/**
  ******************************************************************************
  * @file    stm32wl33.h
  * @author  MCD Application Team
  * @brief   CMSIS Cortex Device Peripheral Access Layer Header File.
  *          This file contains all the peripheral register's definitions, bits
  *          definitions and memory mapping for stm32wl33 devices.
  * 
  *          This file contains:
  *           - Data structures and the address mapping for all peripherals
  *           - Peripheral's registers declarations and bits definition
  *           - Macros to access peripheral's registers hardware
  *
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2024 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */

#ifndef STM32WL33_H
#define STM32WL33_H

#ifdef __cplusplus
extern "C" {
#endif


/** @addtogroup ST
  * @{
  */


/** @addtogroup STM32WL3x
  * @{
  */




/** @addtogroup Configuration_of_CMSIS
  * @{
  */


/* =========================================================================================================================== */
/* ================                                Interrupt Number Definition                                ================ */
/* =========================================================================================================================== */

typedef enum {
/* =======================================  ARM Cortex-M0+ Specific Interrupt Numbers  ======================================= */
  Reset_IRQn                = -15,              /*!< -15  Reset Vector, invoked on Power up and warm reset                     */
  NonMaskableInt_IRQn       = -14,              /*!< -14  Non maskable Interrupt, cannot be stopped or preempted               */
  HardFault_IRQn            = -13,              /*!< -13  Hard Fault, all classes of Fault                                     */
  SVCall_IRQn               =  -5,              /*!< -5 System Service Call via SVC instruction                                */
  PendSV_IRQn               =  -2,              /*!< -2 Pendable request for system service                                    */
  SysTick_IRQn              =  -1,              /*!< -1 System Tick Timer                                                      */
/* ========================================  STM32WL3x Specific Interrupt Numbers  =========================================== */
  FLASH_IRQn                =   0,              /*!< 0  NVM interrupt                                                          */
  RCC_IRQn                  =   1,              /*!< 1  RCC interrupt                                                          */
  PVD_IRQn                  =   2,              /*!< 2  PVD interrupt                                                          */
  I2C1_IRQn                 =   3,              /*!< 3  I2C1 interrupt                                                         */
  I2C2_IRQn                 =   4,              /*!< 4  I2C2 interrupt                                                         */
  SPI1_IRQn                 =   5,              /*!< 5  SPI1 interrupt                                                         */
  SPI3_IRQn                 =   7,              /*!< 7  SPI3 interrupt                                                         */
  USART1_IRQn               =   8,              /*!< 8  USART interrupt                                                        */
  LPUART1_IRQn              =   9,              /*!< 9  Low Power UART interrupt                                               */
  TIM2_IRQn                 =  10,              /*!< 10 Timer 2 interrupt                                                      */
  RTC_IRQn                  =  11,              /*!< 11 RTC interrupt                                                          */
  ADC_IRQn                  =  12,              /*!< 12 ADC interrupt                                                          */
  AES_IRQn                 =  13,              /*!< 13 AES interrupt                                                         */
  GPIOA_IRQn                =  15,              /*!< 15 GPIOA interrupt                                                        */
  GPIOB_IRQn                =  16,              /*!< 16 GPIOB interrupt                                                        */
  DMA_IRQn                  =  17,              /*!< 17 DMA interrupt                                                          */
  LPAWUR_IRQn               =  18,              /*!< 18 LPAWUR interrupt                                                       */
  COMP1_IRQn                =  19,              /*!< 19 Comp interrupt through SYSCFGBLE                                       */
  MR_SUBG_BUSY_IRQn         =  20,              /*!< 20 MR_SUBG Busy interrupt                                                 */
  MR_SUBG_IRQn              =  21,              /*!< 21 MR_SUBG interrupt                                                      */
  TX_RX_SEQUENCE_IRQn       =  22,              /*!< 22 MR_SUBG TX/RX Sequence interrupt                                       */
  CPU_WKUP_IRQn             =  23,              /*!< 23 CPU Wakeup interrupt                                                   */
  SUBG_WKUP_IRQn            =  24,              /*!< 24 SUBG Wakeup interrupt                                                  */
  DAC_IRQn                  =  25,              /*!< 25 DAC interrupt                                                          */
  TIM16_IRQn                =  26,              /*!< 26 TIM16 interrupt                                                        */
  LCD_IRQn                  =  27,              /*!< 27 LCD interrupt                                                          */
  LCSC_IRQn                 =  28,              /*!< 28 LCSC interrupt                                                         */
  LCSC_LC_ACTIVITY_IRQn     =  29               /*!< 28 LCSC LC activity interrupt                                             */
} IRQn_Type;



/* =========================================================================================================================== */
/* ================                           Processor and Core Peripheral Section                           ================ */
/* =========================================================================================================================== */

/* ==========================  Configuration of the ARM Cortex-M0+ Processor and Core Peripherals  =========================== */
/** @addtogroup Configuration_of_CMSIS
  * @{
  */
/**
  * @brief Configuration of the Cortex-M0+ Processor and Core Peripherals
  */

#define __CM0PLUS_REV                  1        /*!< CM0PLUS Core Revision r0p1                                                */
#define __NVIC_PRIO_BITS               2        /*!< Number of Bits used for Priority Levels                                   */
#define __Vendor_SysTickConfig         0        /*!< Set to 1 if different SysTick Config is used                              */
#define __VTOR_PRESENT                 1        /*!< Vector Table Offset Register supported                                    */
#define __MPU_PRESENT                  1        /*!< M0+ provides an MPU                                                       */
#define __FPU_PRESENT                  0        /*!< FPU not present                                                           */
/**
  * @}
  */


 /*!< Device Electronic Signature */
#define PACKAGE_BASE          ((uint32_t)0x40001014U)        /*!< Package size data register base address               */
#define UID64_BASE            ((uint32_t)0x10001EF0U)        /*!< 64-bit Unique device Identification                   */
#define FLASHSIZE_BASE        ((uint32_t)0x40001014U)        /*!< Flash size data register base address                 */
#define RAMSIZE_BASE          ((uint32_t)0x40001014U)        /*!< RAM size data register base address                   */
#define DEV_ID_BASE           ((uint32_t)0x40000000U)        /*!< Device version and cut version register base address  */



/** @} */ /* End of group Configuration_of_CMSIS */

#include "core_cm0plus.h"                       /*!< ARM Cortex-M0+ processor and core peripherals                             */
#include "system_stm32wl3x.h"                   /*!< system_stm32wl3x macros and typedefs System util                          */



/* ========================================  Start of section using anonymous unions  ======================================== */
#if defined (__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined (__ICCARM__)
  #pragma language=extended
#elif defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
  #pragma clang diagnostic push
  #pragma clang diagnostic ignored "-Wc11-extensions"
  #pragma clang diagnostic ignored "-Wreserved-id-macro"
  #pragma clang diagnostic ignored "-Wgnu-anonymous-struct"
  #pragma clang diagnostic ignored "-Wnested-anon-types"
#elif defined (__GNUC__)
  /* anonymous unions are enabled by default */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by default */
#elif defined (__TASKING__)
  #pragma warning 586
#elif defined (__CSMC__)
  /* anonymous unions are enabled by default */
#else
  #warning Not supported compiler type
#endif


/* =========================================================================================================================== */
/* ================                            Device Specific Peripheral Section                             ================ */
/* =========================================================================================================================== */


/** @addtogroup Device_Peripheral_peripherals
  * @{
  */



/* =========================================================================================================================== */
/* ================                                            DMA                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Direct memory access controller (DMA)
  */

typedef struct {                                /*!< DMA Structure                                                             */
  __IO  uint32_t  ISR;                          /*!< (@ 0x00000000) Interrupt status register                                  */
  __IO  uint32_t  IFCR;                         /*!< (@ 0x00000004) Interrupt flag clear register                              */
} DMA_TypeDef;

typedef struct {
  __IO uint32_t CCR;         /*!< DMA channel x configuration register        */
  __IO uint32_t CNDTR;       /*!< DMA channel x number of data register       */
  __IO uint32_t CPAR;        /*!< DMA channel x peripheral address register   */
  __IO uint32_t CMAR;        /*!< DMA channel x memory address register       */
  __IO  uint32_t RESERVED;
} DMA_Channel_TypeDef;




/* =========================================================================================================================== */
/* ================                                          DMAMUX                                           ================ */
/* =========================================================================================================================== */


/**
  * @brief Direct memory access Multiplexer (DMAMUX)
  */

/**
  * @brief DMA Multiplexer
  */
typedef struct {              /*!< DMAMUX Structure                                                                   */
  __IO uint32_t   CxCR;       /*!< DMA Multiplexer Channel x Control Register    Address offset: 0x0004 * (channel x) */
} DMAMUX_Channel_TypeDef;


/* =========================================================================================================================== */
/* ================                                            CRC                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Cyclic redundancy check calculation unit (CRC)
  */

typedef struct {                                /*!< CRC Structure                                                             */
  __IO uint32_t  DR;                           /*!< (@ 0x00000000) Data register                                              */
  __IO uint32_t  IDR;                          /*!< (@ 0x00000004) Independent data register                                  */
  __IO uint32_t  CR;                           /*!< (@ 0x00000008) Control register                                           */
  __IO  uint32_t  RESERVED;
  __IO uint32_t  INIT;                         /*!< (@ 0x00000010) Initial CRC value                                          */
  __IO uint32_t  POL;                          /*!< (@ 0x00000014) Polynomial                                                 */
} CRC_TypeDef;                                  /*!< Size = 24 (0x18)                                                          */



/* =========================================================================================================================== */
/* ================                                           IWDG                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Independent watchdog (IWDG)
  */

typedef struct {                                /*!< IWDG Structure                                                            */
  __IO  uint32_t  KR;                           /*!< (@ 0x00000000) Key register                                               */
  __IO uint32_t  PR;                           /*!< (@ 0x00000004) Prescaler register                                         */
  __IO uint32_t  RLR;                          /*!< (@ 0x00000008) Reload register                                            */
  __IO  uint32_t  SR;                           /*!< (@ 0x0000000C) Status register                                            */
  __IO uint32_t  WINR;                         /*!< (@ 0x00000010) Window register                                            */
} IWDG_TypeDef;                                 /*!< Size = 20 (0x14)                                                          */


/* =========================================================================================================================== */
/* ================                                            I2C                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Inter-integrated circuit (I2C)
  */

typedef struct {                                /*!< I2C1 Structure                                                            */
  __IO uint32_t  CR1;                          /*!< (@ 0x00000000) Control register 1                                         */
  __IO uint32_t  CR2;                          /*!< (@ 0x00000004) Control register 2                                         */
  __IO uint32_t  OAR1;                         /*!< (@ 0x00000008) Own address register 1                                     */
  __IO uint32_t  OAR2;                         /*!< (@ 0x0000000C) Own address register 2                                     */
  __IO uint32_t  TIMINGR;                      /*!< (@ 0x00000010) Timing register                                            */
  __IO uint32_t  TIMEOUTR;                     /*!< (@ 0x00000014) Timeout register                                           */
  __IO uint32_t  ISR;                          /*!< (@ 0x00000018) Interrupt and Status register                              */
  __IO  uint32_t  ICR;                          /*!< (@ 0x0000001C) Interrupt clear register                                   */
  __IO  uint32_t  PECR;                         /*!< (@ 0x00000020) PEC register                                               */
  __IO  uint32_t  RXDR;                         /*!< (@ 0x00000024) Receive data register                                      */
  __IO uint32_t  TXDR;                         /*!< (@ 0x00000028) Transmit data register                                     */
} I2C_TypeDef;                                  /*!< Size = 44 (0x2C)                                                          */



/* =========================================================================================================================== */
/* ================                                           FLASH                                           ================ */
/* =========================================================================================================================== */


/**
  * @brief FLASH (FLASH)
  */

typedef struct {                                /*!< FLASH Structure                                                           */
  __IO uint32_t  COMMAND;                      /*!< (@ 0x00000000) Command register                                           */
  __IO uint32_t  CONFIG;                       /*!< (@ 0x00000004) Configuration register                                     */
  __IO uint32_t  IRQSTAT;                      /*!< (@ 0x00000008) Interrupt status register                                  */
  __IO uint32_t  IRQMASK;                      /*!< (@ 0x0000000C) Interrupt mask register                                    */
  __IO uint32_t  IRQRAW;                       /*!< (@ 0x00000010) Intertupt raw status register                              */
  __IO  uint32_t  SIZE;                         /*!< (@ 0x00000014) SIZE register                                              */
  __IO uint32_t  ADDRESS;                      /*!< (@ 0x00000018) Address register                                           */
  __IO  uint32_t  RESERVED[2];
  __IO  uint32_t  LFSRVAL;            /*!< (@ 0x00000024) LFSRVAL register            */
  __IO  uint32_t RESERVED2[3];
  __IO uint32_t  PAGEPROT0;                    /*!< (@ 0x00000034) Main Flash page protection register 0                      */
  __IO uint32_t  PAGEPROT1;                    /*!< (@ 0x00000038) Main Flash page protection register 1                      */
  __IO  uint32_t  RESERVED1;
  __IO uint32_t  DATA0;                        /*!< (@ 0x00000040) Data register 0                                            */
  __IO uint32_t  DATA1;                        /*!< (@ 0x00000044) Data register 1                                            */
  __IO uint32_t  DATA2;                        /*!< (@ 0x00000048) Data register 2                                            */
  __IO uint32_t  DATA3;                        /*!< (@ 0x0000004C) Data register 3                                            */
} FLASH_TypeDef;                                /*!< Size = 80 (0x50)                                                          */



/* =========================================================================================================================== */
/* ================                                            SPI                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Serial peripheral interface/Inter-IC sound (SPI)
  */

typedef struct {                                /*!< SPI Structure                                                             */
  __IO uint32_t  CR1;                          /*!< (@ 0x00000000) Control register 1                                         */
  __IO uint32_t  CR2;                          /*!< (@ 0x00000004) Control register 2                                         */
  __IO uint32_t  SR;                           /*!< (@ 0x00000008) Status register                                            */
  __IO uint32_t  DR;                           /*!< (@ 0x0000000C) Data register                                              */
  __IO uint32_t  CRCPR;                        /*!< (@ 0x00000010) CRC polynomial register                                    */
  __IO uint32_t   RXCRCR;                       /*!< (@ 0x00000014) RX CRC register                                            */
  __IO uint32_t   TXCRCR;                       /*!< (@ 0x00000018) TX CRC register                                            */
  __IO uint32_t  I2SCFGR;                      /*!< (@ 0x0000001C) I2S configuration register                                 */
  __IO uint32_t  I2SPR;                        /*!< (@ 0x00000020) I2S prescaler register                                     */
} SPI_TypeDef;                                  /*!< Size = 36 (0x24)                                                          */



/* =========================================================================================================================== */
/* ================                                            RCC                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Reset and clock control (RCC)
  */

typedef struct{ /*!< RCC Structure  */
  __IO uint32_t CR;       /*!< (@ 0x00000000) CR register                 */
  __IO uint32_t ICSCR;    /*!< (@ 0x00000004) ICSCR register              */
  __IO uint32_t CFGR;     /*!< (@ 0x00000008) CFGR register               */
  __IO uint32_t CSSWCR;   /*!< (@ 0x0000000C) CSSWCR register             */
  __IO uint32_t KRMR;     /*!< (@ 0x00000010) KRMR register               */
  __IO  uint32_t RESERVED;
  __IO uint32_t CIER;     /*!< (@ 0x00000018) CIER register               */
  __IO uint32_t CIFR;     /*!< (@ 0x0000001C) CIFR register               */
  __IO uint32_t CSCMDR;   /*!< (@ 0x00000020) CSCMDR register             */
  __IO  uint32_t RESERVED1[3];
  __IO uint32_t AHBRSTR;  /*!< (@ 0x00000030) AHBRSTR register            */
  __IO uint32_t APB0RSTR; /*!< (@ 0x00000034) APB0RSTR register           */
  __IO uint32_t APB1RSTR; /*!< (@ 0x00000038) APB1RSTR register           */
  __IO  uint32_t RESERVED2;
  __IO uint32_t APB2RSTR; /*!< (@ 0x00000040) APB2RSTR register           */
  __IO  uint32_t RESERVED3[3];
  __IO uint32_t AHBENR;   /*!< (@ 0x00000050) AHBENR register             */
  __IO uint32_t APB0ENR;  /*!< (@ 0x00000054) APB0ENR register            */
  __IO uint32_t APB1ENR;  /*!< (@ 0x00000058) APB1ENR register            */
  __IO  uint32_t RESERVED4;
  __IO uint32_t APB2ENR;  /*!< (@ 0x00000060) APB2ENR register            */
  __IO  uint32_t RESERVED5[12];
  __IO uint32_t CSR;      /*!< (@ 0x00000094) CSR register                */
  __IO uint32_t RFSWHSECR;/*!< (@ 0x00000098) RFSWHSECR register          */
  __IO  uint32_t RFHSECR;  /*!< (@ 0x0000009C) RFHSECR register            */
  __IO uint32_t AHBSMENR; /*!< (@ 0x000000A0) AHBSMENR register           */
  __IO uint32_t APB0SMENR;/*!< (@ 0x000000A4) APB0SMENR register          */
  __IO uint32_t APB1SMENR;/*!< (@ 0x000000A8) APB1SMENR register          */
} RCC_TypeDef;             /*!< Size = 172 (0xAC) */


  
/* =========================================================================================================================== */
/* ================                                            PWR                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Power control (PWR)
  */

typedef struct{ /*!< PWR Structure  */
  __IO uint32_t CR1;       /*!< (@ 0x00000000) CR1 register                 */
  __IO uint32_t CR2;       /*!< (@ 0x00000004) CR2 register                 */
  __IO uint32_t IEWU;      /*!< (@ 0x00000008) IEWU register                */
  __IO uint32_t IWUP;      /*!< (@ 0x0000000C) IWUP register                */
  __IO uint32_t IWUF;      /*!< (@ 0x00000010) IWUF register                */
  __IO  uint32_t SR2;       /*!< (@ 0x00000014) SR2 register                 */
  __IO  uint32_t RESERVED;
  __IO uint32_t CR5;       /*!< (@ 0x0000001C) CR5 register                 */
  __IO uint32_t PUCRA;     /*!< (@ 0x00000020) PUCRA register               */
  __IO uint32_t PDCRA;     /*!< (@ 0x00000024) PDCRA register               */
  __IO uint32_t PUCRB;     /*!< (@ 0x00000028) PUCRB register               */
  __IO uint32_t PDCRB;     /*!< (@ 0x0000002C) PDCRB register               */
  __IO uint32_t EWUA;      /*!< (@ 0x00000030) EWUA register                */
  __IO uint32_t WUPA;      /*!< (@ 0x00000034) WUPA register                */
  __IO uint32_t WUFA;      /*!< (@ 0x00000038) WUFA register                */
  __IO  uint32_t RESERVED1;
  __IO uint32_t EWUB;      /*!< (@ 0x00000040) EWUB register                */
  __IO uint32_t WUPB;      /*!< (@ 0x00000044) WUPB register                */
  __IO uint32_t WUFB;      /*!< (@ 0x00000048) WUFB register                */
  __IO uint32_t SDWN_WUEN; /*!< (@ 0x0000004C) SDWN_WUEN register           */
  __IO uint32_t SDWN_WUPOL;/*!< (@ 0x00000050) SDWN_WUPOL register          */
  __IO uint32_t SDWN_WUF;  /*!< (@ 0x00000054) SDWN_WUF register            */
  __IO uint32_t BOF_TUNE;  /*!< (@ 0x00000058) BOF_TUNE register            */
  __IO  uint32_t RESERVED2[10];
  __IO uint32_t DBGR;      /*!< (@ 0x00000084) DBGR register                */
  __IO uint32_t EXTSRR;    /*!< (@ 0x00000088) EXTSRR register              */
  __IO  uint32_t RESERVED3;
  __IO  uint32_t TRIMR;     /*!< (@ 0x00000090) TRIMR register               */
  __IO uint32_t ENGTRIM;   /*!< (@ 0x00000094) ENGTRIM register             */
  __IO  uint32_t RESERVED4[2];
  __IO uint32_t ENGTRIM2;  /*!< (@ 0x000000A0) ENGTRIM register             */
} PWR_TypeDef;              /*!< Size = 164 (0xA4) */



/* =========================================================================================================================== */
/* ================                                          SYSCFG                                           ================ */
/* =========================================================================================================================== */


/**
  * @brief System configuration controller (SYSCFG)
  */

typedef struct {                                /*!< SYSCFG Structure                                                                        */
  __IO  uint32_t  DIE_ID;                       /*!< (@ 0x00000000) Die ID register                                                          */
  __IO  uint32_t  JTAG_ID;                      /*!< (@ 0x00000004) JTAG ID register                                                         */
  __IO uint32_t  I2C_FMP_CTRL;                 /*!< (@ 0x00000008) I2C Fast-Mode Plus pin capability control register                       */
  __IO uint32_t  IO_DTR;                       /*!< (@ 0x0000000C) I/O Interrupt detection type register                                    */
  __IO uint32_t  IO_IBER;                      /*!< (@ 0x00000010) I/O Interrupt Edge register                                              */
  __IO uint32_t  IO_IEVR;                      /*!< (@ 0x00000014) I/O Interrupt polarity event register                                    */
  __IO uint32_t  IO_IER;                       /*!< (@ 0x00000018) I/O Interrupt Enable register                                            */
  __IO uint32_t  IO_ISCR;                      /*!< (@ 0x0000001C) I/O Interrupt Status and Clear register                                  */
  __IO uint32_t  PWRC_IER;                     /*!< (@ 0x00000020) Power Controller Interrupt Enable register                               */
  __IO uint32_t  PWRC_ISCR;                    /*!< (@ 0x00000024) Power Controller Interrupt Status and Clear register                     */
  __IO uint32_t  GPIO_SWA_CTRL;                /*!< (@ 0x00000028) I/O analog switch control register                                       */
  __IO uint32_t  INTAI_DTR;                    /*!< (@ 0x0000002C) Internal asynchronous interrupt detection type register                  */
  __IO uint32_t  INTAI_IBER;                   /*!< (@ 0x00000030) Internal asynchronous interrupt edge register                            */
  __IO uint32_t  INTAI_IEVR;                   /*!< (@ 0x00000034) Internal asynchronous interrupt polarity event register                  */
  __IO uint32_t  INTAI_IER;                    /*!< (@ 0x00000038) Internal asynchronous interrupt enable register                          */
  __IO uint32_t  INTAI_ISCR;                   /*!< (@ 0x0000003C) Internal asynchronous interrupt detection status and clear register      */
  __IO  uint32_t  SR1;                          /*!< (@ 0x00000040) SYSCFG status register 1                                                 */
} SYSCFG_TypeDef;                               /*!< Size = 68 (0x44)                                                                        */



/* =========================================================================================================================== */
/* ================                                            RNG                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Random number generator (RNG)
  */

typedef struct {                                /*!< RNG Structure                                                             */
  __IO uint32_t  CR;                           /*!< (@ 0x00000000) Control register                                           */
  __IO uint32_t  SR;                           /*!< (@ 0x00000004) Status register                                            */
  __IO  uint32_t  VAL;                          /*!< (@ 0x00000008) Data register                                              */
} RNG_TypeDef;                                  /*!< Size = 12 (0xC)                                                           */



/* =========================================================================================================================== */
/* ================                                           GPIO                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief General-purpose I/Os (GPIO)
  */

typedef struct {                                /*!< GPIO Structure                                                            */
  __IO uint32_t  MODER;                        /*!< (@ 0x00000000) GPIO port mode register                                    */
  __IO uint32_t  OTYPER;                       /*!< (@ 0x00000004) GPIO port output type register                             */
  __IO uint32_t  OSPEEDR;                      /*!< (@ 0x00000008) GPIO port output speed register                            */
  __IO uint32_t  PUPDR;                        /*!< (@ 0x0000000C) GPIO port pull-up/pull-down register                       */
  __IO  uint32_t  IDR;                          /*!< (@ 0x00000010) GPIO port input data register                              */
  __IO uint32_t  ODR;                          /*!< (@ 0x00000014) GPIO port output data register                             */
  __IO  uint32_t  BSRR;                         /*!< (@ 0x00000018) GPIO port bit set/reset register                           */
  __IO uint32_t  LCKR;                         /*!< (@ 0x0000001C) GPIO port configuration lock register                      */
  __IO uint32_t  AFR[2];                       /*!< (@ 0x00000020) GPIO alternate function register                           */
  __IO  uint32_t  BRR;                          /*!< (@ 0x00000028) GPIO bit reset register                                    */
} GPIO_TypeDef;                                 /*!< Size = 44 (0x2c)                                                          */



/* =========================================================================================================================== */
/* ================                                            TIM                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Advanced-timers (TIM)
  */

typedef struct {                                /*!< TIMx Structure                                                            */
  __IO uint32_t  CR1;                          /*!< (@ 0x00000000) Control register 1                                         */
  __IO uint32_t  CR2;                          /*!< (@ 0x00000004) Control register 2                                         */
  __IO uint32_t  SMCR;                         /*!< (@ 0x00000008) Slave mode control register                                */
  __IO uint32_t  DIER;                         /*!< (@ 0x0000000C) DMA/Interrupt enable register                              */
  __IO uint32_t  SR;                           /*!< (@ 0x00000010) Status register                                            */
  __IO  uint32_t  EGR;                          /*!< (@ 0x00000014) Event generation register                                  */
  __IO uint32_t  CCMR1;                        /*!< (@ 0x00000018) Input capture and output compare mode register 1           */
  __IO uint32_t  CCMR2;                        /*!< (@ 0x0000001C) Input capture and output compare mode register 2           */
  __IO uint32_t  CCER;                         /*!< (@ 0x00000020) Capture/compare enable register                            */
  __IO uint32_t  CNT;                          /*!< (@ 0x00000024) Counter                                                    */
  __IO uint32_t  PSC;                          /*!< (@ 0x00000028) Prescaler                                                  */
  __IO uint32_t  ARR;                          /*!< (@ 0x0000002C) Auto-reload register                                       */
  __IO uint32_t  RCR;                          /*!< (@ 0x00000030) Repetition counter register                                */
  __IO uint32_t  CCR1;                         /*!< (@ 0x00000034) Capture/compare register 1                                 */
  __IO uint32_t  CCR2;                         /*!< (@ 0x00000038) Capture/compare register 2                                 */
  __IO uint32_t  CCR3;                         /*!< (@ 0x0000003C) Capture/compare register 3                                 */
  __IO uint32_t  CCR4;                         /*!< (@ 0x00000040) Capture/compare register 4                                 */
  __IO uint32_t  BDTR;                         /*!< (@ 0x00000044) Break and dead-time register                               */
  __IO uint32_t  DCR;                          /*!< (@ 0x00000048) DMA control register                                       */
  __IO uint32_t  DMAR;                         /*!< (@ 0x0000004C) DMA address for full transfer                              */
  __IO uint32_t  OR;                           /*!< (@ 0x00000050) Option register 1                                          */
  __IO  uint32_t  RESERVED[3];
  __IO uint32_t  AF1;                          /*!< (@ 0x00000060) TIM alternate function option register 1                   */
  __IO  uint32_t  RESERVED1;  
  __IO uint32_t  TISEL;                        /*!< (@ 0x00000068) Input selection register                                   */
} TIM_TypeDef;                                  /*!< Size = 108 (0x6C)                                                         */



/* =========================================================================================================================== */
/* ================                                           USART                                           ================ */
/* =========================================================================================================================== */


/**
  * @brief Universal synchronous asynchronous receiver transmitter (USART)
  */

typedef struct {                                /*!< USART Structure                                                           */
  __IO uint32_t  CR1;                          /*!< (@ 0x00000000) Control register 1                                         */
  __IO uint32_t  CR2;                          /*!< (@ 0x00000004) Control register 2                                         */
  __IO uint32_t  CR3;                          /*!< (@ 0x00000008) Control register 3                                         */
  __IO uint32_t  BRR;                          /*!< (@ 0x0000000C) Baud rate register                                         */
  __IO uint32_t  GTPR;                         /*!< (@ 0x00000010) Guard time and prescaler register                          */
  __IO uint32_t  RTOR;                         /*!< (@ 0x00000014) Receiver timeout register                                  */
  __IO  uint32_t  RQR;                          /*!< (@ 0x00000018) Request register                                           */
  __IO  uint32_t  ISR;                          /*!< (@ 0x0000001C) Interrupt & status register                                */
  __IO  uint32_t  ICR;                          /*!< (@ 0x00000020) Interrupt flag clear register                              */
  __IO  uint32_t  RDR;                          /*!< (@ 0x00000024) Receive data register                                      */
  __IO uint32_t  TDR;                          /*!< (@ 0x00000028) Transmit data register                                     */
  __IO uint32_t  PRESC;                        /*!< (@ 0x0000002C) Prescaler register                                         */
} USART_TypeDef;                                /*!< Size = 48 (0x30)                                                          */



/* =========================================================================================================================== */
/* ================                                            RTC                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Real-time clock (RTC)
  */

typedef struct {                                /*!< RTC Structure                                                             */
  __IO uint32_t  TR;                           /*!< (@ 0x00000000) Time register                                              */
  __IO uint32_t  DR;                           /*!< (@ 0x00000004) Date register                                              */
  __IO uint32_t  CR;                           /*!< (@ 0x00000008) Control register                                           */
  __IO uint32_t  ISR;                          /*!< (@ 0x0000000C) Initialization and status register                         */
  __IO uint32_t  PRER;                         /*!< (@ 0x00000010) Prescaler register                                         */
  __IO uint32_t  WUTR;                         /*!< (@ 0x00000014) Wakeup timer register                                      */
  __IO  uint32_t  RESERVED;
  __IO uint32_t  ALRMAR;                       /*!< (@ 0x0000001C) Alarm A register                                           */
  __IO  uint32_t  RESERVED1;
  __IO  uint32_t  WPR;                          /*!< (@ 0x00000024) Write protection register                                  */
  __IO  uint32_t  SSR;                          /*!< (@ 0x00000028) Sub second register                                        */
  __IO  uint32_t  SHIFTR;                       /*!< (@ 0x0000002C) Shift control register                                     */
  __IO  uint32_t  TSTR;                         /*!< (@ 0x00000030) Timestamp time register                                    */
  __IO  uint32_t  TSDR;                         /*!< (@ 0x00000034) Timestamp date register                                    */
  __IO  uint32_t  TSSSR;                        /*!< (@ 0x00000038) Timestamp sub second register                              */
  __IO uint32_t  CALR;                         /*!< (@ 0x0000003C) Calibration register                                       */
  __IO uint32_t  TAMPCR;                       /*!< (@ 0x00000040) Tamper configuration register                              */
  __IO uint32_t  ALRMASSR;                     /*!< (@ 0x00000044) Alarm A sub second register                                */
  __IO  uint32_t  RESERVED2;
  __IO uint32_t  OR;                           /*!< (@ 0x0000004C) Option register                                            */
  __IO uint32_t  BKP0R;                        /*!< (@ 0x00000050) Backup register 0                                          */
  __IO uint32_t  BKP1R;                        /*!< (@ 0x00000054) Backup register 1                                          */
} RTC_TypeDef;                                  /*!< Size = 88 (0x58)                                                          */



/* =========================================================================================================================== */
/* ================                                            AES                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief Advanced encryption standard hardware accelerator (AES)
  */

typedef struct {              /*!< AES Structure                                                                               */
  __IO uint32_t   CR;        /*!< (@ 0x00000000) Control Register                                                             */
  __IO  uint32_t   SR;        /*!< (@ 0x00000004) Status Register                                                              */
  __IO uint32_t   DINR;      /*!< (@ 0x00000008) Data Input register                                                          */
  __IO  uint32_t   DOUTR;     /*!< (@ 0x0000000C) Data Output register                                                         */
  __IO uint32_t   KEYR0;     /*!< (@ 0x00000010) Key register 0                                                               */
  __IO uint32_t   KEYR1;     /*!< (@ 0x00000014) Key register 1                                                               */
  __IO uint32_t   KEYR2;     /*!< (@ 0x00000018) Key register 2                                                               */
  __IO uint32_t   KEYR3;     /*!< (@ 0x0000001C) Key register 3                                                               */
  __IO uint32_t   IVR0;      /*!< (@ 0x00000020) Initialization vector register 0                                             */
  __IO uint32_t   IVR1;      /*!< (@ 0x00000024) Initialization vector register 1                                             */
  __IO uint32_t   IVR2;      /*!< (@ 0x00000028) Initialization vector register 2                                             */
  __IO uint32_t   IVR3;      /*!< (@ 0x0000002C) Initialization vector register 3                                             */
  __IO uint32_t   KEYR4;     /*!< (@ 0x00000030) Key register 4                                                               */
  __IO uint32_t   KEYR5;     /*!< (@ 0x00000034) Key register 5                                                               */
  __IO uint32_t   KEYR6;     /*!< (@ 0x00000038) Key register 6                                                               */
  __IO uint32_t   KEYR7;     /*!< (@ 0x0000003C) Key register 7                                                               */
  __IO uint32_t   SUSP0;     /*!< (@ 0x00000040) Suspend register 0                                                           */
  __IO uint32_t   SUSP1;     /*!< (@ 0x00000044) Suspend register 1                                                           */
  __IO uint32_t   SUSP2;     /*!< (@ 0x00000048) Suspend register 2                                                           */
  __IO uint32_t   SUSP3;     /*!< (@ 0x0000004C) Suspend register 3                                                           */
  __IO uint32_t   SUSP4;     /*!< (@ 0x00000050) Suspend register 4                                                           */
  __IO uint32_t   SUSP5;     /*!< (@ 0x00000054) Suspend register 5                                                           */
  __IO uint32_t   SUSP6;     /*!< (@ 0x00000058) Suspend register 6                                                           */
  __IO uint32_t   SUSP7;     /*!< (@ 0x0000005C) Suspend register 7                                                           */
} AES_TypeDef;                /*!< Size = 96 (0x60)                                                                            */



/* =========================================================================================================================== */
/* ================                                            ADC                                            ================ */
/* =========================================================================================================================== */


/**
  * @brief ADC (ADC)
  */

typedef struct {                                /*!< ADC Structure                                                             */
  __IO  uint32_t  VERSION_ID;                   /*!< (@ 0x00000000) VERSION_ID register                                        */
  __IO uint32_t  CONF;                         /*!< (@ 0x00000004) ADC configuration register                                 */
  __IO uint32_t  CTRL;                         /*!< (@ 0x00000008) ADC control register                                       */
  __IO  uint32_t  RESERVED[2];
  __IO uint32_t  SWITCH;                       /*!< (@ 0x00000014) ADC switch control for Input Selection                     */
  __IO  uint32_t  RESERVED1;
  __IO uint32_t  DS_CONF;                      /*!< (@ 0x0000001C) Downsampler configuration register                         */
  __IO uint32_t  SEQ_1;                        /*!< (@ 0x00000020) ADC regular sequence programming register 1                */
  __IO uint32_t  SEQ_2;                        /*!< (@ 0x00000024) ADC regular sequence programming register 2                */
  __IO uint32_t  COMP_1;                       /*!< (@ 0x00000028) ADC Gain & offset correction values register 1             */
  __IO uint32_t  COMP_2;                       /*!< (@ 0x0000002C) ADC Gain & offset correction values register 2             */
  __IO uint32_t  COMP_3;                       /*!< (@ 0x00000030) ADC Gain & offset correction values register 3             */
  __IO uint32_t  COMP_4;                       /*!< (@ 0x00000034) ADC Gain & offset correction values register 4             */
  __IO uint32_t  COMP_SEL;                     /*!< (@ 0x00000038) ADC Gain & Offset selection values register                */
  __IO uint32_t  WD_TH;                        /*!< (@ 0x0000003C) ADC watchdog threshold register register                   */
  __IO uint32_t  WD_CONF;                      /*!< (@ 0x00000040) ADC watchdog configuration register                        */
  __IO  uint32_t  DS_DATAOUT;                   /*!< (@ 0x00000044) Downsampler Data output register                           */
  __IO  uint32_t  RESERVED2;
  __IO uint32_t  IRQ_STATUS;                   /*!< (@ 0x0000004C) Interrupt Status register                                  */
  __IO uint32_t  IRQ_ENABLE;                   /*!< (@ 0x00000050) Enable/disable Interrupts                                  */
} ADC_TypeDef;                                  /*!< Size = 84 (0x54)                                                          */



/* =========================================================================================================================== */
/* ================                                     COMP                                                  ================ */
/* =========================================================================================================================== */


/**
  * @brief Comparator (COMP)
  */

typedef struct {                              /*!< COMP Structure                                                              */
  __IO  uint32_t  CSR;                       /*!< (@ 0x00000000) Control and status register                                  */
} COMP_TypeDef;                               /*!< Size = 4 (0x04)                                                             */



/* =========================================================================================================================== */
/* ================                                     DAC                                                   ================ */
/* =========================================================================================================================== */


/**
  * @brief Digital to Analog Converter (DAC)
  */

typedef struct {                                /*!< DAC Structure                                                 */
  __IO uint32_t  CR;                           /*!< (@ 0x00000000) Control register                               */
  __IO  uint32_t  SWTRIGR;                      /*!< (@ 0x00000004) Software trigger register                      */
  __IO  uint32_t  RESERVED[2];
  __IO uint32_t  DHR;                          /*!< (@ 0x00000010) Channel data holding register                  */
  __IO  uint32_t  RESERVED1[6];
  __IO  uint32_t  DOR;                          /*!< (@ 0x0000002C) Channel data output register                   */
  __IO  uint32_t  RESERVED2;
  __IO uint32_t  SR;                           /*!< (@ 0x00000034) Status register                                */
} DAC_TypeDef;                                  /*!< Size = 56 (0x38)                                              */



/* =========================================================================================================================== */
/* ================                                     LCSC                                                  ================ */
/* =========================================================================================================================== */


/**
  * @brief LC Sensor Controller (LCSC)
  */

typedef struct{ /*!< LCSC Structure  */
  __IO uint32_t CR0;       /*!< (@ 0x00000000) LCSC_CR0 register               */
  __IO uint32_t CR1;       /*!< (@ 0x00000004) LCSC_CR1 register               */
  __IO uint32_t CR2;       /*!< (@ 0x00000008) LCSC_CR2 register               */
  __IO uint32_t PULSE_CR;  /*!< (@ 0x0000000C) LCSC_PULSE_CR register          */
  __IO uint32_t ENR;       /*!< (@ 0x00000010) LCSC_ENR register               */
  __IO  uint32_t WHEEL_SR;  /*!< (@ 0x00000014) LCSC_WHEEL_SR register          */
  __IO uint32_t CONFR;     /*!< (@ 0x00000018) LCSC_CONFR register             */
  __IO  uint32_t COMP_CTN;  /*!< (@ 0x0000001C) LCSC_COMP_CTN register          */
  __IO  uint32_t SR;        /*!< (@ 0x00000020) LCSC_SR register                */
  __IO uint32_t STAT;      /*!< (@ 0x00000024) LCSC_STAT register              */
  __IO  uint32_t RESERVED[6];
  __IO  uint32_t VER;       /*!< (@ 0x00000040) LCSC_VER register               */
  __IO uint32_t ISR;       /*!< (@ 0x00000044) LCSC_ISR register               */
} LCSC_TypeDef;              /*!< Size = 72 (0x48) */



/* =========================================================================================================================== */
/* ================                                     DBGMCU                                                ================ */
/* =========================================================================================================================== */


/**
  * @brief MCU debug component (DBGMCU)
  */

typedef struct{ /*!< DBGMCU Structure  */
  __IO uint32_t CR;                /*!< (@ 0x00000000) CR register                   */
  __IO uint32_t DBG_APB0_FZ;       /*!< (@ 0x00000004) DBG_APB0_FZ register          */
  __IO uint32_t DBG_APB1_FZ;       /*!< (@ 0x00000008) DBG_APB1_FZ register          */
} DBGMCU_TypeDef;                      /*!< Size = 12 (0x0C) */



/* =========================================================================================================================== */
/* ================                                     LCD                                                   ================ */
/* =========================================================================================================================== */


/**
  * @brief Liquid crystal display controller (LCD)
  */

typedef struct {                                /*!< LCD Structure                                                             */
  __IO  uint32_t  CR;                          /*!< (@ 0x00000000) Control register                                           */
  __IO  uint32_t  FCR;                         /*!< (@ 0x00000004) Frame control register                                     */
  __IO  uint32_t  SR;                          /*!< (@ 0x00000008) Status register                                            */
  __IO   uint32_t  CLR;                         /*!< (@ 0x0000000C) Clear register                                             */
  __IO   uint32_t  RESERVED;
  __IO  uint32_t  RAM_COM0;                    /*!< (@ 0x00000014) Disaplay Memory COM 0                                      */
  __IO   uint32_t  RESERVED1;
  __IO  uint32_t  RAM_COM1;                    /*!< (@ 0x0000001C) Disaplay Memory COM 1                                      */
  __IO   uint32_t  RESERVED2;
  __IO  uint32_t  RAM_COM2;                    /*!< (@ 0x00000024) Disaplay Memory COM 2                                      */
  __IO   uint32_t  RESERVED3;
  __IO  uint32_t  RAM_COM3;                    /*!< (@ 0x0000002C) Disaplay Memory COM 3                                      */
  __IO   uint32_t  RESERVED4;
  __IO  uint32_t  RAM_COM4;                    /*!< (@ 0x00000034) Disaplay Memory COM 4                                      */
  __IO   uint32_t  RESERVED5;
  __IO  uint32_t  RAM_COM5;                    /*!< (@ 0x0000003C) Disaplay Memory COM 5                                      */
  __IO   uint32_t  RESERVED6;
  __IO  uint32_t  RAM_COM6;                    /*!< (@ 0x00000044) Disaplay Memory COM 6                                      */
  __IO   uint32_t  RESERVED7;
  __IO  uint32_t  RAM_COM7;                    /*!< (@ 0x0000004C) Disaplay Memory COM 7                                      */
} LCD_TypeDef;                                  /*!< Size = 80 (0x50)                                                          */



/* =========================================================================================================================== */
/* ================                                   MR_SUBG_RADIO                                           ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Radio
  */

typedef struct{ /*!< MR_SUBG_RADIO Structure  */
  __IO uint32_t RF_FSM0_TIMEOUT;       /*!< (@ 0x00000000) RF_FSM0_TIMEOUT register                */
  __IO uint32_t RF_FSM1_TIMEOUT;       /*!< (@ 0x00000004) RF_FSM1_TIMEOUT register                */
  __IO uint32_t RF_FSM2_TIMEOUT;       /*!< (@ 0x00000008) RF_FSM2_TIMEOUT register                */
  __IO uint32_t RF_FSM3_TIMEOUT;       /*!< (@ 0x0000000C) RF_FSM3_TIMEOUT register                */
  __IO uint32_t RF_FSM4_TIMEOUT;       /*!< (@ 0x00000010) RF_FSM4_TIMEOUT register                */
  __IO uint32_t RF_FSM5_TIMEOUT;       /*!< (@ 0x00000014) RF_FSM5_TIMEOUT register                */
  __IO uint32_t RF_FSM6_TIMEOUT;       /*!< (@ 0x00000018) RF_FSM6_TIMEOUT register                */
  __IO uint32_t RF_FSM7_TIMEOUT;       /*!< (@ 0x0000001C) RF_FSM7_TIMEOUT register                */
  __IO uint32_t AFC0_CONFIG;           /*!< (@ 0x00000020) AFC0_CONFIG register                    */
  __IO uint32_t AFC1_CONFIG;           /*!< (@ 0x00000024) AFC1_CONFIG register                    */
  __IO uint32_t AFC2_CONFIG;           /*!< (@ 0x00000028) AFC2_CONFIG register                    */
  __IO uint32_t AFC3_CONFIG;           /*!< (@ 0x0000002C) AFC3_CONFIG register                    */
  __IO uint32_t CLKREC_CTRL0;          /*!< (@ 0x00000030) CLKREC_CTRL0 register                   */
  __IO uint32_t CLKREC_CTRL1;          /*!< (@ 0x00000034) CLKREC_CTRL1 register                   */
  __IO uint32_t DCREM_CTRL0;           /*!< (@ 0x00000038) DCREM_CTRL0 register                    */
  __IO uint32_t DCREM_CTRL1;           /*!< (@ 0x0000003C) DCREM_CTRL1 register                    */
  __IO uint32_t IQC_CTRL0;             /*!< (@ 0x00000040) IQC_CTRL0 register                      */
  __IO uint32_t IQC_CTRL1;             /*!< (@ 0x00000044) IQC_CTRL1 register                      */
  __IO uint32_t IQC_CTRL2;             /*!< (@ 0x00000048) IQC_CTRL2 register                      */
  __IO uint32_t IQC_CTRL3;             /*!< (@ 0x0000004C) IQC_CTRL3 register                      */
  __IO  uint32_t RESERVED;
  __IO uint32_t AGC0_CTRL;             /*!< (@ 0x00000054) AGC0_CTRL register                      */
  __IO uint32_t AGC1_CTRL;             /*!< (@ 0x00000058) AGC1_CTRL register                      */
  __IO uint32_t AGC2_CTRL;             /*!< (@ 0x0000005C) AGC2_CTRL register                      */
  __IO uint32_t AGC3_CTRL;             /*!< (@ 0x00000060) AGC3_CTRL register                      */
  __IO uint32_t AGC4_CTRL;             /*!< (@ 0x00000064) AGC4_CTRL register                      */
  __IO uint32_t AGC_ATTEN0;            /*!< (@ 0x00000068) AGC_ATTEN0 register                     */
  __IO uint32_t AGC_ATTEN1;            /*!< (@ 0x0000006C) AGC_ATTEN1 register                     */
  __IO uint32_t AGC_ATTEN2;            /*!< (@ 0x00000070) AGC_ATTEN2 register                     */
  __IO uint32_t AGC_ATTEN3;            /*!< (@ 0x00000074) AGC_ATTEN3 register                     */
  __IO uint32_t AGC_ATTEN4;            /*!< (@ 0x00000078) AGC_ATTEN4 register                     */
  __IO uint32_t AGC_ATTEN5;            /*!< (@ 0x0000007C) AGC_ATTEN5 register                     */
  __IO uint32_t AGC_ATTEN6;            /*!< (@ 0x00000080) AGC_ATTEN6 register                     */
  __IO uint32_t AGC_ATTEN7;            /*!< (@ 0x00000084) AGC_ATTEN7 register                     */
  __IO uint32_t AGC_ATTEN8;            /*!< (@ 0x00000088) AGC_ATTEN8 register                     */
  __IO uint32_t AGC_ATTEN9;            /*!< (@ 0x0000008C) AGC_ATTEN9 register                     */
  __IO uint32_t AGC1_ATTEN_TRIM;       /*!< (@ 0x00000090) AGC1_ATTEN_TRIM register                */
  __IO uint32_t AGC2_ATTEN_TRIM;       /*!< (@ 0x00000094) AGC2_ATTEN_TRIM register                */
  __IO uint32_t AGC3_ATTEN_TRIM;       /*!< (@ 0x00000098) AGC3_ATTEN_TRIM register                */
  __IO uint32_t AGC4_ATTEN_TRIM;       /*!< (@ 0x0000009C) AGC4_ATTEN_TRIM register                */
  __IO  uint32_t AGC_PGA_HWTRIM_OUT;    /*!< (@ 0x000000A0) AGC_PGA_HWTRIM_OUT register             */
  __IO  uint32_t RESERVED1;
  __IO uint32_t PA_REG;                /*!< (@ 0x000000A8) PA_REG register                         */
  __IO  uint32_t PA_HWTRIM_OUT;         /*!< (@ 0x000000AC) PA_HWTRIM_OUT register                  */
  __IO  uint32_t RESERVED2[3];
  __IO uint32_t RSSI_FLT;              /*!< (@ 0x000000BC) RSSI_FLT register                       */
  __IO  uint32_t RESERVED3[2];
  __IO uint32_t SYNTH2_ANA_ENG;        /*!< (@ 0x000000C8) SYNTH2_ANA_ENG register                 */
  __IO  uint32_t RESERVED4[7];
  __IO  uint32_t RXADC_HWDELAYTRIM_OUT; /*!< (@ 0x000000E8) RXADC_HWDELAYTRIM_OUT register          */
  __IO  uint32_t RESERVED5[2];
  __IO  uint32_t RX_AAF_HWTRIM_OUT;     /*!< (@ 0x000000F4) RX_AAF_HWTRIM_OUT register              */
  __IO  uint32_t RESERVED6[2];
  __IO uint32_t SINGEN_ANA_ENG;        /*!< (@ 0x00000100) SINGEN_ANA_ENG register                 */
  __IO  uint32_t RESERVED7;
  __IO  uint32_t RF_INFO_OUT;          /*!< (@ 0x00000108) RF_INFO_OUT register                */
  __IO  uint32_t RESERVED8[6];
  __IO uint32_t RF_FSM8_TIMEOUT;       /*!< (@ 0x00000124) RF_FSM8_TIMEOUT register                */
  __IO uint32_t RF_FSM9_TIMEOUT;       /*!< (@ 0x00000128) RF_FSM9_TIMEOUT register                */
  __IO uint32_t RF_FSM10_TIMEOUT;      /*!< (@ 0x0000012C) RF_FSM10_TIMEOUT register               */
  __IO  uint32_t RESERVED9[5];
  __IO uint32_t SUBG_DIG_CTRL0;        /*!< (@ 0x00000144) SUBG_DIG_CTRL0 register                 */
} MR_SUBG_RADIO_TypeDef;                          /*!< Size = 328 (0x148) */



/* =========================================================================================================================== */
/* ================                                   MR_SUBG_GLOB_STATIC                                     ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Global Static
  */

typedef struct{ /*!< MR_SUBG_GLOB_STATIC Structure  */
  __IO uint32_t PCKT_CONFIG;       /*!< (@ 0x00000000) PCKT_CONFIG register              */
  __IO uint32_t SYNC;              /*!< (@ 0x00000004) SYNC register                     */
  __IO uint32_t SEC_SYNC;          /*!< (@ 0x00000008) SEC_SYNC register                 */
  __IO uint32_t CRC_INIT;          /*!< (@ 0x0000000C) CRC_INIT register                 */
  __IO uint32_t PCKT_CTRL;         /*!< (@ 0x00000010) PCKT_CTRL register                */
  __IO uint32_t DATABUFFER0_PTR;   /*!< (@ 0x00000014) DATABUFFER0_PTR register          */
  __IO uint32_t DATABUFFER1_PTR;   /*!< (@ 0x00000018) DATABUFFER1_PTR register          */
  __IO uint32_t DATABUFFER_SIZE;   /*!< (@ 0x0000001C) DATABUFFER_SIZE register          */
  __IO uint32_t PA_LEVEL_3_0;      /*!< (@ 0x00000020) PA_LEVEL_3_0 register             */
  __IO uint32_t PA_LEVEL_7_4;      /*!< (@ 0x00000024) PA_LEVEL_7_4 register             */
  __IO uint32_t PA_CONFIG;         /*!< (@ 0x00000028) PA_CONFIG register                */
  __IO uint32_t IF_CTRL;           /*!< (@ 0x0000002C) IF_CTRL register                  */
  __IO uint32_t AS_QI_CTRL;        /*!< (@ 0x00000030) AS_QI_CTRL register               */
  __IO uint32_t IQC_CONFIG;        /*!< (@ 0x00000034) IQC_CONFIG register               */
  __IO uint32_t DSSS_CTRL;         /*!< (@ 0x00000038) DSSS_CTRL register                */
} MR_SUBG_GLOB_STATIC_TypeDef;                      /*!< Size = 60 (0x3C) */


/* =========================================================================================================================== */
/* ================                                   MR_SUBG_GLOB_DYNAMIC                                    ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Global Dynamic
  */

typedef struct{ /*!< MR_SUBG_GLOB_DYNAMIC Structure  */
  __IO uint32_t PCKTLEN_CONFIG;       /*!< (@ 0x00000000) PCKTLEN_CONFIG register            */
  __IO uint32_t MOD0_CONFIG;          /*!< (@ 0x00000004) MOD0_CONFIG register               */
  __IO uint32_t MOD1_CONFIG;          /*!< (@ 0x00000008) MOD1_CONFIG register               */
  __IO uint32_t SYNTH_FREQ;           /*!< (@ 0x0000000C) SYNTH_FREQ register                */
  __IO uint32_t VCO_CAL_CONFIG;       /*!< (@ 0x00000010) VCO_CAL_CONFIG register            */
  __IO uint32_t RX_TIMER;             /*!< (@ 0x00000014) RX_TIMER register                  */
  __IO uint32_t DATABUFFER_THR;       /*!< (@ 0x00000018) DATABUFFER_THR register            */
  __IO uint32_t RFSEQ_IRQ_ENABLE;     /*!< (@ 0x0000001C) RFSEQ_IRQ_ENABLE register          */
  __IO uint32_t ADDITIONAL_CTRL;      /*!< (@ 0x00000020) ADDITIONAL_CTRL register           */
  __IO uint32_t FAST_RX_TIMER;        /*!< (@ 0x00000024) FAST_RX_TIMER register             */
  __IO uint32_t COMMAND;              /*!< (@ 0x00000028) COMMAND register                   */
} MR_SUBG_GLOB_DYNAMIC_TypeDef;                         /*!< Size = 44 (0x2C) */


/* =========================================================================================================================== */
/* ================                                   MR_SUBG_GLOB_STATUS                                     ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Global Status
  */

typedef struct{ /*!< MR_SUBG_GLOB_STATUS Structure  */
  __IO uint32_t RFSEQ_IRQ_STATUS;       /*!< (@ 0x00000000) RFSEQ_IRQ_STATUS register             */
  __IO uint32_t RFSEQ_STATUS_DETAIL;    /*!< (@ 0x00000004) RFSEQ_STATUS_DETAIL register          */
  __IO  uint32_t RADIO_FSM_INFO;         /*!< (@ 0x00000008) RADIO_FSM_INFO register               */
  __IO  uint32_t RX_INDICATOR;           /*!< (@ 0x0000000C) RX_INDICATOR register                 */
  __IO  uint32_t RX_INFO_REG;            /*!< (@ 0x00000010) RX_INFO_REG register                  */
  __IO  uint32_t RX_CRC_REG;             /*!< (@ 0x00000014) RX_CRC_REG register                   */
  __IO  uint32_t QI_INFO;                /*!< (@ 0x00000018) QI_INFO register                      */
  __IO  uint32_t DATABUFFER_INFO;        /*!< (@ 0x0000001C) DATABUFFER_INFO register              */
  __IO  uint32_t TIME_CAPTURE;           /*!< (@ 0x00000020) TIME_CAPTURE register                 */
  __IO  uint32_t IQC_CORRECTION_OUT;     /*!< (@ 0x00000024) IQC_CORRECTION_OUT register           */
  __IO  uint32_t PA_SAFEASK_OUT;         /*!< (@ 0x00000028) PA_SAFEASK_OUT register               */
  __IO  uint32_t VCO_CALIB_OUT;          /*!< (@ 0x0000002C) VCO_CALIB_OUT register                */
  __IO  uint32_t SEQ_INFO;               /*!< (@ 0x00000030) SEQ_INFO register                     */
  __IO  uint32_t SEQ_EVENT_STATUS;       /*!< (@ 0x00000034) SEQ_EVENT_STATUS register             */
} MR_SUBG_GLOB_STATUS_TypeDef;                           /*!< Size = 56 (0x38) */



/* =========================================================================================================================== */
/* ================                                   MR_SUBG_GLOB_MISC                                       ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Global Misc
  */

typedef struct{ /*!< MR_SUBG_GLOB_MISC Structure  */
  __IO  uint32_t RFIP_VERSION;       /*!< (@ 0x00000000) RFIP_VERSION register               */
  __IO  uint32_t RRM_UDRA_CTRL;      /*!< (@ 0x00000004) RRM_UDRA_CTRL register              */
  __IO uint32_t SEQUENCER_CTRL;     /*!< (@ 0x00000008) SEQUENCER_CTRL register             */
  __IO  uint32_t ABSOLUTE_TIME;      /*!< (@ 0x0000000C) ABSOLUTE_TIME register              */
  __IO  uint32_t SCM_COUNTER_VAL;    /*!< (@ 0x00000010) SCM_COUNTER_VAL register            */
  __IO uint32_t SCM_MIN_MAX;        /*!< (@ 0x00000014) SCM_MIN_MAX register                */
  __IO uint32_t WAKEUP_IRQ_STATUS;  /*!< (@ 0x00000018) WAKEUP_IRQ_STATUS register          */
} MR_SUBG_GLOB_MISC_TypeDef;                       /*!< Size = 28 (0x1C) */



/* =========================================================================================================================== */
/* ================                                   MR_SUBG_GLOB_RETAINED                                   ================ */
/* =========================================================================================================================== */


/**
  * @brief MR_SUBG Global Retained
  */

typedef struct{ /*!< MR_SUBG_GLOB_RETAINED Structure  */
  __IO  uint32_t RFIP_WAKEUPTIME;       /*!< (@ 0x00000000) RFIP_WAKEUPTIME register              */
  __IO uint32_t CPU_WAKEUPTIME;        /*!< (@ 0x00000004) CPU_WAKEUPTIME register               */
  __IO uint32_t WAKEUP_CTRL;           /*!< (@ 0x00000008) WAKEUP_CTRL register                  */
  __IO uint32_t RRM_CMDLIST_PTR;       /*!< (@ 0x0000000C) RRM_CMDLIST_PTR register              */
  __IO uint32_t SEQ_GLOBALTABLE_PTR;   /*!< (@ 0x00000010) SEQ_GLOBALTABLE_PTR register          */
} MR_SUBG_GLOB_RETAINED_TypeDef;                          /*!< Size = 20 (0x14) */



/* =========================================================================================================================== */
/* ================                                        LPAWUR                                             ================ */
/* =========================================================================================================================== */


/**
  * @brief Low Power Autonomous Wakeup Radio IP
  */

typedef struct{ /*!< LPAWUR Structure  */
  __IO uint32_t FRAME_CONFIG0;       /*!< (@ 0x00000000) FRAME_CONFIG0 register              */
  __IO uint32_t FRAME_CONFIG1;       /*!< (@ 0x00000004) FRAME_CONFIG1 register              */
  __IO uint32_t FRAME_SYNC_CONFIG;   /*!< (@ 0x00000008) FRAME_SYNC_CONFIG register          */
  __IO uint32_t RFIP_CONFIG;         /*!< (@ 0x0000000C) RFIP_CONFIG register                */
  __IO uint32_t RF_CONFIG;           /*!< (@ 0x00000010) RF_CONFIG register                  */
  __IO uint32_t AGC_CONFIG;          /*!< (@ 0x00000014) AGC_CONFIG register                 */
  __IO  uint32_t RESERVED;
  __IO  uint32_t PAYLOAD_0;           /*!< (@ 0x0000001C) PAYLOAD_0 register                  */
  __IO  uint32_t PAYLOAD_1;           /*!< (@ 0x00000020) PAYLOAD_1 register                  */
  __IO  uint32_t RESERVED1[7];
  __IO  uint32_t RFIP_VERSION;        /*!< (@ 0x00000040) RFIP_VERSION register               */
  __IO uint32_t IRQ_ENABLE;          /*!< (@ 0x00000044) IRQ_ENABLE register                 */
  __IO uint32_t STATUS;              /*!< (@ 0x00000048) STATUS register                     */
} LPAWUR_TypeDef;                        /*!< Size = 76 (0x4C) */



/** @} */ /* End of group Device_Peripheral_peripherals */


/* =========================================================================================================================== */
/* ================                          Device Specific Peripheral Address Map                           ================ */
/* =========================================================================================================================== */


/** @addtogroup Device_Peripheral_peripheralAddr
  * @{
  */
#define FLASH_BASE             (0x10040000UL) /*!< Main FLASH base address */
#define SRAM_BASE              (0x20000000UL) /*!< SRAM base address       */
#define PERIPH_BASE            (0x40000000UL) /*!< Peripheral base address */


/*!< System Memory, OTP bytes */

/* Base addresses */
#define SYSTEM_MEMORY_BASE     (0x10000000UL)   /*!< System Memory : 6KB (0x10000000 – 0x100017FF)  */
#define OTP_AREA_BASE          (0x10001800UL)   /*!< OTP area : 1kB (0x10001800 – 0x10001BFF)       */

#define SRAM0_BASE             SRAM_BASE                  /*!< SRAM0 (16 KB) base address           */
#define SRAM1_BASE            (SRAM_BASE + 0x00004000UL)  /*!< SRAM1 (16 KB) base address           */

/* End addresses */
#define SRAM0_END_ADDR         (0x20003FFFUL)   /*!< RAM0  : 16KB (0x20000000 – 0x20003FFF)         */
#define SRAM1_END_ADDR         (0x20007FFFUL)   /*!< RAM1  : 16KB (0x20004000 – 0x20007FFF)         */

#define SYSTEM_MEMORY_END_ADDR (0x100017FFUL)   /*!< System Memory : 6KB (0x10000000 – 0x100017FF)  */
#define OTP_AREA_END_ADDR      (0x10001BFFUL)   /*!< OTP area : 1KB (0x10001800 – 0x10001BFF)       */

/*!< Peripheral memory map */
#define APB0PERIPH_BASE        PERIPH_BASE
#define APB1PERIPH_BASE       (PERIPH_BASE + 0x01000000LU)
#define AHBPERIPH_BASE        (PERIPH_BASE + 0x08000000LU)
#define APB2PERIPH_BASE       (PERIPH_BASE + 0x09000000LU)


/*!< APB0 peripherals */
#define SYSCFG_BASE        (APB0PERIPH_BASE + 0x0000UL)
#define FLASH_R_BASE       (APB0PERIPH_BASE + 0x1000UL)
#define TIM2_BASE          (APB0PERIPH_BASE + 0x2000UL)
#define IWDG_BASE          (APB0PERIPH_BASE + 0x3000UL)
#define RTC_BASE           (APB0PERIPH_BASE + 0x4000UL)
#define TIM16_BASE         (APB0PERIPH_BASE + 0x5000UL)
#define DAC1_BASE           (APB0PERIPH_BASE + 0x6000UL)
#define LCD_BASE           (APB0PERIPH_BASE + 0x7000UL)
#define DBGMCU_BASE        (APB0PERIPH_BASE + 0x8000UL)
#define COMP_BASE          (APB0PERIPH_BASE + 0x9000UL)
#define LCSC_BASE          (APB0PERIPH_BASE + 0xA000UL)


/*!< APB1 peripherals */
#define I2C1_BASE          (APB1PERIPH_BASE + 0x0000UL)
#define I2C2_BASE          (APB1PERIPH_BASE + 0x1000UL)
#define SPI1_BASE          (APB1PERIPH_BASE + 0x2000UL)
#define USART1_BASE        (APB1PERIPH_BASE + 0x4000UL)
#define LPUART1_BASE       (APB1PERIPH_BASE + 0x5000UL)
#define ADC1_BASE           (APB1PERIPH_BASE + 0x6000UL)
#define SPI3_BASE          (APB1PERIPH_BASE + 0x7000UL)

/*!< AHB peripherals */
#define GPIOA_BASE                 (AHBPERIPH_BASE + 0x000000UL)
#define GPIOB_BASE                 (AHBPERIPH_BASE + 0x100000UL)
#define CRC_BASE                   (AHBPERIPH_BASE + 0x200000UL)
#define RCC_BASE                   (AHBPERIPH_BASE + 0x400000UL)
#define PWR_BASE                   (AHBPERIPH_BASE + 0x500000UL)
#define RNG_BASE                   (AHBPERIPH_BASE + 0x600000UL)
#define DMA1_BASE                  (AHBPERIPH_BASE + 0x700000UL)
#define DMAMUX1_BASE               (AHBPERIPH_BASE + 0x800000UL)
#define AES_BASE                   (AHBPERIPH_BASE + 0x900000UL)

#define DMA1_Channel1_BASE         (DMA1_BASE + 0x0008UL)
#define DMA1_Channel2_BASE         (DMA1_BASE + 0x001CUL)
#define DMA1_Channel3_BASE         (DMA1_BASE + 0x0030UL)
#define DMA1_Channel4_BASE         (DMA1_BASE + 0x0044UL)
#define DMA1_Channel5_BASE         (DMA1_BASE + 0x0058UL)
#define DMA1_Channel6_BASE         (DMA1_BASE + 0x006CUL)
#define DMA1_Channel7_BASE         (DMA1_BASE + 0x0080UL)
#define DMA1_Channel8_BASE         (DMA1_BASE + 0x0094UL)

#define DMAMUX1_Channel0_BASE      (DMAMUX1_BASE)
#define DMAMUX1_Channel1_BASE      (DMAMUX1_BASE + 0x00000004UL)
#define DMAMUX1_Channel2_BASE      (DMAMUX1_BASE + 0x00000008UL)
#define DMAMUX1_Channel3_BASE      (DMAMUX1_BASE + 0x0000000CUL)
#define DMAMUX1_Channel4_BASE      (DMAMUX1_BASE + 0x00000010UL)
#define DMAMUX1_Channel5_BASE      (DMAMUX1_BASE + 0x00000014UL)
#define DMAMUX1_Channel6_BASE      (DMAMUX1_BASE + 0x00000018UL)
#define DMAMUX1_Channel7_BASE      (DMAMUX1_BASE + 0x0000001CUL)

/*!< APB2 peripherals */
#define MR_SUBG_BASE               (APB2PERIPH_BASE + 0x0000UL)
#define MR_SUBG_RADIO_BASE         (MR_SUBG_BASE    + 0x0000UL)
#define MR_SUBG_GLOB_STATIC_BASE   (MR_SUBG_BASE    + 0x0400UL)
#define MR_SUBG_GLOB_DYNAMIC_BASE  (MR_SUBG_BASE    + 0x0500UL)
#define MR_SUBG_GLOB_STATUS_BASE   (MR_SUBG_BASE    + 0x0600UL)
#define MR_SUBG_GLOB_MISC_BASE     (MR_SUBG_BASE    + 0x0700UL)
#define MR_SUBG_GLOB_RETAINED_BASE (MR_SUBG_BASE    + 0x0780UL)
#define LPAWUR_BASE                (APB2PERIPH_BASE + 0x1000UL)


/** @} */ /* End of group Device_Peripheral_peripheralAddr */


/* =========================================================================================================================== */
/* ================                                  Peripheral declaration                                   ================ */
/* =========================================================================================================================== */


/** @addtogroup Device_Peripheral_declaration
  * @{
  */


/* Peripherals available on APB0 bus */
#define SYSCFG                      ((SYSCFG_TypeDef*)           SYSCFG_BASE)
#define FLASH                       ((FLASH_TypeDef*)           FLASH_R_BASE)
#define TIM2                        ((TIM_TypeDef*)                TIM2_BASE)
#define IWDG                        ((IWDG_TypeDef*)               IWDG_BASE)
#define RTC                         ((RTC_TypeDef*)                 RTC_BASE)
#define TIM16                       ((TIM_TypeDef*)               TIM16_BASE)
#define DAC1                        ((DAC_TypeDef*)                DAC1_BASE)
#define LCD                         ((LCD_TypeDef*)                 LCD_BASE)
#define DBGMCU                      ((DBGMCU_TypeDef*)           DBGMCU_BASE)
#define COMP1                       ((COMP_TypeDef*)               COMP_BASE)
#define LCSC                        ((LCSC_TypeDef*)               LCSC_BASE)

/* Peripherals available on APB1 bus */
#define I2C1                        ((I2C_TypeDef*)                I2C1_BASE)
#define I2C2                        ((I2C_TypeDef*)                I2C2_BASE)
#define SPI1                        ((SPI_TypeDef*)                SPI1_BASE)
#define USART1                      ((USART_TypeDef*)            USART1_BASE)
#define LPUART1                     ((USART_TypeDef*)           LPUART1_BASE)
#define ADC1                        ((ADC_TypeDef*)                ADC1_BASE)
#define SPI3                        ((SPI_TypeDef*)                SPI3_BASE)

/* Peripherals available on AHB bus */
#define GPIOA                       ((GPIO_TypeDef*)                                  GPIOA_BASE)
#define GPIOB                       ((GPIO_TypeDef*)                                  GPIOB_BASE)
#define CRC                         ((CRC_TypeDef*)                                     CRC_BASE)
#define RCC                         ((RCC_TypeDef*)                                     RCC_BASE)
#define PWR                         ((PWR_TypeDef*)                                     PWR_BASE)
#define RNG                         ((RNG_TypeDef*)                                     RNG_BASE)
#define DMA1                        ((DMA_TypeDef*)                                    DMA1_BASE)
#define DMA1_Channel1               ((DMA_Channel_TypeDef *)                  DMA1_Channel1_BASE)
#define DMA1_Channel2               ((DMA_Channel_TypeDef *)                  DMA1_Channel2_BASE)
#define DMA1_Channel3               ((DMA_Channel_TypeDef *)                  DMA1_Channel3_BASE)
#define DMA1_Channel4               ((DMA_Channel_TypeDef *)                  DMA1_Channel4_BASE)
#define DMA1_Channel5               ((DMA_Channel_TypeDef *)                  DMA1_Channel5_BASE)
#define DMA1_Channel6               ((DMA_Channel_TypeDef *)                  DMA1_Channel6_BASE)
#define DMA1_Channel7               ((DMA_Channel_TypeDef *)                  DMA1_Channel7_BASE)
#define DMA1_Channel8               ((DMA_Channel_TypeDef *)                  DMA1_Channel8_BASE)
#define DMAMUX1                     ((DMAMUX_Channel_TypeDef *)                     DMAMUX1_BASE)
#define DMAMUX1_Channel0            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel0_BASE)
#define DMAMUX1_Channel1            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel1_BASE)
#define DMAMUX1_Channel2            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel2_BASE)
#define DMAMUX1_Channel3            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel3_BASE)
#define DMAMUX1_Channel4            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel4_BASE)
#define DMAMUX1_Channel5            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel5_BASE)
#define DMAMUX1_Channel6            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel6_BASE)
#define DMAMUX1_Channel7            ((DMAMUX_Channel_TypeDef *)            DMAMUX1_Channel7_BASE)
#define AES                         ((AES_TypeDef*)                                     AES_BASE)

/* Peripherals available on APB2 bus */
#define MR_SUBG_RADIO               ((MR_SUBG_RADIO_TypeDef*)                 MR_SUBG_RADIO_BASE)
#define MR_SUBG_GLOB_STATIC         ((MR_SUBG_GLOB_STATIC_TypeDef*)     MR_SUBG_GLOB_STATIC_BASE)
#define MR_SUBG_GLOB_DYNAMIC        ((MR_SUBG_GLOB_DYNAMIC_TypeDef*)   MR_SUBG_GLOB_DYNAMIC_BASE)
#define MR_SUBG_GLOB_STATUS         ((MR_SUBG_GLOB_STATUS_TypeDef*)     MR_SUBG_GLOB_STATUS_BASE)
#define MR_SUBG_GLOB_MISC           ((MR_SUBG_GLOB_MISC_TypeDef*)         MR_SUBG_GLOB_MISC_BASE)
#define MR_SUBG_GLOB_RETAINED       ((MR_SUBG_GLOB_RETAINED_TypeDef*) MR_SUBG_GLOB_RETAINED_BASE)
#define LPAWUR                      ((LPAWUR_TypeDef*)                               LPAWUR_BASE)

/** @} */ /* End of group Device_Peripheral_declaration */

/* =========================================  End of section using anonymous unions  ========================================= */
#if defined (__CC_ARM)
  #pragma pop
#elif defined (__ICCARM__)
  /* leave anonymous unions enabled */
#elif (__ARMCC_VERSION >= 6010050)
  #pragma clang diagnostic pop
#elif defined (__GNUC__)
  /* anonymous unions are enabled by default */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by default */
#elif defined (__TASKING__)
  #pragma warning restore
#elif defined (__CSMC__)
  /* anonymous unions are enabled by default */
#endif


/* =========================================================================================================================== */
/* ================                                Pos/Mask Peripheral Section                                ================ */
/* =========================================================================================================================== */


/** @addtogroup PosMask_peripherals
  * @{
  */

/* ============================================================================================================================*/ 
/*=====================                                      SYSCFG                                      =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    DIE_ID    =====================================================*/ 
#define SYSCFG_DIE_ID_PRODUCT_Pos                                          (8UL)		/*!<SYSCFG DIE_ID: PRODUCT (Bit 8) */
#define SYSCFG_DIE_ID_PRODUCT_Msk                                          (0xf00UL)		/*!< SYSCFG DIE_ID: PRODUCT (Bitfield-Mask: 0x0f) */
#define SYSCFG_DIE_ID_PRODUCT                                              SYSCFG_DIE_ID_PRODUCT_Msk
#define SYSCFG_DIE_ID_PRODUCT_0                                            (0x1U << SYSCFG_DIE_ID_PRODUCT_Pos)
#define SYSCFG_DIE_ID_PRODUCT_1                                            (0x2U << SYSCFG_DIE_ID_PRODUCT_Pos)
#define SYSCFG_DIE_ID_PRODUCT_2                                            (0x4U << SYSCFG_DIE_ID_PRODUCT_Pos)
#define SYSCFG_DIE_ID_PRODUCT_3                                            (0x8U << SYSCFG_DIE_ID_PRODUCT_Pos)
#define SYSCFG_DIE_ID_VERSION_Pos                                          (4UL)		/*!<SYSCFG DIE_ID: VERSION (Bit 4) */
#define SYSCFG_DIE_ID_VERSION_Msk                                          (0xf0UL)		/*!< SYSCFG DIE_ID: VERSION (Bitfield-Mask: 0x0f) */
#define SYSCFG_DIE_ID_VERSION                                              SYSCFG_DIE_ID_VERSION_Msk
#define SYSCFG_DIE_ID_VERSION_0                                            (0x1U << SYSCFG_DIE_ID_VERSION_Pos)
#define SYSCFG_DIE_ID_VERSION_1                                            (0x2U << SYSCFG_DIE_ID_VERSION_Pos)
#define SYSCFG_DIE_ID_VERSION_2                                            (0x4U << SYSCFG_DIE_ID_VERSION_Pos)
#define SYSCFG_DIE_ID_VERSION_3                                            (0x8U << SYSCFG_DIE_ID_VERSION_Pos)
#define SYSCFG_DIE_ID_REVISION_Pos                                         (0UL)		/*!<SYSCFG DIE_ID: REVISION (Bit 0) */
#define SYSCFG_DIE_ID_REVISION_Msk                                         (0xfUL)		/*!< SYSCFG DIE_ID: REVISION (Bitfield-Mask: 0x0f) */
#define SYSCFG_DIE_ID_REVISION                                             SYSCFG_DIE_ID_REVISION_Msk
#define SYSCFG_DIE_ID_REVISION_0                                           (0x1U << SYSCFG_DIE_ID_REVISION_Pos)
#define SYSCFG_DIE_ID_REVISION_1                                           (0x2U << SYSCFG_DIE_ID_REVISION_Pos)
#define SYSCFG_DIE_ID_REVISION_2                                           (0x4U << SYSCFG_DIE_ID_REVISION_Pos)
#define SYSCFG_DIE_ID_REVISION_3                                           (0x8U << SYSCFG_DIE_ID_REVISION_Pos)

/* =====================================================    JTAG_ID    =====================================================*/ 
#define SYSCFG_JTAG_ID_VERSION_NUMBER_Pos                                  (28UL)		/*!<SYSCFG JTAG_ID: VERSION_NUMBER (Bit 28) */
#define SYSCFG_JTAG_ID_VERSION_NUMBER_Msk                                  (0xf0000000UL)		/*!< SYSCFG JTAG_ID: VERSION_NUMBER (Bitfield-Mask: 0x0f) */
#define SYSCFG_JTAG_ID_VERSION_NUMBER                                      SYSCFG_JTAG_ID_VERSION_NUMBER_Msk
#define SYSCFG_JTAG_ID_VERSION_NUMBER_0                                    (0x1U << SYSCFG_JTAG_ID_VERSION_NUMBER_Pos)
#define SYSCFG_JTAG_ID_VERSION_NUMBER_1                                    (0x2U << SYSCFG_JTAG_ID_VERSION_NUMBER_Pos)
#define SYSCFG_JTAG_ID_VERSION_NUMBER_2                                    (0x4U << SYSCFG_JTAG_ID_VERSION_NUMBER_Pos)
#define SYSCFG_JTAG_ID_VERSION_NUMBER_3                                    (0x8U << SYSCFG_JTAG_ID_VERSION_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_Pos                                     (12UL)		/*!<SYSCFG JTAG_ID: PART_NUMBER (Bit 12) */
#define SYSCFG_JTAG_ID_PART_NUMBER_Msk                                     (0xffff000UL)		/*!< SYSCFG JTAG_ID: PART_NUMBER (Bitfield-Mask: 0xffff) */
#define SYSCFG_JTAG_ID_PART_NUMBER                                         SYSCFG_JTAG_ID_PART_NUMBER_Msk
#define SYSCFG_JTAG_ID_PART_NUMBER_0                                       (0x1U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_1                                       (0x2U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_2                                       (0x4U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_3                                       (0x8U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_4                                       (0x10U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_5                                       (0x20U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_6                                       (0x40U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_7                                       (0x80U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_8                                       (0x100U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_9                                       (0x200U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_10                                      (0x400U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_11                                      (0x800U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_12                                      (0x1000U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_13                                      (0x2000U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_14                                      (0x4000U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_PART_NUMBER_15                                      (0x8000U << SYSCFG_JTAG_ID_PART_NUMBER_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_Pos                                        (1UL)		/*!<SYSCFG JTAG_ID: MANUF_ID (Bit 1) */
#define SYSCFG_JTAG_ID_MANUF_ID_Msk                                        (0xffeUL)		/*!< SYSCFG JTAG_ID: MANUF_ID (Bitfield-Mask: 0x7ff) */
#define SYSCFG_JTAG_ID_MANUF_ID                                            SYSCFG_JTAG_ID_MANUF_ID_Msk
#define SYSCFG_JTAG_ID_MANUF_ID_0                                          (0x1U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_1                                          (0x2U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_2                                          (0x4U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_3                                          (0x8U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_4                                          (0x10U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_5                                          (0x20U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_6                                          (0x40U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_7                                          (0x80U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_8                                          (0x100U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_9                                          (0x200U << SYSCFG_JTAG_ID_MANUF_ID_Pos)
#define SYSCFG_JTAG_ID_MANUF_ID_10                                         (0x400U << SYSCFG_JTAG_ID_MANUF_ID_Pos)

/* =====================================================    I2C_FMP_CTRL    =====================================================*/ 
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA14_FMP_Pos                              (9UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C2_PA14_FMP (Bit 9) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA14_FMP_Msk                              (0x200UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C2_PA14_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA14_FMP                                  SYSCFG_I2C_FMP_CTRL_I2C2_PA14_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA13_FMP_Pos                              (8UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C2_PA13_FMP (Bit 8) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA13_FMP_Msk                              (0x100UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C2_PA13_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA13_FMP                                  SYSCFG_I2C_FMP_CTRL_I2C2_PA13_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA7_FMP_Pos                               (7UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C2_PA7_FMP (Bit 7) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA7_FMP_Msk                               (0x80UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C2_PA7_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA7_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C2_PA7_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA6_FMP_Pos                               (6UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C2_PA6_FMP (Bit 6) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA6_FMP_Msk                               (0x40UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C2_PA6_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C2_PA6_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C2_PA6_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB11_FMP_Pos                              (5UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PB11_FMP (Bit 5) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB11_FMP_Msk                              (0x20UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PB11_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB11_FMP                                  SYSCFG_I2C_FMP_CTRL_I2C1_PB11_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB10_FMP_Pos                              (4UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PB10_FMP (Bit 4) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB10_FMP_Msk                              (0x10UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PB10_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB10_FMP                                  SYSCFG_I2C_FMP_CTRL_I2C1_PB10_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB7_FMP_Pos                               (3UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PB7_FMP (Bit 3) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB7_FMP_Msk                               (0x8UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PB7_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB7_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C1_PB7_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB6_FMP_Pos                               (2UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PB6_FMP (Bit 2) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB6_FMP_Msk                               (0x4UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PB6_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PB6_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C1_PB6_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA1_FMP_Pos                               (1UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PA1_FMP (Bit 1) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA1_FMP_Msk                               (0x2UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PA1_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA1_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C1_PA1_FMP_Msk
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA0_FMP_Pos                               (0UL)		/*!<SYSCFG I2C_FMP_CTRL: I2C1_PA0_FMP (Bit 0) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA0_FMP_Msk                               (0x1UL)		/*!< SYSCFG I2C_FMP_CTRL: I2C1_PA0_FMP (Bitfield-Mask: 0x01) */
#define SYSCFG_I2C_FMP_CTRL_I2C1_PA0_FMP                                   SYSCFG_I2C_FMP_CTRL_I2C1_PA0_FMP_Msk

/* =====================================================    IO_DTR    =====================================================*/ 
#define SYSCFG_IO_DTR_PB15_DT_Pos                                          (31UL)		/*!<SYSCFG IO_DTR: PB15_DT (Bit 31) */
#define SYSCFG_IO_DTR_PB15_DT_Msk                                          (0x80000000UL)		/*!< SYSCFG IO_DTR: PB15_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB15_DT                                              SYSCFG_IO_DTR_PB15_DT_Msk
#define SYSCFG_IO_DTR_PB14_DT_Pos                                          (30UL)		/*!<SYSCFG IO_DTR: PB14_DT (Bit 30) */
#define SYSCFG_IO_DTR_PB14_DT_Msk                                          (0x40000000UL)		/*!< SYSCFG IO_DTR: PB14_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB14_DT                                              SYSCFG_IO_DTR_PB14_DT_Msk
#define SYSCFG_IO_DTR_PB13_DT_Pos                                          (29UL)		/*!<SYSCFG IO_DTR: PB13_DT (Bit 29) */
#define SYSCFG_IO_DTR_PB13_DT_Msk                                          (0x20000000UL)		/*!< SYSCFG IO_DTR: PB13_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB13_DT                                              SYSCFG_IO_DTR_PB13_DT_Msk
#define SYSCFG_IO_DTR_PB12_DT_Pos                                          (28UL)		/*!<SYSCFG IO_DTR: PB12_DT (Bit 28) */
#define SYSCFG_IO_DTR_PB12_DT_Msk                                          (0x10000000UL)		/*!< SYSCFG IO_DTR: PB12_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB12_DT                                              SYSCFG_IO_DTR_PB12_DT_Msk
#define SYSCFG_IO_DTR_PB11_DT_Pos                                          (27UL)		/*!<SYSCFG IO_DTR: PB11_DT (Bit 27) */
#define SYSCFG_IO_DTR_PB11_DT_Msk                                          (0x8000000UL)		/*!< SYSCFG IO_DTR: PB11_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB11_DT                                              SYSCFG_IO_DTR_PB11_DT_Msk
#define SYSCFG_IO_DTR_PB10_DT_Pos                                          (26UL)		/*!<SYSCFG IO_DTR: PB10_DT (Bit 26) */
#define SYSCFG_IO_DTR_PB10_DT_Msk                                          (0x4000000UL)		/*!< SYSCFG IO_DTR: PB10_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB10_DT                                              SYSCFG_IO_DTR_PB10_DT_Msk
#define SYSCFG_IO_DTR_PB9_DT_Pos                                           (25UL)		/*!<SYSCFG IO_DTR: PB9_DT (Bit 25) */
#define SYSCFG_IO_DTR_PB9_DT_Msk                                           (0x2000000UL)		/*!< SYSCFG IO_DTR: PB9_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB9_DT                                               SYSCFG_IO_DTR_PB9_DT_Msk
#define SYSCFG_IO_DTR_PB8_DT_Pos                                           (24UL)		/*!<SYSCFG IO_DTR: PB8_DT (Bit 24) */
#define SYSCFG_IO_DTR_PB8_DT_Msk                                           (0x1000000UL)		/*!< SYSCFG IO_DTR: PB8_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB8_DT                                               SYSCFG_IO_DTR_PB8_DT_Msk
#define SYSCFG_IO_DTR_PB7_DT_Pos                                           (23UL)		/*!<SYSCFG IO_DTR: PB7_DT (Bit 23) */
#define SYSCFG_IO_DTR_PB7_DT_Msk                                           (0x800000UL)		/*!< SYSCFG IO_DTR: PB7_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB7_DT                                               SYSCFG_IO_DTR_PB7_DT_Msk
#define SYSCFG_IO_DTR_PB6_DT_Pos                                           (22UL)		/*!<SYSCFG IO_DTR: PB6_DT (Bit 22) */
#define SYSCFG_IO_DTR_PB6_DT_Msk                                           (0x400000UL)		/*!< SYSCFG IO_DTR: PB6_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB6_DT                                               SYSCFG_IO_DTR_PB6_DT_Msk
#define SYSCFG_IO_DTR_PB5_DT_Pos                                           (21UL)		/*!<SYSCFG IO_DTR: PB5_DT (Bit 21) */
#define SYSCFG_IO_DTR_PB5_DT_Msk                                           (0x200000UL)		/*!< SYSCFG IO_DTR: PB5_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB5_DT                                               SYSCFG_IO_DTR_PB5_DT_Msk
#define SYSCFG_IO_DTR_PB4_DT_Pos                                           (20UL)		/*!<SYSCFG IO_DTR: PB4_DT (Bit 20) */
#define SYSCFG_IO_DTR_PB4_DT_Msk                                           (0x100000UL)		/*!< SYSCFG IO_DTR: PB4_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB4_DT                                               SYSCFG_IO_DTR_PB4_DT_Msk
#define SYSCFG_IO_DTR_PB3_DT_Pos                                           (19UL)		/*!<SYSCFG IO_DTR: PB3_DT (Bit 19) */
#define SYSCFG_IO_DTR_PB3_DT_Msk                                           (0x80000UL)		/*!< SYSCFG IO_DTR: PB3_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB3_DT                                               SYSCFG_IO_DTR_PB3_DT_Msk
#define SYSCFG_IO_DTR_PB2_DT_Pos                                           (18UL)		/*!<SYSCFG IO_DTR: PB2_DT (Bit 18) */
#define SYSCFG_IO_DTR_PB2_DT_Msk                                           (0x40000UL)		/*!< SYSCFG IO_DTR: PB2_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB2_DT                                               SYSCFG_IO_DTR_PB2_DT_Msk
#define SYSCFG_IO_DTR_PB1_DT_Pos                                           (17UL)		/*!<SYSCFG IO_DTR: PB1_DT (Bit 17) */
#define SYSCFG_IO_DTR_PB1_DT_Msk                                           (0x20000UL)		/*!< SYSCFG IO_DTR: PB1_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB1_DT                                               SYSCFG_IO_DTR_PB1_DT_Msk
#define SYSCFG_IO_DTR_PB0_DT_Pos                                           (16UL)		/*!<SYSCFG IO_DTR: PB0_DT (Bit 16) */
#define SYSCFG_IO_DTR_PB0_DT_Msk                                           (0x10000UL)		/*!< SYSCFG IO_DTR: PB0_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PB0_DT                                               SYSCFG_IO_DTR_PB0_DT_Msk
#define SYSCFG_IO_DTR_PA15_DT_Pos                                          (15UL)		/*!<SYSCFG IO_DTR: PA15_DT (Bit 15) */
#define SYSCFG_IO_DTR_PA15_DT_Msk                                          (0x8000UL)		/*!< SYSCFG IO_DTR: PA15_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA15_DT                                              SYSCFG_IO_DTR_PA15_DT_Msk
#define SYSCFG_IO_DTR_PA14_DT_Pos                                          (14UL)		/*!<SYSCFG IO_DTR: PA14_DT (Bit 14) */
#define SYSCFG_IO_DTR_PA14_DT_Msk                                          (0x4000UL)		/*!< SYSCFG IO_DTR: PA14_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA14_DT                                              SYSCFG_IO_DTR_PA14_DT_Msk
#define SYSCFG_IO_DTR_PA13_DT_Pos                                          (13UL)		/*!<SYSCFG IO_DTR: PA13_DT (Bit 13) */
#define SYSCFG_IO_DTR_PA13_DT_Msk                                          (0x2000UL)		/*!< SYSCFG IO_DTR: PA13_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA13_DT                                              SYSCFG_IO_DTR_PA13_DT_Msk
#define SYSCFG_IO_DTR_PA12_DT_Pos                                          (12UL)		/*!<SYSCFG IO_DTR: PA12_DT (Bit 12) */
#define SYSCFG_IO_DTR_PA12_DT_Msk                                          (0x1000UL)		/*!< SYSCFG IO_DTR: PA12_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA12_DT                                              SYSCFG_IO_DTR_PA12_DT_Msk
#define SYSCFG_IO_DTR_PA11_DT_Pos                                          (11UL)		/*!<SYSCFG IO_DTR: PA11_DT (Bit 11) */
#define SYSCFG_IO_DTR_PA11_DT_Msk                                          (0x800UL)		/*!< SYSCFG IO_DTR: PA11_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA11_DT                                              SYSCFG_IO_DTR_PA11_DT_Msk
#define SYSCFG_IO_DTR_PA10_DT_Pos                                          (10UL)		/*!<SYSCFG IO_DTR: PA10_DT (Bit 10) */
#define SYSCFG_IO_DTR_PA10_DT_Msk                                          (0x400UL)		/*!< SYSCFG IO_DTR: PA10_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA10_DT                                              SYSCFG_IO_DTR_PA10_DT_Msk
#define SYSCFG_IO_DTR_PA9_DT_Pos                                           (9UL)		/*!<SYSCFG IO_DTR: PA9_DT (Bit 9) */
#define SYSCFG_IO_DTR_PA9_DT_Msk                                           (0x200UL)		/*!< SYSCFG IO_DTR: PA9_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA9_DT                                               SYSCFG_IO_DTR_PA9_DT_Msk
#define SYSCFG_IO_DTR_PA8_DT_Pos                                           (8UL)		/*!<SYSCFG IO_DTR: PA8_DT (Bit 8) */
#define SYSCFG_IO_DTR_PA8_DT_Msk                                           (0x100UL)		/*!< SYSCFG IO_DTR: PA8_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA8_DT                                               SYSCFG_IO_DTR_PA8_DT_Msk
#define SYSCFG_IO_DTR_PA7_DT_Pos                                           (7UL)		/*!<SYSCFG IO_DTR: PA7_DT (Bit 7) */
#define SYSCFG_IO_DTR_PA7_DT_Msk                                           (0x80UL)		/*!< SYSCFG IO_DTR: PA7_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA7_DT                                               SYSCFG_IO_DTR_PA7_DT_Msk
#define SYSCFG_IO_DTR_PA6_DT_Pos                                           (6UL)		/*!<SYSCFG IO_DTR: PA6_DT (Bit 6) */
#define SYSCFG_IO_DTR_PA6_DT_Msk                                           (0x40UL)		/*!< SYSCFG IO_DTR: PA6_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA6_DT                                               SYSCFG_IO_DTR_PA6_DT_Msk
#define SYSCFG_IO_DTR_PA5_DT_Pos                                           (5UL)		/*!<SYSCFG IO_DTR: PA5_DT (Bit 5) */
#define SYSCFG_IO_DTR_PA5_DT_Msk                                           (0x20UL)		/*!< SYSCFG IO_DTR: PA5_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA5_DT                                               SYSCFG_IO_DTR_PA5_DT_Msk
#define SYSCFG_IO_DTR_PA4_DT_Pos                                           (4UL)		/*!<SYSCFG IO_DTR: PA4_DT (Bit 4) */
#define SYSCFG_IO_DTR_PA4_DT_Msk                                           (0x10UL)		/*!< SYSCFG IO_DTR: PA4_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA4_DT                                               SYSCFG_IO_DTR_PA4_DT_Msk
#define SYSCFG_IO_DTR_PA3_DT_Pos                                           (3UL)		/*!<SYSCFG IO_DTR: PA3_DT (Bit 3) */
#define SYSCFG_IO_DTR_PA3_DT_Msk                                           (0x8UL)		/*!< SYSCFG IO_DTR: PA3_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA3_DT                                               SYSCFG_IO_DTR_PA3_DT_Msk
#define SYSCFG_IO_DTR_PA2_DT_Pos                                           (2UL)		/*!<SYSCFG IO_DTR: PA2_DT (Bit 2) */
#define SYSCFG_IO_DTR_PA2_DT_Msk                                           (0x4UL)		/*!< SYSCFG IO_DTR: PA2_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA2_DT                                               SYSCFG_IO_DTR_PA2_DT_Msk
#define SYSCFG_IO_DTR_PA1_DT_Pos                                           (1UL)		/*!<SYSCFG IO_DTR: PA1_DT (Bit 1) */
#define SYSCFG_IO_DTR_PA1_DT_Msk                                           (0x2UL)		/*!< SYSCFG IO_DTR: PA1_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA1_DT                                               SYSCFG_IO_DTR_PA1_DT_Msk
#define SYSCFG_IO_DTR_PA0_DT_Pos                                           (0UL)		/*!<SYSCFG IO_DTR: PA0_DT (Bit 0) */
#define SYSCFG_IO_DTR_PA0_DT_Msk                                           (0x1UL)		/*!< SYSCFG IO_DTR: PA0_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_DTR_PA0_DT                                               SYSCFG_IO_DTR_PA0_DT_Msk

/* =====================================================    IO_IBER    =====================================================*/ 
#define SYSCFG_IO_IBER_PB15_IBE_Pos                                        (31UL)		/*!<SYSCFG IO_IBER: PB15_IBE (Bit 31) */
#define SYSCFG_IO_IBER_PB15_IBE_Msk                                        (0x80000000UL)		/*!< SYSCFG IO_IBER: PB15_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB15_IBE                                            SYSCFG_IO_IBER_PB15_IBE_Msk
#define SYSCFG_IO_IBER_PB14_IBE_Pos                                        (30UL)		/*!<SYSCFG IO_IBER: PB14_IBE (Bit 30) */
#define SYSCFG_IO_IBER_PB14_IBE_Msk                                        (0x40000000UL)		/*!< SYSCFG IO_IBER: PB14_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB14_IBE                                            SYSCFG_IO_IBER_PB14_IBE_Msk
#define SYSCFG_IO_IBER_PB13_IBE_Pos                                        (29UL)		/*!<SYSCFG IO_IBER: PB13_IBE (Bit 29) */
#define SYSCFG_IO_IBER_PB13_IBE_Msk                                        (0x20000000UL)		/*!< SYSCFG IO_IBER: PB13_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB13_IBE                                            SYSCFG_IO_IBER_PB13_IBE_Msk
#define SYSCFG_IO_IBER_PB12_IBE_Pos                                        (28UL)		/*!<SYSCFG IO_IBER: PB12_IBE (Bit 28) */
#define SYSCFG_IO_IBER_PB12_IBE_Msk                                        (0x10000000UL)		/*!< SYSCFG IO_IBER: PB12_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB12_IBE                                            SYSCFG_IO_IBER_PB12_IBE_Msk
#define SYSCFG_IO_IBER_PB11_IBE_Pos                                        (27UL)		/*!<SYSCFG IO_IBER: PB11_IBE (Bit 27) */
#define SYSCFG_IO_IBER_PB11_IBE_Msk                                        (0x8000000UL)		/*!< SYSCFG IO_IBER: PB11_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB11_IBE                                            SYSCFG_IO_IBER_PB11_IBE_Msk
#define SYSCFG_IO_IBER_PB10_IBE_Pos                                        (26UL)		/*!<SYSCFG IO_IBER: PB10_IBE (Bit 26) */
#define SYSCFG_IO_IBER_PB10_IBE_Msk                                        (0x4000000UL)		/*!< SYSCFG IO_IBER: PB10_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB10_IBE                                            SYSCFG_IO_IBER_PB10_IBE_Msk
#define SYSCFG_IO_IBER_PB9_IBE_Pos                                         (25UL)		/*!<SYSCFG IO_IBER: PB9_IBE (Bit 25) */
#define SYSCFG_IO_IBER_PB9_IBE_Msk                                         (0x2000000UL)		/*!< SYSCFG IO_IBER: PB9_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB9_IBE                                             SYSCFG_IO_IBER_PB9_IBE_Msk
#define SYSCFG_IO_IBER_PB8_IBE_Pos                                         (24UL)		/*!<SYSCFG IO_IBER: PB8_IBE (Bit 24) */
#define SYSCFG_IO_IBER_PB8_IBE_Msk                                         (0x1000000UL)		/*!< SYSCFG IO_IBER: PB8_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB8_IBE                                             SYSCFG_IO_IBER_PB8_IBE_Msk
#define SYSCFG_IO_IBER_PB7_IBE_Pos                                         (23UL)		/*!<SYSCFG IO_IBER: PB7_IBE (Bit 23) */
#define SYSCFG_IO_IBER_PB7_IBE_Msk                                         (0x800000UL)		/*!< SYSCFG IO_IBER: PB7_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB7_IBE                                             SYSCFG_IO_IBER_PB7_IBE_Msk
#define SYSCFG_IO_IBER_PB6_IBE_Pos                                         (22UL)		/*!<SYSCFG IO_IBER: PB6_IBE (Bit 22) */
#define SYSCFG_IO_IBER_PB6_IBE_Msk                                         (0x400000UL)		/*!< SYSCFG IO_IBER: PB6_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB6_IBE                                             SYSCFG_IO_IBER_PB6_IBE_Msk
#define SYSCFG_IO_IBER_PB5_IBE_Pos                                         (21UL)		/*!<SYSCFG IO_IBER: PB5_IBE (Bit 21) */
#define SYSCFG_IO_IBER_PB5_IBE_Msk                                         (0x200000UL)		/*!< SYSCFG IO_IBER: PB5_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB5_IBE                                             SYSCFG_IO_IBER_PB5_IBE_Msk
#define SYSCFG_IO_IBER_PB4_IBE_Pos                                         (20UL)		/*!<SYSCFG IO_IBER: PB4_IBE (Bit 20) */
#define SYSCFG_IO_IBER_PB4_IBE_Msk                                         (0x100000UL)		/*!< SYSCFG IO_IBER: PB4_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB4_IBE                                             SYSCFG_IO_IBER_PB4_IBE_Msk
#define SYSCFG_IO_IBER_PB3_IBE_Pos                                         (19UL)		/*!<SYSCFG IO_IBER: PB3_IBE (Bit 19) */
#define SYSCFG_IO_IBER_PB3_IBE_Msk                                         (0x80000UL)		/*!< SYSCFG IO_IBER: PB3_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB3_IBE                                             SYSCFG_IO_IBER_PB3_IBE_Msk
#define SYSCFG_IO_IBER_PB2_IBE_Pos                                         (18UL)		/*!<SYSCFG IO_IBER: PB2_IBE (Bit 18) */
#define SYSCFG_IO_IBER_PB2_IBE_Msk                                         (0x40000UL)		/*!< SYSCFG IO_IBER: PB2_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB2_IBE                                             SYSCFG_IO_IBER_PB2_IBE_Msk
#define SYSCFG_IO_IBER_PB1_IBE_Pos                                         (17UL)		/*!<SYSCFG IO_IBER: PB1_IBE (Bit 17) */
#define SYSCFG_IO_IBER_PB1_IBE_Msk                                         (0x20000UL)		/*!< SYSCFG IO_IBER: PB1_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB1_IBE                                             SYSCFG_IO_IBER_PB1_IBE_Msk
#define SYSCFG_IO_IBER_PB0_IBE_Pos                                         (16UL)		/*!<SYSCFG IO_IBER: PB0_IBE (Bit 16) */
#define SYSCFG_IO_IBER_PB0_IBE_Msk                                         (0x10000UL)		/*!< SYSCFG IO_IBER: PB0_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PB0_IBE                                             SYSCFG_IO_IBER_PB0_IBE_Msk
#define SYSCFG_IO_IBER_PA15_IBE_Pos                                        (15UL)		/*!<SYSCFG IO_IBER: PA15_IBE (Bit 15) */
#define SYSCFG_IO_IBER_PA15_IBE_Msk                                        (0x8000UL)		/*!< SYSCFG IO_IBER: PA15_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA15_IBE                                            SYSCFG_IO_IBER_PA15_IBE_Msk
#define SYSCFG_IO_IBER_PA14_IBE_Pos                                        (14UL)		/*!<SYSCFG IO_IBER: PA14_IBE (Bit 14) */
#define SYSCFG_IO_IBER_PA14_IBE_Msk                                        (0x4000UL)		/*!< SYSCFG IO_IBER: PA14_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA14_IBE                                            SYSCFG_IO_IBER_PA14_IBE_Msk
#define SYSCFG_IO_IBER_PA13_IBE_Pos                                        (13UL)		/*!<SYSCFG IO_IBER: PA13_IBE (Bit 13) */
#define SYSCFG_IO_IBER_PA13_IBE_Msk                                        (0x2000UL)		/*!< SYSCFG IO_IBER: PA13_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA13_IBE                                            SYSCFG_IO_IBER_PA13_IBE_Msk
#define SYSCFG_IO_IBER_PA12_IBE_Pos                                        (12UL)		/*!<SYSCFG IO_IBER: PA12_IBE (Bit 12) */
#define SYSCFG_IO_IBER_PA12_IBE_Msk                                        (0x1000UL)		/*!< SYSCFG IO_IBER: PA12_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA12_IBE                                            SYSCFG_IO_IBER_PA12_IBE_Msk
#define SYSCFG_IO_IBER_PA11_IBE_Pos                                        (11UL)		/*!<SYSCFG IO_IBER: PA11_IBE (Bit 11) */
#define SYSCFG_IO_IBER_PA11_IBE_Msk                                        (0x800UL)		/*!< SYSCFG IO_IBER: PA11_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA11_IBE                                            SYSCFG_IO_IBER_PA11_IBE_Msk
#define SYSCFG_IO_IBER_PA10_IBE_Pos                                        (10UL)		/*!<SYSCFG IO_IBER: PA10_IBE (Bit 10) */
#define SYSCFG_IO_IBER_PA10_IBE_Msk                                        (0x400UL)		/*!< SYSCFG IO_IBER: PA10_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA10_IBE                                            SYSCFG_IO_IBER_PA10_IBE_Msk
#define SYSCFG_IO_IBER_PA9_IBE_Pos                                         (9UL)		/*!<SYSCFG IO_IBER: PA9_IBE (Bit 9) */
#define SYSCFG_IO_IBER_PA9_IBE_Msk                                         (0x200UL)		/*!< SYSCFG IO_IBER: PA9_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA9_IBE                                             SYSCFG_IO_IBER_PA9_IBE_Msk
#define SYSCFG_IO_IBER_PA8_IBE_Pos                                         (8UL)		/*!<SYSCFG IO_IBER: PA8_IBE (Bit 8) */
#define SYSCFG_IO_IBER_PA8_IBE_Msk                                         (0x100UL)		/*!< SYSCFG IO_IBER: PA8_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA8_IBE                                             SYSCFG_IO_IBER_PA8_IBE_Msk
#define SYSCFG_IO_IBER_PA7_IBE_Pos                                         (7UL)		/*!<SYSCFG IO_IBER: PA7_IBE (Bit 7) */
#define SYSCFG_IO_IBER_PA7_IBE_Msk                                         (0x80UL)		/*!< SYSCFG IO_IBER: PA7_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA7_IBE                                             SYSCFG_IO_IBER_PA7_IBE_Msk
#define SYSCFG_IO_IBER_PA6_IBE_Pos                                         (6UL)		/*!<SYSCFG IO_IBER: PA6_IBE (Bit 6) */
#define SYSCFG_IO_IBER_PA6_IBE_Msk                                         (0x40UL)		/*!< SYSCFG IO_IBER: PA6_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA6_IBE                                             SYSCFG_IO_IBER_PA6_IBE_Msk
#define SYSCFG_IO_IBER_PA5_IBE_Pos                                         (5UL)		/*!<SYSCFG IO_IBER: PA5_IBE (Bit 5) */
#define SYSCFG_IO_IBER_PA5_IBE_Msk                                         (0x20UL)		/*!< SYSCFG IO_IBER: PA5_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA5_IBE                                             SYSCFG_IO_IBER_PA5_IBE_Msk
#define SYSCFG_IO_IBER_PA4_IBE_Pos                                         (4UL)		/*!<SYSCFG IO_IBER: PA4_IBE (Bit 4) */
#define SYSCFG_IO_IBER_PA4_IBE_Msk                                         (0x10UL)		/*!< SYSCFG IO_IBER: PA4_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA4_IBE                                             SYSCFG_IO_IBER_PA4_IBE_Msk
#define SYSCFG_IO_IBER_PA3_IBE_Pos                                         (3UL)		/*!<SYSCFG IO_IBER: PA3_IBE (Bit 3) */
#define SYSCFG_IO_IBER_PA3_IBE_Msk                                         (0x8UL)		/*!< SYSCFG IO_IBER: PA3_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA3_IBE                                             SYSCFG_IO_IBER_PA3_IBE_Msk
#define SYSCFG_IO_IBER_PA2_IBE_Pos                                         (2UL)		/*!<SYSCFG IO_IBER: PA2_IBE (Bit 2) */
#define SYSCFG_IO_IBER_PA2_IBE_Msk                                         (0x4UL)		/*!< SYSCFG IO_IBER: PA2_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA2_IBE                                             SYSCFG_IO_IBER_PA2_IBE_Msk
#define SYSCFG_IO_IBER_PA1_IBE_Pos                                         (1UL)		/*!<SYSCFG IO_IBER: PA1_IBE (Bit 1) */
#define SYSCFG_IO_IBER_PA1_IBE_Msk                                         (0x2UL)		/*!< SYSCFG IO_IBER: PA1_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA1_IBE                                             SYSCFG_IO_IBER_PA1_IBE_Msk
#define SYSCFG_IO_IBER_PA0_IBE_Pos                                         (0UL)		/*!<SYSCFG IO_IBER: PA0_IBE (Bit 0) */
#define SYSCFG_IO_IBER_PA0_IBE_Msk                                         (0x1UL)		/*!< SYSCFG IO_IBER: PA0_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IBER_PA0_IBE                                             SYSCFG_IO_IBER_PA0_IBE_Msk

/* =====================================================    IO_IEVR    =====================================================*/ 
#define SYSCFG_IO_IEVR_PB15_IEV_Pos                                        (31UL)		/*!<SYSCFG IO_IEVR: PB15_IEV (Bit 31) */
#define SYSCFG_IO_IEVR_PB15_IEV_Msk                                        (0x80000000UL)		/*!< SYSCFG IO_IEVR: PB15_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB15_IEV                                            SYSCFG_IO_IEVR_PB15_IEV_Msk
#define SYSCFG_IO_IEVR_PB14_IEV_Pos                                        (30UL)		/*!<SYSCFG IO_IEVR: PB14_IEV (Bit 30) */
#define SYSCFG_IO_IEVR_PB14_IEV_Msk                                        (0x40000000UL)		/*!< SYSCFG IO_IEVR: PB14_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB14_IEV                                            SYSCFG_IO_IEVR_PB14_IEV_Msk
#define SYSCFG_IO_IEVR_PB13_IEV_Pos                                        (29UL)		/*!<SYSCFG IO_IEVR: PB13_IEV (Bit 29) */
#define SYSCFG_IO_IEVR_PB13_IEV_Msk                                        (0x20000000UL)		/*!< SYSCFG IO_IEVR: PB13_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB13_IEV                                            SYSCFG_IO_IEVR_PB13_IEV_Msk
#define SYSCFG_IO_IEVR_PB12_IEV_Pos                                        (28UL)		/*!<SYSCFG IO_IEVR: PB12_IEV (Bit 28) */
#define SYSCFG_IO_IEVR_PB12_IEV_Msk                                        (0x10000000UL)		/*!< SYSCFG IO_IEVR: PB12_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB12_IEV                                            SYSCFG_IO_IEVR_PB12_IEV_Msk
#define SYSCFG_IO_IEVR_PB11_IEV_Pos                                        (27UL)		/*!<SYSCFG IO_IEVR: PB11_IEV (Bit 27) */
#define SYSCFG_IO_IEVR_PB11_IEV_Msk                                        (0x8000000UL)		/*!< SYSCFG IO_IEVR: PB11_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB11_IEV                                            SYSCFG_IO_IEVR_PB11_IEV_Msk
#define SYSCFG_IO_IEVR_PB10_IEV_Pos                                        (26UL)		/*!<SYSCFG IO_IEVR: PB10_IEV (Bit 26) */
#define SYSCFG_IO_IEVR_PB10_IEV_Msk                                        (0x4000000UL)		/*!< SYSCFG IO_IEVR: PB10_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB10_IEV                                            SYSCFG_IO_IEVR_PB10_IEV_Msk
#define SYSCFG_IO_IEVR_PB9_IEV_Pos                                         (25UL)		/*!<SYSCFG IO_IEVR: PB9_IEV (Bit 25) */
#define SYSCFG_IO_IEVR_PB9_IEV_Msk                                         (0x2000000UL)		/*!< SYSCFG IO_IEVR: PB9_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB9_IEV                                             SYSCFG_IO_IEVR_PB9_IEV_Msk
#define SYSCFG_IO_IEVR_PB8_IEV_Pos                                         (24UL)		/*!<SYSCFG IO_IEVR: PB8_IEV (Bit 24) */
#define SYSCFG_IO_IEVR_PB8_IEV_Msk                                         (0x1000000UL)		/*!< SYSCFG IO_IEVR: PB8_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB8_IEV                                             SYSCFG_IO_IEVR_PB8_IEV_Msk
#define SYSCFG_IO_IEVR_PB7_IEV_Pos                                         (23UL)		/*!<SYSCFG IO_IEVR: PB7_IEV (Bit 23) */
#define SYSCFG_IO_IEVR_PB7_IEV_Msk                                         (0x800000UL)		/*!< SYSCFG IO_IEVR: PB7_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB7_IEV                                             SYSCFG_IO_IEVR_PB7_IEV_Msk
#define SYSCFG_IO_IEVR_PB6_IEV_Pos                                         (22UL)		/*!<SYSCFG IO_IEVR: PB6_IEV (Bit 22) */
#define SYSCFG_IO_IEVR_PB6_IEV_Msk                                         (0x400000UL)		/*!< SYSCFG IO_IEVR: PB6_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB6_IEV                                             SYSCFG_IO_IEVR_PB6_IEV_Msk
#define SYSCFG_IO_IEVR_PB5_IEV_Pos                                         (21UL)		/*!<SYSCFG IO_IEVR: PB5_IEV (Bit 21) */
#define SYSCFG_IO_IEVR_PB5_IEV_Msk                                         (0x200000UL)		/*!< SYSCFG IO_IEVR: PB5_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB5_IEV                                             SYSCFG_IO_IEVR_PB5_IEV_Msk
#define SYSCFG_IO_IEVR_PB4_IEV_Pos                                         (20UL)		/*!<SYSCFG IO_IEVR: PB4_IEV (Bit 20) */
#define SYSCFG_IO_IEVR_PB4_IEV_Msk                                         (0x100000UL)		/*!< SYSCFG IO_IEVR: PB4_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB4_IEV                                             SYSCFG_IO_IEVR_PB4_IEV_Msk
#define SYSCFG_IO_IEVR_PB3_IEV_Pos                                         (19UL)		/*!<SYSCFG IO_IEVR: PB3_IEV (Bit 19) */
#define SYSCFG_IO_IEVR_PB3_IEV_Msk                                         (0x80000UL)		/*!< SYSCFG IO_IEVR: PB3_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB3_IEV                                             SYSCFG_IO_IEVR_PB3_IEV_Msk
#define SYSCFG_IO_IEVR_PB2_IEV_Pos                                         (18UL)		/*!<SYSCFG IO_IEVR: PB2_IEV (Bit 18) */
#define SYSCFG_IO_IEVR_PB2_IEV_Msk                                         (0x40000UL)		/*!< SYSCFG IO_IEVR: PB2_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB2_IEV                                             SYSCFG_IO_IEVR_PB2_IEV_Msk
#define SYSCFG_IO_IEVR_PB1_IEV_Pos                                         (17UL)		/*!<SYSCFG IO_IEVR: PB1_IEV (Bit 17) */
#define SYSCFG_IO_IEVR_PB1_IEV_Msk                                         (0x20000UL)		/*!< SYSCFG IO_IEVR: PB1_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB1_IEV                                             SYSCFG_IO_IEVR_PB1_IEV_Msk
#define SYSCFG_IO_IEVR_PB0_IEV_Pos                                         (16UL)		/*!<SYSCFG IO_IEVR: PB0_IEV (Bit 16) */
#define SYSCFG_IO_IEVR_PB0_IEV_Msk                                         (0x10000UL)		/*!< SYSCFG IO_IEVR: PB0_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PB0_IEV                                             SYSCFG_IO_IEVR_PB0_IEV_Msk
#define SYSCFG_IO_IEVR_PA15_IEV_Pos                                        (15UL)		/*!<SYSCFG IO_IEVR: PA15_IEV (Bit 15) */
#define SYSCFG_IO_IEVR_PA15_IEV_Msk                                        (0x8000UL)		/*!< SYSCFG IO_IEVR: PA15_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA15_IEV                                            SYSCFG_IO_IEVR_PA15_IEV_Msk
#define SYSCFG_IO_IEVR_PA14_IEV_Pos                                        (14UL)		/*!<SYSCFG IO_IEVR: PA14_IEV (Bit 14) */
#define SYSCFG_IO_IEVR_PA14_IEV_Msk                                        (0x4000UL)		/*!< SYSCFG IO_IEVR: PA14_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA14_IEV                                            SYSCFG_IO_IEVR_PA14_IEV_Msk
#define SYSCFG_IO_IEVR_PA13_IEV_Pos                                        (13UL)		/*!<SYSCFG IO_IEVR: PA13_IEV (Bit 13) */
#define SYSCFG_IO_IEVR_PA13_IEV_Msk                                        (0x2000UL)		/*!< SYSCFG IO_IEVR: PA13_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA13_IEV                                            SYSCFG_IO_IEVR_PA13_IEV_Msk
#define SYSCFG_IO_IEVR_PA12_IEV_Pos                                        (12UL)		/*!<SYSCFG IO_IEVR: PA12_IEV (Bit 12) */
#define SYSCFG_IO_IEVR_PA12_IEV_Msk                                        (0x1000UL)		/*!< SYSCFG IO_IEVR: PA12_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA12_IEV                                            SYSCFG_IO_IEVR_PA12_IEV_Msk
#define SYSCFG_IO_IEVR_PA11_IEV_Pos                                        (11UL)		/*!<SYSCFG IO_IEVR: PA11_IEV (Bit 11) */
#define SYSCFG_IO_IEVR_PA11_IEV_Msk                                        (0x800UL)		/*!< SYSCFG IO_IEVR: PA11_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA11_IEV                                            SYSCFG_IO_IEVR_PA11_IEV_Msk
#define SYSCFG_IO_IEVR_PA10_IEV_Pos                                        (10UL)		/*!<SYSCFG IO_IEVR: PA10_IEV (Bit 10) */
#define SYSCFG_IO_IEVR_PA10_IEV_Msk                                        (0x400UL)		/*!< SYSCFG IO_IEVR: PA10_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA10_IEV                                            SYSCFG_IO_IEVR_PA10_IEV_Msk
#define SYSCFG_IO_IEVR_PA9_IEV_Pos                                         (9UL)		/*!<SYSCFG IO_IEVR: PA9_IEV (Bit 9) */
#define SYSCFG_IO_IEVR_PA9_IEV_Msk                                         (0x200UL)		/*!< SYSCFG IO_IEVR: PA9_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA9_IEV                                             SYSCFG_IO_IEVR_PA9_IEV_Msk
#define SYSCFG_IO_IEVR_PA8_IEV_Pos                                         (8UL)		/*!<SYSCFG IO_IEVR: PA8_IEV (Bit 8) */
#define SYSCFG_IO_IEVR_PA8_IEV_Msk                                         (0x100UL)		/*!< SYSCFG IO_IEVR: PA8_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA8_IEV                                             SYSCFG_IO_IEVR_PA8_IEV_Msk
#define SYSCFG_IO_IEVR_PA7_IEV_Pos                                         (7UL)		/*!<SYSCFG IO_IEVR: PA7_IEV (Bit 7) */
#define SYSCFG_IO_IEVR_PA7_IEV_Msk                                         (0x80UL)		/*!< SYSCFG IO_IEVR: PA7_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA7_IEV                                             SYSCFG_IO_IEVR_PA7_IEV_Msk
#define SYSCFG_IO_IEVR_PA6_IEV_Pos                                         (6UL)		/*!<SYSCFG IO_IEVR: PA6_IEV (Bit 6) */
#define SYSCFG_IO_IEVR_PA6_IEV_Msk                                         (0x40UL)		/*!< SYSCFG IO_IEVR: PA6_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA6_IEV                                             SYSCFG_IO_IEVR_PA6_IEV_Msk
#define SYSCFG_IO_IEVR_PA5_IEV_Pos                                         (5UL)		/*!<SYSCFG IO_IEVR: PA5_IEV (Bit 5) */
#define SYSCFG_IO_IEVR_PA5_IEV_Msk                                         (0x20UL)		/*!< SYSCFG IO_IEVR: PA5_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA5_IEV                                             SYSCFG_IO_IEVR_PA5_IEV_Msk
#define SYSCFG_IO_IEVR_PA4_IEV_Pos                                         (4UL)		/*!<SYSCFG IO_IEVR: PA4_IEV (Bit 4) */
#define SYSCFG_IO_IEVR_PA4_IEV_Msk                                         (0x10UL)		/*!< SYSCFG IO_IEVR: PA4_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA4_IEV                                             SYSCFG_IO_IEVR_PA4_IEV_Msk
#define SYSCFG_IO_IEVR_PA3_IEV_Pos                                         (3UL)		/*!<SYSCFG IO_IEVR: PA3_IEV (Bit 3) */
#define SYSCFG_IO_IEVR_PA3_IEV_Msk                                         (0x8UL)		/*!< SYSCFG IO_IEVR: PA3_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA3_IEV                                             SYSCFG_IO_IEVR_PA3_IEV_Msk
#define SYSCFG_IO_IEVR_PA2_IEV_Pos                                         (2UL)		/*!<SYSCFG IO_IEVR: PA2_IEV (Bit 2) */
#define SYSCFG_IO_IEVR_PA2_IEV_Msk                                         (0x4UL)		/*!< SYSCFG IO_IEVR: PA2_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA2_IEV                                             SYSCFG_IO_IEVR_PA2_IEV_Msk
#define SYSCFG_IO_IEVR_PA1_IEV_Pos                                         (1UL)		/*!<SYSCFG IO_IEVR: PA1_IEV (Bit 1) */
#define SYSCFG_IO_IEVR_PA1_IEV_Msk                                         (0x2UL)		/*!< SYSCFG IO_IEVR: PA1_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA1_IEV                                             SYSCFG_IO_IEVR_PA1_IEV_Msk
#define SYSCFG_IO_IEVR_PA0_IEV_Pos                                         (0UL)		/*!<SYSCFG IO_IEVR: PA0_IEV (Bit 0) */
#define SYSCFG_IO_IEVR_PA0_IEV_Msk                                         (0x1UL)		/*!< SYSCFG IO_IEVR: PA0_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IEVR_PA0_IEV                                             SYSCFG_IO_IEVR_PA0_IEV_Msk

/* =====================================================    IO_IER    =====================================================*/ 
#define SYSCFG_IO_IER_PB15_IE_Pos                                          (31UL)		/*!<SYSCFG IO_IER: PB15_IE (Bit 31) */
#define SYSCFG_IO_IER_PB15_IE_Msk                                          (0x80000000UL)		/*!< SYSCFG IO_IER: PB15_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB15_IE                                              SYSCFG_IO_IER_PB15_IE_Msk
#define SYSCFG_IO_IER_PB14_IE_Pos                                          (30UL)		/*!<SYSCFG IO_IER: PB14_IE (Bit 30) */
#define SYSCFG_IO_IER_PB14_IE_Msk                                          (0x40000000UL)		/*!< SYSCFG IO_IER: PB14_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB14_IE                                              SYSCFG_IO_IER_PB14_IE_Msk
#define SYSCFG_IO_IER_PB13_IE_Pos                                          (29UL)		/*!<SYSCFG IO_IER: PB13_IE (Bit 29) */
#define SYSCFG_IO_IER_PB13_IE_Msk                                          (0x20000000UL)		/*!< SYSCFG IO_IER: PB13_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB13_IE                                              SYSCFG_IO_IER_PB13_IE_Msk
#define SYSCFG_IO_IER_PB12_IE_Pos                                          (28UL)		/*!<SYSCFG IO_IER: PB12_IE (Bit 28) */
#define SYSCFG_IO_IER_PB12_IE_Msk                                          (0x10000000UL)		/*!< SYSCFG IO_IER: PB12_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB12_IE                                              SYSCFG_IO_IER_PB12_IE_Msk
#define SYSCFG_IO_IER_PB11_IE_Pos                                          (27UL)		/*!<SYSCFG IO_IER: PB11_IE (Bit 27) */
#define SYSCFG_IO_IER_PB11_IE_Msk                                          (0x8000000UL)		/*!< SYSCFG IO_IER: PB11_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB11_IE                                              SYSCFG_IO_IER_PB11_IE_Msk
#define SYSCFG_IO_IER_PB10_IE_Pos                                          (26UL)		/*!<SYSCFG IO_IER: PB10_IE (Bit 26) */
#define SYSCFG_IO_IER_PB10_IE_Msk                                          (0x4000000UL)		/*!< SYSCFG IO_IER: PB10_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB10_IE                                              SYSCFG_IO_IER_PB10_IE_Msk
#define SYSCFG_IO_IER_PB9_IE_Pos                                           (25UL)		/*!<SYSCFG IO_IER: PB9_IE (Bit 25) */
#define SYSCFG_IO_IER_PB9_IE_Msk                                           (0x2000000UL)		/*!< SYSCFG IO_IER: PB9_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB9_IE                                               SYSCFG_IO_IER_PB9_IE_Msk
#define SYSCFG_IO_IER_PB8_IE_Pos                                           (24UL)		/*!<SYSCFG IO_IER: PB8_IE (Bit 24) */
#define SYSCFG_IO_IER_PB8_IE_Msk                                           (0x1000000UL)		/*!< SYSCFG IO_IER: PB8_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB8_IE                                               SYSCFG_IO_IER_PB8_IE_Msk
#define SYSCFG_IO_IER_PB7_IE_Pos                                           (23UL)		/*!<SYSCFG IO_IER: PB7_IE (Bit 23) */
#define SYSCFG_IO_IER_PB7_IE_Msk                                           (0x800000UL)		/*!< SYSCFG IO_IER: PB7_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB7_IE                                               SYSCFG_IO_IER_PB7_IE_Msk
#define SYSCFG_IO_IER_PB6_IE_Pos                                           (22UL)		/*!<SYSCFG IO_IER: PB6_IE (Bit 22) */
#define SYSCFG_IO_IER_PB6_IE_Msk                                           (0x400000UL)		/*!< SYSCFG IO_IER: PB6_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB6_IE                                               SYSCFG_IO_IER_PB6_IE_Msk
#define SYSCFG_IO_IER_PB5_IE_Pos                                           (21UL)		/*!<SYSCFG IO_IER: PB5_IE (Bit 21) */
#define SYSCFG_IO_IER_PB5_IE_Msk                                           (0x200000UL)		/*!< SYSCFG IO_IER: PB5_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB5_IE                                               SYSCFG_IO_IER_PB5_IE_Msk
#define SYSCFG_IO_IER_PB4_IE_Pos                                           (20UL)		/*!<SYSCFG IO_IER: PB4_IE (Bit 20) */
#define SYSCFG_IO_IER_PB4_IE_Msk                                           (0x100000UL)		/*!< SYSCFG IO_IER: PB4_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB4_IE                                               SYSCFG_IO_IER_PB4_IE_Msk
#define SYSCFG_IO_IER_PB3_IE_Pos                                           (19UL)		/*!<SYSCFG IO_IER: PB3_IE (Bit 19) */
#define SYSCFG_IO_IER_PB3_IE_Msk                                           (0x80000UL)		/*!< SYSCFG IO_IER: PB3_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB3_IE                                               SYSCFG_IO_IER_PB3_IE_Msk
#define SYSCFG_IO_IER_PB2_IE_Pos                                           (18UL)		/*!<SYSCFG IO_IER: PB2_IE (Bit 18) */
#define SYSCFG_IO_IER_PB2_IE_Msk                                           (0x40000UL)		/*!< SYSCFG IO_IER: PB2_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB2_IE                                               SYSCFG_IO_IER_PB2_IE_Msk
#define SYSCFG_IO_IER_PB1_IE_Pos                                           (17UL)		/*!<SYSCFG IO_IER: PB1_IE (Bit 17) */
#define SYSCFG_IO_IER_PB1_IE_Msk                                           (0x20000UL)		/*!< SYSCFG IO_IER: PB1_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB1_IE                                               SYSCFG_IO_IER_PB1_IE_Msk
#define SYSCFG_IO_IER_PB0_IE_Pos                                           (16UL)		/*!<SYSCFG IO_IER: PB0_IE (Bit 16) */
#define SYSCFG_IO_IER_PB0_IE_Msk                                           (0x10000UL)		/*!< SYSCFG IO_IER: PB0_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PB0_IE                                               SYSCFG_IO_IER_PB0_IE_Msk
#define SYSCFG_IO_IER_PA15_IE_Pos                                          (15UL)		/*!<SYSCFG IO_IER: PA15_IE (Bit 15) */
#define SYSCFG_IO_IER_PA15_IE_Msk                                          (0x8000UL)		/*!< SYSCFG IO_IER: PA15_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA15_IE                                              SYSCFG_IO_IER_PA15_IE_Msk
#define SYSCFG_IO_IER_PA14_IE_Pos                                          (14UL)		/*!<SYSCFG IO_IER: PA14_IE (Bit 14) */
#define SYSCFG_IO_IER_PA14_IE_Msk                                          (0x4000UL)		/*!< SYSCFG IO_IER: PA14_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA14_IE                                              SYSCFG_IO_IER_PA14_IE_Msk
#define SYSCFG_IO_IER_PA13_IE_Pos                                          (13UL)		/*!<SYSCFG IO_IER: PA13_IE (Bit 13) */
#define SYSCFG_IO_IER_PA13_IE_Msk                                          (0x2000UL)		/*!< SYSCFG IO_IER: PA13_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA13_IE                                              SYSCFG_IO_IER_PA13_IE_Msk
#define SYSCFG_IO_IER_PA12_IE_Pos                                          (12UL)		/*!<SYSCFG IO_IER: PA12_IE (Bit 12) */
#define SYSCFG_IO_IER_PA12_IE_Msk                                          (0x1000UL)		/*!< SYSCFG IO_IER: PA12_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA12_IE                                              SYSCFG_IO_IER_PA12_IE_Msk
#define SYSCFG_IO_IER_PA11_IE_Pos                                          (11UL)		/*!<SYSCFG IO_IER: PA11_IE (Bit 11) */
#define SYSCFG_IO_IER_PA11_IE_Msk                                          (0x800UL)		/*!< SYSCFG IO_IER: PA11_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA11_IE                                              SYSCFG_IO_IER_PA11_IE_Msk
#define SYSCFG_IO_IER_PA10_IE_Pos                                          (10UL)		/*!<SYSCFG IO_IER: PA10_IE (Bit 10) */
#define SYSCFG_IO_IER_PA10_IE_Msk                                          (0x400UL)		/*!< SYSCFG IO_IER: PA10_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA10_IE                                              SYSCFG_IO_IER_PA10_IE_Msk
#define SYSCFG_IO_IER_PA9_IE_Pos                                           (9UL)		/*!<SYSCFG IO_IER: PA9_IE (Bit 9) */
#define SYSCFG_IO_IER_PA9_IE_Msk                                           (0x200UL)		/*!< SYSCFG IO_IER: PA9_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA9_IE                                               SYSCFG_IO_IER_PA9_IE_Msk
#define SYSCFG_IO_IER_PA8_IE_Pos                                           (8UL)		/*!<SYSCFG IO_IER: PA8_IE (Bit 8) */
#define SYSCFG_IO_IER_PA8_IE_Msk                                           (0x100UL)		/*!< SYSCFG IO_IER: PA8_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA8_IE                                               SYSCFG_IO_IER_PA8_IE_Msk
#define SYSCFG_IO_IER_PA7_IE_Pos                                           (7UL)		/*!<SYSCFG IO_IER: PA7_IE (Bit 7) */
#define SYSCFG_IO_IER_PA7_IE_Msk                                           (0x80UL)		/*!< SYSCFG IO_IER: PA7_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA7_IE                                               SYSCFG_IO_IER_PA7_IE_Msk
#define SYSCFG_IO_IER_PA6_IE_Pos                                           (6UL)		/*!<SYSCFG IO_IER: PA6_IE (Bit 6) */
#define SYSCFG_IO_IER_PA6_IE_Msk                                           (0x40UL)		/*!< SYSCFG IO_IER: PA6_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA6_IE                                               SYSCFG_IO_IER_PA6_IE_Msk
#define SYSCFG_IO_IER_PA5_IE_Pos                                           (5UL)		/*!<SYSCFG IO_IER: PA5_IE (Bit 5) */
#define SYSCFG_IO_IER_PA5_IE_Msk                                           (0x20UL)		/*!< SYSCFG IO_IER: PA5_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA5_IE                                               SYSCFG_IO_IER_PA5_IE_Msk
#define SYSCFG_IO_IER_PA4_IE_Pos                                           (4UL)		/*!<SYSCFG IO_IER: PA4_IE (Bit 4) */
#define SYSCFG_IO_IER_PA4_IE_Msk                                           (0x10UL)		/*!< SYSCFG IO_IER: PA4_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA4_IE                                               SYSCFG_IO_IER_PA4_IE_Msk
#define SYSCFG_IO_IER_PA3_IE_Pos                                           (3UL)		/*!<SYSCFG IO_IER: PA3_IE (Bit 3) */
#define SYSCFG_IO_IER_PA3_IE_Msk                                           (0x8UL)		/*!< SYSCFG IO_IER: PA3_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA3_IE                                               SYSCFG_IO_IER_PA3_IE_Msk
#define SYSCFG_IO_IER_PA2_IE_Pos                                           (2UL)		/*!<SYSCFG IO_IER: PA2_IE (Bit 2) */
#define SYSCFG_IO_IER_PA2_IE_Msk                                           (0x4UL)		/*!< SYSCFG IO_IER: PA2_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA2_IE                                               SYSCFG_IO_IER_PA2_IE_Msk
#define SYSCFG_IO_IER_PA1_IE_Pos                                           (1UL)		/*!<SYSCFG IO_IER: PA1_IE (Bit 1) */
#define SYSCFG_IO_IER_PA1_IE_Msk                                           (0x2UL)		/*!< SYSCFG IO_IER: PA1_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA1_IE                                               SYSCFG_IO_IER_PA1_IE_Msk
#define SYSCFG_IO_IER_PA0_IE_Pos                                           (0UL)		/*!<SYSCFG IO_IER: PA0_IE (Bit 0) */
#define SYSCFG_IO_IER_PA0_IE_Msk                                           (0x1UL)		/*!< SYSCFG IO_IER: PA0_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_IER_PA0_IE                                               SYSCFG_IO_IER_PA0_IE_Msk

/* =====================================================    IO_ISCR    =====================================================*/ 
#define SYSCFG_IO_ISCR_PB15_ISC_Pos                                        (31UL)		/*!<SYSCFG IO_ISCR: PB15_ISC (Bit 31) */
#define SYSCFG_IO_ISCR_PB15_ISC_Msk                                        (0x80000000UL)		/*!< SYSCFG IO_ISCR: PB15_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB15_ISC                                            SYSCFG_IO_ISCR_PB15_ISC_Msk
#define SYSCFG_IO_ISCR_PB14_ISC_Pos                                        (30UL)		/*!<SYSCFG IO_ISCR: PB14_ISC (Bit 30) */
#define SYSCFG_IO_ISCR_PB14_ISC_Msk                                        (0x40000000UL)		/*!< SYSCFG IO_ISCR: PB14_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB14_ISC                                            SYSCFG_IO_ISCR_PB14_ISC_Msk
#define SYSCFG_IO_ISCR_PB13_ISC_Pos                                        (29UL)		/*!<SYSCFG IO_ISCR: PB13_ISC (Bit 29) */
#define SYSCFG_IO_ISCR_PB13_ISC_Msk                                        (0x20000000UL)		/*!< SYSCFG IO_ISCR: PB13_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB13_ISC                                            SYSCFG_IO_ISCR_PB13_ISC_Msk
#define SYSCFG_IO_ISCR_PB12_ISC_Pos                                        (28UL)		/*!<SYSCFG IO_ISCR: PB12_ISC (Bit 28) */
#define SYSCFG_IO_ISCR_PB12_ISC_Msk                                        (0x10000000UL)		/*!< SYSCFG IO_ISCR: PB12_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB12_ISC                                            SYSCFG_IO_ISCR_PB12_ISC_Msk
#define SYSCFG_IO_ISCR_PB11_ISC_Pos                                        (27UL)		/*!<SYSCFG IO_ISCR: PB11_ISC (Bit 27) */
#define SYSCFG_IO_ISCR_PB11_ISC_Msk                                        (0x8000000UL)		/*!< SYSCFG IO_ISCR: PB11_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB11_ISC                                            SYSCFG_IO_ISCR_PB11_ISC_Msk
#define SYSCFG_IO_ISCR_PB10_ISC_Pos                                        (26UL)		/*!<SYSCFG IO_ISCR: PB10_ISC (Bit 26) */
#define SYSCFG_IO_ISCR_PB10_ISC_Msk                                        (0x4000000UL)		/*!< SYSCFG IO_ISCR: PB10_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB10_ISC                                            SYSCFG_IO_ISCR_PB10_ISC_Msk
#define SYSCFG_IO_ISCR_PB9_ISC_Pos                                         (25UL)		/*!<SYSCFG IO_ISCR: PB9_ISC (Bit 25) */
#define SYSCFG_IO_ISCR_PB9_ISC_Msk                                         (0x2000000UL)		/*!< SYSCFG IO_ISCR: PB9_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB9_ISC                                             SYSCFG_IO_ISCR_PB9_ISC_Msk
#define SYSCFG_IO_ISCR_PB8_ISC_Pos                                         (24UL)		/*!<SYSCFG IO_ISCR: PB8_ISC (Bit 24) */
#define SYSCFG_IO_ISCR_PB8_ISC_Msk                                         (0x1000000UL)		/*!< SYSCFG IO_ISCR: PB8_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB8_ISC                                             SYSCFG_IO_ISCR_PB8_ISC_Msk
#define SYSCFG_IO_ISCR_PB7_ISC_Pos                                         (23UL)		/*!<SYSCFG IO_ISCR: PB7_ISC (Bit 23) */
#define SYSCFG_IO_ISCR_PB7_ISC_Msk                                         (0x800000UL)		/*!< SYSCFG IO_ISCR: PB7_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB7_ISC                                             SYSCFG_IO_ISCR_PB7_ISC_Msk
#define SYSCFG_IO_ISCR_PB6_ISC_Pos                                         (22UL)		/*!<SYSCFG IO_ISCR: PB6_ISC (Bit 22) */
#define SYSCFG_IO_ISCR_PB6_ISC_Msk                                         (0x400000UL)		/*!< SYSCFG IO_ISCR: PB6_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB6_ISC                                             SYSCFG_IO_ISCR_PB6_ISC_Msk
#define SYSCFG_IO_ISCR_PB5_ISC_Pos                                         (21UL)		/*!<SYSCFG IO_ISCR: PB5_ISC (Bit 21) */
#define SYSCFG_IO_ISCR_PB5_ISC_Msk                                         (0x200000UL)		/*!< SYSCFG IO_ISCR: PB5_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB5_ISC                                             SYSCFG_IO_ISCR_PB5_ISC_Msk
#define SYSCFG_IO_ISCR_PB4_ISC_Pos                                         (20UL)		/*!<SYSCFG IO_ISCR: PB4_ISC (Bit 20) */
#define SYSCFG_IO_ISCR_PB4_ISC_Msk                                         (0x100000UL)		/*!< SYSCFG IO_ISCR: PB4_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB4_ISC                                             SYSCFG_IO_ISCR_PB4_ISC_Msk
#define SYSCFG_IO_ISCR_PB3_ISC_Pos                                         (19UL)		/*!<SYSCFG IO_ISCR: PB3_ISC (Bit 19) */
#define SYSCFG_IO_ISCR_PB3_ISC_Msk                                         (0x80000UL)		/*!< SYSCFG IO_ISCR: PB3_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB3_ISC                                             SYSCFG_IO_ISCR_PB3_ISC_Msk
#define SYSCFG_IO_ISCR_PB2_ISC_Pos                                         (18UL)		/*!<SYSCFG IO_ISCR: PB2_ISC (Bit 18) */
#define SYSCFG_IO_ISCR_PB2_ISC_Msk                                         (0x40000UL)		/*!< SYSCFG IO_ISCR: PB2_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB2_ISC                                             SYSCFG_IO_ISCR_PB2_ISC_Msk
#define SYSCFG_IO_ISCR_PB1_ISC_Pos                                         (17UL)		/*!<SYSCFG IO_ISCR: PB1_ISC (Bit 17) */
#define SYSCFG_IO_ISCR_PB1_ISC_Msk                                         (0x20000UL)		/*!< SYSCFG IO_ISCR: PB1_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB1_ISC                                             SYSCFG_IO_ISCR_PB1_ISC_Msk
#define SYSCFG_IO_ISCR_PB0_ISC_Pos                                         (16UL)		/*!<SYSCFG IO_ISCR: PB0_ISC (Bit 16) */
#define SYSCFG_IO_ISCR_PB0_ISC_Msk                                         (0x10000UL)		/*!< SYSCFG IO_ISCR: PB0_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PB0_ISC                                             SYSCFG_IO_ISCR_PB0_ISC_Msk
#define SYSCFG_IO_ISCR_PA15_ISC_Pos                                        (15UL)		/*!<SYSCFG IO_ISCR: PA15_ISC (Bit 15) */
#define SYSCFG_IO_ISCR_PA15_ISC_Msk                                        (0x8000UL)		/*!< SYSCFG IO_ISCR: PA15_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA15_ISC                                            SYSCFG_IO_ISCR_PA15_ISC_Msk
#define SYSCFG_IO_ISCR_PA14_ISC_Pos                                        (14UL)		/*!<SYSCFG IO_ISCR: PA14_ISC (Bit 14) */
#define SYSCFG_IO_ISCR_PA14_ISC_Msk                                        (0x4000UL)		/*!< SYSCFG IO_ISCR: PA14_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA14_ISC                                            SYSCFG_IO_ISCR_PA14_ISC_Msk
#define SYSCFG_IO_ISCR_PA13_ISC_Pos                                        (13UL)		/*!<SYSCFG IO_ISCR: PA13_ISC (Bit 13) */
#define SYSCFG_IO_ISCR_PA13_ISC_Msk                                        (0x2000UL)		/*!< SYSCFG IO_ISCR: PA13_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA13_ISC                                            SYSCFG_IO_ISCR_PA13_ISC_Msk
#define SYSCFG_IO_ISCR_PA12_ISC_Pos                                        (12UL)		/*!<SYSCFG IO_ISCR: PA12_ISC (Bit 12) */
#define SYSCFG_IO_ISCR_PA12_ISC_Msk                                        (0x1000UL)		/*!< SYSCFG IO_ISCR: PA12_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA12_ISC                                            SYSCFG_IO_ISCR_PA12_ISC_Msk
#define SYSCFG_IO_ISCR_PA11_ISC_Pos                                        (11UL)		/*!<SYSCFG IO_ISCR: PA11_ISC (Bit 11) */
#define SYSCFG_IO_ISCR_PA11_ISC_Msk                                        (0x800UL)		/*!< SYSCFG IO_ISCR: PA11_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA11_ISC                                            SYSCFG_IO_ISCR_PA11_ISC_Msk
#define SYSCFG_IO_ISCR_PA10_ISC_Pos                                        (10UL)		/*!<SYSCFG IO_ISCR: PA10_ISC (Bit 10) */
#define SYSCFG_IO_ISCR_PA10_ISC_Msk                                        (0x400UL)		/*!< SYSCFG IO_ISCR: PA10_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA10_ISC                                            SYSCFG_IO_ISCR_PA10_ISC_Msk
#define SYSCFG_IO_ISCR_PA9_ISC_Pos                                         (9UL)		/*!<SYSCFG IO_ISCR: PA9_ISC (Bit 9) */
#define SYSCFG_IO_ISCR_PA9_ISC_Msk                                         (0x200UL)		/*!< SYSCFG IO_ISCR: PA9_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA9_ISC                                             SYSCFG_IO_ISCR_PA9_ISC_Msk
#define SYSCFG_IO_ISCR_PA8_ISC_Pos                                         (8UL)		/*!<SYSCFG IO_ISCR: PA8_ISC (Bit 8) */
#define SYSCFG_IO_ISCR_PA8_ISC_Msk                                         (0x100UL)		/*!< SYSCFG IO_ISCR: PA8_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA8_ISC                                             SYSCFG_IO_ISCR_PA8_ISC_Msk
#define SYSCFG_IO_ISCR_PA7_ISC_Pos                                         (7UL)		/*!<SYSCFG IO_ISCR: PA7_ISC (Bit 7) */
#define SYSCFG_IO_ISCR_PA7_ISC_Msk                                         (0x80UL)		/*!< SYSCFG IO_ISCR: PA7_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA7_ISC                                             SYSCFG_IO_ISCR_PA7_ISC_Msk
#define SYSCFG_IO_ISCR_PA6_ISC_Pos                                         (6UL)		/*!<SYSCFG IO_ISCR: PA6_ISC (Bit 6) */
#define SYSCFG_IO_ISCR_PA6_ISC_Msk                                         (0x40UL)		/*!< SYSCFG IO_ISCR: PA6_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA6_ISC                                             SYSCFG_IO_ISCR_PA6_ISC_Msk
#define SYSCFG_IO_ISCR_PA5_ISC_Pos                                         (5UL)		/*!<SYSCFG IO_ISCR: PA5_ISC (Bit 5) */
#define SYSCFG_IO_ISCR_PA5_ISC_Msk                                         (0x20UL)		/*!< SYSCFG IO_ISCR: PA5_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA5_ISC                                             SYSCFG_IO_ISCR_PA5_ISC_Msk
#define SYSCFG_IO_ISCR_PA4_ISC_Pos                                         (4UL)		/*!<SYSCFG IO_ISCR: PA4_ISC (Bit 4) */
#define SYSCFG_IO_ISCR_PA4_ISC_Msk                                         (0x10UL)		/*!< SYSCFG IO_ISCR: PA4_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA4_ISC                                             SYSCFG_IO_ISCR_PA4_ISC_Msk
#define SYSCFG_IO_ISCR_PA3_ISC_Pos                                         (3UL)		/*!<SYSCFG IO_ISCR: PA3_ISC (Bit 3) */
#define SYSCFG_IO_ISCR_PA3_ISC_Msk                                         (0x8UL)		/*!< SYSCFG IO_ISCR: PA3_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA3_ISC                                             SYSCFG_IO_ISCR_PA3_ISC_Msk
#define SYSCFG_IO_ISCR_PA2_ISC_Pos                                         (2UL)		/*!<SYSCFG IO_ISCR: PA2_ISC (Bit 2) */
#define SYSCFG_IO_ISCR_PA2_ISC_Msk                                         (0x4UL)		/*!< SYSCFG IO_ISCR: PA2_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA2_ISC                                             SYSCFG_IO_ISCR_PA2_ISC_Msk
#define SYSCFG_IO_ISCR_PA1_ISC_Pos                                         (1UL)		/*!<SYSCFG IO_ISCR: PA1_ISC (Bit 1) */
#define SYSCFG_IO_ISCR_PA1_ISC_Msk                                         (0x2UL)		/*!< SYSCFG IO_ISCR: PA1_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA1_ISC                                             SYSCFG_IO_ISCR_PA1_ISC_Msk
#define SYSCFG_IO_ISCR_PA0_ISC_Pos                                         (0UL)		/*!<SYSCFG IO_ISCR: PA0_ISC (Bit 0) */
#define SYSCFG_IO_ISCR_PA0_ISC_Msk                                         (0x1UL)		/*!< SYSCFG IO_ISCR: PA0_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_IO_ISCR_PA0_ISC                                             SYSCFG_IO_ISCR_PA0_ISC_Msk

/* =====================================================    PWRC_IER    =====================================================*/ 
#define SYSCFG_PWRC_IER_WKUP_IE_Pos                                        (2UL)		/*!<SYSCFG PWRC_IER: WKUP_IE (Bit 2) */
#define SYSCFG_PWRC_IER_WKUP_IE_Msk                                        (0x4UL)		/*!< SYSCFG PWRC_IER: WKUP_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_IER_WKUP_IE                                            SYSCFG_PWRC_IER_WKUP_IE_Msk
#define SYSCFG_PWRC_IER_PVD_IE_Pos                                         (1UL)		/*!<SYSCFG PWRC_IER: PVD_IE (Bit 1) */
#define SYSCFG_PWRC_IER_PVD_IE_Msk                                         (0x2UL)		/*!< SYSCFG PWRC_IER: PVD_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_IER_PVD_IE                                             SYSCFG_PWRC_IER_PVD_IE_Msk
#define SYSCFG_PWRC_IER_BORH_IE_Pos                                        (0UL)		/*!<SYSCFG PWRC_IER: BORH_IE (Bit 0) */
#define SYSCFG_PWRC_IER_BORH_IE_Msk                                        (0x1UL)		/*!< SYSCFG PWRC_IER: BORH_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_IER_BORH_IE                                            SYSCFG_PWRC_IER_BORH_IE_Msk

/* =====================================================    PWRC_ISCR    =====================================================*/ 
#define SYSCFG_PWRC_ISCR_WKUP_ISC_Pos                                      (2UL)		/*!<SYSCFG PWRC_ISCR: WKUP_ISC (Bit 2) */
#define SYSCFG_PWRC_ISCR_WKUP_ISC_Msk                                      (0x4UL)		/*!< SYSCFG PWRC_ISCR: WKUP_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_ISCR_WKUP_ISC                                          SYSCFG_PWRC_ISCR_WKUP_ISC_Msk
#define SYSCFG_PWRC_ISCR_PVD_ISC_Pos                                       (1UL)		/*!<SYSCFG PWRC_ISCR: PVD_ISC (Bit 1) */
#define SYSCFG_PWRC_ISCR_PVD_ISC_Msk                                       (0x2UL)		/*!< SYSCFG PWRC_ISCR: PVD_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_ISCR_PVD_ISC                                           SYSCFG_PWRC_ISCR_PVD_ISC_Msk
#define SYSCFG_PWRC_ISCR_BORH_ISC_Pos                                      (0UL)		/*!<SYSCFG PWRC_ISCR: BORH_ISC (Bit 0) */
#define SYSCFG_PWRC_ISCR_BORH_ISC_Msk                                      (0x1UL)		/*!< SYSCFG PWRC_ISCR: BORH_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_PWRC_ISCR_BORH_ISC                                          SYSCFG_PWRC_ISCR_BORH_ISC_Msk

/* =====================================================    GPIO_SWA_CTRL    =====================================================*/ 
#define SYSCFG_GPIO_SWA_CTRL_ATB1_nPVD_Pos                                 (0UL)		/*!<SYSCFG GPIO_SWA_CTRL: ATB1_nPVD (Bit 0) */
#define SYSCFG_GPIO_SWA_CTRL_ATB1_nPVD_Msk                                 (0x1UL)		/*!< SYSCFG GPIO_SWA_CTRL: ATB1_nPVD (Bitfield-Mask: 0x01) */
#define SYSCFG_GPIO_SWA_CTRL_ATB1_nPVD                                     SYSCFG_GPIO_SWA_CTRL_ATB1_nPVD_Msk

/* =====================================================    INTAI_DTR    =====================================================*/ 
#define SYSCFG_INTAI_DTR_LC_ACTIVITY_DT_Pos                                (6UL)		/*!<SYSCFG INTAI_DTR: LC_ACTIVITY_DT (Bit 6) */
#define SYSCFG_INTAI_DTR_LC_ACTIVITY_DT_Msk                                (0x40UL)		/*!< SYSCFG INTAI_DTR: LC_ACTIVITY_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_DTR_LC_ACTIVITY_DT                                    SYSCFG_INTAI_DTR_LC_ACTIVITY_DT_Msk
#define SYSCFG_INTAI_DTR_RFIP_BUSY_STATUS_DT_Pos                           (5UL)		/*!<SYSCFG INTAI_DTR: RFIP_BUSY_STATUS_DT (Bit 5) */
#define SYSCFG_INTAI_DTR_RFIP_BUSY_STATUS_DT_Msk                           (0x20UL)		/*!< SYSCFG INTAI_DTR: RFIP_BUSY_STATUS_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_DTR_RFIP_BUSY_STATUS_DT                               SYSCFG_INTAI_DTR_RFIP_BUSY_STATUS_DT_Msk
#define SYSCFG_INTAI_DTR_COMP_DT_Pos                                       (4UL)		/*!<SYSCFG INTAI_DTR: COMP_DT (Bit 4) */
#define SYSCFG_INTAI_DTR_COMP_DT_Msk                                       (0x10UL)		/*!< SYSCFG INTAI_DTR: COMP_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_DTR_COMP_DT                                           SYSCFG_INTAI_DTR_COMP_DT_Msk
#define SYSCFG_INTAI_DTR_RX_DT_Pos                                         (1UL)		/*!<SYSCFG INTAI_DTR: RX_DT (Bit 1) */
#define SYSCFG_INTAI_DTR_RX_DT_Msk                                         (0x2UL)		/*!< SYSCFG INTAI_DTR: RX_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_DTR_RX_DT                                             SYSCFG_INTAI_DTR_RX_DT_Msk
#define SYSCFG_INTAI_DTR_TX_DT_Pos                                         (0UL)		/*!<SYSCFG INTAI_DTR: TX_DT (Bit 0) */
#define SYSCFG_INTAI_DTR_TX_DT_Msk                                         (0x1UL)		/*!< SYSCFG INTAI_DTR: TX_DT (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_DTR_TX_DT                                             SYSCFG_INTAI_DTR_TX_DT_Msk

/* =====================================================    INTAI_IBER    =====================================================*/ 
#define SYSCFG_INTAI_IBER_LC_ACTIVITY_IBE_Pos                              (6UL)		/*!<SYSCFG INTAI_IBER: LC_ACTIVITY_IBE (Bit 6) */
#define SYSCFG_INTAI_IBER_LC_ACTIVITY_IBE_Msk                              (0x40UL)		/*!< SYSCFG INTAI_IBER: LC_ACTIVITY_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IBER_LC_ACTIVITY_IBE                                  SYSCFG_INTAI_IBER_LC_ACTIVITY_IBE_Msk
#define SYSCFG_INTAI_IBER_RFIP_BUSY_STATUS_IBE_Pos                         (5UL)		/*!<SYSCFG INTAI_IBER: RFIP_BUSY_STATUS_IBE (Bit 5) */
#define SYSCFG_INTAI_IBER_RFIP_BUSY_STATUS_IBE_Msk                         (0x20UL)		/*!< SYSCFG INTAI_IBER: RFIP_BUSY_STATUS_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IBER_RFIP_BUSY_STATUS_IBE                             SYSCFG_INTAI_IBER_RFIP_BUSY_STATUS_IBE_Msk
#define SYSCFG_INTAI_IBER_COMP_IBE_Pos                                     (4UL)		/*!<SYSCFG INTAI_IBER: COMP_IBE (Bit 4) */
#define SYSCFG_INTAI_IBER_COMP_IBE_Msk                                     (0x10UL)		/*!< SYSCFG INTAI_IBER: COMP_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IBER_COMP_IBE                                         SYSCFG_INTAI_IBER_COMP_IBE_Msk
#define SYSCFG_INTAI_IBER_RX_IBE_Pos                                       (1UL)		/*!<SYSCFG INTAI_IBER: RX_IBE (Bit 1) */
#define SYSCFG_INTAI_IBER_RX_IBE_Msk                                       (0x2UL)		/*!< SYSCFG INTAI_IBER: RX_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IBER_RX_IBE                                           SYSCFG_INTAI_IBER_RX_IBE_Msk
#define SYSCFG_INTAI_IBER_TX_IBE_Pos                                       (0UL)		/*!<SYSCFG INTAI_IBER: TX_IBE (Bit 0) */
#define SYSCFG_INTAI_IBER_TX_IBE_Msk                                       (0x1UL)		/*!< SYSCFG INTAI_IBER: TX_IBE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IBER_TX_IBE                                           SYSCFG_INTAI_IBER_TX_IBE_Msk

/* =====================================================    INTAI_IEVR    =====================================================*/ 
#define SYSCFG_INTAI_IEVR_LC_ACTIVITY_IEV_Pos                              (6UL)		/*!<SYSCFG INTAI_IEVR: LC_ACTIVITY_IEV (Bit 6) */
#define SYSCFG_INTAI_IEVR_LC_ACTIVITY_IEV_Msk                              (0x40UL)		/*!< SYSCFG INTAI_IEVR: LC_ACTIVITY_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IEVR_LC_ACTIVITY_IEV                                  SYSCFG_INTAI_IEVR_LC_ACTIVITY_IEV_Msk
#define SYSCFG_INTAI_IEVR_RFIP_BUSY_STATUS_IEV_Pos                         (5UL)		/*!<SYSCFG INTAI_IEVR: RFIP_BUSY_STATUS_IEV (Bit 5) */
#define SYSCFG_INTAI_IEVR_RFIP_BUSY_STATUS_IEV_Msk                         (0x20UL)		/*!< SYSCFG INTAI_IEVR: RFIP_BUSY_STATUS_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IEVR_RFIP_BUSY_STATUS_IEV                             SYSCFG_INTAI_IEVR_RFIP_BUSY_STATUS_IEV_Msk
#define SYSCFG_INTAI_IEVR_COMP_IEV_Pos                                     (4UL)		/*!<SYSCFG INTAI_IEVR: COMP_IEV (Bit 4) */
#define SYSCFG_INTAI_IEVR_COMP_IEV_Msk                                     (0x10UL)		/*!< SYSCFG INTAI_IEVR: COMP_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IEVR_COMP_IEV                                         SYSCFG_INTAI_IEVR_COMP_IEV_Msk
#define SYSCFG_INTAI_IEVR_RX_IEV_Pos                                       (1UL)		/*!<SYSCFG INTAI_IEVR: RX_IEV (Bit 1) */
#define SYSCFG_INTAI_IEVR_RX_IEV_Msk                                       (0x2UL)		/*!< SYSCFG INTAI_IEVR: RX_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IEVR_RX_IEV                                           SYSCFG_INTAI_IEVR_RX_IEV_Msk
#define SYSCFG_INTAI_IEVR_TX_IEV_Pos                                       (0UL)		/*!<SYSCFG INTAI_IEVR: TX_IEV (Bit 0) */
#define SYSCFG_INTAI_IEVR_TX_IEV_Msk                                       (0x1UL)		/*!< SYSCFG INTAI_IEVR: TX_IEV (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IEVR_TX_IEV                                           SYSCFG_INTAI_IEVR_TX_IEV_Msk

/* =====================================================    INTAI_IER    =====================================================*/ 
#define SYSCFG_INTAI_IER_LC_ACTIVITY_IE_Pos                                (6UL)		/*!<SYSCFG INTAI_IER: LC_ACTIVITY_IE (Bit 6) */
#define SYSCFG_INTAI_IER_LC_ACTIVITY_IE_Msk                                (0x40UL)		/*!< SYSCFG INTAI_IER: LC_ACTIVITY_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IER_LC_ACTIVITY_IE                                    SYSCFG_INTAI_IER_LC_ACTIVITY_IE_Msk
#define SYSCFG_INTAI_IER_RFIP_BUSY_STATUS_IE_Pos                           (5UL)		/*!<SYSCFG INTAI_IER: RFIP_BUSY_STATUS_IE (Bit 5) */
#define SYSCFG_INTAI_IER_RFIP_BUSY_STATUS_IE_Msk                           (0x20UL)		/*!< SYSCFG INTAI_IER: RFIP_BUSY_STATUS_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IER_RFIP_BUSY_STATUS_IE                               SYSCFG_INTAI_IER_RFIP_BUSY_STATUS_IE_Msk
#define SYSCFG_INTAI_IER_COMP_IE_Pos                                       (4UL)		/*!<SYSCFG INTAI_IER: COMP_IE (Bit 4) */
#define SYSCFG_INTAI_IER_COMP_IE_Msk                                       (0x10UL)		/*!< SYSCFG INTAI_IER: COMP_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IER_COMP_IE                                           SYSCFG_INTAI_IER_COMP_IE_Msk
#define SYSCFG_INTAI_IER_RX_IE_Pos                                         (1UL)		/*!<SYSCFG INTAI_IER: RX_IE (Bit 1) */
#define SYSCFG_INTAI_IER_RX_IE_Msk                                         (0x2UL)		/*!< SYSCFG INTAI_IER: RX_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IER_RX_IE                                             SYSCFG_INTAI_IER_RX_IE_Msk
#define SYSCFG_INTAI_IER_TX_IE_Pos                                         (0UL)		/*!<SYSCFG INTAI_IER: TX_IE (Bit 0) */
#define SYSCFG_INTAI_IER_TX_IE_Msk                                         (0x1UL)		/*!< SYSCFG INTAI_IER: TX_IE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_IER_TX_IE                                             SYSCFG_INTAI_IER_TX_IE_Msk

/* =====================================================    INTAI_ISCR    =====================================================*/ 
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISEDGE_Pos                           (7UL)		/*!<SYSCFG INTAI_ISCR: LC_ACTIVITY_ISEDGE (Bit 7) */
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISEDGE_Msk                           (0x80UL)		/*!< SYSCFG INTAI_ISCR: LC_ACTIVITY_ISEDGE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISEDGE                               SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISEDGE_Msk
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISC_Pos                              (6UL)		/*!<SYSCFG INTAI_ISCR: LC_ACTIVITY_ISC (Bit 6) */
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISC_Msk                              (0x40UL)		/*!< SYSCFG INTAI_ISCR: LC_ACTIVITY_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISC                                  SYSCFG_INTAI_ISCR_LC_ACTIVITY_ISC_Msk
#define SYSCFG_INTAI_ISCR_RFIP_BUSY_STATUS_ISC_Pos                         (5UL)		/*!<SYSCFG INTAI_ISCR: RFIP_BUSY_STATUS_ISC (Bit 5) */
#define SYSCFG_INTAI_ISCR_RFIP_BUSY_STATUS_ISC_Msk                         (0x20UL)		/*!< SYSCFG INTAI_ISCR: RFIP_BUSY_STATUS_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_RFIP_BUSY_STATUS_ISC                             SYSCFG_INTAI_ISCR_RFIP_BUSY_STATUS_ISC_Msk
#define SYSCFG_INTAI_ISCR_COMP_ISC_Pos                                     (4UL)		/*!<SYSCFG INTAI_ISCR: COMP_ISC (Bit 4) */
#define SYSCFG_INTAI_ISCR_COMP_ISC_Msk                                     (0x10UL)		/*!< SYSCFG INTAI_ISCR: COMP_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_COMP_ISC                                         SYSCFG_INTAI_ISCR_COMP_ISC_Msk
#define SYSCFG_INTAI_ISCR_RX_ISEDGE_Pos                                    (3UL)		/*!<SYSCFG INTAI_ISCR: RX_ISEDGE (Bit 3) */
#define SYSCFG_INTAI_ISCR_RX_ISEDGE_Msk                                    (0x8UL)		/*!< SYSCFG INTAI_ISCR: RX_ISEDGE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_RX_ISEDGE                                        SYSCFG_INTAI_ISCR_RX_ISEDGE_Msk
#define SYSCFG_INTAI_ISCR_TX_ISEDGE_Pos                                    (2UL)		/*!<SYSCFG INTAI_ISCR: TX_ISEDGE (Bit 2) */
#define SYSCFG_INTAI_ISCR_TX_ISEDGE_Msk                                    (0x4UL)		/*!< SYSCFG INTAI_ISCR: TX_ISEDGE (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_TX_ISEDGE                                        SYSCFG_INTAI_ISCR_TX_ISEDGE_Msk
#define SYSCFG_INTAI_ISCR_RX_ISC_Pos                                       (1UL)		/*!<SYSCFG INTAI_ISCR: RX_ISC (Bit 1) */
#define SYSCFG_INTAI_ISCR_RX_ISC_Msk                                       (0x2UL)		/*!< SYSCFG INTAI_ISCR: RX_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_RX_ISC                                           SYSCFG_INTAI_ISCR_RX_ISC_Msk
#define SYSCFG_INTAI_ISCR_TX_ISC_Pos                                       (0UL)		/*!<SYSCFG INTAI_ISCR: TX_ISC (Bit 0) */
#define SYSCFG_INTAI_ISCR_TX_ISC_Msk                                       (0x1UL)		/*!< SYSCFG INTAI_ISCR: TX_ISC (Bitfield-Mask: 0x01) */
#define SYSCFG_INTAI_ISCR_TX_ISC                                           SYSCFG_INTAI_ISCR_TX_ISC_Msk

/* =====================================================    SR1    =====================================================*/ 
#define SYSCFG_SR1_RFIP_BUSY_STATUS_Pos                                    (5UL)		/*!<SYSCFG SR1: RFIP_BUSY_STATUS (Bit 5) */
#define SYSCFG_SR1_RFIP_BUSY_STATUS_Msk                                    (0x20UL)		/*!< SYSCFG SR1: RFIP_BUSY_STATUS (Bitfield-Mask: 0x01) */
#define SYSCFG_SR1_RFIP_BUSY_STATUS                                        SYSCFG_SR1_RFIP_BUSY_STATUS_Msk


/* ============================================================================================================================*/ 
/*=====================                                       FLASH                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    COMMAND    =====================================================*/ 
#define FLASH_COMMAND_COMMAND_Pos                                          (0UL)		/*!<FLASH COMMAND: COMMAND (Bit 0) */
#define FLASH_COMMAND_COMMAND_Msk                                          (0xffUL)		/*!< FLASH COMMAND: COMMAND (Bitfield-Mask: 0xff) */
#define FLASH_COMMAND_COMMAND                                              FLASH_COMMAND_COMMAND_Msk
#define FLASH_COMMAND_COMMAND_0                                            (0x1U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_1                                            (0x2U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_2                                            (0x4U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_3                                            (0x8U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_4                                            (0x10U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_5                                            (0x20U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_6                                            (0x40U << FLASH_COMMAND_COMMAND_Pos)
#define FLASH_COMMAND_COMMAND_7                                            (0x80U << FLASH_COMMAND_COMMAND_Pos)

/* =====================================================    CONFIG    =====================================================*/ 
#define FLASH_CONFIG_SLEEP_SM_Pos                                          (6UL)		/*!<FLASH CONFIG: SLEEP_SM (Bit 6) */
#define FLASH_CONFIG_SLEEP_SM_Msk                                          (0x40UL)		/*!< FLASH CONFIG: SLEEP_SM (Bitfield-Mask: 0x01) */
#define FLASH_CONFIG_SLEEP_SM                                              FLASH_CONFIG_SLEEP_SM_Msk
#define FLASH_CONFIG_WAIT_STATES_Pos                                       (4UL)		/*!<FLASH CONFIG: WAIT_STATES (Bit 4) */
#define FLASH_CONFIG_WAIT_STATES_Msk                                       (0x30UL)		/*!< FLASH CONFIG: WAIT_STATES (Bitfield-Mask: 0x03) */
#define FLASH_CONFIG_WAIT_STATES                                           FLASH_CONFIG_WAIT_STATES_Msk
#define FLASH_CONFIG_WAIT_STATES_0                                         (0x1U << FLASH_CONFIG_WAIT_STATES_Pos)
#define FLASH_CONFIG_WAIT_STATES_1                                         (0x2U << FLASH_CONFIG_WAIT_STATES_Pos)
#define FLASH_CONFIG_DIS_GROUP_WRITE_Pos                                   (2UL)		/*!<FLASH CONFIG: DIS_GROUP_WRITE (Bit 2) */
#define FLASH_CONFIG_DIS_GROUP_WRITE_Msk                                   (0x4UL)		/*!< FLASH CONFIG: DIS_GROUP_WRITE (Bitfield-Mask: 0x01) */
#define FLASH_CONFIG_DIS_GROUP_WRITE                                       FLASH_CONFIG_DIS_GROUP_WRITE_Msk
#define FLASH_CONFIG_REMAP_Pos                                             (1UL)		/*!<FLASH CONFIG: REMAP (Bit 1) */
#define FLASH_CONFIG_REMAP_Msk                                             (0x2UL)		/*!< FLASH CONFIG: REMAP (Bitfield-Mask: 0x01) */
#define FLASH_CONFIG_REMAP                                                 FLASH_CONFIG_REMAP_Msk

/* =====================================================    IRQSTAT    =====================================================*/ 
#define FLASH_IRQSTAT_READOK_MIS_Pos                                       (4UL)		/*!<FLASH IRQSTAT: READOK_MIS (Bit 4) */
#define FLASH_IRQSTAT_READOK_MIS_Msk                                       (0x10UL)		/*!< FLASH IRQSTAT: READOK_MIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQSTAT_READOK_MIS                                           FLASH_IRQSTAT_READOK_MIS_Msk
#define FLASH_IRQSTAT_ILLCMD_MIS_Pos                                       (3UL)		/*!<FLASH IRQSTAT: ILLCMD_MIS (Bit 3) */
#define FLASH_IRQSTAT_ILLCMD_MIS_Msk                                       (0x8UL)		/*!< FLASH IRQSTAT: ILLCMD_MIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQSTAT_ILLCMD_MIS                                           FLASH_IRQSTAT_ILLCMD_MIS_Msk
#define FLASH_IRQSTAT_CMDERR_MIS_Pos                                       (2UL)		/*!<FLASH IRQSTAT: CMDERR_MIS (Bit 2) */
#define FLASH_IRQSTAT_CMDERR_MIS_Msk                                       (0x4UL)		/*!< FLASH IRQSTAT: CMDERR_MIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQSTAT_CMDERR_MIS                                           FLASH_IRQSTAT_CMDERR_MIS_Msk
#define FLASH_IRQSTAT_CMDSTART_MIS_Pos                                     (1UL)		/*!<FLASH IRQSTAT: CMDSTART_MIS (Bit 1) */
#define FLASH_IRQSTAT_CMDSTART_MIS_Msk                                     (0x2UL)		/*!< FLASH IRQSTAT: CMDSTART_MIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQSTAT_CMDSTART_MIS                                         FLASH_IRQSTAT_CMDSTART_MIS_Msk
#define FLASH_IRQSTAT_CMDDONE_MIS_Pos                                      (0UL)		/*!<FLASH IRQSTAT: CMDDONE_MIS (Bit 0) */
#define FLASH_IRQSTAT_CMDDONE_MIS_Msk                                      (0x1UL)		/*!< FLASH IRQSTAT: CMDDONE_MIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQSTAT_CMDDONE_MIS                                          FLASH_IRQSTAT_CMDDONE_MIS_Msk

/* =====================================================    IRQMASK    =====================================================*/ 
#define FLASH_IRQMASK_READOKM_Pos                                          (4UL)		/*!<FLASH IRQMASK: READOKM (Bit 4) */
#define FLASH_IRQMASK_READOKM_Msk                                          (0x10UL)		/*!< FLASH IRQMASK: READOKM (Bitfield-Mask: 0x01) */
#define FLASH_IRQMASK_READOKM                                              FLASH_IRQMASK_READOKM_Msk
#define FLASH_IRQMASK_ILLCMDM_Pos                                          (3UL)		/*!<FLASH IRQMASK: ILLCMDM (Bit 3) */
#define FLASH_IRQMASK_ILLCMDM_Msk                                          (0x8UL)		/*!< FLASH IRQMASK: ILLCMDM (Bitfield-Mask: 0x01) */
#define FLASH_IRQMASK_ILLCMDM                                              FLASH_IRQMASK_ILLCMDM_Msk
#define FLASH_IRQMASK_CMDERRM_Pos                                          (2UL)		/*!<FLASH IRQMASK: CMDERRM (Bit 2) */
#define FLASH_IRQMASK_CMDERRM_Msk                                          (0x4UL)		/*!< FLASH IRQMASK: CMDERRM (Bitfield-Mask: 0x01) */
#define FLASH_IRQMASK_CMDERRM                                              FLASH_IRQMASK_CMDERRM_Msk
#define FLASH_IRQMASK_CMDSTARTM_Pos                                        (1UL)		/*!<FLASH IRQMASK: CMDSTARTM (Bit 1) */
#define FLASH_IRQMASK_CMDSTARTM_Msk                                        (0x2UL)		/*!< FLASH IRQMASK: CMDSTARTM (Bitfield-Mask: 0x01) */
#define FLASH_IRQMASK_CMDSTARTM                                            FLASH_IRQMASK_CMDSTARTM_Msk
#define FLASH_IRQMASK_CMDDONEM_Pos                                         (0UL)		/*!<FLASH IRQMASK: CMDDONEM (Bit 0) */
#define FLASH_IRQMASK_CMDDONEM_Msk                                         (0x1UL)		/*!< FLASH IRQMASK: CMDDONEM (Bitfield-Mask: 0x01) */
#define FLASH_IRQMASK_CMDDONEM                                             FLASH_IRQMASK_CMDDONEM_Msk

/* =====================================================    IRQRAW    =====================================================*/
#define FLASH_IRQRAW_READOK_RIS_Pos                                        (4UL)		/*!<FLASH IRQRAW: READOK_RIS (Bit 4) */
#define FLASH_IRQRAW_READOK_RIS_Msk                                        (0x10UL)		/*!< FLASH IRQRAW: READOK_RIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQRAW_READOK_RIS                                            FLASH_IRQRAW_READOK_RIS_Msk
#define FLASH_IRQRAW_ILLCMD_RIS_Pos                                        (3UL)		/*!<FLASH IRQRAW: ILLCMD_RIS (Bit 3) */
#define FLASH_IRQRAW_ILLCMD_RIS_Msk                                        (0x8UL)		/*!< FLASH IRQRAW: ILLCMD_RIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQRAW_ILLCMD_RIS                                            FLASH_IRQRAW_ILLCMD_RIS_Msk
#define FLASH_IRQRAW_CMDERR_RIS_Pos                                        (2UL)		/*!<FLASH IRQRAW: CMDERR_RIS (Bit 2) */
#define FLASH_IRQRAW_CMDERR_RIS_Msk                                        (0x4UL)		/*!< FLASH IRQRAW: CMDERR_RIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQRAW_CMDERR_RIS                                            FLASH_IRQRAW_CMDERR_RIS_Msk
#define FLASH_IRQRAW_CMDSTART_RIS_Pos                                      (1UL)		/*!<FLASH IRQRAW: CMDSTART_RIS (Bit 1) */
#define FLASH_IRQRAW_CMDSTART_RIS_Msk                                      (0x2UL)		/*!< FLASH IRQRAW: CMDSTART_RIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQRAW_CMDSTART_RIS                                          FLASH_IRQRAW_CMDSTART_RIS_Msk
#define FLASH_IRQRAW_CMDDONE_RIS_Pos                                       (0UL)		/*!<FLASH IRQRAW: CMDDONE_RIS (Bit 0) */
#define FLASH_IRQRAW_CMDDONE_RIS_Msk                                       (0x1UL)		/*!< FLASH IRQRAW: CMDDONE_RIS (Bitfield-Mask: 0x01) */
#define FLASH_IRQRAW_CMDDONE_RIS                                           FLASH_IRQRAW_CMDDONE_RIS_Msk

/* =====================================================    FLASH_SIZE    =====================================================*/ 
#define FLASH_FLASH_SIZE_PACKAGE_SIZE_Pos                                  (21UL)		/*!<FLASH FLASH_SIZE: PACKAGE_SIZE (Bit 21) */
#define FLASH_FLASH_SIZE_PACKAGE_SIZE_Msk                                  (0x600000UL)		/*!< FLASH FLASH_SIZE: PACKAGE_SIZE (Bitfield-Mask: 0x03) */
#define FLASH_FLASH_SIZE_PACKAGE_SIZE                                      FLASH_FLASH_SIZE_PACKAGE_SIZE_Msk
#define FLASH_FLASH_SIZE_PACKAGE_SIZE_0                                    (0x1U << FLASH_FLASH_SIZE_PACKAGE_SIZE_Pos)
#define FLASH_FLASH_SIZE_PACKAGE_SIZE_1                                    (0x2U << FLASH_FLASH_SIZE_PACKAGE_SIZE_Pos)
#define FLASH_FLASH_SIZE_SWD_DISABLE_Pos                                   (20UL)		/*!<FLASH FLASH_SIZE: SWD_DISABLE (Bit 20) */
#define FLASH_FLASH_SIZE_SWD_DISABLE_Msk                                   (0x100000UL)		/*!< FLASH FLASH_SIZE: SWD_DISABLE (Bitfield-Mask: 0x01) */
#define FLASH_FLASH_SIZE_SWD_DISABLE                                       FLASH_FLASH_SIZE_SWD_DISABLE_Msk
#define FLASH_FLASH_SIZE_FLASH_SECURE_Pos                                  (19UL)		/*!<FLASH FLASH_SIZE: FLASH_SECURE (Bit 19) */
#define FLASH_FLASH_SIZE_FLASH_SECURE_Msk                                  (0x80000UL)		/*!< FLASH FLASH_SIZE: FLASH_SECURE (Bitfield-Mask: 0x01) */
#define FLASH_FLASH_SIZE_FLASH_SECURE                                      FLASH_FLASH_SIZE_FLASH_SECURE_Msk
#define FLASH_FLASH_SIZE_RAM_SIZE_Pos                                      (17UL)		/*!<FLASH FLASH_SIZE: RAM_SIZE (Bit 17) */
#define FLASH_FLASH_SIZE_RAM_SIZE_Msk                                      (0x20000UL)		/*!< FLASH FLASH_SIZE: RAM_SIZE (Bitfield-Mask: 0x01) */
#define FLASH_FLASH_SIZE_RAM_SIZE                                          FLASH_FLASH_SIZE_RAM_SIZE_Msk
#define FLASH_FLASH_SIZE_FLASH_SIZE_Pos                                    (0UL)		/*!<FLASH FLASH_SIZE: FLASH_SIZE (Bit 0) */
#define FLASH_FLASH_SIZE_FLASH_SIZE_Msk                                    (0xffffUL)		/*!< FLASH FLASH_SIZE: FLASH_SIZE (Bitfield-Mask: 0xffff) */
#define FLASH_FLASH_SIZE_FLASH_SIZE                                        FLASH_FLASH_SIZE_FLASH_SIZE_Msk
#define FLASH_FLASH_SIZE_FLASH_SIZE_0                                      (0x1U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_1                                      (0x2U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_2                                      (0x4U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_3                                      (0x8U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_4                                      (0x10U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_5                                      (0x20U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_6                                      (0x40U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_7                                      (0x80U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_8                                      (0x100U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_9                                      (0x200U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_10                                     (0x400U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_11                                     (0x800U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_12                                     (0x1000U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_13                                     (0x2000U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_14                                     (0x4000U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)
#define FLASH_FLASH_SIZE_FLASH_SIZE_15                                     (0x8000U << FLASH_FLASH_SIZE_FLASH_SIZE_Pos)

/* =====================================================    ADDRESS    =====================================================*/ 
#define FLASH_ADDRESS_XADDR_Pos                                            (6UL)		/*!<FLASH ADDRESS: XADDR (Bit 6) */
#define FLASH_ADDRESS_XADDR_Msk                                            (0xffc0UL)		/*!< FLASH ADDRESS: XADDR (Bitfield-Mask: 0x3ff) */
#define FLASH_ADDRESS_XADDR                                                FLASH_ADDRESS_XADDR_Msk
#define FLASH_ADDRESS_XADDR_0                                              (0x1U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_1                                              (0x2U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_2                                              (0x4U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_3                                              (0x8U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_4                                              (0x10U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_5                                              (0x20U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_6                                              (0x40U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_7                                              (0x80U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_8                                              (0x100U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_XADDR_9                                              (0x200U << FLASH_ADDRESS_XADDR_Pos)
#define FLASH_ADDRESS_YADDR_Pos                                            (0UL)		/*!<FLASH ADDRESS: YADDR (Bit 0) */
#define FLASH_ADDRESS_YADDR_Msk                                            (0x3fUL)		/*!< FLASH ADDRESS: YADDR (Bitfield-Mask: 0x3f) */
#define FLASH_ADDRESS_YADDR                                                FLASH_ADDRESS_YADDR_Msk
#define FLASH_ADDRESS_YADDR_0                                              (0x1U << FLASH_ADDRESS_YADDR_Pos)
#define FLASH_ADDRESS_YADDR_1                                              (0x2U << FLASH_ADDRESS_YADDR_Pos)
#define FLASH_ADDRESS_YADDR_2                                              (0x4U << FLASH_ADDRESS_YADDR_Pos)
#define FLASH_ADDRESS_YADDR_3                                              (0x8U << FLASH_ADDRESS_YADDR_Pos)
#define FLASH_ADDRESS_YADDR_4                                              (0x10U << FLASH_ADDRESS_YADDR_Pos)
#define FLASH_ADDRESS_YADDR_5                                              (0x20U << FLASH_ADDRESS_YADDR_Pos)

/* =====================================================    LFSRVAL      =====================================================*/
#define FLASH_LFSRVAL_Pos                                                  (0UL)		                          /*!< Linear feedback shift*/
#define FLASH_LFSRVAL_Msk                                                  (0xFFFFFFFFUL << FLASH_LFSRVAL_Pos)		  /*! 0xFFFFFFFF */
#define FLASH_LFSRVAL                                                      FLASH_LFSRVAL_Msk
#define FLASH_LFSRVAL_0                                                    (0x00000001U << FLASH_LFSRVAL_Pos)             /*! 0x00000001 */
#define FLASH_LFSRVAL_1                                                    (0x00000002U << FLASH_LFSRVAL_Pos)             /*! 0x00000002 */
#define FLASH_LFSRVAL_2                                                    (0x00000004U << FLASH_LFSRVAL_Pos)             /*! 0x00000004 */
#define FLASH_LFSRVAL_3                                                    (0x00000008U << FLASH_LFSRVAL_Pos)             /*! 0x00000008 */
#define FLASH_LFSRVAL_4                                                    (0x00000010U << FLASH_LFSRVAL_Pos)             /*! 0x00000010 */
#define FLASH_LFSRVAL_5                                                    (0x00000020U << FLASH_LFSRVAL_Pos)             /*! 0x00000020 */
#define FLASH_LFSRVAL_6                                                    (0x00000040U << FLASH_LFSRVAL_Pos)             /*! 0x00000040 */
#define FLASH_LFSRVAL_7                                                    (0x00000080U << FLASH_LFSRVAL_Pos)             /*! 0x00000080 */
#define FLASH_LFSRVAL_8                                                    (0x00000100U << FLASH_LFSRVAL_Pos)             /*! 0x00000100 */
#define FLASH_LFSRVAL_9                                                    (0x00000200U << FLASH_LFSRVAL_Pos)             /*! 0x00000200 */
#define FLASH_LFSRVAL_10                                                   (0x00000400U << FLASH_LFSRVAL_Pos)             /*! 0x00000400 */
#define FLASH_LFSRVAL_11                                                   (0x00000800U << FLASH_LFSRVAL_Pos)             /*! 0x00000800 */
#define FLASH_LFSRVAL_12                                                   (0x00001000U << FLASH_LFSRVAL_Pos)             /*! 0x00001000 */
#define FLASH_LFSRVAL_13                                                   (0x00002000U << FLASH_LFSRVAL_Pos)             /*! 0x00002000 */
#define FLASH_LFSRVAL_14                                                   (0x00004000U << FLASH_LFSRVAL_Pos)             /*! 0x00004000 */
#define FLASH_LFSRVAL_15                                                   (0x00008000U << FLASH_LFSRVAL_Pos)             /*! 0x00008000 */
#define FLASH_LFSRVAL_16                                                   (0x00010000U << FLASH_LFSRVAL_Pos)             /*! 0x00010000 */
#define FLASH_LFSRVAL_17                                                   (0x00020000U << FLASH_LFSRVAL_Pos)             /*! 0x00020000 */
#define FLASH_LFSRVAL_18                                                   (0x00040000U << FLASH_LFSRVAL_Pos)             /*! 0x00040000 */
#define FLASH_LFSRVAL_19                                                   (0x00080000U << FLASH_LFSRVAL_Pos)             /*! 0x00080000 */
#define FLASH_LFSRVAL_20                                                   (0x00100000U << FLASH_LFSRVAL_Pos)             /*! 0x00100000 */
#define FLASH_LFSRVAL_21                                                   (0x00200000U << FLASH_LFSRVAL_Pos)             /*! 0x00200000 */
#define FLASH_LFSRVAL_22                                                   (0x00400000U << FLASH_LFSRVAL_Pos)             /*! 0x00400000 */
#define FLASH_LFSRVAL_23                                                   (0x00800000U << FLASH_LFSRVAL_Pos)             /*! 0x00800000 */
#define FLASH_LFSRVAL_24                                                   (0x01000000U << FLASH_LFSRVAL_Pos)             /*! 0x01000000 */
#define FLASH_LFSRVAL_25                                                   (0x02000000U << FLASH_LFSRVAL_Pos)             /*! 0x02000000 */
#define FLASH_LFSRVAL_26                                                   (0x04000000U << FLASH_LFSRVAL_Pos)             /*! 0x04000000 */
#define FLASH_LFSRVAL_27                                                   (0x08000000U << FLASH_LFSRVAL_Pos)             /*! 0x08000000 */
#define FLASH_LFSRVAL_28                                                   (0x10000000U << FLASH_LFSRVAL_Pos)             /*! 0x10000000 */
#define FLASH_LFSRVAL_29                                                   (0x20000000U << FLASH_LFSRVAL_Pos)             /*! 0x20000000 */
#define FLASH_LFSRVAL_30                                                   (0x40000000U << FLASH_LFSRVAL_Pos)             /*! 0x40000000 */
#define FLASH_LFSRVAL_31                                                   (0x80000000U << FLASH_LFSRVAL_Pos)             /*! 0x80000000 */

/* =====================================================    PAGEPROT0    =====================================================*/ 
#define FLASH_PAGEPROT0_SEG1_Pos                                           (16UL)		/*!<FLASH PAGEPROT0: SEG1 (Bit 16) */
#define FLASH_PAGEPROT0_SEG1_Msk                                           (0xffff0000UL)		/*!< FLASH PAGEPROT0: SEG1 (Bitfield-Mask: 0xffff) */
#define FLASH_PAGEPROT0_SEG1                                               FLASH_PAGEPROT0_SEG1_Msk
#define FLASH_PAGEPROT0_SEG1_0                                             (0x1U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_1                                             (0x2U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_2                                             (0x4U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_3                                             (0x8U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_4                                             (0x10U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_5                                             (0x20U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_6                                             (0x40U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_7                                             (0x80U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_8                                             (0x100U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_9                                             (0x200U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_10                                            (0x400U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_11                                            (0x800U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_12                                            (0x1000U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_13                                            (0x2000U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_14                                            (0x4000U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG1_15                                            (0x8000U << FLASH_PAGEPROT0_SEG1_Pos)
#define FLASH_PAGEPROT0_SEG0_Pos                                           (0UL)		/*!<FLASH PAGEPROT0: SEG0 (Bit 0) */
#define FLASH_PAGEPROT0_SEG0_Msk                                           (0xffffUL)		/*!< FLASH PAGEPROT0: SEG0 (Bitfield-Mask: 0xffff) */
#define FLASH_PAGEPROT0_SEG0                                               FLASH_PAGEPROT0_SEG0_Msk
#define FLASH_PAGEPROT0_SEG0_0                                             (0x1U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_1                                             (0x2U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_2                                             (0x4U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_3                                             (0x8U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_4                                             (0x10U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_5                                             (0x20U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_6                                             (0x40U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_7                                             (0x80U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_8                                             (0x100U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_9                                             (0x200U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_10                                            (0x400U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_11                                            (0x800U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_12                                            (0x1000U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_13                                            (0x2000U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_14                                            (0x4000U << FLASH_PAGEPROT0_SEG0_Pos)
#define FLASH_PAGEPROT0_SEG0_15                                            (0x8000U << FLASH_PAGEPROT0_SEG0_Pos)

/* =====================================================    PAGEPROT1    =====================================================*/ 
#define FLASH_PAGEPROT1_SEG3_Pos                                           (16UL)		/*!<FLASH PAGEPROT1: SEG3 (Bit 16) */
#define FLASH_PAGEPROT1_SEG3_Msk                                           (0xffff0000UL)		/*!< FLASH PAGEPROT1: SEG3 (Bitfield-Mask: 0xffff) */
#define FLASH_PAGEPROT1_SEG3                                               FLASH_PAGEPROT1_SEG3_Msk
#define FLASH_PAGEPROT1_SEG3_0                                             (0x1U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_1                                             (0x2U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_2                                             (0x4U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_3                                             (0x8U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_4                                             (0x10U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_5                                             (0x20U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_6                                             (0x40U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_7                                             (0x80U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_8                                             (0x100U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_9                                             (0x200U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_10                                            (0x400U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_11                                            (0x800U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_12                                            (0x1000U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_13                                            (0x2000U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_14                                            (0x4000U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG3_15                                            (0x8000U << FLASH_PAGEPROT1_SEG3_Pos)
#define FLASH_PAGEPROT1_SEG2_Pos                                           (0UL)		/*!<FLASH PAGEPROT1: SEG2 (Bit 0) */
#define FLASH_PAGEPROT1_SEG2_Msk                                           (0xffffUL)		/*!< FLASH PAGEPROT1: SEG2 (Bitfield-Mask: 0xffff) */
#define FLASH_PAGEPROT1_SEG2                                               FLASH_PAGEPROT1_SEG2_Msk
#define FLASH_PAGEPROT1_SEG2_0                                             (0x1U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_1                                             (0x2U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_2                                             (0x4U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_3                                             (0x8U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_4                                             (0x10U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_5                                             (0x20U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_6                                             (0x40U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_7                                             (0x80U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_8                                             (0x100U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_9                                             (0x200U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_10                                            (0x400U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_11                                            (0x800U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_12                                            (0x1000U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_13                                            (0x2000U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_14                                            (0x4000U << FLASH_PAGEPROT1_SEG2_Pos)
#define FLASH_PAGEPROT1_SEG2_15                                            (0x8000U << FLASH_PAGEPROT1_SEG2_Pos)

/* =====================================================    DATA0    =====================================================*/ 
#define FLASH_DATA0_DATA0_Pos                                              (0UL)		/*!<FLASH DATA0: DATA0 (Bit 0) */
#define FLASH_DATA0_DATA0_Msk                                              (0xffffffffUL)		/*!< FLASH DATA0: DATA0 (Bitfield-Mask: 0xffffffff) */
#define FLASH_DATA0_DATA0                                                  FLASH_DATA0_DATA0_Msk
#define FLASH_DATA0_DATA0_0                                                (0x1U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_1                                                (0x2U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_2                                                (0x4U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_3                                                (0x8U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_4                                                (0x10U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_5                                                (0x20U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_6                                                (0x40U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_7                                                (0x80U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_8                                                (0x100U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_9                                                (0x200U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_10                                               (0x400U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_11                                               (0x800U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_12                                               (0x1000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_13                                               (0x2000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_14                                               (0x4000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_15                                               (0x8000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_16                                               (0x10000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_17                                               (0x20000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_18                                               (0x40000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_19                                               (0x80000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_20                                               (0x100000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_21                                               (0x200000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_22                                               (0x400000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_23                                               (0x800000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_24                                               (0x1000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_25                                               (0x2000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_26                                               (0x4000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_27                                               (0x8000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_28                                               (0x10000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_29                                               (0x20000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_30                                               (0x40000000U << FLASH_DATA0_DATA0_Pos)
#define FLASH_DATA0_DATA0_31                                               (0x80000000UL << FLASH_DATA0_DATA0_Pos)

/* =====================================================    DATA1    =====================================================*/ 
#define FLASH_DATA1_DATA1_Pos                                              (0UL)		/*!<FLASH DATA1: DATA1 (Bit 0) */
#define FLASH_DATA1_DATA1_Msk                                              (0xffffffffUL)		/*!< FLASH DATA1: DATA1 (Bitfield-Mask: 0xffffffff) */
#define FLASH_DATA1_DATA1                                                  FLASH_DATA1_DATA1_Msk
#define FLASH_DATA1_DATA1_0                                                (0x1U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_1                                                (0x2U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_2                                                (0x4U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_3                                                (0x8U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_4                                                (0x10U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_5                                                (0x20U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_6                                                (0x40U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_7                                                (0x80U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_8                                                (0x100U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_9                                                (0x200U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_10                                               (0x400U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_11                                               (0x800U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_12                                               (0x1000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_13                                               (0x2000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_14                                               (0x4000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_15                                               (0x8000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_16                                               (0x10000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_17                                               (0x20000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_18                                               (0x40000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_19                                               (0x80000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_20                                               (0x100000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_21                                               (0x200000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_22                                               (0x400000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_23                                               (0x800000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_24                                               (0x1000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_25                                               (0x2000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_26                                               (0x4000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_27                                               (0x8000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_28                                               (0x10000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_29                                               (0x20000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_30                                               (0x40000000U << FLASH_DATA1_DATA1_Pos)
#define FLASH_DATA1_DATA1_31                                               (0x80000000UL << FLASH_DATA1_DATA1_Pos)

/* =====================================================    DATA2    =====================================================*/ 
#define FLASH_DATA2_DATA2_Pos                                              (0UL)		/*!<FLASH DATA2: DATA2 (Bit 0) */
#define FLASH_DATA2_DATA2_Msk                                              (0xffffffffUL)		/*!< FLASH DATA2: DATA2 (Bitfield-Mask: 0xffffffff) */
#define FLASH_DATA2_DATA2                                                  FLASH_DATA2_DATA2_Msk
#define FLASH_DATA2_DATA2_0                                                (0x1U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_1                                                (0x2U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_2                                                (0x4U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_3                                                (0x8U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_4                                                (0x10U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_5                                                (0x20U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_6                                                (0x40U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_7                                                (0x80U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_8                                                (0x100U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_9                                                (0x200U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_10                                               (0x400U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_11                                               (0x800U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_12                                               (0x1000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_13                                               (0x2000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_14                                               (0x4000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_15                                               (0x8000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_16                                               (0x10000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_17                                               (0x20000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_18                                               (0x40000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_19                                               (0x80000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_20                                               (0x100000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_21                                               (0x200000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_22                                               (0x400000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_23                                               (0x800000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_24                                               (0x1000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_25                                               (0x2000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_26                                               (0x4000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_27                                               (0x8000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_28                                               (0x10000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_29                                               (0x20000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_30                                               (0x40000000U << FLASH_DATA2_DATA2_Pos)
#define FLASH_DATA2_DATA2_31                                               (0x80000000UL << FLASH_DATA2_DATA2_Pos)

/* =====================================================    DATA3    =====================================================*/ 
#define FLASH_DATA3_DATA3_Pos                                              (0UL)		/*!<FLASH DATA3: DATA3 (Bit 0) */
#define FLASH_DATA3_DATA3_Msk                                              (0xffffffffUL)		/*!< FLASH DATA3: DATA3 (Bitfield-Mask: 0xffffffff) */
#define FLASH_DATA3_DATA3                                                  FLASH_DATA3_DATA3_Msk
#define FLASH_DATA3_DATA3_0                                                (0x1U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_1                                                (0x2U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_2                                                (0x4U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_3                                                (0x8U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_4                                                (0x10U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_5                                                (0x20U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_6                                                (0x40U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_7                                                (0x80U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_8                                                (0x100U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_9                                                (0x200U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_10                                               (0x400U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_11                                               (0x800U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_12                                               (0x1000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_13                                               (0x2000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_14                                               (0x4000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_15                                               (0x8000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_16                                               (0x10000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_17                                               (0x20000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_18                                               (0x40000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_19                                               (0x80000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_20                                               (0x100000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_21                                               (0x200000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_22                                               (0x400000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_23                                               (0x800000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_24                                               (0x1000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_25                                               (0x2000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_26                                               (0x4000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_27                                               (0x8000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_28                                               (0x10000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_29                                               (0x20000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_30                                               (0x40000000U << FLASH_DATA3_DATA3_Pos)
#define FLASH_DATA3_DATA3_31                                               (0x80000000UL << FLASH_DATA3_DATA3_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        TIM                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR1    =====================================================*/ 
#define TIM_CR1_UIFREMAP_Pos                                               (11UL)		/*!<TIM CR1: UIFREMAP (Bit 11) */
#define TIM_CR1_UIFREMAP_Msk                                               (0x800UL)		/*!< TIM CR1: UIFREMAP (Bitfield-Mask: 0x01) */
#define TIM_CR1_UIFREMAP                                                   TIM_CR1_UIFREMAP_Msk
#define TIM_CR1_CKD_Pos                                                    (8UL)		/*!<TIM CR1: CKD (Bit 8) */
#define TIM_CR1_CKD_Msk                                                    (0x300UL)		/*!< TIM CR1: CKD (Bitfield-Mask: 0x03) */
#define TIM_CR1_CKD                                                        TIM_CR1_CKD_Msk
#define TIM_CR1_CKD_0                                                      (0x1U << TIM_CR1_CKD_Pos)
#define TIM_CR1_CKD_1                                                      (0x2U << TIM_CR1_CKD_Pos)
#define TIM_CR1_ARPE_Pos                                                   (7UL)		/*!<TIM CR1: ARPE (Bit 7) */
#define TIM_CR1_ARPE_Msk                                                   (0x80UL)		/*!< TIM CR1: ARPE (Bitfield-Mask: 0x01) */
#define TIM_CR1_ARPE                                                       TIM_CR1_ARPE_Msk
#define TIM_CR1_CMS_Pos                                                    (5UL)		/*!<TIM CR1: CMS (Bit 5) */
#define TIM_CR1_CMS_Msk                                                    (0x60UL)		/*!< TIM CR1: CMS (Bitfield-Mask: 0x03) */
#define TIM_CR1_CMS                                                        TIM_CR1_CMS_Msk
#define TIM_CR1_CMS_0                                                      (0x1U << TIM_CR1_CMS_Pos)
#define TIM_CR1_CMS_1                                                      (0x2U << TIM_CR1_CMS_Pos)
#define TIM_CR1_DIR_Pos                                                    (4UL)		/*!<TIM CR1: DIR (Bit 4) */
#define TIM_CR1_DIR_Msk                                                    (0x10UL)		/*!< TIM CR1: DIR (Bitfield-Mask: 0x01) */
#define TIM_CR1_DIR                                                        TIM_CR1_DIR_Msk
#define TIM_CR1_OPM_Pos                                                    (3UL)		/*!<TIM CR1: OPM (Bit 3) */
#define TIM_CR1_OPM_Msk                                                    (0x8UL)		/*!< TIM CR1: OPM (Bitfield-Mask: 0x01) */
#define TIM_CR1_OPM                                                        TIM_CR1_OPM_Msk
#define TIM_CR1_URS_Pos                                                    (2UL)		/*!<TIM CR1: URS (Bit 2) */
#define TIM_CR1_URS_Msk                                                    (0x4UL)		/*!< TIM CR1: URS (Bitfield-Mask: 0x01) */
#define TIM_CR1_URS                                                        TIM_CR1_URS_Msk
#define TIM_CR1_UDIS_Pos                                                   (1UL)		/*!<TIM CR1: UDIS (Bit 1) */
#define TIM_CR1_UDIS_Msk                                                   (0x2UL)		/*!< TIM CR1: UDIS (Bitfield-Mask: 0x01) */
#define TIM_CR1_UDIS                                                       TIM_CR1_UDIS_Msk
#define TIM_CR1_CEN_Pos                                                    (0UL)		/*!<TIM CR1: CEN (Bit 0) */
#define TIM_CR1_CEN_Msk                                                    (0x1UL)		/*!< TIM CR1: CEN (Bitfield-Mask: 0x01) */
#define TIM_CR1_CEN                                                        TIM_CR1_CEN_Msk

/* =====================================================    CR2    =====================================================*/ 
#define TIM_CR2_OIS1N_Pos                                                  (9UL)		/*!<TIM CR2: OIS1N (Bit 9) */
#define TIM_CR2_OIS1N_Msk                                                  (0x200UL)		/*!< TIM CR2: OIS1N (Bitfield-Mask: 0x01) */
#define TIM_CR2_OIS1N                                                      TIM_CR2_OIS1N_Msk
#define TIM_CR2_OIS1_Pos                                                   (8UL)		/*!<TIM CR2: OIS1 (Bit 8) */
#define TIM_CR2_OIS1_Msk                                                   (0x100UL)		/*!< TIM CR2: OIS1 (Bitfield-Mask: 0x01) */
#define TIM_CR2_OIS1                                                       TIM_CR2_OIS1_Msk
#define TIM_CR2_TI1S_Pos                                                   (7UL)		/*!<TIM CR2: TI1S (Bit 7) */
#define TIM_CR2_TI1S_Msk                                                   (0x80UL)		/*!< TIM CR2: TI1S (Bitfield-Mask: 0x01) */
#define TIM_CR2_TI1S                                                       TIM_CR2_TI1S_Msk
#define TIM_CR2_MMS_Pos                                                    (4UL)		/*!<TIM CR2: MMS (Bit 4) */
#define TIM_CR2_MMS_Msk                                                    (0x70UL)		/*!< TIM CR2: MMS (Bitfield-Mask: 0x07) */
#define TIM_CR2_MMS                                                        TIM_CR2_MMS_Msk
#define TIM_CR2_MMS_0                                                      (0x1U << TIM_CR2_MMS_Pos)
#define TIM_CR2_MMS_1                                                      (0x2U << TIM_CR2_MMS_Pos)
#define TIM_CR2_MMS_2                                                      (0x4U << TIM_CR2_MMS_Pos)
#define TIM_CR2_CCDS_Pos                                                   (3UL)		/*!<TIM CR2: CCDS (Bit 3) */
#define TIM_CR2_CCDS_Msk                                                   (0x8UL)		/*!< TIM CR2: CCDS (Bitfield-Mask: 0x01) */
#define TIM_CR2_CCDS                                                       TIM_CR2_CCDS_Msk
#define TIM_CR2_CCUS_Pos                                                   (2UL)		/*!<TIM CR2: CCUS (Bit 2) */
#define TIM_CR2_CCUS_Msk                                                   (0x4UL)		/*!< TIM CR2: CCUS (Bitfield-Mask: 0x01) */
#define TIM_CR2_CCUS                                                       TIM_CR2_CCUS_Msk
#define TIM_CR2_CCPC_Pos                                                   (0UL)		/*!<TIM CR2: CCPC (Bit 0) */
#define TIM_CR2_CCPC_Msk                                                   (0x1UL)		/*!< TIM CR2: CCPC (Bitfield-Mask: 0x01) */
#define TIM_CR2_CCPC                                                       TIM_CR2_CCPC_Msk

/* =====================================================    SMCR    =====================================================*/ 
#define TIM_SMCR_ETP_Pos                                                   (15UL)		/*!<TIM SMCR: ETP (Bit 15) */
#define TIM_SMCR_ETP_Msk                                                   (0x8000UL)		/*!< TIM SMCR: ETP (Bitfield-Mask: 0x01) */
#define TIM_SMCR_ETP                                                       TIM_SMCR_ETP_Msk
#define TIM_SMCR_ECE_Pos                                                   (14UL)		/*!<TIM SMCR: ECE (Bit 14) */
#define TIM_SMCR_ECE_Msk                                                   (0x4000UL)		/*!< TIM SMCR: ECE (Bitfield-Mask: 0x01) */
#define TIM_SMCR_ECE                                                       TIM_SMCR_ECE_Msk
#define TIM_SMCR_ETPS_Pos                                                  (12UL)		/*!<TIM SMCR: ETPS (Bit 12) */
#define TIM_SMCR_ETPS_Msk                                                  (0x3000UL)		/*!< TIM SMCR: ETPS (Bitfield-Mask: 0x03) */
#define TIM_SMCR_ETPS                                                      TIM_SMCR_ETPS_Msk
#define TIM_SMCR_ETPS_0                                                    (0x1U << TIM_SMCR_ETPS_Pos)
#define TIM_SMCR_ETPS_1                                                    (0x2U << TIM_SMCR_ETPS_Pos)
#define TIM_SMCR_ETF_Pos                                                   (8UL)		/*!<TIM SMCR: ETF (Bit 8) */
#define TIM_SMCR_ETF_Msk                                                   (0xf00UL)		/*!< TIM SMCR: ETF (Bitfield-Mask: 0x0f) */
#define TIM_SMCR_ETF                                                       TIM_SMCR_ETF_Msk
#define TIM_SMCR_ETF_0                                                     (0x1U << TIM_SMCR_ETF_Pos)
#define TIM_SMCR_ETF_1                                                     (0x2U << TIM_SMCR_ETF_Pos)
#define TIM_SMCR_ETF_2                                                     (0x4U << TIM_SMCR_ETF_Pos)
#define TIM_SMCR_ETF_3                                                     (0x8U << TIM_SMCR_ETF_Pos)
#define TIM_SMCR_MSM_Pos                                                   (7UL)		/*!<TIM SMCR: MSM (Bit 7) */
#define TIM_SMCR_MSM_Msk                                                   (0x80UL)		/*!< TIM SMCR: MSM (Bitfield-Mask: 0x01) */
#define TIM_SMCR_MSM                                                       TIM_SMCR_MSM_Msk
#define TIM_SMCR_TS_Pos                                                    (4UL)		/*!<TIM SMCR: TS (Bit 4) */
#define TIM_SMCR_TS_Msk                                                    (0x300070UL)		/*!< TIM SMCR: TS (Bitfield-Mask: 0x30007) */
#define TIM_SMCR_TS                                                        TIM_SMCR_TS_Msk
#define TIM_SMCR_TS_0                                                      (0x1U << TIM_SMCR_TS_Pos)
#define TIM_SMCR_TS_1                                                      (0x2U << TIM_SMCR_TS_Pos)
#define TIM_SMCR_TS_2                                                      (0x4U << TIM_SMCR_TS_Pos)
#define TIM_SMCR_TS_3                                                      (0x10000U << TIM_SMCR_TS_Pos)
#define TIM_SMCR_TS_4                                                      (0x20000U << TIM_SMCR_TS_Pos)
#define TIM_SMCR_OCCS_Pos                                                  (3UL)		/*!<TIM SMCR: OCCS (Bit 3) */
#define TIM_SMCR_OCCS_Msk                                                  (0x8UL)		/*!< TIM SMCR: OCCS (Bitfield-Mask: 0x01) */
#define TIM_SMCR_OCCS                                                      TIM_SMCR_OCCS_Msk
#define TIM_SMCR_SMS_Pos                                                   (0UL)		/*!<TIM SMCR: SMS (Bit 0) */
#define TIM_SMCR_SMS_Msk                                                   (0x10007UL)		/*!< TIM SMCR: SMS (Bitfield-Mask: 0x10007) */
#define TIM_SMCR_SMS                                                       TIM_SMCR_SMS_Msk
#define TIM_SMCR_SMS_0                                                     (0x1U << TIM_SMCR_SMS_Pos)
#define TIM_SMCR_SMS_1                                                     (0x2U << TIM_SMCR_SMS_Pos)
#define TIM_SMCR_SMS_2                                                     (0x4U << TIM_SMCR_SMS_Pos)
#define TIM_SMCR_SMS_3                                                     (0x10000U << TIM_SMCR_SMS_Pos)

/* =====================================================    DIER    =====================================================*/ 
#define TIM_DIER_BDE_Pos                                                   (15UL)		/*!<TIM DIER: BDE (Bit 15) */
#define TIM_DIER_BDE_Msk                                                   (0x8000UL)		/*!< TIM DIER: BDE (Bitfield-Mask: 0x01) */
#define TIM_DIER_BDE                                                       TIM_DIER_BDE_Msk
#define TIM_DIER_TDE_Pos                                                   (14UL)		/*!<TIM DIER: TDE (Bit 14) */
#define TIM_DIER_TDE_Msk                                                   (0x4000UL)		/*!< TIM DIER: TDE (Bitfield-Mask: 0x01) */
#define TIM_DIER_TDE                                                       TIM_DIER_TDE_Msk
#define TIM_DIER_CCUDE_Pos                                                 (13UL)		/*!<TIM DIER: CCUDE (Bit 13) */
#define TIM_DIER_CCUDE_Msk                                                 (0x2000UL)		/*!< TIM DIER: CCUDE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CCUDE                                                     TIM_DIER_CCUDE_Msk
#define TIM_DIER_CC4DE_Pos                                                 (12UL)		/*!<TIM DIER: CC4DE (Bit 12) */
#define TIM_DIER_CC4DE_Msk                                                 (0x1000UL)		/*!< TIM DIER: CC4DE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC4DE                                                     TIM_DIER_CC4DE_Msk
#define TIM_DIER_CC3DE_Pos                                                 (11UL)		/*!<TIM DIER: CC3DE (Bit 11) */
#define TIM_DIER_CC3DE_Msk                                                 (0x800UL)		/*!< TIM DIER: CC3DE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC3DE                                                     TIM_DIER_CC3DE_Msk
#define TIM_DIER_CC2DE_Pos                                                 (10UL)		/*!<TIM DIER: CC2DE (Bit 10) */
#define TIM_DIER_CC2DE_Msk                                                 (0x400UL)		/*!< TIM DIER: CC2DE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC2DE                                                     TIM_DIER_CC2DE_Msk
#define TIM_DIER_CC1DE_Pos                                                 (9UL)		/*!<TIM DIER: CC1DE (Bit 9) */
#define TIM_DIER_CC1DE_Msk                                                 (0x200UL)		/*!< TIM DIER: CC1DE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC1DE                                                     TIM_DIER_CC1DE_Msk
#define TIM_DIER_UDE_Pos                                                   (8UL)		/*!<TIM DIER: UDE (Bit 8) */
#define TIM_DIER_UDE_Msk                                                   (0x100UL)		/*!< TIM DIER: UDE (Bitfield-Mask: 0x01) */
#define TIM_DIER_UDE                                                       TIM_DIER_UDE_Msk
#define TIM_DIER_BIE_Pos                                                   (7UL)		/*!<TIM DIER: BIE (Bit 7) */
#define TIM_DIER_BIE_Msk                                                   (0x80UL)		/*!< TIM DIER: BIE (Bitfield-Mask: 0x01) */
#define TIM_DIER_BIE                                                       TIM_DIER_BIE_Msk
#define TIM_DIER_TIE_Pos                                                   (6UL)		/*!<TIM DIER: TIE (Bit 6) */
#define TIM_DIER_TIE_Msk                                                   (0x40UL)		/*!< TIM DIER: TIE (Bitfield-Mask: 0x01) */
#define TIM_DIER_TIE                                                       TIM_DIER_TIE_Msk
#define TIM_DIER_COMIE_Pos                                                 (5UL)		/*!<TIM DIER: COMIE (Bit 5) */
#define TIM_DIER_COMIE_Msk                                                 (0x20UL)		/*!< TIM DIER: COMIE (Bitfield-Mask: 0x01) */
#define TIM_DIER_COMIE                                                     TIM_DIER_COMIE_Msk
#define TIM_DIER_CC4IE_Pos                                                 (4UL)		/*!<TIM DIER: CC4IE (Bit 4) */
#define TIM_DIER_CC4IE_Msk                                                 (0x10UL)		/*!< TIM DIER: CC4IE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC4IE                                                     TIM_DIER_CC4IE_Msk
#define TIM_DIER_CC3IE_Pos                                                 (3UL)		/*!<TIM DIER: CC3IE (Bit 3) */
#define TIM_DIER_CC3IE_Msk                                                 (0x8UL)		/*!< TIM DIER: CC3IE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC3IE                                                     TIM_DIER_CC3IE_Msk
#define TIM_DIER_CC2IE_Pos                                                 (2UL)		/*!<TIM DIER: CC2IE (Bit 2) */
#define TIM_DIER_CC2IE_Msk                                                 (0x4UL)		/*!< TIM DIER: CC2IE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC2IE                                                     TIM_DIER_CC2IE_Msk
#define TIM_DIER_CC1IE_Pos                                                 (1UL)		/*!<TIM DIER: CC1IE (Bit 1) */
#define TIM_DIER_CC1IE_Msk                                                 (0x2UL)		/*!< TIM DIER: CC1IE (Bitfield-Mask: 0x01) */
#define TIM_DIER_CC1IE                                                     TIM_DIER_CC1IE_Msk
#define TIM_DIER_UIE_Pos                                                   (0UL)		/*!<TIM DIER: UIE (Bit 0) */
#define TIM_DIER_UIE_Msk                                                   (0x1UL)		/*!< TIM DIER: UIE (Bitfield-Mask: 0x01) */
#define TIM_DIER_UIE                                                       TIM_DIER_UIE_Msk

/* =====================================================    SR    =====================================================*/ 
#define TIM_SR_CC4OF_Pos                                                   (12UL)		/*!<TIM SR: CC4OF (Bit 12) */
#define TIM_SR_CC4OF_Msk                                                   (0x1000UL)		/*!< TIM SR: CC4OF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC4OF                                                       TIM_SR_CC4OF_Msk
#define TIM_SR_CC3OF_Pos                                                   (11UL)		/*!<TIM SR: CC3OF (Bit 11) */
#define TIM_SR_CC3OF_Msk                                                   (0x800UL)		/*!< TIM SR: CC3OF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC3OF                                                       TIM_SR_CC3OF_Msk
#define TIM_SR_CC2OF_Pos                                                   (10UL)		/*!<TIM SR: CC2OF (Bit 10) */
#define TIM_SR_CC2OF_Msk                                                   (0x400UL)		/*!< TIM SR: CC2OF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC2OF                                                       TIM_SR_CC2OF_Msk
#define TIM_SR_CC1OF_Pos                                                   (9UL)		/*!<TIM SR: CC1OF (Bit 9) */
#define TIM_SR_CC1OF_Msk                                                   (0x200UL)		/*!< TIM SR: CC1OF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC1OF                                                       TIM_SR_CC1OF_Msk
#define TIM_SR_BIF_Pos                                                     (7UL)		/*!<TIM SR: BIF (Bit 7) */
#define TIM_SR_BIF_Msk                                                     (0x80UL)		/*!< TIM SR: BIF (Bitfield-Mask: 0x01) */
#define TIM_SR_BIF                                                         TIM_SR_BIF_Msk
#define TIM_SR_TIF_Pos                                                     (6UL)		/*!<TIM SR: TIF (Bit 6) */
#define TIM_SR_TIF_Msk                                                     (0x40UL)		/*!< TIM SR: TIF (Bitfield-Mask: 0x01) */
#define TIM_SR_TIF                                                         TIM_SR_TIF_Msk
#define TIM_SR_COMIF_Pos                                                   (5UL)		/*!<TIM SR: COMIF (Bit 5) */
#define TIM_SR_COMIF_Msk                                                   (0x20UL)		/*!< TIM SR: COMIF (Bitfield-Mask: 0x01) */
#define TIM_SR_COMIF                                                       TIM_SR_COMIF_Msk
#define TIM_SR_CC4IF_Pos                                                   (4UL)		/*!<TIM SR: CC4IF (Bit 4) */
#define TIM_SR_CC4IF_Msk                                                   (0x10UL)		/*!< TIM SR: CC4IF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC4IF                                                       TIM_SR_CC4IF_Msk
#define TIM_SR_CC3IF_Pos                                                   (3UL)		/*!<TIM SR: CC3IF (Bit 3) */
#define TIM_SR_CC3IF_Msk                                                   (0x8UL)		/*!< TIM SR: CC3IF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC3IF                                                       TIM_SR_CC3IF_Msk
#define TIM_SR_CC2IF_Pos                                                   (2UL)		/*!<TIM SR: CC2IF (Bit 2) */
#define TIM_SR_CC2IF_Msk                                                   (0x4UL)		/*!< TIM SR: CC2IF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC2IF                                                       TIM_SR_CC2IF_Msk
#define TIM_SR_CC1IF_Pos                                                   (1UL)		/*!<TIM SR: CC1IF (Bit 1) */
#define TIM_SR_CC1IF_Msk                                                   (0x2UL)		/*!< TIM SR: CC1IF (Bitfield-Mask: 0x01) */
#define TIM_SR_CC1IF                                                       TIM_SR_CC1IF_Msk
#define TIM_SR_UIF_Pos                                                     (0UL)		/*!<TIM SR: UIF (Bit 0) */
#define TIM_SR_UIF_Msk                                                     (0x1UL)		/*!< TIM SR: UIF (Bitfield-Mask: 0x01) */
#define TIM_SR_UIF                                                         TIM_SR_UIF_Msk

/* =====================================================    EGR    =====================================================*/ 
#define TIM_EGR_BG_Pos                                                     (7UL)		/*!<TIM EGR: BG (Bit 7) */
#define TIM_EGR_BG_Msk                                                     (0x80UL)		/*!< TIM EGR: BG (Bitfield-Mask: 0x01) */
#define TIM_EGR_BG                                                         TIM_EGR_BG_Msk
#define TIM_EGR_TG_Pos                                                     (6UL)		/*!<TIM EGR: TG (Bit 6) */
#define TIM_EGR_TG_Msk                                                     (0x40UL)		/*!< TIM EGR: TG (Bitfield-Mask: 0x01) */
#define TIM_EGR_TG                                                         TIM_EGR_TG_Msk
#define TIM_EGR_COMG_Pos                                                   (5UL)		/*!<TIM EGR: COMG (Bit 5) */
#define TIM_EGR_COMG_Msk                                                   (0x20UL)		/*!< TIM EGR: COMG (Bitfield-Mask: 0x01) */
#define TIM_EGR_COMG                                                       TIM_EGR_COMG_Msk
#define TIM_EGR_CC4G_Pos                                                   (4UL)		/*!<TIM EGR: CC4G (Bit 4) */
#define TIM_EGR_CC4G_Msk                                                   (0x10UL)		/*!< TIM EGR: CC4G (Bitfield-Mask: 0x01) */
#define TIM_EGR_CC4G                                                       TIM_EGR_CC4G_Msk
#define TIM_EGR_CC3G_Pos                                                   (3UL)		/*!<TIM EGR: CC3G (Bit 3) */
#define TIM_EGR_CC3G_Msk                                                   (0x8UL)		/*!< TIM EGR: CC3G (Bitfield-Mask: 0x01) */
#define TIM_EGR_CC3G                                                       TIM_EGR_CC3G_Msk
#define TIM_EGR_CC2G_Pos                                                   (2UL)		/*!<TIM EGR: CC2G (Bit 2) */
#define TIM_EGR_CC2G_Msk                                                   (0x4UL)		/*!< TIM EGR: CC2G (Bitfield-Mask: 0x01) */
#define TIM_EGR_CC2G                                                       TIM_EGR_CC2G_Msk
#define TIM_EGR_CC1G_Pos                                                   (1UL)		/*!<TIM EGR: CC1G (Bit 1) */
#define TIM_EGR_CC1G_Msk                                                   (0x2UL)		/*!< TIM EGR: CC1G (Bitfield-Mask: 0x01) */
#define TIM_EGR_CC1G                                                       TIM_EGR_CC1G_Msk
#define TIM_EGR_UG_Pos                                                     (0UL)		/*!<TIM EGR: UG (Bit 0) */
#define TIM_EGR_UG_Msk                                                     (0x1UL)		/*!< TIM EGR: UG (Bitfield-Mask: 0x01) */
#define TIM_EGR_UG                                                         TIM_EGR_UG_Msk

/* =====================================================    CCMR1    =====================================================*/ 
#define TIM_CCMR1_OC2CE_Pos                                                (15UL)		/*!<TIM CCMR1: OC2CE (Bit 15) */
#define TIM_CCMR1_OC2CE_Msk                                                (0x8000UL)		/*!< TIM CCMR1: OC2CE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC2CE                                                    TIM_CCMR1_OC2CE_Msk
#define TIM_CCMR1_OC2M_Pos                                                 (12UL)		/*!<TIM CCMR1: OC2M (Bit 12) */
#define TIM_CCMR1_OC2M_Msk                                                 (0x1007000UL)		/*!< TIM CCMR1: OC2M (Bitfield-Mask: 0x1007) */
#define TIM_CCMR1_OC2M                                                     TIM_CCMR1_OC2M_Msk
#define TIM_CCMR1_OC2M_0                                                   (0x1U << TIM_CCMR1_OC2M_Pos)
#define TIM_CCMR1_OC2M_1                                                   (0x2U << TIM_CCMR1_OC2M_Pos)
#define TIM_CCMR1_OC2M_2                                                   (0x4U << TIM_CCMR1_OC2M_Pos)
#define TIM_CCMR1_OC2M_3                                                   (0x1000U << TIM_CCMR1_OC2M_Pos)
#define TIM_CCMR1_OC2PE_Pos                                                (11UL)		/*!<TIM CCMR1: OC2PE (Bit 11) */
#define TIM_CCMR1_OC2PE_Msk                                                (0x800UL)		/*!< TIM CCMR1: OC2PE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC2PE                                                    TIM_CCMR1_OC2PE_Msk
#define TIM_CCMR1_OC2FE_Pos                                                (10UL)		/*!<TIM CCMR1: OC2FE (Bit 10) */
#define TIM_CCMR1_OC2FE_Msk                                                (0x400UL)		/*!< TIM CCMR1: OC2FE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC2FE                                                    TIM_CCMR1_OC2FE_Msk
#define TIM_CCMR1_CC2S_Pos                                                 (8UL)		/*!<TIM CCMR1: CC2S (Bit 8) */
#define TIM_CCMR1_CC2S_Msk                                                 (0x300UL)		/*!< TIM CCMR1: CC2S (Bitfield-Mask: 0x03) */
#define TIM_CCMR1_CC2S                                                     TIM_CCMR1_CC2S_Msk
#define TIM_CCMR1_CC2S_0                                                   (0x1U << TIM_CCMR1_CC2S_Pos)
#define TIM_CCMR1_CC2S_1                                                   (0x2U << TIM_CCMR1_CC2S_Pos)
#define TIM_CCMR1_OC1CE_Pos                                                (7UL)		/*!<TIM CCMR1: OC1CE (Bit 7) */
#define TIM_CCMR1_OC1CE_Msk                                                (0x80UL)		/*!< TIM CCMR1: OC1CE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC1CE                                                    TIM_CCMR1_OC1CE_Msk
#define TIM_CCMR1_OC1M_Pos                                                 (4UL)		/*!<TIM CCMR1: OC1M (Bit 4) */
#define TIM_CCMR1_OC1M_Msk                                                 (0x10070UL)		/*!< TIM CCMR1: OC1M (Bitfield-Mask: 0x1007) */
#define TIM_CCMR1_OC1M                                                     TIM_CCMR1_OC1M_Msk
#define TIM_CCMR1_OC1M_0                                                   (0x1U << TIM_CCMR1_OC1M_Pos)
#define TIM_CCMR1_OC1M_1                                                   (0x2U << TIM_CCMR1_OC1M_Pos)
#define TIM_CCMR1_OC1M_2                                                   (0x4U << TIM_CCMR1_OC1M_Pos)
#define TIM_CCMR1_OC1M_3                                                   (0x1000U << TIM_CCMR1_OC1M_Pos)
#define TIM_CCMR1_OC1PE_Pos                                                (3UL)		/*!<TIM CCMR1: OC1PE (Bit 3) */
#define TIM_CCMR1_OC1PE_Msk                                                (0x8UL)		/*!< TIM CCMR1: OC1PE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC1PE                                                    TIM_CCMR1_OC1PE_Msk
#define TIM_CCMR1_OC1FE_Pos                                                (2UL)		/*!<TIM CCMR1: OC1FE (Bit 2) */
#define TIM_CCMR1_OC1FE_Msk                                                (0x4UL)		/*!< TIM CCMR1: OC1FE (Bitfield-Mask: 0x01) */
#define TIM_CCMR1_OC1FE                                                    TIM_CCMR1_OC1FE_Msk
#define TIM_CCMR1_CC1S_Pos                                                 (0UL)		/*!<TIM CCMR1: CC1S (Bit 0) */
#define TIM_CCMR1_CC1S_Msk                                                 (0x3UL)		/*!< TIM CCMR1: CC1S (Bitfield-Mask: 0x03) */
#define TIM_CCMR1_CC1S                                                     TIM_CCMR1_CC1S_Msk
#define TIM_CCMR1_CC1S_0                                                   (0x1U << TIM_CCMR1_CC1S_Pos)
#define TIM_CCMR1_CC1S_1                                                   (0x2U << TIM_CCMR1_CC1S_Pos)

/* =====================================================    CCMR1    =====================================================*/ 
#define TIM_CCMR1_IC2F_Pos                                                 (12UL)		/*!<TIM CCMR1: IC2F (Bit 12) */
#define TIM_CCMR1_IC2F_Msk                                                 (0xf000UL)		/*!< TIM CCMR1: IC2F (Bitfield-Mask: 0x0f) */
#define TIM_CCMR1_IC2F                                                     TIM_CCMR1_IC2F_Msk
#define TIM_CCMR1_IC2F_0                                                   (0x1U << TIM_CCMR1_IC2F_Pos)
#define TIM_CCMR1_IC2F_1                                                   (0x2U << TIM_CCMR1_IC2F_Pos)
#define TIM_CCMR1_IC2F_2                                                   (0x4U << TIM_CCMR1_IC2F_Pos)
#define TIM_CCMR1_IC2F_3                                                   (0x8U << TIM_CCMR1_IC2F_Pos)
#define TIM_CCMR1_IC2PSC_Pos                                               (10UL)		/*!<TIM CCMR1: IC2PSC (Bit 10) */
#define TIM_CCMR1_IC2PSC_Msk                                               (0xc00UL)		/*!< TIM CCMR1: IC2PSC (Bitfield-Mask: 0x03) */
#define TIM_CCMR1_IC2PSC                                                   TIM_CCMR1_IC2PSC_Msk
#define TIM_CCMR1_IC2PSC_0                                                 (0x1U << TIM_CCMR1_IC2PSC_Pos)
#define TIM_CCMR1_IC2PSC_1                                                 (0x2U << TIM_CCMR1_IC2PSC_Pos)
#define TIM_CCMR1_IC1F_Pos                                                 (4UL)		/*!<TIM CCMR1: IC1F (Bit 4) */
#define TIM_CCMR1_IC1F_Msk                                                 (0xf0UL)		/*!< TIM CCMR1: IC1F (Bitfield-Mask: 0x0f) */
#define TIM_CCMR1_IC1F                                                     TIM_CCMR1_IC1F_Msk
#define TIM_CCMR1_IC1F_0                                                   (0x1U << TIM_CCMR1_IC1F_Pos)
#define TIM_CCMR1_IC1F_1                                                   (0x2U << TIM_CCMR1_IC1F_Pos)
#define TIM_CCMR1_IC1F_2                                                   (0x4U << TIM_CCMR1_IC1F_Pos)
#define TIM_CCMR1_IC1F_3                                                   (0x8U << TIM_CCMR1_IC1F_Pos)
#define TIM_CCMR1_IC1PSC_Pos                                               (2UL)		/*!<TIM CCMR1: IC1PSC (Bit 2) */
#define TIM_CCMR1_IC1PSC_Msk                                               (0xcUL)		/*!< TIM CCMR1: IC1PSC (Bitfield-Mask: 0x03) */
#define TIM_CCMR1_IC1PSC                                                   TIM_CCMR1_IC1PSC_Msk
#define TIM_CCMR1_IC1PSC_0                                                 (0x1U << TIM_CCMR1_IC1PSC_Pos)
#define TIM_CCMR1_IC1PSC_1                                                 (0x2U << TIM_CCMR1_IC1PSC_Pos)

/* =====================================================    CCMR2    =====================================================*/ 
#define TIM_CCMR2_IC4F_Pos                                                 (12UL)		/*!<TIM CCMR2: IC4F (Bit 12) */
#define TIM_CCMR2_IC4F_Msk                                                 (0xf000UL)		/*!< TIM CCMR2: IC4F (Bitfield-Mask: 0x0f) */
#define TIM_CCMR2_IC4F                                                     TIM_CCMR2_IC4F_Msk
#define TIM_CCMR2_IC4F_0                                                   (0x1U << TIM_CCMR2_IC4F_Pos)
#define TIM_CCMR2_IC4F_1                                                   (0x2U << TIM_CCMR2_IC4F_Pos)
#define TIM_CCMR2_IC4F_2                                                   (0x4U << TIM_CCMR2_IC4F_Pos)
#define TIM_CCMR2_IC4F_3                                                   (0x8U << TIM_CCMR2_IC4F_Pos)
#define TIM_CCMR2_IC4PSC_Pos                                               (10UL)		/*!<TIM CCMR2: IC4PSC (Bit 10) */
#define TIM_CCMR2_IC4PSC_Msk                                               (0xc00UL)		/*!< TIM CCMR2: IC4PSC (Bitfield-Mask: 0x03) */
#define TIM_CCMR2_IC4PSC                                                   TIM_CCMR2_IC4PSC_Msk
#define TIM_CCMR2_IC4PSC_0                                                 (0x1U << TIM_CCMR2_IC4PSC_Pos)
#define TIM_CCMR2_IC4PSC_1                                                 (0x2U << TIM_CCMR2_IC4PSC_Pos)
#define TIM_CCMR2_CC4S_Pos                                                 (8UL)		/*!<TIM CCMR2: CC4S (Bit 8) */
#define TIM_CCMR2_CC4S_Msk                                                 (0x300UL)		/*!< TIM CCMR2: CC4S (Bitfield-Mask: 0x03) */
#define TIM_CCMR2_CC4S                                                     TIM_CCMR2_CC4S_Msk
#define TIM_CCMR2_CC4S_0                                                   (0x1U << TIM_CCMR2_CC4S_Pos)
#define TIM_CCMR2_CC4S_1                                                   (0x2U << TIM_CCMR2_CC4S_Pos)
#define TIM_CCMR2_IC3F_Pos                                                 (4UL)		/*!<TIM CCMR2: IC3F (Bit 4) */
#define TIM_CCMR2_IC3F_Msk                                                 (0xf0UL)		/*!< TIM CCMR2: IC3F (Bitfield-Mask: 0x0f) */
#define TIM_CCMR2_IC3F                                                     TIM_CCMR2_IC3F_Msk
#define TIM_CCMR2_IC3F_0                                                   (0x1U << TIM_CCMR2_IC3F_Pos)
#define TIM_CCMR2_IC3F_1                                                   (0x2U << TIM_CCMR2_IC3F_Pos)
#define TIM_CCMR2_IC3F_2                                                   (0x4U << TIM_CCMR2_IC3F_Pos)
#define TIM_CCMR2_IC3F_3                                                   (0x8U << TIM_CCMR2_IC3F_Pos)
#define TIM_CCMR2_IC3PSC_Pos                                               (2UL)		/*!<TIM CCMR2: IC3PSC (Bit 2) */
#define TIM_CCMR2_IC3PSC_Msk                                               (0xcUL)		/*!< TIM CCMR2: IC3PSC (Bitfield-Mask: 0x03) */
#define TIM_CCMR2_IC3PSC                                                   TIM_CCMR2_IC3PSC_Msk
#define TIM_CCMR2_IC3PSC_0                                                 (0x1U << TIM_CCMR2_IC3PSC_Pos)
#define TIM_CCMR2_IC3PSC_1                                                 (0x2U << TIM_CCMR2_IC3PSC_Pos)
#define TIM_CCMR2_CC3S_Pos                                                 (0UL)		/*!<TIM CCMR2: CC3S (Bit 0) */
#define TIM_CCMR2_CC3S_Msk                                                 (0x3UL)		/*!< TIM CCMR2: CC3S (Bitfield-Mask: 0x03) */
#define TIM_CCMR2_CC3S                                                     TIM_CCMR2_CC3S_Msk
#define TIM_CCMR2_CC3S_0                                                   (0x1U << TIM_CCMR2_CC3S_Pos)
#define TIM_CCMR2_CC3S_1                                                   (0x2U << TIM_CCMR2_CC3S_Pos)

/* =====================================================    CCMR2    =====================================================*/ 
#define TIM_CCMR2_OC4CE_Pos                                                (15UL)		/*!<TIM CCMR2: OC4CE (Bit 15) */
#define TIM_CCMR2_OC4CE_Msk                                                (0x8000UL)		/*!< TIM CCMR2: OC4CE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC4CE                                                    TIM_CCMR2_OC4CE_Msk
#define TIM_CCMR2_OC4M_Pos                                                 (12UL)		/*!<TIM CCMR2: OC4M (Bit 12) */
#define TIM_CCMR2_OC4M_Msk                                                 (0x1007000UL)		/*!< TIM CCMR2: OC4M (Bitfield-Mask: 0x1007) */
#define TIM_CCMR2_OC4M                                                     TIM_CCMR2_OC4M_Msk
#define TIM_CCMR2_OC4M_0                                                   (0x1U << TIM_CCMR2_OC4M_Pos)
#define TIM_CCMR2_OC4M_1                                                   (0x2U << TIM_CCMR2_OC4M_Pos)
#define TIM_CCMR2_OC4M_2                                                   (0x4U << TIM_CCMR2_OC4M_Pos)
#define TIM_CCMR2_OC4M_3                                                   (0x1000U << TIM_CCMR2_OC4M_Pos)
#define TIM_CCMR2_OC4PE_Pos                                                (11UL)		/*!<TIM CCMR2: OC4PE (Bit 11) */
#define TIM_CCMR2_OC4PE_Msk                                                (0x800UL)		/*!< TIM CCMR2: OC4PE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC4PE                                                    TIM_CCMR2_OC4PE_Msk
#define TIM_CCMR2_OC4FE_Pos                                                (10UL)		/*!<TIM CCMR2: OC4FE (Bit 10) */
#define TIM_CCMR2_OC4FE_Msk                                                (0x400UL)		/*!< TIM CCMR2: OC4FE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC4FE                                                    TIM_CCMR2_OC4FE_Msk
#define TIM_CCMR2_OC3CE_Pos                                                (7UL)		/*!<TIM CCMR2: OC3CE (Bit 7) */
#define TIM_CCMR2_OC3CE_Msk                                                (0x80UL)		/*!< TIM CCMR2: OC3CE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC3CE                                                    TIM_CCMR2_OC3CE_Msk
#define TIM_CCMR2_OC3M_Pos                                                 (4UL)		/*!<TIM CCMR2: OC3M (Bit 4) */
#define TIM_CCMR2_OC3M_Msk                                                 (0x10070UL)		/*!< TIM CCMR2: OC3M (Bitfield-Mask: 0x1007) */
#define TIM_CCMR2_OC3M                                                     TIM_CCMR2_OC3M_Msk
#define TIM_CCMR2_OC3M_0                                                   (0x1U << TIM_CCMR2_OC3M_Pos)
#define TIM_CCMR2_OC3M_1                                                   (0x2U << TIM_CCMR2_OC3M_Pos)
#define TIM_CCMR2_OC3M_2                                                   (0x4U << TIM_CCMR2_OC3M_Pos)
#define TIM_CCMR2_OC3M_3                                                   (0x1000U << TIM_CCMR2_OC3M_Pos)
#define TIM_CCMR2_OC3PE_Pos                                                (3UL)		/*!<TIM CCMR2: OC3PE (Bit 3) */
#define TIM_CCMR2_OC3PE_Msk                                                (0x8UL)		/*!< TIM CCMR2: OC3PE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC3PE                                                    TIM_CCMR2_OC3PE_Msk
#define TIM_CCMR2_OC3FE_Pos                                                (2UL)		/*!<TIM CCMR2: OC3FE (Bit 2) */
#define TIM_CCMR2_OC3FE_Msk                                                (0x4UL)		/*!< TIM CCMR2: OC3FE (Bitfield-Mask: 0x01) */
#define TIM_CCMR2_OC3FE                                                    TIM_CCMR2_OC3FE_Msk

/* =====================================================    CCER    =====================================================*/ 
#define TIM_CCER_CC4NP_Pos                                                 (15UL)		/*!<TIM CCER: CC4NP (Bit 15) */
#define TIM_CCER_CC4NP_Msk                                                 (0x8000UL)		/*!< TIM CCER: CC4NP (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC4NP                                                     TIM_CCER_CC4NP_Msk
#define TIM_CCER_CC4P_Pos                                                  (13UL)		/*!<TIM CCER: CC4P (Bit 13) */
#define TIM_CCER_CC4P_Msk                                                  (0x2000UL)		/*!< TIM CCER: CC4P (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC4P                                                      TIM_CCER_CC4P_Msk
#define TIM_CCER_CC4E_Pos                                                  (12UL)		/*!<TIM CCER: CC4E (Bit 12) */
#define TIM_CCER_CC4E_Msk                                                  (0x1000UL)		/*!< TIM CCER: CC4E (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC4E                                                      TIM_CCER_CC4E_Msk
#define TIM_CCER_CC3NP_Pos                                                 (11UL)		/*!<TIM CCER: CC3NP (Bit 11) */
#define TIM_CCER_CC3NP_Msk                                                 (0x800UL)		/*!< TIM CCER: CC3NP (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC3NP                                                     TIM_CCER_CC3NP_Msk
#define TIM_CCER_CC3P_Pos                                                  (9UL)		/*!<TIM CCER: CC3P (Bit 9) */
#define TIM_CCER_CC3P_Msk                                                  (0x200UL)		/*!< TIM CCER: CC3P (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC3P                                                      TIM_CCER_CC3P_Msk
#define TIM_CCER_CC3E_Pos                                                  (8UL)		/*!<TIM CCER: CC3E (Bit 8) */
#define TIM_CCER_CC3E_Msk                                                  (0x100UL)		/*!< TIM CCER: CC3E (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC3E                                                      TIM_CCER_CC3E_Msk
#define TIM_CCER_CC2NP_Pos                                                 (7UL)		/*!<TIM CCER: CC2NP (Bit 7) */
#define TIM_CCER_CC2NP_Msk                                                 (0x80UL)		/*!< TIM CCER: CC2NP (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC2NP                                                     TIM_CCER_CC2NP_Msk
#define TIM_CCER_CC2P_Pos                                                  (5UL)		/*!<TIM CCER: CC2P (Bit 5) */
#define TIM_CCER_CC2P_Msk                                                  (0x20UL)		/*!< TIM CCER: CC2P (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC2P                                                      TIM_CCER_CC2P_Msk
#define TIM_CCER_CC2E_Pos                                                  (4UL)		/*!<TIM CCER: CC2E (Bit 4) */
#define TIM_CCER_CC2E_Msk                                                  (0x10UL)		/*!< TIM CCER: CC2E (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC2E                                                      TIM_CCER_CC2E_Msk
#define TIM_CCER_CC1NP_Pos                                                 (3UL)		/*!<TIM CCER: CC1NP (Bit 3) */
#define TIM_CCER_CC1NP_Msk                                                 (0x8UL)		/*!< TIM CCER: CC1NP (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC1NP                                                     TIM_CCER_CC1NP_Msk
#define TIM_CCER_CC1NE_Pos                                                 (2UL)		/*!<TIM CCER: CC1NE (Bit 2) */
#define TIM_CCER_CC1NE_Msk                                                 (0x4UL)		/*!< TIM CCER: CC1NE (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC1NE                                                     TIM_CCER_CC1NE_Msk
#define TIM_CCER_CC1P_Pos                                                  (1UL)		/*!<TIM CCER: CC1P (Bit 1) */
#define TIM_CCER_CC1P_Msk                                                  (0x2UL)		/*!< TIM CCER: CC1P (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC1P                                                      TIM_CCER_CC1P_Msk
#define TIM_CCER_CC1E_Pos                                                  (0UL)		/*!<TIM CCER: CC1E (Bit 0) */
#define TIM_CCER_CC1E_Msk                                                  (0x1UL)		/*!< TIM CCER: CC1E (Bitfield-Mask: 0x01) */
#define TIM_CCER_CC1E                                                      TIM_CCER_CC1E_Msk

/* =====================================================    CNT    =====================================================*/ 
#define TIM_CNT_UIFCPY_Pos                                                 (31UL)		/*!<TIM CNT: UIFCPY (Bit 31) */
#define TIM_CNT_UIFCPY_Msk                                                 (0x80000000UL)		/*!< TIM CNT: UIFCPY (Bitfield-Mask: 0x01) */
#define TIM_CNT_UIFCPY                                                     TIM_CNT_UIFCPY_Msk
#define TIM_CNT_CNT_Pos                                                    (0UL)		/*!<TIM CNT: CNT (Bit 0) */
#define TIM_CNT_CNT_Msk                                                    (0xffffUL)		/*!< TIM CNT: CNT (Bitfield-Mask: 0xffff) */
#define TIM_CNT_CNT                                                        TIM_CNT_CNT_Msk
#define TIM_CNT_CNT_0                                                      (0x1U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_1                                                      (0x2U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_2                                                      (0x4U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_3                                                      (0x8U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_4                                                      (0x10U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_5                                                      (0x20U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_6                                                      (0x40U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_7                                                      (0x80U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_8                                                      (0x100U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_9                                                      (0x200U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_10                                                     (0x400U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_11                                                     (0x800U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_12                                                     (0x1000U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_13                                                     (0x2000U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_14                                                     (0x4000U << TIM_CNT_CNT_Pos)
#define TIM_CNT_CNT_15                                                     (0x8000U << TIM_CNT_CNT_Pos)

/* =====================================================    PSC    =====================================================*/ 
#define TIM_PSC_PSC_Pos                                                    (0UL)		/*!<TIM PSC: PSC (Bit 0) */
#define TIM_PSC_PSC_Msk                                                    (0xffffUL)		/*!< TIM PSC: PSC (Bitfield-Mask: 0xffff) */
#define TIM_PSC_PSC                                                        TIM_PSC_PSC_Msk
#define TIM_PSC_PSC_0                                                      (0x1U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_1                                                      (0x2U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_2                                                      (0x4U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_3                                                      (0x8U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_4                                                      (0x10U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_5                                                      (0x20U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_6                                                      (0x40U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_7                                                      (0x80U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_8                                                      (0x100U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_9                                                      (0x200U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_10                                                     (0x400U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_11                                                     (0x800U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_12                                                     (0x1000U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_13                                                     (0x2000U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_14                                                     (0x4000U << TIM_PSC_PSC_Pos)
#define TIM_PSC_PSC_15                                                     (0x8000U << TIM_PSC_PSC_Pos)

/* =====================================================    ARR    =====================================================*/ 
#define TIM_ARR_ARR_Pos                                                    (0UL)		/*!<TIM ARR: ARR (Bit 0) */
#define TIM_ARR_ARR_Msk                                                    (0xffffUL)		/*!< TIM ARR: ARR (Bitfield-Mask: 0xffff) */
#define TIM_ARR_ARR                                                        TIM_ARR_ARR_Msk
#define TIM_ARR_ARR_0                                                      (0x1U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_1                                                      (0x2U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_2                                                      (0x4U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_3                                                      (0x8U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_4                                                      (0x10U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_5                                                      (0x20U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_6                                                      (0x40U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_7                                                      (0x80U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_8                                                      (0x100U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_9                                                      (0x200U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_10                                                     (0x400U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_11                                                     (0x800U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_12                                                     (0x1000U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_13                                                     (0x2000U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_14                                                     (0x4000U << TIM_ARR_ARR_Pos)
#define TIM_ARR_ARR_15                                                     (0x8000U << TIM_ARR_ARR_Pos)

/* =====================================================    RCR    =====================================================*/ 
#define TIM_RCR_REP_Pos                                                    (0UL)		/*!<TIM RCR: REP (Bit 0) */
#define TIM_RCR_REP_Msk                                                    (0xffUL)		/*!< TIM RCR: REP (Bitfield-Mask: 0xff) */
#define TIM_RCR_REP                                                        TIM_RCR_REP_Msk
#define TIM_RCR_REP_0                                                      (0x1U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_1                                                      (0x2U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_2                                                      (0x4U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_3                                                      (0x8U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_4                                                      (0x10U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_5                                                      (0x20U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_6                                                      (0x40U << TIM_RCR_REP_Pos)
#define TIM_RCR_REP_7                                                      (0x80U << TIM_RCR_REP_Pos)

/* =====================================================    CCR1    =====================================================*/ 
#define TIM_CCR1_CCR1_Pos                                                  (0UL)		/*!<TIM CCR1: CCR1 (Bit 0) */
#define TIM_CCR1_CCR1_Msk                                                  (0xffffUL)		/*!< TIM CCR1: CCR1 (Bitfield-Mask: 0xffff) */
#define TIM_CCR1_CCR1                                                      TIM_CCR1_CCR1_Msk
#define TIM_CCR1_CCR1_0                                                    (0x1U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_1                                                    (0x2U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_2                                                    (0x4U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_3                                                    (0x8U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_4                                                    (0x10U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_5                                                    (0x20U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_6                                                    (0x40U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_7                                                    (0x80U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_8                                                    (0x100U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_9                                                    (0x200U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_10                                                   (0x400U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_11                                                   (0x800U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_12                                                   (0x1000U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_13                                                   (0x2000U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_14                                                   (0x4000U << TIM_CCR1_CCR1_Pos)
#define TIM_CCR1_CCR1_15                                                   (0x8000U << TIM_CCR1_CCR1_Pos)

/* =====================================================    CCR2    =====================================================*/ 
#define TIM_CCR2_CCR2_Pos                                                  (0UL)		/*!<TIM CCR2: CCR2 (Bit 0) */
#define TIM_CCR2_CCR2_Msk                                                  (0xffffUL)		/*!< TIM CCR2: CCR2 (Bitfield-Mask: 0xffff) */
#define TIM_CCR2_CCR2                                                      TIM_CCR2_CCR2_Msk
#define TIM_CCR2_CCR2_0                                                    (0x1U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_1                                                    (0x2U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_2                                                    (0x4U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_3                                                    (0x8U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_4                                                    (0x10U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_5                                                    (0x20U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_6                                                    (0x40U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_7                                                    (0x80U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_8                                                    (0x100U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_9                                                    (0x200U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_10                                                   (0x400U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_11                                                   (0x800U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_12                                                   (0x1000U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_13                                                   (0x2000U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_14                                                   (0x4000U << TIM_CCR2_CCR2_Pos)
#define TIM_CCR2_CCR2_15                                                   (0x8000U << TIM_CCR2_CCR2_Pos)

/* =====================================================    CCR3    =====================================================*/ 
#define TIM_CCR3_CCR3_Pos                                                  (0UL)		/*!<TIM CCR3: CCR3 (Bit 0) */
#define TIM_CCR3_CCR3_Msk                                                  (0xffffUL)		/*!< TIM CCR3: CCR3 (Bitfield-Mask: 0xffff) */
#define TIM_CCR3_CCR3                                                      TIM_CCR3_CCR3_Msk
#define TIM_CCR3_CCR3_0                                                    (0x1U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_1                                                    (0x2U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_2                                                    (0x4U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_3                                                    (0x8U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_4                                                    (0x10U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_5                                                    (0x20U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_6                                                    (0x40U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_7                                                    (0x80U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_8                                                    (0x100U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_9                                                    (0x200U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_10                                                   (0x400U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_11                                                   (0x800U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_12                                                   (0x1000U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_13                                                   (0x2000U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_14                                                   (0x4000U << TIM_CCR3_CCR3_Pos)
#define TIM_CCR3_CCR3_15                                                   (0x8000U << TIM_CCR3_CCR3_Pos)

/* =====================================================    CCR4    =====================================================*/ 
#define TIM_CCR4_CCR4_Pos                                                  (0UL)		/*!<TIM CCR4: CCR4 (Bit 0) */
#define TIM_CCR4_CCR4_Msk                                                  (0xffffUL)		/*!< TIM CCR4: CCR4 (Bitfield-Mask: 0xffff) */
#define TIM_CCR4_CCR4                                                      TIM_CCR4_CCR4_Msk
#define TIM_CCR4_CCR4_0                                                    (0x1U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_1                                                    (0x2U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_2                                                    (0x4U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_3                                                    (0x8U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_4                                                    (0x10U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_5                                                    (0x20U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_6                                                    (0x40U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_7                                                    (0x80U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_8                                                    (0x100U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_9                                                    (0x200U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_10                                                   (0x400U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_11                                                   (0x800U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_12                                                   (0x1000U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_13                                                   (0x2000U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_14                                                   (0x4000U << TIM_CCR4_CCR4_Pos)
#define TIM_CCR4_CCR4_15                                                   (0x8000U << TIM_CCR4_CCR4_Pos)

/* =====================================================    BDTR    =====================================================*/ 
#define TIM_BDTR_BKBID_Pos                                                 (28UL)		/*!<TIM BDTR: BKBID (Bit 28) */
#define TIM_BDTR_BKBID_Msk                                                 (0x10000000UL)		/*!< TIM BDTR: BKBID (Bitfield-Mask: 0x01) */
#define TIM_BDTR_BKBID                                                     TIM_BDTR_BKBID_Msk
#define TIM_BDTR_BKDSRM_Pos                                                (26UL)		/*!<TIM BDTR: BKDSRM (Bit 26) */
#define TIM_BDTR_BKDSRM_Msk                                                (0x4000000UL)		/*!< TIM BDTR: BKDSRM (Bitfield-Mask: 0x01) */
#define TIM_BDTR_BKDSRM                                                    TIM_BDTR_BKDSRM_Msk
#define TIM_BDTR_MOE_Pos                                                   (15UL)		/*!<TIM BDTR: MOE (Bit 15) */
#define TIM_BDTR_MOE_Msk                                                   (0x8000UL)		/*!< TIM BDTR: MOE (Bitfield-Mask: 0x01) */
#define TIM_BDTR_MOE                                                       TIM_BDTR_MOE_Msk
#define TIM_BDTR_AOE_Pos                                                   (14UL)		/*!<TIM BDTR: AOE (Bit 14) */
#define TIM_BDTR_AOE_Msk                                                   (0x4000UL)		/*!< TIM BDTR: AOE (Bitfield-Mask: 0x01) */
#define TIM_BDTR_AOE                                                       TIM_BDTR_AOE_Msk
#define TIM_BDTR_BKP_Pos                                                   (13UL)		/*!<TIM BDTR: BKP (Bit 13) */
#define TIM_BDTR_BKP_Msk                                                   (0x2000UL)		/*!< TIM BDTR: BKP (Bitfield-Mask: 0x01) */
#define TIM_BDTR_BKP                                                       TIM_BDTR_BKP_Msk
#define TIM_BDTR_BKE_Pos                                                   (12UL)		/*!<TIM BDTR: BKE (Bit 12) */
#define TIM_BDTR_BKE_Msk                                                   (0x1000UL)		/*!< TIM BDTR: BKE (Bitfield-Mask: 0x01) */
#define TIM_BDTR_BKE                                                       TIM_BDTR_BKE_Msk
#define TIM_BDTR_OSSR_Pos                                                  (11UL)		/*!<TIM BDTR: OSSR (Bit 11) */
#define TIM_BDTR_OSSR_Msk                                                  (0x800UL)		/*!< TIM BDTR: OSSR (Bitfield-Mask: 0x01) */
#define TIM_BDTR_OSSR                                                      TIM_BDTR_OSSR_Msk
#define TIM_BDTR_OSSI_Pos                                                  (10UL)		/*!<TIM BDTR: OSSI (Bit 10) */
#define TIM_BDTR_OSSI_Msk                                                  (0x400UL)		/*!< TIM BDTR: OSSI (Bitfield-Mask: 0x01) */
#define TIM_BDTR_OSSI                                                      TIM_BDTR_OSSI_Msk
#define TIM_BDTR_LOCK_Pos                                                  (8UL)		/*!<TIM BDTR: LOCK (Bit 8) */
#define TIM_BDTR_LOCK_Msk                                                  (0x300UL)		/*!< TIM BDTR: LOCK (Bitfield-Mask: 0x03) */
#define TIM_BDTR_LOCK                                                      TIM_BDTR_LOCK_Msk
#define TIM_BDTR_LOCK_0                                                    (0x1U << TIM_BDTR_LOCK_Pos)
#define TIM_BDTR_LOCK_1                                                    (0x2U << TIM_BDTR_LOCK_Pos)
#define TIM_BDTR_DTG_Pos                                                   (0UL)		/*!<TIM BDTR: DTG (Bit 0) */
#define TIM_BDTR_DTG_Msk                                                   (0xffUL)		/*!< TIM BDTR: DTG (Bitfield-Mask: 0xff) */
#define TIM_BDTR_DTG                                                       TIM_BDTR_DTG_Msk
#define TIM_BDTR_DTG_0                                                     (0x1U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_1                                                     (0x2U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_2                                                     (0x4U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_3                                                     (0x8U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_4                                                     (0x10U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_5                                                     (0x20U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_6                                                     (0x40U << TIM_BDTR_DTG_Pos)
#define TIM_BDTR_DTG_7                                                     (0x80U << TIM_BDTR_DTG_Pos)

/* =====================================================    DCR    =====================================================*/ 
#define TIM_DCR_DBL_Pos                                                    (8UL)		/*!<TIM DCR: DBL (Bit 8) */
#define TIM_DCR_DBL_Msk                                                    (0x1f00UL)		/*!< TIM DCR: DBL (Bitfield-Mask: 0x1f) */
#define TIM_DCR_DBL                                                        TIM_DCR_DBL_Msk
#define TIM_DCR_DBL_0                                                      (0x1U << TIM_DCR_DBL_Pos)
#define TIM_DCR_DBL_1                                                      (0x2U << TIM_DCR_DBL_Pos)
#define TIM_DCR_DBL_2                                                      (0x4U << TIM_DCR_DBL_Pos)
#define TIM_DCR_DBL_3                                                      (0x8U << TIM_DCR_DBL_Pos)
#define TIM_DCR_DBL_4                                                      (0x10U << TIM_DCR_DBL_Pos)
#define TIM_DCR_DBA_Pos                                                    (0UL)		/*!<TIM DCR: DBA (Bit 0) */
#define TIM_DCR_DBA_Msk                                                    (0x1fUL)		/*!< TIM DCR: DBA (Bitfield-Mask: 0x1f) */
#define TIM_DCR_DBA                                                        TIM_DCR_DBA_Msk
#define TIM_DCR_DBA_0                                                      (0x1U << TIM_DCR_DBA_Pos)
#define TIM_DCR_DBA_1                                                      (0x2U << TIM_DCR_DBA_Pos)
#define TIM_DCR_DBA_2                                                      (0x4U << TIM_DCR_DBA_Pos)
#define TIM_DCR_DBA_3                                                      (0x8U << TIM_DCR_DBA_Pos)
#define TIM_DCR_DBA_4                                                      (0x10U << TIM_DCR_DBA_Pos)

/* =====================================================    DMAR    =====================================================*/ 
#define TIM_DMAR_DMAB_Pos                                                  (0UL)		/*!<TIM DMAR: DMAB (Bit 0) */
#define TIM_DMAR_DMAB_Msk                                                  (0xffffUL)		/*!< TIM DMAR: DMAB (Bitfield-Mask: 0xffff) */
#define TIM_DMAR_DMAB                                                      TIM_DMAR_DMAB_Msk
#define TIM_DMAR_DMAB_0                                                    (0x1U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_1                                                    (0x2U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_2                                                    (0x4U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_3                                                    (0x8U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_4                                                    (0x10U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_5                                                    (0x20U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_6                                                    (0x40U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_7                                                    (0x80U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_8                                                    (0x100U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_9                                                    (0x200U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_10                                                   (0x400U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_11                                                   (0x800U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_12                                                   (0x1000U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_13                                                   (0x2000U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_14                                                   (0x4000U << TIM_DMAR_DMAB_Pos)
#define TIM_DMAR_DMAB_15                                                   (0x8000U << TIM_DMAR_DMAB_Pos)

/* ==================================================    TIM2 OR    =================================================== */ 
#define TIM2_OR_TI4_RMP_Pos       (2U)
#define TIM2_OR_TI4_RMP_Msk       (0x1UL << TIM2_OR_TI4_RMP_Pos)             /*!< 0x00000004 */
#define TIM2_OR_TI4_RMP           TIM2_OR_TI4_RMP_Msk                        /*!< TI4 RMP Input capture 4 remap*/
#define TIM2_OR_ETR_RMP_Pos       (0U)
#define TIM2_OR_ETR_RMP_Msk       (0x1UL << TIM2_OR_ETR_RMP_Pos)             /*!< 0x00000001 */
#define TIM2_OR_ETR_RMP           TIM2_OR_ETR_RMP_Msk                        /*!< ETR_RMP External trigger remap */

/* ==================================================    TIM16 OR    =================================================== */ 
#define TIM16_OR_TI1_RMP_Pos      (1U)
#define TIM16_OR_TI1_RMP_Msk      (0x3UL << TIM16_OR_TI1_RMP_Pos)            /*!< 0x00000006 */
#define TIM16_OR_TI1_RMP          TIM16_OR_TI1_RMP_Msk                       /*!<Timer 16 input 1 connection. */
#define TIM16_OR_TI1_RMP_0        (0x1UL << TIM16_OR_TI1_RMP_Pos)            /*!< 0x00000002 */
#define TIM16_OR_TI1_RMP_1        (0x2UL << TIM16_OR_TI1_RMP_Pos)            /*!< 0x00000004 */

/* =====================================================    AF1    =====================================================*/
#define TIM_AF1_ETRSEL_Pos                                                 (14UL)		/*!<TIM AF1: ETRSEL (Bit 14) */
#define TIM_AF1_ETRSEL_Msk                                                 (0x3c000UL)		/*!< TIM AF1: ETRSEL (Bitfield-Mask: 0x0f) */
#define TIM_AF1_ETRSEL                                                     TIM_AF1_ETRSEL_Msk
#define TIM_AF1_ETRSEL_0                                                   (0x1U << TIM_AF1_ETRSEL_Pos)
#define TIM_AF1_ETRSEL_1                                                   (0x2U << TIM_AF1_ETRSEL_Pos)
#define TIM_AF1_ETRSEL_2                                                   (0x4U << TIM_AF1_ETRSEL_Pos)
#define TIM_AF1_ETRSEL_3                                                   (0x8U << TIM_AF1_ETRSEL_Pos)
#define TIM_AF1_BKCMP1P_Pos                                                (10UL)		/*!<TIM AF1: BKCMP1P (Bit 10) */
#define TIM_AF1_BKCMP1P_Msk                                                (0x400UL)		/*!< TIM AF1: BKCMP1P (Bitfield-Mask: 0x01) */
#define TIM_AF1_BKCMP1P                                                    TIM_AF1_BKCMP1P_Msk
#define TIM_AF1_BKINP_Pos                                                  (9UL)		/*!<TIM AF1: BKINP (Bit 9) */
#define TIM_AF1_BKINP_Msk                                                  (0x200UL)		/*!< TIM AF1: BKINP (Bitfield-Mask: 0x01) */
#define TIM_AF1_BKINP                                                      TIM_AF1_BKINP_Msk
#define TIM_AF1_BKCMP1E_Pos                                                (1UL)		/*!<TIM AF1: BKCMP1E (Bit 1) */
#define TIM_AF1_BKCMP1E_Msk                                                (0x2UL)		/*!< TIM AF1: BKCMP1E (Bitfield-Mask: 0x01) */
#define TIM_AF1_BKCMP1E                                                    TIM_AF1_BKCMP1E_Msk
#define TIM_AF1_BKINE_Pos                                                  (0UL)		/*!<TIM AF1: BKINE (Bit 0) */
#define TIM_AF1_BKINE_Msk                                                  (0x1UL)		/*!< TIM AF1: BKINE (Bitfield-Mask: 0x01) */
#define TIM_AF1_BKINE                                                      TIM_AF1_BKINE_Msk

/* =====================================================    TISEL    =====================================================*/ 
#define TIM_TISEL_TI4SEL_Pos                                               (24UL)		/*!<TIM TISEL: TI4SEL (Bit 24) */
#define TIM_TISEL_TI4SEL_Msk                                               (0xf000000UL)		/*!< TIM TISEL: TI4SEL (Bitfield-Mask: 0x0f) */
#define TIM_TISEL_TI4SEL                                                   TIM_TISEL_TI4SEL_Msk
#define TIM_TISEL_TI4SEL_0                                                 (0x1U << TIM_TISEL_TI4SEL_Pos)
#define TIM_TISEL_TI4SEL_1                                                 (0x2U << TIM_TISEL_TI4SEL_Pos)
#define TIM_TISEL_TI4SEL_2                                                 (0x4U << TIM_TISEL_TI4SEL_Pos)
#define TIM_TISEL_TI4SEL_3                                                 (0x8U << TIM_TISEL_TI4SEL_Pos)
#define TIM_TISEL_TI3SEL_Pos                                               (16UL)		/*!<TIM TISEL: TI3SEL (Bit 16) */
#define TIM_TISEL_TI3SEL_Msk                                               (0xf0000UL)		/*!< TIM TISEL: TI3SEL (Bitfield-Mask: 0x0f) */
#define TIM_TISEL_TI3SEL                                                   TIM_TISEL_TI3SEL_Msk
#define TIM_TISEL_TI3SEL_0                                                 (0x1U << TIM_TISEL_TI3SEL_Pos)
#define TIM_TISEL_TI3SEL_1                                                 (0x2U << TIM_TISEL_TI3SEL_Pos)
#define TIM_TISEL_TI3SEL_2                                                 (0x4U << TIM_TISEL_TI3SEL_Pos)
#define TIM_TISEL_TI3SEL_3                                                 (0x8U << TIM_TISEL_TI3SEL_Pos)
#define TIM_TISEL_TI2SEL_Pos                                               (8UL)		/*!<TIM TISEL: TI2SEL (Bit 8) */
#define TIM_TISEL_TI2SEL_Msk                                               (0xf00UL)		/*!< TIM TISEL: TI2SEL (Bitfield-Mask: 0x0f) */
#define TIM_TISEL_TI2SEL                                                   TIM_TISEL_TI2SEL_Msk
#define TIM_TISEL_TI2SEL_0                                                 (0x1U << TIM_TISEL_TI2SEL_Pos)
#define TIM_TISEL_TI2SEL_1                                                 (0x2U << TIM_TISEL_TI2SEL_Pos)
#define TIM_TISEL_TI2SEL_2                                                 (0x4U << TIM_TISEL_TI2SEL_Pos)
#define TIM_TISEL_TI2SEL_3                                                 (0x8U << TIM_TISEL_TI2SEL_Pos)
#define TIM_TISEL_TI1SEL_Pos                                               (0UL)		/*!<TIM TISEL: TI1SEL (Bit 0) */
#define TIM_TISEL_TI1SEL_Msk                                               (0xfUL)		/*!< TIM TISEL: TI1SEL (Bitfield-Mask: 0x0f) */
#define TIM_TISEL_TI1SEL                                                   TIM_TISEL_TI1SEL_Msk
#define TIM_TISEL_TI1SEL_0                                                 (0x1U << TIM_TISEL_TI1SEL_Pos)
#define TIM_TISEL_TI1SEL_1                                                 (0x2U << TIM_TISEL_TI1SEL_Pos)
#define TIM_TISEL_TI1SEL_2                                                 (0x4U << TIM_TISEL_TI1SEL_Pos)
#define TIM_TISEL_TI1SEL_3                                                 (0x8U << TIM_TISEL_TI1SEL_Pos)


/* ============================================================================================================================*/ 
/*=====================                                       IWDG                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    KR    =====================================================*/ 
#define IWDG_KR_KEY_Pos                                                    (0UL)		/*!<IWDG KR: KEY (Bit 0) */
#define IWDG_KR_KEY_Msk                                                    (0xffffUL)		/*!< IWDG KR: KEY (Bitfield-Mask: 0xffff) */
#define IWDG_KR_KEY                                                        IWDG_KR_KEY_Msk
#define IWDG_KR_KEY_0                                                      (0x1U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_1                                                      (0x2U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_2                                                      (0x4U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_3                                                      (0x8U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_4                                                      (0x10U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_5                                                      (0x20U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_6                                                      (0x40U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_7                                                      (0x80U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_8                                                      (0x100U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_9                                                      (0x200U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_10                                                     (0x400U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_11                                                     (0x800U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_12                                                     (0x1000U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_13                                                     (0x2000U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_14                                                     (0x4000U << IWDG_KR_KEY_Pos)
#define IWDG_KR_KEY_15                                                     (0x8000U << IWDG_KR_KEY_Pos)

/* =====================================================    PR    =====================================================*/ 
#define IWDG_PR_PR_Pos                                                     (0UL)		/*!<IWDG PR: PR (Bit 0) */
#define IWDG_PR_PR_Msk                                                     (0x7UL)		/*!< IWDG PR: PR (Bitfield-Mask: 0x07) */
#define IWDG_PR_PR                                                         IWDG_PR_PR_Msk
#define IWDG_PR_PR_0                                                       (0x1U << IWDG_PR_PR_Pos)
#define IWDG_PR_PR_1                                                       (0x2U << IWDG_PR_PR_Pos)
#define IWDG_PR_PR_2                                                       (0x4U << IWDG_PR_PR_Pos)

/* =====================================================    RLR    =====================================================*/ 
#define IWDG_RLR_RL_Pos                                                    (0UL)		/*!<IWDG RLR: RL (Bit 0) */
#define IWDG_RLR_RL_Msk                                                    (0xfffUL)		/*!< IWDG RLR: RL (Bitfield-Mask: 0xfff) */
#define IWDG_RLR_RL                                                        IWDG_RLR_RL_Msk
#define IWDG_RLR_RL_0                                                      (0x1U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_1                                                      (0x2U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_2                                                      (0x4U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_3                                                      (0x8U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_4                                                      (0x10U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_5                                                      (0x20U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_6                                                      (0x40U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_7                                                      (0x80U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_8                                                      (0x100U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_9                                                      (0x200U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_10                                                     (0x400U << IWDG_RLR_RL_Pos)
#define IWDG_RLR_RL_11                                                     (0x800U << IWDG_RLR_RL_Pos)

/* =====================================================    SR    =====================================================*/ 
#define IWDG_SR_WVU_Pos                                                    (2UL)		/*!<IWDG SR: WVU (Bit 2) */
#define IWDG_SR_WVU_Msk                                                    (0x4UL)		/*!< IWDG SR: WVU (Bitfield-Mask: 0x01) */
#define IWDG_SR_WVU                                                        IWDG_SR_WVU_Msk
#define IWDG_SR_RVU_Pos                                                    (1UL)		/*!<IWDG SR: RVU (Bit 1) */
#define IWDG_SR_RVU_Msk                                                    (0x2UL)		/*!< IWDG SR: RVU (Bitfield-Mask: 0x01) */
#define IWDG_SR_RVU                                                        IWDG_SR_RVU_Msk
#define IWDG_SR_PVU_Pos                                                    (0UL)		/*!<IWDG SR: PVU (Bit 0) */
#define IWDG_SR_PVU_Msk                                                    (0x1UL)		/*!< IWDG SR: PVU (Bitfield-Mask: 0x01) */
#define IWDG_SR_PVU                                                        IWDG_SR_PVU_Msk

/* =====================================================    WINR    =====================================================*/ 
#define IWDG_WINR_WIN_Pos                                                  (0UL)		/*!<IWDG WINR: WIN (Bit 0) */
#define IWDG_WINR_WIN_Msk                                                  (0xfffUL)		/*!< IWDG WINR: WIN (Bitfield-Mask: 0xfff) */
#define IWDG_WINR_WIN                                                      IWDG_WINR_WIN_Msk
#define IWDG_WINR_WIN_0                                                    (0x1U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_1                                                    (0x2U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_2                                                    (0x4U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_3                                                    (0x8U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_4                                                    (0x10U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_5                                                    (0x20U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_6                                                    (0x40U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_7                                                    (0x80U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_8                                                    (0x100U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_9                                                    (0x200U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_10                                                   (0x400U << IWDG_WINR_WIN_Pos)
#define IWDG_WINR_WIN_11                                                   (0x800U << IWDG_WINR_WIN_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        RTC                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    TR    =====================================================*/ 
#define RTC_TR_PM_Pos                                                      (22UL)		/*!<RTC TR: PM (Bit 22) */
#define RTC_TR_PM_Msk                                                      (0x400000UL)		/*!< RTC TR: PM (Bitfield-Mask: 0x01) */
#define RTC_TR_PM                                                          RTC_TR_PM_Msk
#define RTC_TR_HT_Pos                                                      (20UL)		/*!<RTC TR: HT (Bit 20) */
#define RTC_TR_HT_Msk                                                      (0x300000UL)		/*!< RTC TR: HT (Bitfield-Mask: 0x03) */
#define RTC_TR_HT                                                          RTC_TR_HT_Msk
#define RTC_TR_HT_0                                                        (0x1U << RTC_TR_HT_Pos)
#define RTC_TR_HT_1                                                        (0x2U << RTC_TR_HT_Pos)
#define RTC_TR_HU_Pos                                                      (16UL)		/*!<RTC TR: HU (Bit 16) */
#define RTC_TR_HU_Msk                                                      (0xf0000UL)		/*!< RTC TR: HU (Bitfield-Mask: 0x0f) */
#define RTC_TR_HU                                                          RTC_TR_HU_Msk
#define RTC_TR_HU_0                                                        (0x1U << RTC_TR_HU_Pos)
#define RTC_TR_HU_1                                                        (0x2U << RTC_TR_HU_Pos)
#define RTC_TR_HU_2                                                        (0x4U << RTC_TR_HU_Pos)
#define RTC_TR_HU_3                                                        (0x8U << RTC_TR_HU_Pos)
#define RTC_TR_MNT_Pos                                                     (12UL)		/*!<RTC TR: MNT (Bit 12) */
#define RTC_TR_MNT_Msk                                                     (0x7000UL)		/*!< RTC TR: MNT (Bitfield-Mask: 0x07) */
#define RTC_TR_MNT                                                         RTC_TR_MNT_Msk
#define RTC_TR_MNT_0                                                       (0x1U << RTC_TR_MNT_Pos)
#define RTC_TR_MNT_1                                                       (0x2U << RTC_TR_MNT_Pos)
#define RTC_TR_MNT_2                                                       (0x4U << RTC_TR_MNT_Pos)
#define RTC_TR_MNU_Pos                                                     (8UL)		/*!<RTC TR: MNU (Bit 8) */
#define RTC_TR_MNU_Msk                                                     (0xf00UL)		/*!< RTC TR: MNU (Bitfield-Mask: 0x0f) */
#define RTC_TR_MNU                                                         RTC_TR_MNU_Msk
#define RTC_TR_MNU_0                                                       (0x1U << RTC_TR_MNU_Pos)
#define RTC_TR_MNU_1                                                       (0x2U << RTC_TR_MNU_Pos)
#define RTC_TR_MNU_2                                                       (0x4U << RTC_TR_MNU_Pos)
#define RTC_TR_MNU_3                                                       (0x8U << RTC_TR_MNU_Pos)
#define RTC_TR_ST_Pos                                                      (4UL)		/*!<RTC TR: ST (Bit 4) */
#define RTC_TR_ST_Msk                                                      (0x70UL)		/*!< RTC TR: ST (Bitfield-Mask: 0x07) */
#define RTC_TR_ST                                                          RTC_TR_ST_Msk
#define RTC_TR_ST_0                                                        (0x1U << RTC_TR_ST_Pos)
#define RTC_TR_ST_1                                                        (0x2U << RTC_TR_ST_Pos)
#define RTC_TR_ST_2                                                        (0x4U << RTC_TR_ST_Pos)
#define RTC_TR_SU_Pos                                                      (0UL)		/*!<RTC TR: SU (Bit 0) */
#define RTC_TR_SU_Msk                                                      (0xfUL)		/*!< RTC TR: SU (Bitfield-Mask: 0x0f) */
#define RTC_TR_SU                                                          RTC_TR_SU_Msk
#define RTC_TR_SU_0                                                        (0x1U << RTC_TR_SU_Pos)
#define RTC_TR_SU_1                                                        (0x2U << RTC_TR_SU_Pos)
#define RTC_TR_SU_2                                                        (0x4U << RTC_TR_SU_Pos)
#define RTC_TR_SU_3                                                        (0x8U << RTC_TR_SU_Pos)

/* =====================================================    DR    =====================================================*/ 
#define RTC_DR_YT_Pos                                                      (20UL)		/*!<RTC DR: YT (Bit 20) */
#define RTC_DR_YT_Msk                                                      (0xf00000UL)		/*!< RTC DR: YT (Bitfield-Mask: 0x0f) */
#define RTC_DR_YT                                                          RTC_DR_YT_Msk
#define RTC_DR_YT_0                                                        (0x1U << RTC_DR_YT_Pos)
#define RTC_DR_YT_1                                                        (0x2U << RTC_DR_YT_Pos)
#define RTC_DR_YT_2                                                        (0x4U << RTC_DR_YT_Pos)
#define RTC_DR_YT_3                                                        (0x8U << RTC_DR_YT_Pos)
#define RTC_DR_YU_Pos                                                      (16UL)		/*!<RTC DR: YU (Bit 16) */
#define RTC_DR_YU_Msk                                                      (0xf0000UL)		/*!< RTC DR: YU (Bitfield-Mask: 0x0f) */
#define RTC_DR_YU                                                          RTC_DR_YU_Msk
#define RTC_DR_YU_0                                                        (0x1U << RTC_DR_YU_Pos)
#define RTC_DR_YU_1                                                        (0x2U << RTC_DR_YU_Pos)
#define RTC_DR_YU_2                                                        (0x4U << RTC_DR_YU_Pos)
#define RTC_DR_YU_3                                                        (0x8U << RTC_DR_YU_Pos)
#define RTC_DR_WDU_Pos                                                     (13UL)		/*!<RTC DR: WDU (Bit 13) */
#define RTC_DR_WDU_Msk                                                     (0xe000UL)		/*!< RTC DR: WDU (Bitfield-Mask: 0x07) */
#define RTC_DR_WDU                                                         RTC_DR_WDU_Msk
#define RTC_DR_WDU_0                                                       (0x1U << RTC_DR_WDU_Pos)
#define RTC_DR_WDU_1                                                       (0x2U << RTC_DR_WDU_Pos)
#define RTC_DR_WDU_2                                                       (0x4U << RTC_DR_WDU_Pos)
#define RTC_DR_MT_Pos                                                      (12UL)		/*!<RTC DR: MT (Bit 12) */
#define RTC_DR_MT_Msk                                                      (0x1000UL)		/*!< RTC DR: MT (Bitfield-Mask: 0x01) */
#define RTC_DR_MT                                                          RTC_DR_MT_Msk
#define RTC_DR_MU_Pos                                                      (8UL)		/*!<RTC DR: MU (Bit 8) */
#define RTC_DR_MU_Msk                                                      (0xf00UL)		/*!< RTC DR: MU (Bitfield-Mask: 0x0f) */
#define RTC_DR_MU                                                          RTC_DR_MU_Msk
#define RTC_DR_MU_0                                                        (0x1U << RTC_DR_MU_Pos)
#define RTC_DR_MU_1                                                        (0x2U << RTC_DR_MU_Pos)
#define RTC_DR_MU_2                                                        (0x4U << RTC_DR_MU_Pos)
#define RTC_DR_MU_3                                                        (0x8U << RTC_DR_MU_Pos)
#define RTC_DR_DT_Pos                                                      (4UL)		/*!<RTC DR: DT (Bit 4) */
#define RTC_DR_DT_Msk                                                      (0x30UL)		/*!< RTC DR: DT (Bitfield-Mask: 0x03) */
#define RTC_DR_DT                                                          RTC_DR_DT_Msk
#define RTC_DR_DT_0                                                        (0x1U << RTC_DR_DT_Pos)
#define RTC_DR_DT_1                                                        (0x2U << RTC_DR_DT_Pos)
#define RTC_DR_DU_Pos                                                      (0UL)		/*!<RTC DR: DU (Bit 0) */
#define RTC_DR_DU_Msk                                                      (0xfUL)		/*!< RTC DR: DU (Bitfield-Mask: 0x0f) */
#define RTC_DR_DU                                                          RTC_DR_DU_Msk
#define RTC_DR_DU_0                                                        (0x1U << RTC_DR_DU_Pos)
#define RTC_DR_DU_1                                                        (0x2U << RTC_DR_DU_Pos)
#define RTC_DR_DU_2                                                        (0x4U << RTC_DR_DU_Pos)
#define RTC_DR_DU_3                                                        (0x8U << RTC_DR_DU_Pos)

/* =====================================================    CR    =====================================================*/ 
#define RTC_CR_ITSE_Pos                                                    (24UL)		/*!<RTC CR: ITSE (Bit 24) */
#define RTC_CR_ITSE_Msk                                                    (0x1000000UL)		/*!< RTC CR: ITSE (Bitfield-Mask: 0x01) */
#define RTC_CR_ITSE                                                        RTC_CR_ITSE_Msk
#define RTC_CR_COE_Pos                                                     (23UL)		/*!<RTC CR: COE (Bit 23) */
#define RTC_CR_COE_Msk                                                     (0x800000UL)		/*!< RTC CR: COE (Bitfield-Mask: 0x01) */
#define RTC_CR_COE                                                         RTC_CR_COE_Msk
#define RTC_CR_OSEL_Pos                                                    (21UL)		/*!<RTC CR: OSEL (Bit 21) */
#define RTC_CR_OSEL_Msk                                                    (0x600000UL)		/*!< RTC CR: OSEL (Bitfield-Mask: 0x03) */
#define RTC_CR_OSEL                                                        RTC_CR_OSEL_Msk
#define RTC_CR_OSEL_0                                                      (0x1U << RTC_CR_OSEL_Pos)
#define RTC_CR_OSEL_1                                                      (0x2U << RTC_CR_OSEL_Pos)
#define RTC_CR_POL_Pos                                                     (20UL)		/*!<RTC CR: POL (Bit 20) */
#define RTC_CR_POL_Msk                                                     (0x100000UL)		/*!< RTC CR: POL (Bitfield-Mask: 0x01) */
#define RTC_CR_POL                                                         RTC_CR_POL_Msk
#define RTC_CR_COSEL_Pos                                                   (19UL)		/*!<RTC CR: COSEL (Bit 19) */
#define RTC_CR_COSEL_Msk                                                   (0x80000UL)		/*!< RTC CR: COSEL (Bitfield-Mask: 0x01) */
#define RTC_CR_COSEL                                                       RTC_CR_COSEL_Msk
#define RTC_CR_BKP_Pos                                                     (18UL)		/*!<RTC CR: BKP (Bit 18) */
#define RTC_CR_BKP_Msk                                                     (0x40000UL)		/*!< RTC CR: BKP (Bitfield-Mask: 0x01) */
#define RTC_CR_BKP                                                         RTC_CR_BKP_Msk
#define RTC_CR_SUB1H_Pos                                                   (17UL)		/*!<RTC CR: SUB1H (Bit 17) */
#define RTC_CR_SUB1H_Msk                                                   (0x20000UL)		/*!< RTC CR: SUB1H (Bitfield-Mask: 0x01) */
#define RTC_CR_SUB1H                                                       RTC_CR_SUB1H_Msk
#define RTC_CR_ADD1H_Pos                                                   (16UL)		/*!<RTC CR: ADD1H (Bit 16) */
#define RTC_CR_ADD1H_Msk                                                   (0x10000UL)		/*!< RTC CR: ADD1H (Bitfield-Mask: 0x01) */
#define RTC_CR_ADD1H                                                       RTC_CR_ADD1H_Msk
#define RTC_CR_TSIE_Pos                                                    (15UL)		/*!<RTC CR: TSIE (Bit 15) */
#define RTC_CR_TSIE_Msk                                                    (0x8000UL)		/*!< RTC CR: TSIE (Bitfield-Mask: 0x01) */
#define RTC_CR_TSIE                                                        RTC_CR_TSIE_Msk
#define RTC_CR_WUTIE_Pos                                                   (14UL)		/*!<RTC CR: WUTIE (Bit 14) */
#define RTC_CR_WUTIE_Msk                                                   (0x4000UL)		/*!< RTC CR: WUTIE (Bitfield-Mask: 0x01) */
#define RTC_CR_WUTIE                                                       RTC_CR_WUTIE_Msk
#define RTC_CR_ALRAIE_Pos                                                  (12UL)		/*!<RTC CR: ALRAIE (Bit 12) */
#define RTC_CR_ALRAIE_Msk                                                  (0x1000UL)		/*!< RTC CR: ALRAIE (Bitfield-Mask: 0x01) */
#define RTC_CR_ALRAIE                                                      RTC_CR_ALRAIE_Msk
#define RTC_CR_TSE_Pos                                                     (11UL)		/*!<RTC CR: TSE (Bit 11) */
#define RTC_CR_TSE_Msk                                                     (0x800UL)		/*!< RTC CR: TSE (Bitfield-Mask: 0x01) */
#define RTC_CR_TSE                                                         RTC_CR_TSE_Msk
#define RTC_CR_WUTE_Pos                                                    (10UL)		/*!<RTC CR: WUTE (Bit 10) */
#define RTC_CR_WUTE_Msk                                                    (0x400UL)		/*!< RTC CR: WUTE (Bitfield-Mask: 0x01) */
#define RTC_CR_WUTE                                                        RTC_CR_WUTE_Msk
#define RTC_CR_ALRAE_Pos                                                   (8UL)		/*!<RTC CR: ALRAE (Bit 8) */
#define RTC_CR_ALRAE_Msk                                                   (0x100UL)		/*!< RTC CR: ALRAE (Bitfield-Mask: 0x01) */
#define RTC_CR_ALRAE                                                       RTC_CR_ALRAE_Msk
#define RTC_CR_FMT_Pos                                                     (6UL)		/*!<RTC CR: FMT (Bit 6) */
#define RTC_CR_FMT_Msk                                                     (0x40UL)		/*!< RTC CR: FMT (Bitfield-Mask: 0x01) */
#define RTC_CR_FMT                                                         RTC_CR_FMT_Msk
#define RTC_CR_BYPSHAD_Pos                                                 (5UL)		/*!<RTC CR: BYPSHAD (Bit 5) */
#define RTC_CR_BYPSHAD_Msk                                                 (0x20UL)		/*!< RTC CR: BYPSHAD (Bitfield-Mask: 0x01) */
#define RTC_CR_BYPSHAD                                                     RTC_CR_BYPSHAD_Msk
#define RTC_CR_TSEDGE_Pos                                                  (3UL)		/*!<RTC CR: TSEDGE (Bit 3) */
#define RTC_CR_TSEDGE_Msk                                                  (0x8UL)		/*!< RTC CR: TSEDGE (Bitfield-Mask: 0x01) */
#define RTC_CR_TSEDGE                                                      RTC_CR_TSEDGE_Msk
#define RTC_CR_WUCKSEL_Pos                                                 (0UL)		/*!<RTC CR: WUCKSEL (Bit 0) */
#define RTC_CR_WUCKSEL_Msk                                                 (0x7UL)		/*!< RTC CR: WUCKSEL (Bitfield-Mask: 0x07) */
#define RTC_CR_WUCKSEL                                                     RTC_CR_WUCKSEL_Msk
#define RTC_CR_WUCKSEL_0                                                   (0x1U << RTC_CR_WUCKSEL_Pos)
#define RTC_CR_WUCKSEL_1                                                   (0x2U << RTC_CR_WUCKSEL_Pos)
#define RTC_CR_WUCKSEL_2                                                   (0x4U << RTC_CR_WUCKSEL_Pos)

/* =====================================================    ISR    =====================================================*/ 
#define RTC_ISR_ITSF_Pos                                                   (17UL)		/*!<RTC ISR: ITSF (Bit 17) */
#define RTC_ISR_ITSF_Msk                                                   (0x20000UL)		/*!< RTC ISR: ITSF (Bitfield-Mask: 0x01) */
#define RTC_ISR_ITSF                                                       RTC_ISR_ITSF_Msk
#define RTC_ISR_RECALPF_Pos                                                (16UL)		/*!<RTC ISR: RECALPF (Bit 16) */
#define RTC_ISR_RECALPF_Msk                                                (0x10000UL)		/*!< RTC ISR: RECALPF (Bitfield-Mask: 0x01) */
#define RTC_ISR_RECALPF                                                    RTC_ISR_RECALPF_Msk
#define RTC_ISR_TAMP1F_Pos                                                 (13UL)		/*!<RTC ISR: TAMP1F (Bit 13) */
#define RTC_ISR_TAMP1F_Msk                                                 (0x2000UL)		/*!< RTC ISR: TAMP1F (Bitfield-Mask: 0x01) */
#define RTC_ISR_TAMP1F                                                     RTC_ISR_TAMP1F_Msk
#define RTC_ISR_TSOVF_Pos                                                  (12UL)		/*!<RTC ISR: TSOVF (Bit 12) */
#define RTC_ISR_TSOVF_Msk                                                  (0x1000UL)		/*!< RTC ISR: TSOVF (Bitfield-Mask: 0x01) */
#define RTC_ISR_TSOVF                                                      RTC_ISR_TSOVF_Msk
#define RTC_ISR_TSF_Pos                                                    (11UL)		/*!<RTC ISR: TSF (Bit 11) */
#define RTC_ISR_TSF_Msk                                                    (0x800UL)		/*!< RTC ISR: TSF (Bitfield-Mask: 0x01) */
#define RTC_ISR_TSF                                                        RTC_ISR_TSF_Msk
#define RTC_ISR_WUTF_Pos                                                   (10UL)		/*!<RTC ISR: WUTF (Bit 10) */
#define RTC_ISR_WUTF_Msk                                                   (0x400UL)		/*!< RTC ISR: WUTF (Bitfield-Mask: 0x01) */
#define RTC_ISR_WUTF                                                       RTC_ISR_WUTF_Msk
#define RTC_ISR_ALRAF_Pos                                                  (8UL)		/*!<RTC ISR: ALRAF (Bit 8) */
#define RTC_ISR_ALRAF_Msk                                                  (0x100UL)		/*!< RTC ISR: ALRAF (Bitfield-Mask: 0x01) */
#define RTC_ISR_ALRAF                                                      RTC_ISR_ALRAF_Msk
#define RTC_ISR_INIT_Pos                                                   (7UL)		/*!<RTC ISR: INIT (Bit 7) */
#define RTC_ISR_INIT_Msk                                                   (0x80UL)		/*!< RTC ISR: INIT (Bitfield-Mask: 0x01) */
#define RTC_ISR_INIT                                                       RTC_ISR_INIT_Msk
#define RTC_ISR_INITF_Pos                                                  (6UL)		/*!<RTC ISR: INITF (Bit 6) */
#define RTC_ISR_INITF_Msk                                                  (0x40UL)		/*!< RTC ISR: INITF (Bitfield-Mask: 0x01) */
#define RTC_ISR_INITF                                                      RTC_ISR_INITF_Msk
#define RTC_ISR_RSF_Pos                                                    (5UL)		/*!<RTC ISR: RSF (Bit 5) */
#define RTC_ISR_RSF_Msk                                                    (0x20UL)		/*!< RTC ISR: RSF (Bitfield-Mask: 0x01) */
#define RTC_ISR_RSF                                                        RTC_ISR_RSF_Msk
#define RTC_ISR_INITS_Pos                                                  (4UL)		/*!<RTC ISR: INITS (Bit 4) */
#define RTC_ISR_INITS_Msk                                                  (0x10UL)		/*!< RTC ISR: INITS (Bitfield-Mask: 0x01) */
#define RTC_ISR_INITS                                                      RTC_ISR_INITS_Msk
#define RTC_ISR_SHPF_Pos                                                   (3UL)		/*!<RTC ISR: SHPF (Bit 3) */
#define RTC_ISR_SHPF_Msk                                                   (0x8UL)		/*!< RTC ISR: SHPF (Bitfield-Mask: 0x01) */
#define RTC_ISR_SHPF                                                       RTC_ISR_SHPF_Msk
#define RTC_ISR_WUTWF_Pos                                                  (2UL)		/*!<RTC ISR: WUTWF (Bit 2) */
#define RTC_ISR_WUTWF_Msk                                                  (0x4UL)		/*!< RTC ISR: WUTWF (Bitfield-Mask: 0x01) */
#define RTC_ISR_WUTWF                                                      RTC_ISR_WUTWF_Msk
#define RTC_ISR_ALRAWF_Pos                                                 (0UL)		/*!<RTC ISR: ALRAWF (Bit 0) */
#define RTC_ISR_ALRAWF_Msk                                                 (0x1UL)		/*!< RTC ISR: ALRAWF (Bitfield-Mask: 0x01) */
#define RTC_ISR_ALRAWF                                                     RTC_ISR_ALRAWF_Msk

/* =====================================================    PRER    =====================================================*/ 
#define RTC_PRER_PREDIV_A_Pos                                              (16UL)		/*!<RTC PRER: PREDIV_A (Bit 16) */
#define RTC_PRER_PREDIV_A_Msk                                              (0x7f0000UL)		/*!< RTC PRER: PREDIV_A (Bitfield-Mask: 0x7f) */
#define RTC_PRER_PREDIV_A                                                  RTC_PRER_PREDIV_A_Msk
#define RTC_PRER_PREDIV_A_0                                                (0x1U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_1                                                (0x2U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_2                                                (0x4U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_3                                                (0x8U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_4                                                (0x10U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_5                                                (0x20U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_A_6                                                (0x40U << RTC_PRER_PREDIV_A_Pos)
#define RTC_PRER_PREDIV_S_Pos                                              (0UL)		/*!<RTC PRER: PREDIV_S (Bit 0) */
#define RTC_PRER_PREDIV_S_Msk                                              (0x7fffUL)		/*!< RTC PRER: PREDIV_S (Bitfield-Mask: 0x7fff) */
#define RTC_PRER_PREDIV_S                                                  RTC_PRER_PREDIV_S_Msk
#define RTC_PRER_PREDIV_S_0                                                (0x1U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_1                                                (0x2U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_2                                                (0x4U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_3                                                (0x8U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_4                                                (0x10U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_5                                                (0x20U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_6                                                (0x40U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_7                                                (0x80U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_8                                                (0x100U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_9                                                (0x200U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_10                                               (0x400U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_11                                               (0x800U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_12                                               (0x1000U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_13                                               (0x2000U << RTC_PRER_PREDIV_S_Pos)
#define RTC_PRER_PREDIV_S_14                                               (0x4000U << RTC_PRER_PREDIV_S_Pos)

/* =====================================================    WUTR    =====================================================*/ 
#define RTC_WUTR_WUT_Pos                                                   (0UL)		/*!<RTC WUTR: WUT (Bit 0) */
#define RTC_WUTR_WUT_Msk                                                   (0xffffUL)		/*!< RTC WUTR: WUT (Bitfield-Mask: 0xffff) */
#define RTC_WUTR_WUT                                                       RTC_WUTR_WUT_Msk
#define RTC_WUTR_WUT_0                                                     (0x1U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_1                                                     (0x2U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_2                                                     (0x4U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_3                                                     (0x8U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_4                                                     (0x10U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_5                                                     (0x20U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_6                                                     (0x40U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_7                                                     (0x80U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_8                                                     (0x100U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_9                                                     (0x200U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_10                                                    (0x400U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_11                                                    (0x800U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_12                                                    (0x1000U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_13                                                    (0x2000U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_14                                                    (0x4000U << RTC_WUTR_WUT_Pos)
#define RTC_WUTR_WUT_15                                                    (0x8000U << RTC_WUTR_WUT_Pos)

/* =====================================================    ALRMAR    =====================================================*/ 
#define RTC_ALRMAR_MSK4_Pos                                                (31UL)		/*!<RTC ALRMAR: MSK4 (Bit 31) */
#define RTC_ALRMAR_MSK4_Msk                                                (0x80000000UL)		/*!< RTC ALRMAR: MSK4 (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_MSK4                                                    RTC_ALRMAR_MSK4_Msk
#define RTC_ALRMAR_WDSEL_Pos                                               (30UL)		/*!<RTC ALRMAR: WDSEL (Bit 30) */
#define RTC_ALRMAR_WDSEL_Msk                                               (0x40000000UL)		/*!< RTC ALRMAR: WDSEL (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_WDSEL                                                   RTC_ALRMAR_WDSEL_Msk
#define RTC_ALRMAR_DT_Pos                                                  (28UL)		/*!<RTC ALRMAR: DT (Bit 28) */
#define RTC_ALRMAR_DT_Msk                                                  (0x30000000UL)		/*!< RTC ALRMAR: DT (Bitfield-Mask: 0x03) */
#define RTC_ALRMAR_DT                                                      RTC_ALRMAR_DT_Msk
#define RTC_ALRMAR_DT_0                                                    (0x1U << RTC_ALRMAR_DT_Pos)
#define RTC_ALRMAR_DT_1                                                    (0x2U << RTC_ALRMAR_DT_Pos)
#define RTC_ALRMAR_DU_Pos                                                  (24UL)		/*!<RTC ALRMAR: DU (Bit 24) */
#define RTC_ALRMAR_DU_Msk                                                  (0xf000000UL)		/*!< RTC ALRMAR: DU (Bitfield-Mask: 0x0f) */
#define RTC_ALRMAR_DU                                                      RTC_ALRMAR_DU_Msk
#define RTC_ALRMAR_DU_0                                                    (0x1U << RTC_ALRMAR_DU_Pos)
#define RTC_ALRMAR_DU_1                                                    (0x2U << RTC_ALRMAR_DU_Pos)
#define RTC_ALRMAR_DU_2                                                    (0x4U << RTC_ALRMAR_DU_Pos)
#define RTC_ALRMAR_DU_3                                                    (0x8U << RTC_ALRMAR_DU_Pos)
#define RTC_ALRMAR_MSK3_Pos                                                (23UL)		/*!<RTC ALRMAR: MSK3 (Bit 23) */
#define RTC_ALRMAR_MSK3_Msk                                                (0x800000UL)		/*!< RTC ALRMAR: MSK3 (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_MSK3                                                    RTC_ALRMAR_MSK3_Msk
#define RTC_ALRMAR_PM_Pos                                                  (22UL)		/*!<RTC ALRMAR: PM (Bit 22) */
#define RTC_ALRMAR_PM_Msk                                                  (0x400000UL)		/*!< RTC ALRMAR: PM (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_PM                                                      RTC_ALRMAR_PM_Msk
#define RTC_ALRMAR_HT_Pos                                                  (20UL)		/*!<RTC ALRMAR: HT (Bit 20) */
#define RTC_ALRMAR_HT_Msk                                                  (0x300000UL)		/*!< RTC ALRMAR: HT (Bitfield-Mask: 0x03) */
#define RTC_ALRMAR_HT                                                      RTC_ALRMAR_HT_Msk
#define RTC_ALRMAR_HT_0                                                    (0x1U << RTC_ALRMAR_HT_Pos)
#define RTC_ALRMAR_HT_1                                                    (0x2U << RTC_ALRMAR_HT_Pos)
#define RTC_ALRMAR_HU_Pos                                                  (16UL)		/*!<RTC ALRMAR: HU (Bit 16) */
#define RTC_ALRMAR_HU_Msk                                                  (0xf0000UL)		/*!< RTC ALRMAR: HU (Bitfield-Mask: 0x0f) */
#define RTC_ALRMAR_HU                                                      RTC_ALRMAR_HU_Msk
#define RTC_ALRMAR_HU_0                                                    (0x1U << RTC_ALRMAR_HU_Pos)
#define RTC_ALRMAR_HU_1                                                    (0x2U << RTC_ALRMAR_HU_Pos)
#define RTC_ALRMAR_HU_2                                                    (0x4U << RTC_ALRMAR_HU_Pos)
#define RTC_ALRMAR_HU_3                                                    (0x8U << RTC_ALRMAR_HU_Pos)
#define RTC_ALRMAR_MSK2_Pos                                                (15UL)		/*!<RTC ALRMAR: MSK2 (Bit 15) */
#define RTC_ALRMAR_MSK2_Msk                                                (0x8000UL)		/*!< RTC ALRMAR: MSK2 (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_MSK2                                                    RTC_ALRMAR_MSK2_Msk
#define RTC_ALRMAR_MNT_Pos                                                 (12UL)		/*!<RTC ALRMAR: MNT (Bit 12) */
#define RTC_ALRMAR_MNT_Msk                                                 (0x7000UL)		/*!< RTC ALRMAR: MNT (Bitfield-Mask: 0x07) */
#define RTC_ALRMAR_MNT                                                     RTC_ALRMAR_MNT_Msk
#define RTC_ALRMAR_MNT_0                                                   (0x1U << RTC_ALRMAR_MNT_Pos)
#define RTC_ALRMAR_MNT_1                                                   (0x2U << RTC_ALRMAR_MNT_Pos)
#define RTC_ALRMAR_MNT_2                                                   (0x4U << RTC_ALRMAR_MNT_Pos)
#define RTC_ALRMAR_MNU_Pos                                                 (8UL)		/*!<RTC ALRMAR: MNU (Bit 8) */
#define RTC_ALRMAR_MNU_Msk                                                 (0xf00UL)		/*!< RTC ALRMAR: MNU (Bitfield-Mask: 0x0f) */
#define RTC_ALRMAR_MNU                                                     RTC_ALRMAR_MNU_Msk
#define RTC_ALRMAR_MNU_0                                                   (0x1U << RTC_ALRMAR_MNU_Pos)
#define RTC_ALRMAR_MNU_1                                                   (0x2U << RTC_ALRMAR_MNU_Pos)
#define RTC_ALRMAR_MNU_2                                                   (0x4U << RTC_ALRMAR_MNU_Pos)
#define RTC_ALRMAR_MNU_3                                                   (0x8U << RTC_ALRMAR_MNU_Pos)
#define RTC_ALRMAR_MSK1_Pos                                                (7UL)		/*!<RTC ALRMAR: MSK1 (Bit 7) */
#define RTC_ALRMAR_MSK1_Msk                                                (0x80UL)		/*!< RTC ALRMAR: MSK1 (Bitfield-Mask: 0x01) */
#define RTC_ALRMAR_MSK1                                                    RTC_ALRMAR_MSK1_Msk
#define RTC_ALRMAR_ST_Pos                                                  (4UL)		/*!<RTC ALRMAR: ST (Bit 4) */
#define RTC_ALRMAR_ST_Msk                                                  (0x70UL)		/*!< RTC ALRMAR: ST (Bitfield-Mask: 0x07) */
#define RTC_ALRMAR_ST                                                      RTC_ALRMAR_ST_Msk
#define RTC_ALRMAR_ST_0                                                    (0x1U << RTC_ALRMAR_ST_Pos)
#define RTC_ALRMAR_ST_1                                                    (0x2U << RTC_ALRMAR_ST_Pos)
#define RTC_ALRMAR_ST_2                                                    (0x4U << RTC_ALRMAR_ST_Pos)
#define RTC_ALRMAR_SU_Pos                                                  (0UL)		/*!<RTC ALRMAR: SU (Bit 0) */
#define RTC_ALRMAR_SU_Msk                                                  (0xfUL)		/*!< RTC ALRMAR: SU (Bitfield-Mask: 0x0f) */
#define RTC_ALRMAR_SU                                                      RTC_ALRMAR_SU_Msk
#define RTC_ALRMAR_SU_0                                                    (0x1U << RTC_ALRMAR_SU_Pos)
#define RTC_ALRMAR_SU_1                                                    (0x2U << RTC_ALRMAR_SU_Pos)
#define RTC_ALRMAR_SU_2                                                    (0x4U << RTC_ALRMAR_SU_Pos)
#define RTC_ALRMAR_SU_3                                                    (0x8U << RTC_ALRMAR_SU_Pos)

/* =====================================================    WPR    =====================================================*/ 
#define RTC_WPR_KEY_Pos                                                    (0UL)		/*!<RTC WPR: KEY (Bit 0) */
#define RTC_WPR_KEY_Msk                                                    (0xffUL)		/*!< RTC WPR: KEY (Bitfield-Mask: 0xff) */
#define RTC_WPR_KEY                                                        RTC_WPR_KEY_Msk
#define RTC_WPR_KEY_0                                                      (0x1U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_1                                                      (0x2U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_2                                                      (0x4U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_3                                                      (0x8U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_4                                                      (0x10U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_5                                                      (0x20U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_6                                                      (0x40U << RTC_WPR_KEY_Pos)
#define RTC_WPR_KEY_7                                                      (0x80U << RTC_WPR_KEY_Pos)

/* =====================================================    SSR    =====================================================*/ 
#define RTC_SSR_SS_Pos                                                     (0UL)		/*!<RTC SSR: SS (Bit 0) */
#define RTC_SSR_SS_Msk                                                     (0xffffUL)		/*!< RTC SSR: SS (Bitfield-Mask: 0xffff) */
#define RTC_SSR_SS                                                         RTC_SSR_SS_Msk
#define RTC_SSR_SS_0                                                       (0x1U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_1                                                       (0x2U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_2                                                       (0x4U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_3                                                       (0x8U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_4                                                       (0x10U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_5                                                       (0x20U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_6                                                       (0x40U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_7                                                       (0x80U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_8                                                       (0x100U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_9                                                       (0x200U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_10                                                      (0x400U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_11                                                      (0x800U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_12                                                      (0x1000U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_13                                                      (0x2000U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_14                                                      (0x4000U << RTC_SSR_SS_Pos)
#define RTC_SSR_SS_15                                                      (0x8000U << RTC_SSR_SS_Pos)

/* =====================================================    SHIFTR    =====================================================*/ 
#define RTC_SHIFTR_ADD1S_Pos                                               (31UL)		/*!<RTC SHIFTR: ADD1S (Bit 31) */
#define RTC_SHIFTR_ADD1S_Msk                                               (0x80000000UL)		/*!< RTC SHIFTR: ADD1S (Bitfield-Mask: 0x01) */
#define RTC_SHIFTR_ADD1S                                                   RTC_SHIFTR_ADD1S_Msk
#define RTC_SHIFTR_SUBFS_Pos                                               (0UL)		/*!<RTC SHIFTR: SUBFS (Bit 0) */
#define RTC_SHIFTR_SUBFS_Msk                                               (0x7fffUL)		/*!< RTC SHIFTR: SUBFS (Bitfield-Mask: 0x7fff) */
#define RTC_SHIFTR_SUBFS                                                   RTC_SHIFTR_SUBFS_Msk
#define RTC_SHIFTR_SUBFS_0                                                 (0x1U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_1                                                 (0x2U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_2                                                 (0x4U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_3                                                 (0x8U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_4                                                 (0x10U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_5                                                 (0x20U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_6                                                 (0x40U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_7                                                 (0x80U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_8                                                 (0x100U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_9                                                 (0x200U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_10                                                (0x400U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_11                                                (0x800U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_12                                                (0x1000U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_13                                                (0x2000U << RTC_SHIFTR_SUBFS_Pos)
#define RTC_SHIFTR_SUBFS_14                                                (0x4000U << RTC_SHIFTR_SUBFS_Pos)

/* =====================================================    TSTR    =====================================================*/ 
#define RTC_TSTR_PM_Pos                                                    (22UL)		/*!<RTC TSTR: PM (Bit 22) */
#define RTC_TSTR_PM_Msk                                                    (0x400000UL)		/*!< RTC TSTR: PM (Bitfield-Mask: 0x01) */
#define RTC_TSTR_PM                                                        RTC_TSTR_PM_Msk
#define RTC_TSTR_HT_Pos                                                    (20UL)		/*!<RTC TSTR: HT (Bit 20) */
#define RTC_TSTR_HT_Msk                                                    (0x300000UL)		/*!< RTC TSTR: HT (Bitfield-Mask: 0x03) */
#define RTC_TSTR_HT                                                        RTC_TSTR_HT_Msk
#define RTC_TSTR_HT_0                                                      (0x1U << RTC_TSTR_HT_Pos)
#define RTC_TSTR_HT_1                                                      (0x2U << RTC_TSTR_HT_Pos)
#define RTC_TSTR_HU_Pos                                                    (16UL)		/*!<RTC TSTR: HU (Bit 16) */
#define RTC_TSTR_HU_Msk                                                    (0xf0000UL)		/*!< RTC TSTR: HU (Bitfield-Mask: 0x0f) */
#define RTC_TSTR_HU                                                        RTC_TSTR_HU_Msk
#define RTC_TSTR_HU_0                                                      (0x1U << RTC_TSTR_HU_Pos)
#define RTC_TSTR_HU_1                                                      (0x2U << RTC_TSTR_HU_Pos)
#define RTC_TSTR_HU_2                                                      (0x4U << RTC_TSTR_HU_Pos)
#define RTC_TSTR_HU_3                                                      (0x8U << RTC_TSTR_HU_Pos)
#define RTC_TSTR_MNT_Pos                                                   (12UL)		/*!<RTC TSTR: MNT (Bit 12) */
#define RTC_TSTR_MNT_Msk                                                   (0x7000UL)		/*!< RTC TSTR: MNT (Bitfield-Mask: 0x07) */
#define RTC_TSTR_MNT                                                       RTC_TSTR_MNT_Msk
#define RTC_TSTR_MNT_0                                                     (0x1U << RTC_TSTR_MNT_Pos)
#define RTC_TSTR_MNT_1                                                     (0x2U << RTC_TSTR_MNT_Pos)
#define RTC_TSTR_MNT_2                                                     (0x4U << RTC_TSTR_MNT_Pos)
#define RTC_TSTR_MNU_Pos                                                   (8UL)		/*!<RTC TSTR: MNU (Bit 8) */
#define RTC_TSTR_MNU_Msk                                                   (0xf00UL)		/*!< RTC TSTR: MNU (Bitfield-Mask: 0x0f) */
#define RTC_TSTR_MNU                                                       RTC_TSTR_MNU_Msk
#define RTC_TSTR_MNU_0                                                     (0x1U << RTC_TSTR_MNU_Pos)
#define RTC_TSTR_MNU_1                                                     (0x2U << RTC_TSTR_MNU_Pos)
#define RTC_TSTR_MNU_2                                                     (0x4U << RTC_TSTR_MNU_Pos)
#define RTC_TSTR_MNU_3                                                     (0x8U << RTC_TSTR_MNU_Pos)
#define RTC_TSTR_ST_Pos                                                    (4UL)		/*!<RTC TSTR: ST (Bit 4) */
#define RTC_TSTR_ST_Msk                                                    (0x70UL)		/*!< RTC TSTR: ST (Bitfield-Mask: 0x07) */
#define RTC_TSTR_ST                                                        RTC_TSTR_ST_Msk
#define RTC_TSTR_ST_0                                                      (0x1U << RTC_TSTR_ST_Pos)
#define RTC_TSTR_ST_1                                                      (0x2U << RTC_TSTR_ST_Pos)
#define RTC_TSTR_ST_2                                                      (0x4U << RTC_TSTR_ST_Pos)
#define RTC_TSTR_SU_Pos                                                    (0UL)		/*!<RTC TSTR: SU (Bit 0) */
#define RTC_TSTR_SU_Msk                                                    (0xfUL)		/*!< RTC TSTR: SU (Bitfield-Mask: 0x0f) */
#define RTC_TSTR_SU                                                        RTC_TSTR_SU_Msk
#define RTC_TSTR_SU_0                                                      (0x1U << RTC_TSTR_SU_Pos)
#define RTC_TSTR_SU_1                                                      (0x2U << RTC_TSTR_SU_Pos)
#define RTC_TSTR_SU_2                                                      (0x4U << RTC_TSTR_SU_Pos)
#define RTC_TSTR_SU_3                                                      (0x8U << RTC_TSTR_SU_Pos)

/* =====================================================    TSDR    =====================================================*/ 
#define RTC_TSDR_WDU_Pos                                                   (13UL)		/*!<RTC TSDR: WDU (Bit 13) */
#define RTC_TSDR_WDU_Msk                                                   (0xe000UL)		/*!< RTC TSDR: WDU (Bitfield-Mask: 0x07) */
#define RTC_TSDR_WDU                                                       RTC_TSDR_WDU_Msk
#define RTC_TSDR_WDU_0                                                     (0x1U << RTC_TSDR_WDU_Pos)
#define RTC_TSDR_WDU_1                                                     (0x2U << RTC_TSDR_WDU_Pos)
#define RTC_TSDR_WDU_2                                                     (0x4U << RTC_TSDR_WDU_Pos)
#define RTC_TSDR_MT_Pos                                                    (12UL)		/*!<RTC TSDR: MT (Bit 12) */
#define RTC_TSDR_MT_Msk                                                    (0x1000UL)		/*!< RTC TSDR: MT (Bitfield-Mask: 0x01) */
#define RTC_TSDR_MT                                                        RTC_TSDR_MT_Msk
#define RTC_TSDR_MU_Pos                                                    (8UL)		/*!<RTC TSDR: MU (Bit 8) */
#define RTC_TSDR_MU_Msk                                                    (0xf00UL)		/*!< RTC TSDR: MU (Bitfield-Mask: 0x0f) */
#define RTC_TSDR_MU                                                        RTC_TSDR_MU_Msk
#define RTC_TSDR_MU_0                                                      (0x1U << RTC_TSDR_MU_Pos)
#define RTC_TSDR_MU_1                                                      (0x2U << RTC_TSDR_MU_Pos)
#define RTC_TSDR_MU_2                                                      (0x4U << RTC_TSDR_MU_Pos)
#define RTC_TSDR_MU_3                                                      (0x8U << RTC_TSDR_MU_Pos)
#define RTC_TSDR_DT_Pos                                                    (4UL)		/*!<RTC TSDR: DT (Bit 4) */
#define RTC_TSDR_DT_Msk                                                    (0x30UL)		/*!< RTC TSDR: DT (Bitfield-Mask: 0x03) */
#define RTC_TSDR_DT                                                        RTC_TSDR_DT_Msk
#define RTC_TSDR_DT_0                                                      (0x1U << RTC_TSDR_DT_Pos)
#define RTC_TSDR_DT_1                                                      (0x2U << RTC_TSDR_DT_Pos)
#define RTC_TSDR_DU_Pos                                                    (0UL)		/*!<RTC TSDR: DU (Bit 0) */
#define RTC_TSDR_DU_Msk                                                    (0xfUL)		/*!< RTC TSDR: DU (Bitfield-Mask: 0x0f) */
#define RTC_TSDR_DU                                                        RTC_TSDR_DU_Msk
#define RTC_TSDR_DU_0                                                      (0x1U << RTC_TSDR_DU_Pos)
#define RTC_TSDR_DU_1                                                      (0x2U << RTC_TSDR_DU_Pos)
#define RTC_TSDR_DU_2                                                      (0x4U << RTC_TSDR_DU_Pos)
#define RTC_TSDR_DU_3                                                      (0x8U << RTC_TSDR_DU_Pos)

/* =====================================================    TSSSR    =====================================================*/ 
#define RTC_TSSSR_SS_Pos                                                   (0UL)		/*!<RTC TSSSR: SS (Bit 0) */
#define RTC_TSSSR_SS_Msk                                                   (0xffffUL)		/*!< RTC TSSSR: SS (Bitfield-Mask: 0xffff) */
#define RTC_TSSSR_SS                                                       RTC_TSSSR_SS_Msk
#define RTC_TSSSR_SS_0                                                     (0x1U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_1                                                     (0x2U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_2                                                     (0x4U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_3                                                     (0x8U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_4                                                     (0x10U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_5                                                     (0x20U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_6                                                     (0x40U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_7                                                     (0x80U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_8                                                     (0x100U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_9                                                     (0x200U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_10                                                    (0x400U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_11                                                    (0x800U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_12                                                    (0x1000U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_13                                                    (0x2000U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_14                                                    (0x4000U << RTC_TSSSR_SS_Pos)
#define RTC_TSSSR_SS_15                                                    (0x8000U << RTC_TSSSR_SS_Pos)

/* =====================================================    CALR    =====================================================*/ 
#define RTC_CALR_CALP_Pos                                                  (15UL)		/*!<RTC CALR: CALP (Bit 15) */
#define RTC_CALR_CALP_Msk                                                  (0x8000UL)		/*!< RTC CALR: CALP (Bitfield-Mask: 0x01) */
#define RTC_CALR_CALP                                                      RTC_CALR_CALP_Msk
#define RTC_CALR_CALW8_Pos                                                 (14UL)		/*!<RTC CALR: CALW8 (Bit 14) */
#define RTC_CALR_CALW8_Msk                                                 (0x4000UL)		/*!< RTC CALR: CALW8 (Bitfield-Mask: 0x01) */
#define RTC_CALR_CALW8                                                     RTC_CALR_CALW8_Msk
#define RTC_CALR_CALW16_Pos                                                (13UL)		/*!<RTC CALR: CALW16 (Bit 13) */
#define RTC_CALR_CALW16_Msk                                                (0x2000UL)		/*!< RTC CALR: CALW16 (Bitfield-Mask: 0x01) */
#define RTC_CALR_CALW16                                                    RTC_CALR_CALW16_Msk
#define RTC_CALR_CALM_Pos                                                  (0UL)		/*!<RTC CALR: CALM (Bit 0) */
#define RTC_CALR_CALM_Msk                                                  (0x1ffUL)		/*!< RTC CALR: CALM (Bitfield-Mask: 0x1ff) */
#define RTC_CALR_CALM                                                      RTC_CALR_CALM_Msk
#define RTC_CALR_CALM_0                                                    (0x1U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_1                                                    (0x2U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_2                                                    (0x4U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_3                                                    (0x8U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_4                                                    (0x10U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_5                                                    (0x20U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_6                                                    (0x40U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_7                                                    (0x80U << RTC_CALR_CALM_Pos)
#define RTC_CALR_CALM_8                                                    (0x100U << RTC_CALR_CALM_Pos)

/* =====================================================    TAMPCR    =====================================================*/ 
#define RTC_TAMPCR_TAMP1MF_Pos                                             (18UL)		/*!<RTC TAMPCR: TAMP1MF (Bit 18) */
#define RTC_TAMPCR_TAMP1MF_Msk                                             (0x40000UL)		/*!< RTC TAMPCR: TAMP1MF (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMP1MF                                                 RTC_TAMPCR_TAMP1MF_Msk
#define RTC_TAMPCR_TAMP1NOERASE_Pos                                        (17UL)		/*!<RTC TAMPCR: TAMP1NOERASE (Bit 17) */
#define RTC_TAMPCR_TAMP1NOERASE_Msk                                        (0x20000UL)		/*!< RTC TAMPCR: TAMP1NOERASE (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMP1NOERASE                                            RTC_TAMPCR_TAMP1NOERASE_Msk
#define RTC_TAMPCR_TAMP1IE_Pos                                             (16UL)		/*!<RTC TAMPCR: TAMP1IE (Bit 16) */
#define RTC_TAMPCR_TAMP1IE_Msk                                             (0x10000UL)		/*!< RTC TAMPCR: TAMP1IE (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMP1IE                                                 RTC_TAMPCR_TAMP1IE_Msk
#define RTC_TAMPCR_TAMPPUDIS_Pos                                           (15UL)		/*!<RTC TAMPCR: TAMPPUDIS (Bit 15) */
#define RTC_TAMPCR_TAMPPUDIS_Msk                                           (0x8000UL)		/*!< RTC TAMPCR: TAMPPUDIS (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMPPUDIS                                               RTC_TAMPCR_TAMPPUDIS_Msk
#define RTC_TAMPCR_TAMPPRCH_Pos                                            (13UL)		/*!<RTC TAMPCR: TAMPPRCH (Bit 13) */
#define RTC_TAMPCR_TAMPPRCH_Msk                                            (0x6000UL)		/*!< RTC TAMPCR: TAMPPRCH (Bitfield-Mask: 0x03) */
#define RTC_TAMPCR_TAMPPRCH                                                RTC_TAMPCR_TAMPPRCH_Msk
#define RTC_TAMPCR_TAMPPRCH_0                                              (0x1U << RTC_TAMPCR_TAMPPRCH_Pos)
#define RTC_TAMPCR_TAMPPRCH_1                                              (0x2U << RTC_TAMPCR_TAMPPRCH_Pos)
#define RTC_TAMPCR_TAMPFLT_Pos                                             (11UL)		/*!<RTC TAMPCR: TAMPFLT (Bit 11) */
#define RTC_TAMPCR_TAMPFLT_Msk                                             (0x1800UL)		/*!< RTC TAMPCR: TAMPFLT (Bitfield-Mask: 0x03) */
#define RTC_TAMPCR_TAMPFLT                                                 RTC_TAMPCR_TAMPFLT_Msk
#define RTC_TAMPCR_TAMPFLT_0                                               (0x1U << RTC_TAMPCR_TAMPFLT_Pos)
#define RTC_TAMPCR_TAMPFLT_1                                               (0x2U << RTC_TAMPCR_TAMPFLT_Pos)
#define RTC_TAMPCR_TAMPFREQ_Pos                                            (8UL)		/*!<RTC TAMPCR: TAMPFREQ (Bit 8) */
#define RTC_TAMPCR_TAMPFREQ_Msk                                            (0x700UL)		/*!< RTC TAMPCR: TAMPFREQ (Bitfield-Mask: 0x07) */
#define RTC_TAMPCR_TAMPFREQ                                                RTC_TAMPCR_TAMPFREQ_Msk
#define RTC_TAMPCR_TAMPFREQ_0                                              (0x1U << RTC_TAMPCR_TAMPFREQ_Pos)
#define RTC_TAMPCR_TAMPFREQ_1                                              (0x2U << RTC_TAMPCR_TAMPFREQ_Pos)
#define RTC_TAMPCR_TAMPFREQ_2                                              (0x4U << RTC_TAMPCR_TAMPFREQ_Pos)
#define RTC_TAMPCR_TAMPTS_Pos                                              (7UL)		/*!<RTC TAMPCR: TAMPTS (Bit 7) */
#define RTC_TAMPCR_TAMPTS_Msk                                              (0x80UL)		/*!< RTC TAMPCR: TAMPTS (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMPTS                                                  RTC_TAMPCR_TAMPTS_Msk
#define RTC_TAMPCR_TAMPIE_Pos                                              (2UL)		/*!<RTC TAMPCR: TAMPIE (Bit 2) */
#define RTC_TAMPCR_TAMPIE_Msk                                              (0x4UL)		/*!< RTC TAMPCR: TAMPIE (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMPIE                                                  RTC_TAMPCR_TAMPIE_Msk
#define RTC_TAMPCR_TAMP1TRG_Pos                                            (1UL)		/*!<RTC TAMPCR: TAMP1TRG (Bit 1) */
#define RTC_TAMPCR_TAMP1TRG_Msk                                            (0x2UL)		/*!< RTC TAMPCR: TAMP1TRG (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMP1TRG                                                RTC_TAMPCR_TAMP1TRG_Msk
#define RTC_TAMPCR_TAMP1E_Pos                                              (0UL)		/*!<RTC TAMPCR: TAMP1E (Bit 0) */
#define RTC_TAMPCR_TAMP1E_Msk                                              (0x1UL)		/*!< RTC TAMPCR: TAMP1E (Bitfield-Mask: 0x01) */
#define RTC_TAMPCR_TAMP1E                                                  RTC_TAMPCR_TAMP1E_Msk

/* =====================================================    ALRMASSR    =====================================================*/ 
#define RTC_ALRMASSR_MASKSS_Pos                                            (24UL)		/*!<RTC ALRMASSR: MASKSS (Bit 24) */
#define RTC_ALRMASSR_MASKSS_Msk                                            (0xf000000UL)		/*!< RTC ALRMASSR: MASKSS (Bitfield-Mask: 0x0f) */
#define RTC_ALRMASSR_MASKSS                                                RTC_ALRMASSR_MASKSS_Msk
#define RTC_ALRMASSR_MASKSS_0                                              (0x1U << RTC_ALRMASSR_MASKSS_Pos)
#define RTC_ALRMASSR_MASKSS_1                                              (0x2U << RTC_ALRMASSR_MASKSS_Pos)
#define RTC_ALRMASSR_MASKSS_2                                              (0x4U << RTC_ALRMASSR_MASKSS_Pos)
#define RTC_ALRMASSR_MASKSS_3                                              (0x8U << RTC_ALRMASSR_MASKSS_Pos)
#define RTC_ALRMASSR_SS_Pos                                                (0UL)		/*!<RTC ALRMASSR: SS (Bit 0) */
#define RTC_ALRMASSR_SS_Msk                                                (0x7fffUL)		/*!< RTC ALRMASSR: SS (Bitfield-Mask: 0x7fff) */
#define RTC_ALRMASSR_SS                                                    RTC_ALRMASSR_SS_Msk
#define RTC_ALRMASSR_SS_0                                                  (0x1U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_1                                                  (0x2U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_2                                                  (0x4U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_3                                                  (0x8U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_4                                                  (0x10U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_5                                                  (0x20U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_6                                                  (0x40U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_7                                                  (0x80U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_8                                                  (0x100U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_9                                                  (0x200U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_10                                                 (0x400U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_11                                                 (0x800U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_12                                                 (0x1000U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_13                                                 (0x2000U << RTC_ALRMASSR_SS_Pos)
#define RTC_ALRMASSR_SS_14                                                 (0x4000U << RTC_ALRMASSR_SS_Pos)

/* =====================================================    OR    =====================================================*/ 
#define RTC_OR_OUT_RMP_Pos                                                 (1UL)		/*!<RTC OR: RTC_OUT_RMP (Bit 1) */
#define RTC_OR_OUT_RMP_Msk                                                 (0x2UL)		/*!< RTC OR: RTC_OUT_RMP (Bitfield-Mask: 0x01) */
#define RTC_OR_OUT_RMP                                                     RTC_OR_OUT_RMP_Msk
#define RTC_OR_ALARMOUTTYPE_Pos                                            (0UL)		/*!<RTC OR: ALARMOUTTYPE (Bit 0) */
#define RTC_OR_ALARMOUTTYPE_Msk                                            (0x1UL)		/*!< RTC OR: ALARMOUTTYPE (Bitfield-Mask: 0x01) */
#define RTC_OR_ALARMOUTTYPE                                                RTC_OR_ALARMOUTTYPE_Msk

/* =====================================================    BKP0R    =====================================================*/ 
#define RTC_BKP0R_BKP_Pos                                                  (0UL)		/*!<RTC BKP0R: BKP (Bit 0) */
#define RTC_BKP0R_BKP_Msk                                                  (0xffffffffUL)		/*!< RTC BKP0R: BKP (Bitfield-Mask: 0xffffffff) */
#define RTC_BKP0R_BKP                                                      RTC_BKP0R_BKP_Msk
#define RTC_BKP0R_BKP_0                                                    (0x1U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_1                                                    (0x2U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_2                                                    (0x4U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_3                                                    (0x8U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_4                                                    (0x10U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_5                                                    (0x20U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_6                                                    (0x40U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_7                                                    (0x80U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_8                                                    (0x100U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_9                                                    (0x200U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_10                                                   (0x400U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_11                                                   (0x800U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_12                                                   (0x1000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_13                                                   (0x2000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_14                                                   (0x4000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_15                                                   (0x8000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_16                                                   (0x10000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_17                                                   (0x20000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_18                                                   (0x40000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_19                                                   (0x80000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_20                                                   (0x100000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_21                                                   (0x200000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_22                                                   (0x400000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_23                                                   (0x800000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_24                                                   (0x1000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_25                                                   (0x2000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_26                                                   (0x4000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_27                                                   (0x8000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_28                                                   (0x10000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_29                                                   (0x20000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_30                                                   (0x40000000U << RTC_BKP0R_BKP_Pos)
#define RTC_BKP0R_BKP_31                                                   (0x80000000UL << RTC_BKP0R_BKP_Pos)

/* =====================================================    BKP1R    =====================================================*/ 
#define RTC_BKP1R_BKP_Pos                                                  (0UL)		/*!<RTC BKP1R: BKP (Bit 0) */
#define RTC_BKP1R_BKP_Msk                                                  (0xffffffffUL)		/*!< RTC BKP1R: BKP (Bitfield-Mask: 0xffffffff) */
#define RTC_BKP1R_BKP                                                      RTC_BKP1R_BKP_Msk
#define RTC_BKP1R_BKP_0                                                    (0x1U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_1                                                    (0x2U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_2                                                    (0x4U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_3                                                    (0x8U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_4                                                    (0x10U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_5                                                    (0x20U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_6                                                    (0x40U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_7                                                    (0x80U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_8                                                    (0x100U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_9                                                    (0x200U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_10                                                   (0x400U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_11                                                   (0x800U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_12                                                   (0x1000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_13                                                   (0x2000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_14                                                   (0x4000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_15                                                   (0x8000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_16                                                   (0x10000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_17                                                   (0x20000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_18                                                   (0x40000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_19                                                   (0x80000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_20                                                   (0x100000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_21                                                   (0x200000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_22                                                   (0x400000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_23                                                   (0x800000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_24                                                   (0x1000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_25                                                   (0x2000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_26                                                   (0x4000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_27                                                   (0x8000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_28                                                   (0x10000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_29                                                   (0x20000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_30                                                   (0x40000000U << RTC_BKP1R_BKP_Pos)
#define RTC_BKP1R_BKP_31                                                   (0x80000000UL << RTC_BKP1R_BKP_Pos)

/******************** Number of backup registers ******************************/
#define RTC_BKP_NUMBER                 (2U)

/* ============================================================================================================================*/ 
/*=====================                                        LCD                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define LCD_CR_BUFEN_Pos                                                   (8UL)		/*!<LCD CR: BUFEN (Bit 8) */
#define LCD_CR_BUFEN_Msk                                                   (0x100UL)		/*!< LCD CR: BUFEN (Bitfield-Mask: 0x01) */
#define LCD_CR_BUFEN                                                       LCD_CR_BUFEN_Msk
#define LCD_CR_BIAS_Pos                                                    (5UL)		/*!<LCD CR: BIAS (Bit 5) */
#define LCD_CR_BIAS_Msk                                                    (0x60UL)		/*!< LCD CR: BIAS (Bitfield-Mask: 0x03) */
#define LCD_CR_BIAS                                                        LCD_CR_BIAS_Msk
#define LCD_CR_BIAS_0                                                      (0x1U << LCD_CR_BIAS_Pos)
#define LCD_CR_BIAS_1                                                      (0x2U << LCD_CR_BIAS_Pos)
#define LCD_CR_DUTY_Pos                                                    (2UL)		/*!<LCD CR: DUTY (Bit 2) */
#define LCD_CR_DUTY_Msk                                                    (0x1cUL)		/*!< LCD CR: DUTY (Bitfield-Mask: 0x07) */
#define LCD_CR_DUTY                                                        LCD_CR_DUTY_Msk
#define LCD_CR_DUTY_0                                                      (0x1U << LCD_CR_DUTY_Pos)
#define LCD_CR_DUTY_1                                                      (0x2U << LCD_CR_DUTY_Pos)
#define LCD_CR_DUTY_2                                                      (0x4U << LCD_CR_DUTY_Pos)
#define LCD_CR_VSEL_Pos                                                    (1UL)		/*!<LCD CR: VSEL (Bit 1) */
#define LCD_CR_VSEL_Msk                                                    (0x2UL)		/*!< LCD CR: VSEL (Bitfield-Mask: 0x01) */
#define LCD_CR_VSEL                                                        LCD_CR_VSEL_Msk
#define LCD_CR_LCDEN_Pos                                                   (0UL)		/*!<LCD CR: LCDEN (Bit 0) */
#define LCD_CR_LCDEN_Msk                                                   (0x1UL)		/*!< LCD CR: LCDEN (Bitfield-Mask: 0x01) */
#define LCD_CR_LCDEN                                                       LCD_CR_LCDEN_Msk

/* =====================================================    FCR    =====================================================*/ 
#define LCD_FCR_PS_Pos                                                     (22UL)		/*!<LCD FCR: PS (Bit 22) */
#define LCD_FCR_PS_Msk                                                     (0x3c00000UL)		/*!< LCD FCR: PS (Bitfield-Mask: 0x0f) */
#define LCD_FCR_PS                                                         LCD_FCR_PS_Msk
#define LCD_FCR_PS_0                                                       (0x1U << LCD_FCR_PS_Pos)
#define LCD_FCR_PS_1                                                       (0x2U << LCD_FCR_PS_Pos)
#define LCD_FCR_PS_2                                                       (0x4U << LCD_FCR_PS_Pos)
#define LCD_FCR_PS_3                                                       (0x8U << LCD_FCR_PS_Pos)
#define LCD_FCR_DIV_Pos                                                    (18UL)		/*!<LCD FCR: DIV (Bit 18) */
#define LCD_FCR_DIV_Msk                                                    (0x3c0000UL)		/*!< LCD FCR: DIV (Bitfield-Mask: 0x0f) */
#define LCD_FCR_DIV                                                        LCD_FCR_DIV_Msk
#define LCD_FCR_DIV_0                                                      (0x1U << LCD_FCR_DIV_Pos)
#define LCD_FCR_DIV_1                                                      (0x2U << LCD_FCR_DIV_Pos)
#define LCD_FCR_DIV_2                                                      (0x4U << LCD_FCR_DIV_Pos)
#define LCD_FCR_DIV_3                                                      (0x8U << LCD_FCR_DIV_Pos)
#define LCD_FCR_BLINK_Pos                                                  (16UL)		/*!<LCD FCR: BLINK (Bit 16) */
#define LCD_FCR_BLINK_Msk                                                  (0x30000UL)		/*!< LCD FCR: BLINK (Bitfield-Mask: 0x03) */
#define LCD_FCR_BLINK                                                      LCD_FCR_BLINK_Msk
#define LCD_FCR_BLINK_0                                                    (0x1U << LCD_FCR_BLINK_Pos)
#define LCD_FCR_BLINK_1                                                    (0x2U << LCD_FCR_BLINK_Pos)
#define LCD_FCR_BLINKF_Pos                                                 (13UL)		/*!<LCD FCR: BLINKF (Bit 13) */
#define LCD_FCR_BLINKF_Msk                                                 (0xe000UL)		/*!< LCD FCR: BLINKF (Bitfield-Mask: 0x07) */
#define LCD_FCR_BLINKF                                                     LCD_FCR_BLINKF_Msk
#define LCD_FCR_BLINKF_0                                                   (0x1U << LCD_FCR_BLINKF_Pos)
#define LCD_FCR_BLINKF_1                                                   (0x2U << LCD_FCR_BLINKF_Pos)
#define LCD_FCR_BLINKF_2                                                   (0x4U << LCD_FCR_BLINKF_Pos)
#define LCD_FCR_CC_Pos                                                     (10UL)		/*!<LCD FCR: CC (Bit 10) */
#define LCD_FCR_CC_Msk                                                     (0x1c00UL)		/*!< LCD FCR: CC (Bitfield-Mask: 0x07) */
#define LCD_FCR_CC                                                         LCD_FCR_CC_Msk
#define LCD_FCR_CC_0                                                       (0x1U << LCD_FCR_CC_Pos)
#define LCD_FCR_CC_1                                                       (0x2U << LCD_FCR_CC_Pos)
#define LCD_FCR_CC_2                                                       (0x4U << LCD_FCR_CC_Pos)
#define LCD_FCR_DEAD_Pos                                                   (7UL)		/*!<LCD FCR: DEAD (Bit 7) */
#define LCD_FCR_DEAD_Msk                                                   (0x380UL)		/*!< LCD FCR: DEAD (Bitfield-Mask: 0x07) */
#define LCD_FCR_DEAD                                                       LCD_FCR_DEAD_Msk
#define LCD_FCR_DEAD_0                                                     (0x1U << LCD_FCR_DEAD_Pos)
#define LCD_FCR_DEAD_1                                                     (0x2U << LCD_FCR_DEAD_Pos)
#define LCD_FCR_DEAD_2                                                     (0x4U << LCD_FCR_DEAD_Pos)
#define LCD_FCR_PON_Pos                                                    (4UL)		/*!<LCD FCR: PON (Bit 4) */
#define LCD_FCR_PON_Msk                                                    (0x70UL)		/*!< LCD FCR: PON (Bitfield-Mask: 0x07) */
#define LCD_FCR_PON                                                        LCD_FCR_PON_Msk
#define LCD_FCR_PON_0                                                      (0x1U << LCD_FCR_PON_Pos)
#define LCD_FCR_PON_1                                                      (0x2U << LCD_FCR_PON_Pos)
#define LCD_FCR_PON_2                                                      (0x4U << LCD_FCR_PON_Pos)
#define LCD_FCR_UDDIE_Pos                                                  (3UL)		/*!<LCD FCR: UDDIE (Bit 3) */
#define LCD_FCR_UDDIE_Msk                                                  (0x8UL)		/*!< LCD FCR: UDDIE (Bitfield-Mask: 0x01) */
#define LCD_FCR_UDDIE                                                      LCD_FCR_UDDIE_Msk
#define LCD_FCR_SOFIE_Pos                                                  (1UL)		/*!<LCD FCR: SOFIE (Bit 1) */
#define LCD_FCR_SOFIE_Msk                                                  (0x2UL)		/*!< LCD FCR: SOFIE (Bitfield-Mask: 0x01) */
#define LCD_FCR_SOFIE                                                      LCD_FCR_SOFIE_Msk
#define LCD_FCR_HD_Pos                                                     (0UL)		/*!<LCD FCR: HD (Bit 0) */
#define LCD_FCR_HD_Msk                                                     (0x1UL)		/*!< LCD FCR: HD (Bitfield-Mask: 0x01) */
#define LCD_FCR_HD                                                         LCD_FCR_HD_Msk

/* =====================================================    SR    =====================================================*/ 
#define LCD_SR_FCRSF_Pos                                                   (5UL)		/*!<LCD SR: FCRSF (Bit 5) */
#define LCD_SR_FCRSF_Msk                                                   (0x20UL)		/*!< LCD SR: FCRSF (Bitfield-Mask: 0x01) */
#define LCD_SR_FCRSF                                                       LCD_SR_FCRSF_Msk
#define LCD_SR_RDY_Pos                                                     (4UL)		/*!<LCD SR: RDY (Bit 4) */
#define LCD_SR_RDY_Msk                                                     (0x10UL)		/*!< LCD SR: RDY (Bitfield-Mask: 0x01) */
#define LCD_SR_RDY                                                         LCD_SR_RDY_Msk
#define LCD_SR_UDD_Pos                                                     (3UL)		/*!<LCD SR: UDD (Bit 3) */
#define LCD_SR_UDD_Msk                                                     (0x8UL)		/*!< LCD SR: UDD (Bitfield-Mask: 0x01) */
#define LCD_SR_UDD                                                         LCD_SR_UDD_Msk
#define LCD_SR_UDR_Pos                                                     (2UL)		/*!<LCD SR: UDR (Bit 2) */
#define LCD_SR_UDR_Msk                                                     (0x4UL)		/*!< LCD SR: UDR (Bitfield-Mask: 0x01) */
#define LCD_SR_UDR                                                         LCD_SR_UDR_Msk
#define LCD_SR_SOF_Pos                                                     (1UL)		/*!<LCD SR: SOF (Bit 1) */
#define LCD_SR_SOF_Msk                                                     (0x2UL)		/*!< LCD SR: SOF (Bitfield-Mask: 0x01) */
#define LCD_SR_SOF                                                         LCD_SR_SOF_Msk
#define LCD_SR_ENS_Pos                                                     (0UL)		/*!<LCD SR: ENS (Bit 0) */
#define LCD_SR_ENS_Msk                                                     (0x1UL)		/*!< LCD SR: ENS (Bitfield-Mask: 0x01) */
#define LCD_SR_ENS                                                         LCD_SR_ENS_Msk

/* =====================================================    CLR    =====================================================*/ 
#define LCD_CLR_UDDC_Pos                                                   (3UL)		/*!<LCD CLR: UDDC (Bit 3) */
#define LCD_CLR_UDDC_Msk                                                   (0x8UL)		/*!< LCD CLR: UDDC (Bitfield-Mask: 0x01) */
#define LCD_CLR_UDDC                                                       LCD_CLR_UDDC_Msk
#define LCD_CLR_SOFC_Pos                                                   (1UL)		/*!<LCD CLR: SOFC (Bit 1) */
#define LCD_CLR_SOFC_Msk                                                   (0x2UL)		/*!< LCD CLR: SOFC (Bitfield-Mask: 0x01) */
#define LCD_CLR_SOFC                                                       LCD_CLR_SOFC_Msk

/* =====================================================    RAM_COM0    =====================================================*/ 
#define LCD_RAM_COM0_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM0: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM0_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM0: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM0_SEGMENT_DATA                                          LCD_RAM_COM0_SEGMENT_DATA_Msk
#define LCD_RAM_COM0_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_12                                       (0x1000U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_13                                       (0x2000U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_14                                       (0x4000U << LCD_RAM_COM0_SEGMENT_DATA_Pos)
#define LCD_RAM_COM0_SEGMENT_DATA_15                                       (0x8000U << LCD_RAM_COM0_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM1    =====================================================*/ 
#define LCD_RAM_COM1_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM1: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM1_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM1: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM1_SEGMENT_DATA                                          LCD_RAM_COM1_SEGMENT_DATA_Msk
#define LCD_RAM_COM1_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_12                                       (0x1000U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_13                                       (0x2000U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_14                                       (0x4000U << LCD_RAM_COM1_SEGMENT_DATA_Pos)
#define LCD_RAM_COM1_SEGMENT_DATA_15                                       (0x8000U << LCD_RAM_COM1_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM2    =====================================================*/ 
#define LCD_RAM_COM2_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM2: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM2_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM2: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM2_SEGMENT_DATA                                          LCD_RAM_COM2_SEGMENT_DATA_Msk
#define LCD_RAM_COM2_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_12                                       (0x1000U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_13                                       (0x2000U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_14                                       (0x4000U << LCD_RAM_COM2_SEGMENT_DATA_Pos)
#define LCD_RAM_COM2_SEGMENT_DATA_15                                       (0x8000U << LCD_RAM_COM2_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM3    =====================================================*/ 
#define LCD_RAM_COM3_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM3: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM3_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM3: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM3_SEGMENT_DATA                                          LCD_RAM_COM3_SEGMENT_DATA_Msk
#define LCD_RAM_COM3_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_12                                       (0x1000U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_13                                       (0x2000U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_14                                       (0x4000U << LCD_RAM_COM3_SEGMENT_DATA_Pos)
#define LCD_RAM_COM3_SEGMENT_DATA_15                                       (0x8000U << LCD_RAM_COM3_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM4    =====================================================*/ 
#define LCD_RAM_COM4_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM4: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM4_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM4: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM4_SEGMENT_DATA                                          LCD_RAM_COM4_SEGMENT_DATA_Msk
#define LCD_RAM_COM4_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM4_SEGMENT_DATA_Pos)
#define LCD_RAM_COM4_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM4_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM5    =====================================================*/ 
#define LCD_RAM_COM5_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM5: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM5_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM5: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM5_SEGMENT_DATA                                          LCD_RAM_COM5_SEGMENT_DATA_Msk
#define LCD_RAM_COM5_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM5_SEGMENT_DATA_Pos)
#define LCD_RAM_COM5_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM5_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM6    =====================================================*/ 
#define LCD_RAM_COM6_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM6: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM6_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM6: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM6_SEGMENT_DATA                                          LCD_RAM_COM6_SEGMENT_DATA_Msk
#define LCD_RAM_COM6_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM6_SEGMENT_DATA_Pos)
#define LCD_RAM_COM6_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM6_SEGMENT_DATA_Pos)

/* =====================================================    RAM_COM7    =====================================================*/ 
#define LCD_RAM_COM7_SEGMENT_DATA_Pos                                      (0UL)		/*!<LCD RAM_COM7: SEGMENT_DATA (Bit 0) */
#define LCD_RAM_COM7_SEGMENT_DATA_Msk                                      (0xffffUL)		/*!< LCD RAM_COM7: SEGMENT_DATA (Bitfield-Mask: 0xffff) */
#define LCD_RAM_COM7_SEGMENT_DATA                                          LCD_RAM_COM7_SEGMENT_DATA_Msk
#define LCD_RAM_COM7_SEGMENT_DATA_0                                        (0x1U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_1                                        (0x2U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_2                                        (0x4U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_3                                        (0x8U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_4                                        (0x10U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_5                                        (0x20U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_6                                        (0x40U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_7                                        (0x80U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_8                                        (0x100U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_9                                        (0x200U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_10                                       (0x400U << LCD_RAM_COM7_SEGMENT_DATA_Pos)
#define LCD_RAM_COM7_SEGMENT_DATA_11                                       (0x800U << LCD_RAM_COM7_SEGMENT_DATA_Pos)


/* ============================================================================================================================*/ 
/*=====================                                      DBGMCU                                      =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define DBGMCU_CR_DBG_STOP_Pos                                             (1UL)		/*!<DBGMCU CR: DBG_STOP (Bit 1) */
#define DBGMCU_CR_DBG_STOP_Msk                                             (0x2UL)		/*!< DBGMCU CR: DBG_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_CR_DBG_STOP                                                 DBGMCU_CR_DBG_STOP_Msk
#define DBGMCU_CR_DBG_SLEEP_Pos                                            (0UL)		/*!<DBGMCU CR: DBG_SLEEP (Bit 0) */
#define DBGMCU_CR_DBG_SLEEP_Msk                                            (0x1UL)		/*!< DBGMCU CR: DBG_SLEEP (Bitfield-Mask: 0x01) */
#define DBGMCU_CR_DBG_SLEEP                                                DBGMCU_CR_DBG_SLEEP_Msk

/* =====================================================    DBG_APB0_FZ    =====================================================*/ 
#define DBGMCU_DBG_APB0_FZ_DBG_IWDG_STOP_Pos                               (14UL)		/*!<DBGMCU DBG_APB0_FZ: DBG_IWDG_STOP (Bit 14) */
#define DBGMCU_DBG_APB0_FZ_DBG_IWDG_STOP_Msk                               (0x4000UL)		/*!< DBGMCU DBG_APB0_FZ: DBG_IWDG_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB0_FZ_DBG_IWDG_STOP                                   DBGMCU_DBG_APB0_FZ_DBG_IWDG_STOP_Msk
#define DBGMCU_DBG_APB0_FZ_DBG_RTC_STOP_Pos                                (12UL)		/*!<DBGMCU DBG_APB0_FZ: DBG_RTC_STOP (Bit 12) */
#define DBGMCU_DBG_APB0_FZ_DBG_RTC_STOP_Msk                                (0x1000UL)		/*!< DBGMCU DBG_APB0_FZ: DBG_RTC_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB0_FZ_DBG_RTC_STOP                                    DBGMCU_DBG_APB0_FZ_DBG_RTC_STOP_Msk
#define DBGMCU_DBG_APB0_FZ_DBG_TIM16_STOP_Pos                              (1UL)		/*!<DBGMCU DBG_APB0_FZ: DBG_TIM16_STOP (Bit 1) */
#define DBGMCU_DBG_APB0_FZ_DBG_TIM16_STOP_Msk                              (0x2UL)		/*!< DBGMCU DBG_APB0_FZ: DBG_TIM16_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB0_FZ_DBG_TIM16_STOP                                  DBGMCU_DBG_APB0_FZ_DBG_TIM16_STOP_Msk
#define DBGMCU_DBG_APB0_FZ_DBG_TIM2_STOP_Pos                               (0UL)		/*!<DBGMCU DBG_APB0_FZ: DBG_TIM2_STOP (Bit 0) */
#define DBGMCU_DBG_APB0_FZ_DBG_TIM2_STOP_Msk                               (0x1UL)		/*!< DBGMCU DBG_APB0_FZ: DBG_TIM2_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB0_FZ_DBG_TIM2_STOP                                   DBGMCU_DBG_APB0_FZ_DBG_TIM2_STOP_Msk

/* =====================================================    DBG_APB1_FZ    =====================================================*/ 
#define DBGMCU_DBG_APB1_FZ_DBG_I2C2_STOP_Pos                               (23UL)		/*!<DBGMCU DBG_APB1_FZ: DBG_I2C2_STOP (Bit 23) */
#define DBGMCU_DBG_APB1_FZ_DBG_I2C2_STOP_Msk                               (0x800000UL)		/*!< DBGMCU DBG_APB1_FZ: DBG_I2C2_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB1_FZ_DBG_I2C2_STOP                                   DBGMCU_DBG_APB1_FZ_DBG_I2C2_STOP_Msk
#define DBGMCU_DBG_APB1_FZ_DBG_I2C1_STOP_Pos                               (21UL)		/*!<DBGMCU DBG_APB1_FZ: DBG_I2C1_STOP (Bit 21) */
#define DBGMCU_DBG_APB1_FZ_DBG_I2C1_STOP_Msk                               (0x200000UL)		/*!< DBGMCU DBG_APB1_FZ: DBG_I2C1_STOP (Bitfield-Mask: 0x01) */
#define DBGMCU_DBG_APB1_FZ_DBG_I2C1_STOP                                   DBGMCU_DBG_APB1_FZ_DBG_I2C1_STOP_Msk


/* ============================================================================================================================*/ 
/*=====================                                       COMP                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CSR    =====================================================*/ 
#define COMP_CSR_LOCK_Pos                                                  (31UL)		/*!<COMP CSR: LOCK (Bit 31) */
#define COMP_CSR_LOCK_Msk                                                  (0x80000000UL)		/*!< COMP CSR: LOCK (Bitfield-Mask: 0x01) */
#define COMP_CSR_LOCK                                                      COMP_CSR_LOCK_Msk
#define COMP_CSR_VALUE_Pos                                                 (30UL)		/*!<COMP CSR: VALUE (Bit 30) */
#define COMP_CSR_VALUE_Msk                                                 (0x40000000UL)		/*!< COMP CSR: VALUE (Bitfield-Mask: 0x01) */
#define COMP_CSR_VALUE                                                     COMP_CSR_VALUE_Msk
#define COMP_CSR_SCALEN_Pos                                                (23UL)		/*!<COMP CSR: SCALEN (Bit 23) */
#define COMP_CSR_SCALEN_Msk                                                (0x800000UL)		/*!< COMP CSR: SCALEN (Bitfield-Mask: 0x01) */
#define COMP_CSR_SCALEN                                                    COMP_CSR_SCALEN_Msk
#define COMP_CSR_BRGEN_Pos                                                 (22UL)		/*!<COMP CSR: BRGEN (Bit 22) */
#define COMP_CSR_BRGEN_Msk                                                 (0x400000UL)		/*!< COMP CSR: BRGEN (Bitfield-Mask: 0x01) */
#define COMP_CSR_BRGEN                                                     COMP_CSR_BRGEN_Msk
#define COMP_CSR_BLANKING_Pos                                              (18UL)		/*!<COMP CSR: BLANKING (Bit 18) */
#define COMP_CSR_BLANKING_Msk                                              (0x1c0000UL)		/*!< COMP CSR: BLANKING (Bitfield-Mask: 0x07) */
#define COMP_CSR_BLANKING                                                  COMP_CSR_BLANKING_Msk
#define COMP_CSR_BLANKING_0                                                (0x1U << COMP_CSR_BLANKING_Pos)
#define COMP_CSR_BLANKING_1                                                (0x2U << COMP_CSR_BLANKING_Pos)
#define COMP_CSR_BLANKING_2                                                (0x4U << COMP_CSR_BLANKING_Pos)
#define COMP_CSR_HYST_Pos                                                  (16UL)		/*!<COMP CSR: HYST (Bit 16) */
#define COMP_CSR_HYST_Msk                                                  (0x30000UL)		/*!< COMP CSR: HYST (Bitfield-Mask: 0x03) */
#define COMP_CSR_HYST                                                      COMP_CSR_HYST_Msk
#define COMP_CSR_HYST_0                                                    (0x1U << COMP_CSR_HYST_Pos)
#define COMP_CSR_HYST_1                                                    (0x2U << COMP_CSR_HYST_Pos)
#define COMP_CSR_POLARITY_Pos                                              (15UL)		/*!<COMP CSR: POLARITY (Bit 15) */
#define COMP_CSR_POLARITY_Msk                                              (0x8000UL)		/*!< COMP CSR: POLARITY (Bitfield-Mask: 0x01) */
#define COMP_CSR_POLARITY                                                  COMP_CSR_POLARITY_Msk
#define COMP_CSR_INPSEL_Pos                                                (7UL)		/*!<COMP CSR: INPSEL (Bit 7) */
#define COMP_CSR_INPSEL_Msk                                                (0x180UL)		/*!< COMP CSR: INPSEL (Bitfield-Mask: 0x03) */
#define COMP_CSR_INPSEL                                                    COMP_CSR_INPSEL_Msk
#define COMP_CSR_INPSEL_0                                                  (0x1U << COMP_CSR_INPSEL_Pos)
#define COMP_CSR_INPSEL_1                                                  (0x2U << COMP_CSR_INPSEL_Pos)
#define COMP_CSR_INMSEL_Pos                                                (4UL)		/*!<COMP CSR: INMSEL (Bit 4) */
#define COMP_CSR_INMSEL_Msk                                                (0x70UL)		/*!< COMP CSR: INMSEL (Bitfield-Mask: 0x07) */
#define COMP_CSR_INMSEL                                                    COMP_CSR_INMSEL_Msk
#define COMP_CSR_INMSEL_0                                                  (0x1U << COMP_CSR_INMSEL_Pos)
#define COMP_CSR_INMSEL_1                                                  (0x2U << COMP_CSR_INMSEL_Pos)
#define COMP_CSR_INMSEL_2                                                  (0x4U << COMP_CSR_INMSEL_Pos)
#define COMP_CSR_PWRMODE_Pos                                               (2UL)		/*!<COMP CSR: PWRMODE (Bit 2) */
#define COMP_CSR_PWRMODE_Msk                                               (0xcUL)		/*!< COMP CSR: PWRMODE (Bitfield-Mask: 0x03) */
#define COMP_CSR_PWRMODE                                                   COMP_CSR_PWRMODE_Msk
#define COMP_CSR_PWRMODE_0                                                 (0x1U << COMP_CSR_PWRMODE_Pos)
#define COMP_CSR_PWRMODE_1                                                 (0x2U << COMP_CSR_PWRMODE_Pos)
#define COMP_CSR_EN_Pos                                                    (0UL)		/*!<COMP CSR: EN (Bit 0) */
#define COMP_CSR_EN_Msk                                                    (0x1UL)		/*!< COMP CSR: EN (Bitfield-Mask: 0x01) */
#define COMP_CSR_EN                                                        COMP_CSR_EN_Msk


/* ============================================================================================================================*/ 
/*=====================                                        DAC                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define DAC_CR_VCMON_Pos                                                   (16UL)		/*!<DAC CR: VCMON (Bit 16) */
#define DAC_CR_VCMON_Msk                                                   (0x10000UL)		/*!< DAC CR: VCMON (Bitfield-Mask: 0x01) */
#define DAC_CR_VCMON                                                       DAC_CR_VCMON_Msk
#define DAC_CR_VCMEN_Pos                                                   (15UL)		/*!<DAC CR: VCMEN (Bit 15) */
#define DAC_CR_VCMEN_Msk                                                   (0x8000UL)		/*!< DAC CR: VCMEN (Bitfield-Mask: 0x01) */
#define DAC_CR_VCMEN                                                       DAC_CR_VCMEN_Msk
#define DAC_CR_CMPEN_Pos                                                   (14UL)		/*!<DAC CR: CMPEN (Bit 14) */
#define DAC_CR_CMPEN_Msk                                                   (0x4000UL)		/*!< DAC CR: CMPEN (Bitfield-Mask: 0x01) */
#define DAC_CR_CMPEN                                                       DAC_CR_CMPEN_Msk
#define DAC_CR_DMAUDRIE_Pos                                                (13UL)		/*!<DAC CR: DMAUDRIE (Bit 13) */
#define DAC_CR_DMAUDRIE_Msk                                                (0x2000UL)		/*!< DAC CR: DMAUDRIE (Bitfield-Mask: 0x01) */
#define DAC_CR_DMAUDRIE                                                    DAC_CR_DMAUDRIE_Msk
#define DAC_CR_DMAEN_Pos                                                   (12UL)		/*!<DAC CR: DMAEN (Bit 12) */
#define DAC_CR_DMAEN_Msk                                                   (0x1000UL)		/*!< DAC CR: DMAEN (Bitfield-Mask: 0x01) */
#define DAC_CR_DMAEN                                                       DAC_CR_DMAEN_Msk
#define DAC_CR_MAMP_Pos                                                    (8UL)		/*!<DAC CR: MAMP (Bit 8) */
#define DAC_CR_MAMP_Msk                                                    (0xf00UL)		/*!< DAC CR: MAMP (Bitfield-Mask: 0x0f) */
#define DAC_CR_MAMP                                                        DAC_CR_MAMP_Msk
#define DAC_CR_MAMP_0                                                      (0x1U << DAC_CR_MAMP_Pos)
#define DAC_CR_MAMP_1                                                      (0x2U << DAC_CR_MAMP_Pos)
#define DAC_CR_MAMP_2                                                      (0x4U << DAC_CR_MAMP_Pos)
#define DAC_CR_MAMP_3                                                      (0x8U << DAC_CR_MAMP_Pos)
#define DAC_CR_WAVE_Pos                                                    (6UL)		/*!<DAC CR: WAVE (Bit 6) */
#define DAC_CR_WAVE_Msk                                                    (0xc0UL)		/*!< DAC CR: WAVE (Bitfield-Mask: 0x03) */
#define DAC_CR_WAVE                                                        DAC_CR_WAVE_Msk
#define DAC_CR_WAVE_0                                                      (0x1U << DAC_CR_WAVE_Pos)
#define DAC_CR_WAVE_1                                                      (0x2U << DAC_CR_WAVE_Pos)
#define DAC_CR_TSEL_Pos                                                    (3UL)		/*!<DAC CR: TSEL (Bit 3) */
#define DAC_CR_TSEL_Msk                                                    (0x38UL)		/*!< DAC CR: TSEL (Bitfield-Mask: 0x07) */
#define DAC_CR_TSEL                                                        DAC_CR_TSEL_Msk
#define DAC_CR_TSEL_0                                                      (0x1U << DAC_CR_TSEL_Pos)
#define DAC_CR_TSEL_1                                                      (0x2U << DAC_CR_TSEL_Pos)
#define DAC_CR_TSEL_2                                                      (0x4U << DAC_CR_TSEL_Pos)
#define DAC_CR_TEN_Pos                                                     (2UL)		/*!<DAC CR: TEN (Bit 2) */
#define DAC_CR_TEN_Msk                                                     (0x4UL)		/*!< DAC CR: TEN (Bitfield-Mask: 0x01) */
#define DAC_CR_TEN                                                         DAC_CR_TEN_Msk
#define DAC_CR_BON_Pos                                                     (1UL)		/*!<DAC CR: BON (Bit 1) */
#define DAC_CR_BON_Msk                                                     (0x2UL)		/*!< DAC CR: BON (Bitfield-Mask: 0x01) */
#define DAC_CR_BON                                                         DAC_CR_BON_Msk
#define DAC_CR_EN_Pos                                                      (0UL)		/*!<DAC CR: EN (Bit 0) */
#define DAC_CR_EN_Msk                                                      (0x1UL)		/*!< DAC CR: EN (Bitfield-Mask: 0x01) */
#define DAC_CR_EN                                                          DAC_CR_EN_Msk

/* =====================================================    SWTRIGR    =====================================================*/ 
#define DAC_SWTRIGR_SWTRIG_Pos                                             (0UL)		/*!<DAC SWTRIGR: SWTRIG (Bit 0) */
#define DAC_SWTRIGR_SWTRIG_Msk                                             (0x1UL)		/*!< DAC SWTRIGR: SWTRIG (Bitfield-Mask: 0x01) */
#define DAC_SWTRIGR_SWTRIG                                                 DAC_SWTRIGR_SWTRIG_Msk

/* =====================================================    DHR    =====================================================*/ 
#define DAC_DHR_DACDHR_Pos                                                 (0UL)		/*!<DAC DHR: DACDHR (Bit 0) */
#define DAC_DHR_DACDHR_Msk                                                 (0x3fUL)		/*!< DAC DHR: DACDHR (Bitfield-Mask: 0x3f) */
#define DAC_DHR_DACDHR                                                     DAC_DHR_DACDHR_Msk
#define DAC_DHR_DACDHR_0                                                   (0x1U << DAC_DHR_DACDHR_Pos)
#define DAC_DHR_DACDHR_1                                                   (0x2U << DAC_DHR_DACDHR_Pos)
#define DAC_DHR_DACDHR_2                                                   (0x4U << DAC_DHR_DACDHR_Pos)
#define DAC_DHR_DACDHR_3                                                   (0x8U << DAC_DHR_DACDHR_Pos)
#define DAC_DHR_DACDHR_4                                                   (0x10U << DAC_DHR_DACDHR_Pos)
#define DAC_DHR_DACDHR_5                                                   (0x20U << DAC_DHR_DACDHR_Pos)

/* =====================================================    DOR    =====================================================*/ 
#define DAC_DOR_DACDOR_Pos                                                 (0UL)		/*!<DAC DOR: DACDOR (Bit 0) */
#define DAC_DOR_DACDOR_Msk                                                 (0x3fUL)		/*!< DAC DOR: DACDOR (Bitfield-Mask: 0x3f) */
#define DAC_DOR_DACDOR                                                     DAC_DOR_DACDOR_Msk
#define DAC_DOR_DACDOR_0                                                   (0x1U << DAC_DOR_DACDOR_Pos)
#define DAC_DOR_DACDOR_1                                                   (0x2U << DAC_DOR_DACDOR_Pos)
#define DAC_DOR_DACDOR_2                                                   (0x4U << DAC_DOR_DACDOR_Pos)
#define DAC_DOR_DACDOR_3                                                   (0x8U << DAC_DOR_DACDOR_Pos)
#define DAC_DOR_DACDOR_4                                                   (0x10U << DAC_DOR_DACDOR_Pos)
#define DAC_DOR_DACDOR_5                                                   (0x20U << DAC_DOR_DACDOR_Pos)

/* =====================================================    SR    =====================================================*/ 
#define DAC_SR_DMAUDR_Pos                                                  (13UL)		/*!<DAC SR: DMAUDR (Bit 13) */
#define DAC_SR_DMAUDR_Msk                                                  (0x2000UL)		/*!< DAC SR: DMAUDR (Bitfield-Mask: 0x01) */
#define DAC_SR_DMAUDR                                                      DAC_SR_DMAUDR_Msk


/* ============================================================================================================================*/ 
/*=====================                                       LCSC                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR0    =====================================================*/ 
#define LCSC_CR0_TICAP_Pos                                                 (24UL)		/*!<LCSC CR0: TICAP (Bit 24) */
#define LCSC_CR0_TICAP_Msk                                                 (0x7000000UL)		/*!< LCSC CR0: TICAP (Bitfield-Mask: 0x07) */
#define LCSC_CR0_TICAP                                                     LCSC_CR0_TICAP_Msk
#define LCSC_CR0_TICAP_0                                                   (0x1U << LCSC_CR0_TICAP_Pos)
#define LCSC_CR0_TICAP_1                                                   (0x2U << LCSC_CR0_TICAP_Pos)
#define LCSC_CR0_TICAP_2                                                   (0x4U << LCSC_CR0_TICAP_Pos)
#define LCSC_CR0_TCAP_Pos                                                  (16UL)		/*!<LCSC CR0: TCAP (Bit 16) */
#define LCSC_CR0_TCAP_Msk                                                  (0x3f0000UL)		/*!< LCSC CR0: TCAP (Bitfield-Mask: 0x3f) */
#define LCSC_CR0_TCAP                                                      LCSC_CR0_TCAP_Msk
#define LCSC_CR0_TCAP_0                                                    (0x1U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TCAP_1                                                    (0x2U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TCAP_2                                                    (0x4U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TCAP_3                                                    (0x8U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TCAP_4                                                    (0x10U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TCAP_5                                                    (0x20U << LCSC_CR0_TCAP_Pos)
#define LCSC_CR0_TMEAS_Pos                                                 (0UL)		/*!<LCSC CR0: TMEAS (Bit 0) */
#define LCSC_CR0_TMEAS_Msk                                                 (0x3fffUL)		/*!< LCSC CR0: TMEAS (Bitfield-Mask: 0x3fff) */
#define LCSC_CR0_TMEAS                                                     LCSC_CR0_TMEAS_Msk
#define LCSC_CR0_TMEAS_0                                                   (0x1U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_1                                                   (0x2U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_2                                                   (0x4U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_3                                                   (0x8U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_4                                                   (0x10U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_5                                                   (0x20U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_6                                                   (0x40U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_7                                                   (0x80U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_8                                                   (0x100U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_9                                                   (0x200U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_10                                                  (0x400U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_11                                                  (0x800U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_12                                                  (0x1000U << LCSC_CR0_TMEAS_Pos)
#define LCSC_CR0_TMEAS_13                                                  (0x2000U << LCSC_CR0_TMEAS_Pos)

/* =====================================================    CR1    =====================================================*/ 
#define LCSC_CR1_TSTART_VCM_Pos                                            (20UL)
#define LCSC_CR1_TSTART_VCM_Msk                                            (0x7ff00000UL)
#define LCSC_CR1_TSTART_VCM                                                LCSC_CR1_TSTART_VCM_Msk
#define LCSC_CR1_TREC_VCM_Pos                                              (10UL)		/*!<LCSC CR1: TREC_VCM (Bit 10) */
#define LCSC_CR1_TREC_VCM_Msk                                              (0x7fc00UL)		/*!< LCSC CR1: TREC_VCM (Bitfield-Mask: 0x1ff) */
#define LCSC_CR1_TREC_VCM                                                  LCSC_CR1_TREC_VCM_Msk
#define LCSC_CR1_TREC_VCM_0                                                (0x1U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_1                                                (0x2U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_2                                                (0x4U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_3                                                (0x8U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_4                                                (0x10U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_5                                                (0x20U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_6                                                (0x40U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_7                                                (0x80U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_TREC_VCM_8                                                (0x100U << LCSC_CR1_TREC_VCM_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_Pos                                       (0UL)		/*!<LCSC CR1: LCAB_DAMP_THRES (Bit 0) */
#define LCSC_CR1_LCAB_DAMP_THRES_Msk                                       (0xffUL)		/*!< LCSC CR1: LCAB_DAMP_THRES (Bitfield-Mask: 0xff) */
#define LCSC_CR1_LCAB_DAMP_THRES                                           LCSC_CR1_LCAB_DAMP_THRES_Msk
#define LCSC_CR1_LCAB_DAMP_THRES_0                                         (0x1U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_1                                         (0x2U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_2                                         (0x4U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_3                                         (0x8U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_4                                         (0x10U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_5                                         (0x20U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_6                                         (0x40U << LCSC_CR1_LCAB_DAMP_THRES_Pos)
#define LCSC_CR1_LCAB_DAMP_THRES_7                                         (0x80U << LCSC_CR1_LCAB_DAMP_THRES_Pos)

/* =====================================================    CR2    =====================================================*/ 
#define LCSC_CR2_LCT_DAMP_THRES_Pos                                        (8UL)		/*!<LCSC CR2: LCT_DAMP_THRES (Bit 8) */
#define LCSC_CR2_LCT_DAMP_THRES_Msk                                        (0xff00UL)		/*!< LCSC CR2: LCT_DAMP_THRES (Bitfield-Mask: 0xff) */
#define LCSC_CR2_LCT_DAMP_THRES                                            LCSC_CR2_LCT_DAMP_THRES_Msk
#define LCSC_CR2_LCT_DAMP_THRES_0                                          (0x1U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_1                                          (0x2U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_2                                          (0x4U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_3                                          (0x8U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_4                                          (0x10U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_5                                          (0x20U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_6                                          (0x40U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_LCT_DAMP_THRES_7                                          (0x80U << LCSC_CR2_LCT_DAMP_THRES_Pos)
#define LCSC_CR2_TAMP_PSC_Pos                                              (0UL)		/*!<LCSC CR2: TAMP_PSC (Bit 0) */
#define LCSC_CR2_TAMP_PSC_Msk                                              (0xffUL)		/*!< LCSC CR2: TAMP_PSC (Bitfield-Mask: 0xff) */
#define LCSC_CR2_TAMP_PSC                                                  LCSC_CR2_TAMP_PSC_Msk
#define LCSC_CR2_TAMP_PSC_0                                                (0x1U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_1                                                (0x2U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_2                                                (0x4U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_3                                                (0x8U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_4                                                (0x10U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_5                                                (0x20U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_6                                                (0x40U << LCSC_CR2_TAMP_PSC_Pos)
#define LCSC_CR2_TAMP_PSC_7                                                (0x80U << LCSC_CR2_TAMP_PSC_Pos)

/* =====================================================    PULSE_CR    =====================================================*/ 
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_Pos                                  (8UL)		/*!<LCSC PULSE_CR: LCT_PULSE_WIDTH (Bit 8) */
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_Msk                                  (0xf00UL)		/*!< LCSC PULSE_CR: LCT_PULSE_WIDTH (Bitfield-Mask: 0x0f) */
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH                                      LCSC_PULSE_CR_LCT_PULSE_WIDTH_Msk
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_0                                    (0x1U << LCSC_PULSE_CR_LCT_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_1                                    (0x2U << LCSC_PULSE_CR_LCT_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_2                                    (0x4U << LCSC_PULSE_CR_LCT_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCT_PULSE_WIDTH_3                                    (0x8U << LCSC_PULSE_CR_LCT_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Pos                                 (0UL)		/*!<LCSC PULSE_CR: LCAB_PULSE_WIDTH (Bit 0) */
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Msk                                 (0xfUL)		/*!< LCSC PULSE_CR: LCAB_PULSE_WIDTH (Bitfield-Mask: 0x0f) */
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH                                     LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Msk
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_0                                   (0x1U << LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_1                                   (0x2U << LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_2                                   (0x4U << LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Pos)
#define LCSC_PULSE_CR_LCAB_PULSE_WIDTH_3                                   (0x8U << LCSC_PULSE_CR_LCAB_PULSE_WIDTH_Pos)

/* =====================================================    ENR    =====================================================*/ 
#define LCSC_ENR_LCSC_EN_Pos                                               (31UL)		/*!<LCSC ENR: LCSC_EN (Bit 31) */
#define LCSC_ENR_LCSC_EN_Msk                                               (0x80000000UL)		/*!< LCSC ENR: LCSC_EN (Bitfield-Mask: 0x01) */
#define LCSC_ENR_LCSC_EN                                                   LCSC_ENR_LCSC_EN_Msk
#define LCSC_ENR_CNT_OFB_WKP_IE_Pos                                        (3UL)		/*!<LCSC ENR: CNT_OFB_WKP_IE (Bit 3) */
#define LCSC_ENR_CNT_OFB_WKP_IE_Msk                                        (0x8UL)		/*!< LCSC ENR: CNT_OFB_WKP_IE (Bitfield-Mask: 0x01) */
#define LCSC_ENR_CNT_OFB_WKP_IE                                            LCSC_ENR_CNT_OFB_WKP_IE_Msk
#define LCSC_ENR_TAMP_IE_Pos                                               (2UL)		/*!<LCSC ENR: TAMP_IE (Bit 2) */
#define LCSC_ENR_TAMP_IE_Msk                                               (0x4UL)		/*!< LCSC ENR: TAMP_IE (Bitfield-Mask: 0x01) */
#define LCSC_ENR_TAMP_IE                                                   LCSC_ENR_TAMP_IE_Msk
#define LCSC_ENR_ACLKWISE_IE_Pos                                           (1UL)		/*!<LCSC ENR: ACLKWISE_IE (Bit 1) */
#define LCSC_ENR_ACLKWISE_IE_Msk                                           (0x2UL)		/*!< LCSC ENR: ACLKWISE_IE (Bitfield-Mask: 0x01) */
#define LCSC_ENR_ACLKWISE_IE                                               LCSC_ENR_ACLKWISE_IE_Msk
#define LCSC_ENR_CLKWISE_IE_Pos                                            (0UL)		/*!<LCSC ENR: CLKWISE_IE (Bit 0) */
#define LCSC_ENR_CLKWISE_IE_Msk                                            (0x1UL)		/*!< LCSC ENR: CLKWISE_IE (Bitfield-Mask: 0x01) */
#define LCSC_ENR_CLKWISE_IE                                                LCSC_ENR_CLKWISE_IE_Msk

/* =====================================================    WHEEL_SR    =====================================================*/ 
#define LCSC_WHEEL_SR_ACLKWISE_Pos                                         (16UL)		/*!<LCSC WHEEL_SR: ACLKWISE (Bit 16) */
#define LCSC_WHEEL_SR_ACLKWISE_Msk                                         (0xffff0000UL)		/*!< LCSC WHEEL_SR: ACLKWISE (Bitfield-Mask: 0xffff) */
#define LCSC_WHEEL_SR_ACLKWISE                                             LCSC_WHEEL_SR_ACLKWISE_Msk
#define LCSC_WHEEL_SR_ACLKWISE_0                                           (0x1U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_1                                           (0x2U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_2                                           (0x4U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_3                                           (0x8U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_4                                           (0x10U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_5                                           (0x20U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_6                                           (0x40U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_7                                           (0x80U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_8                                           (0x100U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_9                                           (0x200U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_10                                          (0x400U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_11                                          (0x800U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_12                                          (0x1000U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_13                                          (0x2000U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_14                                          (0x4000U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_ACLKWISE_15                                          (0x8000U << LCSC_WHEEL_SR_ACLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_Pos                                          (0UL)		/*!<LCSC WHEEL_SR: CLKWISE (Bit 0) */
#define LCSC_WHEEL_SR_CLKWISE_Msk                                          (0xffffUL)		/*!< LCSC WHEEL_SR: CLKWISE (Bitfield-Mask: 0xffff) */
#define LCSC_WHEEL_SR_CLKWISE                                              LCSC_WHEEL_SR_CLKWISE_Msk
#define LCSC_WHEEL_SR_CLKWISE_0                                            (0x1U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_1                                            (0x2U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_2                                            (0x4U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_3                                            (0x8U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_4                                            (0x10U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_5                                            (0x20U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_6                                            (0x40U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_7                                            (0x80U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_8                                            (0x100U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_9                                            (0x200U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_10                                           (0x400U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_11                                           (0x800U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_12                                           (0x1000U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_13                                           (0x2000U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_14                                           (0x4000U << LCSC_WHEEL_SR_CLKWISE_Pos)
#define LCSC_WHEEL_SR_CLKWISE_15                                           (0x8000U << LCSC_WHEEL_SR_CLKWISE_Pos)

/* =====================================================    CONFR    =====================================================*/ 
#define LCSC_CONFR_ACLKWISE_THRES_Pos                                      (16UL)		/*!<LCSC CONFR: ACLKWISE_THRES (Bit 16) */
#define LCSC_CONFR_ACLKWISE_THRES_Msk                                      (0xffff0000UL)		/*!< LCSC CONFR: ACLKWISE_THRES (Bitfield-Mask: 0xffff) */
#define LCSC_CONFR_ACLKWISE_THRES                                          LCSC_CONFR_ACLKWISE_THRES_Msk
#define LCSC_CONFR_ACLKWISE_THRES_0                                        (0x1U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_1                                        (0x2U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_2                                        (0x4U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_3                                        (0x8U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_4                                        (0x10U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_5                                        (0x20U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_6                                        (0x40U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_7                                        (0x80U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_8                                        (0x100U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_9                                        (0x200U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_10                                       (0x400U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_11                                       (0x800U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_12                                       (0x1000U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_13                                       (0x2000U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_14                                       (0x4000U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_ACLKWISE_THRES_15                                       (0x8000U << LCSC_CONFR_ACLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_Pos                                       (0UL)		/*!<LCSC CONFR: CLKWISE_THRES (Bit 0) */
#define LCSC_CONFR_CLKWISE_THRES_Msk                                       (0xffffUL)		/*!< LCSC CONFR: CLKWISE_THRES (Bitfield-Mask: 0xffff) */
#define LCSC_CONFR_CLKWISE_THRES                                           LCSC_CONFR_CLKWISE_THRES_Msk
#define LCSC_CONFR_CLKWISE_THRES_0                                         (0x1U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_1                                         (0x2U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_2                                         (0x4U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_3                                         (0x8U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_4                                         (0x10U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_5                                         (0x20U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_6                                         (0x40U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_7                                         (0x80U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_8                                         (0x100U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_9                                         (0x200U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_10                                        (0x400U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_11                                        (0x800U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_12                                        (0x1000U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_13                                        (0x2000U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_14                                        (0x4000U << LCSC_CONFR_CLKWISE_THRES_Pos)
#define LCSC_CONFR_CLKWISE_THRES_15                                        (0x8000U << LCSC_CONFR_CLKWISE_THRES_Pos)

/* =====================================================    COMP_CTN    =====================================================*/ 
#define LCSC_COMP_CTN_CMP_LCT_CNT_Pos                                      (20UL)		/*!<LCSC COMP_CTN: CMP_LCT_CNT (Bit 20) */
#define LCSC_COMP_CTN_CMP_LCT_CNT_Msk                                      (0xff00000UL)		/*!< LCSC COMP_CTN: CMP_LCT_CNT (Bitfield-Mask: 0xff) */
#define LCSC_COMP_CTN_CMP_LCT_CNT                                          LCSC_COMP_CTN_CMP_LCT_CNT_Msk
#define LCSC_COMP_CTN_CMP_LCT_CNT_0                                        (0x1U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_1                                        (0x2U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_2                                        (0x4U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_3                                        (0x8U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_4                                        (0x10U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_5                                        (0x20U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_6                                        (0x40U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCT_CNT_7                                        (0x80U << LCSC_COMP_CTN_CMP_LCT_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_Pos                                      (10UL)		/*!<LCSC COMP_CTN: CMP_LCB_CNT (Bit 10) */
#define LCSC_COMP_CTN_CMP_LCB_CNT_Msk                                      (0x3fc00UL)		/*!< LCSC COMP_CTN: CMP_LCB_CNT (Bitfield-Mask: 0xff) */
#define LCSC_COMP_CTN_CMP_LCB_CNT                                          LCSC_COMP_CTN_CMP_LCB_CNT_Msk
#define LCSC_COMP_CTN_CMP_LCB_CNT_0                                        (0x1U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_1                                        (0x2U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_2                                        (0x4U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_3                                        (0x8U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_4                                        (0x10U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_5                                        (0x20U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_6                                        (0x40U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCB_CNT_7                                        (0x80U << LCSC_COMP_CTN_CMP_LCB_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_Pos                                      (0UL)		/*!<LCSC COMP_CTN: CMP_LCA_CNT (Bit 0) */
#define LCSC_COMP_CTN_CMP_LCA_CNT_Msk                                      (0xffUL)		/*!< LCSC COMP_CTN: CMP_LCA_CNT (Bitfield-Mask: 0xff) */
#define LCSC_COMP_CTN_CMP_LCA_CNT                                          LCSC_COMP_CTN_CMP_LCA_CNT_Msk
#define LCSC_COMP_CTN_CMP_LCA_CNT_0                                        (0x1U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_1                                        (0x2U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_2                                        (0x4U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_3                                        (0x8U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_4                                        (0x10U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_5                                        (0x20U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_6                                        (0x40U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)
#define LCSC_COMP_CTN_CMP_LCA_CNT_7                                        (0x80U << LCSC_COMP_CTN_CMP_LCA_CNT_Pos)

/* =====================================================    SR    =====================================================*/ 
#define LCSC_SR_LAST_DIR_Pos                                               (4UL)		/*!<LCSC SR: LAST_DIR (Bit 4) */
#define LCSC_SR_LAST_DIR_Msk                                               (0x30UL)		/*!< LCSC SR: LAST_DIR (Bitfield-Mask: 0x03) */
#define LCSC_SR_LAST_DIR                                                   LCSC_SR_LAST_DIR_Msk
#define LCSC_SR_LAST_DIR_0                                                 (0x1U << LCSC_SR_LAST_DIR_Pos)
#define LCSC_SR_LAST_DIR_1                                                 (0x2U << LCSC_SR_LAST_DIR_Pos)
#define LCSC_SR_ACLKWISE_STATE_Pos                                         (2UL)		/*!<LCSC SR: ACLKWISE_STATE (Bit 2) */
#define LCSC_SR_ACLKWISE_STATE_Msk                                         (0xcUL)		/*!< LCSC SR: ACLKWISE_STATE (Bitfield-Mask: 0x03) */
#define LCSC_SR_ACLKWISE_STATE                                             LCSC_SR_ACLKWISE_STATE_Msk
#define LCSC_SR_ACLKWISE_STATE_0                                           (0x1U << LCSC_SR_ACLKWISE_STATE_Pos)
#define LCSC_SR_ACLKWISE_STATE_1                                           (0x2U << LCSC_SR_ACLKWISE_STATE_Pos)
#define LCSC_SR_CLKWISE_STATE_Pos                                          (0UL)		/*!<LCSC SR: CLKWISE_STATE (Bit 0) */
#define LCSC_SR_CLKWISE_STATE_Msk                                          (0x3UL)		/*!< LCSC SR: CLKWISE_STATE (Bitfield-Mask: 0x03) */
#define LCSC_SR_CLKWISE_STATE                                              LCSC_SR_CLKWISE_STATE_Msk
#define LCSC_SR_CLKWISE_STATE_0                                            (0x1U << LCSC_SR_CLKWISE_STATE_Pos)
#define LCSC_SR_CLKWISE_STATE_1                                            (0x2U << LCSC_SR_CLKWISE_STATE_Pos)

/* =====================================================    STAT    =====================================================*/ 
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos                                   (24UL)		/*!<LCSC STAT: MAX_LCAB_CNT_BOUND (Bit 24) */
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_Msk                                   (0xff000000UL)		/*!< LCSC STAT: MAX_LCAB_CNT_BOUND (Bitfield-Mask: 0xff) */
#define LCSC_STAT_MAX_LCAB_CNT_BOUND                                       LCSC_STAT_MAX_LCAB_CNT_BOUND_Msk
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_0                                     (0x1U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_1                                     (0x2U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_2                                     (0x4U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_3                                     (0x8U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_4                                     (0x10U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_5                                     (0x20U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_6                                     (0x40U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_BOUND_7                                     (0x80U << LCSC_STAT_MAX_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos                                   (16UL)		/*!<LCSC STAT: MIN_LCAB_CNT_BOUND (Bit 16) */
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_Msk                                   (0xff0000UL)		/*!< LCSC STAT: MIN_LCAB_CNT_BOUND (Bitfield-Mask: 0xff) */
#define LCSC_STAT_MIN_LCAB_CNT_BOUND                                       LCSC_STAT_MIN_LCAB_CNT_BOUND_Msk
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_0                                     (0x1U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_1                                     (0x2U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_2                                     (0x4U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_3                                     (0x8U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_4                                     (0x10U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_5                                     (0x20U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_6                                     (0x40U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_BOUND_7                                     (0x80U << LCSC_STAT_MIN_LCAB_CNT_BOUND_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_Pos                                         (8UL)		/*!<LCSC STAT: MAX_LCAB_CNT (Bit 8) */
#define LCSC_STAT_MAX_LCAB_CNT_Msk                                         (0xff00UL)		/*!< LCSC STAT: MAX_LCAB_CNT (Bitfield-Mask: 0xff) */
#define LCSC_STAT_MAX_LCAB_CNT                                             LCSC_STAT_MAX_LCAB_CNT_Msk
#define LCSC_STAT_MAX_LCAB_CNT_0                                           (0x1U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_1                                           (0x2U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_2                                           (0x4U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_3                                           (0x8U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_4                                           (0x10U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_5                                           (0x20U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_6                                           (0x40U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MAX_LCAB_CNT_7                                           (0x80U << LCSC_STAT_MAX_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_Pos                                         (0UL)		/*!<LCSC STAT: MIN_LCAB_CNT (Bit 0) */
#define LCSC_STAT_MIN_LCAB_CNT_Msk                                         (0xffUL)		/*!< LCSC STAT: MIN_LCAB_CNT (Bitfield-Mask: 0xff) */
#define LCSC_STAT_MIN_LCAB_CNT                                             LCSC_STAT_MIN_LCAB_CNT_Msk
#define LCSC_STAT_MIN_LCAB_CNT_0                                           (0x1U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_1                                           (0x2U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_2                                           (0x4U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_3                                           (0x8U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_4                                           (0x10U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_5                                           (0x20U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_6                                           (0x40U << LCSC_STAT_MIN_LCAB_CNT_Pos)
#define LCSC_STAT_MIN_LCAB_CNT_7                                           (0x80U << LCSC_STAT_MIN_LCAB_CNT_Pos)

/* =====================================================    VER    =====================================================*/ 
#define LCSC_VER_PROD_Pos                                                  (12UL)		/*!<LCSC VER: PROD (Bit 12) */
#define LCSC_VER_PROD_Msk                                                  (0xf000UL)		/*!< LCSC VER: PROD (Bitfield-Mask: 0x0f) */
#define LCSC_VER_PROD                                                      LCSC_VER_PROD_Msk
#define LCSC_VER_PROD_0                                                    (0x1U << LCSC_VER_PROD_Pos)
#define LCSC_VER_PROD_1                                                    (0x2U << LCSC_VER_PROD_Pos)
#define LCSC_VER_PROD_2                                                    (0x4U << LCSC_VER_PROD_Pos)
#define LCSC_VER_PROD_3                                                    (0x8U << LCSC_VER_PROD_Pos)
#define LCSC_VER_VER_Pos                                                   (8UL)		/*!<LCSC VER: VER (Bit 8) */
#define LCSC_VER_VER_Msk                                                   (0xf00UL)		/*!< LCSC VER: VER (Bitfield-Mask: 0x0f) */
#define LCSC_VER_VER                                                       LCSC_VER_VER_Msk
#define LCSC_VER_VER_0                                                     (0x1U << LCSC_VER_VER_Pos)
#define LCSC_VER_VER_1                                                     (0x2U << LCSC_VER_VER_Pos)
#define LCSC_VER_VER_2                                                     (0x4U << LCSC_VER_VER_Pos)
#define LCSC_VER_VER_3                                                     (0x8U << LCSC_VER_VER_Pos)
#define LCSC_VER_REV_Pos                                                   (4UL)		/*!<LCSC VER: REV (Bit 4) */
#define LCSC_VER_REV_Msk                                                   (0xf0UL)		/*!< LCSC VER: REV (Bitfield-Mask: 0x0f) */
#define LCSC_VER_REV                                                       LCSC_VER_REV_Msk
#define LCSC_VER_REV_0                                                     (0x1U << LCSC_VER_REV_Pos)
#define LCSC_VER_REV_1                                                     (0x2U << LCSC_VER_REV_Pos)
#define LCSC_VER_REV_2                                                     (0x4U << LCSC_VER_REV_Pos)
#define LCSC_VER_REV_3                                                     (0x8U << LCSC_VER_REV_Pos)

/* =====================================================    ISR    =====================================================*/ 
#define LCSC_ISR_CNT_OFB_F_Pos                                             (3UL)		/*!<LCSC ISR: CNT_OFB_F (Bit 3) */
#define LCSC_ISR_CNT_OFB_F_Msk                                             (0x8UL)		/*!< LCSC ISR: CNT_OFB_F (Bitfield-Mask: 0x01) */
#define LCSC_ISR_CNT_OFB_F                                                 LCSC_ISR_CNT_OFB_F_Msk
#define LCSC_ISR_TAMP_F_Pos                                                (2UL)		/*!<LCSC ISR: TAMP_F (Bit 2) */
#define LCSC_ISR_TAMP_F_Msk                                                (0x4UL)		/*!< LCSC ISR: TAMP_F (Bitfield-Mask: 0x01) */
#define LCSC_ISR_TAMP_F                                                    LCSC_ISR_TAMP_F_Msk
#define LCSC_ISR_ACLKWISE_F_Pos                                            (1UL)		/*!<LCSC ISR: ACLKWISE_F (Bit 1) */
#define LCSC_ISR_ACLKWISE_F_Msk                                            (0x2UL)		/*!< LCSC ISR: ACLKWISE_F (Bitfield-Mask: 0x01) */
#define LCSC_ISR_ACLKWISE_F                                                LCSC_ISR_ACLKWISE_F_Msk
#define LCSC_ISR_CLKWISE_F_Pos                                             (0UL)		/*!<LCSC ISR: CLKWISE_F (Bit 0) */
#define LCSC_ISR_CLKWISE_F_Msk                                             (0x1UL)		/*!< LCSC ISR: CLKWISE_F (Bitfield-Mask: 0x01) */
#define LCSC_ISR_CLKWISE_F                                                 LCSC_ISR_CLKWISE_F_Msk


/* ============================================================================================================================*/ 
/*=====================                                        I2C                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR1    =====================================================*/ 
#define I2C_CR1_PECEN_Pos                                                  (23UL)		/*!<I2C CR1: PECEN (Bit 23) */
#define I2C_CR1_PECEN_Msk                                                  (0x800000UL)		/*!< I2C CR1: PECEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_PECEN                                                      I2C_CR1_PECEN_Msk
#define I2C_CR1_ALERTEN_Pos                                                (22UL)		/*!<I2C CR1: ALERTEN (Bit 22) */
#define I2C_CR1_ALERTEN_Msk                                                (0x400000UL)		/*!< I2C CR1: ALERTEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_ALERTEN                                                    I2C_CR1_ALERTEN_Msk
#define I2C_CR1_SMBDEN_Pos                                                 (21UL)		/*!<I2C CR1: SMBDEN (Bit 21) */
#define I2C_CR1_SMBDEN_Msk                                                 (0x200000UL)		/*!< I2C CR1: SMBDEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_SMBDEN                                                     I2C_CR1_SMBDEN_Msk
#define I2C_CR1_SMBHEN_Pos                                                 (20UL)		/*!<I2C CR1: SMBHEN (Bit 20) */
#define I2C_CR1_SMBHEN_Msk                                                 (0x100000UL)		/*!< I2C CR1: SMBHEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_SMBHEN                                                     I2C_CR1_SMBHEN_Msk
#define I2C_CR1_GCEN_Pos                                                   (19UL)		/*!<I2C CR1: GCEN (Bit 19) */
#define I2C_CR1_GCEN_Msk                                                   (0x80000UL)		/*!< I2C CR1: GCEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_GCEN                                                       I2C_CR1_GCEN_Msk
#define I2C_CR1_NOSTRETCH_Pos                                              (17UL)		/*!<I2C CR1: NOSTRETCH (Bit 17) */
#define I2C_CR1_NOSTRETCH_Msk                                              (0x20000UL)		/*!< I2C CR1: NOSTRETCH (Bitfield-Mask: 0x01) */
#define I2C_CR1_NOSTRETCH                                                  I2C_CR1_NOSTRETCH_Msk
#define I2C_CR1_SBC_Pos                                                    (16UL)		/*!<I2C CR1: SBC (Bit 16) */
#define I2C_CR1_SBC_Msk                                                    (0x10000UL)		/*!< I2C CR1: SBC (Bitfield-Mask: 0x01) */
#define I2C_CR1_SBC                                                        I2C_CR1_SBC_Msk
#define I2C_CR1_RXDMAEN_Pos                                                (15UL)		/*!<I2C CR1: RXDMAEN (Bit 15) */
#define I2C_CR1_RXDMAEN_Msk                                                (0x8000UL)		/*!< I2C CR1: RXDMAEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_RXDMAEN                                                    I2C_CR1_RXDMAEN_Msk
#define I2C_CR1_TXDMAEN_Pos                                                (14UL)		/*!<I2C CR1: TXDMAEN (Bit 14) */
#define I2C_CR1_TXDMAEN_Msk                                                (0x4000UL)		/*!< I2C CR1: TXDMAEN (Bitfield-Mask: 0x01) */
#define I2C_CR1_TXDMAEN                                                    I2C_CR1_TXDMAEN_Msk
#define I2C_CR1_ANFOFF_Pos                                                 (12UL)		/*!<I2C CR1: ANFOFF (Bit 12) */
#define I2C_CR1_ANFOFF_Msk                                                 (0x1000UL)		/*!< I2C CR1: ANFOFF (Bitfield-Mask: 0x01) */
#define I2C_CR1_ANFOFF                                                     I2C_CR1_ANFOFF_Msk
#define I2C_CR1_DNF_Pos                                                    (8UL)		/*!<I2C CR1: DNF (Bit 8) */
#define I2C_CR1_DNF_Msk                                                    (0xf00UL)		/*!< I2C CR1: DNF (Bitfield-Mask: 0x0f) */
#define I2C_CR1_DNF                                                        I2C_CR1_DNF_Msk
#define I2C_CR1_DNF_0                                                      (0x1U << I2C_CR1_DNF_Pos)
#define I2C_CR1_DNF_1                                                      (0x2U << I2C_CR1_DNF_Pos)
#define I2C_CR1_DNF_2                                                      (0x4U << I2C_CR1_DNF_Pos)
#define I2C_CR1_DNF_3                                                      (0x8U << I2C_CR1_DNF_Pos)
#define I2C_CR1_ERRIE_Pos                                                  (7UL)		/*!<I2C CR1: ERRIE (Bit 7) */
#define I2C_CR1_ERRIE_Msk                                                  (0x80UL)		/*!< I2C CR1: ERRIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_ERRIE                                                      I2C_CR1_ERRIE_Msk
#define I2C_CR1_TCIE_Pos                                                   (6UL)		/*!<I2C CR1: TCIE (Bit 6) */
#define I2C_CR1_TCIE_Msk                                                   (0x40UL)		/*!< I2C CR1: TCIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_TCIE                                                       I2C_CR1_TCIE_Msk
#define I2C_CR1_STOPIE_Pos                                                 (5UL)		/*!<I2C CR1: STOPIE (Bit 5) */
#define I2C_CR1_STOPIE_Msk                                                 (0x20UL)		/*!< I2C CR1: STOPIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_STOPIE                                                     I2C_CR1_STOPIE_Msk
#define I2C_CR1_NACKIE_Pos                                                 (4UL)		/*!<I2C CR1: NACKIE (Bit 4) */
#define I2C_CR1_NACKIE_Msk                                                 (0x10UL)		/*!< I2C CR1: NACKIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_NACKIE                                                     I2C_CR1_NACKIE_Msk
#define I2C_CR1_ADDRIE_Pos                                                 (3UL)		/*!<I2C CR1: ADDRIE (Bit 3) */
#define I2C_CR1_ADDRIE_Msk                                                 (0x8UL)		/*!< I2C CR1: ADDRIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_ADDRIE                                                     I2C_CR1_ADDRIE_Msk
#define I2C_CR1_RXIE_Pos                                                   (2UL)		/*!<I2C CR1: RXIE (Bit 2) */
#define I2C_CR1_RXIE_Msk                                                   (0x4UL)		/*!< I2C CR1: RXIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_RXIE                                                       I2C_CR1_RXIE_Msk
#define I2C_CR1_TXIE_Pos                                                   (1UL)		/*!<I2C CR1: TXIE (Bit 1) */
#define I2C_CR1_TXIE_Msk                                                   (0x2UL)		/*!< I2C CR1: TXIE (Bitfield-Mask: 0x01) */
#define I2C_CR1_TXIE                                                       I2C_CR1_TXIE_Msk
#define I2C_CR1_PE_Pos                                                     (0UL)		/*!<I2C CR1: PE (Bit 0) */
#define I2C_CR1_PE_Msk                                                     (0x1UL)		/*!< I2C CR1: PE (Bitfield-Mask: 0x01) */
#define I2C_CR1_PE                                                         I2C_CR1_PE_Msk

/* =====================================================    CR2    =====================================================*/ 
#define I2C_CR2_PECBYTE_Pos                                                (26UL)		/*!<I2C CR2: PECBYTE (Bit 26) */
#define I2C_CR2_PECBYTE_Msk                                                (0x4000000UL)		/*!< I2C CR2: PECBYTE (Bitfield-Mask: 0x01) */
#define I2C_CR2_PECBYTE                                                    I2C_CR2_PECBYTE_Msk
#define I2C_CR2_AUTOEND_Pos                                                (25UL)		/*!<I2C CR2: AUTOEND (Bit 25) */
#define I2C_CR2_AUTOEND_Msk                                                (0x2000000UL)		/*!< I2C CR2: AUTOEND (Bitfield-Mask: 0x01) */
#define I2C_CR2_AUTOEND                                                    I2C_CR2_AUTOEND_Msk
#define I2C_CR2_RELOAD_Pos                                                 (24UL)		/*!<I2C CR2: RELOAD (Bit 24) */
#define I2C_CR2_RELOAD_Msk                                                 (0x1000000UL)		/*!< I2C CR2: RELOAD (Bitfield-Mask: 0x01) */
#define I2C_CR2_RELOAD                                                     I2C_CR2_RELOAD_Msk
#define I2C_CR2_NBYTES_Pos                                                 (16UL)		/*!<I2C CR2: NBYTES (Bit 16) */
#define I2C_CR2_NBYTES_Msk                                                 (0xff0000UL)		/*!< I2C CR2: NBYTES (Bitfield-Mask: 0xff) */
#define I2C_CR2_NBYTES                                                     I2C_CR2_NBYTES_Msk
#define I2C_CR2_NBYTES_0                                                   (0x1U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_1                                                   (0x2U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_2                                                   (0x4U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_3                                                   (0x8U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_4                                                   (0x10U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_5                                                   (0x20U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_6                                                   (0x40U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NBYTES_7                                                   (0x80U << I2C_CR2_NBYTES_Pos)
#define I2C_CR2_NACK_Pos                                                   (15UL)		/*!<I2C CR2: NACK (Bit 15) */
#define I2C_CR2_NACK_Msk                                                   (0x8000UL)		/*!< I2C CR2: NACK (Bitfield-Mask: 0x01) */
#define I2C_CR2_NACK                                                       I2C_CR2_NACK_Msk
#define I2C_CR2_STOP_Pos                                                   (14UL)		/*!<I2C CR2: STOP (Bit 14) */
#define I2C_CR2_STOP_Msk                                                   (0x4000UL)		/*!< I2C CR2: STOP (Bitfield-Mask: 0x01) */
#define I2C_CR2_STOP                                                       I2C_CR2_STOP_Msk
#define I2C_CR2_START_Pos                                                  (13UL)		/*!<I2C CR2: START (Bit 13) */
#define I2C_CR2_START_Msk                                                  (0x2000UL)		/*!< I2C CR2: START (Bitfield-Mask: 0x01) */
#define I2C_CR2_START                                                      I2C_CR2_START_Msk
#define I2C_CR2_HEAD10R_Pos                                                (12UL)		/*!<I2C CR2: HEAD10R (Bit 12) */
#define I2C_CR2_HEAD10R_Msk                                                (0x1000UL)		/*!< I2C CR2: HEAD10R (Bitfield-Mask: 0x01) */
#define I2C_CR2_HEAD10R                                                    I2C_CR2_HEAD10R_Msk
#define I2C_CR2_ADD10_Pos                                                  (11UL)		/*!<I2C CR2: ADD10 (Bit 11) */
#define I2C_CR2_ADD10_Msk                                                  (0x800UL)		/*!< I2C CR2: ADD10 (Bitfield-Mask: 0x01) */
#define I2C_CR2_ADD10                                                      I2C_CR2_ADD10_Msk
#define I2C_CR2_RD_WRN_Pos                                                 (10UL)		/*!<I2C CR2: RD_WRN (Bit 10) */
#define I2C_CR2_RD_WRN_Msk                                                 (0x400UL)		/*!< I2C CR2: RD_WRN (Bitfield-Mask: 0x01) */
#define I2C_CR2_RD_WRN                                                     I2C_CR2_RD_WRN_Msk
#define I2C_CR2_SADD_Pos                                                   (0UL)		/*!<I2C CR2: SADD (Bit 0) */
#define I2C_CR2_SADD_Msk                                                   (0x3ffUL)		/*!< I2C CR2: SADD (Bitfield-Mask: 0x3ff) */
#define I2C_CR2_SADD                                                       I2C_CR2_SADD_Msk
#define I2C_CR2_SADD_0                                                     (0x1U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_1                                                     (0x2U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_2                                                     (0x4U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_3                                                     (0x8U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_4                                                     (0x10U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_5                                                     (0x20U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_6                                                     (0x40U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_7                                                     (0x80U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_8                                                     (0x100U << I2C_CR2_SADD_Pos)
#define I2C_CR2_SADD_9                                                     (0x200U << I2C_CR2_SADD_Pos)

/* =====================================================    OAR1    =====================================================*/ 
#define I2C_OAR1_OA1EN_Pos                                                 (15UL)		/*!<I2C OAR1: OA1EN (Bit 15) */
#define I2C_OAR1_OA1EN_Msk                                                 (0x8000UL)		/*!< I2C OAR1: OA1EN (Bitfield-Mask: 0x01) */
#define I2C_OAR1_OA1EN                                                     I2C_OAR1_OA1EN_Msk
#define I2C_OAR1_OA1MODE_Pos                                               (10UL)		/*!<I2C OAR1: OA1MODE (Bit 10) */
#define I2C_OAR1_OA1MODE_Msk                                               (0x400UL)		/*!< I2C OAR1: OA1MODE (Bitfield-Mask: 0x01) */
#define I2C_OAR1_OA1MODE                                                   I2C_OAR1_OA1MODE_Msk
#define I2C_OAR1_OA1_Pos                                                   (0UL)		/*!<I2C OAR1: OA1 (Bit 0) */
#define I2C_OAR1_OA1_Msk                                                   (0x3ffUL)		/*!< I2C OAR1: OA1 (Bitfield-Mask: 0x3ff) */
#define I2C_OAR1_OA1                                                       I2C_OAR1_OA1_Msk
#define I2C_OAR1_OA1_0                                                     (0x1U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_1                                                     (0x2U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_2                                                     (0x4U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_3                                                     (0x8U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_4                                                     (0x10U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_5                                                     (0x20U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_6                                                     (0x40U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_7                                                     (0x80U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_8                                                     (0x100U << I2C_OAR1_OA1_Pos)
#define I2C_OAR1_OA1_9                                                     (0x200U << I2C_OAR1_OA1_Pos)

/* =====================================================    OAR2    =====================================================*/
#define I2C_OAR2_OA2EN_Pos                                                 (15UL)                             /*!<I2C OAR2: OA2EN (Bit 15) */
#define I2C_OAR2_OA2EN_Msk                                                 (0x1UL << I2C_OAR2_OA2EN_Pos)      /*!< I2C OAR2: OA2EN (Bitfield-Mask: 0x01) */
#define I2C_OAR2_OA2EN                                                     I2C_OAR2_OA2EN_Msk
#define I2C_OAR2_OA2MSK_Pos                                                (8UL)                              /*!<I2C OAR2: OA2MSK (Bit 8) */
#define I2C_OAR2_OA2MSK_Msk                                                (0x7UL << I2C_OAR2_OA2MSK_Pos)     /*!< I2C OAR2: OA2MSK (Bitfield-Mask: 0x07) */
#define I2C_OAR2_OA2MSK                                                    I2C_OAR2_OA2MSK_Msk
#define I2C_OAR2_OA2NOMASK                                                 (0x00000000UL)                     /*!< No mask                                        */
#define I2C_OAR2_OA2MASK01_Pos                                             (8U)
#define I2C_OAR2_OA2MASK01_Msk                                             (0x1UL << I2C_OAR2_OA2MASK01_Pos)  /*!< 0x00000100 */
#define I2C_OAR2_OA2MASK01                                                 I2C_OAR2_OA2MASK01_Msk             /*!< OA2[1] is masked, Only OA2[7:2] are compared   */
#define I2C_OAR2_OA2MASK02_Pos                                             (9U)
#define I2C_OAR2_OA2MASK02_Msk                                             (0x1UL << I2C_OAR2_OA2MASK02_Pos)  /*!< 0x00000200 */
#define I2C_OAR2_OA2MASK02                                                 I2C_OAR2_OA2MASK02_Msk             /*!< OA2[2:1] is masked, Only OA2[7:3] are compared */
#define I2C_OAR2_OA2MASK03_Pos                                             (8U)
#define I2C_OAR2_OA2MASK03_Msk                                             (0x3UL << I2C_OAR2_OA2MASK03_Pos)  /*!< 0x00000300 */
#define I2C_OAR2_OA2MASK03                                                 I2C_OAR2_OA2MASK03_Msk             /*!< OA2[3:1] is masked, Only OA2[7:4] are compared */
#define I2C_OAR2_OA2MASK04_Pos                                             (10U)
#define I2C_OAR2_OA2MASK04_Msk                                             (0x1UL << I2C_OAR2_OA2MASK04_Pos)  /*!< 0x00000400 */
#define I2C_OAR2_OA2MASK04                                                 I2C_OAR2_OA2MASK04_Msk             /*!< OA2[4:1] is masked, Only OA2[7:5] are compared */
#define I2C_OAR2_OA2MASK05_Pos                                             (8U)
#define I2C_OAR2_OA2MASK05_Msk                                             (0x5UL << I2C_OAR2_OA2MASK05_Pos)  /*!< 0x00000500 */
#define I2C_OAR2_OA2MASK05                                                 I2C_OAR2_OA2MASK05_Msk             /*!< OA2[5:1] is masked, Only OA2[7:6] are compared */
#define I2C_OAR2_OA2MASK06_Pos                                             (9U)
#define I2C_OAR2_OA2MASK06_Msk                                             (0x3UL << I2C_OAR2_OA2MASK06_Pos)  /*!< 0x00000600 */
#define I2C_OAR2_OA2MASK06                                                 I2C_OAR2_OA2MASK06_Msk             /*!< OA2[6:1] is masked, Only OA2[7] are compared   */
#define I2C_OAR2_OA2MASK07_Pos                                             (8U)
#define I2C_OAR2_OA2MASK07_Msk                                             (0x7UL << I2C_OAR2_OA2MASK07_Pos)  /*!< 0x00000700 */
#define I2C_OAR2_OA2MASK07                                                 I2C_OAR2_OA2MASK07_Msk             /*!< OA2[7:1] is masked, No comparison is done      */
#define I2C_OAR2_OA2_Pos                                                   (1UL)                              /*!<I2C OAR2: OA2 (Bit 1) */
#define I2C_OAR2_OA2_Msk                                                   (0x7FUL << I2C_OAR2_OA2_Pos)	      /*!< I2C OAR2: OA2 (Bitfield-Mask: 0x7f) */
#define I2C_OAR2_OA2                                                       I2C_OAR2_OA2_Msk
#define I2C_OAR2_OA2_0                                                     (0x1U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_1                                                     (0x2U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_2                                                     (0x4U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_3                                                     (0x8U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_4                                                     (0x10U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_5                                                     (0x20U << I2C_OAR2_OA2_Pos)
#define I2C_OAR2_OA2_6                                                     (0x40U << I2C_OAR2_OA2_Pos)

/* =====================================================    TIMINGR    =====================================================*/ 
#define I2C_TIMINGR_PRESC_Pos                                              (28UL)		/*!<I2C TIMINGR: PRESC (Bit 28) */
#define I2C_TIMINGR_PRESC_Msk                                              (0xf0000000UL)		/*!< I2C TIMINGR: PRESC (Bitfield-Mask: 0x0f) */
#define I2C_TIMINGR_PRESC                                                  I2C_TIMINGR_PRESC_Msk
#define I2C_TIMINGR_PRESC_0                                                (0x1U << I2C_TIMINGR_PRESC_Pos)
#define I2C_TIMINGR_PRESC_1                                                (0x2U << I2C_TIMINGR_PRESC_Pos)
#define I2C_TIMINGR_PRESC_2                                                (0x4U << I2C_TIMINGR_PRESC_Pos)
#define I2C_TIMINGR_PRESC_3                                                (0x8U << I2C_TIMINGR_PRESC_Pos)
#define I2C_TIMINGR_SCLDEL_Pos                                             (20UL)		/*!<I2C TIMINGR: SCLDEL (Bit 20) */
#define I2C_TIMINGR_SCLDEL_Msk                                             (0xf00000UL)		/*!< I2C TIMINGR: SCLDEL (Bitfield-Mask: 0x0f) */
#define I2C_TIMINGR_SCLDEL                                                 I2C_TIMINGR_SCLDEL_Msk
#define I2C_TIMINGR_SCLDEL_0                                               (0x1U << I2C_TIMINGR_SCLDEL_Pos)
#define I2C_TIMINGR_SCLDEL_1                                               (0x2U << I2C_TIMINGR_SCLDEL_Pos)
#define I2C_TIMINGR_SCLDEL_2                                               (0x4U << I2C_TIMINGR_SCLDEL_Pos)
#define I2C_TIMINGR_SCLDEL_3                                               (0x8U << I2C_TIMINGR_SCLDEL_Pos)
#define I2C_TIMINGR_SDADEL_Pos                                             (16UL)		/*!<I2C TIMINGR: SDADEL (Bit 16) */
#define I2C_TIMINGR_SDADEL_Msk                                             (0xf0000UL)		/*!< I2C TIMINGR: SDADEL (Bitfield-Mask: 0x0f) */
#define I2C_TIMINGR_SDADEL                                                 I2C_TIMINGR_SDADEL_Msk
#define I2C_TIMINGR_SDADEL_0                                               (0x1U << I2C_TIMINGR_SDADEL_Pos)
#define I2C_TIMINGR_SDADEL_1                                               (0x2U << I2C_TIMINGR_SDADEL_Pos)
#define I2C_TIMINGR_SDADEL_2                                               (0x4U << I2C_TIMINGR_SDADEL_Pos)
#define I2C_TIMINGR_SDADEL_3                                               (0x8U << I2C_TIMINGR_SDADEL_Pos)
#define I2C_TIMINGR_SCLH_Pos                                               (8UL)		/*!<I2C TIMINGR: SCLH (Bit 8) */
#define I2C_TIMINGR_SCLH_Msk                                               (0xff00UL)		/*!< I2C TIMINGR: SCLH (Bitfield-Mask: 0xff) */
#define I2C_TIMINGR_SCLH                                                   I2C_TIMINGR_SCLH_Msk
#define I2C_TIMINGR_SCLH_0                                                 (0x1U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_1                                                 (0x2U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_2                                                 (0x4U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_3                                                 (0x8U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_4                                                 (0x10U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_5                                                 (0x20U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_6                                                 (0x40U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLH_7                                                 (0x80U << I2C_TIMINGR_SCLH_Pos)
#define I2C_TIMINGR_SCLL_Pos                                               (0UL)		/*!<I2C TIMINGR: SCLL (Bit 0) */
#define I2C_TIMINGR_SCLL_Msk                                               (0xffUL)		/*!< I2C TIMINGR: SCLL (Bitfield-Mask: 0xff) */
#define I2C_TIMINGR_SCLL                                                   I2C_TIMINGR_SCLL_Msk
#define I2C_TIMINGR_SCLL_0                                                 (0x1U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_1                                                 (0x2U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_2                                                 (0x4U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_3                                                 (0x8U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_4                                                 (0x10U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_5                                                 (0x20U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_6                                                 (0x40U << I2C_TIMINGR_SCLL_Pos)
#define I2C_TIMINGR_SCLL_7                                                 (0x80U << I2C_TIMINGR_SCLL_Pos)

/* =====================================================    TIMEOUTR    =====================================================*/ 
#define I2C_TIMEOUTR_TEXTEN_Pos                                            (31UL)		/*!<I2C TIMEOUTR: TEXTEN (Bit 31) */
#define I2C_TIMEOUTR_TEXTEN_Msk                                            (0x80000000UL)		/*!< I2C TIMEOUTR: TEXTEN (Bitfield-Mask: 0x01) */
#define I2C_TIMEOUTR_TEXTEN                                                I2C_TIMEOUTR_TEXTEN_Msk
#define I2C_TIMEOUTR_TIMEOUTB_Pos                                          (16UL)		/*!<I2C TIMEOUTR: TIMEOUTB (Bit 16) */
#define I2C_TIMEOUTR_TIMEOUTB_Msk                                          (0xfff0000UL)		/*!< I2C TIMEOUTR: TIMEOUTB (Bitfield-Mask: 0xfff) */
#define I2C_TIMEOUTR_TIMEOUTB                                              I2C_TIMEOUTR_TIMEOUTB_Msk
#define I2C_TIMEOUTR_TIMEOUTB_0                                            (0x1U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_1                                            (0x2U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_2                                            (0x4U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_3                                            (0x8U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_4                                            (0x10U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_5                                            (0x20U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_6                                            (0x40U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_7                                            (0x80U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_8                                            (0x100U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_9                                            (0x200U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_10                                           (0x400U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMEOUTB_11                                           (0x800U << I2C_TIMEOUTR_TIMEOUTB_Pos)
#define I2C_TIMEOUTR_TIMOUTEN_Pos                                          (15UL)		/*!<I2C TIMEOUTR: TIMOUTEN (Bit 15) */
#define I2C_TIMEOUTR_TIMOUTEN_Msk                                          (0x8000UL)		/*!< I2C TIMEOUTR: TIMOUTEN (Bitfield-Mask: 0x01) */
#define I2C_TIMEOUTR_TIMOUTEN                                              I2C_TIMEOUTR_TIMOUTEN_Msk
#define I2C_TIMEOUTR_TIDLE_Pos                                             (12UL)		/*!<I2C TIMEOUTR: TIDLE (Bit 12) */
#define I2C_TIMEOUTR_TIDLE_Msk                                             (0x1000UL)		/*!< I2C TIMEOUTR: TIDLE (Bitfield-Mask: 0x01) */
#define I2C_TIMEOUTR_TIDLE                                                 I2C_TIMEOUTR_TIDLE_Msk
#define I2C_TIMEOUTR_TIMEOUTA_Pos                                          (0UL)		/*!<I2C TIMEOUTR: TIMEOUTA (Bit 0) */
#define I2C_TIMEOUTR_TIMEOUTA_Msk                                          (0xfffUL)		/*!< I2C TIMEOUTR: TIMEOUTA (Bitfield-Mask: 0xfff) */
#define I2C_TIMEOUTR_TIMEOUTA                                              I2C_TIMEOUTR_TIMEOUTA_Msk
#define I2C_TIMEOUTR_TIMEOUTA_0                                            (0x1U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_1                                            (0x2U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_2                                            (0x4U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_3                                            (0x8U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_4                                            (0x10U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_5                                            (0x20U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_6                                            (0x40U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_7                                            (0x80U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_8                                            (0x100U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_9                                            (0x200U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_10                                           (0x400U << I2C_TIMEOUTR_TIMEOUTA_Pos)
#define I2C_TIMEOUTR_TIMEOUTA_11                                           (0x800U << I2C_TIMEOUTR_TIMEOUTA_Pos)

/* =====================================================    ISR    =====================================================*/ 
#define I2C_ISR_ADDCODE_Pos                                                (17UL)		/*!<I2C ISR: ADDCODE (Bit 17) */
#define I2C_ISR_ADDCODE_Msk                                                (0xfe0000UL)		/*!< I2C ISR: ADDCODE (Bitfield-Mask: 0x7f) */
#define I2C_ISR_ADDCODE                                                    I2C_ISR_ADDCODE_Msk
#define I2C_ISR_ADDCODE_0                                                  (0x1U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_1                                                  (0x2U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_2                                                  (0x4U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_3                                                  (0x8U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_4                                                  (0x10U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_5                                                  (0x20U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_ADDCODE_6                                                  (0x40U << I2C_ISR_ADDCODE_Pos)
#define I2C_ISR_DIR_Pos                                                    (16UL)		/*!<I2C ISR: DIR (Bit 16) */
#define I2C_ISR_DIR_Msk                                                    (0x10000UL)		/*!< I2C ISR: DIR (Bitfield-Mask: 0x01) */
#define I2C_ISR_DIR                                                        I2C_ISR_DIR_Msk
#define I2C_ISR_BUSY_Pos                                                   (15UL)		/*!<I2C ISR: BUSY (Bit 15) */
#define I2C_ISR_BUSY_Msk                                                   (0x8000UL)		/*!< I2C ISR: BUSY (Bitfield-Mask: 0x01) */
#define I2C_ISR_BUSY                                                       I2C_ISR_BUSY_Msk
#define I2C_ISR_ALERT_Pos                                                  (13UL)		/*!<I2C ISR: ALERT (Bit 13) */
#define I2C_ISR_ALERT_Msk                                                  (0x2000UL)		/*!< I2C ISR: ALERT (Bitfield-Mask: 0x01) */
#define I2C_ISR_ALERT                                                      I2C_ISR_ALERT_Msk
#define I2C_ISR_TIMEOUT_Pos                                                (12UL)		/*!<I2C ISR: TIMEOUT (Bit 12) */
#define I2C_ISR_TIMEOUT_Msk                                                (0x1000UL)		/*!< I2C ISR: TIMEOUT (Bitfield-Mask: 0x01) */
#define I2C_ISR_TIMEOUT                                                    I2C_ISR_TIMEOUT_Msk
#define I2C_ISR_PECERR_Pos                                                 (11UL)		/*!<I2C ISR: PECERR (Bit 11) */
#define I2C_ISR_PECERR_Msk                                                 (0x800UL)		/*!< I2C ISR: PECERR (Bitfield-Mask: 0x01) */
#define I2C_ISR_PECERR                                                     I2C_ISR_PECERR_Msk
#define I2C_ISR_OVR_Pos                                                    (10UL)		/*!<I2C ISR: OVR (Bit 10) */
#define I2C_ISR_OVR_Msk                                                    (0x400UL)		/*!< I2C ISR: OVR (Bitfield-Mask: 0x01) */
#define I2C_ISR_OVR                                                        I2C_ISR_OVR_Msk
#define I2C_ISR_ARLO_Pos                                                   (9UL)		/*!<I2C ISR: ARLO (Bit 9) */
#define I2C_ISR_ARLO_Msk                                                   (0x200UL)		/*!< I2C ISR: ARLO (Bitfield-Mask: 0x01) */
#define I2C_ISR_ARLO                                                       I2C_ISR_ARLO_Msk
#define I2C_ISR_BERR_Pos                                                   (8UL)		/*!<I2C ISR: BERR (Bit 8) */
#define I2C_ISR_BERR_Msk                                                   (0x100UL)		/*!< I2C ISR: BERR (Bitfield-Mask: 0x01) */
#define I2C_ISR_BERR                                                       I2C_ISR_BERR_Msk
#define I2C_ISR_TCR_Pos                                                    (7UL)		/*!<I2C ISR: TCR (Bit 7) */
#define I2C_ISR_TCR_Msk                                                    (0x80UL)		/*!< I2C ISR: TCR (Bitfield-Mask: 0x01) */
#define I2C_ISR_TCR                                                        I2C_ISR_TCR_Msk
#define I2C_ISR_TC_Pos                                                     (6UL)		/*!<I2C ISR: TC (Bit 6) */
#define I2C_ISR_TC_Msk                                                     (0x40UL)		/*!< I2C ISR: TC (Bitfield-Mask: 0x01) */
#define I2C_ISR_TC                                                         I2C_ISR_TC_Msk
#define I2C_ISR_STOPF_Pos                                                  (5UL)		/*!<I2C ISR: STOPF (Bit 5) */
#define I2C_ISR_STOPF_Msk                                                  (0x20UL)		/*!< I2C ISR: STOPF (Bitfield-Mask: 0x01) */
#define I2C_ISR_STOPF                                                      I2C_ISR_STOPF_Msk
#define I2C_ISR_NACKF_Pos                                                  (4UL)		/*!<I2C ISR: NACKF (Bit 4) */
#define I2C_ISR_NACKF_Msk                                                  (0x10UL)		/*!< I2C ISR: NACKF (Bitfield-Mask: 0x01) */
#define I2C_ISR_NACKF                                                      I2C_ISR_NACKF_Msk
#define I2C_ISR_ADDR_Pos                                                   (3UL)		/*!<I2C ISR: ADDR (Bit 3) */
#define I2C_ISR_ADDR_Msk                                                   (0x8UL)		/*!< I2C ISR: ADDR (Bitfield-Mask: 0x01) */
#define I2C_ISR_ADDR                                                       I2C_ISR_ADDR_Msk
#define I2C_ISR_RXNE_Pos                                                   (2UL)		/*!<I2C ISR: RXNE (Bit 2) */
#define I2C_ISR_RXNE_Msk                                                   (0x4UL)		/*!< I2C ISR: RXNE (Bitfield-Mask: 0x01) */
#define I2C_ISR_RXNE                                                       I2C_ISR_RXNE_Msk
#define I2C_ISR_TXIS_Pos                                                   (1UL)		/*!<I2C ISR: TXIS (Bit 1) */
#define I2C_ISR_TXIS_Msk                                                   (0x2UL)		/*!< I2C ISR: TXIS (Bitfield-Mask: 0x01) */
#define I2C_ISR_TXIS                                                       I2C_ISR_TXIS_Msk
#define I2C_ISR_TXE_Pos                                                    (0UL)		/*!<I2C ISR: TXE (Bit 0) */
#define I2C_ISR_TXE_Msk                                                    (0x1UL)		/*!< I2C ISR: TXE (Bitfield-Mask: 0x01) */
#define I2C_ISR_TXE                                                        I2C_ISR_TXE_Msk

/* =====================================================    ICR    =====================================================*/ 
#define I2C_ICR_ALERTCF_Pos                                                (13UL)		/*!<I2C ICR: ALERTCF (Bit 13) */
#define I2C_ICR_ALERTCF_Msk                                                (0x2000UL)		/*!< I2C ICR: ALERTCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_ALERTCF                                                    I2C_ICR_ALERTCF_Msk
#define I2C_ICR_TIMOUTCF_Pos                                               (12UL)		/*!<I2C ICR: TIMOUTCF (Bit 12) */
#define I2C_ICR_TIMOUTCF_Msk                                               (0x1000UL)		/*!< I2C ICR: TIMOUTCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_TIMOUTCF                                                   I2C_ICR_TIMOUTCF_Msk
#define I2C_ICR_PECCF_Pos                                                  (11UL)		/*!<I2C ICR: PECCF (Bit 11) */
#define I2C_ICR_PECCF_Msk                                                  (0x800UL)		/*!< I2C ICR: PECCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_PECCF                                                      I2C_ICR_PECCF_Msk
#define I2C_ICR_OVRCF_Pos                                                  (10UL)		/*!<I2C ICR: OVRCF (Bit 10) */
#define I2C_ICR_OVRCF_Msk                                                  (0x400UL)		/*!< I2C ICR: OVRCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_OVRCF                                                      I2C_ICR_OVRCF_Msk
#define I2C_ICR_ARLOCF_Pos                                                 (9UL)		/*!<I2C ICR: ARLOCF (Bit 9) */
#define I2C_ICR_ARLOCF_Msk                                                 (0x200UL)		/*!< I2C ICR: ARLOCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_ARLOCF                                                     I2C_ICR_ARLOCF_Msk
#define I2C_ICR_BERRCF_Pos                                                 (8UL)		/*!<I2C ICR: BERRCF (Bit 8) */
#define I2C_ICR_BERRCF_Msk                                                 (0x100UL)		/*!< I2C ICR: BERRCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_BERRCF                                                     I2C_ICR_BERRCF_Msk
#define I2C_ICR_STOPCF_Pos                                                 (5UL)		/*!<I2C ICR: STOPCF (Bit 5) */
#define I2C_ICR_STOPCF_Msk                                                 (0x20UL)		/*!< I2C ICR: STOPCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_STOPCF                                                     I2C_ICR_STOPCF_Msk
#define I2C_ICR_NACKCF_Pos                                                 (4UL)		/*!<I2C ICR: NACKCF (Bit 4) */
#define I2C_ICR_NACKCF_Msk                                                 (0x10UL)		/*!< I2C ICR: NACKCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_NACKCF                                                     I2C_ICR_NACKCF_Msk
#define I2C_ICR_ADDRCF_Pos                                                 (3UL)		/*!<I2C ICR: ADDRCF (Bit 3) */
#define I2C_ICR_ADDRCF_Msk                                                 (0x8UL)		/*!< I2C ICR: ADDRCF (Bitfield-Mask: 0x01) */
#define I2C_ICR_ADDRCF                                                     I2C_ICR_ADDRCF_Msk

/* =====================================================    PECR    =====================================================*/ 
#define I2C_PECR_PEC_Pos                                                   (0UL)		/*!<I2C PECR: PEC (Bit 0) */
#define I2C_PECR_PEC_Msk                                                   (0xffUL)		/*!< I2C PECR: PEC (Bitfield-Mask: 0xff) */
#define I2C_PECR_PEC                                                       I2C_PECR_PEC_Msk
#define I2C_PECR_PEC_0                                                     (0x1U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_1                                                     (0x2U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_2                                                     (0x4U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_3                                                     (0x8U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_4                                                     (0x10U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_5                                                     (0x20U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_6                                                     (0x40U << I2C_PECR_PEC_Pos)
#define I2C_PECR_PEC_7                                                     (0x80U << I2C_PECR_PEC_Pos)

/* =====================================================    RXDR    =====================================================*/ 
#define I2C_RXDR_RXDATA_Pos                                                (0UL)		/*!<I2C RXDR: RXDATA (Bit 0) */
#define I2C_RXDR_RXDATA_Msk                                                (0xffUL)		/*!< I2C RXDR: RXDATA (Bitfield-Mask: 0xff) */
#define I2C_RXDR_RXDATA                                                    I2C_RXDR_RXDATA_Msk
#define I2C_RXDR_RXDATA_0                                                  (0x1U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_1                                                  (0x2U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_2                                                  (0x4U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_3                                                  (0x8U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_4                                                  (0x10U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_5                                                  (0x20U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_6                                                  (0x40U << I2C_RXDR_RXDATA_Pos)
#define I2C_RXDR_RXDATA_7                                                  (0x80U << I2C_RXDR_RXDATA_Pos)

/* =====================================================    TXDR    =====================================================*/ 
#define I2C_TXDR_TXDATA_Pos                                                (0UL)		/*!<I2C TXDR: TXDATA (Bit 0) */
#define I2C_TXDR_TXDATA_Msk                                                (0xffUL)		/*!< I2C TXDR: TXDATA (Bitfield-Mask: 0xff) */
#define I2C_TXDR_TXDATA                                                    I2C_TXDR_TXDATA_Msk
#define I2C_TXDR_TXDATA_0                                                  (0x1U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_1                                                  (0x2U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_2                                                  (0x4U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_3                                                  (0x8U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_4                                                  (0x10U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_5                                                  (0x20U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_6                                                  (0x40U << I2C_TXDR_TXDATA_Pos)
#define I2C_TXDR_TXDATA_7                                                  (0x80U << I2C_TXDR_TXDATA_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        SPI                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR1    =====================================================*/ 
#define SPI_CR1_BIDIMODE_Pos                                               (15UL)		/*!<SPI CR1: BIDIMODE (Bit 15) */
#define SPI_CR1_BIDIMODE_Msk                                               (0x8000UL)		/*!< SPI CR1: BIDIMODE (Bitfield-Mask: 0x01) */
#define SPI_CR1_BIDIMODE                                                   SPI_CR1_BIDIMODE_Msk
#define SPI_CR1_BIDIOE_Pos                                                 (14UL)		/*!<SPI CR1: BIDIOE (Bit 14) */
#define SPI_CR1_BIDIOE_Msk                                                 (0x4000UL)		/*!< SPI CR1: BIDIOE (Bitfield-Mask: 0x01) */
#define SPI_CR1_BIDIOE                                                     SPI_CR1_BIDIOE_Msk
#define SPI_CR1_CRCEN_Pos                                                  (13UL)		/*!<SPI CR1: CRCEN (Bit 13) */
#define SPI_CR1_CRCEN_Msk                                                  (0x2000UL)		/*!< SPI CR1: CRCEN (Bitfield-Mask: 0x01) */
#define SPI_CR1_CRCEN                                                      SPI_CR1_CRCEN_Msk
#define SPI_CR1_CRCNEXT_Pos                                                (12UL)		/*!<SPI CR1: CRCNEXT (Bit 12) */
#define SPI_CR1_CRCNEXT_Msk                                                (0x1000UL)		/*!< SPI CR1: CRCNEXT (Bitfield-Mask: 0x01) */
#define SPI_CR1_CRCNEXT                                                    SPI_CR1_CRCNEXT_Msk
#define SPI_CR1_CRCL_Pos                                                   (11UL)		/*!<SPI CR1: CRCL (Bit 11) */
#define SPI_CR1_CRCL_Msk                                                   (0x800UL)		/*!< SPI CR1: CRCL (Bitfield-Mask: 0x01) */
#define SPI_CR1_CRCL                                                       SPI_CR1_CRCL_Msk
#define SPI_CR1_RXONLY_Pos                                                 (10UL)		/*!<SPI CR1: RXONLY (Bit 10) */
#define SPI_CR1_RXONLY_Msk                                                 (0x400UL)		/*!< SPI CR1: RXONLY (Bitfield-Mask: 0x01) */
#define SPI_CR1_RXONLY                                                     SPI_CR1_RXONLY_Msk
#define SPI_CR1_SSM_Pos                                                    (9UL)		/*!<SPI CR1: SSM (Bit 9) */
#define SPI_CR1_SSM_Msk                                                    (0x200UL)		/*!< SPI CR1: SSM (Bitfield-Mask: 0x01) */
#define SPI_CR1_SSM                                                        SPI_CR1_SSM_Msk
#define SPI_CR1_SSI_Pos                                                    (8UL)		/*!<SPI CR1: SSI (Bit 8) */
#define SPI_CR1_SSI_Msk                                                    (0x100UL)		/*!< SPI CR1: SSI (Bitfield-Mask: 0x01) */
#define SPI_CR1_SSI                                                        SPI_CR1_SSI_Msk
#define SPI_CR1_LSBFIRST_Pos                                               (7UL)		/*!<SPI CR1: LSBFIRST (Bit 7) */
#define SPI_CR1_LSBFIRST_Msk                                               (0x80UL)		/*!< SPI CR1: LSBFIRST (Bitfield-Mask: 0x01) */
#define SPI_CR1_LSBFIRST                                                   SPI_CR1_LSBFIRST_Msk
#define SPI_CR1_SPE_Pos                                                    (6UL)		/*!<SPI CR1: SPE (Bit 6) */
#define SPI_CR1_SPE_Msk                                                    (0x40UL)		/*!< SPI CR1: SPE (Bitfield-Mask: 0x01) */
#define SPI_CR1_SPE                                                        SPI_CR1_SPE_Msk
#define SPI_CR1_BR_Pos                                                     (3UL)		/*!<SPI CR1: BR (Bit 3) */
#define SPI_CR1_BR_Msk                                                     (0x38UL)		/*!< SPI CR1: BR (Bitfield-Mask: 0x07) */
#define SPI_CR1_BR                                                         SPI_CR1_BR_Msk
#define SPI_CR1_BR_0                                                       (0x1U << SPI_CR1_BR_Pos)
#define SPI_CR1_BR_1                                                       (0x2U << SPI_CR1_BR_Pos)
#define SPI_CR1_BR_2                                                       (0x4U << SPI_CR1_BR_Pos)
#define SPI_CR1_MSTR_Pos                                                   (2UL)		/*!<SPI CR1: MSTR (Bit 2) */
#define SPI_CR1_MSTR_Msk                                                   (0x4UL)		/*!< SPI CR1: MSTR (Bitfield-Mask: 0x01) */
#define SPI_CR1_MSTR                                                       SPI_CR1_MSTR_Msk
#define SPI_CR1_CPOL_Pos                                                   (1UL)		/*!<SPI CR1: CPOL (Bit 1) */
#define SPI_CR1_CPOL_Msk                                                   (0x2UL)		/*!< SPI CR1: CPOL (Bitfield-Mask: 0x01) */
#define SPI_CR1_CPOL                                                       SPI_CR1_CPOL_Msk
#define SPI_CR1_CPHA_Pos                                                   (0UL)		/*!<SPI CR1: CPHA (Bit 0) */
#define SPI_CR1_CPHA_Msk                                                   (0x1UL)		/*!< SPI CR1: CPHA (Bitfield-Mask: 0x01) */
#define SPI_CR1_CPHA                                                       SPI_CR1_CPHA_Msk

/* =====================================================    CR2    =====================================================*/ 
#define SPI_CR2_LDMATX_Pos                                                 (14UL)		/*!<SPI CR2: LDMATX (Bit 14) */
#define SPI_CR2_LDMATX_Msk                                                 (0x4000UL)		/*!< SPI CR2: LDMATX (Bitfield-Mask: 0x01) */
#define SPI_CR2_LDMATX                                                     SPI_CR2_LDMATX_Msk
#define SPI_CR2_LDMARX_Pos                                                 (13UL)		/*!<SPI CR2: LDMARX (Bit 13) */
#define SPI_CR2_LDMARX_Msk                                                 (0x2000UL)		/*!< SPI CR2: LDMARX (Bitfield-Mask: 0x01) */
#define SPI_CR2_LDMARX                                                     SPI_CR2_LDMARX_Msk
#define SPI_CR2_FRXTH_Pos                                                  (12UL)		/*!<SPI CR2: FRXTH (Bit 12) */
#define SPI_CR2_FRXTH_Msk                                                  (0x1000UL)		/*!< SPI CR2: FRXTH (Bitfield-Mask: 0x01) */
#define SPI_CR2_FRXTH                                                      SPI_CR2_FRXTH_Msk
#define SPI_CR2_DS_Pos                                                     (8UL)		/*!<SPI CR2: DS (Bit 8) */
#define SPI_CR2_DS_Msk                                                     (0xf00UL)		/*!< SPI CR2: DS (Bitfield-Mask: 0x0f) */
#define SPI_CR2_DS                                                         SPI_CR2_DS_Msk
#define SPI_CR2_DS_0                                                       (0x1U << SPI_CR2_DS_Pos)
#define SPI_CR2_DS_1                                                       (0x2U << SPI_CR2_DS_Pos)
#define SPI_CR2_DS_2                                                       (0x4U << SPI_CR2_DS_Pos)
#define SPI_CR2_DS_3                                                       (0x8U << SPI_CR2_DS_Pos)
#define SPI_CR2_TXEIE_Pos                                                  (7UL)		/*!<SPI CR2: TXEIE (Bit 7) */
#define SPI_CR2_TXEIE_Msk                                                  (0x80UL)		/*!< SPI CR2: TXEIE (Bitfield-Mask: 0x01) */
#define SPI_CR2_TXEIE                                                      SPI_CR2_TXEIE_Msk
#define SPI_CR2_RXNEIE_Pos                                                 (6UL)		/*!<SPI CR2: RXNEIE (Bit 6) */
#define SPI_CR2_RXNEIE_Msk                                                 (0x40UL)		/*!< SPI CR2: RXNEIE (Bitfield-Mask: 0x01) */
#define SPI_CR2_RXNEIE                                                     SPI_CR2_RXNEIE_Msk
#define SPI_CR2_ERRIE_Pos                                                  (5UL)		/*!<SPI CR2: ERRIE (Bit 5) */
#define SPI_CR2_ERRIE_Msk                                                  (0x20UL)		/*!< SPI CR2: ERRIE (Bitfield-Mask: 0x01) */
#define SPI_CR2_ERRIE                                                      SPI_CR2_ERRIE_Msk
#define SPI_CR2_FRF_Pos                                                    (4UL)		/*!<SPI CR2: FRF (Bit 4) */
#define SPI_CR2_FRF_Msk                                                    (0x10UL)		/*!< SPI CR2: FRF (Bitfield-Mask: 0x01) */
#define SPI_CR2_FRF                                                        SPI_CR2_FRF_Msk
#define SPI_CR2_NSSP_Pos                                                   (3UL)		/*!<SPI CR2: NSSP (Bit 3) */
#define SPI_CR2_NSSP_Msk                                                   (0x8UL)		/*!< SPI CR2: NSSP (Bitfield-Mask: 0x01) */
#define SPI_CR2_NSSP                                                       SPI_CR2_NSSP_Msk
#define SPI_CR2_SSOE_Pos                                                   (2UL)		/*!<SPI CR2: SSOE (Bit 2) */
#define SPI_CR2_SSOE_Msk                                                   (0x4UL)		/*!< SPI CR2: SSOE (Bitfield-Mask: 0x01) */
#define SPI_CR2_SSOE                                                       SPI_CR2_SSOE_Msk
#define SPI_CR2_TXDMAEN_Pos                                                (1UL)		/*!<SPI CR2: TXDMAEN (Bit 1) */
#define SPI_CR2_TXDMAEN_Msk                                                (0x2UL)		/*!< SPI CR2: TXDMAEN (Bitfield-Mask: 0x01) */
#define SPI_CR2_TXDMAEN                                                    SPI_CR2_TXDMAEN_Msk
#define SPI_CR2_RXDMAEN_Pos                                                (0UL)		/*!<SPI CR2: RXDMAEN (Bit 0) */
#define SPI_CR2_RXDMAEN_Msk                                                (0x1UL)		/*!< SPI CR2: RXDMAEN (Bitfield-Mask: 0x01) */
#define SPI_CR2_RXDMAEN                                                    SPI_CR2_RXDMAEN_Msk

/* =====================================================    SR    =====================================================*/ 
#define SPI_SR_FTLVL_Pos                                                   (11UL)		/*!<SPI SR: FTLVL (Bit 11) */
#define SPI_SR_FTLVL_Msk                                                   (0x1800UL)		/*!< SPI SR: FTLVL (Bitfield-Mask: 0x03) */
#define SPI_SR_FTLVL                                                       SPI_SR_FTLVL_Msk
#define SPI_SR_FTLVL_0                                                     (0x1U << SPI_SR_FTLVL_Pos)
#define SPI_SR_FTLVL_1                                                     (0x2U << SPI_SR_FTLVL_Pos)
#define SPI_SR_FRLVL_Pos                                                   (9UL)		/*!<SPI SR: FRLVL (Bit 9) */
#define SPI_SR_FRLVL_Msk                                                   (0x600UL)		/*!< SPI SR: FRLVL (Bitfield-Mask: 0x03) */
#define SPI_SR_FRLVL                                                       SPI_SR_FRLVL_Msk
#define SPI_SR_FRLVL_0                                                     (0x1U << SPI_SR_FRLVL_Pos)
#define SPI_SR_FRLVL_1                                                     (0x2U << SPI_SR_FRLVL_Pos)
#define SPI_SR_FRE_Pos                                                     (8UL)		/*!<SPI SR: FRE (Bit 8) */
#define SPI_SR_FRE_Msk                                                     (0x100UL)		/*!< SPI SR: FRE (Bitfield-Mask: 0x01) */
#define SPI_SR_FRE                                                         SPI_SR_FRE_Msk
#define SPI_SR_BSY_Pos                                                     (7UL)		/*!<SPI SR: BSY (Bit 7) */
#define SPI_SR_BSY_Msk                                                     (0x80UL)		/*!< SPI SR: BSY (Bitfield-Mask: 0x01) */
#define SPI_SR_BSY                                                         SPI_SR_BSY_Msk
#define SPI_SR_OVR_Pos                                                     (6UL)		/*!<SPI SR: OVR (Bit 6) */
#define SPI_SR_OVR_Msk                                                     (0x40UL)		/*!< SPI SR: OVR (Bitfield-Mask: 0x01) */
#define SPI_SR_OVR                                                         SPI_SR_OVR_Msk
#define SPI_SR_MODF_Pos                                                    (5UL)		/*!<SPI SR: MODF (Bit 5) */
#define SPI_SR_MODF_Msk                                                    (0x20UL)		/*!< SPI SR: MODF (Bitfield-Mask: 0x01) */
#define SPI_SR_MODF                                                        SPI_SR_MODF_Msk
#define SPI_SR_CRCERR_Pos                                                  (4UL)		/*!<SPI SR: CRCERR (Bit 4) */
#define SPI_SR_CRCERR_Msk                                                  (0x10UL)		/*!< SPI SR: CRCERR (Bitfield-Mask: 0x01) */
#define SPI_SR_CRCERR                                                      SPI_SR_CRCERR_Msk
#define SPI_SR_UDR_Pos                                                     (3UL)		/*!<SPI SR: UDR (Bit 3) */
#define SPI_SR_UDR_Msk                                                     (0x8UL)		/*!< SPI SR: UDR (Bitfield-Mask: 0x01) */
#define SPI_SR_UDR                                                         SPI_SR_UDR_Msk
#define SPI_SR_CHSIDE_Pos                                                  (2UL)		/*!<SPI SR: CHSIDE (Bit 2) */
#define SPI_SR_CHSIDE_Msk                                                  (0x4UL)		/*!< SPI SR: CHSIDE (Bitfield-Mask: 0x01) */
#define SPI_SR_CHSIDE                                                      SPI_SR_CHSIDE_Msk
#define SPI_SR_TXE_Pos                                                     (1UL)		/*!<SPI SR: TXE (Bit 1) */
#define SPI_SR_TXE_Msk                                                     (0x2UL)		/*!< SPI SR: TXE (Bitfield-Mask: 0x01) */
#define SPI_SR_TXE                                                         SPI_SR_TXE_Msk
#define SPI_SR_RXNE_Pos                                                    (0UL)		/*!<SPI SR: RXNE (Bit 0) */
#define SPI_SR_RXNE_Msk                                                    (0x1UL)		/*!< SPI SR: RXNE (Bitfield-Mask: 0x01) */
#define SPI_SR_RXNE                                                        SPI_SR_RXNE_Msk

/* =====================================================    DR    =====================================================*/ 
#define SPI_DR_DR_Pos                                                      (0UL)		/*!<SPI DR: DR (Bit 0) */
#define SPI_DR_DR_Msk                                                      (0xffffUL)		/*!< SPI DR: DR (Bitfield-Mask: 0xffff) */
#define SPI_DR_DR                                                          SPI_DR_DR_Msk
#define SPI_DR_DR_0                                                        (0x1U << SPI_DR_DR_Pos)
#define SPI_DR_DR_1                                                        (0x2U << SPI_DR_DR_Pos)
#define SPI_DR_DR_2                                                        (0x4U << SPI_DR_DR_Pos)
#define SPI_DR_DR_3                                                        (0x8U << SPI_DR_DR_Pos)
#define SPI_DR_DR_4                                                        (0x10U << SPI_DR_DR_Pos)
#define SPI_DR_DR_5                                                        (0x20U << SPI_DR_DR_Pos)
#define SPI_DR_DR_6                                                        (0x40U << SPI_DR_DR_Pos)
#define SPI_DR_DR_7                                                        (0x80U << SPI_DR_DR_Pos)
#define SPI_DR_DR_8                                                        (0x100U << SPI_DR_DR_Pos)
#define SPI_DR_DR_9                                                        (0x200U << SPI_DR_DR_Pos)
#define SPI_DR_DR_10                                                       (0x400U << SPI_DR_DR_Pos)
#define SPI_DR_DR_11                                                       (0x800U << SPI_DR_DR_Pos)
#define SPI_DR_DR_12                                                       (0x1000U << SPI_DR_DR_Pos)
#define SPI_DR_DR_13                                                       (0x2000U << SPI_DR_DR_Pos)
#define SPI_DR_DR_14                                                       (0x4000U << SPI_DR_DR_Pos)
#define SPI_DR_DR_15                                                       (0x8000U << SPI_DR_DR_Pos)

/* =====================================================    CRCPR    =====================================================*/ 
#define SPI_CRCPR_CRCPOLY_Pos                                              (0UL)		/*!<SPI CRCPR: CRCPOLY (Bit 0) */
#define SPI_CRCPR_CRCPOLY_Msk                                              (0xffffUL)		/*!< SPI CRCPR: CRCPOLY (Bitfield-Mask: 0xffff) */
#define SPI_CRCPR_CRCPOLY                                                  SPI_CRCPR_CRCPOLY_Msk
#define SPI_CRCPR_CRCPOLY_0                                                (0x1U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_1                                                (0x2U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_2                                                (0x4U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_3                                                (0x8U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_4                                                (0x10U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_5                                                (0x20U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_6                                                (0x40U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_7                                                (0x80U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_8                                                (0x100U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_9                                                (0x200U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_10                                               (0x400U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_11                                               (0x800U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_12                                               (0x1000U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_13                                               (0x2000U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_14                                               (0x4000U << SPI_CRCPR_CRCPOLY_Pos)
#define SPI_CRCPR_CRCPOLY_15                                               (0x8000U << SPI_CRCPR_CRCPOLY_Pos)

/* =====================================================    RXCRCR    =====================================================*/ 
#define SPI_RXCRCR_RXCRC_Pos                                               (0UL)		/*!<SPI RXCRCR: RXCRC (Bit 0) */
#define SPI_RXCRCR_RXCRC_Msk                                               (0xffffUL)		/*!< SPI RXCRCR: RXCRC (Bitfield-Mask: 0xffff) */
#define SPI_RXCRCR_RXCRC                                                   SPI_RXCRCR_RXCRC_Msk
#define SPI_RXCRCR_RXCRC_0                                                 (0x1U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_1                                                 (0x2U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_2                                                 (0x4U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_3                                                 (0x8U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_4                                                 (0x10U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_5                                                 (0x20U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_6                                                 (0x40U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_7                                                 (0x80U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_8                                                 (0x100U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_9                                                 (0x200U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_10                                                (0x400U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_11                                                (0x800U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_12                                                (0x1000U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_13                                                (0x2000U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_14                                                (0x4000U << SPI_RXCRCR_RXCRC_Pos)
#define SPI_RXCRCR_RXCRC_15                                                (0x8000U << SPI_RXCRCR_RXCRC_Pos)

/* =====================================================    TXCRCR    =====================================================*/ 
#define SPI_TXCRCR_TXCRC_Pos                                               (0UL)		/*!<SPI TXCRCR: TXCRC (Bit 0) */
#define SPI_TXCRCR_TXCRC_Msk                                               (0xffffUL)		/*!< SPI TXCRCR: TXCRC (Bitfield-Mask: 0xffff) */
#define SPI_TXCRCR_TXCRC                                                   SPI_TXCRCR_TXCRC_Msk
#define SPI_TXCRCR_TXCRC_0                                                 (0x1U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_1                                                 (0x2U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_2                                                 (0x4U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_3                                                 (0x8U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_4                                                 (0x10U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_5                                                 (0x20U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_6                                                 (0x40U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_7                                                 (0x80U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_8                                                 (0x100U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_9                                                 (0x200U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_10                                                (0x400U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_11                                                (0x800U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_12                                                (0x1000U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_13                                                (0x2000U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_14                                                (0x4000U << SPI_TXCRCR_TXCRC_Pos)
#define SPI_TXCRCR_TXCRC_15                                                (0x8000U << SPI_TXCRCR_TXCRC_Pos)

/* =====================================================    I2SCFGR    =====================================================*/ 
#define SPI_I2SCFGR_ASTRTEN_Pos                                            (12UL)		/*!<SPI I2SCFGR: ASTRTEN (Bit 12) */
#define SPI_I2SCFGR_ASTRTEN_Msk                                            (0x1000UL)		/*!< SPI I2SCFGR: ASTRTEN (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_ASTRTEN                                                SPI_I2SCFGR_ASTRTEN_Msk
#define SPI_I2SCFGR_I2SMOD_Pos                                             (11UL)		/*!<SPI I2SCFGR: I2SMOD (Bit 11) */
#define SPI_I2SCFGR_I2SMOD_Msk                                             (0x800UL)		/*!< SPI I2SCFGR: I2SMOD (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_I2SMOD                                                 SPI_I2SCFGR_I2SMOD_Msk
#define SPI_I2SCFGR_I2SE_Pos                                               (10UL)		/*!<SPI I2SCFGR: I2SE (Bit 10) */
#define SPI_I2SCFGR_I2SE_Msk                                               (0x400UL)		/*!< SPI I2SCFGR: I2SE (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_I2SE                                                   SPI_I2SCFGR_I2SE_Msk
#define SPI_I2SCFGR_I2SCFG_Pos                                             (8UL)		/*!<SPI I2SCFGR: I2SCFG (Bit 8) */
#define SPI_I2SCFGR_I2SCFG_Msk                                             (0x300UL)		/*!< SPI I2SCFGR: I2SCFG (Bitfield-Mask: 0x03) */
#define SPI_I2SCFGR_I2SCFG                                                 SPI_I2SCFGR_I2SCFG_Msk
#define SPI_I2SCFGR_I2SCFG_0                                               (0x1U << SPI_I2SCFGR_I2SCFG_Pos)
#define SPI_I2SCFGR_I2SCFG_1                                               (0x2U << SPI_I2SCFGR_I2SCFG_Pos)
#define SPI_I2SCFGR_PCMSYNC_Pos                                            (7UL)		/*!<SPI I2SCFGR: PCMSYNC (Bit 7) */
#define SPI_I2SCFGR_PCMSYNC_Msk                                            (0x80UL)		/*!< SPI I2SCFGR: PCMSYNC (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_PCMSYNC                                                SPI_I2SCFGR_PCMSYNC_Msk
#define SPI_I2SCFGR_I2SSTD_Pos                                             (4UL)		/*!<SPI I2SCFGR: I2SSTD (Bit 4) */
#define SPI_I2SCFGR_I2SSTD_Msk                                             (0x30UL)		/*!< SPI I2SCFGR: I2SSTD (Bitfield-Mask: 0x03) */
#define SPI_I2SCFGR_I2SSTD                                                 SPI_I2SCFGR_I2SSTD_Msk
#define SPI_I2SCFGR_I2SSTD_0                                               (0x1U << SPI_I2SCFGR_I2SSTD_Pos)
#define SPI_I2SCFGR_I2SSTD_1                                               (0x2U << SPI_I2SCFGR_I2SSTD_Pos)
#define SPI_I2SCFGR_CKPOL_Pos                                              (3UL)		/*!<SPI I2SCFGR: CKPOL (Bit 3) */
#define SPI_I2SCFGR_CKPOL_Msk                                              (0x8UL)		/*!< SPI I2SCFGR: CKPOL (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_CKPOL                                                  SPI_I2SCFGR_CKPOL_Msk
#define SPI_I2SCFGR_DATLEN_Pos                                             (1UL)		/*!<SPI I2SCFGR: DATLEN (Bit 1) */
#define SPI_I2SCFGR_DATLEN_Msk                                             (0x6UL)		/*!< SPI I2SCFGR: DATLEN (Bitfield-Mask: 0x03) */
#define SPI_I2SCFGR_DATLEN                                                 SPI_I2SCFGR_DATLEN_Msk
#define SPI_I2SCFGR_DATLEN_0                                               (0x1U << SPI_I2SCFGR_DATLEN_Pos)
#define SPI_I2SCFGR_DATLEN_1                                               (0x2U << SPI_I2SCFGR_DATLEN_Pos)
#define SPI_I2SCFGR_CHLEN_Pos                                              (0UL)		/*!<SPI I2SCFGR: CHLEN (Bit 0) */
#define SPI_I2SCFGR_CHLEN_Msk                                              (0x1UL)		/*!< SPI I2SCFGR: CHLEN (Bitfield-Mask: 0x01) */
#define SPI_I2SCFGR_CHLEN                                                  SPI_I2SCFGR_CHLEN_Msk

/* =====================================================    I2SPR    =====================================================*/ 
#define SPI_I2SPR_MCKOE_Pos                                                (9UL)		/*!<SPI I2SPR: MCKOE (Bit 9) */
#define SPI_I2SPR_MCKOE_Msk                                                (0x200UL)		/*!< SPI I2SPR: MCKOE (Bitfield-Mask: 0x01) */
#define SPI_I2SPR_MCKOE                                                    SPI_I2SPR_MCKOE_Msk
#define SPI_I2SPR_ODD_Pos                                                  (8UL)		/*!<SPI I2SPR: ODD (Bit 8) */
#define SPI_I2SPR_ODD_Msk                                                  (0x100UL)		/*!< SPI I2SPR: ODD (Bitfield-Mask: 0x01) */
#define SPI_I2SPR_ODD                                                      SPI_I2SPR_ODD_Msk
#define SPI_I2SPR_I2SDIV_Pos                                               (0UL)		/*!<SPI I2SPR: I2SDIV (Bit 0) */
#define SPI_I2SPR_I2SDIV_Msk                                               (0xffUL)		/*!< SPI I2SPR: I2SDIV (Bitfield-Mask: 0xff) */
#define SPI_I2SPR_I2SDIV                                                   SPI_I2SPR_I2SDIV_Msk
#define SPI_I2SPR_I2SDIV_0                                                 (0x1U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_1                                                 (0x2U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_2                                                 (0x4U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_3                                                 (0x8U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_4                                                 (0x10U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_5                                                 (0x20U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_6                                                 (0x40U << SPI_I2SPR_I2SDIV_Pos)
#define SPI_I2SPR_I2SDIV_7                                                 (0x80U << SPI_I2SPR_I2SDIV_Pos)


/* ============================================================================================================================*/ 
/*=====================                                       USART                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR1    =====================================================*/ 
#define USART_CR1_RXFFIE_Pos                                               (31UL)		/*!<USART CR1: RXFFIE (Bit 31) */
#define USART_CR1_RXFFIE_Msk                                               (0x80000000UL)		/*!< USART CR1: RXFFIE (Bitfield-Mask: 0x01) */
#define USART_CR1_RXFFIE                                                   USART_CR1_RXFFIE_Msk
#define USART_CR1_TXFEIE_Pos                                               (30UL)		/*!<USART CR1: TXFEIE (Bit 30) */
#define USART_CR1_TXFEIE_Msk                                               (0x40000000UL)		/*!< USART CR1: TXFEIE (Bitfield-Mask: 0x01) */
#define USART_CR1_TXFEIE                                                   USART_CR1_TXFEIE_Msk
#define USART_CR1_FIFOEN_Pos                                               (29UL)		/*!<USART CR1: FIFOEN (Bit 29) */
#define USART_CR1_FIFOEN_Msk                                               (0x20000000UL)		/*!< USART CR1: FIFOEN (Bitfield-Mask: 0x01) */
#define USART_CR1_FIFOEN                                                   USART_CR1_FIFOEN_Msk
#define USART_CR1_EOBIE_Pos                                                (27UL)		/*!<USART CR1: EOBIE (Bit 27) */
#define USART_CR1_EOBIE_Msk                                                (0x8000000UL)		/*!< USART CR1: EOBIE (Bitfield-Mask: 0x01) */
#define USART_CR1_EOBIE                                                    USART_CR1_EOBIE_Msk
#define USART_CR1_RTOIE_Pos                                                (26UL)		/*!<USART CR1: RTOIE (Bit 26) */
#define USART_CR1_RTOIE_Msk                                                (0x4000000UL)		/*!< USART CR1: RTOIE (Bitfield-Mask: 0x01) */
#define USART_CR1_RTOIE                                                    USART_CR1_RTOIE_Msk
#define USART_CR1_DEAT_Pos                                                 (21UL)		/*!<USART CR1: DEAT (Bit 21) */
#define USART_CR1_DEAT_Msk                                                 (0x3e00000UL)		/*!< USART CR1: DEAT (Bitfield-Mask: 0x1f) */
#define USART_CR1_DEAT                                                     USART_CR1_DEAT_Msk
#define USART_CR1_DEAT_0                                                   (0x1U << USART_CR1_DEAT_Pos)
#define USART_CR1_DEAT_1                                                   (0x2U << USART_CR1_DEAT_Pos)
#define USART_CR1_DEAT_2                                                   (0x4U << USART_CR1_DEAT_Pos)
#define USART_CR1_DEAT_3                                                   (0x8U << USART_CR1_DEAT_Pos)
#define USART_CR1_DEAT_4                                                   (0x10U << USART_CR1_DEAT_Pos)
#define USART_CR1_DEDT_Pos                                                 (16UL)		/*!<USART CR1: DEDT (Bit 16) */
#define USART_CR1_DEDT_Msk                                                 (0x1f0000UL)		/*!< USART CR1: DEDT (Bitfield-Mask: 0x1f) */
#define USART_CR1_DEDT                                                     USART_CR1_DEDT_Msk
#define USART_CR1_DEDT_0                                                   (0x1U << USART_CR1_DEDT_Pos)
#define USART_CR1_DEDT_1                                                   (0x2U << USART_CR1_DEDT_Pos)
#define USART_CR1_DEDT_2                                                   (0x4U << USART_CR1_DEDT_Pos)
#define USART_CR1_DEDT_3                                                   (0x8U << USART_CR1_DEDT_Pos)
#define USART_CR1_DEDT_4                                                   (0x10U << USART_CR1_DEDT_Pos)
#define USART_CR1_OVER8_Pos                                                (15UL)		/*!<USART CR1: OVER8 (Bit 15) */
#define USART_CR1_OVER8_Msk                                                (0x8000UL)		/*!< USART CR1: OVER8 (Bitfield-Mask: 0x01) */
#define USART_CR1_OVER8                                                    USART_CR1_OVER8_Msk
#define USART_CR1_CMIE_Pos                                                 (14UL)		/*!<USART CR1: CMIE (Bit 14) */
#define USART_CR1_CMIE_Msk                                                 (0x4000UL)		/*!< USART CR1: CMIE (Bitfield-Mask: 0x01) */
#define USART_CR1_CMIE                                                     USART_CR1_CMIE_Msk
#define USART_CR1_MME_Pos                                                  (13UL)		/*!<USART CR1: MME (Bit 13) */
#define USART_CR1_MME_Msk                                                  (0x2000UL)		/*!< USART CR1: MME (Bitfield-Mask: 0x01) */
#define USART_CR1_MME                                                      USART_CR1_MME_Msk
#define USART_CR1_M_Pos                                                    (12UL)		/*!<USART CR1: M (Bit 12) */
#define USART_CR1_M_Msk                                                    (0x10001000UL)		/*!< USART CR1: M (Bitfield-Mask: 0x10001) */
#define USART_CR1_M                                                        USART_CR1_M_Msk
#define USART_CR1_M0                                                       (0x1U << USART_CR1_M_Pos)
#define USART_CR1_M1                                                       (0x10000U << USART_CR1_M_Pos)
#define USART_CR1_WAKE_Pos                                                 (11UL)		/*!<USART CR1: WAKE (Bit 11) */
#define USART_CR1_WAKE_Msk                                                 (0x800UL)		/*!< USART CR1: WAKE (Bitfield-Mask: 0x01) */
#define USART_CR1_WAKE                                                     USART_CR1_WAKE_Msk
#define USART_CR1_PCE_Pos                                                  (10UL)		/*!<USART CR1: PCE (Bit 10) */
#define USART_CR1_PCE_Msk                                                  (0x400UL)		/*!< USART CR1: PCE (Bitfield-Mask: 0x01) */
#define USART_CR1_PCE                                                      USART_CR1_PCE_Msk
#define USART_CR1_PS_Pos                                                   (9UL)		/*!<USART CR1: PS (Bit 9) */
#define USART_CR1_PS_Msk                                                   (0x200UL)		/*!< USART CR1: PS (Bitfield-Mask: 0x01) */
#define USART_CR1_PS                                                       USART_CR1_PS_Msk
#define USART_CR1_PEIE_Pos                                                 (8UL)		/*!<USART CR1: PEIE (Bit 8) */
#define USART_CR1_PEIE_Msk                                                 (0x100UL)		/*!< USART CR1: PEIE (Bitfield-Mask: 0x01) */
#define USART_CR1_PEIE                                                     USART_CR1_PEIE_Msk
#define USART_CR1_TXEIE_TXFNFIE_Pos                                        (7UL)		/*!<USART CR1: TXEIE_TXFNFIE (Bit 7) */
#define USART_CR1_TXEIE_TXFNFIE_Msk                                        (0x80UL)		/*!< USART CR1: TXEIE_TXFNFIE (Bitfield-Mask: 0x01) */
#define USART_CR1_TXEIE_TXFNFIE                                            USART_CR1_TXEIE_TXFNFIE_Msk
#define USART_CR1_TCIE_Pos                                                 (6UL)		/*!<USART CR1: TCIE (Bit 6) */
#define USART_CR1_TCIE_Msk                                                 (0x40UL)		/*!< USART CR1: TCIE (Bitfield-Mask: 0x01) */
#define USART_CR1_TCIE                                                     USART_CR1_TCIE_Msk
#define USART_CR1_RXNEIE_RXFNEIE_Pos                                       (5UL)		/*!<USART CR1: RXNEIE_RXFNEIE (Bit 5) */
#define USART_CR1_RXNEIE_RXFNEIE_Msk                                       (0x20UL)		/*!< USART CR1: RXNEIE_RXFNEIE (Bitfield-Mask: 0x01) */
#define USART_CR1_RXNEIE_RXFNEIE                                           USART_CR1_RXNEIE_RXFNEIE_Msk
#define USART_CR1_IDLEIE_Pos                                               (4UL)		/*!<USART CR1: IDLEIE (Bit 4) */
#define USART_CR1_IDLEIE_Msk                                               (0x10UL)		/*!< USART CR1: IDLEIE (Bitfield-Mask: 0x01) */
#define USART_CR1_IDLEIE                                                   USART_CR1_IDLEIE_Msk
#define USART_CR1_TE_Pos                                                   (3UL)		/*!<USART CR1: TE (Bit 3) */
#define USART_CR1_TE_Msk                                                   (0x8UL)		/*!< USART CR1: TE (Bitfield-Mask: 0x01) */
#define USART_CR1_TE                                                       USART_CR1_TE_Msk
#define USART_CR1_RE_Pos                                                   (2UL)		/*!<USART CR1: RE (Bit 2) */
#define USART_CR1_RE_Msk                                                   (0x4UL)		/*!< USART CR1: RE (Bitfield-Mask: 0x01) */
#define USART_CR1_RE                                                       USART_CR1_RE_Msk
#define USART_CR1_UESM_Pos                                                 (1UL)		/*!<USART CR1: UESM (Bit 1) */
#define USART_CR1_UESM_Msk                                                 (0x2UL)		/*!< USART CR1: UESM (Bitfield-Mask: 0x01) */
#define USART_CR1_UESM                                                     USART_CR1_UESM_Msk
#define USART_CR1_UE_Pos                                                   (0UL)		/*!<USART CR1: UE (Bit 0) */
#define USART_CR1_UE_Msk                                                   (0x1UL)		/*!< USART CR1: UE (Bitfield-Mask: 0x01) */
#define USART_CR1_UE                                                       USART_CR1_UE_Msk

/* =====================================================    CR2    =====================================================*/ 
#define USART_CR2_ADD_Pos                                                  (24UL)		/*!<USART CR2: ADD (Bit 24) */
#define USART_CR2_ADD_Msk                                                  (0xff000000UL)		/*!< USART CR2: ADD (Bitfield-Mask: 0xff) */
#define USART_CR2_ADD                                                      USART_CR2_ADD_Msk
#define USART_CR2_ADD_0                                                    (0x1U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_1                                                    (0x2U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_2                                                    (0x4U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_3                                                    (0x8U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_4                                                    (0x10U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_5                                                    (0x20U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_6                                                    (0x40U << USART_CR2_ADD_Pos)
#define USART_CR2_ADD_7                                                    (0x80U << USART_CR2_ADD_Pos)
#define USART_CR2_RTOEN_Pos                                                (23UL)		/*!<USART CR2: RTOEN (Bit 23) */
#define USART_CR2_RTOEN_Msk                                                (0x800000UL)		/*!< USART CR2: RTOEN (Bitfield-Mask: 0x01) */
#define USART_CR2_RTOEN                                                    USART_CR2_RTOEN_Msk
#define USART_CR2_ABRMODE_Pos                                              (21UL)		/*!<USART CR2: ABRMODE (Bit 21) */
#define USART_CR2_ABRMODE_Msk                                              (0x600000UL)		/*!< USART CR2: ABRMODE (Bitfield-Mask: 0x03) */
#define USART_CR2_ABRMODE                                                  USART_CR2_ABRMODE_Msk
#define USART_CR2_ABRMODE_0                                                (0x1U << USART_CR2_ABRMODE_Pos)
#define USART_CR2_ABRMODE_1                                                (0x2U << USART_CR2_ABRMODE_Pos)
#define USART_CR2_ABREN_Pos                                                (20UL)		/*!<USART CR2: ABREN (Bit 20) */
#define USART_CR2_ABREN_Msk                                                (0x100000UL)		/*!< USART CR2: ABREN (Bitfield-Mask: 0x01) */
#define USART_CR2_ABREN                                                    USART_CR2_ABREN_Msk
#define USART_CR2_MSBFIRST_Pos                                             (19UL)		/*!<USART CR2: MSBFIRST (Bit 19) */
#define USART_CR2_MSBFIRST_Msk                                             (0x80000UL)		/*!< USART CR2: MSBFIRST (Bitfield-Mask: 0x01) */
#define USART_CR2_MSBFIRST                                                 USART_CR2_MSBFIRST_Msk
#define USART_CR2_DATAINV_Pos                                              (18UL)		/*!<USART CR2: DATAINV (Bit 18) */
#define USART_CR2_DATAINV_Msk                                              (0x40000UL)		/*!< USART CR2: DATAINV (Bitfield-Mask: 0x01) */
#define USART_CR2_DATAINV                                                  USART_CR2_DATAINV_Msk
#define USART_CR2_TXINV_Pos                                                (17UL)		/*!<USART CR2: TXINV (Bit 17) */
#define USART_CR2_TXINV_Msk                                                (0x20000UL)		/*!< USART CR2: TXINV (Bitfield-Mask: 0x01) */
#define USART_CR2_TXINV                                                    USART_CR2_TXINV_Msk
#define USART_CR2_RXINV_Pos                                                (16UL)		/*!<USART CR2: RXINV (Bit 16) */
#define USART_CR2_RXINV_Msk                                                (0x10000UL)		/*!< USART CR2: RXINV (Bitfield-Mask: 0x01) */
#define USART_CR2_RXINV                                                    USART_CR2_RXINV_Msk
#define USART_CR2_SWAP_Pos                                                 (15UL)		/*!<USART CR2: SWAP (Bit 15) */
#define USART_CR2_SWAP_Msk                                                 (0x8000UL)		/*!< USART CR2: SWAP (Bitfield-Mask: 0x01) */
#define USART_CR2_SWAP                                                     USART_CR2_SWAP_Msk
#define USART_CR2_LINEN_Pos                                                (14UL)		/*!<USART CR2: LINEN (Bit 14) */
#define USART_CR2_LINEN_Msk                                                (0x4000UL)		/*!< USART CR2: LINEN (Bitfield-Mask: 0x01) */
#define USART_CR2_LINEN                                                    USART_CR2_LINEN_Msk
#define USART_CR2_STOP_Pos                                                 (12UL)		/*!<USART CR2: STOP (Bit 12) */
#define USART_CR2_STOP_Msk                                                 (0x3000UL)		/*!< USART CR2: STOP (Bitfield-Mask: 0x03) */
#define USART_CR2_STOP                                                     USART_CR2_STOP_Msk
#define USART_CR2_STOP_0                                                   (0x1U << USART_CR2_STOP_Pos)
#define USART_CR2_STOP_1                                                   (0x2U << USART_CR2_STOP_Pos)
#define USART_CR2_CLKEN_Pos                                                (11UL)		/*!<USART CR2: CLKEN (Bit 11) */
#define USART_CR2_CLKEN_Msk                                                (0x800UL)		/*!< USART CR2: CLKEN (Bitfield-Mask: 0x01) */
#define USART_CR2_CLKEN                                                    USART_CR2_CLKEN_Msk
#define USART_CR2_CPOL_Pos                                                 (10UL)		/*!<USART CR2: CPOL (Bit 10) */
#define USART_CR2_CPOL_Msk                                                 (0x400UL)		/*!< USART CR2: CPOL (Bitfield-Mask: 0x01) */
#define USART_CR2_CPOL                                                     USART_CR2_CPOL_Msk
#define USART_CR2_CPHA_Pos                                                 (9UL)		/*!<USART CR2: CPHA (Bit 9) */
#define USART_CR2_CPHA_Msk                                                 (0x200UL)		/*!< USART CR2: CPHA (Bitfield-Mask: 0x01) */
#define USART_CR2_CPHA                                                     USART_CR2_CPHA_Msk
#define USART_CR2_LBCL_Pos                                                 (8UL)		/*!<USART CR2: LBCL (Bit 8) */
#define USART_CR2_LBCL_Msk                                                 (0x100UL)		/*!< USART CR2: LBCL (Bitfield-Mask: 0x01) */
#define USART_CR2_LBCL                                                     USART_CR2_LBCL_Msk
#define USART_CR2_LBDIE_Pos                                                (6UL)		/*!<USART CR2: LBDIE (Bit 6) */
#define USART_CR2_LBDIE_Msk                                                (0x40UL)		/*!< USART CR2: LBDIE (Bitfield-Mask: 0x01) */
#define USART_CR2_LBDIE                                                    USART_CR2_LBDIE_Msk
#define USART_CR2_LBDL_Pos                                                 (5UL)		/*!<USART CR2: LBDL (Bit 5) */
#define USART_CR2_LBDL_Msk                                                 (0x20UL)		/*!< USART CR2: LBDL (Bitfield-Mask: 0x01) */
#define USART_CR2_LBDL                                                     USART_CR2_LBDL_Msk
#define USART_CR2_ADDM7_Pos                                                (4UL)		/*!<USART CR2: ADDM7 (Bit 4) */
#define USART_CR2_ADDM7_Msk                                                (0x10UL)		/*!< USART CR2: ADDM7 (Bitfield-Mask: 0x01) */
#define USART_CR2_ADDM7                                                    USART_CR2_ADDM7_Msk
#define USART_CR2_DIS_NSS_Pos                                              (3UL)		/*!<USART CR2: DIS_NSS (Bit 3) */
#define USART_CR2_DIS_NSS_Msk                                              (0x8UL)		/*!< USART CR2: DIS_NSS (Bitfield-Mask: 0x01) */
#define USART_CR2_DIS_NSS                                                  USART_CR2_DIS_NSS_Msk
#define USART_CR2_SLVEN_Pos                                                (0UL)		/*!<USART CR2: SLVEN (Bit 0) */
#define USART_CR2_SLVEN_Msk                                                (0x1UL)		/*!< USART CR2: SLVEN (Bitfield-Mask: 0x01) */
#define USART_CR2_SLVEN                                                    USART_CR2_SLVEN_Msk

/* =====================================================    CR3    =====================================================*/ 
#define USART_CR3_TXFTCFG_Pos                                              (29UL)		/*!<USART CR3: TXFTCFG (Bit 29) */
#define USART_CR3_TXFTCFG_Msk                                              (0xe0000000UL)		/*!< USART CR3: TXFTCFG (Bitfield-Mask: 0x07) */
#define USART_CR3_TXFTCFG                                                  USART_CR3_TXFTCFG_Msk
#define USART_CR3_TXFTCFG_0                                                (0x1U << USART_CR3_TXFTCFG_Pos)
#define USART_CR3_TXFTCFG_1                                                (0x2U << USART_CR3_TXFTCFG_Pos)
#define USART_CR3_TXFTCFG_2                                                (0x4U << USART_CR3_TXFTCFG_Pos)
#define USART_CR3_RXFTIE_Pos                                               (28UL)		/*!<USART CR3: RXFTIE (Bit 28) */
#define USART_CR3_RXFTIE_Msk                                               (0x10000000UL)		/*!< USART CR3: RXFTIE (Bitfield-Mask: 0x01) */
#define USART_CR3_RXFTIE                                                   USART_CR3_RXFTIE_Msk
#define USART_CR3_RXFTCFG_Pos                                              (25UL)		/*!<USART CR3: RXFTCFG (Bit 25) */
#define USART_CR3_RXFTCFG_Msk                                              (0xe000000UL)		/*!< USART CR3: RXFTCFG (Bitfield-Mask: 0x07) */
#define USART_CR3_RXFTCFG                                                  USART_CR3_RXFTCFG_Msk
#define USART_CR3_RXFTCFG_0                                                (0x1U << USART_CR3_RXFTCFG_Pos)
#define USART_CR3_RXFTCFG_1                                                (0x2U << USART_CR3_RXFTCFG_Pos)
#define USART_CR3_RXFTCFG_2                                                (0x4U << USART_CR3_RXFTCFG_Pos)
#define USART_CR3_TCBGTIE_Pos                                              (24UL)		/*!<USART CR3: TCBGTIE (Bit 24) */
#define USART_CR3_TCBGTIE_Msk                                              (0x1000000UL)		/*!< USART CR3: TCBGTIE (Bitfield-Mask: 0x01) */
#define USART_CR3_TCBGTIE                                                  USART_CR3_TCBGTIE_Msk
#define USART_CR3_TXFTIE_Pos                                               (23UL)		/*!<USART CR3: TXFTIE (Bit 23) */
#define USART_CR3_TXFTIE_Msk                                               (0x800000UL)		/*!< USART CR3: TXFTIE (Bitfield-Mask: 0x01) */
#define USART_CR3_TXFTIE                                                   USART_CR3_TXFTIE_Msk
#define USART_CR3_WUFIE_Pos                                                (22UL)		/*!<USART CR3: WUFIE (Bit 22) */
#define USART_CR3_WUFIE_Msk                                                (0x400000UL)		/*!< USART CR3: WUFIE (Bitfield-Mask: 0x01) */
#define USART_CR3_WUFIE                                                    USART_CR3_WUFIE_Msk
#define USART_CR3_WUS_Pos                                                  (20UL)		/*!<USART CR3: WUS (Bit 20) */
#define USART_CR3_WUS_Msk                                                  (0x300000UL)		/*!< USART CR3: WUS (Bitfield-Mask: 0x03) */
#define USART_CR3_WUS                                                      USART_CR3_WUS_Msk
#define USART_CR3_WUS_0                                                    (0x1U << USART_CR3_WUS_Pos)
#define USART_CR3_WUS_1                                                    (0x2U << USART_CR3_WUS_Pos)
#define USART_CR3_SCARCNT_Pos                                              (17UL)		/*!<USART CR3: SCARCNT (Bit 17) */
#define USART_CR3_SCARCNT_Msk                                              (0xe0000UL)		/*!< USART CR3: SCARCNT (Bitfield-Mask: 0x07) */
#define USART_CR3_SCARCNT                                                  USART_CR3_SCARCNT_Msk
#define USART_CR3_SCARCNT_0                                                (0x1U << USART_CR3_SCARCNT_Pos)
#define USART_CR3_SCARCNT_1                                                (0x2U << USART_CR3_SCARCNT_Pos)
#define USART_CR3_SCARCNT_2                                                (0x4U << USART_CR3_SCARCNT_Pos)
#define USART_CR3_DEP_Pos                                                  (15UL)		/*!<USART CR3: DEP (Bit 15) */
#define USART_CR3_DEP_Msk                                                  (0x8000UL)		/*!< USART CR3: DEP (Bitfield-Mask: 0x01) */
#define USART_CR3_DEP                                                      USART_CR3_DEP_Msk
#define USART_CR3_DEM_Pos                                                  (14UL)		/*!<USART CR3: DEM (Bit 14) */
#define USART_CR3_DEM_Msk                                                  (0x4000UL)		/*!< USART CR3: DEM (Bitfield-Mask: 0x01) */
#define USART_CR3_DEM                                                      USART_CR3_DEM_Msk
#define USART_CR3_DDRE_Pos                                                 (13UL)		/*!<USART CR3: DDRE (Bit 13) */
#define USART_CR3_DDRE_Msk                                                 (0x2000UL)		/*!< USART CR3: DDRE (Bitfield-Mask: 0x01) */
#define USART_CR3_DDRE                                                     USART_CR3_DDRE_Msk
#define USART_CR3_OVRDIS_Pos                                               (12UL)		/*!<USART CR3: OVRDIS (Bit 12) */
#define USART_CR3_OVRDIS_Msk                                               (0x1000UL)		/*!< USART CR3: OVRDIS (Bitfield-Mask: 0x01) */
#define USART_CR3_OVRDIS                                                   USART_CR3_OVRDIS_Msk
#define USART_CR3_ONEBIT_Pos                                               (11UL)		/*!<USART CR3: ONEBIT (Bit 11) */
#define USART_CR3_ONEBIT_Msk                                               (0x800UL)		/*!< USART CR3: ONEBIT (Bitfield-Mask: 0x01) */
#define USART_CR3_ONEBIT                                                   USART_CR3_ONEBIT_Msk
#define USART_CR3_CTSIE_Pos                                                (10UL)		/*!<USART CR3: CTSIE (Bit 10) */
#define USART_CR3_CTSIE_Msk                                                (0x400UL)		/*!< USART CR3: CTSIE (Bitfield-Mask: 0x01) */
#define USART_CR3_CTSIE                                                    USART_CR3_CTSIE_Msk
#define USART_CR3_CTSE_Pos                                                 (9UL)		/*!<USART CR3: CTSE (Bit 9) */
#define USART_CR3_CTSE_Msk                                                 (0x200UL)		/*!< USART CR3: CTSE (Bitfield-Mask: 0x01) */
#define USART_CR3_CTSE                                                     USART_CR3_CTSE_Msk
#define USART_CR3_RTSE_Pos                                                 (8UL)		/*!<USART CR3: RTSE (Bit 8) */
#define USART_CR3_RTSE_Msk                                                 (0x100UL)		/*!< USART CR3: RTSE (Bitfield-Mask: 0x01) */
#define USART_CR3_RTSE                                                     USART_CR3_RTSE_Msk
#define USART_CR3_DMAT_Pos                                                 (7UL)		/*!<USART CR3: DMAT (Bit 7) */
#define USART_CR3_DMAT_Msk                                                 (0x80UL)		/*!< USART CR3: DMAT (Bitfield-Mask: 0x01) */
#define USART_CR3_DMAT                                                     USART_CR3_DMAT_Msk
#define USART_CR3_DMAR_Pos                                                 (6UL)		/*!<USART CR3: DMAR (Bit 6) */
#define USART_CR3_DMAR_Msk                                                 (0x40UL)		/*!< USART CR3: DMAR (Bitfield-Mask: 0x01) */
#define USART_CR3_DMAR                                                     USART_CR3_DMAR_Msk
#define USART_CR3_SCEN_Pos                                                 (5UL)		/*!<USART CR3: SCEN (Bit 5) */
#define USART_CR3_SCEN_Msk                                                 (0x20UL)		/*!< USART CR3: SCEN (Bitfield-Mask: 0x01) */
#define USART_CR3_SCEN                                                     USART_CR3_SCEN_Msk
#define USART_CR3_NACK_Pos                                                 (4UL)		/*!<USART CR3: NACK (Bit 4) */
#define USART_CR3_NACK_Msk                                                 (0x10UL)		/*!< USART CR3: NACK (Bitfield-Mask: 0x01) */
#define USART_CR3_NACK                                                     USART_CR3_NACK_Msk
#define USART_CR3_HDSEL_Pos                                                (3UL)		/*!<USART CR3: HDSEL (Bit 3) */
#define USART_CR3_HDSEL_Msk                                                (0x8UL)		/*!< USART CR3: HDSEL (Bitfield-Mask: 0x01) */
#define USART_CR3_HDSEL                                                    USART_CR3_HDSEL_Msk
#define USART_CR3_IRLP_Pos                                                 (2UL)		/*!<USART CR3: IRLP (Bit 2) */
#define USART_CR3_IRLP_Msk                                                 (0x4UL)		/*!< USART CR3: IRLP (Bitfield-Mask: 0x01) */
#define USART_CR3_IRLP                                                     USART_CR3_IRLP_Msk
#define USART_CR3_IREN_Pos                                                 (1UL)		/*!<USART CR3: IREN (Bit 1) */
#define USART_CR3_IREN_Msk                                                 (0x2UL)		/*!< USART CR3: IREN (Bitfield-Mask: 0x01) */
#define USART_CR3_IREN                                                     USART_CR3_IREN_Msk
#define USART_CR3_EIE_Pos                                                  (0UL)		/*!<USART CR3: EIE (Bit 0) */
#define USART_CR3_EIE_Msk                                                  (0x1UL)		/*!< USART CR3: EIE (Bitfield-Mask: 0x01) */
#define USART_CR3_EIE                                                      USART_CR3_EIE_Msk

/* =====================================================    BRR    =====================================================*/ 
#define USART_BRR_BRR_Pos                                                  (0UL)		/*!<USART BRR: BRR (Bit 0) */
#define USART_BRR_BRR_Msk                                                  (0xfffffUL)		/*!< USART BRR: BRR (Bitfield-Mask: 0xfffff) */
#define USART_BRR_BRR                                                      USART_BRR_BRR_Msk
#define USART_BRR_BRR_0                                                    (0x1U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_1                                                    (0x2U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_2                                                    (0x4U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_3                                                    (0x8U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_4                                                    (0x10U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_5                                                    (0x20U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_6                                                    (0x40U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_7                                                    (0x80U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_8                                                    (0x100U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_9                                                    (0x200U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_10                                                   (0x400U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_11                                                   (0x800U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_12                                                   (0x1000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_13                                                   (0x2000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_14                                                   (0x4000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_15                                                   (0x8000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_16                                                   (0x10000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_17                                                   (0x20000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_18                                                   (0x40000U << USART_BRR_BRR_Pos)
#define USART_BRR_BRR_19                                                   (0x80000U << USART_BRR_BRR_Pos)

/* =====================================================    GTPR    =====================================================*/ 
#define USART_GTPR_GT_Pos                                                  (8UL)		/*!<USART GTPR: GT (Bit 8) */
#define USART_GTPR_GT_Msk                                                  (0xff00UL)		/*!< USART GTPR: GT (Bitfield-Mask: 0xff) */
#define USART_GTPR_GT                                                      USART_GTPR_GT_Msk
#define USART_GTPR_GT_0                                                    (0x1U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_1                                                    (0x2U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_2                                                    (0x4U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_3                                                    (0x8U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_4                                                    (0x10U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_5                                                    (0x20U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_6                                                    (0x40U << USART_GTPR_GT_Pos)
#define USART_GTPR_GT_7                                                    (0x80U << USART_GTPR_GT_Pos)
#define USART_GTPR_PSC_Pos                                                 (0UL)		/*!<USART GTPR: PSC (Bit 0) */
#define USART_GTPR_PSC_Msk                                                 (0xffUL)		/*!< USART GTPR: PSC (Bitfield-Mask: 0xff) */
#define USART_GTPR_PSC                                                     USART_GTPR_PSC_Msk
#define USART_GTPR_PSC_0                                                   (0x1U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_1                                                   (0x2U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_2                                                   (0x4U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_3                                                   (0x8U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_4                                                   (0x10U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_5                                                   (0x20U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_6                                                   (0x40U << USART_GTPR_PSC_Pos)
#define USART_GTPR_PSC_7                                                   (0x80U << USART_GTPR_PSC_Pos)

/* =====================================================    RTOR    =====================================================*/ 
#define USART_RTOR_BLEN_Pos                                                (24UL)		/*!<USART RTOR: BLEN (Bit 24) */
#define USART_RTOR_BLEN_Msk                                                (0xff000000UL)		/*!< USART RTOR: BLEN (Bitfield-Mask: 0xff) */
#define USART_RTOR_BLEN                                                    USART_RTOR_BLEN_Msk
#define USART_RTOR_BLEN_0                                                  (0x1U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_1                                                  (0x2U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_2                                                  (0x4U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_3                                                  (0x8U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_4                                                  (0x10U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_5                                                  (0x20U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_6                                                  (0x40U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_BLEN_7                                                  (0x80U << USART_RTOR_BLEN_Pos)
#define USART_RTOR_RTO_Pos                                                 (0UL)		/*!<USART RTOR: RTO (Bit 0) */
#define USART_RTOR_RTO_Msk                                                 (0xffffffUL)		/*!< USART RTOR: RTO (Bitfield-Mask: 0xffffff) */
#define USART_RTOR_RTO                                                     USART_RTOR_RTO_Msk
#define USART_RTOR_RTO_0                                                   (0x1U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_1                                                   (0x2U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_2                                                   (0x4U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_3                                                   (0x8U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_4                                                   (0x10U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_5                                                   (0x20U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_6                                                   (0x40U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_7                                                   (0x80U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_8                                                   (0x100U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_9                                                   (0x200U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_10                                                  (0x400U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_11                                                  (0x800U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_12                                                  (0x1000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_13                                                  (0x2000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_14                                                  (0x4000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_15                                                  (0x8000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_16                                                  (0x10000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_17                                                  (0x20000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_18                                                  (0x40000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_19                                                  (0x80000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_20                                                  (0x100000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_21                                                  (0x200000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_22                                                  (0x400000U << USART_RTOR_RTO_Pos)
#define USART_RTOR_RTO_23                                                  (0x800000U << USART_RTOR_RTO_Pos)

/* =====================================================    RQR    =====================================================*/ 
#define USART_RQR_TXFRQ_Pos                                                (4UL)		/*!<USART RQR: TXFRQ (Bit 4) */
#define USART_RQR_TXFRQ_Msk                                                (0x10UL)		/*!< USART RQR: TXFRQ (Bitfield-Mask: 0x01) */
#define USART_RQR_TXFRQ                                                    USART_RQR_TXFRQ_Msk
#define USART_RQR_RXFRQ_Pos                                                (3UL)		/*!<USART RQR: RXFRQ (Bit 3) */
#define USART_RQR_RXFRQ_Msk                                                (0x8UL)		/*!< USART RQR: RXFRQ (Bitfield-Mask: 0x01) */
#define USART_RQR_RXFRQ                                                    USART_RQR_RXFRQ_Msk
#define USART_RQR_MMRQ_Pos                                                 (2UL)		/*!<USART RQR: MMRQ (Bit 2) */
#define USART_RQR_MMRQ_Msk                                                 (0x4UL)		/*!< USART RQR: MMRQ (Bitfield-Mask: 0x01) */
#define USART_RQR_MMRQ                                                     USART_RQR_MMRQ_Msk
#define USART_RQR_SBKRQ_Pos                                                (1UL)		/*!<USART RQR: SBKRQ (Bit 1) */
#define USART_RQR_SBKRQ_Msk                                                (0x2UL)		/*!< USART RQR: SBKRQ (Bitfield-Mask: 0x01) */
#define USART_RQR_SBKRQ                                                    USART_RQR_SBKRQ_Msk
#define USART_RQR_ABRRQ_Pos                                                (0UL)		/*!<USART RQR: ABRRQ (Bit 0) */
#define USART_RQR_ABRRQ_Msk                                                (0x1UL)		/*!< USART RQR: ABRRQ (Bitfield-Mask: 0x01) */
#define USART_RQR_ABRRQ                                                    USART_RQR_ABRRQ_Msk

/* =====================================================    ISR    =====================================================*/ 
#define USART_ISR_TXFT_Pos                                                 (27UL)		/*!<USART ISR: TXFT (Bit 27) */
#define USART_ISR_TXFT_Msk                                                 (0x8000000UL)		/*!< USART ISR: TXFT (Bitfield-Mask: 0x01) */
#define USART_ISR_TXFT                                                     USART_ISR_TXFT_Msk
#define USART_ISR_RXFT_Pos                                                 (26UL)		/*!<USART ISR: RXFT (Bit 26) */
#define USART_ISR_RXFT_Msk                                                 (0x4000000UL)		/*!< USART ISR: RXFT (Bitfield-Mask: 0x01) */
#define USART_ISR_RXFT                                                     USART_ISR_RXFT_Msk
#define USART_ISR_TCBGT_Pos                                                (25UL)		/*!<USART ISR: TCBGT (Bit 25) */
#define USART_ISR_TCBGT_Msk                                                (0x2000000UL)		/*!< USART ISR: TCBGT (Bitfield-Mask: 0x01) */
#define USART_ISR_TCBGT                                                    USART_ISR_TCBGT_Msk
#define USART_ISR_RXFF_Pos                                                 (24UL)		/*!<USART ISR: RXFF (Bit 24) */
#define USART_ISR_RXFF_Msk                                                 (0x1000000UL)		/*!< USART ISR: RXFF (Bitfield-Mask: 0x01) */
#define USART_ISR_RXFF                                                     USART_ISR_RXFF_Msk
#define USART_ISR_TXFE_Pos                                                 (23UL)		/*!<USART ISR: TXFE (Bit 23) */
#define USART_ISR_TXFE_Msk                                                 (0x800000UL)		/*!< USART ISR: TXFE (Bitfield-Mask: 0x01) */
#define USART_ISR_TXFE                                                     USART_ISR_TXFE_Msk
#define USART_ISR_REACK_Pos                                                (22UL)		/*!<USART ISR: REACK (Bit 22) */
#define USART_ISR_REACK_Msk                                                (0x400000UL)		/*!< USART ISR: REACK (Bitfield-Mask: 0x01) */
#define USART_ISR_REACK                                                    USART_ISR_REACK_Msk
#define USART_ISR_TEACK_Pos                                                (21UL)		/*!<USART ISR: TEACK (Bit 21) */
#define USART_ISR_TEACK_Msk                                                (0x200000UL)		/*!< USART ISR: TEACK (Bitfield-Mask: 0x01) */
#define USART_ISR_TEACK                                                    USART_ISR_TEACK_Msk
#define USART_ISR_WUF_Pos                                                  (20UL)		/*!<USART ISR: WUF (Bit 20) */
#define USART_ISR_WUF_Msk                                                  (0x100000UL)		/*!< USART ISR: WUF (Bitfield-Mask: 0x01) */
#define USART_ISR_WUF                                                      USART_ISR_WUF_Msk
#define USART_ISR_RWU_Pos                                                  (19UL)		/*!<USART ISR: RWU (Bit 19) */
#define USART_ISR_RWU_Msk                                                  (0x80000UL)		/*!< USART ISR: RWU (Bitfield-Mask: 0x01) */
#define USART_ISR_RWU                                                      USART_ISR_RWU_Msk
#define USART_ISR_SBKF_Pos                                                 (18UL)		/*!<USART ISR: SBKF (Bit 18) */
#define USART_ISR_SBKF_Msk                                                 (0x40000UL)		/*!< USART ISR: SBKF (Bitfield-Mask: 0x01) */
#define USART_ISR_SBKF                                                     USART_ISR_SBKF_Msk
#define USART_ISR_CMF_Pos                                                  (17UL)		/*!<USART ISR: CMF (Bit 17) */
#define USART_ISR_CMF_Msk                                                  (0x20000UL)		/*!< USART ISR: CMF (Bitfield-Mask: 0x01) */
#define USART_ISR_CMF                                                      USART_ISR_CMF_Msk
#define USART_ISR_BUSY_Pos                                                 (16UL)		/*!<USART ISR: BUSY (Bit 16) */
#define USART_ISR_BUSY_Msk                                                 (0x10000UL)		/*!< USART ISR: BUSY (Bitfield-Mask: 0x01) */
#define USART_ISR_BUSY                                                     USART_ISR_BUSY_Msk
#define USART_ISR_ABRF_Pos                                                 (15UL)		/*!<USART ISR: ABRF (Bit 15) */
#define USART_ISR_ABRF_Msk                                                 (0x8000UL)		/*!< USART ISR: ABRF (Bitfield-Mask: 0x01) */
#define USART_ISR_ABRF                                                     USART_ISR_ABRF_Msk
#define USART_ISR_ABRE_Pos                                                 (14UL)		/*!<USART ISR: ABRE (Bit 14) */
#define USART_ISR_ABRE_Msk                                                 (0x4000UL)		/*!< USART ISR: ABRE (Bitfield-Mask: 0x01) */
#define USART_ISR_ABRE                                                     USART_ISR_ABRE_Msk
#define USART_ISR_UDR_Pos                                                  (13UL)		/*!<USART ISR: UDR (Bit 13) */
#define USART_ISR_UDR_Msk                                                  (0x2000UL)		/*!< USART ISR: UDR (Bitfield-Mask: 0x01) */
#define USART_ISR_UDR                                                      USART_ISR_UDR_Msk
#define USART_ISR_EOBF_Pos                                                 (12UL)		/*!<USART ISR: EOBF (Bit 12) */
#define USART_ISR_EOBF_Msk                                                 (0x1000UL)		/*!< USART ISR: EOBF (Bitfield-Mask: 0x01) */
#define USART_ISR_EOBF                                                     USART_ISR_EOBF_Msk
#define USART_ISR_RTOF_Pos                                                 (11UL)		/*!<USART ISR: RTOF (Bit 11) */
#define USART_ISR_RTOF_Msk                                                 (0x800UL)		/*!< USART ISR: RTOF (Bitfield-Mask: 0x01) */
#define USART_ISR_RTOF                                                     USART_ISR_RTOF_Msk
#define USART_ISR_CTS_Pos                                                  (10UL)		/*!<USART ISR: CTS (Bit 10) */
#define USART_ISR_CTS_Msk                                                  (0x400UL)		/*!< USART ISR: CTS (Bitfield-Mask: 0x01) */
#define USART_ISR_CTS                                                      USART_ISR_CTS_Msk
#define USART_ISR_CTSIF_Pos                                                (9UL)		/*!<USART ISR: CTSIF (Bit 9) */
#define USART_ISR_CTSIF_Msk                                                (0x200UL)		/*!< USART ISR: CTSIF (Bitfield-Mask: 0x01) */
#define USART_ISR_CTSIF                                                    USART_ISR_CTSIF_Msk
#define USART_ISR_LBDF_Pos                                                 (8UL)		/*!<USART ISR: LBDF (Bit 8) */
#define USART_ISR_LBDF_Msk                                                 (0x100UL)		/*!< USART ISR: LBDF (Bitfield-Mask: 0x01) */
#define USART_ISR_LBDF                                                     USART_ISR_LBDF_Msk
#define USART_ISR_TXE_TXFNF_Pos                                            (7UL)		/*!<USART ISR: TXE_TXFNF (Bit 7) */
#define USART_ISR_TXE_TXFNF_Msk                                            (0x80UL)		/*!< USART ISR: TXE_TXFNF (Bitfield-Mask: 0x01) */
#define USART_ISR_TXE_TXFNF                                                USART_ISR_TXE_TXFNF_Msk
#define USART_ISR_TC_Pos                                                   (6UL)		/*!<USART ISR: TC (Bit 6) */
#define USART_ISR_TC_Msk                                                   (0x40UL)		/*!< USART ISR: TC (Bitfield-Mask: 0x01) */
#define USART_ISR_TC                                                       USART_ISR_TC_Msk
#define USART_ISR_RXNE_RXFNE_Pos                                           (5UL)		/*!<USART ISR: RXNE_RXFNE (Bit 5) */
#define USART_ISR_RXNE_RXFNE_Msk                                           (0x20UL)		/*!< USART ISR: RXNE_RXFNE (Bitfield-Mask: 0x01) */
#define USART_ISR_RXNE_RXFNE                                               USART_ISR_RXNE_RXFNE_Msk
#define USART_ISR_IDLE_Pos                                                 (4UL)		/*!<USART ISR: IDLE (Bit 4) */
#define USART_ISR_IDLE_Msk                                                 (0x10UL)		/*!< USART ISR: IDLE (Bitfield-Mask: 0x01) */
#define USART_ISR_IDLE                                                     USART_ISR_IDLE_Msk
#define USART_ISR_ORE_Pos                                                  (3UL)		/*!<USART ISR: ORE (Bit 3) */
#define USART_ISR_ORE_Msk                                                  (0x8UL)		/*!< USART ISR: ORE (Bitfield-Mask: 0x01) */
#define USART_ISR_ORE                                                      USART_ISR_ORE_Msk
#define USART_ISR_NE_Pos                                                   (2UL)		/*!<USART ISR: NE (Bit 2) */
#define USART_ISR_NE_Msk                                                   (0x4UL)		/*!< USART ISR: NE (Bitfield-Mask: 0x01) */
#define USART_ISR_NE                                                       USART_ISR_NE_Msk
#define USART_ISR_FE_Pos                                                   (1UL)		/*!<USART ISR: FE (Bit 1) */
#define USART_ISR_FE_Msk                                                   (0x2UL)		/*!< USART ISR: FE (Bitfield-Mask: 0x01) */
#define USART_ISR_FE                                                       USART_ISR_FE_Msk
#define USART_ISR_PE_Pos                                                   (0UL)		/*!<USART ISR: PE (Bit 0) */
#define USART_ISR_PE_Msk                                                   (0x1UL)		/*!< USART ISR: PE (Bitfield-Mask: 0x01) */
#define USART_ISR_PE                                                       USART_ISR_PE_Msk

/* =====================================================    ICR    =====================================================*/ 
#define USART_ICR_WUCF_Pos                                                 (20UL)		/*!<USART ICR: WUCF (Bit 20) */
#define USART_ICR_WUCF_Msk                                                 (0x100000UL)		/*!< USART ICR: WUCF (Bitfield-Mask: 0x01) */
#define USART_ICR_WUCF                                                     USART_ICR_WUCF_Msk
#define USART_ICR_CMCF_Pos                                                 (17UL)		/*!<USART ICR: CMCF (Bit 17) */
#define USART_ICR_CMCF_Msk                                                 (0x20000UL)		/*!< USART ICR: CMCF (Bitfield-Mask: 0x01) */
#define USART_ICR_CMCF                                                     USART_ICR_CMCF_Msk
#define USART_ICR_UDRCF_Pos                                                (13UL)		/*!<USART ICR: UDRCF (Bit 13) */
#define USART_ICR_UDRCF_Msk                                                (0x2000UL)		/*!< USART ICR: UDRCF (Bitfield-Mask: 0x01) */
#define USART_ICR_UDRCF                                                    USART_ICR_UDRCF_Msk
#define USART_ICR_EOBCF_Pos                                                (12UL)		/*!<USART ICR: EOBCF (Bit 12) */
#define USART_ICR_EOBCF_Msk                                                (0x1000UL)		/*!< USART ICR: EOBCF (Bitfield-Mask: 0x01) */
#define USART_ICR_EOBCF                                                    USART_ICR_EOBCF_Msk
#define USART_ICR_RTOCF_Pos                                                (11UL)		/*!<USART ICR: RTOCF (Bit 11) */
#define USART_ICR_RTOCF_Msk                                                (0x800UL)		/*!< USART ICR: RTOCF (Bitfield-Mask: 0x01) */
#define USART_ICR_RTOCF                                                    USART_ICR_RTOCF_Msk
#define USART_ICR_CTSCF_Pos                                                (9UL)		/*!<USART ICR: CTSCF (Bit 9) */
#define USART_ICR_CTSCF_Msk                                                (0x200UL)		/*!< USART ICR: CTSCF (Bitfield-Mask: 0x01) */
#define USART_ICR_CTSCF                                                    USART_ICR_CTSCF_Msk
#define USART_ICR_LBDCF_Pos                                                (8UL)		/*!<USART ICR: LBDCF (Bit 8) */
#define USART_ICR_LBDCF_Msk                                                (0x100UL)		/*!< USART ICR: LBDCF (Bitfield-Mask: 0x01) */
#define USART_ICR_LBDCF                                                    USART_ICR_LBDCF_Msk
#define USART_ICR_TCBGTCF_Pos                                              (7UL)		/*!<USART ICR: TCBGTCF (Bit 7) */
#define USART_ICR_TCBGTCF_Msk                                              (0x80UL)		/*!< USART ICR: TCBGTCF (Bitfield-Mask: 0x01) */
#define USART_ICR_TCBGTCF                                                  USART_ICR_TCBGTCF_Msk
#define USART_ICR_TCCF_Pos                                                 (6UL)		/*!<USART ICR: TCCF (Bit 6) */
#define USART_ICR_TCCF_Msk                                                 (0x40UL)		/*!< USART ICR: TCCF (Bitfield-Mask: 0x01) */
#define USART_ICR_TCCF                                                     USART_ICR_TCCF_Msk
#define USART_ICR_TXFECF_Pos                                               (5UL)		/*!<USART ICR: TXFECF (Bit 5) */
#define USART_ICR_TXFECF_Msk                                               (0x20UL)		/*!< USART ICR: TXFECF (Bitfield-Mask: 0x01) */
#define USART_ICR_TXFECF                                                   USART_ICR_TXFECF_Msk
#define USART_ICR_IDLECF_Pos                                               (4UL)		/*!<USART ICR: IDLECF (Bit 4) */
#define USART_ICR_IDLECF_Msk                                               (0x10UL)		/*!< USART ICR: IDLECF (Bitfield-Mask: 0x01) */
#define USART_ICR_IDLECF                                                   USART_ICR_IDLECF_Msk
#define USART_ICR_ORECF_Pos                                                (3UL)		/*!<USART ICR: ORECF (Bit 3) */
#define USART_ICR_ORECF_Msk                                                (0x8UL)		/*!< USART ICR: ORECF (Bitfield-Mask: 0x01) */
#define USART_ICR_ORECF                                                    USART_ICR_ORECF_Msk
#define USART_ICR_NECF_Pos                                                 (2UL)		/*!<USART ICR: NECF (Bit 2) */
#define USART_ICR_NECF_Msk                                                 (0x4UL)		/*!< USART ICR: NECF (Bitfield-Mask: 0x01) */
#define USART_ICR_NECF                                                     USART_ICR_NECF_Msk
#define USART_ICR_FECF_Pos                                                 (1UL)		/*!<USART ICR: FECF (Bit 1) */
#define USART_ICR_FECF_Msk                                                 (0x2UL)		/*!< USART ICR: FECF (Bitfield-Mask: 0x01) */
#define USART_ICR_FECF                                                     USART_ICR_FECF_Msk
#define USART_ICR_PECF_Pos                                                 (0UL)		/*!<USART ICR: PECF (Bit 0) */
#define USART_ICR_PECF_Msk                                                 (0x1UL)		/*!< USART ICR: PECF (Bitfield-Mask: 0x01) */
#define USART_ICR_PECF                                                     USART_ICR_PECF_Msk

/* =====================================================    RDR    =====================================================*/ 
#define USART_RDR_RDR_Pos                                                  (0UL)		/*!<USART RDR: RDR (Bit 0) */
#define USART_RDR_RDR_Msk                                                  (0x1ffUL)		/*!< USART RDR: RDR (Bitfield-Mask: 0x1ff) */
#define USART_RDR_RDR                                                      USART_RDR_RDR_Msk
#define USART_RDR_RDR_0                                                    (0x1U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_1                                                    (0x2U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_2                                                    (0x4U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_3                                                    (0x8U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_4                                                    (0x10U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_5                                                    (0x20U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_6                                                    (0x40U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_7                                                    (0x80U << USART_RDR_RDR_Pos)
#define USART_RDR_RDR_8                                                    (0x100U << USART_RDR_RDR_Pos)

/* =====================================================    TDR    =====================================================*/ 
#define USART_TDR_TDR_Pos                                                  (0UL)		/*!<USART TDR: TDR (Bit 0) */
#define USART_TDR_TDR_Msk                                                  (0x1ffUL)		/*!< USART TDR: TDR (Bitfield-Mask: 0x1ff) */
#define USART_TDR_TDR                                                      USART_TDR_TDR_Msk
#define USART_TDR_TDR_0                                                    (0x1U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_1                                                    (0x2U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_2                                                    (0x4U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_3                                                    (0x8U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_4                                                    (0x10U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_5                                                    (0x20U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_6                                                    (0x40U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_7                                                    (0x80U << USART_TDR_TDR_Pos)
#define USART_TDR_TDR_8                                                    (0x100U << USART_TDR_TDR_Pos)

/* =====================================================    PRESC    =====================================================*/ 
#define USART_PRESC_PRESCALER_Pos                                          (0UL)		/*!<USART PRESC: PRESCALER (Bit 0) */
#define USART_PRESC_PRESCALER_Msk                                          (0xfUL)		/*!< USART PRESC: PRESCALER (Bitfield-Mask: 0x0f) */
#define USART_PRESC_PRESCALER                                              USART_PRESC_PRESCALER_Msk
#define USART_PRESC_PRESCALER_0                                            (0x1U << USART_PRESC_PRESCALER_Pos)
#define USART_PRESC_PRESCALER_1                                            (0x2U << USART_PRESC_PRESCALER_Pos)
#define USART_PRESC_PRESCALER_2                                            (0x4U << USART_PRESC_PRESCALER_Pos)
#define USART_PRESC_PRESCALER_3                                            (0x8U << USART_PRESC_PRESCALER_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        ADC                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    VERSION_ID    =====================================================*/ 
#define ADC_VERSION_ID_VERSION_ID_Pos                                      (0UL)		/*!<ADC VERSION_ID: VERSION_ID (Bit 0) */
#define ADC_VERSION_ID_VERSION_ID_Msk                                      (0xffUL)		/*!< ADC VERSION_ID: VERSION_ID (Bitfield-Mask: 0xff) */
#define ADC_VERSION_ID_VERSION_ID                                          ADC_VERSION_ID_VERSION_ID_Msk
#define ADC_VERSION_ID_VERSION_ID_0                                        (0x1U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_1                                        (0x2U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_2                                        (0x4U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_3                                        (0x8U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_4                                        (0x10U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_5                                        (0x20U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_6                                        (0x40U << ADC_VERSION_ID_VERSION_ID_Pos)
#define ADC_VERSION_ID_VERSION_ID_7                                        (0x80U << ADC_VERSION_ID_VERSION_ID_Pos)

/* =====================================================    CONF    =====================================================*/ 
#define ADC_CONF_SAMPLE_RATE_MSB_Pos                                       (21UL)		/*!<ADC CONF: SAMPLE_RATE_MSB (Bit 21) */
#define ADC_CONF_SAMPLE_RATE_MSB_Msk                                       (0xe00000UL)		/*!< ADC CONF: SAMPLE_RATE_MSB (Bitfield-Mask: 0x07) */
#define ADC_CONF_SAMPLE_RATE_MSB                                           ADC_CONF_SAMPLE_RATE_MSB_Msk
#define ADC_CONF_SAMPLE_RATE_MSB_0                                         (0x1U << ADC_CONF_SAMPLE_RATE_MSB_Pos)
#define ADC_CONF_SAMPLE_RATE_MSB_1                                         (0x2U << ADC_CONF_SAMPLE_RATE_MSB_Pos)
#define ADC_CONF_SAMPLE_RATE_MSB_2                                         (0x4U << ADC_CONF_SAMPLE_RATE_MSB_Pos)
#define ADC_CONF_ADC_CONT_1V2_Pos                                          (19UL)		/*!<ADC CONF: ADC_CONT_1V2 (Bit 19) */
#define ADC_CONF_ADC_CONT_1V2_Msk                                          (0x80000UL)		/*!< ADC CONF: ADC_CONT_1V2 (Bitfield-Mask: 0x01) */
#define ADC_CONF_ADC_CONT_1V2                                              ADC_CONF_ADC_CONT_1V2_Msk
#define ADC_CONF_BIT_INVERT_DIFF_Pos                                       (18UL)		/*!<ADC CONF: BIT_INVERT_DIFF (Bit 18) */
#define ADC_CONF_BIT_INVERT_DIFF_Msk                                       (0x40000UL)		/*!< ADC CONF: BIT_INVERT_DIFF (Bitfield-Mask: 0x01) */
#define ADC_CONF_BIT_INVERT_DIFF                                           ADC_CONF_BIT_INVERT_DIFF_Msk
#define ADC_CONF_BIT_INVERT_SN_Pos                                         (17UL)		/*!<ADC CONF: BIT_INVERT_SN (Bit 17) */
#define ADC_CONF_BIT_INVERT_SN_Msk                                         (0x20000UL)		/*!< ADC CONF: BIT_INVERT_SN (Bitfield-Mask: 0x01) */
#define ADC_CONF_BIT_INVERT_SN                                             ADC_CONF_BIT_INVERT_SN_Msk
#define ADC_CONF_OVR_DS_CFG_Pos                                            (15UL)		/*!<ADC CONF: OVR_DS_CFG (Bit 15) */
#define ADC_CONF_OVR_DS_CFG_Msk                                            (0x8000UL)		/*!< ADC CONF: OVR_DS_CFG (Bitfield-Mask: 0x01) */
#define ADC_CONF_OVR_DS_CFG                                                ADC_CONF_OVR_DS_CFG_Msk
#define ADC_CONF_DMA_DS_ENA_Pos                                            (13UL)		/*!<ADC CONF: DMA_DS_ENA (Bit 13) */
#define ADC_CONF_DMA_DS_ENA_Msk                                            (0x2000UL)		/*!< ADC CONF: DMA_DS_ENA (Bitfield-Mask: 0x01) */
#define ADC_CONF_DMA_DS_ENA                                                ADC_CONF_DMA_DS_ENA_Msk
#define ADC_CONF_SAMPLE_RATE_Pos                                           (11UL)		/*!<ADC CONF: SAMPLE_RATE (Bit 11) */
#define ADC_CONF_SAMPLE_RATE_Msk                                           (0x1800UL)		/*!< ADC CONF: SAMPLE_RATE (Bitfield-Mask: 0x03) */
#define ADC_CONF_SAMPLE_RATE                                               ADC_CONF_SAMPLE_RATE_Msk
#define ADC_CONF_SAMPLE_RATE_0                                             (0x1U << ADC_CONF_SAMPLE_RATE_Pos)
#define ADC_CONF_SAMPLE_RATE_1                                             (0x2U << ADC_CONF_SAMPLE_RATE_Pos)
#define ADC_CONF_SAMPLE_RATE_LSB_Pos                                       (9UL)		/*!<ADC CONF: SAMPLE_RATE_LSB (Bit 9) */
#define ADC_CONF_SAMPLE_RATE_LSB_Msk                                       (0x600UL)		/*!< ADC CONF: SAMPLE_RATE_LSB (Bitfield-Mask: 0x03) */
#define ADC_CONF_SAMPLE_RATE_LSB                                           ADC_CONF_SAMPLE_RATE_LSB_Msk
#define ADC_CONF_SAMPLE_RATE_LSB_0                                         (0x1U << ADC_CONF_SAMPLE_RATE_LSB_Pos)
#define ADC_CONF_SAMPLE_RATE_LSB_1                                         (0x2U << ADC_CONF_SAMPLE_RATE_LSB_Pos)
#define ADC_CONF_SMPS_SYNCHRO_ENA_Pos                                      (6UL)		/*!<ADC CONF: SMPS_SYNCHRO_ENA (Bit 6) */
#define ADC_CONF_SMPS_SYNCHRO_ENA_Msk                                      (0x40UL)		/*!< ADC CONF: SMPS_SYNCHRO_ENA (Bitfield-Mask: 0x01) */
#define ADC_CONF_SMPS_SYNCHRO_ENA                                          ADC_CONF_SMPS_SYNCHRO_ENA_Msk
#define ADC_CONF_SEQ_LEN_Pos                                               (2UL)		/*!<ADC CONF: SEQ_LEN (Bit 2) */
#define ADC_CONF_SEQ_LEN_Msk                                               (0x3cUL)		/*!< ADC CONF: SEQ_LEN (Bitfield-Mask: 0x0f) */
#define ADC_CONF_SEQ_LEN                                                   ADC_CONF_SEQ_LEN_Msk
#define ADC_CONF_SEQ_LEN_0                                                 (0x1U << ADC_CONF_SEQ_LEN_Pos)
#define ADC_CONF_SEQ_LEN_1                                                 (0x2U << ADC_CONF_SEQ_LEN_Pos)
#define ADC_CONF_SEQ_LEN_2                                                 (0x4U << ADC_CONF_SEQ_LEN_Pos)
#define ADC_CONF_SEQ_LEN_3                                                 (0x8U << ADC_CONF_SEQ_LEN_Pos)
#define ADC_CONF_SEQUENCE_Pos                                              (1UL)		/*!<ADC CONF: SEQUENCE (Bit 1) */
#define ADC_CONF_SEQUENCE_Msk                                              (0x2UL)		/*!< ADC CONF: SEQUENCE (Bitfield-Mask: 0x01) */
#define ADC_CONF_SEQUENCE                                                  ADC_CONF_SEQUENCE_Msk
#define ADC_CONF_CONT_Pos                                                  (0UL)		/*!<ADC CONF: CONT (Bit 0) */
#define ADC_CONF_CONT_Msk                                                  (0x1UL)		/*!< ADC CONF: CONT (Bitfield-Mask: 0x01) */
#define ADC_CONF_CONT                                                      ADC_CONF_CONT_Msk

/* =====================================================    CTRL    =====================================================*/ 
#define ADC_CTRL_STOP_OP_MODE_Pos                                          (2UL)		/*!<ADC CTRL: STOP_OP_MODE (Bit 2) */
#define ADC_CTRL_STOP_OP_MODE_Msk                                          (0x4UL)		/*!< ADC CTRL: STOP_OP_MODE (Bitfield-Mask: 0x01) */
#define ADC_CTRL_STOP_OP_MODE                                              ADC_CTRL_STOP_OP_MODE_Msk
#define ADC_CTRL_START_CONV_Pos                                            (1UL)		/*!<ADC CTRL: START_CONV (Bit 1) */
#define ADC_CTRL_START_CONV_Msk                                            (0x2UL)		/*!< ADC CTRL: START_CONV (Bitfield-Mask: 0x01) */
#define ADC_CTRL_START_CONV                                                ADC_CTRL_START_CONV_Msk
#define ADC_CTRL_ADC_ON_OFF_Pos                                            (0UL)		/*!<ADC CTRL: ADC_ON_OFF (Bit 0) */
#define ADC_CTRL_ADC_ON_OFF_Msk                                            (0x1UL)		/*!< ADC CTRL: ADC_ON_OFF (Bitfield-Mask: 0x01) */
#define ADC_CTRL_ADC_ON_OFF                                                ADC_CTRL_ADC_ON_OFF_Msk

/* =====================================================    SWITCH    =====================================================*/ 
#define ADC_SWITCH_SE_VIN_7_Pos                                            (14UL)		/*!<ADC SWITCH: SE_VIN_7 (Bit 14) */
#define ADC_SWITCH_SE_VIN_7_Msk                                            (0xc000UL)		/*!< ADC SWITCH: SE_VIN_7 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_7                                                ADC_SWITCH_SE_VIN_7_Msk
#define ADC_SWITCH_SE_VIN_7_0                                              (0x1U << ADC_SWITCH_SE_VIN_7_Pos)
#define ADC_SWITCH_SE_VIN_7_1                                              (0x2U << ADC_SWITCH_SE_VIN_7_Pos)
#define ADC_SWITCH_SE_VIN_6_Pos                                            (12UL)		/*!<ADC SWITCH: SE_VIN_6 (Bit 12) */
#define ADC_SWITCH_SE_VIN_6_Msk                                            (0x3000UL)		/*!< ADC SWITCH: SE_VIN_6 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_6                                                ADC_SWITCH_SE_VIN_6_Msk
#define ADC_SWITCH_SE_VIN_6_0                                              (0x1U << ADC_SWITCH_SE_VIN_6_Pos)
#define ADC_SWITCH_SE_VIN_6_1                                              (0x2U << ADC_SWITCH_SE_VIN_6_Pos)
#define ADC_SWITCH_SE_VIN_5_Pos                                            (10UL)		/*!<ADC SWITCH: SE_VIN_5 (Bit 10) */
#define ADC_SWITCH_SE_VIN_5_Msk                                            (0xc00UL)		/*!< ADC SWITCH: SE_VIN_5 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_5                                                ADC_SWITCH_SE_VIN_5_Msk
#define ADC_SWITCH_SE_VIN_5_0                                              (0x1U << ADC_SWITCH_SE_VIN_5_Pos)
#define ADC_SWITCH_SE_VIN_5_1                                              (0x2U << ADC_SWITCH_SE_VIN_5_Pos)
#define ADC_SWITCH_SE_VIN_4_Pos                                            (8UL)		/*!<ADC SWITCH: SE_VIN_4 (Bit 8) */
#define ADC_SWITCH_SE_VIN_4_Msk                                            (0x300UL)		/*!< ADC SWITCH: SE_VIN_4 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_4                                                ADC_SWITCH_SE_VIN_4_Msk
#define ADC_SWITCH_SE_VIN_4_0                                              (0x1U << ADC_SWITCH_SE_VIN_4_Pos)
#define ADC_SWITCH_SE_VIN_4_1                                              (0x2U << ADC_SWITCH_SE_VIN_4_Pos)
#define ADC_SWITCH_SE_VIN_3_Pos                                            (6UL)		/*!<ADC SWITCH: SE_VIN_3 (Bit 6) */
#define ADC_SWITCH_SE_VIN_3_Msk                                            (0xc0UL)		/*!< ADC SWITCH: SE_VIN_3 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_3                                                ADC_SWITCH_SE_VIN_3_Msk
#define ADC_SWITCH_SE_VIN_3_0                                              (0x1U << ADC_SWITCH_SE_VIN_3_Pos)
#define ADC_SWITCH_SE_VIN_3_1                                              (0x2U << ADC_SWITCH_SE_VIN_3_Pos)
#define ADC_SWITCH_SE_VIN_2_Pos                                            (4UL)		/*!<ADC SWITCH: SE_VIN_2 (Bit 4) */
#define ADC_SWITCH_SE_VIN_2_Msk                                            (0x30UL)		/*!< ADC SWITCH: SE_VIN_2 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_2                                                ADC_SWITCH_SE_VIN_2_Msk
#define ADC_SWITCH_SE_VIN_2_0                                              (0x1U << ADC_SWITCH_SE_VIN_2_Pos)
#define ADC_SWITCH_SE_VIN_2_1                                              (0x2U << ADC_SWITCH_SE_VIN_2_Pos)
#define ADC_SWITCH_SE_VIN_1_Pos                                            (2UL)		/*!<ADC SWITCH: SE_VIN_1 (Bit 2) */
#define ADC_SWITCH_SE_VIN_1_Msk                                            (0xcUL)		/*!< ADC SWITCH: SE_VIN_1 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_1                                                ADC_SWITCH_SE_VIN_1_Msk
#define ADC_SWITCH_SE_VIN_1_0                                              (0x1U << ADC_SWITCH_SE_VIN_1_Pos)
#define ADC_SWITCH_SE_VIN_1_1                                              (0x2U << ADC_SWITCH_SE_VIN_1_Pos)
#define ADC_SWITCH_SE_VIN_0_Pos                                            (0UL)		/*!<ADC SWITCH: SE_VIN_0 (Bit 0) */
#define ADC_SWITCH_SE_VIN_0_Msk                                            (0x3UL)		/*!< ADC SWITCH: SE_VIN_0 (Bitfield-Mask: 0x03) */
#define ADC_SWITCH_SE_VIN_0                                                ADC_SWITCH_SE_VIN_0_Msk
#define ADC_SWITCH_SE_VIN_0_0                                              (0x1U << ADC_SWITCH_SE_VIN_0_Pos)
#define ADC_SWITCH_SE_VIN_0_1                                              (0x2U << ADC_SWITCH_SE_VIN_0_Pos)

/* =====================================================    DS_CONF    =====================================================*/ 
#define ADC_DS_CONF_DS_WIDTH_Pos                                           (3UL)		/*!<ADC DS_CONF: DS_WIDTH (Bit 3) */
#define ADC_DS_CONF_DS_WIDTH_Msk                                           (0x38UL)		/*!< ADC DS_CONF: DS_WIDTH (Bitfield-Mask: 0x07) */
#define ADC_DS_CONF_DS_WIDTH                                               ADC_DS_CONF_DS_WIDTH_Msk
#define ADC_DS_CONF_DS_WIDTH_0                                             (0x1U << ADC_DS_CONF_DS_WIDTH_Pos)
#define ADC_DS_CONF_DS_WIDTH_1                                             (0x2U << ADC_DS_CONF_DS_WIDTH_Pos)
#define ADC_DS_CONF_DS_WIDTH_2                                             (0x4U << ADC_DS_CONF_DS_WIDTH_Pos)
#define ADC_DS_CONF_DS_RATIO_Pos                                           (0UL)		/*!<ADC DS_CONF: DS_RATIO (Bit 0) */
#define ADC_DS_CONF_DS_RATIO_Msk                                           (0x7UL)		/*!< ADC DS_CONF: DS_RATIO (Bitfield-Mask: 0x07) */
#define ADC_DS_CONF_DS_RATIO                                               ADC_DS_CONF_DS_RATIO_Msk
#define ADC_DS_CONF_DS_RATIO_0                                             (0x1U << ADC_DS_CONF_DS_RATIO_Pos)
#define ADC_DS_CONF_DS_RATIO_1                                             (0x2U << ADC_DS_CONF_DS_RATIO_Pos)
#define ADC_DS_CONF_DS_RATIO_2                                             (0x4U << ADC_DS_CONF_DS_RATIO_Pos)

/* =====================================================    SEQ_1    =====================================================*/ 
#define ADC_SEQ_1_SEQ7_Pos                                                 (28UL)		/*!<ADC SEQ_1: SEQ7 (Bit 28) */
#define ADC_SEQ_1_SEQ7_Msk                                                 (0xf0000000UL)		/*!< ADC SEQ_1: SEQ7 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ7                                                     ADC_SEQ_1_SEQ7_Msk
#define ADC_SEQ_1_SEQ7_0                                                   (0x1U << ADC_SEQ_1_SEQ7_Pos)
#define ADC_SEQ_1_SEQ7_1                                                   (0x2U << ADC_SEQ_1_SEQ7_Pos)
#define ADC_SEQ_1_SEQ7_2                                                   (0x4U << ADC_SEQ_1_SEQ7_Pos)
#define ADC_SEQ_1_SEQ7_3                                                   (0x8U << ADC_SEQ_1_SEQ7_Pos)
#define ADC_SEQ_1_SEQ6_Pos                                                 (24UL)		/*!<ADC SEQ_1: SEQ6 (Bit 24) */
#define ADC_SEQ_1_SEQ6_Msk                                                 (0xf000000UL)		/*!< ADC SEQ_1: SEQ6 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ6                                                     ADC_SEQ_1_SEQ6_Msk
#define ADC_SEQ_1_SEQ6_0                                                   (0x1U << ADC_SEQ_1_SEQ6_Pos)
#define ADC_SEQ_1_SEQ6_1                                                   (0x2U << ADC_SEQ_1_SEQ6_Pos)
#define ADC_SEQ_1_SEQ6_2                                                   (0x4U << ADC_SEQ_1_SEQ6_Pos)
#define ADC_SEQ_1_SEQ6_3                                                   (0x8U << ADC_SEQ_1_SEQ6_Pos)
#define ADC_SEQ_1_SEQ5_Pos                                                 (20UL)		/*!<ADC SEQ_1: SEQ5 (Bit 20) */
#define ADC_SEQ_1_SEQ5_Msk                                                 (0xf00000UL)		/*!< ADC SEQ_1: SEQ5 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ5                                                     ADC_SEQ_1_SEQ5_Msk
#define ADC_SEQ_1_SEQ5_0                                                   (0x1U << ADC_SEQ_1_SEQ5_Pos)
#define ADC_SEQ_1_SEQ5_1                                                   (0x2U << ADC_SEQ_1_SEQ5_Pos)
#define ADC_SEQ_1_SEQ5_2                                                   (0x4U << ADC_SEQ_1_SEQ5_Pos)
#define ADC_SEQ_1_SEQ5_3                                                   (0x8U << ADC_SEQ_1_SEQ5_Pos)
#define ADC_SEQ_1_SEQ4_Pos                                                 (16UL)		/*!<ADC SEQ_1: SEQ4 (Bit 16) */
#define ADC_SEQ_1_SEQ4_Msk                                                 (0xf0000UL)		/*!< ADC SEQ_1: SEQ4 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ4                                                     ADC_SEQ_1_SEQ4_Msk
#define ADC_SEQ_1_SEQ4_0                                                   (0x1U << ADC_SEQ_1_SEQ4_Pos)
#define ADC_SEQ_1_SEQ4_1                                                   (0x2U << ADC_SEQ_1_SEQ4_Pos)
#define ADC_SEQ_1_SEQ4_2                                                   (0x4U << ADC_SEQ_1_SEQ4_Pos)
#define ADC_SEQ_1_SEQ4_3                                                   (0x8U << ADC_SEQ_1_SEQ4_Pos)
#define ADC_SEQ_1_SEQ3_Pos                                                 (12UL)		/*!<ADC SEQ_1: SEQ3 (Bit 12) */
#define ADC_SEQ_1_SEQ3_Msk                                                 (0xf000UL)		/*!< ADC SEQ_1: SEQ3 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ3                                                     ADC_SEQ_1_SEQ3_Msk
#define ADC_SEQ_1_SEQ3_0                                                   (0x1U << ADC_SEQ_1_SEQ3_Pos)
#define ADC_SEQ_1_SEQ3_1                                                   (0x2U << ADC_SEQ_1_SEQ3_Pos)
#define ADC_SEQ_1_SEQ3_2                                                   (0x4U << ADC_SEQ_1_SEQ3_Pos)
#define ADC_SEQ_1_SEQ3_3                                                   (0x8U << ADC_SEQ_1_SEQ3_Pos)
#define ADC_SEQ_1_SEQ2_Pos                                                 (8UL)		/*!<ADC SEQ_1: SEQ2 (Bit 8) */
#define ADC_SEQ_1_SEQ2_Msk                                                 (0xf00UL)		/*!< ADC SEQ_1: SEQ2 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ2                                                     ADC_SEQ_1_SEQ2_Msk
#define ADC_SEQ_1_SEQ2_0                                                   (0x1U << ADC_SEQ_1_SEQ2_Pos)
#define ADC_SEQ_1_SEQ2_1                                                   (0x2U << ADC_SEQ_1_SEQ2_Pos)
#define ADC_SEQ_1_SEQ2_2                                                   (0x4U << ADC_SEQ_1_SEQ2_Pos)
#define ADC_SEQ_1_SEQ2_3                                                   (0x8U << ADC_SEQ_1_SEQ2_Pos)
#define ADC_SEQ_1_SEQ1_Pos                                                 (4UL)		/*!<ADC SEQ_1: SEQ1 (Bit 4) */
#define ADC_SEQ_1_SEQ1_Msk                                                 (0xf0UL)		/*!< ADC SEQ_1: SEQ1 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ1                                                     ADC_SEQ_1_SEQ1_Msk
#define ADC_SEQ_1_SEQ1_0                                                   (0x1U << ADC_SEQ_1_SEQ1_Pos)
#define ADC_SEQ_1_SEQ1_1                                                   (0x2U << ADC_SEQ_1_SEQ1_Pos)
#define ADC_SEQ_1_SEQ1_2                                                   (0x4U << ADC_SEQ_1_SEQ1_Pos)
#define ADC_SEQ_1_SEQ1_3                                                   (0x8U << ADC_SEQ_1_SEQ1_Pos)
#define ADC_SEQ_1_SEQ0_Pos                                                 (0UL)		/*!<ADC SEQ_1: SEQ0 (Bit 0) */
#define ADC_SEQ_1_SEQ0_Msk                                                 (0xfUL)		/*!< ADC SEQ_1: SEQ0 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_1_SEQ0                                                     ADC_SEQ_1_SEQ0_Msk
#define ADC_SEQ_1_SEQ0_0                                                   (0x1U << ADC_SEQ_1_SEQ0_Pos)
#define ADC_SEQ_1_SEQ0_1                                                   (0x2U << ADC_SEQ_1_SEQ0_Pos)
#define ADC_SEQ_1_SEQ0_2                                                   (0x4U << ADC_SEQ_1_SEQ0_Pos)
#define ADC_SEQ_1_SEQ0_3                                                   (0x8U << ADC_SEQ_1_SEQ0_Pos)

/* =====================================================    SEQ_2    =====================================================*/ 
#define ADC_SEQ_2_SEQ15_Pos                                                (28UL)		/*!<ADC SEQ_2: SEQ15 (Bit 28) */
#define ADC_SEQ_2_SEQ15_Msk                                                (0xf0000000UL)		/*!< ADC SEQ_2: SEQ15 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ15                                                    ADC_SEQ_2_SEQ15_Msk
#define ADC_SEQ_2_SEQ15_0                                                  (0x1U << ADC_SEQ_2_SEQ15_Pos)
#define ADC_SEQ_2_SEQ15_1                                                  (0x2U << ADC_SEQ_2_SEQ15_Pos)
#define ADC_SEQ_2_SEQ15_2                                                  (0x4U << ADC_SEQ_2_SEQ15_Pos)
#define ADC_SEQ_2_SEQ15_3                                                  (0x8U << ADC_SEQ_2_SEQ15_Pos)
#define ADC_SEQ_2_SEQ14_Pos                                                (24UL)		/*!<ADC SEQ_2: SEQ14 (Bit 24) */
#define ADC_SEQ_2_SEQ14_Msk                                                (0xf000000UL)		/*!< ADC SEQ_2: SEQ14 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ14                                                    ADC_SEQ_2_SEQ14_Msk
#define ADC_SEQ_2_SEQ14_0                                                  (0x1U << ADC_SEQ_2_SEQ14_Pos)
#define ADC_SEQ_2_SEQ14_1                                                  (0x2U << ADC_SEQ_2_SEQ14_Pos)
#define ADC_SEQ_2_SEQ14_2                                                  (0x4U << ADC_SEQ_2_SEQ14_Pos)
#define ADC_SEQ_2_SEQ14_3                                                  (0x8U << ADC_SEQ_2_SEQ14_Pos)
#define ADC_SEQ_2_SEQ13_Pos                                                (20UL)		/*!<ADC SEQ_2: SEQ13 (Bit 20) */
#define ADC_SEQ_2_SEQ13_Msk                                                (0xf00000UL)		/*!< ADC SEQ_2: SEQ13 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ13                                                    ADC_SEQ_2_SEQ13_Msk
#define ADC_SEQ_2_SEQ13_0                                                  (0x1U << ADC_SEQ_2_SEQ13_Pos)
#define ADC_SEQ_2_SEQ13_1                                                  (0x2U << ADC_SEQ_2_SEQ13_Pos)
#define ADC_SEQ_2_SEQ13_2                                                  (0x4U << ADC_SEQ_2_SEQ13_Pos)
#define ADC_SEQ_2_SEQ13_3                                                  (0x8U << ADC_SEQ_2_SEQ13_Pos)
#define ADC_SEQ_2_SEQ12_Pos                                                (16UL)		/*!<ADC SEQ_2: SEQ12 (Bit 16) */
#define ADC_SEQ_2_SEQ12_Msk                                                (0xf0000UL)		/*!< ADC SEQ_2: SEQ12 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ12                                                    ADC_SEQ_2_SEQ12_Msk
#define ADC_SEQ_2_SEQ12_0                                                  (0x1U << ADC_SEQ_2_SEQ12_Pos)
#define ADC_SEQ_2_SEQ12_1                                                  (0x2U << ADC_SEQ_2_SEQ12_Pos)
#define ADC_SEQ_2_SEQ12_2                                                  (0x4U << ADC_SEQ_2_SEQ12_Pos)
#define ADC_SEQ_2_SEQ12_3                                                  (0x8U << ADC_SEQ_2_SEQ12_Pos)
#define ADC_SEQ_2_SEQ11_Pos                                                (12UL)		/*!<ADC SEQ_2: SEQ11 (Bit 12) */
#define ADC_SEQ_2_SEQ11_Msk                                                (0xf000UL)		/*!< ADC SEQ_2: SEQ11 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ11                                                    ADC_SEQ_2_SEQ11_Msk
#define ADC_SEQ_2_SEQ11_0                                                  (0x1U << ADC_SEQ_2_SEQ11_Pos)
#define ADC_SEQ_2_SEQ11_1                                                  (0x2U << ADC_SEQ_2_SEQ11_Pos)
#define ADC_SEQ_2_SEQ11_2                                                  (0x4U << ADC_SEQ_2_SEQ11_Pos)
#define ADC_SEQ_2_SEQ11_3                                                  (0x8U << ADC_SEQ_2_SEQ11_Pos)
#define ADC_SEQ_2_SEQ10_Pos                                                (8UL)		/*!<ADC SEQ_2: SEQ10 (Bit 8) */
#define ADC_SEQ_2_SEQ10_Msk                                                (0xf00UL)		/*!< ADC SEQ_2: SEQ10 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ10                                                    ADC_SEQ_2_SEQ10_Msk
#define ADC_SEQ_2_SEQ10_0                                                  (0x1U << ADC_SEQ_2_SEQ10_Pos)
#define ADC_SEQ_2_SEQ10_1                                                  (0x2U << ADC_SEQ_2_SEQ10_Pos)
#define ADC_SEQ_2_SEQ10_2                                                  (0x4U << ADC_SEQ_2_SEQ10_Pos)
#define ADC_SEQ_2_SEQ10_3                                                  (0x8U << ADC_SEQ_2_SEQ10_Pos)
#define ADC_SEQ_2_SEQ9_Pos                                                 (4UL)		/*!<ADC SEQ_2: SEQ9 (Bit 4) */
#define ADC_SEQ_2_SEQ9_Msk                                                 (0xf0UL)		/*!< ADC SEQ_2: SEQ9 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ9                                                     ADC_SEQ_2_SEQ9_Msk
#define ADC_SEQ_2_SEQ9_0                                                   (0x1U << ADC_SEQ_2_SEQ9_Pos)
#define ADC_SEQ_2_SEQ9_1                                                   (0x2U << ADC_SEQ_2_SEQ9_Pos)
#define ADC_SEQ_2_SEQ9_2                                                   (0x4U << ADC_SEQ_2_SEQ9_Pos)
#define ADC_SEQ_2_SEQ9_3                                                   (0x8U << ADC_SEQ_2_SEQ9_Pos)
#define ADC_SEQ_2_SEQ8_Pos                                                 (0UL)		/*!<ADC SEQ_2: SEQ8 (Bit 0) */
#define ADC_SEQ_2_SEQ8_Msk                                                 (0xfUL)		/*!< ADC SEQ_2: SEQ8 (Bitfield-Mask: 0x0f) */
#define ADC_SEQ_2_SEQ8                                                     ADC_SEQ_2_SEQ8_Msk
#define ADC_SEQ_2_SEQ8_0                                                   (0x1U << ADC_SEQ_2_SEQ8_Pos)
#define ADC_SEQ_2_SEQ8_1                                                   (0x2U << ADC_SEQ_2_SEQ8_Pos)
#define ADC_SEQ_2_SEQ8_2                                                   (0x4U << ADC_SEQ_2_SEQ8_Pos)
#define ADC_SEQ_2_SEQ8_3                                                   (0x8U << ADC_SEQ_2_SEQ8_Pos)

/* =====================================================    COMP_1    =====================================================*/ 
#define ADC_COMP_1_OFFSET1_Pos                                             (12UL)		/*!<ADC COMP_1: OFFSET1 (Bit 12) */
#define ADC_COMP_1_OFFSET1_Msk                                             (0xff000UL)		/*!< ADC COMP_1: OFFSET1 (Bitfield-Mask: 0xff) */
#define ADC_COMP_1_OFFSET1                                                 ADC_COMP_1_OFFSET1_Msk
#define ADC_COMP_1_OFFSET1_0                                               (0x1U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_1                                               (0x2U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_2                                               (0x4U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_3                                               (0x8U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_4                                               (0x10U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_5                                               (0x20U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_6                                               (0x40U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_OFFSET1_7                                               (0x80U << ADC_COMP_1_OFFSET1_Pos)
#define ADC_COMP_1_GAIN1_Pos                                               (0UL)		/*!<ADC COMP_1: GAIN1 (Bit 0) */
#define ADC_COMP_1_GAIN1_Msk                                               (0xfffUL)		/*!< ADC COMP_1: GAIN1 (Bitfield-Mask: 0xfff) */
#define ADC_COMP_1_GAIN1                                                   ADC_COMP_1_GAIN1_Msk
#define ADC_COMP_1_GAIN1_0                                                 (0x1U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_1                                                 (0x2U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_2                                                 (0x4U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_3                                                 (0x8U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_4                                                 (0x10U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_5                                                 (0x20U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_6                                                 (0x40U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_7                                                 (0x80U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_8                                                 (0x100U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_9                                                 (0x200U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_10                                                (0x400U << ADC_COMP_1_GAIN1_Pos)
#define ADC_COMP_1_GAIN1_11                                                (0x800U << ADC_COMP_1_GAIN1_Pos)

/* =====================================================    COMP_2    =====================================================*/ 
#define ADC_COMP_2_OFFSET2_Pos                                             (12UL)		/*!<ADC COMP_2: OFFSET2 (Bit 12) */
#define ADC_COMP_2_OFFSET2_Msk                                             (0xff000UL)		/*!< ADC COMP_2: OFFSET2 (Bitfield-Mask: 0xff) */
#define ADC_COMP_2_OFFSET2                                                 ADC_COMP_2_OFFSET2_Msk
#define ADC_COMP_2_OFFSET2_0                                               (0x1U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_1                                               (0x2U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_2                                               (0x4U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_3                                               (0x8U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_4                                               (0x10U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_5                                               (0x20U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_6                                               (0x40U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_OFFSET2_7                                               (0x80U << ADC_COMP_2_OFFSET2_Pos)
#define ADC_COMP_2_GAIN2_Pos                                               (0UL)		/*!<ADC COMP_2: GAIN2 (Bit 0) */
#define ADC_COMP_2_GAIN2_Msk                                               (0xfffUL)		/*!< ADC COMP_2: GAIN2 (Bitfield-Mask: 0xfff) */
#define ADC_COMP_2_GAIN2                                                   ADC_COMP_2_GAIN2_Msk
#define ADC_COMP_2_GAIN2_0                                                 (0x1U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_1                                                 (0x2U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_2                                                 (0x4U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_3                                                 (0x8U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_4                                                 (0x10U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_5                                                 (0x20U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_6                                                 (0x40U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_7                                                 (0x80U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_8                                                 (0x100U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_9                                                 (0x200U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_10                                                (0x400U << ADC_COMP_2_GAIN2_Pos)
#define ADC_COMP_2_GAIN2_11                                                (0x800U << ADC_COMP_2_GAIN2_Pos)

/* =====================================================    COMP_3    =====================================================*/ 
#define ADC_COMP_3_OFFSET3_Pos                                             (12UL)		/*!<ADC COMP_3: OFFSET3 (Bit 12) */
#define ADC_COMP_3_OFFSET3_Msk                                             (0xff000UL)		/*!< ADC COMP_3: OFFSET3 (Bitfield-Mask: 0xff) */
#define ADC_COMP_3_OFFSET3                                                 ADC_COMP_3_OFFSET3_Msk
#define ADC_COMP_3_OFFSET3_0                                               (0x1U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_1                                               (0x2U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_2                                               (0x4U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_3                                               (0x8U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_4                                               (0x10U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_5                                               (0x20U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_6                                               (0x40U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_OFFSET3_7                                               (0x80U << ADC_COMP_3_OFFSET3_Pos)
#define ADC_COMP_3_GAIN3_Pos                                               (0UL)		/*!<ADC COMP_3: GAIN3 (Bit 0) */
#define ADC_COMP_3_GAIN3_Msk                                               (0xfffUL)		/*!< ADC COMP_3: GAIN3 (Bitfield-Mask: 0xfff) */
#define ADC_COMP_3_GAIN3                                                   ADC_COMP_3_GAIN3_Msk
#define ADC_COMP_3_GAIN3_0                                                 (0x1U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_1                                                 (0x2U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_2                                                 (0x4U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_3                                                 (0x8U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_4                                                 (0x10U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_5                                                 (0x20U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_6                                                 (0x40U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_7                                                 (0x80U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_8                                                 (0x100U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_9                                                 (0x200U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_10                                                (0x400U << ADC_COMP_3_GAIN3_Pos)
#define ADC_COMP_3_GAIN3_11                                                (0x800U << ADC_COMP_3_GAIN3_Pos)

/* =====================================================    COMP_4    =====================================================*/ 
#define ADC_COMP_4_OFFSET4_Pos                                             (12UL)		/*!<ADC COMP_4: OFFSET4 (Bit 12) */
#define ADC_COMP_4_OFFSET4_Msk                                             (0xff000UL)		/*!< ADC COMP_4: OFFSET4 (Bitfield-Mask: 0xff) */
#define ADC_COMP_4_OFFSET4                                                 ADC_COMP_4_OFFSET4_Msk
#define ADC_COMP_4_OFFSET4_0                                               (0x1U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_1                                               (0x2U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_2                                               (0x4U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_3                                               (0x8U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_4                                               (0x10U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_5                                               (0x20U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_6                                               (0x40U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_OFFSET4_7                                               (0x80U << ADC_COMP_4_OFFSET4_Pos)
#define ADC_COMP_4_GAIN4_Pos                                               (0UL)		/*!<ADC COMP_4: GAIN4 (Bit 0) */
#define ADC_COMP_4_GAIN4_Msk                                               (0xfffUL)		/*!< ADC COMP_4: GAIN4 (Bitfield-Mask: 0xfff) */
#define ADC_COMP_4_GAIN4                                                   ADC_COMP_4_GAIN4_Msk
#define ADC_COMP_4_GAIN4_0                                                 (0x1U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_1                                                 (0x2U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_2                                                 (0x4U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_3                                                 (0x8U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_4                                                 (0x10U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_5                                                 (0x20U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_6                                                 (0x40U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_7                                                 (0x80U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_8                                                 (0x100U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_9                                                 (0x200U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_10                                                (0x400U << ADC_COMP_4_GAIN4_Pos)
#define ADC_COMP_4_GAIN4_11                                                (0x800U << ADC_COMP_4_GAIN4_Pos)

/* =====================================================    COMP_SEL    =====================================================*/ 
#define ADC_COMP_SEL_OFFSET_GAIN8_Pos                                      (16UL)		/*!<ADC COMP_SEL: OFFSET_GAIN8 (Bit 16) */
#define ADC_COMP_SEL_OFFSET_GAIN8_Msk                                      (0x30000UL)		/*!< ADC COMP_SEL: OFFSET_GAIN8 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN8                                          ADC_COMP_SEL_OFFSET_GAIN8_Msk
#define ADC_COMP_SEL_OFFSET_GAIN8_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN8_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN8_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN8_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN7_Pos                                      (14UL)		/*!<ADC COMP_SEL: OFFSET_GAIN7 (Bit 14) */
#define ADC_COMP_SEL_OFFSET_GAIN7_Msk                                      (0xc000UL)		/*!< ADC COMP_SEL: OFFSET_GAIN7 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN7                                          ADC_COMP_SEL_OFFSET_GAIN7_Msk
#define ADC_COMP_SEL_OFFSET_GAIN7_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN7_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN7_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN7_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN6_Pos                                      (12UL)		/*!<ADC COMP_SEL: OFFSET_GAIN6 (Bit 12) */
#define ADC_COMP_SEL_OFFSET_GAIN6_Msk                                      (0x3000UL)		/*!< ADC COMP_SEL: OFFSET_GAIN6 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN6                                          ADC_COMP_SEL_OFFSET_GAIN6_Msk
#define ADC_COMP_SEL_OFFSET_GAIN6_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN6_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN6_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN6_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN5_Pos                                      (10UL)		/*!<ADC COMP_SEL: OFFSET_GAIN5 (Bit 10) */
#define ADC_COMP_SEL_OFFSET_GAIN5_Msk                                      (0xc00UL)		/*!< ADC COMP_SEL: OFFSET_GAIN5 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN5                                          ADC_COMP_SEL_OFFSET_GAIN5_Msk
#define ADC_COMP_SEL_OFFSET_GAIN5_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN5_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN5_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN5_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN4_Pos                                      (8UL)		/*!<ADC COMP_SEL: OFFSET_GAIN4 (Bit 8) */
#define ADC_COMP_SEL_OFFSET_GAIN4_Msk                                      (0x300UL)		/*!< ADC COMP_SEL: OFFSET_GAIN4 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN4                                          ADC_COMP_SEL_OFFSET_GAIN4_Msk
#define ADC_COMP_SEL_OFFSET_GAIN4_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN4_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN4_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN4_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN3_Pos                                      (6UL)		/*!<ADC COMP_SEL: OFFSET_GAIN3 (Bit 6) */
#define ADC_COMP_SEL_OFFSET_GAIN3_Msk                                      (0xc0UL)		/*!< ADC COMP_SEL: OFFSET_GAIN3 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN3                                          ADC_COMP_SEL_OFFSET_GAIN3_Msk
#define ADC_COMP_SEL_OFFSET_GAIN3_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN3_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN3_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN3_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN2_Pos                                      (4UL)		/*!<ADC COMP_SEL: OFFSET_GAIN2 (Bit 4) */
#define ADC_COMP_SEL_OFFSET_GAIN2_Msk                                      (0x30UL)		/*!< ADC COMP_SEL: OFFSET_GAIN2 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN2                                          ADC_COMP_SEL_OFFSET_GAIN2_Msk
#define ADC_COMP_SEL_OFFSET_GAIN2_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN2_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN2_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN2_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN1_Pos                                      (2UL)		/*!<ADC COMP_SEL: OFFSET_GAIN1 (Bit 2) */
#define ADC_COMP_SEL_OFFSET_GAIN1_Msk                                      (0xcUL)		/*!< ADC COMP_SEL: OFFSET_GAIN1 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN1                                          ADC_COMP_SEL_OFFSET_GAIN1_Msk
#define ADC_COMP_SEL_OFFSET_GAIN1_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN1_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN1_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN1_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN0_Pos                                      (0UL)		/*!<ADC COMP_SEL: OFFSET_GAIN0 (Bit 0) */
#define ADC_COMP_SEL_OFFSET_GAIN0_Msk                                      (0x3UL)		/*!< ADC COMP_SEL: OFFSET_GAIN0 (Bitfield-Mask: 0x03) */
#define ADC_COMP_SEL_OFFSET_GAIN0                                          ADC_COMP_SEL_OFFSET_GAIN0_Msk
#define ADC_COMP_SEL_OFFSET_GAIN0_0                                        (0x1U << ADC_COMP_SEL_OFFSET_GAIN0_Pos)
#define ADC_COMP_SEL_OFFSET_GAIN0_1                                        (0x2U << ADC_COMP_SEL_OFFSET_GAIN0_Pos)

/* =====================================================    WD_TH    =====================================================*/ 
#define ADC_WD_TH_WD_HT_Pos                                                (16UL)		/*!<ADC WD_TH: WD_HT (Bit 16) */
#define ADC_WD_TH_WD_HT_Msk                                                (0xfff0000UL)		/*!< ADC WD_TH: WD_HT (Bitfield-Mask: 0xfff) */
#define ADC_WD_TH_WD_HT                                                    ADC_WD_TH_WD_HT_Msk
#define ADC_WD_TH_WD_HT_0                                                  (0x1U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_1                                                  (0x2U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_2                                                  (0x4U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_3                                                  (0x8U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_4                                                  (0x10U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_5                                                  (0x20U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_6                                                  (0x40U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_7                                                  (0x80U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_8                                                  (0x100U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_9                                                  (0x200U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_10                                                 (0x400U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_HT_11                                                 (0x800U << ADC_WD_TH_WD_HT_Pos)
#define ADC_WD_TH_WD_LT_Pos                                                (0UL)		/*!<ADC WD_TH: WD_LT (Bit 0) */
#define ADC_WD_TH_WD_LT_Msk                                                (0xfffUL)		/*!< ADC WD_TH: WD_LT (Bitfield-Mask: 0xfff) */
#define ADC_WD_TH_WD_LT                                                    ADC_WD_TH_WD_LT_Msk
#define ADC_WD_TH_WD_LT_0                                                  (0x1U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_1                                                  (0x2U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_2                                                  (0x4U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_3                                                  (0x8U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_4                                                  (0x10U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_5                                                  (0x20U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_6                                                  (0x40U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_7                                                  (0x80U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_8                                                  (0x100U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_9                                                  (0x200U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_10                                                 (0x400U << ADC_WD_TH_WD_LT_Pos)
#define ADC_WD_TH_WD_LT_11                                                 (0x800U << ADC_WD_TH_WD_LT_Pos)

/* =====================================================    WD_CONF    =====================================================*/ 
#define ADC_WD_CONF_AWD_CHX_Pos                                            (0UL)		/*!<ADC WD_CONF: AWD_CHX (Bit 0) */
#define ADC_WD_CONF_AWD_CHX_Msk                                            (0xffffUL)		/*!< ADC WD_CONF: AWD_CHX (Bitfield-Mask: 0xffff) */
#define ADC_WD_CONF_AWD_CHX                                                ADC_WD_CONF_AWD_CHX_Msk
#define ADC_WD_CONF_AWD_CHX_0                                              (0x1U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_1                                              (0x2U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_2                                              (0x4U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_3                                              (0x8U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_4                                              (0x10U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_5                                              (0x20U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_6                                              (0x40U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_7                                              (0x80U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_8                                              (0x100U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_9                                              (0x200U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_10                                             (0x400U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_11                                             (0x800U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_12                                             (0x1000U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_13                                             (0x2000U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_14                                             (0x4000U << ADC_WD_CONF_AWD_CHX_Pos)
#define ADC_WD_CONF_AWD_CHX_15                                             (0x8000U << ADC_WD_CONF_AWD_CHX_Pos)

/* =====================================================    DS_DATAOUT    =====================================================*/ 
#define ADC_DS_DATAOUT_DS_DATA_Pos                                         (0UL)		/*!<ADC DS_DATAOUT: DS_DATA (Bit 0) */
#define ADC_DS_DATAOUT_DS_DATA_Msk                                         (0xffffUL)		/*!< ADC DS_DATAOUT: DS_DATA (Bitfield-Mask: 0xffff) */
#define ADC_DS_DATAOUT_DS_DATA                                             ADC_DS_DATAOUT_DS_DATA_Msk
#define ADC_DS_DATAOUT_DS_DATA_0                                           (0x1U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_1                                           (0x2U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_2                                           (0x4U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_3                                           (0x8U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_4                                           (0x10U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_5                                           (0x20U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_6                                           (0x40U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_7                                           (0x80U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_8                                           (0x100U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_9                                           (0x200U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_10                                          (0x400U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_11                                          (0x800U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_12                                          (0x1000U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_13                                          (0x2000U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_14                                          (0x4000U << ADC_DS_DATAOUT_DS_DATA_Pos)
#define ADC_DS_DATAOUT_DS_DATA_15                                          (0x8000U << ADC_DS_DATAOUT_DS_DATA_Pos)

/* =====================================================    IRQ_STATUS    =====================================================*/ 
#define ADC_IRQ_STATUS_OVR_DS_IRQ_Pos                                      (5UL)		/*!<ADC IRQ_STATUS: OVR_DS_IRQ (Bit 5) */
#define ADC_IRQ_STATUS_OVR_DS_IRQ_Msk                                      (0x20UL)		/*!< ADC IRQ_STATUS: OVR_DS_IRQ (Bitfield-Mask: 0x01) */
#define ADC_IRQ_STATUS_OVR_DS_IRQ                                          ADC_IRQ_STATUS_OVR_DS_IRQ_Msk
#define ADC_IRQ_STATUS_AWD_IRQ_Pos                                         (4UL)		/*!<ADC IRQ_STATUS: AWD_IRQ (Bit 4) */
#define ADC_IRQ_STATUS_AWD_IRQ_Msk                                         (0x10UL)		/*!< ADC IRQ_STATUS: AWD_IRQ (Bitfield-Mask: 0x01) */
#define ADC_IRQ_STATUS_AWD_IRQ                                             ADC_IRQ_STATUS_AWD_IRQ_Msk
#define ADC_IRQ_STATUS_EOS_IRQ_Pos                                         (3UL)		/*!<ADC IRQ_STATUS: EOS_IRQ (Bit 3) */
#define ADC_IRQ_STATUS_EOS_IRQ_Msk                                         (0x8UL)		/*!< ADC IRQ_STATUS: EOS_IRQ (Bitfield-Mask: 0x01) */
#define ADC_IRQ_STATUS_EOS_IRQ                                             ADC_IRQ_STATUS_EOS_IRQ_Msk
#define ADC_IRQ_STATUS_EODS_IRQ_Pos                                        (1UL)		/*!<ADC IRQ_STATUS: EODS_IRQ (Bit 1) */
#define ADC_IRQ_STATUS_EODS_IRQ_Msk                                        (0x2UL)		/*!< ADC IRQ_STATUS: EODS_IRQ (Bitfield-Mask: 0x01) */
#define ADC_IRQ_STATUS_EODS_IRQ                                            ADC_IRQ_STATUS_EODS_IRQ_Msk
#define ADC_IRQ_STATUS_EOC_IRQ_Pos                                         (0UL)		/*!<ADC IRQ_STATUS: EOC_IRQ (Bit 0) */
#define ADC_IRQ_STATUS_EOC_IRQ_Msk                                         (0x1UL)		/*!< ADC IRQ_STATUS: EOC_IRQ (Bitfield-Mask: 0x01) */
#define ADC_IRQ_STATUS_EOC_IRQ                                             ADC_IRQ_STATUS_EOC_IRQ_Msk

/* =====================================================    IRQ_ENABLE    =====================================================*/ 
#define ADC_IRQ_ENABLE_OVR_DS_IRQ_ENA_Pos                                  (5UL)		/*!<ADC IRQ_ENABLE: OVR_DS_IRQ_ENA (Bit 5) */
#define ADC_IRQ_ENABLE_OVR_DS_IRQ_ENA_Msk                                  (0x20UL)		/*!< ADC IRQ_ENABLE: OVR_DS_IRQ_ENA (Bitfield-Mask: 0x01) */
#define ADC_IRQ_ENABLE_OVR_DS_IRQ_ENA                                      ADC_IRQ_ENABLE_OVR_DS_IRQ_ENA_Msk
#define ADC_IRQ_ENABLE_AWD_IRQ_ENA_Pos                                     (4UL)		/*!<ADC IRQ_ENABLE: AWD_IRQ_ENA (Bit 4) */
#define ADC_IRQ_ENABLE_AWD_IRQ_ENA_Msk                                     (0x10UL)		/*!< ADC IRQ_ENABLE: AWD_IRQ_ENA (Bitfield-Mask: 0x01) */
#define ADC_IRQ_ENABLE_AWD_IRQ_ENA                                         ADC_IRQ_ENABLE_AWD_IRQ_ENA_Msk
#define ADC_IRQ_ENABLE_EOS_IRQ_ENA_Pos                                     (3UL)		/*!<ADC IRQ_ENABLE: EOS_IRQ_ENA (Bit 3) */
#define ADC_IRQ_ENABLE_EOS_IRQ_ENA_Msk                                     (0x8UL)		/*!< ADC IRQ_ENABLE: EOS_IRQ_ENA (Bitfield-Mask: 0x01) */
#define ADC_IRQ_ENABLE_EOS_IRQ_ENA                                         ADC_IRQ_ENABLE_EOS_IRQ_ENA_Msk
#define ADC_IRQ_ENABLE_EODS_IRQ_ENA_Pos                                    (1UL)		/*!<ADC IRQ_ENABLE: EODS_IRQ_ENA (Bit 1) */
#define ADC_IRQ_ENABLE_EODS_IRQ_ENA_Msk                                    (0x2UL)		/*!< ADC IRQ_ENABLE: EODS_IRQ_ENA (Bitfield-Mask: 0x01) */
#define ADC_IRQ_ENABLE_EODS_IRQ_ENA                                        ADC_IRQ_ENABLE_EODS_IRQ_ENA_Msk
#define ADC_IRQ_ENABLE_EOC_IRQ_ENA_Pos                                     (0UL)		/*!<ADC IRQ_ENABLE: EOC_IRQ_ENA (Bit 0) */
#define ADC_IRQ_ENABLE_EOC_IRQ_ENA_Msk                                     (0x1UL)		/*!< ADC IRQ_ENABLE: EOC_IRQ_ENA (Bitfield-Mask: 0x01) */
#define ADC_IRQ_ENABLE_EOC_IRQ_ENA                                         ADC_IRQ_ENABLE_EOC_IRQ_ENA_Msk


/* ============================================================================================================================*/ 
/*=====================                                       GPIO                                       =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    MODER    =====================================================*/ 
#define GPIO_MODER_MODE15_Pos                                              (30UL)		/*!<GPIO MODER: MODE15 (Bit 30) */
#define GPIO_MODER_MODE15_Msk                                              (0xc0000000UL)		/*!< GPIO MODER: MODE15 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE15                                                  GPIO_MODER_MODE15_Msk
#define GPIO_MODER_MODE15_0                                                (0x1U << GPIO_MODER_MODE15_Pos)
#define GPIO_MODER_MODE15_1                                                (0x2U << GPIO_MODER_MODE15_Pos)
#define GPIO_MODER_MODE14_Pos                                              (28UL)		/*!<GPIO MODER: MODE14 (Bit 28) */
#define GPIO_MODER_MODE14_Msk                                              (0x30000000UL)		/*!< GPIO MODER: MODE14 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE14                                                  GPIO_MODER_MODE14_Msk
#define GPIO_MODER_MODE14_0                                                (0x1U << GPIO_MODER_MODE14_Pos)
#define GPIO_MODER_MODE14_1                                                (0x2U << GPIO_MODER_MODE14_Pos)
#define GPIO_MODER_MODE13_Pos                                              (26UL)		/*!<GPIO MODER: MODE13 (Bit 26) */
#define GPIO_MODER_MODE13_Msk                                              (0xc000000UL)		/*!< GPIO MODER: MODE13 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE13                                                  GPIO_MODER_MODE13_Msk
#define GPIO_MODER_MODE13_0                                                (0x1U << GPIO_MODER_MODE13_Pos)
#define GPIO_MODER_MODE13_1                                                (0x2U << GPIO_MODER_MODE13_Pos)
#define GPIO_MODER_MODE12_Pos                                              (24UL)		/*!<GPIO MODER: MODE12 (Bit 24) */
#define GPIO_MODER_MODE12_Msk                                              (0x3000000UL)		/*!< GPIO MODER: MODE12 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE12                                                  GPIO_MODER_MODE12_Msk
#define GPIO_MODER_MODE12_0                                                (0x1U << GPIO_MODER_MODE12_Pos)
#define GPIO_MODER_MODE12_1                                                (0x2U << GPIO_MODER_MODE12_Pos)
#define GPIO_MODER_MODE11_Pos                                              (22UL)		/*!<GPIO MODER: MODE11 (Bit 22) */
#define GPIO_MODER_MODE11_Msk                                              (0xc00000UL)		/*!< GPIO MODER: MODE11 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE11                                                  GPIO_MODER_MODE11_Msk
#define GPIO_MODER_MODE11_0                                                (0x1U << GPIO_MODER_MODE11_Pos)
#define GPIO_MODER_MODE11_1                                                (0x2U << GPIO_MODER_MODE11_Pos)
#define GPIO_MODER_MODE10_Pos                                              (20UL)		/*!<GPIO MODER: MODE10 (Bit 20) */
#define GPIO_MODER_MODE10_Msk                                              (0x300000UL)		/*!< GPIO MODER: MODE10 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE10                                                  GPIO_MODER_MODE10_Msk
#define GPIO_MODER_MODE10_0                                                (0x1U << GPIO_MODER_MODE10_Pos)
#define GPIO_MODER_MODE10_1                                                (0x2U << GPIO_MODER_MODE10_Pos)
#define GPIO_MODER_MODE9_Pos                                               (18UL)		/*!<GPIO MODER: MODE9 (Bit 18) */
#define GPIO_MODER_MODE9_Msk                                               (0xc0000UL)		/*!< GPIO MODER: MODE9 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE9                                                   GPIO_MODER_MODE9_Msk
#define GPIO_MODER_MODE9_0                                                 (0x1U << GPIO_MODER_MODE9_Pos)
#define GPIO_MODER_MODE9_1                                                 (0x2U << GPIO_MODER_MODE9_Pos)
#define GPIO_MODER_MODE8_Pos                                               (16UL)		/*!<GPIO MODER: MODE8 (Bit 16) */
#define GPIO_MODER_MODE8_Msk                                               (0x30000UL)		/*!< GPIO MODER: MODE8 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE8                                                   GPIO_MODER_MODE8_Msk
#define GPIO_MODER_MODE8_0                                                 (0x1U << GPIO_MODER_MODE8_Pos)
#define GPIO_MODER_MODE8_1                                                 (0x2U << GPIO_MODER_MODE8_Pos)
#define GPIO_MODER_MODE7_Pos                                               (14UL)		/*!<GPIO MODER: MODE7 (Bit 14) */
#define GPIO_MODER_MODE7_Msk                                               (0xc000UL)		/*!< GPIO MODER: MODE7 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE7                                                   GPIO_MODER_MODE7_Msk
#define GPIO_MODER_MODE7_0                                                 (0x1U << GPIO_MODER_MODE7_Pos)
#define GPIO_MODER_MODE7_1                                                 (0x2U << GPIO_MODER_MODE7_Pos)
#define GPIO_MODER_MODE6_Pos                                               (12UL)		/*!<GPIO MODER: MODE6 (Bit 12) */
#define GPIO_MODER_MODE6_Msk                                               (0x3000UL)		/*!< GPIO MODER: MODE6 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE6                                                   GPIO_MODER_MODE6_Msk
#define GPIO_MODER_MODE6_0                                                 (0x1U << GPIO_MODER_MODE6_Pos)
#define GPIO_MODER_MODE6_1                                                 (0x2U << GPIO_MODER_MODE6_Pos)
#define GPIO_MODER_MODE5_Pos                                               (10UL)		/*!<GPIO MODER: MODE5 (Bit 10) */
#define GPIO_MODER_MODE5_Msk                                               (0xc00UL)		/*!< GPIO MODER: MODE5 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE5                                                   GPIO_MODER_MODE5_Msk
#define GPIO_MODER_MODE5_0                                                 (0x1U << GPIO_MODER_MODE5_Pos)
#define GPIO_MODER_MODE5_1                                                 (0x2U << GPIO_MODER_MODE5_Pos)
#define GPIO_MODER_MODE4_Pos                                               (8UL)		/*!<GPIO MODER: MODE4 (Bit 8) */
#define GPIO_MODER_MODE4_Msk                                               (0x300UL)		/*!< GPIO MODER: MODE4 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE4                                                   GPIO_MODER_MODE4_Msk
#define GPIO_MODER_MODE4_0                                                 (0x1U << GPIO_MODER_MODE4_Pos)
#define GPIO_MODER_MODE4_1                                                 (0x2U << GPIO_MODER_MODE4_Pos)
#define GPIO_MODER_MODE3_Pos                                               (6UL)		/*!<GPIO MODER: MODE3 (Bit 6) */
#define GPIO_MODER_MODE3_Msk                                               (0xc0UL)		/*!< GPIO MODER: MODE3 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE3                                                   GPIO_MODER_MODE3_Msk
#define GPIO_MODER_MODE3_0                                                 (0x1U << GPIO_MODER_MODE3_Pos)
#define GPIO_MODER_MODE3_1                                                 (0x2U << GPIO_MODER_MODE3_Pos)
#define GPIO_MODER_MODE2_Pos                                               (4UL)		/*!<GPIO MODER: MODE2 (Bit 4) */
#define GPIO_MODER_MODE2_Msk                                               (0x30UL)		/*!< GPIO MODER: MODE2 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE2                                                   GPIO_MODER_MODE2_Msk
#define GPIO_MODER_MODE2_0                                                 (0x1U << GPIO_MODER_MODE2_Pos)
#define GPIO_MODER_MODE2_1                                                 (0x2U << GPIO_MODER_MODE2_Pos)
#define GPIO_MODER_MODE1_Pos                                               (2UL)		/*!<GPIO MODER: MODE1 (Bit 2) */
#define GPIO_MODER_MODE1_Msk                                               (0xcUL)		/*!< GPIO MODER: MODE1 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE1                                                   GPIO_MODER_MODE1_Msk
#define GPIO_MODER_MODE1_0                                                 (0x1U << GPIO_MODER_MODE1_Pos)
#define GPIO_MODER_MODE1_1                                                 (0x2U << GPIO_MODER_MODE1_Pos)
#define GPIO_MODER_MODE0_Pos                                               (0UL)		/*!<GPIO MODER: MODE0 (Bit 0) */
#define GPIO_MODER_MODE0_Msk                                               (0x3UL)		/*!< GPIO MODER: MODE0 (Bitfield-Mask: 0x03) */
#define GPIO_MODER_MODE0                                                   GPIO_MODER_MODE0_Msk
#define GPIO_MODER_MODE0_0                                                 (0x1U << GPIO_MODER_MODE0_Pos)
#define GPIO_MODER_MODE0_1                                                 (0x2U << GPIO_MODER_MODE0_Pos)

/* =====================================================    OTYPER    =====================================================*/ 
#define GPIO_OTYPER_OT15_Pos                                               (15UL)		/*!<GPIO OTYPER: OT15 (Bit 15) */
#define GPIO_OTYPER_OT15_Msk                                               (0x8000UL)		/*!< GPIO OTYPER: OT15 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT15                                                   GPIO_OTYPER_OT15_Msk
#define GPIO_OTYPER_OT14_Pos                                               (14UL)		/*!<GPIO OTYPER: OT14 (Bit 14) */
#define GPIO_OTYPER_OT14_Msk                                               (0x4000UL)		/*!< GPIO OTYPER: OT14 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT14                                                   GPIO_OTYPER_OT14_Msk
#define GPIO_OTYPER_OT13_Pos                                               (13UL)		/*!<GPIO OTYPER: OT13 (Bit 13) */
#define GPIO_OTYPER_OT13_Msk                                               (0x2000UL)		/*!< GPIO OTYPER: OT13 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT13                                                   GPIO_OTYPER_OT13_Msk
#define GPIO_OTYPER_OT12_Pos                                               (12UL)		/*!<GPIO OTYPER: OT12 (Bit 12) */
#define GPIO_OTYPER_OT12_Msk                                               (0x1000UL)		/*!< GPIO OTYPER: OT12 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT12                                                   GPIO_OTYPER_OT12_Msk
#define GPIO_OTYPER_OT11_Pos                                               (11UL)		/*!<GPIO OTYPER: OT11 (Bit 11) */
#define GPIO_OTYPER_OT11_Msk                                               (0x800UL)		/*!< GPIO OTYPER: OT11 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT11                                                   GPIO_OTYPER_OT11_Msk
#define GPIO_OTYPER_OT10_Pos                                               (10UL)		/*!<GPIO OTYPER: OT10 (Bit 10) */
#define GPIO_OTYPER_OT10_Msk                                               (0x400UL)		/*!< GPIO OTYPER: OT10 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT10                                                   GPIO_OTYPER_OT10_Msk
#define GPIO_OTYPER_OT9_Pos                                                (9UL)		/*!<GPIO OTYPER: OT9 (Bit 9) */
#define GPIO_OTYPER_OT9_Msk                                                (0x200UL)		/*!< GPIO OTYPER: OT9 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT9                                                    GPIO_OTYPER_OT9_Msk
#define GPIO_OTYPER_OT8_Pos                                                (8UL)		/*!<GPIO OTYPER: OT8 (Bit 8) */
#define GPIO_OTYPER_OT8_Msk                                                (0x100UL)		/*!< GPIO OTYPER: OT8 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT8                                                    GPIO_OTYPER_OT8_Msk
#define GPIO_OTYPER_OT7_Pos                                                (7UL)		/*!<GPIO OTYPER: OT7 (Bit 7) */
#define GPIO_OTYPER_OT7_Msk                                                (0x80UL)		/*!< GPIO OTYPER: OT7 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT7                                                    GPIO_OTYPER_OT7_Msk
#define GPIO_OTYPER_OT6_Pos                                                (6UL)		/*!<GPIO OTYPER: OT6 (Bit 6) */
#define GPIO_OTYPER_OT6_Msk                                                (0x40UL)		/*!< GPIO OTYPER: OT6 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT6                                                    GPIO_OTYPER_OT6_Msk
#define GPIO_OTYPER_OT5_Pos                                                (5UL)		/*!<GPIO OTYPER: OT5 (Bit 5) */
#define GPIO_OTYPER_OT5_Msk                                                (0x20UL)		/*!< GPIO OTYPER: OT5 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT5                                                    GPIO_OTYPER_OT5_Msk
#define GPIO_OTYPER_OT4_Pos                                                (4UL)		/*!<GPIO OTYPER: OT4 (Bit 4) */
#define GPIO_OTYPER_OT4_Msk                                                (0x10UL)		/*!< GPIO OTYPER: OT4 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT4                                                    GPIO_OTYPER_OT4_Msk
#define GPIO_OTYPER_OT3_Pos                                                (3UL)		/*!<GPIO OTYPER: OT3 (Bit 3) */
#define GPIO_OTYPER_OT3_Msk                                                (0x8UL)		/*!< GPIO OTYPER: OT3 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT3                                                    GPIO_OTYPER_OT3_Msk
#define GPIO_OTYPER_OT2_Pos                                                (2UL)		/*!<GPIO OTYPER: OT2 (Bit 2) */
#define GPIO_OTYPER_OT2_Msk                                                (0x4UL)		/*!< GPIO OTYPER: OT2 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT2                                                    GPIO_OTYPER_OT2_Msk
#define GPIO_OTYPER_OT1_Pos                                                (1UL)		/*!<GPIO OTYPER: OT1 (Bit 1) */
#define GPIO_OTYPER_OT1_Msk                                                (0x2UL)		/*!< GPIO OTYPER: OT1 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT1                                                    GPIO_OTYPER_OT1_Msk
#define GPIO_OTYPER_OT0_Pos                                                (0UL)		/*!<GPIO OTYPER: OT0 (Bit 0) */
#define GPIO_OTYPER_OT0_Msk                                                (0x1UL)		/*!< GPIO OTYPER: OT0 (Bitfield-Mask: 0x01) */
#define GPIO_OTYPER_OT0                                                    GPIO_OTYPER_OT0_Msk

/* =====================================================    OSPEEDR    =====================================================*/ 
#define GPIO_OSPEEDR_OSPEED15_Pos                                          (30UL)		/*!<GPIO OSPEEDR: OSPEED15 (Bit 30) */
#define GPIO_OSPEEDR_OSPEED15_Msk                                          (0xc0000000UL)		/*!< GPIO OSPEEDR: OSPEED15 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED15                                              GPIO_OSPEEDR_OSPEED15_Msk
#define GPIO_OSPEEDR_OSPEED15_0                                            (0x1U << GPIO_OSPEEDR_OSPEED15_Pos)
#define GPIO_OSPEEDR_OSPEED15_1                                            (0x2U << GPIO_OSPEEDR_OSPEED15_Pos)
#define GPIO_OSPEEDR_OSPEED14_Pos                                          (28UL)		/*!<GPIO OSPEEDR: OSPEED14 (Bit 28) */
#define GPIO_OSPEEDR_OSPEED14_Msk                                          (0x30000000UL)		/*!< GPIO OSPEEDR: OSPEED14 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED14                                              GPIO_OSPEEDR_OSPEED14_Msk
#define GPIO_OSPEEDR_OSPEED14_0                                            (0x1U << GPIO_OSPEEDR_OSPEED14_Pos)
#define GPIO_OSPEEDR_OSPEED14_1                                            (0x2U << GPIO_OSPEEDR_OSPEED14_Pos)
#define GPIO_OSPEEDR_OSPEED13_Pos                                          (26UL)		/*!<GPIO OSPEEDR: OSPEED13 (Bit 26) */
#define GPIO_OSPEEDR_OSPEED13_Msk                                          (0xc000000UL)		/*!< GPIO OSPEEDR: OSPEED13 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED13                                              GPIO_OSPEEDR_OSPEED13_Msk
#define GPIO_OSPEEDR_OSPEED13_0                                            (0x1U << GPIO_OSPEEDR_OSPEED13_Pos)
#define GPIO_OSPEEDR_OSPEED13_1                                            (0x2U << GPIO_OSPEEDR_OSPEED13_Pos)
#define GPIO_OSPEEDR_OSPEED12_Pos                                          (24UL)		/*!<GPIO OSPEEDR: OSPEED12 (Bit 24) */
#define GPIO_OSPEEDR_OSPEED12_Msk                                          (0x3000000UL)		/*!< GPIO OSPEEDR: OSPEED12 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED12                                              GPIO_OSPEEDR_OSPEED12_Msk
#define GPIO_OSPEEDR_OSPEED12_0                                            (0x1U << GPIO_OSPEEDR_OSPEED12_Pos)
#define GPIO_OSPEEDR_OSPEED12_1                                            (0x2U << GPIO_OSPEEDR_OSPEED12_Pos)
#define GPIO_OSPEEDR_OSPEED11_Pos                                          (22UL)		/*!<GPIO OSPEEDR: OSPEED11 (Bit 22) */
#define GPIO_OSPEEDR_OSPEED11_Msk                                          (0xc00000UL)		/*!< GPIO OSPEEDR: OSPEED11 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED11                                              GPIO_OSPEEDR_OSPEED11_Msk
#define GPIO_OSPEEDR_OSPEED11_0                                            (0x1U << GPIO_OSPEEDR_OSPEED11_Pos)
#define GPIO_OSPEEDR_OSPEED11_1                                            (0x2U << GPIO_OSPEEDR_OSPEED11_Pos)
#define GPIO_OSPEEDR_OSPEED10_Pos                                          (20UL)		/*!<GPIO OSPEEDR: OSPEED10 (Bit 20) */
#define GPIO_OSPEEDR_OSPEED10_Msk                                          (0x300000UL)		/*!< GPIO OSPEEDR: OSPEED10 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED10                                              GPIO_OSPEEDR_OSPEED10_Msk
#define GPIO_OSPEEDR_OSPEED10_0                                            (0x1U << GPIO_OSPEEDR_OSPEED10_Pos)
#define GPIO_OSPEEDR_OSPEED10_1                                            (0x2U << GPIO_OSPEEDR_OSPEED10_Pos)
#define GPIO_OSPEEDR_OSPEED9_Pos                                           (18UL)		/*!<GPIO OSPEEDR: OSPEED9 (Bit 18) */
#define GPIO_OSPEEDR_OSPEED9_Msk                                           (0xc0000UL)		/*!< GPIO OSPEEDR: OSPEED9 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED9                                               GPIO_OSPEEDR_OSPEED9_Msk
#define GPIO_OSPEEDR_OSPEED9_0                                             (0x1U << GPIO_OSPEEDR_OSPEED9_Pos)
#define GPIO_OSPEEDR_OSPEED9_1                                             (0x2U << GPIO_OSPEEDR_OSPEED9_Pos)
#define GPIO_OSPEEDR_OSPEED8_Pos                                           (16UL)		/*!<GPIO OSPEEDR: OSPEED8 (Bit 16) */
#define GPIO_OSPEEDR_OSPEED8_Msk                                           (0x30000UL)		/*!< GPIO OSPEEDR: OSPEED8 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED8                                               GPIO_OSPEEDR_OSPEED8_Msk
#define GPIO_OSPEEDR_OSPEED8_0                                             (0x1U << GPIO_OSPEEDR_OSPEED8_Pos)
#define GPIO_OSPEEDR_OSPEED8_1                                             (0x2U << GPIO_OSPEEDR_OSPEED8_Pos)
#define GPIO_OSPEEDR_OSPEED7_Pos                                           (14UL)		/*!<GPIO OSPEEDR: OSPEED7 (Bit 14) */
#define GPIO_OSPEEDR_OSPEED7_Msk                                           (0xc000UL)		/*!< GPIO OSPEEDR: OSPEED7 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED7                                               GPIO_OSPEEDR_OSPEED7_Msk
#define GPIO_OSPEEDR_OSPEED7_0                                             (0x1U << GPIO_OSPEEDR_OSPEED7_Pos)
#define GPIO_OSPEEDR_OSPEED7_1                                             (0x2U << GPIO_OSPEEDR_OSPEED7_Pos)
#define GPIO_OSPEEDR_OSPEED6_Pos                                           (12UL)		/*!<GPIO OSPEEDR: OSPEED6 (Bit 12) */
#define GPIO_OSPEEDR_OSPEED6_Msk                                           (0x3000UL)		/*!< GPIO OSPEEDR: OSPEED6 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED6                                               GPIO_OSPEEDR_OSPEED6_Msk
#define GPIO_OSPEEDR_OSPEED6_0                                             (0x1U << GPIO_OSPEEDR_OSPEED6_Pos)
#define GPIO_OSPEEDR_OSPEED6_1                                             (0x2U << GPIO_OSPEEDR_OSPEED6_Pos)
#define GPIO_OSPEEDR_OSPEED5_Pos                                           (10UL)		/*!<GPIO OSPEEDR: OSPEED5 (Bit 10) */
#define GPIO_OSPEEDR_OSPEED5_Msk                                           (0xc00UL)		/*!< GPIO OSPEEDR: OSPEED5 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED5                                               GPIO_OSPEEDR_OSPEED5_Msk
#define GPIO_OSPEEDR_OSPEED5_0                                             (0x1U << GPIO_OSPEEDR_OSPEED5_Pos)
#define GPIO_OSPEEDR_OSPEED5_1                                             (0x2U << GPIO_OSPEEDR_OSPEED5_Pos)
#define GPIO_OSPEEDR_OSPEED4_Pos                                           (8UL)		/*!<GPIO OSPEEDR: OSPEED4 (Bit 8) */
#define GPIO_OSPEEDR_OSPEED4_Msk                                           (0x300UL)		/*!< GPIO OSPEEDR: OSPEED4 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED4                                               GPIO_OSPEEDR_OSPEED4_Msk
#define GPIO_OSPEEDR_OSPEED4_0                                             (0x1U << GPIO_OSPEEDR_OSPEED4_Pos)
#define GPIO_OSPEEDR_OSPEED4_1                                             (0x2U << GPIO_OSPEEDR_OSPEED4_Pos)
#define GPIO_OSPEEDR_OSPEED3_Pos                                           (6UL)		/*!<GPIO OSPEEDR: OSPEED3 (Bit 6) */
#define GPIO_OSPEEDR_OSPEED3_Msk                                           (0xc0UL)		/*!< GPIO OSPEEDR: OSPEED3 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED3                                               GPIO_OSPEEDR_OSPEED3_Msk
#define GPIO_OSPEEDR_OSPEED3_0                                             (0x1U << GPIO_OSPEEDR_OSPEED3_Pos)
#define GPIO_OSPEEDR_OSPEED3_1                                             (0x2U << GPIO_OSPEEDR_OSPEED3_Pos)
#define GPIO_OSPEEDR_OSPEED2_Pos                                           (4UL)		/*!<GPIO OSPEEDR: OSPEED2 (Bit 4) */
#define GPIO_OSPEEDR_OSPEED2_Msk                                           (0x30UL)		/*!< GPIO OSPEEDR: OSPEED2 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED2                                               GPIO_OSPEEDR_OSPEED2_Msk
#define GPIO_OSPEEDR_OSPEED2_0                                             (0x1U << GPIO_OSPEEDR_OSPEED2_Pos)
#define GPIO_OSPEEDR_OSPEED2_1                                             (0x2U << GPIO_OSPEEDR_OSPEED2_Pos)
#define GPIO_OSPEEDR_OSPEED1_Pos                                           (2UL)		/*!<GPIO OSPEEDR: OSPEED1 (Bit 2) */
#define GPIO_OSPEEDR_OSPEED1_Msk                                           (0xcUL)		/*!< GPIO OSPEEDR: OSPEED1 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED1                                               GPIO_OSPEEDR_OSPEED1_Msk
#define GPIO_OSPEEDR_OSPEED1_0                                             (0x1U << GPIO_OSPEEDR_OSPEED1_Pos)
#define GPIO_OSPEEDR_OSPEED1_1                                             (0x2U << GPIO_OSPEEDR_OSPEED1_Pos)
#define GPIO_OSPEEDR_OSPEED0_Pos                                           (0UL)		/*!<GPIO OSPEEDR: OSPEED0 (Bit 0) */
#define GPIO_OSPEEDR_OSPEED0_Msk                                           (0x3UL)		/*!< GPIO OSPEEDR: OSPEED0 (Bitfield-Mask: 0x03) */
#define GPIO_OSPEEDR_OSPEED0                                               GPIO_OSPEEDR_OSPEED0_Msk
#define GPIO_OSPEEDR_OSPEED0_0                                             (0x1U << GPIO_OSPEEDR_OSPEED0_Pos)
#define GPIO_OSPEEDR_OSPEED0_1                                             (0x2U << GPIO_OSPEEDR_OSPEED0_Pos)

/* =====================================================    PUPDR    =====================================================*/ 
#define GPIO_PUPDR_PUPD15_Pos                                              (30UL)		/*!<GPIO PUPDR: PUPD15 (Bit 30) */
#define GPIO_PUPDR_PUPD15_Msk                                              (0xc0000000UL)		/*!< GPIO PUPDR: PUPD15 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD15                                                  GPIO_PUPDR_PUPD15_Msk
#define GPIO_PUPDR_PUPD15_0                                                (0x1U << GPIO_PUPDR_PUPD15_Pos)
#define GPIO_PUPDR_PUPD15_1                                                (0x2U << GPIO_PUPDR_PUPD15_Pos)
#define GPIO_PUPDR_PUPD14_Pos                                              (28UL)		/*!<GPIO PUPDR: PUPD14 (Bit 28) */
#define GPIO_PUPDR_PUPD14_Msk                                              (0x30000000UL)		/*!< GPIO PUPDR: PUPD14 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD14                                                  GPIO_PUPDR_PUPD14_Msk
#define GPIO_PUPDR_PUPD14_0                                                (0x1U << GPIO_PUPDR_PUPD14_Pos)
#define GPIO_PUPDR_PUPD14_1                                                (0x2U << GPIO_PUPDR_PUPD14_Pos)
#define GPIO_PUPDR_PUPD13_Pos                                              (26UL)		/*!<GPIO PUPDR: PUPD13 (Bit 26) */
#define GPIO_PUPDR_PUPD13_Msk                                              (0xc000000UL)		/*!< GPIO PUPDR: PUPD13 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD13                                                  GPIO_PUPDR_PUPD13_Msk
#define GPIO_PUPDR_PUPD13_0                                                (0x1U << GPIO_PUPDR_PUPD13_Pos)
#define GPIO_PUPDR_PUPD13_1                                                (0x2U << GPIO_PUPDR_PUPD13_Pos)
#define GPIO_PUPDR_PUPD12_Pos                                              (24UL)		/*!<GPIO PUPDR: PUPD12 (Bit 24) */
#define GPIO_PUPDR_PUPD12_Msk                                              (0x3000000UL)		/*!< GPIO PUPDR: PUPD12 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD12                                                  GPIO_PUPDR_PUPD12_Msk
#define GPIO_PUPDR_PUPD12_0                                                (0x1U << GPIO_PUPDR_PUPD12_Pos)
#define GPIO_PUPDR_PUPD12_1                                                (0x2U << GPIO_PUPDR_PUPD12_Pos)
#define GPIO_PUPDR_PUPD11_Pos                                              (22UL)		/*!<GPIO PUPDR: PUPD11 (Bit 22) */
#define GPIO_PUPDR_PUPD11_Msk                                              (0xc00000UL)		/*!< GPIO PUPDR: PUPD11 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD11                                                  GPIO_PUPDR_PUPD11_Msk
#define GPIO_PUPDR_PUPD11_0                                                (0x1U << GPIO_PUPDR_PUPD11_Pos)
#define GPIO_PUPDR_PUPD11_1                                                (0x2U << GPIO_PUPDR_PUPD11_Pos)
#define GPIO_PUPDR_PUPD10_Pos                                              (20UL)		/*!<GPIO PUPDR: PUPD10 (Bit 20) */
#define GPIO_PUPDR_PUPD10_Msk                                              (0x300000UL)		/*!< GPIO PUPDR: PUPD10 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD10                                                  GPIO_PUPDR_PUPD10_Msk
#define GPIO_PUPDR_PUPD10_0                                                (0x1U << GPIO_PUPDR_PUPD10_Pos)
#define GPIO_PUPDR_PUPD10_1                                                (0x2U << GPIO_PUPDR_PUPD10_Pos)
#define GPIO_PUPDR_PUPD9_Pos                                               (18UL)		/*!<GPIO PUPDR: PUPD9 (Bit 18) */
#define GPIO_PUPDR_PUPD9_Msk                                               (0xc0000UL)		/*!< GPIO PUPDR: PUPD9 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD9                                                   GPIO_PUPDR_PUPD9_Msk
#define GPIO_PUPDR_PUPD9_0                                                 (0x1U << GPIO_PUPDR_PUPD9_Pos)
#define GPIO_PUPDR_PUPD9_1                                                 (0x2U << GPIO_PUPDR_PUPD9_Pos)
#define GPIO_PUPDR_PUPD8_Pos                                               (16UL)		/*!<GPIO PUPDR: PUPD8 (Bit 16) */
#define GPIO_PUPDR_PUPD8_Msk                                               (0x30000UL)		/*!< GPIO PUPDR: PUPD8 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD8                                                   GPIO_PUPDR_PUPD8_Msk
#define GPIO_PUPDR_PUPD8_0                                                 (0x1U << GPIO_PUPDR_PUPD8_Pos)
#define GPIO_PUPDR_PUPD8_1                                                 (0x2U << GPIO_PUPDR_PUPD8_Pos)
#define GPIO_PUPDR_PUPD7_Pos                                               (14UL)		/*!<GPIO PUPDR: PUPD7 (Bit 14) */
#define GPIO_PUPDR_PUPD7_Msk                                               (0xc000UL)		/*!< GPIO PUPDR: PUPD7 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD7                                                   GPIO_PUPDR_PUPD7_Msk
#define GPIO_PUPDR_PUPD7_0                                                 (0x1U << GPIO_PUPDR_PUPD7_Pos)
#define GPIO_PUPDR_PUPD7_1                                                 (0x2U << GPIO_PUPDR_PUPD7_Pos)
#define GPIO_PUPDR_PUPD6_Pos                                               (12UL)		/*!<GPIO PUPDR: PUPD6 (Bit 12) */
#define GPIO_PUPDR_PUPD6_Msk                                               (0x3000UL)		/*!< GPIO PUPDR: PUPD6 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD6                                                   GPIO_PUPDR_PUPD6_Msk
#define GPIO_PUPDR_PUPD6_0                                                 (0x1U << GPIO_PUPDR_PUPD6_Pos)
#define GPIO_PUPDR_PUPD6_1                                                 (0x2U << GPIO_PUPDR_PUPD6_Pos)
#define GPIO_PUPDR_PUPD5_Pos                                               (10UL)		/*!<GPIO PUPDR: PUPD5 (Bit 10) */
#define GPIO_PUPDR_PUPD5_Msk                                               (0xc00UL)		/*!< GPIO PUPDR: PUPD5 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD5                                                   GPIO_PUPDR_PUPD5_Msk
#define GPIO_PUPDR_PUPD5_0                                                 (0x1U << GPIO_PUPDR_PUPD5_Pos)
#define GPIO_PUPDR_PUPD5_1                                                 (0x2U << GPIO_PUPDR_PUPD5_Pos)
#define GPIO_PUPDR_PUPD4_Pos                                               (8UL)		/*!<GPIO PUPDR: PUPD4 (Bit 8) */
#define GPIO_PUPDR_PUPD4_Msk                                               (0x300UL)		/*!< GPIO PUPDR: PUPD4 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD4                                                   GPIO_PUPDR_PUPD4_Msk
#define GPIO_PUPDR_PUPD4_0                                                 (0x1U << GPIO_PUPDR_PUPD4_Pos)
#define GPIO_PUPDR_PUPD4_1                                                 (0x2U << GPIO_PUPDR_PUPD4_Pos)
#define GPIO_PUPDR_PUPD3_Pos                                               (6UL)		/*!<GPIO PUPDR: PUPD3 (Bit 6) */
#define GPIO_PUPDR_PUPD3_Msk                                               (0xc0UL)		/*!< GPIO PUPDR: PUPD3 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD3                                                   GPIO_PUPDR_PUPD3_Msk
#define GPIO_PUPDR_PUPD3_0                                                 (0x1U << GPIO_PUPDR_PUPD3_Pos)
#define GPIO_PUPDR_PUPD3_1                                                 (0x2U << GPIO_PUPDR_PUPD3_Pos)
#define GPIO_PUPDR_PUPD2_Pos                                               (4UL)		/*!<GPIO PUPDR: PUPD2 (Bit 4) */
#define GPIO_PUPDR_PUPD2_Msk                                               (0x30UL)		/*!< GPIO PUPDR: PUPD2 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD2                                                   GPIO_PUPDR_PUPD2_Msk
#define GPIO_PUPDR_PUPD2_0                                                 (0x1U << GPIO_PUPDR_PUPD2_Pos)
#define GPIO_PUPDR_PUPD2_1                                                 (0x2U << GPIO_PUPDR_PUPD2_Pos)
#define GPIO_PUPDR_PUPD1_Pos                                               (2UL)		/*!<GPIO PUPDR: PUPD1 (Bit 2) */
#define GPIO_PUPDR_PUPD1_Msk                                               (0xcUL)		/*!< GPIO PUPDR: PUPD1 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD1                                                   GPIO_PUPDR_PUPD1_Msk
#define GPIO_PUPDR_PUPD1_0                                                 (0x1U << GPIO_PUPDR_PUPD1_Pos)
#define GPIO_PUPDR_PUPD1_1                                                 (0x2U << GPIO_PUPDR_PUPD1_Pos)
#define GPIO_PUPDR_PUPD0_Pos                                               (0UL)		/*!<GPIO PUPDR: PUPD0 (Bit 0) */
#define GPIO_PUPDR_PUPD0_Msk                                               (0x3UL)		/*!< GPIO PUPDR: PUPD0 (Bitfield-Mask: 0x03) */
#define GPIO_PUPDR_PUPD0                                                   GPIO_PUPDR_PUPD0_Msk
#define GPIO_PUPDR_PUPD0_0                                                 (0x1U << GPIO_PUPDR_PUPD0_Pos)
#define GPIO_PUPDR_PUPD0_1                                                 (0x2U << GPIO_PUPDR_PUPD0_Pos)

/* =====================================================    IDR    =====================================================*/ 
#define GPIO_IDR_ID15_Pos                                                  (15UL)		/*!<GPIO IDR: ID15 (Bit 15) */
#define GPIO_IDR_ID15_Msk                                                  (0x8000UL)		/*!< GPIO IDR: ID15 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID15                                                      GPIO_IDR_ID15_Msk
#define GPIO_IDR_ID14_Pos                                                  (14UL)		/*!<GPIO IDR: ID14 (Bit 14) */
#define GPIO_IDR_ID14_Msk                                                  (0x4000UL)		/*!< GPIO IDR: ID14 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID14                                                      GPIO_IDR_ID14_Msk
#define GPIO_IDR_ID13_Pos                                                  (13UL)		/*!<GPIO IDR: ID13 (Bit 13) */
#define GPIO_IDR_ID13_Msk                                                  (0x2000UL)		/*!< GPIO IDR: ID13 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID13                                                      GPIO_IDR_ID13_Msk
#define GPIO_IDR_ID12_Pos                                                  (12UL)		/*!<GPIO IDR: ID12 (Bit 12) */
#define GPIO_IDR_ID12_Msk                                                  (0x1000UL)		/*!< GPIO IDR: ID12 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID12                                                      GPIO_IDR_ID12_Msk
#define GPIO_IDR_ID11_Pos                                                  (11UL)		/*!<GPIO IDR: ID11 (Bit 11) */
#define GPIO_IDR_ID11_Msk                                                  (0x800UL)		/*!< GPIO IDR: ID11 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID11                                                      GPIO_IDR_ID11_Msk
#define GPIO_IDR_ID10_Pos                                                  (10UL)		/*!<GPIO IDR: ID10 (Bit 10) */
#define GPIO_IDR_ID10_Msk                                                  (0x400UL)		/*!< GPIO IDR: ID10 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID10                                                      GPIO_IDR_ID10_Msk
#define GPIO_IDR_ID9_Pos                                                   (9UL)		/*!<GPIO IDR: ID9 (Bit 9) */
#define GPIO_IDR_ID9_Msk                                                   (0x200UL)		/*!< GPIO IDR: ID9 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID9                                                       GPIO_IDR_ID9_Msk
#define GPIO_IDR_ID8_Pos                                                   (8UL)		/*!<GPIO IDR: ID8 (Bit 8) */
#define GPIO_IDR_ID8_Msk                                                   (0x100UL)		/*!< GPIO IDR: ID8 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID8                                                       GPIO_IDR_ID8_Msk
#define GPIO_IDR_ID7_Pos                                                   (7UL)		/*!<GPIO IDR: ID7 (Bit 7) */
#define GPIO_IDR_ID7_Msk                                                   (0x80UL)		/*!< GPIO IDR: ID7 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID7                                                       GPIO_IDR_ID7_Msk
#define GPIO_IDR_ID6_Pos                                                   (6UL)		/*!<GPIO IDR: ID6 (Bit 6) */
#define GPIO_IDR_ID6_Msk                                                   (0x40UL)		/*!< GPIO IDR: ID6 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID6                                                       GPIO_IDR_ID6_Msk
#define GPIO_IDR_ID5_Pos                                                   (5UL)		/*!<GPIO IDR: ID5 (Bit 5) */
#define GPIO_IDR_ID5_Msk                                                   (0x20UL)		/*!< GPIO IDR: ID5 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID5                                                       GPIO_IDR_ID5_Msk
#define GPIO_IDR_ID4_Pos                                                   (4UL)		/*!<GPIO IDR: ID4 (Bit 4) */
#define GPIO_IDR_ID4_Msk                                                   (0x10UL)		/*!< GPIO IDR: ID4 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID4                                                       GPIO_IDR_ID4_Msk
#define GPIO_IDR_ID3_Pos                                                   (3UL)		/*!<GPIO IDR: ID3 (Bit 3) */
#define GPIO_IDR_ID3_Msk                                                   (0x8UL)		/*!< GPIO IDR: ID3 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID3                                                       GPIO_IDR_ID3_Msk
#define GPIO_IDR_ID2_Pos                                                   (2UL)		/*!<GPIO IDR: ID2 (Bit 2) */
#define GPIO_IDR_ID2_Msk                                                   (0x4UL)		/*!< GPIO IDR: ID2 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID2                                                       GPIO_IDR_ID2_Msk
#define GPIO_IDR_ID1_Pos                                                   (1UL)		/*!<GPIO IDR: ID1 (Bit 1) */
#define GPIO_IDR_ID1_Msk                                                   (0x2UL)		/*!< GPIO IDR: ID1 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID1                                                       GPIO_IDR_ID1_Msk
#define GPIO_IDR_ID0_Pos                                                   (0UL)		/*!<GPIO IDR: ID0 (Bit 0) */
#define GPIO_IDR_ID0_Msk                                                   (0x1UL)		/*!< GPIO IDR: ID0 (Bitfield-Mask: 0x01) */
#define GPIO_IDR_ID0                                                       GPIO_IDR_ID0_Msk

/* =====================================================    ODR    =====================================================*/ 
#define GPIO_ODR_OD15_Pos                                                  (15UL)		/*!<GPIO ODR: OD15 (Bit 15) */
#define GPIO_ODR_OD15_Msk                                                  (0x8000UL)		/*!< GPIO ODR: OD15 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD15                                                      GPIO_ODR_OD15_Msk
#define GPIO_ODR_OD14_Pos                                                  (14UL)		/*!<GPIO ODR: OD14 (Bit 14) */
#define GPIO_ODR_OD14_Msk                                                  (0x4000UL)		/*!< GPIO ODR: OD14 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD14                                                      GPIO_ODR_OD14_Msk
#define GPIO_ODR_OD13_Pos                                                  (13UL)		/*!<GPIO ODR: OD13 (Bit 13) */
#define GPIO_ODR_OD13_Msk                                                  (0x2000UL)		/*!< GPIO ODR: OD13 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD13                                                      GPIO_ODR_OD13_Msk
#define GPIO_ODR_OD12_Pos                                                  (12UL)		/*!<GPIO ODR: OD12 (Bit 12) */
#define GPIO_ODR_OD12_Msk                                                  (0x1000UL)		/*!< GPIO ODR: OD12 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD12                                                      GPIO_ODR_OD12_Msk
#define GPIO_ODR_OD11_Pos                                                  (11UL)		/*!<GPIO ODR: OD11 (Bit 11) */
#define GPIO_ODR_OD11_Msk                                                  (0x800UL)		/*!< GPIO ODR: OD11 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD11                                                      GPIO_ODR_OD11_Msk
#define GPIO_ODR_OD10_Pos                                                  (10UL)		/*!<GPIO ODR: OD10 (Bit 10) */
#define GPIO_ODR_OD10_Msk                                                  (0x400UL)		/*!< GPIO ODR: OD10 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD10                                                      GPIO_ODR_OD10_Msk
#define GPIO_ODR_OD9_Pos                                                   (9UL)		/*!<GPIO ODR: OD9 (Bit 9) */
#define GPIO_ODR_OD9_Msk                                                   (0x200UL)		/*!< GPIO ODR: OD9 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD9                                                       GPIO_ODR_OD9_Msk
#define GPIO_ODR_OD8_Pos                                                   (8UL)		/*!<GPIO ODR: OD8 (Bit 8) */
#define GPIO_ODR_OD8_Msk                                                   (0x100UL)		/*!< GPIO ODR: OD8 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD8                                                       GPIO_ODR_OD8_Msk
#define GPIO_ODR_OD7_Pos                                                   (7UL)		/*!<GPIO ODR: OD7 (Bit 7) */
#define GPIO_ODR_OD7_Msk                                                   (0x80UL)		/*!< GPIO ODR: OD7 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD7                                                       GPIO_ODR_OD7_Msk
#define GPIO_ODR_OD6_Pos                                                   (6UL)		/*!<GPIO ODR: OD6 (Bit 6) */
#define GPIO_ODR_OD6_Msk                                                   (0x40UL)		/*!< GPIO ODR: OD6 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD6                                                       GPIO_ODR_OD6_Msk
#define GPIO_ODR_OD5_Pos                                                   (5UL)		/*!<GPIO ODR: OD5 (Bit 5) */
#define GPIO_ODR_OD5_Msk                                                   (0x20UL)		/*!< GPIO ODR: OD5 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD5                                                       GPIO_ODR_OD5_Msk
#define GPIO_ODR_OD4_Pos                                                   (4UL)		/*!<GPIO ODR: OD4 (Bit 4) */
#define GPIO_ODR_OD4_Msk                                                   (0x10UL)		/*!< GPIO ODR: OD4 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD4                                                       GPIO_ODR_OD4_Msk
#define GPIO_ODR_OD3_Pos                                                   (3UL)		/*!<GPIO ODR: OD3 (Bit 3) */
#define GPIO_ODR_OD3_Msk                                                   (0x8UL)		/*!< GPIO ODR: OD3 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD3                                                       GPIO_ODR_OD3_Msk
#define GPIO_ODR_OD2_Pos                                                   (2UL)		/*!<GPIO ODR: OD2 (Bit 2) */
#define GPIO_ODR_OD2_Msk                                                   (0x4UL)		/*!< GPIO ODR: OD2 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD2                                                       GPIO_ODR_OD2_Msk
#define GPIO_ODR_OD1_Pos                                                   (1UL)		/*!<GPIO ODR: OD1 (Bit 1) */
#define GPIO_ODR_OD1_Msk                                                   (0x2UL)		/*!< GPIO ODR: OD1 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD1                                                       GPIO_ODR_OD1_Msk
#define GPIO_ODR_OD0_Pos                                                   (0UL)		/*!<GPIO ODR: OD0 (Bit 0) */
#define GPIO_ODR_OD0_Msk                                                   (0x1UL)		/*!< GPIO ODR: OD0 (Bitfield-Mask: 0x01) */
#define GPIO_ODR_OD0                                                       GPIO_ODR_OD0_Msk

/* =====================================================    BSRR    =====================================================*/ 
#define GPIO_BSRR_BR15_Pos                                                 (31UL)		/*!<GPIO BSRR: BR15 (Bit 31) */
#define GPIO_BSRR_BR15_Msk                                                 (0x80000000UL)		/*!< GPIO BSRR: BR15 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR15                                                     GPIO_BSRR_BR15_Msk
#define GPIO_BSRR_BR14_Pos                                                 (30UL)		/*!<GPIO BSRR: BR14 (Bit 30) */
#define GPIO_BSRR_BR14_Msk                                                 (0x40000000UL)		/*!< GPIO BSRR: BR14 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR14                                                     GPIO_BSRR_BR14_Msk
#define GPIO_BSRR_BR13_Pos                                                 (29UL)		/*!<GPIO BSRR: BR13 (Bit 29) */
#define GPIO_BSRR_BR13_Msk                                                 (0x20000000UL)		/*!< GPIO BSRR: BR13 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR13                                                     GPIO_BSRR_BR13_Msk
#define GPIO_BSRR_BR12_Pos                                                 (28UL)		/*!<GPIO BSRR: BR12 (Bit 28) */
#define GPIO_BSRR_BR12_Msk                                                 (0x10000000UL)		/*!< GPIO BSRR: BR12 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR12                                                     GPIO_BSRR_BR12_Msk
#define GPIO_BSRR_BR11_Pos                                                 (27UL)		/*!<GPIO BSRR: BR11 (Bit 27) */
#define GPIO_BSRR_BR11_Msk                                                 (0x8000000UL)		/*!< GPIO BSRR: BR11 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR11                                                     GPIO_BSRR_BR11_Msk
#define GPIO_BSRR_BR10_Pos                                                 (26UL)		/*!<GPIO BSRR: BR10 (Bit 26) */
#define GPIO_BSRR_BR10_Msk                                                 (0x4000000UL)		/*!< GPIO BSRR: BR10 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR10                                                     GPIO_BSRR_BR10_Msk
#define GPIO_BSRR_BR9_Pos                                                  (25UL)		/*!<GPIO BSRR: BR9 (Bit 25) */
#define GPIO_BSRR_BR9_Msk                                                  (0x2000000UL)		/*!< GPIO BSRR: BR9 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR9                                                      GPIO_BSRR_BR9_Msk
#define GPIO_BSRR_BR8_Pos                                                  (24UL)		/*!<GPIO BSRR: BR8 (Bit 24) */
#define GPIO_BSRR_BR8_Msk                                                  (0x1000000UL)		/*!< GPIO BSRR: BR8 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR8                                                      GPIO_BSRR_BR8_Msk
#define GPIO_BSRR_BR7_Pos                                                  (23UL)		/*!<GPIO BSRR: BR7 (Bit 23) */
#define GPIO_BSRR_BR7_Msk                                                  (0x800000UL)		/*!< GPIO BSRR: BR7 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR7                                                      GPIO_BSRR_BR7_Msk
#define GPIO_BSRR_BR6_Pos                                                  (22UL)		/*!<GPIO BSRR: BR6 (Bit 22) */
#define GPIO_BSRR_BR6_Msk                                                  (0x400000UL)		/*!< GPIO BSRR: BR6 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR6                                                      GPIO_BSRR_BR6_Msk
#define GPIO_BSRR_BR5_Pos                                                  (21UL)		/*!<GPIO BSRR: BR5 (Bit 21) */
#define GPIO_BSRR_BR5_Msk                                                  (0x200000UL)		/*!< GPIO BSRR: BR5 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR5                                                      GPIO_BSRR_BR5_Msk
#define GPIO_BSRR_BR4_Pos                                                  (20UL)		/*!<GPIO BSRR: BR4 (Bit 20) */
#define GPIO_BSRR_BR4_Msk                                                  (0x100000UL)		/*!< GPIO BSRR: BR4 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR4                                                      GPIO_BSRR_BR4_Msk
#define GPIO_BSRR_BR3_Pos                                                  (19UL)		/*!<GPIO BSRR: BR3 (Bit 19) */
#define GPIO_BSRR_BR3_Msk                                                  (0x80000UL)		/*!< GPIO BSRR: BR3 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR3                                                      GPIO_BSRR_BR3_Msk
#define GPIO_BSRR_BR2_Pos                                                  (18UL)		/*!<GPIO BSRR: BR2 (Bit 18) */
#define GPIO_BSRR_BR2_Msk                                                  (0x40000UL)		/*!< GPIO BSRR: BR2 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR2                                                      GPIO_BSRR_BR2_Msk
#define GPIO_BSRR_BR1_Pos                                                  (17UL)		/*!<GPIO BSRR: BR1 (Bit 17) */
#define GPIO_BSRR_BR1_Msk                                                  (0x20000UL)		/*!< GPIO BSRR: BR1 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR1                                                      GPIO_BSRR_BR1_Msk
#define GPIO_BSRR_BR0_Pos                                                  (16UL)		/*!<GPIO BSRR: BR0 (Bit 16) */
#define GPIO_BSRR_BR0_Msk                                                  (0x10000UL)		/*!< GPIO BSRR: BR0 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BR0                                                      GPIO_BSRR_BR0_Msk
#define GPIO_BSRR_BS15_Pos                                                 (15UL)		/*!<GPIO BSRR: BS15 (Bit 15) */
#define GPIO_BSRR_BS15_Msk                                                 (0x8000UL)		/*!< GPIO BSRR: BS15 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS15                                                     GPIO_BSRR_BS15_Msk
#define GPIO_BSRR_BS14_Pos                                                 (14UL)		/*!<GPIO BSRR: BS14 (Bit 14) */
#define GPIO_BSRR_BS14_Msk                                                 (0x4000UL)		/*!< GPIO BSRR: BS14 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS14                                                     GPIO_BSRR_BS14_Msk
#define GPIO_BSRR_BS13_Pos                                                 (13UL)		/*!<GPIO BSRR: BS13 (Bit 13) */
#define GPIO_BSRR_BS13_Msk                                                 (0x2000UL)		/*!< GPIO BSRR: BS13 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS13                                                     GPIO_BSRR_BS13_Msk
#define GPIO_BSRR_BS12_Pos                                                 (12UL)		/*!<GPIO BSRR: BS12 (Bit 12) */
#define GPIO_BSRR_BS12_Msk                                                 (0x1000UL)		/*!< GPIO BSRR: BS12 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS12                                                     GPIO_BSRR_BS12_Msk
#define GPIO_BSRR_BS11_Pos                                                 (11UL)		/*!<GPIO BSRR: BS11 (Bit 11) */
#define GPIO_BSRR_BS11_Msk                                                 (0x800UL)		/*!< GPIO BSRR: BS11 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS11                                                     GPIO_BSRR_BS11_Msk
#define GPIO_BSRR_BS10_Pos                                                 (10UL)		/*!<GPIO BSRR: BS10 (Bit 10) */
#define GPIO_BSRR_BS10_Msk                                                 (0x400UL)		/*!< GPIO BSRR: BS10 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS10                                                     GPIO_BSRR_BS10_Msk
#define GPIO_BSRR_BS9_Pos                                                  (9UL)		/*!<GPIO BSRR: BS9 (Bit 9) */
#define GPIO_BSRR_BS9_Msk                                                  (0x200UL)		/*!< GPIO BSRR: BS9 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS9                                                      GPIO_BSRR_BS9_Msk
#define GPIO_BSRR_BS8_Pos                                                  (8UL)		/*!<GPIO BSRR: BS8 (Bit 8) */
#define GPIO_BSRR_BS8_Msk                                                  (0x100UL)		/*!< GPIO BSRR: BS8 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS8                                                      GPIO_BSRR_BS8_Msk
#define GPIO_BSRR_BS7_Pos                                                  (7UL)		/*!<GPIO BSRR: BS7 (Bit 7) */
#define GPIO_BSRR_BS7_Msk                                                  (0x80UL)		/*!< GPIO BSRR: BS7 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS7                                                      GPIO_BSRR_BS7_Msk
#define GPIO_BSRR_BS6_Pos                                                  (6UL)		/*!<GPIO BSRR: BS6 (Bit 6) */
#define GPIO_BSRR_BS6_Msk                                                  (0x40UL)		/*!< GPIO BSRR: BS6 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS6                                                      GPIO_BSRR_BS6_Msk
#define GPIO_BSRR_BS5_Pos                                                  (5UL)		/*!<GPIO BSRR: BS5 (Bit 5) */
#define GPIO_BSRR_BS5_Msk                                                  (0x20UL)		/*!< GPIO BSRR: BS5 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS5                                                      GPIO_BSRR_BS5_Msk
#define GPIO_BSRR_BS4_Pos                                                  (4UL)		/*!<GPIO BSRR: BS4 (Bit 4) */
#define GPIO_BSRR_BS4_Msk                                                  (0x10UL)		/*!< GPIO BSRR: BS4 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS4                                                      GPIO_BSRR_BS4_Msk
#define GPIO_BSRR_BS3_Pos                                                  (3UL)		/*!<GPIO BSRR: BS3 (Bit 3) */
#define GPIO_BSRR_BS3_Msk                                                  (0x8UL)		/*!< GPIO BSRR: BS3 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS3                                                      GPIO_BSRR_BS3_Msk
#define GPIO_BSRR_BS2_Pos                                                  (2UL)		/*!<GPIO BSRR: BS2 (Bit 2) */
#define GPIO_BSRR_BS2_Msk                                                  (0x4UL)		/*!< GPIO BSRR: BS2 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS2                                                      GPIO_BSRR_BS2_Msk
#define GPIO_BSRR_BS1_Pos                                                  (1UL)		/*!<GPIO BSRR: BS1 (Bit 1) */
#define GPIO_BSRR_BS1_Msk                                                  (0x2UL)		/*!< GPIO BSRR: BS1 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS1                                                      GPIO_BSRR_BS1_Msk
#define GPIO_BSRR_BS0_Pos                                                  (0UL)		/*!<GPIO BSRR: BS0 (Bit 0) */
#define GPIO_BSRR_BS0_Msk                                                  (0x1UL)		/*!< GPIO BSRR: BS0 (Bitfield-Mask: 0x01) */
#define GPIO_BSRR_BS0                                                      GPIO_BSRR_BS0_Msk

/* =====================================================    LCKR    =====================================================*/ 
#define GPIO_LCKR_LCKK_Pos                                                 (16UL)		/*!<GPIO LCKR: LCKK (Bit 16) */
#define GPIO_LCKR_LCKK_Msk                                                 (0x10000UL)		/*!< GPIO LCKR: LCKK (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCKK                                                     GPIO_LCKR_LCKK_Msk
#define GPIO_LCKR_LCK15_Pos                                                (15UL)		/*!<GPIO LCKR: LCK15 (Bit 15) */
#define GPIO_LCKR_LCK15_Msk                                                (0x8000UL)		/*!< GPIO LCKR: LCK15 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK15                                                    GPIO_LCKR_LCK15_Msk
#define GPIO_LCKR_LCK14_Pos                                                (14UL)		/*!<GPIO LCKR: LCK14 (Bit 14) */
#define GPIO_LCKR_LCK14_Msk                                                (0x4000UL)		/*!< GPIO LCKR: LCK14 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK14                                                    GPIO_LCKR_LCK14_Msk
#define GPIO_LCKR_LCK13_Pos                                                (13UL)		/*!<GPIO LCKR: LCK13 (Bit 13) */
#define GPIO_LCKR_LCK13_Msk                                                (0x2000UL)		/*!< GPIO LCKR: LCK13 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK13                                                    GPIO_LCKR_LCK13_Msk
#define GPIO_LCKR_LCK12_Pos                                                (12UL)		/*!<GPIO LCKR: LCK12 (Bit 12) */
#define GPIO_LCKR_LCK12_Msk                                                (0x1000UL)		/*!< GPIO LCKR: LCK12 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK12                                                    GPIO_LCKR_LCK12_Msk
#define GPIO_LCKR_LCK11_Pos                                                (11UL)		/*!<GPIO LCKR: LCK11 (Bit 11) */
#define GPIO_LCKR_LCK11_Msk                                                (0x800UL)		/*!< GPIO LCKR: LCK11 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK11                                                    GPIO_LCKR_LCK11_Msk
#define GPIO_LCKR_LCK10_Pos                                                (10UL)		/*!<GPIO LCKR: LCK10 (Bit 10) */
#define GPIO_LCKR_LCK10_Msk                                                (0x400UL)		/*!< GPIO LCKR: LCK10 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK10                                                    GPIO_LCKR_LCK10_Msk
#define GPIO_LCKR_LCK9_Pos                                                 (9UL)		/*!<GPIO LCKR: LCK9 (Bit 9) */
#define GPIO_LCKR_LCK9_Msk                                                 (0x200UL)		/*!< GPIO LCKR: LCK9 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK9                                                     GPIO_LCKR_LCK9_Msk
#define GPIO_LCKR_LCK8_Pos                                                 (8UL)		/*!<GPIO LCKR: LCK8 (Bit 8) */
#define GPIO_LCKR_LCK8_Msk                                                 (0x100UL)		/*!< GPIO LCKR: LCK8 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK8                                                     GPIO_LCKR_LCK8_Msk
#define GPIO_LCKR_LCK7_Pos                                                 (7UL)		/*!<GPIO LCKR: LCK7 (Bit 7) */
#define GPIO_LCKR_LCK7_Msk                                                 (0x80UL)		/*!< GPIO LCKR: LCK7 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK7                                                     GPIO_LCKR_LCK7_Msk
#define GPIO_LCKR_LCK6_Pos                                                 (6UL)		/*!<GPIO LCKR: LCK6 (Bit 6) */
#define GPIO_LCKR_LCK6_Msk                                                 (0x40UL)		/*!< GPIO LCKR: LCK6 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK6                                                     GPIO_LCKR_LCK6_Msk
#define GPIO_LCKR_LCK5_Pos                                                 (5UL)		/*!<GPIO LCKR: LCK5 (Bit 5) */
#define GPIO_LCKR_LCK5_Msk                                                 (0x20UL)		/*!< GPIO LCKR: LCK5 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK5                                                     GPIO_LCKR_LCK5_Msk
#define GPIO_LCKR_LCK4_Pos                                                 (4UL)		/*!<GPIO LCKR: LCK4 (Bit 4) */
#define GPIO_LCKR_LCK4_Msk                                                 (0x10UL)		/*!< GPIO LCKR: LCK4 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK4                                                     GPIO_LCKR_LCK4_Msk
#define GPIO_LCKR_LCK3_Pos                                                 (3UL)		/*!<GPIO LCKR: LCK3 (Bit 3) */
#define GPIO_LCKR_LCK3_Msk                                                 (0x8UL)		/*!< GPIO LCKR: LCK3 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK3                                                     GPIO_LCKR_LCK3_Msk
#define GPIO_LCKR_LCK2_Pos                                                 (2UL)		/*!<GPIO LCKR: LCK2 (Bit 2) */
#define GPIO_LCKR_LCK2_Msk                                                 (0x4UL)		/*!< GPIO LCKR: LCK2 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK2                                                     GPIO_LCKR_LCK2_Msk
#define GPIO_LCKR_LCK1_Pos                                                 (1UL)		/*!<GPIO LCKR: LCK1 (Bit 1) */
#define GPIO_LCKR_LCK1_Msk                                                 (0x2UL)		/*!< GPIO LCKR: LCK1 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK1                                                     GPIO_LCKR_LCK1_Msk
#define GPIO_LCKR_LCK0_Pos                                                 (0UL)		/*!<GPIO LCKR: LCK0 (Bit 0) */
#define GPIO_LCKR_LCK0_Msk                                                 (0x1UL)		/*!< GPIO LCKR: LCK0 (Bitfield-Mask: 0x01) */
#define GPIO_LCKR_LCK0                                                     GPIO_LCKR_LCK0_Msk

/* =====================================================    AFRL    =====================================================*/ 
#define GPIO_AFRL_AFSEL7_Pos                                               (28UL)		/*!<GPIO AFRL: AFSEL7 (Bit 28) */
#define GPIO_AFRL_AFSEL7_Msk                                               (0xf0000000UL)		/*!< GPIO AFRL: AFSEL7 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL7                                                   GPIO_AFRL_AFSEL7_Msk
#define GPIO_AFRL_AFSEL7_0                                                 (0x1U << GPIO_AFRL_AFSEL7_Pos)
#define GPIO_AFRL_AFSEL7_1                                                 (0x2U << GPIO_AFRL_AFSEL7_Pos)
#define GPIO_AFRL_AFSEL7_2                                                 (0x4U << GPIO_AFRL_AFSEL7_Pos)
#define GPIO_AFRL_AFSEL7_3                                                 (0x8U << GPIO_AFRL_AFSEL7_Pos)
#define GPIO_AFRL_AFSEL6_Pos                                               (24UL)		/*!<GPIO AFRL: AFSEL6 (Bit 24) */
#define GPIO_AFRL_AFSEL6_Msk                                               (0xf000000UL)		/*!< GPIO AFRL: AFSEL6 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL6                                                   GPIO_AFRL_AFSEL6_Msk
#define GPIO_AFRL_AFSEL6_0                                                 (0x1U << GPIO_AFRL_AFSEL6_Pos)
#define GPIO_AFRL_AFSEL6_1                                                 (0x2U << GPIO_AFRL_AFSEL6_Pos)
#define GPIO_AFRL_AFSEL6_2                                                 (0x4U << GPIO_AFRL_AFSEL6_Pos)
#define GPIO_AFRL_AFSEL6_3                                                 (0x8U << GPIO_AFRL_AFSEL6_Pos)
#define GPIO_AFRL_AFSEL5_Pos                                               (20UL)		/*!<GPIO AFRL: AFSEL5 (Bit 20) */
#define GPIO_AFRL_AFSEL5_Msk                                               (0xf00000UL)		/*!< GPIO AFRL: AFSEL5 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL5                                                   GPIO_AFRL_AFSEL5_Msk
#define GPIO_AFRL_AFSEL5_0                                                 (0x1U << GPIO_AFRL_AFSEL5_Pos)
#define GPIO_AFRL_AFSEL5_1                                                 (0x2U << GPIO_AFRL_AFSEL5_Pos)
#define GPIO_AFRL_AFSEL5_2                                                 (0x4U << GPIO_AFRL_AFSEL5_Pos)
#define GPIO_AFRL_AFSEL5_3                                                 (0x8U << GPIO_AFRL_AFSEL5_Pos)
#define GPIO_AFRL_AFSEL4_Pos                                               (16UL)		/*!<GPIO AFRL: AFSEL4 (Bit 16) */
#define GPIO_AFRL_AFSEL4_Msk                                               (0xf0000UL)		/*!< GPIO AFRL: AFSEL4 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL4                                                   GPIO_AFRL_AFSEL4_Msk
#define GPIO_AFRL_AFSEL4_0                                                 (0x1U << GPIO_AFRL_AFSEL4_Pos)
#define GPIO_AFRL_AFSEL4_1                                                 (0x2U << GPIO_AFRL_AFSEL4_Pos)
#define GPIO_AFRL_AFSEL4_2                                                 (0x4U << GPIO_AFRL_AFSEL4_Pos)
#define GPIO_AFRL_AFSEL4_3                                                 (0x8U << GPIO_AFRL_AFSEL4_Pos)
#define GPIO_AFRL_AFSEL3_Pos                                               (12UL)		/*!<GPIO AFRL: AFSEL3 (Bit 12) */
#define GPIO_AFRL_AFSEL3_Msk                                               (0xf000UL)		/*!< GPIO AFRL: AFSEL3 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL3                                                   GPIO_AFRL_AFSEL3_Msk
#define GPIO_AFRL_AFSEL3_0                                                 (0x1U << GPIO_AFRL_AFSEL3_Pos)
#define GPIO_AFRL_AFSEL3_1                                                 (0x2U << GPIO_AFRL_AFSEL3_Pos)
#define GPIO_AFRL_AFSEL3_2                                                 (0x4U << GPIO_AFRL_AFSEL3_Pos)
#define GPIO_AFRL_AFSEL3_3                                                 (0x8U << GPIO_AFRL_AFSEL3_Pos)
#define GPIO_AFRL_AFSEL2_Pos                                               (8UL)		/*!<GPIO AFRL: AFSEL2 (Bit 8) */
#define GPIO_AFRL_AFSEL2_Msk                                               (0xf00UL)		/*!< GPIO AFRL: AFSEL2 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL2                                                   GPIO_AFRL_AFSEL2_Msk
#define GPIO_AFRL_AFSEL2_0                                                 (0x1U << GPIO_AFRL_AFSEL2_Pos)
#define GPIO_AFRL_AFSEL2_1                                                 (0x2U << GPIO_AFRL_AFSEL2_Pos)
#define GPIO_AFRL_AFSEL2_2                                                 (0x4U << GPIO_AFRL_AFSEL2_Pos)
#define GPIO_AFRL_AFSEL2_3                                                 (0x8U << GPIO_AFRL_AFSEL2_Pos)
#define GPIO_AFRL_AFSEL1_Pos                                               (4UL)		/*!<GPIO AFRL: AFSEL1 (Bit 4) */
#define GPIO_AFRL_AFSEL1_Msk                                               (0xf0UL)		/*!< GPIO AFRL: AFSEL1 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL1                                                   GPIO_AFRL_AFSEL1_Msk
#define GPIO_AFRL_AFSEL1_0                                                 (0x1U << GPIO_AFRL_AFSEL1_Pos)
#define GPIO_AFRL_AFSEL1_1                                                 (0x2U << GPIO_AFRL_AFSEL1_Pos)
#define GPIO_AFRL_AFSEL1_2                                                 (0x4U << GPIO_AFRL_AFSEL1_Pos)
#define GPIO_AFRL_AFSEL1_3                                                 (0x8U << GPIO_AFRL_AFSEL1_Pos)
#define GPIO_AFRL_AFSEL0_Pos                                               (0UL)		/*!<GPIO AFRL: AFSEL0 (Bit 0) */
#define GPIO_AFRL_AFSEL0_Msk                                               (0xfUL)		/*!< GPIO AFRL: AFSEL0 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRL_AFSEL0                                                   GPIO_AFRL_AFSEL0_Msk
#define GPIO_AFRL_AFSEL0_0                                                 (0x1U << GPIO_AFRL_AFSEL0_Pos)
#define GPIO_AFRL_AFSEL0_1                                                 (0x2U << GPIO_AFRL_AFSEL0_Pos)
#define GPIO_AFRL_AFSEL0_2                                                 (0x4U << GPIO_AFRL_AFSEL0_Pos)
#define GPIO_AFRL_AFSEL0_3                                                 (0x8U << GPIO_AFRL_AFSEL0_Pos)

/* =====================================================    AFRH    =====================================================*/ 
#define GPIO_AFRH_AFSEL15_Pos                                              (28UL)		/*!<GPIO AFRH: AFSEL15 (Bit 28) */
#define GPIO_AFRH_AFSEL15_Msk                                              (0xf0000000UL)		/*!< GPIO AFRH: AFSEL15 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL15                                                  GPIO_AFRH_AFSEL15_Msk
#define GPIO_AFRH_AFSEL15_0                                                (0x1U << GPIO_AFRH_AFSEL15_Pos)
#define GPIO_AFRH_AFSEL15_1                                                (0x2U << GPIO_AFRH_AFSEL15_Pos)
#define GPIO_AFRH_AFSEL15_2                                                (0x4U << GPIO_AFRH_AFSEL15_Pos)
#define GPIO_AFRH_AFSEL15_3                                                (0x8U << GPIO_AFRH_AFSEL15_Pos)
#define GPIO_AFRH_AFSEL14_Pos                                              (24UL)		/*!<GPIO AFRH: AFSEL14 (Bit 24) */
#define GPIO_AFRH_AFSEL14_Msk                                              (0xf000000UL)		/*!< GPIO AFRH: AFSEL14 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL14                                                  GPIO_AFRH_AFSEL14_Msk
#define GPIO_AFRH_AFSEL14_0                                                (0x1U << GPIO_AFRH_AFSEL14_Pos)
#define GPIO_AFRH_AFSEL14_1                                                (0x2U << GPIO_AFRH_AFSEL14_Pos)
#define GPIO_AFRH_AFSEL14_2                                                (0x4U << GPIO_AFRH_AFSEL14_Pos)
#define GPIO_AFRH_AFSEL14_3                                                (0x8U << GPIO_AFRH_AFSEL14_Pos)
#define GPIO_AFRH_AFSEL13_Pos                                              (20UL)		/*!<GPIO AFRH: AFSEL13 (Bit 20) */
#define GPIO_AFRH_AFSEL13_Msk                                              (0xf00000UL)		/*!< GPIO AFRH: AFSEL13 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL13                                                  GPIO_AFRH_AFSEL13_Msk
#define GPIO_AFRH_AFSEL13_0                                                (0x1U << GPIO_AFRH_AFSEL13_Pos)
#define GPIO_AFRH_AFSEL13_1                                                (0x2U << GPIO_AFRH_AFSEL13_Pos)
#define GPIO_AFRH_AFSEL13_2                                                (0x4U << GPIO_AFRH_AFSEL13_Pos)
#define GPIO_AFRH_AFSEL13_3                                                (0x8U << GPIO_AFRH_AFSEL13_Pos)
#define GPIO_AFRH_AFSEL12_Pos                                              (16UL)		/*!<GPIO AFRH: AFSEL12 (Bit 16) */
#define GPIO_AFRH_AFSEL12_Msk                                              (0xf0000UL)		/*!< GPIO AFRH: AFSEL12 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL12                                                  GPIO_AFRH_AFSEL12_Msk
#define GPIO_AFRH_AFSEL12_0                                                (0x1U << GPIO_AFRH_AFSEL12_Pos)
#define GPIO_AFRH_AFSEL12_1                                                (0x2U << GPIO_AFRH_AFSEL12_Pos)
#define GPIO_AFRH_AFSEL12_2                                                (0x4U << GPIO_AFRH_AFSEL12_Pos)
#define GPIO_AFRH_AFSEL12_3                                                (0x8U << GPIO_AFRH_AFSEL12_Pos)
#define GPIO_AFRH_AFSEL11_Pos                                              (12UL)		/*!<GPIO AFRH: AFSEL11 (Bit 12) */
#define GPIO_AFRH_AFSEL11_Msk                                              (0xf000UL)		/*!< GPIO AFRH: AFSEL11 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL11                                                  GPIO_AFRH_AFSEL11_Msk
#define GPIO_AFRH_AFSEL11_0                                                (0x1U << GPIO_AFRH_AFSEL11_Pos)
#define GPIO_AFRH_AFSEL11_1                                                (0x2U << GPIO_AFRH_AFSEL11_Pos)
#define GPIO_AFRH_AFSEL11_2                                                (0x4U << GPIO_AFRH_AFSEL11_Pos)
#define GPIO_AFRH_AFSEL11_3                                                (0x8U << GPIO_AFRH_AFSEL11_Pos)
#define GPIO_AFRH_AFSEL10_Pos                                              (8UL)		/*!<GPIO AFRH: AFSEL10 (Bit 8) */
#define GPIO_AFRH_AFSEL10_Msk                                              (0xf00UL)		/*!< GPIO AFRH: AFSEL10 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL10                                                  GPIO_AFRH_AFSEL10_Msk
#define GPIO_AFRH_AFSEL10_0                                                (0x1U << GPIO_AFRH_AFSEL10_Pos)
#define GPIO_AFRH_AFSEL10_1                                                (0x2U << GPIO_AFRH_AFSEL10_Pos)
#define GPIO_AFRH_AFSEL10_2                                                (0x4U << GPIO_AFRH_AFSEL10_Pos)
#define GPIO_AFRH_AFSEL10_3                                                (0x8U << GPIO_AFRH_AFSEL10_Pos)
#define GPIO_AFRH_AFSEL9_Pos                                               (4UL)		/*!<GPIO AFRH: AFSEL9 (Bit 4) */
#define GPIO_AFRH_AFSEL9_Msk                                               (0xf0UL)		/*!< GPIO AFRH: AFSEL9 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL9                                                   GPIO_AFRH_AFSEL9_Msk
#define GPIO_AFRH_AFSEL9_0                                                 (0x1U << GPIO_AFRH_AFSEL9_Pos)
#define GPIO_AFRH_AFSEL9_1                                                 (0x2U << GPIO_AFRH_AFSEL9_Pos)
#define GPIO_AFRH_AFSEL9_2                                                 (0x4U << GPIO_AFRH_AFSEL9_Pos)
#define GPIO_AFRH_AFSEL9_3                                                 (0x8U << GPIO_AFRH_AFSEL9_Pos)
#define GPIO_AFRH_AFSEL8_Pos                                               (0UL)		/*!<GPIO AFRH: AFSEL8 (Bit 0) */
#define GPIO_AFRH_AFSEL8_Msk                                               (0xfUL)		/*!< GPIO AFRH: AFSEL8 (Bitfield-Mask: 0x0f) */
#define GPIO_AFRH_AFSEL8                                                   GPIO_AFRH_AFSEL8_Msk
#define GPIO_AFRH_AFSEL8_0                                                 (0x1U << GPIO_AFRH_AFSEL8_Pos)
#define GPIO_AFRH_AFSEL8_1                                                 (0x2U << GPIO_AFRH_AFSEL8_Pos)
#define GPIO_AFRH_AFSEL8_2                                                 (0x4U << GPIO_AFRH_AFSEL8_Pos)
#define GPIO_AFRH_AFSEL8_3                                                 (0x8U << GPIO_AFRH_AFSEL8_Pos)

/* =====================================================    BRR    =====================================================*/ 
#define GPIO_BRR_BR15_Pos                                                  (15UL)		/*!<GPIO BRR: BR15 (Bit 15) */
#define GPIO_BRR_BR15_Msk                                                  (0x8000UL)		/*!< GPIO BRR: BR15 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR15                                                      GPIO_BRR_BR15_Msk
#define GPIO_BRR_BR14_Pos                                                  (14UL)		/*!<GPIO BRR: BR14 (Bit 14) */
#define GPIO_BRR_BR14_Msk                                                  (0x4000UL)		/*!< GPIO BRR: BR14 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR14                                                      GPIO_BRR_BR14_Msk
#define GPIO_BRR_BR13_Pos                                                  (13UL)		/*!<GPIO BRR: BR13 (Bit 13) */
#define GPIO_BRR_BR13_Msk                                                  (0x2000UL)		/*!< GPIO BRR: BR13 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR13                                                      GPIO_BRR_BR13_Msk
#define GPIO_BRR_BR12_Pos                                                  (12UL)		/*!<GPIO BRR: BR12 (Bit 12) */
#define GPIO_BRR_BR12_Msk                                                  (0x1000UL)		/*!< GPIO BRR: BR12 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR12                                                      GPIO_BRR_BR12_Msk
#define GPIO_BRR_BR11_Pos                                                  (11UL)		/*!<GPIO BRR: BR11 (Bit 11) */
#define GPIO_BRR_BR11_Msk                                                  (0x800UL)		/*!< GPIO BRR: BR11 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR11                                                      GPIO_BRR_BR11_Msk
#define GPIO_BRR_BR10_Pos                                                  (10UL)		/*!<GPIO BRR: BR10 (Bit 10) */
#define GPIO_BRR_BR10_Msk                                                  (0x400UL)		/*!< GPIO BRR: BR10 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR10                                                      GPIO_BRR_BR10_Msk
#define GPIO_BRR_BR9_Pos                                                   (9UL)		/*!<GPIO BRR: BR9 (Bit 9) */
#define GPIO_BRR_BR9_Msk                                                   (0x200UL)		/*!< GPIO BRR: BR9 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR9                                                       GPIO_BRR_BR9_Msk
#define GPIO_BRR_BR8_Pos                                                   (8UL)		/*!<GPIO BRR: BR8 (Bit 8) */
#define GPIO_BRR_BR8_Msk                                                   (0x100UL)		/*!< GPIO BRR: BR8 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR8                                                       GPIO_BRR_BR8_Msk
#define GPIO_BRR_BR7_Pos                                                   (7UL)		/*!<GPIO BRR: BR7 (Bit 7) */
#define GPIO_BRR_BR7_Msk                                                   (0x80UL)		/*!< GPIO BRR: BR7 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR7                                                       GPIO_BRR_BR7_Msk
#define GPIO_BRR_BR6_Pos                                                   (6UL)		/*!<GPIO BRR: BR6 (Bit 6) */
#define GPIO_BRR_BR6_Msk                                                   (0x40UL)		/*!< GPIO BRR: BR6 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR6                                                       GPIO_BRR_BR6_Msk
#define GPIO_BRR_BR5_Pos                                                   (5UL)		/*!<GPIO BRR: BR5 (Bit 5) */
#define GPIO_BRR_BR5_Msk                                                   (0x20UL)		/*!< GPIO BRR: BR5 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR5                                                       GPIO_BRR_BR5_Msk
#define GPIO_BRR_BR4_Pos                                                   (4UL)		/*!<GPIO BRR: BR4 (Bit 4) */
#define GPIO_BRR_BR4_Msk                                                   (0x10UL)		/*!< GPIO BRR: BR4 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR4                                                       GPIO_BRR_BR4_Msk
#define GPIO_BRR_BR3_Pos                                                   (3UL)		/*!<GPIO BRR: BR3 (Bit 3) */
#define GPIO_BRR_BR3_Msk                                                   (0x8UL)		/*!< GPIO BRR: BR3 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR3                                                       GPIO_BRR_BR3_Msk
#define GPIO_BRR_BR2_Pos                                                   (2UL)		/*!<GPIO BRR: BR2 (Bit 2) */
#define GPIO_BRR_BR2_Msk                                                   (0x4UL)		/*!< GPIO BRR: BR2 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR2                                                       GPIO_BRR_BR2_Msk
#define GPIO_BRR_BR1_Pos                                                   (1UL)		/*!<GPIO BRR: BR1 (Bit 1) */
#define GPIO_BRR_BR1_Msk                                                   (0x2UL)		/*!< GPIO BRR: BR1 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR1                                                       GPIO_BRR_BR1_Msk
#define GPIO_BRR_BR0_Pos                                                   (0UL)		/*!<GPIO BRR: BR0 (Bit 0) */
#define GPIO_BRR_BR0_Msk                                                   (0x1UL)		/*!< GPIO BRR: BR0 (Bitfield-Mask: 0x01) */
#define GPIO_BRR_BR0                                                       GPIO_BRR_BR0_Msk


/* ============================================================================================================================*/ 
/*=====================                                        CRC                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    DR    =====================================================*/ 
#define CRC_DR_DR_Pos                                                      (0UL)		/*!<CRC DR: DR (Bit 0) */
#define CRC_DR_DR_Msk                                                      (0xffffffffUL)		/*!< CRC DR: DR (Bitfield-Mask: 0xffffffff) */
#define CRC_DR_DR                                                          CRC_DR_DR_Msk
#define CRC_DR_DR_0                                                        (0x1U << CRC_DR_DR_Pos)
#define CRC_DR_DR_1                                                        (0x2U << CRC_DR_DR_Pos)
#define CRC_DR_DR_2                                                        (0x4U << CRC_DR_DR_Pos)
#define CRC_DR_DR_3                                                        (0x8U << CRC_DR_DR_Pos)
#define CRC_DR_DR_4                                                        (0x10U << CRC_DR_DR_Pos)
#define CRC_DR_DR_5                                                        (0x20U << CRC_DR_DR_Pos)
#define CRC_DR_DR_6                                                        (0x40U << CRC_DR_DR_Pos)
#define CRC_DR_DR_7                                                        (0x80U << CRC_DR_DR_Pos)
#define CRC_DR_DR_8                                                        (0x100U << CRC_DR_DR_Pos)
#define CRC_DR_DR_9                                                        (0x200U << CRC_DR_DR_Pos)
#define CRC_DR_DR_10                                                       (0x400U << CRC_DR_DR_Pos)
#define CRC_DR_DR_11                                                       (0x800U << CRC_DR_DR_Pos)
#define CRC_DR_DR_12                                                       (0x1000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_13                                                       (0x2000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_14                                                       (0x4000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_15                                                       (0x8000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_16                                                       (0x10000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_17                                                       (0x20000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_18                                                       (0x40000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_19                                                       (0x80000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_20                                                       (0x100000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_21                                                       (0x200000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_22                                                       (0x400000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_23                                                       (0x800000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_24                                                       (0x1000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_25                                                       (0x2000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_26                                                       (0x4000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_27                                                       (0x8000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_28                                                       (0x10000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_29                                                       (0x20000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_30                                                       (0x40000000U << CRC_DR_DR_Pos)
#define CRC_DR_DR_31                                                       (0x80000000UL << CRC_DR_DR_Pos)

/* =====================================================    IDR    =====================================================*/ 
#define CRC_IDR_IDR_Pos                                                    (0UL)		/*!<CRC IDR: IDR (Bit 0) */
#define CRC_IDR_IDR_Msk                                                    (0xffffffffUL)		/*!< CRC IDR: IDR (Bitfield-Mask: 0xffffffff) */
#define CRC_IDR_IDR                                                        CRC_IDR_IDR_Msk
#define CRC_IDR_IDR_0                                                      (0x1U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_1                                                      (0x2U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_2                                                      (0x4U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_3                                                      (0x8U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_4                                                      (0x10U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_5                                                      (0x20U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_6                                                      (0x40U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_7                                                      (0x80U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_8                                                      (0x100U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_9                                                      (0x200U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_10                                                     (0x400U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_11                                                     (0x800U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_12                                                     (0x1000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_13                                                     (0x2000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_14                                                     (0x4000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_15                                                     (0x8000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_16                                                     (0x10000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_17                                                     (0x20000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_18                                                     (0x40000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_19                                                     (0x80000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_20                                                     (0x100000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_21                                                     (0x200000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_22                                                     (0x400000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_23                                                     (0x800000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_24                                                     (0x1000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_25                                                     (0x2000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_26                                                     (0x4000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_27                                                     (0x8000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_28                                                     (0x10000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_29                                                     (0x20000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_30                                                     (0x40000000U << CRC_IDR_IDR_Pos)
#define CRC_IDR_IDR_31                                                     (0x80000000UL << CRC_IDR_IDR_Pos)

/* =====================================================    CR    =====================================================*/ 
#define CRC_CR_REV_OUT_Pos                                                 (7UL)		/*!<CRC CR: REV_OUT (Bit 7) */
#define CRC_CR_REV_OUT_Msk                                                 (0x80UL)		/*!< CRC CR: REV_OUT (Bitfield-Mask: 0x01) */
#define CRC_CR_REV_OUT                                                     CRC_CR_REV_OUT_Msk
#define CRC_CR_REV_IN_Pos                                                  (5UL)		/*!<CRC CR: REV_IN (Bit 5) */
#define CRC_CR_REV_IN_Msk                                                  (0x60UL)		/*!< CRC CR: REV_IN (Bitfield-Mask: 0x03) */
#define CRC_CR_REV_IN                                                      CRC_CR_REV_IN_Msk
#define CRC_CR_REV_IN_0                                                    (0x1U << CRC_CR_REV_IN_Pos)
#define CRC_CR_REV_IN_1                                                    (0x2U << CRC_CR_REV_IN_Pos)
#define CRC_CR_POLYSIZE_Pos                                                (3UL)		/*!<CRC CR: POLYSIZE (Bit 3) */
#define CRC_CR_POLYSIZE_Msk                                                (0x18UL)		/*!< CRC CR: POLYSIZE (Bitfield-Mask: 0x03) */
#define CRC_CR_POLYSIZE                                                    CRC_CR_POLYSIZE_Msk
#define CRC_CR_POLYSIZE_0                                                  (0x1U << CRC_CR_POLYSIZE_Pos)
#define CRC_CR_POLYSIZE_1                                                  (0x2U << CRC_CR_POLYSIZE_Pos)
#define CRC_CR_RESET_Pos                                                   (0UL)		/*!<CRC CR: RESET (Bit 0) */
#define CRC_CR_RESET_Msk                                                   (0x1UL)		/*!< CRC CR: RESET (Bitfield-Mask: 0x01) */
#define CRC_CR_RESET                                                       CRC_CR_RESET_Msk

/* =====================================================    INIT    =====================================================*/ 
#define CRC_INIT_CRC_INIT_Pos                                              (0UL)		/*!<CRC INIT: CRC_INIT (Bit 0) */
#define CRC_INIT_CRC_INIT_Msk                                              (0xffffffffUL)		/*!< CRC INIT: CRC_INIT (Bitfield-Mask: 0xffffffff) */
#define CRC_INIT_CRC_INIT                                                  CRC_INIT_CRC_INIT_Msk
#define CRC_INIT_CRC_INIT_0                                                (0x1U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_1                                                (0x2U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_2                                                (0x4U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_3                                                (0x8U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_4                                                (0x10U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_5                                                (0x20U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_6                                                (0x40U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_7                                                (0x80U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_8                                                (0x100U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_9                                                (0x200U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_10                                               (0x400U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_11                                               (0x800U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_12                                               (0x1000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_13                                               (0x2000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_14                                               (0x4000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_15                                               (0x8000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_16                                               (0x10000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_17                                               (0x20000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_18                                               (0x40000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_19                                               (0x80000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_20                                               (0x100000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_21                                               (0x200000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_22                                               (0x400000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_23                                               (0x800000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_24                                               (0x1000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_25                                               (0x2000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_26                                               (0x4000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_27                                               (0x8000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_28                                               (0x10000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_29                                               (0x20000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_30                                               (0x40000000U << CRC_INIT_CRC_INIT_Pos)
#define CRC_INIT_CRC_INIT_31                                               (0x80000000UL << CRC_INIT_CRC_INIT_Pos)

/* =====================================================    POL    =====================================================*/ 
#define CRC_POL_POL_Pos                                                    (0UL)		/*!<CRC POL: POL (Bit 0) */
#define CRC_POL_POL_Msk                                                    (0xffffffffUL)		/*!< CRC POL: POL (Bitfield-Mask: 0xffffffff) */
#define CRC_POL_POL                                                        CRC_POL_POL_Msk
#define CRC_POL_POL_0                                                      (0x1U << CRC_POL_POL_Pos)
#define CRC_POL_POL_1                                                      (0x2U << CRC_POL_POL_Pos)
#define CRC_POL_POL_2                                                      (0x4U << CRC_POL_POL_Pos)
#define CRC_POL_POL_3                                                      (0x8U << CRC_POL_POL_Pos)
#define CRC_POL_POL_4                                                      (0x10U << CRC_POL_POL_Pos)
#define CRC_POL_POL_5                                                      (0x20U << CRC_POL_POL_Pos)
#define CRC_POL_POL_6                                                      (0x40U << CRC_POL_POL_Pos)
#define CRC_POL_POL_7                                                      (0x80U << CRC_POL_POL_Pos)
#define CRC_POL_POL_8                                                      (0x100U << CRC_POL_POL_Pos)
#define CRC_POL_POL_9                                                      (0x200U << CRC_POL_POL_Pos)
#define CRC_POL_POL_10                                                     (0x400U << CRC_POL_POL_Pos)
#define CRC_POL_POL_11                                                     (0x800U << CRC_POL_POL_Pos)
#define CRC_POL_POL_12                                                     (0x1000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_13                                                     (0x2000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_14                                                     (0x4000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_15                                                     (0x8000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_16                                                     (0x10000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_17                                                     (0x20000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_18                                                     (0x40000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_19                                                     (0x80000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_20                                                     (0x100000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_21                                                     (0x200000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_22                                                     (0x400000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_23                                                     (0x800000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_24                                                     (0x1000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_25                                                     (0x2000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_26                                                     (0x4000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_27                                                     (0x8000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_28                                                     (0x10000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_29                                                     (0x20000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_30                                                     (0x40000000U << CRC_POL_POL_Pos)
#define CRC_POL_POL_31                                                     (0x80000000UL << CRC_POL_POL_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        RCC                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define RCC_CR_HSERDY_Pos                                                  (17UL)		/*!<RCC CR: HSERDY (Bit 17) */
#define RCC_CR_HSERDY_Msk                                                  (0x20000UL)		/*!< RCC CR: HSERDY (Bitfield-Mask: 0x01) */
#define RCC_CR_HSERDY                                                      RCC_CR_HSERDY_Msk
#define RCC_CR_HSEON_Pos                                                   (16UL)		/*!<RCC CR: HSEON (Bit 16) */
#define RCC_CR_HSEON_Msk                                                   (0x10000UL)		/*!< RCC CR: HSEON (Bitfield-Mask: 0x01) */
#define RCC_CR_HSEON                                                       RCC_CR_HSEON_Msk
#define RCC_CR_FMRAT_Pos                                                   (15UL)		/*!<RCC CR: FMRAT (Bit 15) */
#define RCC_CR_FMRAT_Msk                                                   (0x8000UL)		/*!< RCC CR: FMRAT (Bitfield-Mask: 0x01) */
#define RCC_CR_FMRAT                                                       RCC_CR_FMRAT_Msk
#define RCC_CR_HSIPLLRDY_Pos                                               (14UL)		/*!<RCC CR: HSIPLLRDY (Bit 14) */
#define RCC_CR_HSIPLLRDY_Msk                                               (0x4000UL)		/*!< RCC CR: HSIPLLRDY (Bitfield-Mask: 0x01) */
#define RCC_CR_HSIPLLRDY                                                   RCC_CR_HSIPLLRDY_Msk
#define RCC_CR_HSIPLLON_Pos                                                (13UL)		/*!<RCC CR: HSIPLLON (Bit 13) */
#define RCC_CR_HSIPLLON_Msk                                                (0x2000UL)		/*!< RCC CR: HSIPLLON (Bitfield-Mask: 0x01) */
#define RCC_CR_HSIPLLON                                                    RCC_CR_HSIPLLON_Msk
#define RCC_CR_HSEPLLBUFON_Pos                                             (12UL)		/*!<RCC CR: HSEPLLBUFON (Bit 12) */
#define RCC_CR_HSEPLLBUFON_Msk                                             (0x1000UL)		/*!< RCC CR: HSEPLLBUFON (Bitfield-Mask: 0x01) */
#define RCC_CR_HSEPLLBUFON                                                 RCC_CR_HSEPLLBUFON_Msk
#define RCC_CR_HSIRDY_Pos                                                  (10UL)		/*!<RCC CR: HSIRDY (Bit 10) */
#define RCC_CR_HSIRDY_Msk                                                  (0x400UL)		/*!< RCC CR: HSIRDY (Bitfield-Mask: 0x01) */
#define RCC_CR_HSIRDY                                                      RCC_CR_HSIRDY_Msk
#define RCC_CR_LOCKDET_NSTOP_Pos                                           (7UL)		/*!<RCC CR: LOCKDET_NSTOP (Bit 7) */
#define RCC_CR_LOCKDET_NSTOP_Msk                                           (0x380UL)		/*!< RCC CR: LOCKDET_NSTOP (Bitfield-Mask: 0x07) */
#define RCC_CR_LOCKDET_NSTOP                                               RCC_CR_LOCKDET_NSTOP_Msk
#define RCC_CR_LOCKDET_NSTOP_0                                             (0x1U << RCC_CR_LOCKDET_NSTOP_Pos)
#define RCC_CR_LOCKDET_NSTOP_1                                             (0x2U << RCC_CR_LOCKDET_NSTOP_Pos)
#define RCC_CR_LOCKDET_NSTOP_2                                             (0x4U << RCC_CR_LOCKDET_NSTOP_Pos)
#define RCC_CR_LSEBYP_Pos                                                  (6UL)		/*!<RCC CR: LSEBYP (Bit 6) */
#define RCC_CR_LSEBYP_Msk                                                  (0x40UL)		/*!< RCC CR: LSEBYP (Bitfield-Mask: 0x01) */
#define RCC_CR_LSEBYP                                                      RCC_CR_LSEBYP_Msk
#define RCC_CR_LSERDY_Pos                                                  (5UL)		/*!<RCC CR: LSERDY (Bit 5) */
#define RCC_CR_LSERDY_Msk                                                  (0x20UL)		/*!< RCC CR: LSERDY (Bitfield-Mask: 0x01) */
#define RCC_CR_LSERDY                                                      RCC_CR_LSERDY_Msk
#define RCC_CR_LSEON_Pos                                                   (4UL)		/*!<RCC CR: LSEON (Bit 4) */
#define RCC_CR_LSEON_Msk                                                   (0x10UL)		/*!< RCC CR: LSEON (Bitfield-Mask: 0x01) */
#define RCC_CR_LSEON                                                       RCC_CR_LSEON_Msk
#define RCC_CR_LSIRDY_Pos                                                  (3UL)		/*!<RCC CR: LSIRDY (Bit 3) */
#define RCC_CR_LSIRDY_Msk                                                  (0x8UL)		/*!< RCC CR: LSIRDY (Bitfield-Mask: 0x01) */
#define RCC_CR_LSIRDY                                                      RCC_CR_LSIRDY_Msk
#define RCC_CR_LSION_Pos                                                   (2UL)		/*!<RCC CR: LSION (Bit 2) */
#define RCC_CR_LSION_Msk                                                   (0x4UL)		/*!< RCC CR: LSION (Bitfield-Mask: 0x01) */
#define RCC_CR_LSION                                                       RCC_CR_LSION_Msk

/* =====================================================    ICSCR    =====================================================*/ 
#define RCC_ICSCR_HSITRIM_Pos                                              (24UL)		/*!<RCC ICSCR: HSITRIM (Bit 24) */
#define RCC_ICSCR_HSITRIM_Msk                                              (0x3f000000UL)		/*!< RCC ICSCR: HSITRIM (Bitfield-Mask: 0x3f) */
#define RCC_ICSCR_HSITRIM                                                  RCC_ICSCR_HSITRIM_Msk
#define RCC_ICSCR_HSITRIM_0                                                (0x1U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIM_1                                                (0x2U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIM_2                                                (0x4U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIM_3                                                (0x8U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIM_4                                                (0x10U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIM_5                                                (0x20U << RCC_ICSCR_HSITRIM_Pos)
#define RCC_ICSCR_HSITRIMOFS_Pos                                           (16UL)		/*!<RCC ICSCR: HSITRIMOFS (Bit 16) */
#define RCC_ICSCR_HSITRIMOFS_Msk                                           (0x70000UL)		/*!< RCC ICSCR: HSITRIMOFS (Bitfield-Mask: 0x07) */
#define RCC_ICSCR_HSITRIMOFS                                               RCC_ICSCR_HSITRIMOFS_Msk
#define RCC_ICSCR_HSITRIMOFS_0                                             (0x1U << RCC_ICSCR_HSITRIMOFS_Pos)
#define RCC_ICSCR_HSITRIMOFS_1                                             (0x2U << RCC_ICSCR_HSITRIMOFS_Pos)
#define RCC_ICSCR_HSITRIMOFS_2                                             (0x4U << RCC_ICSCR_HSITRIMOFS_Pos)
#define RCC_ICSCR_LSIBW_Pos                                                (2UL)		/*!<RCC ICSCR: LSIBW (Bit 2) */
#define RCC_ICSCR_LSIBW_Msk                                                (0x3cUL)		/*!< RCC ICSCR: LSIBW (Bitfield-Mask: 0x0f) */
#define RCC_ICSCR_LSIBW                                                    RCC_ICSCR_LSIBW_Msk
#define RCC_ICSCR_LSIBW_0                                                  (0x1U << RCC_ICSCR_LSIBW_Pos)
#define RCC_ICSCR_LSIBW_1                                                  (0x2U << RCC_ICSCR_LSIBW_Pos)
#define RCC_ICSCR_LSIBW_2                                                  (0x4U << RCC_ICSCR_LSIBW_Pos)
#define RCC_ICSCR_LSIBW_3                                                  (0x8U << RCC_ICSCR_LSIBW_Pos)
#define RCC_ICSCR_LSITRIMOK_Pos                                            (1UL)		/*!<RCC ICSCR: LSITRIMOK (Bit 1) */
#define RCC_ICSCR_LSITRIMOK_Msk                                            (0x2UL)		/*!< RCC ICSCR: LSITRIMOK (Bitfield-Mask: 0x01) */
#define RCC_ICSCR_LSITRIMOK                                                RCC_ICSCR_LSITRIMOK_Msk
#define RCC_ICSCR_LSITRIMEN_Pos                                            (0UL)		/*!<RCC ICSCR: LSITRIMEN (Bit 0) */
#define RCC_ICSCR_LSITRIMEN_Msk                                            (0x1UL)		/*!< RCC ICSCR: LSITRIMEN (Bitfield-Mask: 0x01) */
#define RCC_ICSCR_LSITRIMEN                                                RCC_ICSCR_LSITRIMEN_Msk

/* =====================================================    CFGR    =====================================================*/ 
#define RCC_CFGR_CCOPRE_Pos                                                (29UL)		/*!<RCC CFGR: CCOPRE (Bit 29) */
#define RCC_CFGR_CCOPRE_Msk                                                (0xe0000000UL)		/*!< RCC CFGR: CCOPRE (Bitfield-Mask: 0x07) */
#define RCC_CFGR_CCOPRE                                                    RCC_CFGR_CCOPRE_Msk
#define RCC_CFGR_CCOPRE_0                                                  (0x1U << RCC_CFGR_CCOPRE_Pos)
#define RCC_CFGR_CCOPRE_1                                                  (0x2U << RCC_CFGR_CCOPRE_Pos)
#define RCC_CFGR_CCOPRE_2                                                  (0x4U << RCC_CFGR_CCOPRE_Pos)
#define RCC_CFGR_MCOSEL_Pos                                                (26UL)		/*!<RCC CFGR: MCOSEL (Bit 26) */
#define RCC_CFGR_MCOSEL_Msk                                                (0x1c000000UL)		/*!< RCC CFGR: MCOSEL (Bitfield-Mask: 0x07) */
#define RCC_CFGR_MCOSEL                                                    RCC_CFGR_MCOSEL_Msk
#define RCC_CFGR_MCOSEL_0                                                  (0x1U << RCC_CFGR_MCOSEL_Pos)
#define RCC_CFGR_MCOSEL_1                                                  (0x2U << RCC_CFGR_MCOSEL_Pos)
#define RCC_CFGR_MCOSEL_2                                                  (0x4U << RCC_CFGR_MCOSEL_Pos)
#define RCC_CFGR_LCOSEL_Pos                                                (24UL)		/*!<RCC CFGR: LCOSEL (Bit 24) */
#define RCC_CFGR_LCOSEL_Msk                                                (0x3000000UL)		/*!< RCC CFGR: LCOSEL (Bitfield-Mask: 0x03) */
#define RCC_CFGR_LCOSEL                                                    RCC_CFGR_LCOSEL_Msk
#define RCC_CFGR_LCOSEL_0                                                  (0x1U << RCC_CFGR_LCOSEL_Pos)
#define RCC_CFGR_LCOSEL_1                                                  (0x2U << RCC_CFGR_LCOSEL_Pos)
#define RCC_CFGR_SPI3I2SCLKSEL_Pos                                         (22UL)		/*!<RCC CFGR: SPI3I2SCLKSEL (Bit 22) */
#define RCC_CFGR_SPI3I2SCLKSEL_Msk                                         (0xc00000UL)		/*!< RCC CFGR: SPI3I2SCLKSEL (Bitfield-Mask: 0x03) */
#define RCC_CFGR_SPI3I2SCLKSEL                                             RCC_CFGR_SPI3I2SCLKSEL_Msk
#define RCC_CFGR_SPI3I2SCLKSEL_0                                           (0x1U << RCC_CFGR_SPI3I2SCLKSEL_Pos)
#define RCC_CFGR_SPI3I2SCLKSEL_1                                           (0x2U << RCC_CFGR_SPI3I2SCLKSEL_Pos)
#define RCC_CFGR_LCOEN_Pos                                                 (19UL)		/*!<RCC CFGR: LCOEN (Bit 19) */
#define RCC_CFGR_LCOEN_Msk                                                 (0x80000UL)		/*!< RCC CFGR: LCOEN (Bitfield-Mask: 0x01) */
#define RCC_CFGR_LCOEN                                                     RCC_CFGR_LCOEN_Msk
#define RCC_CFGR_IOBOOSTEN_Pos                                             (17UL)		/*!<RCC CFGR: IOBOOSTEN (Bit 17) */
#define RCC_CFGR_IOBOOSTEN_Msk                                             (0x20000UL)		/*!< RCC CFGR: IOBOOSTEN (Bitfield-Mask: 0x01) */
#define RCC_CFGR_IOBOOSTEN                                                 RCC_CFGR_IOBOOSTEN_Msk
#define RCC_CFGR_CLKSLOWSEL_Pos                                            (15UL)		/*!<RCC CFGR: CLKSLOWSEL (Bit 15) */
#define RCC_CFGR_CLKSLOWSEL_Msk                                            (0x18000UL)		/*!< RCC CFGR: CLKSLOWSEL (Bitfield-Mask: 0x03) */
#define RCC_CFGR_CLKSLOWSEL                                                RCC_CFGR_CLKSLOWSEL_Msk
#define RCC_CFGR_CLKSLOWSEL_0                                              (0x1U << RCC_CFGR_CLKSLOWSEL_Pos)
#define RCC_CFGR_CLKSLOWSEL_1                                              (0x2U << RCC_CFGR_CLKSLOWSEL_Pos)
#define RCC_CFGR_LPUCLKSEL_Pos                                             (13UL)		/*!<RCC CFGR: LPUCLKSEL (Bit 13) */
#define RCC_CFGR_LPUCLKSEL_Msk                                             (0x2000UL)		/*!< RCC CFGR: LPUCLKSEL (Bitfield-Mask: 0x01) */
#define RCC_CFGR_LPUCLKSEL                                                 RCC_CFGR_LPUCLKSEL_Msk
#define RCC_CFGR_SMPSDIV_Pos                                               (12UL)		/*!<RCC CFGR: SMPSDIV (Bit 12) */
#define RCC_CFGR_SMPSDIV_Msk                                               (0x1000UL)		/*!< RCC CFGR: SMPSDIV (Bitfield-Mask: 0x01) */
#define RCC_CFGR_SMPSDIV                                                   RCC_CFGR_SMPSDIV_Msk
#define RCC_CFGR_ADCDIV_Pos                                                (11UL)		/*!<RCC CFGR: ADCDIV (Bit 11) */
#define RCC_CFGR_ADCDIV_Msk                                                (0x800UL)		/*!< RCC CFGR: ADCDIV (Bitfield-Mask: 0x01) */
#define RCC_CFGR_ADCDIV                                                    RCC_CFGR_ADCDIV_Msk
#define RCC_CFGR_CLKSYSDIV_STATUS_Pos                                      (8UL)		/*!<RCC CFGR: CLKSYSDIV_STATUS (Bit 8) */
#define RCC_CFGR_CLKSYSDIV_STATUS_Msk                                      (0x700UL)		/*!< RCC CFGR: CLKSYSDIV_STATUS (Bitfield-Mask: 0x07) */
#define RCC_CFGR_CLKSYSDIV_STATUS                                          RCC_CFGR_CLKSYSDIV_STATUS_Msk
#define RCC_CFGR_CLKSYSDIV_STATUS_0                                        (0x1U << RCC_CFGR_CLKSYSDIV_STATUS_Pos)
#define RCC_CFGR_CLKSYSDIV_STATUS_1                                        (0x2U << RCC_CFGR_CLKSYSDIV_STATUS_Pos)
#define RCC_CFGR_CLKSYSDIV_STATUS_2                                        (0x4U << RCC_CFGR_CLKSYSDIV_STATUS_Pos)
#define RCC_CFGR_CLKSYSDIV_Pos                                             (5UL)		/*!<RCC CFGR: CLKSYSDIV (Bit 5) */
#define RCC_CFGR_CLKSYSDIV_Msk                                             (0xe0UL)		/*!< RCC CFGR: CLKSYSDIV (Bitfield-Mask: 0x07) */
#define RCC_CFGR_CLKSYSDIV                                                 RCC_CFGR_CLKSYSDIV_Msk
#define RCC_CFGR_CLKSYSDIV_0                                               (0x1U << RCC_CFGR_CLKSYSDIV_Pos)
#define RCC_CFGR_CLKSYSDIV_1                                               (0x2U << RCC_CFGR_CLKSYSDIV_Pos)
#define RCC_CFGR_CLKSYSDIV_2                                               (0x4U << RCC_CFGR_CLKSYSDIV_Pos)
#define RCC_CFGR_HSESEL_STATUS_Pos                                         (3UL)		/*!<RCC CFGR: HSESEL_STATUS (Bit 3) */
#define RCC_CFGR_HSESEL_STATUS_Msk                                         (0x8UL)		/*!< RCC CFGR: HSESEL_STATUS (Bitfield-Mask: 0x01) */
#define RCC_CFGR_HSESEL_STATUS                                             RCC_CFGR_HSESEL_STATUS_Msk
#define RCC_CFGR_STOPHSI_Pos                                               (2UL)		/*!<RCC CFGR: STOPHSI (Bit 2) */
#define RCC_CFGR_STOPHSI_Msk                                               (0x4UL)		/*!< RCC CFGR: STOPHSI (Bitfield-Mask: 0x01) */
#define RCC_CFGR_STOPHSI                                                   RCC_CFGR_STOPHSI_Msk
#define RCC_CFGR_HSESEL_Pos                                                (1UL)		/*!<RCC CFGR: HSESEL (Bit 1) */
#define RCC_CFGR_HSESEL_Msk                                                (0x2UL)		/*!< RCC CFGR: HSESEL (Bitfield-Mask: 0x01) */
#define RCC_CFGR_HSESEL                                                    RCC_CFGR_HSESEL_Msk

/* =====================================================    CSSWCR    =====================================================*/ 
#define RCC_CSSWCR_HSITRIMSW_Pos                                           (24UL)		/*!<RCC CSSWCR: HSITRIMSW (Bit 24) */
#define RCC_CSSWCR_HSITRIMSW_Msk                                           (0x3f000000UL)		/*!< RCC CSSWCR: HSITRIMSW (Bitfield-Mask: 0x3f) */
#define RCC_CSSWCR_HSITRIMSW                                               RCC_CSSWCR_HSITRIMSW_Msk
#define RCC_CSSWCR_HSITRIMSW_0                                             (0x1U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSITRIMSW_1                                             (0x2U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSITRIMSW_2                                             (0x4U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSITRIMSW_3                                             (0x8U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSITRIMSW_4                                             (0x10U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSITRIMSW_5                                             (0x20U << RCC_CSSWCR_HSITRIMSW_Pos)
#define RCC_CSSWCR_HSISWTRIMEN_Pos                                         (23UL)		/*!<RCC CSSWCR: HSISWTRIMEN (Bit 23) */
#define RCC_CSSWCR_HSISWTRIMEN_Msk                                         (0x800000UL)		/*!< RCC CSSWCR: HSISWTRIMEN (Bitfield-Mask: 0x01) */
#define RCC_CSSWCR_HSISWTRIMEN                                             RCC_CSSWCR_HSISWTRIMEN_Msk
#define RCC_CSSWCR_LSEDRV_Pos                                              (5UL)		/*!<RCC CSSWCR: LSEDRV (Bit 5) */
#define RCC_CSSWCR_LSEDRV_Msk                                              (0x60UL)		/*!< RCC CSSWCR: LSEDRV (Bitfield-Mask: 0x03) */
#define RCC_CSSWCR_LSEDRV                                                  RCC_CSSWCR_LSEDRV_Msk
#define RCC_CSSWCR_LSEDRV_0                                                (0x1U << RCC_CSSWCR_LSEDRV_Pos)
#define RCC_CSSWCR_LSEDRV_1                                                (0x2U << RCC_CSSWCR_LSEDRV_Pos)
#define RCC_CSSWCR_LSISWBW_Pos                                             (1UL)		/*!<RCC CSSWCR: LSISWBW (Bit 1) */
#define RCC_CSSWCR_LSISWBW_Msk                                             (0x1eUL)		/*!< RCC CSSWCR: LSISWBW (Bitfield-Mask: 0x0f) */
#define RCC_CSSWCR_LSISWBW                                                 RCC_CSSWCR_LSISWBW_Msk
#define RCC_CSSWCR_LSISWBW_0                                               (0x1U << RCC_CSSWCR_LSISWBW_Pos)
#define RCC_CSSWCR_LSISWBW_1                                               (0x2U << RCC_CSSWCR_LSISWBW_Pos)
#define RCC_CSSWCR_LSISWBW_2                                               (0x4U << RCC_CSSWCR_LSISWBW_Pos)
#define RCC_CSSWCR_LSISWBW_3                                               (0x8U << RCC_CSSWCR_LSISWBW_Pos)
#define RCC_CSSWCR_LSISWTRIMEN_Pos                                         (0UL)		/*!<RCC CSSWCR: LSISWTRIMEN (Bit 0) */
#define RCC_CSSWCR_LSISWTRIMEN_Msk                                         (0x1UL)		/*!< RCC CSSWCR: LSISWTRIMEN (Bitfield-Mask: 0x01) */
#define RCC_CSSWCR_LSISWTRIMEN                                             RCC_CSSWCR_LSISWTRIMEN_Msk

/* =====================================================    KRMR    =====================================================*/ 
#define RCC_KRMR_KRM_Pos                                                   (1UL)		/*!<RCC KRMR: KRM (Bit 1) */
#define RCC_KRMR_KRM_Msk                                                   (0xfffeUL)		/*!< RCC KRMR: KRM (Bitfield-Mask: 0x7fff) */
#define RCC_KRMR_KRM                                                       RCC_KRMR_KRM_Msk
#define RCC_KRMR_KRM_0                                                     (0x1U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_1                                                     (0x2U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_2                                                     (0x4U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_3                                                     (0x8U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_4                                                     (0x10U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_5                                                     (0x20U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_6                                                     (0x40U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_7                                                     (0x80U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_8                                                     (0x100U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_9                                                     (0x200U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_10                                                    (0x400U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_11                                                    (0x800U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_12                                                    (0x1000U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_13                                                    (0x2000U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_14                                                    (0x4000U << RCC_KRMR_KRM_Pos)
#define RCC_KRMR_KRM_EN_Pos                                                (0UL)		/*!<RCC KRMR: KRM_EN (Bit 0) */
#define RCC_KRMR_KRM_EN_Msk                                                (0x1UL)		/*!< RCC KRMR: KRM_EN (Bitfield-Mask: 0x01) */
#define RCC_KRMR_KRM_EN                                                    RCC_KRMR_KRM_EN_Msk

/* =====================================================    CIER    =====================================================*/ 
#define RCC_CIER_LCSCRSTIE_Pos                                             (13UL)		/*!<RCC CIER: LCSCRSTIE (Bit 13) */
#define RCC_CIER_LCSCRSTIE_Msk                                             (0x2000UL)		/*!< RCC CIER: LCSCRSTIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_LCSCRSTIE                                                 RCC_CIER_LCSCRSTIE_Msk
#define RCC_CIER_LCDRSTIE_Pos                                              (10UL)		/*!<RCC CIER: LCDRSTIE (Bit 10) */
#define RCC_CIER_LCDRSTIE_Msk                                              (0x400UL)		/*!< RCC CIER: LCDRSTIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_LCDRSTIE                                                  RCC_CIER_LCDRSTIE_Msk
#define RCC_CIER_LPURSTIE_Pos                                              (9UL)		/*!<RCC CIER: LPURSTIE (Bit 9) */
#define RCC_CIER_LPURSTIE_Msk                                              (0x200UL)		/*!< RCC CIER: LPURSTIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_LPURSTIE                                                  RCC_CIER_LPURSTIE_Msk
#define RCC_CIER_WDGRSTIE_Pos                                              (8UL)		/*!<RCC CIER: WDGRSTIE (Bit 8) */
#define RCC_CIER_WDGRSTIE_Msk                                              (0x100UL)		/*!< RCC CIER: WDGRSTIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_WDGRSTIE                                                  RCC_CIER_WDGRSTIE_Msk
#define RCC_CIER_RTCRSTIE_Pos                                              (7UL)		/*!<RCC CIER: RTCRSTIE (Bit 7) */
#define RCC_CIER_RTCRSTIE_Msk                                              (0x80UL)		/*!< RCC CIER: RTCRSTIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_RTCRSTIE                                                  RCC_CIER_RTCRSTIE_Msk
#define RCC_CIER_HSIPLLUNLOCKDETIE_Pos                                     (6UL)		/*!<RCC CIER: HSIPLLUNLOCKDETIE (Bit 6) */
#define RCC_CIER_HSIPLLUNLOCKDETIE_Msk                                     (0x40UL)		/*!< RCC CIER: HSIPLLUNLOCKDETIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_HSIPLLUNLOCKDETIE                                         RCC_CIER_HSIPLLUNLOCKDETIE_Msk
#define RCC_CIER_HSIPLLRDYIE_Pos                                           (5UL)		/*!<RCC CIER: HSIPLLRDYIE (Bit 5) */
#define RCC_CIER_HSIPLLRDYIE_Msk                                           (0x20UL)		/*!< RCC CIER: HSIPLLRDYIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_HSIPLLRDYIE                                               RCC_CIER_HSIPLLRDYIE_Msk
#define RCC_CIER_HSERDYIE_Pos                                              (4UL)		/*!<RCC CIER: HSERDYIE (Bit 4) */
#define RCC_CIER_HSERDYIE_Msk                                              (0x10UL)		/*!< RCC CIER: HSERDYIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_HSERDYIE                                                  RCC_CIER_HSERDYIE_Msk
#define RCC_CIER_HSIRDYIE_Pos                                              (3UL)		/*!<RCC CIER: HSIRDYIE (Bit 3) */
#define RCC_CIER_HSIRDYIE_Msk                                              (0x8UL)		/*!< RCC CIER: HSIRDYIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_HSIRDYIE                                                  RCC_CIER_HSIRDYIE_Msk
#define RCC_CIER_LSERDYIE_Pos                                              (1UL)		/*!<RCC CIER: LSERDYIE (Bit 1) */
#define RCC_CIER_LSERDYIE_Msk                                              (0x2UL)		/*!< RCC CIER: LSERDYIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_LSERDYIE                                                  RCC_CIER_LSERDYIE_Msk
#define RCC_CIER_LSIRDYIE_Pos                                              (0UL)		/*!<RCC CIER: LSIRDYIE (Bit 0) */
#define RCC_CIER_LSIRDYIE_Msk                                              (0x1UL)		/*!< RCC CIER: LSIRDYIE (Bitfield-Mask: 0x01) */
#define RCC_CIER_LSIRDYIE                                                  RCC_CIER_LSIRDYIE_Msk

/* =====================================================    CIFR    =====================================================*/ 
#define RCC_CIFR_LCSCRSTF_Pos                                              (13UL)		/*!<RCC CIFR: LCSCRSTF (Bit 13) */
#define RCC_CIFR_LCSCRSTF_Msk                                              (0x2000UL)		/*!< RCC CIFR: LCSCRSTF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_LCSCRSTF                                                  RCC_CIFR_LCSCRSTF_Msk
#define RCC_CIFR_LCDRSTF_Pos                                               (10UL)		/*!<RCC CIFR: LCDRSTF (Bit 10) */
#define RCC_CIFR_LCDRSTF_Msk                                               (0x400UL)		/*!< RCC CIFR: LCDRSTF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_LCDRSTF                                                   RCC_CIFR_LCDRSTF_Msk
#define RCC_CIFR_LPURSTF_Pos                                               (9UL)		/*!<RCC CIFR: LPURSTF (Bit 9) */
#define RCC_CIFR_LPURSTF_Msk                                               (0x200UL)		/*!< RCC CIFR: LPURSTF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_LPURSTF                                                   RCC_CIFR_LPURSTF_Msk
#define RCC_CIFR_WDGRSTF_Pos                                               (8UL)		/*!<RCC CIFR: WDGRSTF (Bit 8) */
#define RCC_CIFR_WDGRSTF_Msk                                               (0x100UL)		/*!< RCC CIFR: WDGRSTF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_WDGRSTF                                                   RCC_CIFR_WDGRSTF_Msk
#define RCC_CIFR_RTCRSTF_Pos                                               (7UL)		/*!<RCC CIFR: RTCRSTF (Bit 7) */
#define RCC_CIFR_RTCRSTF_Msk                                               (0x80UL)		/*!< RCC CIFR: RTCRSTF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_RTCRSTF                                                   RCC_CIFR_RTCRSTF_Msk
#define RCC_CIFR_HSIPLLUNLOCKDETF_Pos                                      (6UL)		/*!<RCC CIFR: HSIPLLUNLOCKDETF (Bit 6) */
#define RCC_CIFR_HSIPLLUNLOCKDETF_Msk                                      (0x40UL)		/*!< RCC CIFR: HSIPLLUNLOCKDETF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_HSIPLLUNLOCKDETF                                          RCC_CIFR_HSIPLLUNLOCKDETF_Msk
#define RCC_CIFR_HSIPLLRDYF_Pos                                            (5UL)		/*!<RCC CIFR: HSIPLLRDYF (Bit 5) */
#define RCC_CIFR_HSIPLLRDYF_Msk                                            (0x20UL)		/*!< RCC CIFR: HSIPLLRDYF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_HSIPLLRDYF                                                RCC_CIFR_HSIPLLRDYF_Msk
#define RCC_CIFR_HSERDYF_Pos                                               (4UL)		/*!<RCC CIFR: HSERDYF (Bit 4) */
#define RCC_CIFR_HSERDYF_Msk                                               (0x10UL)		/*!< RCC CIFR: HSERDYF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_HSERDYF                                                   RCC_CIFR_HSERDYF_Msk
#define RCC_CIFR_HSIRDYF_Pos                                               (3UL)		/*!<RCC CIFR: HSIRDYF (Bit 3) */
#define RCC_CIFR_HSIRDYF_Msk                                               (0x8UL)		/*!< RCC CIFR: HSIRDYF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_HSIRDYF                                                   RCC_CIFR_HSIRDYF_Msk
#define RCC_CIFR_LSERDYF_Pos                                               (1UL)		/*!<RCC CIFR: LSERDYF (Bit 1) */
#define RCC_CIFR_LSERDYF_Msk                                               (0x2UL)		/*!< RCC CIFR: LSERDYF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_LSERDYF                                                   RCC_CIFR_LSERDYF_Msk
#define RCC_CIFR_LSIRDYF_Pos                                               (0UL)		/*!<RCC CIFR: LSIRDYF (Bit 0) */
#define RCC_CIFR_LSIRDYF_Msk                                               (0x1UL)		/*!< RCC CIFR: LSIRDYF (Bitfield-Mask: 0x01) */
#define RCC_CIFR_LSIRDYF                                                   RCC_CIFR_LSIRDYF_Msk

/* =====================================================    CSCMDR    =====================================================*/ 
#define RCC_CSCMDR_EOFSEQ_IRQ_Pos                                          (7UL)		/*!<RCC CSCMDR: EOFSEQ_IRQ (Bit 7) */
#define RCC_CSCMDR_EOFSEQ_IRQ_Msk                                          (0x80UL)		/*!< RCC CSCMDR: EOFSEQ_IRQ (Bitfield-Mask: 0x01) */
#define RCC_CSCMDR_EOFSEQ_IRQ                                              RCC_CSCMDR_EOFSEQ_IRQ_Msk
#define RCC_CSCMDR_EOFSEQ_IE_Pos                                           (6UL)		/*!<RCC CSCMDR: EOFSEQ_IE (Bit 6) */
#define RCC_CSCMDR_EOFSEQ_IE_Msk                                           (0x40UL)		/*!< RCC CSCMDR: EOFSEQ_IE (Bitfield-Mask: 0x01) */
#define RCC_CSCMDR_EOFSEQ_IE                                               RCC_CSCMDR_EOFSEQ_IE_Msk
#define RCC_CSCMDR_STATUS_Pos                                              (4UL)		/*!<RCC CSCMDR: STATUS (Bit 4) */
#define RCC_CSCMDR_STATUS_Msk                                              (0x30UL)		/*!< RCC CSCMDR: STATUS (Bitfield-Mask: 0x03) */
#define RCC_CSCMDR_STATUS                                                  RCC_CSCMDR_STATUS_Msk
#define RCC_CSCMDR_STATUS_0                                                (0x1U << RCC_CSCMDR_STATUS_Pos)
#define RCC_CSCMDR_STATUS_1                                                (0x2U << RCC_CSCMDR_STATUS_Pos)
#define RCC_CSCMDR_CLKSYSDIV_REQ_Pos                                       (1UL)		/*!<RCC CSCMDR: CLKSYSDIV_REQ (Bit 1) */
#define RCC_CSCMDR_CLKSYSDIV_REQ_Msk                                       (0xeUL)		/*!< RCC CSCMDR: CLKSYSDIV_REQ (Bitfield-Mask: 0x07) */
#define RCC_CSCMDR_CLKSYSDIV_REQ                                           RCC_CSCMDR_CLKSYSDIV_REQ_Msk
#define RCC_CSCMDR_CLKSYSDIV_REQ_0                                         (0x1U << RCC_CSCMDR_CLKSYSDIV_REQ_Pos)
#define RCC_CSCMDR_CLKSYSDIV_REQ_1                                         (0x2U << RCC_CSCMDR_CLKSYSDIV_REQ_Pos)
#define RCC_CSCMDR_CLKSYSDIV_REQ_2                                         (0x4U << RCC_CSCMDR_CLKSYSDIV_REQ_Pos)
#define RCC_CSCMDR_REQUEST_Pos                                             (0UL)		/*!<RCC CSCMDR: REQUEST (Bit 0) */
#define RCC_CSCMDR_REQUEST_Msk                                             (0x1UL)		/*!< RCC CSCMDR: REQUEST (Bitfield-Mask: 0x01) */
#define RCC_CSCMDR_REQUEST                                                 RCC_CSCMDR_REQUEST_Msk

/* =====================================================    AHBRSTR    =====================================================*/ 
#define RCC_AHBRSTR_AESRST_Pos                                             (20UL)		/*!<RCC AHBRSTR: AESRST (Bit 20) */
#define RCC_AHBRSTR_AESRST_Msk                                             (0x100000UL)		/*!< RCC AHBRSTR: AESRST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_AESRST                                                 RCC_AHBRSTR_AESRST_Msk
#define RCC_AHBRSTR_RNGRST_Pos                                             (18UL)		/*!<RCC AHBRSTR: RNGRST (Bit 18) */
#define RCC_AHBRSTR_RNGRST_Msk                                             (0x40000UL)		/*!< RCC AHBRSTR: RNGRST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_RNGRST                                                 RCC_AHBRSTR_RNGRST_Msk
#define RCC_AHBRSTR_CRCRST_Pos                                             (12UL)		/*!<RCC AHBRSTR: CRCRST (Bit 12) */
#define RCC_AHBRSTR_CRCRST_Msk                                             (0x1000UL)		/*!< RCC AHBRSTR: CRCRST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_CRCRST                                                 RCC_AHBRSTR_CRCRST_Msk
#define RCC_AHBRSTR_GPIOBRST_Pos                                           (3UL)		/*!<RCC AHBRSTR: GPIOBRST (Bit 3) */
#define RCC_AHBRSTR_GPIOBRST_Msk                                           (0x8UL)		/*!< RCC AHBRSTR: GPIOBRST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_GPIOBRST                                               RCC_AHBRSTR_GPIOBRST_Msk
#define RCC_AHBRSTR_GPIOARST_Pos                                           (2UL)		/*!<RCC AHBRSTR: GPIOARST (Bit 2) */
#define RCC_AHBRSTR_GPIOARST_Msk                                           (0x4UL)		/*!< RCC AHBRSTR: GPIOARST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_GPIOARST                                               RCC_AHBRSTR_GPIOARST_Msk
#define RCC_AHBRSTR_DMARST_Pos                                             (0UL)		/*!<RCC AHBRSTR: DMARST (Bit 0) */
#define RCC_AHBRSTR_DMARST_Msk                                             (0x1UL)		/*!< RCC AHBRSTR: DMARST (Bitfield-Mask: 0x01) */
#define RCC_AHBRSTR_DMARST                                                 RCC_AHBRSTR_DMARST_Msk

/* =====================================================    APB0RSTR    =====================================================*/ 
#define RCC_APB0RSTR_DBGMCURST_Pos                                         (15UL)		/*!<RCC APB0RSTR: DBGMCURST (Bit 15) */
#define RCC_APB0RSTR_DBGMCURST_Msk                                         (0x8000UL)		/*!< RCC APB0RSTR: DBGMCURST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_DBGMCURST                                             RCC_APB0RSTR_DBGMCURST_Msk
#define RCC_APB0RSTR_WDGRST_Pos                                            (14UL)		/*!<RCC APB0RSTR: WDGRST (Bit 14) */
#define RCC_APB0RSTR_WDGRST_Msk                                            (0x4000UL)		/*!< RCC APB0RSTR: WDGRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_WDGRST                                                RCC_APB0RSTR_WDGRST_Msk
#define RCC_APB0RSTR_LCSCRST_Pos                                           (13UL)		/*!<RCC APB0RSTR: LCSCRST (Bit 13) */
#define RCC_APB0RSTR_LCSCRST_Msk                                           (0x2000UL)		/*!< RCC APB0RSTR: LCSCRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_LCSCRST                                               RCC_APB0RSTR_LCSCRST_Msk
#define RCC_APB0RSTR_RTCRST_Pos                                            (12UL)		/*!<RCC APB0RSTR: RTCRST (Bit 12) */
#define RCC_APB0RSTR_RTCRST_Msk                                            (0x1000UL)		/*!< RCC APB0RSTR: RTCRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_RTCRST                                                RCC_APB0RSTR_RTCRST_Msk
#define RCC_APB0RSTR_DACRST_Pos                                            (11UL)		/*!<RCC APB0RSTR: DACRST (Bit 11) */
#define RCC_APB0RSTR_DACRST_Msk                                            (0x800UL)		/*!< RCC APB0RSTR: DACRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_DACRST                                                RCC_APB0RSTR_DACRST_Msk
#define RCC_APB0RSTR_COMPRST_Pos                                           (10UL)		/*!<RCC APB0RSTR: COMPRST (Bit 10) */
#define RCC_APB0RSTR_COMPRST_Msk                                           (0x400UL)		/*!< RCC APB0RSTR: COMPRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_COMPRST                                               RCC_APB0RSTR_COMPRST_Msk
#define RCC_APB0RSTR_LCDCRST_Pos                                           (9UL)		/*!<RCC APB0RSTR: LCDCRST (Bit 9) */
#define RCC_APB0RSTR_LCDCRST_Msk                                           (0x200UL)		/*!< RCC APB0RSTR: LCDCRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_LCDCRST                                               RCC_APB0RSTR_LCDCRST_Msk
#define RCC_APB0RSTR_SYSCFGRST_Pos                                         (8UL)		/*!<RCC APB0RSTR: SYSCFGRST (Bit 8) */
#define RCC_APB0RSTR_SYSCFGRST_Msk                                         (0x100UL)		/*!< RCC APB0RSTR: SYSCFGRST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_SYSCFGRST                                             RCC_APB0RSTR_SYSCFGRST_Msk
#define RCC_APB0RSTR_TIM16RST_Pos                                          (1UL)		/*!<RCC APB0RSTR: TIM16RST (Bit 1) */
#define RCC_APB0RSTR_TIM16RST_Msk                                          (0x2UL)		/*!< RCC APB0RSTR: TIM16RST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_TIM16RST                                              RCC_APB0RSTR_TIM16RST_Msk
#define RCC_APB0RSTR_TIM2RST_Pos                                           (0UL)		/*!<RCC APB0RSTR: TIM2RST (Bit 0) */
#define RCC_APB0RSTR_TIM2RST_Msk                                           (0x1UL)		/*!< RCC APB0RSTR: TIM2RST (Bitfield-Mask: 0x01) */
#define RCC_APB0RSTR_TIM2RST                                               RCC_APB0RSTR_TIM2RST_Msk

/* =====================================================    APB1RSTR    =====================================================*/ 
#define RCC_APB1RSTR_I2C2RST_Pos                                           (23UL)		/*!<RCC APB1RSTR: I2C2RST (Bit 23) */
#define RCC_APB1RSTR_I2C2RST_Msk                                           (0x800000UL)		/*!< RCC APB1RSTR: I2C2RST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_I2C2RST                                               RCC_APB1RSTR_I2C2RST_Msk
#define RCC_APB1RSTR_I2C1RST_Pos                                           (21UL)		/*!<RCC APB1RSTR: I2C1RST (Bit 21) */
#define RCC_APB1RSTR_I2C1RST_Msk                                           (0x200000UL)		/*!< RCC APB1RSTR: I2C1RST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_I2C1RST                                               RCC_APB1RSTR_I2C1RST_Msk
#define RCC_APB1RSTR_SPI3RST_Pos                                           (14UL)		/*!<RCC APB1RSTR: SPI3RST (Bit 14) */
#define RCC_APB1RSTR_SPI3RST_Msk                                           (0x4000UL)		/*!< RCC APB1RSTR: SPI3RST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_SPI3RST                                               RCC_APB1RSTR_SPI3RST_Msk
#define RCC_APB1RSTR_USARTRST_Pos                                          (10UL)		/*!<RCC APB1RSTR: USARTRST (Bit 10) */
#define RCC_APB1RSTR_USARTRST_Msk                                          (0x400UL)		/*!< RCC APB1RSTR: USARTRST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_USARTRST                                              RCC_APB1RSTR_USARTRST_Msk
#define RCC_APB1RSTR_LPUARTRST_Pos                                         (8UL)		/*!<RCC APB1RSTR: LPUARTRST (Bit 8) */
#define RCC_APB1RSTR_LPUARTRST_Msk                                         (0x100UL)		/*!< RCC APB1RSTR: LPUARTRST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_LPUARTRST                                             RCC_APB1RSTR_LPUARTRST_Msk
#define RCC_APB1RSTR_ADCRST_Pos                                            (4UL)		/*!<RCC APB1RSTR: ADCRST (Bit 4) */
#define RCC_APB1RSTR_ADCRST_Msk                                            (0x10UL)		/*!< RCC APB1RSTR: ADCRST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_ADCRST                                                RCC_APB1RSTR_ADCRST_Msk
#define RCC_APB1RSTR_SPI1RST_Pos                                           (0UL)		/*!<RCC APB1RSTR: SPI1RST (Bit 0) */
#define RCC_APB1RSTR_SPI1RST_Msk                                           (0x1UL)		/*!< RCC APB1RSTR: SPI1RST (Bitfield-Mask: 0x01) */
#define RCC_APB1RSTR_SPI1RST                                               RCC_APB1RSTR_SPI1RST_Msk

/* =====================================================    APB2RSTR    =====================================================*/ 
#define RCC_APB2RSTR_LPAWURRST_Pos                                         (3UL)		/*!<RCC APB2RSTR: LPAWURRST (Bit 3) */
#define RCC_APB2RSTR_LPAWURRST_Msk                                         (0x8UL)		/*!< RCC APB2RSTR: LPAWURRST (Bitfield-Mask: 0x01) */
#define RCC_APB2RSTR_LPAWURRST                                             RCC_APB2RSTR_LPAWURRST_Msk
#define RCC_APB2RSTR_MRSUBGRST_Pos                                         (0UL)		/*!<RCC APB2RSTR: MRSUBGRST (Bit 0) */
#define RCC_APB2RSTR_MRSUBGRST_Msk                                         (0x1UL)		/*!< RCC APB2RSTR: MRSUBGRST (Bitfield-Mask: 0x01) */
#define RCC_APB2RSTR_MRSUBGRST                                             RCC_APB2RSTR_MRSUBGRST_Msk

/* =====================================================    AHBENR    =====================================================*/ 
#define RCC_AHBENR_AESEN_Pos                                               (20UL)		/*!<RCC AHBENR: AESEN (Bit 20) */
#define RCC_AHBENR_AESEN_Msk                                               (0x100000UL)		/*!< RCC AHBENR: AESEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_AESEN                                                   RCC_AHBENR_AESEN_Msk
#define RCC_AHBENR_RNGEN_Pos                                               (18UL)		/*!<RCC AHBENR: RNGEN (Bit 18) */
#define RCC_AHBENR_RNGEN_Msk                                               (0x40000UL)		/*!< RCC AHBENR: RNGEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_RNGEN                                                   RCC_AHBENR_RNGEN_Msk
#define RCC_AHBENR_CRCEN_Pos                                               (12UL)		/*!<RCC AHBENR: CRCEN (Bit 12) */
#define RCC_AHBENR_CRCEN_Msk                                               (0x1000UL)		/*!< RCC AHBENR: CRCEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_CRCEN                                                   RCC_AHBENR_CRCEN_Msk
#define RCC_AHBENR_GPIOBEN_Pos                                             (3UL)		/*!<RCC AHBENR: GPIOBEN (Bit 3) */
#define RCC_AHBENR_GPIOBEN_Msk                                             (0x8UL)		/*!< RCC AHBENR: GPIOBEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_GPIOBEN                                                 RCC_AHBENR_GPIOBEN_Msk
#define RCC_AHBENR_GPIOAEN_Pos                                             (2UL)		/*!<RCC AHBENR: GPIOAEN (Bit 2) */
#define RCC_AHBENR_GPIOAEN_Msk                                             (0x4UL)		/*!< RCC AHBENR: GPIOAEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_GPIOAEN                                                 RCC_AHBENR_GPIOAEN_Msk
#define RCC_AHBENR_DMAEN_Pos                                               (0UL)		/*!<RCC AHBENR: DMAEN (Bit 0) */
#define RCC_AHBENR_DMAEN_Msk                                               (0x1UL)		/*!< RCC AHBENR: DMAEN (Bitfield-Mask: 0x01) */
#define RCC_AHBENR_DMAEN                                                   RCC_AHBENR_DMAEN_Msk

/* =====================================================    APB0ENR    =====================================================*/ 
#define RCC_APB0ENR_DBGMCUEN_Pos                                           (15UL)		/*!<RCC APB0ENR: DBGMCUEN (Bit 15) */
#define RCC_APB0ENR_DBGMCUEN_Msk                                           (0x8000UL)		/*!< RCC APB0ENR: DBGMCUEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_DBGMCUEN                                               RCC_APB0ENR_DBGMCUEN_Msk
#define RCC_APB0ENR_WDGEN_Pos                                              (14UL)		/*!<RCC APB0ENR: WDGEN (Bit 14) */
#define RCC_APB0ENR_WDGEN_Msk                                              (0x4000UL)		/*!< RCC APB0ENR: WDGEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_WDGEN                                                  RCC_APB0ENR_WDGEN_Msk
#define RCC_APB0ENR_LCSCEN_Pos                                             (13UL)		/*!<RCC APB0ENR: LCSCEN (Bit 13) */
#define RCC_APB0ENR_LCSCEN_Msk                                             (0x2000UL)		/*!< RCC APB0ENR: LCSCEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_LCSCEN                                                 RCC_APB0ENR_LCSCEN_Msk
#define RCC_APB0ENR_RTCEN_Pos                                              (12UL)		/*!<RCC APB0ENR: RTCEN (Bit 12) */
#define RCC_APB0ENR_RTCEN_Msk                                              (0x1000UL)		/*!< RCC APB0ENR: RTCEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_RTCEN                                                  RCC_APB0ENR_RTCEN_Msk
#define RCC_APB0ENR_DACEN_Pos                                              (11UL)		/*!<RCC APB0ENR: DACEN (Bit 11) */
#define RCC_APB0ENR_DACEN_Msk                                              (0x800UL)		/*!< RCC APB0ENR: DACEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_DACEN                                                  RCC_APB0ENR_DACEN_Msk
#define RCC_APB0ENR_COMPEN_Pos                                             (10UL)		/*!<RCC APB0ENR: COMPEN (Bit 10) */
#define RCC_APB0ENR_COMPEN_Msk                                             (0x400UL)		/*!< RCC APB0ENR: COMPEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_COMPEN                                                 RCC_APB0ENR_COMPEN_Msk
#define RCC_APB0ENR_LCDEN_Pos                                              (9UL)		/*!<RCC APB0ENR: LCDEN (Bit 9) */
#define RCC_APB0ENR_LCDEN_Msk                                              (0x200UL)		/*!< RCC APB0ENR: LCDEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_LCDEN                                                  RCC_APB0ENR_LCDEN_Msk
#define RCC_APB0ENR_SYSCFGEN_Pos                                           (8UL)		/*!<RCC APB0ENR: SYSCFGEN (Bit 8) */
#define RCC_APB0ENR_SYSCFGEN_Msk                                           (0x100UL)		/*!< RCC APB0ENR: SYSCFGEN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_SYSCFGEN                                               RCC_APB0ENR_SYSCFGEN_Msk
#define RCC_APB0ENR_TIM16EN_Pos                                            (1UL)		/*!<RCC APB0ENR: TIM16EN (Bit 1) */
#define RCC_APB0ENR_TIM16EN_Msk                                            (0x2UL)		/*!< RCC APB0ENR: TIM16EN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_TIM16EN                                                RCC_APB0ENR_TIM16EN_Msk
#define RCC_APB0ENR_TIM2EN_Pos                                             (0UL)		/*!<RCC APB0ENR: TIM2EN (Bit 0) */
#define RCC_APB0ENR_TIM2EN_Msk                                             (0x1UL)		/*!< RCC APB0ENR: TIM2EN (Bitfield-Mask: 0x01) */
#define RCC_APB0ENR_TIM2EN                                                 RCC_APB0ENR_TIM2EN_Msk

/* =====================================================    APB1ENR    =====================================================*/ 
#define RCC_APB1ENR_I2C2EN_Pos                                             (23UL)		/*!<RCC APB1ENR: I2C2EN (Bit 23) */
#define RCC_APB1ENR_I2C2EN_Msk                                             (0x800000UL)		/*!< RCC APB1ENR: I2C2EN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_I2C2EN                                                 RCC_APB1ENR_I2C2EN_Msk
#define RCC_APB1ENR_I2C1EN_Pos                                             (21UL)		/*!<RCC APB1ENR: I2C1EN (Bit 21) */
#define RCC_APB1ENR_I2C1EN_Msk                                             (0x200000UL)		/*!< RCC APB1ENR: I2C1EN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_I2C1EN                                                 RCC_APB1ENR_I2C1EN_Msk
#define RCC_APB1ENR_SPI3EN_Pos                                             (14UL)		/*!<RCC APB1ENR: SPI3EN (Bit 14) */
#define RCC_APB1ENR_SPI3EN_Msk                                             (0x4000UL)		/*!< RCC APB1ENR: SPI3EN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_SPI3EN                                                 RCC_APB1ENR_SPI3EN_Msk
#define RCC_APB1ENR_USARTEN_Pos                                            (10UL)		/*!<RCC APB1ENR: USARTEN (Bit 10) */
#define RCC_APB1ENR_USARTEN_Msk                                            (0x400UL)		/*!< RCC APB1ENR: USARTEN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_USARTEN                                                RCC_APB1ENR_USARTEN_Msk
#define RCC_APB1ENR_LPUARTEN_Pos                                           (8UL)		/*!<RCC APB1ENR: LPUARTEN (Bit 8) */
#define RCC_APB1ENR_LPUARTEN_Msk                                           (0x100UL)		/*!< RCC APB1ENR: LPUARTEN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_LPUARTEN                                               RCC_APB1ENR_LPUARTEN_Msk
#define RCC_APB1ENR_ADCANAEN_Pos                                           (5UL)		/*!<RCC APB1ENR: ADCANAEN (Bit 5) */
#define RCC_APB1ENR_ADCANAEN_Msk                                           (0x20UL)		/*!< RCC APB1ENR: ADCANAEN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_ADCANAEN                                               RCC_APB1ENR_ADCANAEN_Msk
#define RCC_APB1ENR_ADCDIGEN_Pos                                           (4UL)		/*!<RCC APB1ENR: ADCDIGEN (Bit 4) */
#define RCC_APB1ENR_ADCDIGEN_Msk                                           (0x10UL)		/*!< RCC APB1ENR: ADCDIGEN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_ADCDIGEN                                               RCC_APB1ENR_ADCDIGEN_Msk
#define RCC_APB1ENR_SPI1EN_Pos                                             (0UL)		/*!<RCC APB1ENR: SPI1EN (Bit 0) */
#define RCC_APB1ENR_SPI1EN_Msk                                             (0x1UL)		/*!< RCC APB1ENR: SPI1EN (Bitfield-Mask: 0x01) */
#define RCC_APB1ENR_SPI1EN                                                 RCC_APB1ENR_SPI1EN_Msk

/* =====================================================    APB2ENR    =====================================================*/ 
#define RCC_APB2ENR_LPAWUREN_Pos                                           (3UL)		/*!<RCC APB2ENR: LPAWUREN (Bit 3) */
#define RCC_APB2ENR_LPAWUREN_Msk                                           (0x8UL)		/*!< RCC APB2ENR: LPAWUREN (Bitfield-Mask: 0x01) */
#define RCC_APB2ENR_LPAWUREN                                               RCC_APB2ENR_LPAWUREN_Msk
#define RCC_APB2ENR_MRSUBGEN_Pos                                           (0UL)		/*!<RCC APB2ENR: MRSUBGEN (Bit 0) */
#define RCC_APB2ENR_MRSUBGEN_Msk                                           (0x1UL)		/*!< RCC APB2ENR: MRSUBGEN (Bitfield-Mask: 0x01) */
#define RCC_APB2ENR_MRSUBGEN                                               RCC_APB2ENR_MRSUBGEN_Msk

/* =====================================================    CSR    =====================================================*/ 
#define RCC_CSR_LOCKUPRSTF_Pos                                             (30UL)		/*!<RCC CSR: LOCKUPRSTF (Bit 30) */
#define RCC_CSR_LOCKUPRSTF_Msk                                             (0x40000000UL)		/*!< RCC CSR: LOCKUPRSTF (Bitfield-Mask: 0x01) */
#define RCC_CSR_LOCKUPRSTF                                                 RCC_CSR_LOCKUPRSTF_Msk
#define RCC_CSR_WDGRSTF_Pos                                                (29UL)		/*!<RCC CSR: WDGRSTF (Bit 29) */
#define RCC_CSR_WDGRSTF_Msk                                                (0x20000000UL)		/*!< RCC CSR: WDGRSTF (Bitfield-Mask: 0x01) */
#define RCC_CSR_WDGRSTF                                                    RCC_CSR_WDGRSTF_Msk
#define RCC_CSR_SFTRSTF_Pos                                                (28UL)		/*!<RCC CSR: SFTRSTF (Bit 28) */
#define RCC_CSR_SFTRSTF_Msk                                                (0x10000000UL)		/*!< RCC CSR: SFTRSTF (Bitfield-Mask: 0x01) */
#define RCC_CSR_SFTRSTF                                                    RCC_CSR_SFTRSTF_Msk
#define RCC_CSR_PORRSTF_Pos                                                (27UL)		/*!<RCC CSR: PORRSTF (Bit 27) */
#define RCC_CSR_PORRSTF_Msk                                                (0x8000000UL)		/*!< RCC CSR: PORRSTF (Bitfield-Mask: 0x01) */
#define RCC_CSR_PORRSTF                                                    RCC_CSR_PORRSTF_Msk
#define RCC_CSR_PADRSTF_Pos                                                (26UL)		/*!<RCC CSR: PADRSTF (Bit 26) */
#define RCC_CSR_PADRSTF_Msk                                                (0x4000000UL)		/*!< RCC CSR: PADRSTF (Bitfield-Mask: 0x01) */
#define RCC_CSR_PADRSTF                                                    RCC_CSR_PADRSTF_Msk
#define RCC_CSR_RMVF_Pos                                                   (23UL)		/*!<RCC CSR: RMVF (Bit 23) */
#define RCC_CSR_RMVF_Msk                                                   (0x800000UL)		/*!< RCC CSR: RMVF (Bitfield-Mask: 0x01) */
#define RCC_CSR_RMVF                                                       RCC_CSR_RMVF_Msk

/* =====================================================    RFSWHSECR    =====================================================*/ 
#define RCC_RFSWHSECR_AMPLTHRESH_Pos                                       (16UL)		/*!<RCC RFSWHSECR: AMPLTHRESH (Bit 16) */
#define RCC_RFSWHSECR_AMPLTHRESH_Msk                                       (0x70000UL)		/*!< RCC RFSWHSECR: AMPLTHRESH (Bitfield-Mask: 0x07) */
#define RCC_RFSWHSECR_AMPLTHRESH                                           RCC_RFSWHSECR_AMPLTHRESH_Msk
#define RCC_RFSWHSECR_AMPLTHRESH_0                                         (0x1U << RCC_RFSWHSECR_AMPLTHRESH_Pos)
#define RCC_RFSWHSECR_AMPLTHRESH_1                                         (0x2U << RCC_RFSWHSECR_AMPLTHRESH_Pos)
#define RCC_RFSWHSECR_AMPLTHRESH_2                                         (0x4U << RCC_RFSWHSECR_AMPLTHRESH_Pos)
#define RCC_RFSWHSECR_ISTARTUP_Pos                                         (14UL)		/*!<RCC RFSWHSECR: ISTARTUP (Bit 14) */
#define RCC_RFSWHSECR_ISTARTUP_Msk                                         (0xc000UL)		/*!< RCC RFSWHSECR: ISTARTUP (Bitfield-Mask: 0x03) */
#define RCC_RFSWHSECR_ISTARTUP                                             RCC_RFSWHSECR_ISTARTUP_Msk
#define RCC_RFSWHSECR_ISTARTUP_0                                           (0x1U << RCC_RFSWHSECR_ISTARTUP_Pos)
#define RCC_RFSWHSECR_ISTARTUP_1                                           (0x2U << RCC_RFSWHSECR_ISTARTUP_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_Pos                                         (8UL)		/*!<RCC RFSWHSECR: SWXOTUNE (Bit 8) */
#define RCC_RFSWHSECR_SWXOTUNE_Msk                                         (0x3f00UL)		/*!< RCC RFSWHSECR: SWXOTUNE (Bitfield-Mask: 0x3f) */
#define RCC_RFSWHSECR_SWXOTUNE                                             RCC_RFSWHSECR_SWXOTUNE_Msk
#define RCC_RFSWHSECR_SWXOTUNE_0                                           (0x1U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_1                                           (0x2U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_2                                           (0x4U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_3                                           (0x8U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_4                                           (0x10U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNE_5                                           (0x20U << RCC_RFSWHSECR_SWXOTUNE_Pos)
#define RCC_RFSWHSECR_SWXOTUNEEN_Pos                                       (7UL)		/*!<RCC RFSWHSECR: SWXOTUNEEN (Bit 7) */
#define RCC_RFSWHSECR_SWXOTUNEEN_Msk                                       (0x80UL)		/*!< RCC RFSWHSECR: SWXOTUNEEN (Bitfield-Mask: 0x01) */
#define RCC_RFSWHSECR_SWXOTUNEEN                                           RCC_RFSWHSECR_SWXOTUNEEN_Msk
#define RCC_RFSWHSECR_GMC_Pos                                              (0UL)		/*!<RCC RFSWHSECR: GMC (Bit 0) */
#define RCC_RFSWHSECR_GMC_Msk                                              (0x7fUL)		/*!< RCC RFSWHSECR: GMC (Bitfield-Mask: 0x7f) */
#define RCC_RFSWHSECR_GMC                                                  RCC_RFSWHSECR_GMC_Msk
#define RCC_RFSWHSECR_GMC_0                                                (0x1U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_1                                                (0x2U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_2                                                (0x4U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_3                                                (0x8U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_4                                                (0x10U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_5                                                (0x20U << RCC_RFSWHSECR_GMC_Pos)
#define RCC_RFSWHSECR_GMC_6                                                (0x40U << RCC_RFSWHSECR_GMC_Pos)

/* =====================================================    RFHSECR    =====================================================*/ 
#define RCC_RFHSECR_AMPLREADY_Pos                                          (6UL)		/*!<RCC RFHSECR: AMPLREADY (Bit 6) */
#define RCC_RFHSECR_AMPLREADY_Msk                                          (0x40UL)		/*!< RCC RFHSECR: AMPLREADY (Bitfield-Mask: 0x01) */
#define RCC_RFHSECR_AMPLREADY                                              RCC_RFHSECR_AMPLREADY_Msk
#define RCC_RFHSECR_XOTUNE_Pos                                             (0UL)		/*!<RCC RFHSECR: XOTUNE (Bit 0) */
#define RCC_RFHSECR_XOTUNE_Msk                                             (0x3fUL)		/*!< RCC RFHSECR: XOTUNE (Bitfield-Mask: 0x3f) */
#define RCC_RFHSECR_XOTUNE                                                 RCC_RFHSECR_XOTUNE_Msk
#define RCC_RFHSECR_XOTUNE_0                                               (0x1U << RCC_RFHSECR_XOTUNE_Pos)
#define RCC_RFHSECR_XOTUNE_1                                               (0x2U << RCC_RFHSECR_XOTUNE_Pos)
#define RCC_RFHSECR_XOTUNE_2                                               (0x4U << RCC_RFHSECR_XOTUNE_Pos)
#define RCC_RFHSECR_XOTUNE_3                                               (0x8U << RCC_RFHSECR_XOTUNE_Pos)
#define RCC_RFHSECR_XOTUNE_4                                               (0x10U << RCC_RFHSECR_XOTUNE_Pos)
#define RCC_RFHSECR_XOTUNE_5                                               (0x20U << RCC_RFHSECR_XOTUNE_Pos)

/* =====================================================    AHBSMENR    =====================================================*/ 
#define RCC_AHBSMENR_AESSMEN_Pos                                           (20UL)		/*!<RCC AHBSMENR: AESSMEN (Bit 20) */
#define RCC_AHBSMENR_AESSMEN_Msk                                           (0x100000UL)		/*!< RCC AHBSMENR: AESSMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_AESSMEN                                               RCC_AHBSMENR_AESSMEN_Msk
#define RCC_AHBSMENR_RNGSMEN_Pos                                           (18UL)		/*!<RCC AHBSMENR: RNGSMEN (Bit 18) */
#define RCC_AHBSMENR_RNGSMEN_Msk                                           (0x40000UL)		/*!< RCC AHBSMENR: RNGSMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_RNGSMEN                                               RCC_AHBSMENR_RNGSMEN_Msk
#define RCC_AHBSMENR_CRCSMEN_Pos                                           (12UL)		/*!<RCC AHBSMENR: CRCSMEN (Bit 12) */
#define RCC_AHBSMENR_CRCSMEN_Msk                                           (0x1000UL)		/*!< RCC AHBSMENR: CRCSMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_CRCSMEN                                               RCC_AHBSMENR_CRCSMEN_Msk
#define RCC_AHBSMENR_SRAM1SMEN_Pos                                         (10UL)		/*!<RCC AHBSMENR: SRAM1SMEN (Bit 10) */
#define RCC_AHBSMENR_SRAM1SMEN_Msk                                         (0x400UL)		/*!< RCC AHBSMENR: SRAM1SMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_SRAM1SMEN                                             RCC_AHBSMENR_SRAM1SMEN_Msk
#define RCC_AHBSMENR_SRAM0SMEN_Pos                                         (9UL)		/*!<RCC AHBSMENR: SRAM0SMEN (Bit 9) */
#define RCC_AHBSMENR_SRAM0SMEN_Msk                                         (0x200UL)		/*!< RCC AHBSMENR: SRAM0SMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_SRAM0SMEN                                             RCC_AHBSMENR_SRAM0SMEN_Msk
#define RCC_AHBSMENR_GPIOBSMEN_Pos                                         (3UL)		/*!<RCC AHBSMENR: GPIOBSMEN (Bit 3) */
#define RCC_AHBSMENR_GPIOBSMEN_Msk                                         (0x8UL)		/*!< RCC AHBSMENR: GPIOBSMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_GPIOBSMEN                                             RCC_AHBSMENR_GPIOBSMEN_Msk
#define RCC_AHBSMENR_GPIOASMEN_Pos                                         (2UL)		/*!<RCC AHBSMENR: GPIOASMEN (Bit 2) */
#define RCC_AHBSMENR_GPIOASMEN_Msk                                         (0x4UL)		/*!< RCC AHBSMENR: GPIOASMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_GPIOASMEN                                             RCC_AHBSMENR_GPIOASMEN_Msk
#define RCC_AHBSMENR_DMASMEN_Pos                                           (0UL)		/*!<RCC AHBSMENR: DMASMEN (Bit 0) */
#define RCC_AHBSMENR_DMASMEN_Msk                                           (0x1UL)		/*!< RCC AHBSMENR: DMASMEN (Bitfield-Mask: 0x01) */
#define RCC_AHBSMENR_DMASMEN                                               RCC_AHBSMENR_DMASMEN_Msk

/* =====================================================    APB0SMENR    =====================================================*/ 
#define RCC_APB0SMENR_DBGMCUSMEN_Pos                                       (15UL)		/*!<RCC APB0SMENR: DBGMCUSMEN (Bit 15) */
#define RCC_APB0SMENR_DBGMCUSMEN_Msk                                       (0x8000UL)		/*!< RCC APB0SMENR: DBGMCUSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_DBGMCUSMEN                                           RCC_APB0SMENR_DBGMCUSMEN_Msk
#define RCC_APB0SMENR_WDGSMEN_Pos                                          (14UL)		/*!<RCC APB0SMENR: WDGSMEN (Bit 14) */
#define RCC_APB0SMENR_WDGSMEN_Msk                                          (0x4000UL)		/*!< RCC APB0SMENR: WDGSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_WDGSMEN                                              RCC_APB0SMENR_WDGSMEN_Msk
#define RCC_APB0SMENR_LCSCSMEN_Pos                                         (13UL)		/*!<RCC APB0SMENR: LCSCSMEN (Bit 13) */
#define RCC_APB0SMENR_LCSCSMEN_Msk                                         (0x2000UL)		/*!< RCC APB0SMENR: LCSCSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_LCSCSMEN                                             RCC_APB0SMENR_LCSCSMEN_Msk
#define RCC_APB0SMENR_RTCSMEN_Pos                                          (12UL)		/*!<RCC APB0SMENR: RTCSMEN (Bit 12) */
#define RCC_APB0SMENR_RTCSMEN_Msk                                          (0x1000UL)		/*!< RCC APB0SMENR: RTCSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_RTCSMEN                                              RCC_APB0SMENR_RTCSMEN_Msk
#define RCC_APB0SMENR_DACSMEN_Pos                                          (11UL)		/*!<RCC APB0SMENR: DACSMEN (Bit 11) */
#define RCC_APB0SMENR_DACSMEN_Msk                                          (0x800UL)		/*!< RCC APB0SMENR: DACSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_DACSMEN                                              RCC_APB0SMENR_DACSMEN_Msk
#define RCC_APB0SMENR_COMPSMEN_Pos                                         (10UL)		/*!<RCC APB0SMENR: COMPSMEN (Bit 10) */
#define RCC_APB0SMENR_COMPSMEN_Msk                                         (0x400UL)		/*!< RCC APB0SMENR: COMPSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_COMPSMEN                                             RCC_APB0SMENR_COMPSMEN_Msk
#define RCC_APB0SMENR_LCDCSMEN_Pos                                         (9UL)		/*!<RCC APB0SMENR: LCDCSMEN (Bit 9) */
#define RCC_APB0SMENR_LCDCSMEN_Msk                                         (0x200UL)		/*!< RCC APB0SMENR: LCDCSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_LCDCSMEN                                             RCC_APB0SMENR_LCDCSMEN_Msk
#define RCC_APB0SMENR_SYSCFGSMEN_Pos                                       (8UL)		/*!<RCC APB0SMENR: SYSCFGSMEN (Bit 8) */
#define RCC_APB0SMENR_SYSCFGSMEN_Msk                                       (0x100UL)		/*!< RCC APB0SMENR: SYSCFGSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_SYSCFGSMEN                                           RCC_APB0SMENR_SYSCFGSMEN_Msk
#define RCC_APB0SMENR_TIM16SMEN_Pos                                        (1UL)		/*!<RCC APB0SMENR: TIM16SMEN (Bit 1) */
#define RCC_APB0SMENR_TIM16SMEN_Msk                                        (0x2UL)		/*!< RCC APB0SMENR: TIM16SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_TIM16SMEN                                            RCC_APB0SMENR_TIM16SMEN_Msk
#define RCC_APB0SMENR_TIM2SMEN_Pos                                         (0UL)		/*!<RCC APB0SMENR: TIM2SMEN (Bit 0) */
#define RCC_APB0SMENR_TIM2SMEN_Msk                                         (0x1UL)		/*!< RCC APB0SMENR: TIM2SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB0SMENR_TIM2SMEN                                             RCC_APB0SMENR_TIM2SMEN_Msk

/* =====================================================    APB1SMENR    =====================================================*/ 
#define RCC_APB1SMENR_I2C2SMEN_Pos                                         (23UL)		/*!<RCC APB1SMENR: I2C2SMEN (Bit 23) */
#define RCC_APB1SMENR_I2C2SMEN_Msk                                         (0x800000UL)		/*!< RCC APB1SMENR: I2C2SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_I2C2SMEN                                             RCC_APB1SMENR_I2C2SMEN_Msk
#define RCC_APB1SMENR_I2C1SMEN_Pos                                         (21UL)		/*!<RCC APB1SMENR: I2C1SMEN (Bit 21) */
#define RCC_APB1SMENR_I2C1SMEN_Msk                                         (0x200000UL)		/*!< RCC APB1SMENR: I2C1SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_I2C1SMEN                                             RCC_APB1SMENR_I2C1SMEN_Msk
#define RCC_APB1SMENR_SPI3SMEN_Pos                                         (14UL)		/*!<RCC APB1SMENR: SPI3SMEN (Bit 14) */
#define RCC_APB1SMENR_SPI3SMEN_Msk                                         (0x4000UL)		/*!< RCC APB1SMENR: SPI3SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_SPI3SMEN                                             RCC_APB1SMENR_SPI3SMEN_Msk
#define RCC_APB1SMENR_USARTSMEN_Pos                                        (10UL)		/*!<RCC APB1SMENR: USARTSMEN (Bit 10) */
#define RCC_APB1SMENR_USARTSMEN_Msk                                        (0x400UL)		/*!< RCC APB1SMENR: USARTSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_USARTSMEN                                            RCC_APB1SMENR_USARTSMEN_Msk
#define RCC_APB1SMENR_LPUARTSMEN_Pos                                       (8UL)		/*!<RCC APB1SMENR: LPUARTSMEN (Bit 8) */
#define RCC_APB1SMENR_LPUARTSMEN_Msk                                       (0x100UL)		/*!< RCC APB1SMENR: LPUARTSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_LPUARTSMEN                                           RCC_APB1SMENR_LPUARTSMEN_Msk
#define RCC_APB1SMENR_ADCANASMEN_Pos                                       (5UL)		/*!<RCC APB1SMENR: ADCANASMEN (Bit 5) */
#define RCC_APB1SMENR_ADCANASMEN_Msk                                       (0x20UL)		/*!< RCC APB1SMENR: ADCANASMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_ADCANASMEN                                           RCC_APB1SMENR_ADCANASMEN_Msk
#define RCC_APB1SMENR_ADCDIGSMEN_Pos                                       (4UL)		/*!<RCC APB1SMENR: ADCDIGSMEN (Bit 4) */
#define RCC_APB1SMENR_ADCDIGSMEN_Msk                                       (0x10UL)		/*!< RCC APB1SMENR: ADCDIGSMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_ADCDIGSMEN                                           RCC_APB1SMENR_ADCDIGSMEN_Msk
#define RCC_APB1SMENR_SPI1SMEN_Pos                                         (0UL)		/*!<RCC APB1SMENR: SPI1SMEN (Bit 0) */
#define RCC_APB1SMENR_SPI1SMEN_Msk                                         (0x1UL)		/*!< RCC APB1SMENR: SPI1SMEN (Bitfield-Mask: 0x01) */
#define RCC_APB1SMENR_SPI1SMEN                                             RCC_APB1SMENR_SPI1SMEN_Msk


/* ============================================================================================================================*/ 
/*=====================                                        PWR                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR1    =====================================================*/ 
#define PWR_CR1_APC_Pos                                                    (4UL)		/*!<PWR CR1: APC (Bit 4) */
#define PWR_CR1_APC_Msk                                                    (0x10UL)		/*!< PWR CR1: APC (Bitfield-Mask: 0x01) */
#define PWR_CR1_APC                                                        PWR_CR1_APC_Msk
#define PWR_CR1_IBIAS_RUN_STATE_Pos                                        (3UL)		/*!<PWR CR1: IBIAS_RUN_STATE (Bit 3) */
#define PWR_CR1_IBIAS_RUN_STATE_Msk                                        (0x8UL)		/*!< PWR CR1: IBIAS_RUN_STATE (Bitfield-Mask: 0x01) */
#define PWR_CR1_IBIAS_RUN_STATE                                            PWR_CR1_IBIAS_RUN_STATE_Msk
#define PWR_CR1_IBIAS_RUN_AUTO_Pos                                         (2UL)		/*!<PWR CR1: IBIAS_RUN_AUTO (Bit 2) */
#define PWR_CR1_IBIAS_RUN_AUTO_Msk                                         (0x4UL)		/*!< PWR CR1: IBIAS_RUN_AUTO (Bitfield-Mask: 0x01) */
#define PWR_CR1_IBIAS_RUN_AUTO                                             PWR_CR1_IBIAS_RUN_AUTO_Msk
#define PWR_CR1_ENSDNBOR_Pos                                               (1UL)		/*!<PWR CR1: ENSDNBOR (Bit 1) */
#define PWR_CR1_ENSDNBOR_Msk                                               (0x2UL)		/*!< PWR CR1: ENSDNBOR (Bitfield-Mask: 0x01) */
#define PWR_CR1_ENSDNBOR                                                   PWR_CR1_ENSDNBOR_Msk
#define PWR_CR1_LPMS_Pos                                                   (0UL)		/*!<PWR CR1: LPMS (Bit 0) */
#define PWR_CR1_LPMS_Msk                                                   (0x1UL)		/*!< PWR CR1: LPMS (Bitfield-Mask: 0x01) */
#define PWR_CR1_LPMS                                                       PWR_CR1_LPMS_Msk

/* =====================================================    CR2    =====================================================*/ 
#define PWR_CR2_RFREGON_STATUS_Pos                                         (14UL)		/*!<PWR CR2: RFREGON_STATUS (Bit 14) */
#define PWR_CR2_RFREGON_STATUS_Msk                                         (0x4000UL)		/*!< PWR CR2: RFREGON_STATUS (Bitfield-Mask: 0x01) */
#define PWR_CR2_RFREGON_STATUS                                             PWR_CR2_RFREGON_STATUS_Msk
#define PWR_CR2_RFREGRDY_Pos                                               (13UL)		/*!<PWR CR2: RFREGRDY (Bit 13) */
#define PWR_CR2_RFREGRDY_Msk                                               (0x2000UL)		/*!< PWR CR2: RFREGRDY (Bitfield-Mask: 0x01) */
#define PWR_CR2_RFREGRDY                                                   PWR_CR2_RFREGRDY_Msk
#define PWR_CR2_RFREGBYP_Pos                                               (12UL)		/*!<PWR CR2: RFREGBYP (Bit 12) */
#define PWR_CR2_RFREGBYP_Msk                                               (0x1000UL)		/*!< PWR CR2: RFREGBYP (Bitfield-Mask: 0x01) */
#define PWR_CR2_RFREGBYP                                                   PWR_CR2_RFREGBYP_Msk
#define PWR_CR2_RFREGCEXT_Pos                                              (11UL)		/*!<PWR CR2: RFREGCEXT (Bit 11) */
#define PWR_CR2_RFREGCEXT_Msk                                              (0x800UL)		/*!< PWR CR2: RFREGCEXT (Bitfield-Mask: 0x01) */
#define PWR_CR2_RFREGCEXT                                                  PWR_CR2_RFREGCEXT_Msk
#define PWR_CR2_RFREGEN_Pos                                                (10UL)		/*!<PWR CR2: RFREGEN (Bit 10) */
#define PWR_CR2_RFREGEN_Msk                                                (0x400UL)		/*!< PWR CR2: RFREGEN (Bitfield-Mask: 0x01) */
#define PWR_CR2_RFREGEN                                                    PWR_CR2_RFREGEN_Msk
#define PWR_CR2_ENTS_Pos                                                   (9UL)		/*!<PWR CR2: ENTS (Bit 9) */
#define PWR_CR2_ENTS_Msk                                                   (0x200UL)		/*!< PWR CR2: ENTS (Bitfield-Mask: 0x01) */
#define PWR_CR2_ENTS                                                       PWR_CR2_ENTS_Msk
#define PWR_CR2_GPIORET_Pos                                                (8UL)		/*!<PWR CR2: GPIORET (Bit 8) */
#define PWR_CR2_GPIORET_Msk                                                (0x100UL)		/*!< PWR CR2: GPIORET (Bitfield-Mask: 0x01) */
#define PWR_CR2_GPIORET                                                    PWR_CR2_GPIORET_Msk
#define PWR_CR2_LPREG_VH_STATUS_Pos                                        (7UL)		/*!<PWR CR2: LPREG_VH_STATUS (Bit 7) */
#define PWR_CR2_LPREG_VH_STATUS_Msk                                        (0x80UL)		/*!< PWR CR2: LPREG_VH_STATUS (Bitfield-Mask: 0x01) */
#define PWR_CR2_LPREG_VH_STATUS                                            PWR_CR2_LPREG_VH_STATUS_Msk
#define PWR_CR2_LPREG_FORCE_VH_Pos                                         (6UL)		/*!<PWR CR2: LPREG_FORCE_VH (Bit 6) */
#define PWR_CR2_LPREG_FORCE_VH_Msk                                         (0x40UL)		/*!< PWR CR2: LPREG_FORCE_VH (Bitfield-Mask: 0x01) */
#define PWR_CR2_LPREG_FORCE_VH                                             PWR_CR2_LPREG_FORCE_VH_Msk
#define PWR_CR2_RAMRET1_Pos                                                (5UL)		/*!<PWR CR2: RAMRET1 (Bit 5) */
#define PWR_CR2_RAMRET1_Msk                                                (0x20UL)		/*!< PWR CR2: RAMRET1 (Bitfield-Mask: 0x01) */
#define PWR_CR2_RAMRET1                                                    PWR_CR2_RAMRET1_Msk
#define PWR_CR2_DBGRET_Pos                                                 (4UL)		/*!<PWR CR2: DBGRET (Bit 4) */
#define PWR_CR2_DBGRET_Msk                                                 (0x10UL)		/*!< PWR CR2: DBGRET (Bitfield-Mask: 0x01) */
#define PWR_CR2_DBGRET                                                     PWR_CR2_DBGRET_Msk
#define PWR_CR2_PVDLS_Pos                                                  (1UL)		/*!<PWR CR2: PVDLS (Bit 1) */
#define PWR_CR2_PVDLS_Msk                                                  (0xeUL)		/*!< PWR CR2: PVDLS (Bitfield-Mask: 0x07) */
#define PWR_CR2_PVDLS                                                      PWR_CR2_PVDLS_Msk
#define PWR_CR2_PVDLS_0                                                    (0x1U << PWR_CR2_PVDLS_Pos)
#define PWR_CR2_PVDLS_1                                                    (0x2U << PWR_CR2_PVDLS_Pos)
#define PWR_CR2_PVDLS_2                                                    (0x4U << PWR_CR2_PVDLS_Pos)
#define PWR_CR2_PVDE_Pos                                                   (0UL)		/*!<PWR CR2: PVDE (Bit 0) */
#define PWR_CR2_PVDE_Msk                                                   (0x1UL)		/*!< PWR CR2: PVDE (Bitfield-Mask: 0x01) */
#define PWR_CR2_PVDE                                                       PWR_CR2_PVDE_Msk

/* =====================================================    IEWU    =====================================================*/ 
#define PWR_IEWU_EWLPAWUR_Pos                                              (10UL)		/*!<PWR IEWU: EWLPAWUR (Bit 10) */
#define PWR_IEWU_EWLPAWUR_Msk                                              (0x400UL)		/*!< PWR IEWU: EWLPAWUR (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EWLPAWUR                                                   PWR_IEWU_EWLPAWUR_Msk
#define PWR_IEWU_EWMRSUBGHCPU_Pos                                          (9UL)		/*!<PWR IEWU: EWMRSUBGHCPU (Bit 9) */
#define PWR_IEWU_EWMRSUBGHCPU_Msk                                          (0x200UL)		/*!< PWR IEWU: EWMRSUBGHCPU (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EWMRSUBGHCPU                                              PWR_IEWU_EWMRSUBGHCPU_Msk
#define PWR_IEWU_EWMRSUBG_Pos                                              (8UL)		/*!<PWR IEWU: EWMRSUBG (Bit 8) */
#define PWR_IEWU_EWMRSUBG_Msk                                              (0x100UL)		/*!< PWR IEWU: EWMRSUBG (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EWMRSUBG                                                  PWR_IEWU_EWMRSUBG_Msk
#define PWR_IEWU_EIWL4_Pos                                                 (4UL)		/*!<PWR IEWU: EIWL4 (Bit 4) */
#define PWR_IEWU_EIWL4_Msk                                                 (0x10UL)		/*!< PWR IEWU: EIWL4 (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EIWL4                                                     PWR_IEWU_EIWL4_Msk
#define PWR_IEWU_EIWL3_Pos                                                 (3UL)		/*!<PWR IEWU: EIWL3 (Bit 3) */
#define PWR_IEWU_EIWL3_Msk                                                 (0x8UL)		/*!< PWR IEWU: EIWL3 (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EIWL3                                                     PWR_IEWU_EIWL3_Msk
#define PWR_IEWU_EIWL2_Pos                                                 (2UL)		/*!<PWR IEWU: EIWL2 (Bit 2) */
#define PWR_IEWU_EIWL2_Msk                                                 (0x4UL)		/*!< PWR IEWU: EIWL2 (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EIWL2                                                     PWR_IEWU_EIWL2_Msk
#define PWR_IEWU_EIWL1_Pos                                                 (1UL)		/*!<PWR IEWU: EIWL1 (Bit 1) */
#define PWR_IEWU_EIWL1_Msk                                                 (0x2UL)		/*!< PWR IEWU: EIWL1 (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EIWL1                                                     PWR_IEWU_EIWL1_Msk
#define PWR_IEWU_EIWL0_Pos                                                 (0UL)		/*!<PWR IEWU: EIWL0 (Bit 0) */
#define PWR_IEWU_EIWL0_Msk                                                 (0x1UL)		/*!< PWR IEWU: EIWL0 (Bitfield-Mask: 0x01) */
#define PWR_IEWU_EIWL0                                                     PWR_IEWU_EIWL0_Msk

/* =====================================================    IWUP    =====================================================*/ 
#define PWR_IWUP_WLPAWURP_Pos                                              (10UL)		/*!<PWR IWUP: WLPAWURP (Bit 10) */
#define PWR_IWUP_WLPAWURP_Msk                                              (0x400UL)		/*!< PWR IWUP: WLPAWURP (Bitfield-Mask: 0x01) */
#define PWR_IWUP_WLPAWURP                                                  PWR_IWUP_WLPAWURP_Msk
#define PWR_IWUP_WMRSUBGHCPUP_Pos                                          (9UL)		/*!<PWR IWUP: WMRSUBGHCPUP (Bit 9) */
#define PWR_IWUP_WMRSUBGHCPUP_Msk                                          (0x200UL)		/*!< PWR IWUP: WMRSUBGHCPUP (Bitfield-Mask: 0x01) */
#define PWR_IWUP_WMRSUBGHCPUP                                              PWR_IWUP_WMRSUBGHCPUP_Msk
#define PWR_IWUP_WMRSUBGP_Pos                                              (8UL)		/*!<PWR IWUP: WMRSUBGP (Bit 8) */
#define PWR_IWUP_WMRSUBGP_Msk                                              (0x100UL)		/*!< PWR IWUP: WMRSUBGP (Bitfield-Mask: 0x01) */
#define PWR_IWUP_WMRSUBGP                                                  PWR_IWUP_WMRSUBGP_Msk
#define PWR_IWUP_IWUP4_Pos                                                 (4UL)		/*!<PWR IWUP: IWUP4 (Bit 4) */
#define PWR_IWUP_IWUP4_Msk                                                 (0x10UL)		/*!< PWR IWUP: IWUP4 (Bitfield-Mask: 0x01) */
#define PWR_IWUP_IWUP4                                                     PWR_IWUP_IWUP4_Msk
#define PWR_IWUP_IWUP3_Pos                                                 (3UL)		/*!<PWR IWUP: IWUP3 (Bit 3) */
#define PWR_IWUP_IWUP3_Msk                                                 (0x8UL)		/*!< PWR IWUP: IWUP3 (Bitfield-Mask: 0x01) */
#define PWR_IWUP_IWUP3                                                     PWR_IWUP_IWUP3_Msk
#define PWR_IWUP_IWUP2_Pos                                                 (2UL)		/*!<PWR IWUP: IWUP2 (Bit 2) */
#define PWR_IWUP_IWUP2_Msk                                                 (0x4UL)		/*!< PWR IWUP: IWUP2 (Bitfield-Mask: 0x01) */
#define PWR_IWUP_IWUP2                                                     PWR_IWUP_IWUP2_Msk
#define PWR_IWUP_IWUP1_Pos                                                 (1UL)		/*!<PWR IWUP: IWUP1 (Bit 1) */
#define PWR_IWUP_IWUP1_Msk                                                 (0x2UL)		/*!< PWR IWUP: IWUP1 (Bitfield-Mask: 0x01) */
#define PWR_IWUP_IWUP1                                                     PWR_IWUP_IWUP1_Msk
#define PWR_IWUP_IWUP0_Pos                                                 (0UL)		/*!<PWR IWUP: IWUP0 (Bit 0) */
#define PWR_IWUP_IWUP0_Msk                                                 (0x1UL)		/*!< PWR IWUP: IWUP0 (Bitfield-Mask: 0x01) */
#define PWR_IWUP_IWUP0                                                     PWR_IWUP_IWUP0_Msk

/* =====================================================    IWUF    =====================================================*/ 
#define PWR_IWUF_WLPAWURF_Pos                                              (10UL)		/*!<PWR IWUF: WLPAWURF (Bit 10) */
#define PWR_IWUF_WLPAWURF_Msk                                              (0x400UL)		/*!< PWR IWUF: WLPAWURF (Bitfield-Mask: 0x01) */
#define PWR_IWUF_WLPAWURF                                                  PWR_IWUF_WLPAWURF_Msk
#define PWR_IWUF_WMRSUBGHCPUF_Pos                                          (9UL)		/*!<PWR IWUF: WMRSUBGHCPUF (Bit 9) */
#define PWR_IWUF_WMRSUBGHCPUF_Msk                                          (0x200UL)		/*!< PWR IWUF: WMRSUBGHCPUF (Bitfield-Mask: 0x01) */
#define PWR_IWUF_WMRSUBGHCPUF                                              PWR_IWUF_WMRSUBGHCPUF_Msk
#define PWR_IWUF_WMRSUBGF_Pos                                              (8UL)		/*!<PWR IWUF: WMRSUBGF (Bit 8) */
#define PWR_IWUF_WMRSUBGF_Msk                                              (0x100UL)		/*!< PWR IWUF: WMRSUBGF (Bitfield-Mask: 0x01) */
#define PWR_IWUF_WMRSUBGF                                                  PWR_IWUF_WMRSUBGF_Msk
#define PWR_IWUF_IWUF4_Pos                                                 (4UL)		/*!<PWR IWUF: IWUF4 (Bit 4) */
#define PWR_IWUF_IWUF4_Msk                                                 (0x10UL)		/*!< PWR IWUF: IWUF4 (Bitfield-Mask: 0x01) */
#define PWR_IWUF_IWUF4                                                     PWR_IWUF_IWUF4_Msk
#define PWR_IWUF_IWUF3_Pos                                                 (3UL)		/*!<PWR IWUF: IWUF3 (Bit 3) */
#define PWR_IWUF_IWUF3_Msk                                                 (0x8UL)		/*!< PWR IWUF: IWUF3 (Bitfield-Mask: 0x01) */
#define PWR_IWUF_IWUF3                                                     PWR_IWUF_IWUF3_Msk
#define PWR_IWUF_IWUF2_Pos                                                 (2UL)		/*!<PWR IWUF: IWUF2 (Bit 2) */
#define PWR_IWUF_IWUF2_Msk                                                 (0x4UL)		/*!< PWR IWUF: IWUF2 (Bitfield-Mask: 0x01) */
#define PWR_IWUF_IWUF2                                                     PWR_IWUF_IWUF2_Msk
#define PWR_IWUF_IWUF1_Pos                                                 (1UL)		/*!<PWR IWUF: IWUF1 (Bit 1) */
#define PWR_IWUF_IWUF1_Msk                                                 (0x2UL)		/*!< PWR IWUF: IWUF1 (Bitfield-Mask: 0x01) */
#define PWR_IWUF_IWUF1                                                     PWR_IWUF_IWUF1_Msk
#define PWR_IWUF_IWUF0_Pos                                                 (0UL)		/*!<PWR IWUF: IWUF0 (Bit 0) */
#define PWR_IWUF_IWUF0_Msk                                                 (0x1UL)		/*!< PWR IWUF: IWUF0 (Bitfield-Mask: 0x01) */
#define PWR_IWUF_IWUF0                                                     PWR_IWUF_IWUF0_Msk

/* =====================================================    SR2    =====================================================*/ 
#define PWR_SR2_IOBOOTVAL_Pos                                              (12UL)		/*!<PWR SR2: IOBOOTVAL (Bit 12) */
#define PWR_SR2_IOBOOTVAL_Msk                                              (0xf000UL)		/*!< PWR SR2: IOBOOTVAL (Bitfield-Mask: 0x0f) */
#define PWR_SR2_IOBOOTVAL                                                  PWR_SR2_IOBOOTVAL_Msk
#define PWR_SR2_IOBOOTVAL_0                                                (0x1U << PWR_SR2_IOBOOTVAL_Pos)
#define PWR_SR2_IOBOOTVAL_1                                                (0x2U << PWR_SR2_IOBOOTVAL_Pos)
#define PWR_SR2_IOBOOTVAL_2                                                (0x4U << PWR_SR2_IOBOOTVAL_Pos)
#define PWR_SR2_IOBOOTVAL_3                                                (0x8U << PWR_SR2_IOBOOTVAL_Pos)
#define PWR_SR2_PVDO_Pos                                                   (11UL)		/*!<PWR SR2: PVDO (Bit 11) */
#define PWR_SR2_PVDO_Msk                                                   (0x800UL)		/*!< PWR SR2: PVDO (Bitfield-Mask: 0x01) */
#define PWR_SR2_PVDO                                                       PWR_SR2_PVDO_Msk
#define PWR_SR2_REGMS_Pos                                                  (9UL)		/*!<PWR SR2: REGMS (Bit 9) */
#define PWR_SR2_REGMS_Msk                                                  (0x200UL)		/*!< PWR SR2: REGMS (Bitfield-Mask: 0x01) */
#define PWR_SR2_REGMS                                                      PWR_SR2_REGMS_Msk
#define PWR_SR2_REGLPS_Pos                                                 (8UL)		/*!<PWR SR2: REGLPS (Bit 8) */
#define PWR_SR2_REGLPS_Msk                                                 (0x100UL)		/*!< PWR SR2: REGLPS (Bitfield-Mask: 0x01) */
#define PWR_SR2_REGLPS                                                     PWR_SR2_REGLPS_Msk
#define PWR_SR2_IOBOOTVAL2_Pos                                             (4UL)		/*!<PWR SR2: IOBOOTVAL2 (Bit 4) */
#define PWR_SR2_IOBOOTVAL2_Msk                                             (0xf0UL)		/*!< PWR SR2: IOBOOTVAL2 (Bitfield-Mask: 0x0f) */
#define PWR_SR2_IOBOOTVAL2                                                 PWR_SR2_IOBOOTVAL2_Msk
#define PWR_SR2_IOBOOTVAL2_0                                               (0x1U << PWR_SR2_IOBOOTVAL2_Pos)
#define PWR_SR2_IOBOOTVAL2_1                                               (0x2U << PWR_SR2_IOBOOTVAL2_Pos)
#define PWR_SR2_IOBOOTVAL2_2                                               (0x4U << PWR_SR2_IOBOOTVAL2_Pos)
#define PWR_SR2_IOBOOTVAL2_3                                               (0x8U << PWR_SR2_IOBOOTVAL2_Pos)
#define PWR_SR2_SMPSRDY_Pos                                                (2UL)		/*!<PWR SR2: SMPSRDY (Bit 2) */
#define PWR_SR2_SMPSRDY_Msk                                                (0x4UL)		/*!< PWR SR2: SMPSRDY (Bitfield-Mask: 0x01) */
#define PWR_SR2_SMPSRDY                                                    PWR_SR2_SMPSRDY_Msk
#define PWR_SR2_SMPSENR_Pos                                                (1UL)		/*!<PWR SR2: SMPSENR (Bit 1) */
#define PWR_SR2_SMPSENR_Msk                                                (0x2UL)		/*!< PWR SR2: SMPSENR (Bitfield-Mask: 0x01) */
#define PWR_SR2_SMPSENR                                                    PWR_SR2_SMPSENR_Msk
#define PWR_SR2_SMPSBYPR_Pos                                               (0UL)		/*!<PWR SR2: SMPSBYPR (Bit 0) */
#define PWR_SR2_SMPSBYPR_Msk                                               (0x1UL)		/*!< PWR SR2: SMPSBYPR (Bitfield-Mask: 0x01) */
#define PWR_SR2_SMPSBYPR                                                   PWR_SR2_SMPSBYPR_Msk

/* =====================================================    CR5    =====================================================*/ 
#define PWR_CR5_SMPS_BOF_DYN_Pos                                           (15UL)		/*!<PWR CR5: SMPS_BOF_DYN (Bit 15) */
#define PWR_CR5_SMPS_BOF_DYN_Msk                                           (0x8000UL)		/*!< PWR CR5: SMPS_BOF_DYN (Bitfield-Mask: 0x01) */
#define PWR_CR5_SMPS_BOF_DYN                                               PWR_CR5_SMPS_BOF_DYN_Msk
#define PWR_CR5_SMPS_PRECH_CUR_SEL_Pos                                     (13UL)		/*!<PWR CR5: SMPS_PRECH_CUR_SEL (Bit 13) */
#define PWR_CR5_SMPS_PRECH_CUR_SEL_Msk                                     (0x6000UL)		/*!< PWR CR5: SMPS_PRECH_CUR_SEL (Bitfield-Mask: 0x03) */
#define PWR_CR5_SMPS_PRECH_CUR_SEL                                         PWR_CR5_SMPS_PRECH_CUR_SEL_Msk
#define PWR_CR5_SMPS_PRECH_CUR_SEL_0                                       (0x1U << PWR_CR5_SMPS_PRECH_CUR_SEL_Pos)
#define PWR_CR5_SMPS_PRECH_CUR_SEL_1                                       (0x2U << PWR_CR5_SMPS_PRECH_CUR_SEL_Pos)
#define PWR_CR5_CLKDETR_DISABLE_Pos                                        (12UL)		/*!<PWR CR5: CLKDETR_DISABLE (Bit 12) */
#define PWR_CR5_CLKDETR_DISABLE_Msk                                        (0x1000UL)		/*!< PWR CR5: CLKDETR_DISABLE (Bitfield-Mask: 0x01) */
#define PWR_CR5_CLKDETR_DISABLE                                            PWR_CR5_CLKDETR_DISABLE_Msk
#define PWR_CR5_SMPS_ENA_DCM_Pos                                           (11UL)		/*!<PWR CR5: SMPS_ENA_DCM (Bit 11) */
#define PWR_CR5_SMPS_ENA_DCM_Msk                                           (0x800UL)		/*!< PWR CR5: SMPS_ENA_DCM (Bitfield-Mask: 0x01) */
#define PWR_CR5_SMPS_ENA_DCM                                               PWR_CR5_SMPS_ENA_DCM_Msk
#define PWR_CR5_NOSMPS_Pos                                                 (10UL)		/*!<PWR CR5: NOSMPS (Bit 10) */
#define PWR_CR5_NOSMPS_Msk                                                 (0x400UL)		/*!< PWR CR5: NOSMPS (Bitfield-Mask: 0x01) */
#define PWR_CR5_NOSMPS                                                     PWR_CR5_NOSMPS_Msk
#define PWR_CR5_SMPSFBYP_Pos                                               (9UL)		/*!<PWR CR5: SMPSFBYP (Bit 9) */
#define PWR_CR5_SMPSFBYP_Msk                                               (0x200UL)		/*!< PWR CR5: SMPSFBYP (Bitfield-Mask: 0x01) */
#define PWR_CR5_SMPSFBYP                                                   PWR_CR5_SMPSFBYP_Msk
#define PWR_CR5_SMPSLPOPEN_Pos                                             (8UL)		/*!<PWR CR5: SMPSLPOPEN (Bit 8) */
#define PWR_CR5_SMPSLPOPEN_Msk                                             (0x100UL)		/*!< PWR CR5: SMPSLPOPEN (Bitfield-Mask: 0x01) */
#define PWR_CR5_SMPSLPOPEN                                                 PWR_CR5_SMPSLPOPEN_Msk
#define PWR_CR5_NOSMPS_BOF_Pos                                             (7UL)		/*!<PWR CR5: NOSMPS_BOF (Bit 7) */
#define PWR_CR5_NOSMPS_BOF_Msk                                             (0x80UL)		/*!< PWR CR5: NOSMPS_BOF (Bitfield-Mask: 0x01) */
#define PWR_CR5_NOSMPS_BOF                                                 PWR_CR5_NOSMPS_BOF_Msk
#define PWR_CR5_SMPS_BOF_STATIC_Pos                                        (6UL)		/*!<PWR CR5: SMPS_BOF_STATIC (Bit 6) */
#define PWR_CR5_SMPS_BOF_STATIC_Msk                                        (0x40UL)		/*!< PWR CR5: SMPS_BOF_STATIC (Bitfield-Mask: 0x01) */
#define PWR_CR5_SMPS_BOF_STATIC                                            PWR_CR5_SMPS_BOF_STATIC_Msk
#define PWR_CR5_SMPSBOMSEL_Pos                                             (4UL)		/*!<PWR CR5: SMPSBOMSEL (Bit 4) */
#define PWR_CR5_SMPSBOMSEL_Msk                                             (0x30UL)		/*!< PWR CR5: SMPSBOMSEL (Bitfield-Mask: 0x03) */
#define PWR_CR5_SMPSBOMSEL                                                 PWR_CR5_SMPSBOMSEL_Msk
#define PWR_CR5_SMPSBOMSEL_0                                               (0x1U << PWR_CR5_SMPSBOMSEL_Pos)
#define PWR_CR5_SMPSBOMSEL_1                                               (0x2U << PWR_CR5_SMPSBOMSEL_Pos)
#define PWR_CR5_SMPSLVL_Pos                                                (0UL)		/*!<PWR CR5: SMPSLVL (Bit 0) */
#define PWR_CR5_SMPSLVL_Msk                                                (0xfUL)		/*!< PWR CR5: SMPSLVL (Bitfield-Mask: 0x0f) */
#define PWR_CR5_SMPSLVL                                                    PWR_CR5_SMPSLVL_Msk
#define PWR_CR5_SMPSLVL_0                                                  (0x1U << PWR_CR5_SMPSLVL_Pos)
#define PWR_CR5_SMPSLVL_1                                                  (0x2U << PWR_CR5_SMPSLVL_Pos)
#define PWR_CR5_SMPSLVL_2                                                  (0x4U << PWR_CR5_SMPSLVL_Pos)
#define PWR_CR5_SMPSLVL_3                                                  (0x8U << PWR_CR5_SMPSLVL_Pos)

/* =====================================================    PUCRA    =====================================================*/ 
#define PWR_PUCRA_PA15_Pos                                                 (15UL)		/*!<PWR PUCRA: PA15 (Bit 15) */
#define PWR_PUCRA_PA15_Msk                                                 (0x8000UL)		/*!< PWR PUCRA: PA15 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA15                                                     PWR_PUCRA_PA15_Msk
#define PWR_PUCRA_PA14_Pos                                                 (14UL)		/*!<PWR PUCRA: PA14 (Bit 14) */
#define PWR_PUCRA_PA14_Msk                                                 (0x4000UL)		/*!< PWR PUCRA: PA14 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA14                                                     PWR_PUCRA_PA14_Msk
#define PWR_PUCRA_PA13_Pos                                                 (13UL)		/*!<PWR PUCRA: PA13 (Bit 13) */
#define PWR_PUCRA_PA13_Msk                                                 (0x2000UL)		/*!< PWR PUCRA: PA13 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA13                                                     PWR_PUCRA_PA13_Msk
#define PWR_PUCRA_PA12_Pos                                                 (12UL)		/*!<PWR PUCRA: PA12 (Bit 12) */
#define PWR_PUCRA_PA12_Msk                                                 (0x1000UL)		/*!< PWR PUCRA: PA12 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA12                                                     PWR_PUCRA_PA12_Msk
#define PWR_PUCRA_PA11_Pos                                                 (11UL)		/*!<PWR PUCRA: PA11 (Bit 11) */
#define PWR_PUCRA_PA11_Msk                                                 (0x800UL)		/*!< PWR PUCRA: PA11 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA11                                                     PWR_PUCRA_PA11_Msk
#define PWR_PUCRA_PA10_Pos                                                 (10UL)		/*!<PWR PUCRA: PA10 (Bit 10) */
#define PWR_PUCRA_PA10_Msk                                                 (0x400UL)		/*!< PWR PUCRA: PA10 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA10                                                     PWR_PUCRA_PA10_Msk
#define PWR_PUCRA_PA9_Pos                                                  (9UL)		/*!<PWR PUCRA: PA9 (Bit 9) */
#define PWR_PUCRA_PA9_Msk                                                  (0x200UL)		/*!< PWR PUCRA: PA9 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA9                                                      PWR_PUCRA_PA9_Msk
#define PWR_PUCRA_PA8_Pos                                                  (8UL)		/*!<PWR PUCRA: PA8 (Bit 8) */
#define PWR_PUCRA_PA8_Msk                                                  (0x100UL)		/*!< PWR PUCRA: PA8 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA8                                                      PWR_PUCRA_PA8_Msk
#define PWR_PUCRA_PA7_Pos                                                  (7UL)		/*!<PWR PUCRA: PA7 (Bit 7) */
#define PWR_PUCRA_PA7_Msk                                                  (0x80UL)		/*!< PWR PUCRA: PA7 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA7                                                      PWR_PUCRA_PA7_Msk
#define PWR_PUCRA_PA6_Pos                                                  (6UL)		/*!<PWR PUCRA: PA6 (Bit 6) */
#define PWR_PUCRA_PA6_Msk                                                  (0x40UL)		/*!< PWR PUCRA: PA6 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA6                                                      PWR_PUCRA_PA6_Msk
#define PWR_PUCRA_PA5_Pos                                                  (5UL)		/*!<PWR PUCRA: PA5 (Bit 5) */
#define PWR_PUCRA_PA5_Msk                                                  (0x20UL)		/*!< PWR PUCRA: PA5 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA5                                                      PWR_PUCRA_PA5_Msk
#define PWR_PUCRA_PA4_Pos                                                  (4UL)		/*!<PWR PUCRA: PA4 (Bit 4) */
#define PWR_PUCRA_PA4_Msk                                                  (0x10UL)		/*!< PWR PUCRA: PA4 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA4                                                      PWR_PUCRA_PA4_Msk
#define PWR_PUCRA_PA3_Pos                                                  (3UL)		/*!<PWR PUCRA: PA3 (Bit 3) */
#define PWR_PUCRA_PA3_Msk                                                  (0x8UL)		/*!< PWR PUCRA: PA3 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA3                                                      PWR_PUCRA_PA3_Msk
#define PWR_PUCRA_PA2_Pos                                                  (2UL)		/*!<PWR PUCRA: PA2 (Bit 2) */
#define PWR_PUCRA_PA2_Msk                                                  (0x4UL)		/*!< PWR PUCRA: PA2 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA2                                                      PWR_PUCRA_PA2_Msk
#define PWR_PUCRA_PA1_Pos                                                  (1UL)		/*!<PWR PUCRA: PA1 (Bit 1) */
#define PWR_PUCRA_PA1_Msk                                                  (0x2UL)		/*!< PWR PUCRA: PA1 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA1                                                      PWR_PUCRA_PA1_Msk
#define PWR_PUCRA_PA0_Pos                                                  (0UL)		/*!<PWR PUCRA: PA0 (Bit 0) */
#define PWR_PUCRA_PA0_Msk                                                  (0x1UL)		/*!< PWR PUCRA: PA0 (Bitfield-Mask: 0x01) */
#define PWR_PUCRA_PA0                                                      PWR_PUCRA_PA0_Msk

/* =====================================================    PDCRA    =====================================================*/ 
#define PWR_PDCRA_PA15_Pos                                                 (15UL)		/*!<PWR PDCRA: PA15 (Bit 15) */
#define PWR_PDCRA_PA15_Msk                                                 (0x8000UL)		/*!< PWR PDCRA: PA15 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA15                                                     PWR_PDCRA_PA15_Msk
#define PWR_PDCRA_PA14_Pos                                                 (14UL)		/*!<PWR PDCRA: PA14 (Bit 14) */
#define PWR_PDCRA_PA14_Msk                                                 (0x4000UL)		/*!< PWR PDCRA: PA14 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA14                                                     PWR_PDCRA_PA14_Msk
#define PWR_PDCRA_PA13_Pos                                                 (13UL)		/*!<PWR PDCRA: PA13 (Bit 13) */
#define PWR_PDCRA_PA13_Msk                                                 (0x2000UL)		/*!< PWR PDCRA: PA13 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA13                                                     PWR_PDCRA_PA13_Msk
#define PWR_PDCRA_PA12_Pos                                                 (12UL)		/*!<PWR PDCRA: PA12 (Bit 12) */
#define PWR_PDCRA_PA12_Msk                                                 (0x1000UL)		/*!< PWR PDCRA: PA12 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA12                                                     PWR_PDCRA_PA12_Msk
#define PWR_PDCRA_PA11_Pos                                                 (11UL)		/*!<PWR PDCRA: PA11 (Bit 11) */
#define PWR_PDCRA_PA11_Msk                                                 (0x800UL)		/*!< PWR PDCRA: PA11 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA11                                                     PWR_PDCRA_PA11_Msk
#define PWR_PDCRA_PA10_Pos                                                 (10UL)		/*!<PWR PDCRA: PA10 (Bit 10) */
#define PWR_PDCRA_PA10_Msk                                                 (0x400UL)		/*!< PWR PDCRA: PA10 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA10                                                     PWR_PDCRA_PA10_Msk
#define PWR_PDCRA_PA9_Pos                                                  (9UL)		/*!<PWR PDCRA: PA9 (Bit 9) */
#define PWR_PDCRA_PA9_Msk                                                  (0x200UL)		/*!< PWR PDCRA: PA9 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA9                                                      PWR_PDCRA_PA9_Msk
#define PWR_PDCRA_PA8_Pos                                                  (8UL)		/*!<PWR PDCRA: PA8 (Bit 8) */
#define PWR_PDCRA_PA8_Msk                                                  (0x100UL)		/*!< PWR PDCRA: PA8 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA8                                                      PWR_PDCRA_PA8_Msk
#define PWR_PDCRA_PA7_Pos                                                  (7UL)		/*!<PWR PDCRA: PA7 (Bit 7) */
#define PWR_PDCRA_PA7_Msk                                                  (0x80UL)		/*!< PWR PDCRA: PA7 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA7                                                      PWR_PDCRA_PA7_Msk
#define PWR_PDCRA_PA6_Pos                                                  (6UL)		/*!<PWR PDCRA: PA6 (Bit 6) */
#define PWR_PDCRA_PA6_Msk                                                  (0x40UL)		/*!< PWR PDCRA: PA6 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA6                                                      PWR_PDCRA_PA6_Msk
#define PWR_PDCRA_PA5_Pos                                                  (5UL)		/*!<PWR PDCRA: PA5 (Bit 5) */
#define PWR_PDCRA_PA5_Msk                                                  (0x20UL)		/*!< PWR PDCRA: PA5 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA5                                                      PWR_PDCRA_PA5_Msk
#define PWR_PDCRA_PA4_Pos                                                  (4UL)		/*!<PWR PDCRA: PA4 (Bit 4) */
#define PWR_PDCRA_PA4_Msk                                                  (0x10UL)		/*!< PWR PDCRA: PA4 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA4                                                      PWR_PDCRA_PA4_Msk
#define PWR_PDCRA_PA3_Pos                                                  (3UL)		/*!<PWR PDCRA: PA3 (Bit 3) */
#define PWR_PDCRA_PA3_Msk                                                  (0x8UL)		/*!< PWR PDCRA: PA3 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA3                                                      PWR_PDCRA_PA3_Msk
#define PWR_PDCRA_PA2_Pos                                                  (2UL)		/*!<PWR PDCRA: PA2 (Bit 2) */
#define PWR_PDCRA_PA2_Msk                                                  (0x4UL)		/*!< PWR PDCRA: PA2 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA2                                                      PWR_PDCRA_PA2_Msk
#define PWR_PDCRA_PA1_Pos                                                  (1UL)		/*!<PWR PDCRA: PA1 (Bit 1) */
#define PWR_PDCRA_PA1_Msk                                                  (0x2UL)		/*!< PWR PDCRA: PA1 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA1                                                      PWR_PDCRA_PA1_Msk
#define PWR_PDCRA_PA0_Pos                                                  (0UL)		/*!<PWR PDCRA: PA0 (Bit 0) */
#define PWR_PDCRA_PA0_Msk                                                  (0x1UL)		/*!< PWR PDCRA: PA0 (Bitfield-Mask: 0x01) */
#define PWR_PDCRA_PA0                                                      PWR_PDCRA_PA0_Msk

/* =====================================================    PUCRB    =====================================================*/ 
#define PWR_PUCRB_PB15_Pos                                                 (15UL)		/*!<PWR PUCRB: PB15 (Bit 15) */
#define PWR_PUCRB_PB15_Msk                                                 (0x8000UL)		/*!< PWR PUCRB: PB15 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB15                                                     PWR_PUCRB_PB15_Msk
#define PWR_PUCRB_PB14_Pos                                                 (14UL)		/*!<PWR PUCRB: PB14 (Bit 14) */
#define PWR_PUCRB_PB14_Msk                                                 (0x4000UL)		/*!< PWR PUCRB: PB14 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB14                                                     PWR_PUCRB_PB14_Msk
#define PWR_PUCRB_PB13_Pos                                                 (13UL)		/*!<PWR PUCRB: PB13 (Bit 13) */
#define PWR_PUCRB_PB13_Msk                                                 (0x2000UL)		/*!< PWR PUCRB: PB13 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB13                                                     PWR_PUCRB_PB13_Msk
#define PWR_PUCRB_PB12_Pos                                                 (12UL)		/*!<PWR PUCRB: PB12 (Bit 12) */
#define PWR_PUCRB_PB12_Msk                                                 (0x1000UL)		/*!< PWR PUCRB: PB12 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB12                                                     PWR_PUCRB_PB12_Msk
#define PWR_PUCRB_PB11_Pos                                                 (11UL)		/*!<PWR PUCRB: PB11 (Bit 11) */
#define PWR_PUCRB_PB11_Msk                                                 (0x800UL)		/*!< PWR PUCRB: PB11 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB11                                                     PWR_PUCRB_PB11_Msk
#define PWR_PUCRB_PB10_Pos                                                 (10UL)		/*!<PWR PUCRB: PB10 (Bit 10) */
#define PWR_PUCRB_PB10_Msk                                                 (0x400UL)		/*!< PWR PUCRB: PB10 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB10                                                     PWR_PUCRB_PB10_Msk
#define PWR_PUCRB_PB9_Pos                                                  (9UL)		/*!<PWR PUCRB: PB9 (Bit 9) */
#define PWR_PUCRB_PB9_Msk                                                  (0x200UL)		/*!< PWR PUCRB: PB9 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB9                                                      PWR_PUCRB_PB9_Msk
#define PWR_PUCRB_PB8_Pos                                                  (8UL)		/*!<PWR PUCRB: PB8 (Bit 8) */
#define PWR_PUCRB_PB8_Msk                                                  (0x100UL)		/*!< PWR PUCRB: PB8 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB8                                                      PWR_PUCRB_PB8_Msk
#define PWR_PUCRB_PB7_Pos                                                  (7UL)		/*!<PWR PUCRB: PB7 (Bit 7) */
#define PWR_PUCRB_PB7_Msk                                                  (0x80UL)		/*!< PWR PUCRB: PB7 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB7                                                      PWR_PUCRB_PB7_Msk
#define PWR_PUCRB_PB6_Pos                                                  (6UL)		/*!<PWR PUCRB: PB6 (Bit 6) */
#define PWR_PUCRB_PB6_Msk                                                  (0x40UL)		/*!< PWR PUCRB: PB6 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB6                                                      PWR_PUCRB_PB6_Msk
#define PWR_PUCRB_PB5_Pos                                                  (5UL)		/*!<PWR PUCRB: PB5 (Bit 5) */
#define PWR_PUCRB_PB5_Msk                                                  (0x20UL)		/*!< PWR PUCRB: PB5 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB5                                                      PWR_PUCRB_PB5_Msk
#define PWR_PUCRB_PB4_Pos                                                  (4UL)		/*!<PWR PUCRB: PB4 (Bit 4) */
#define PWR_PUCRB_PB4_Msk                                                  (0x10UL)		/*!< PWR PUCRB: PB4 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB4                                                      PWR_PUCRB_PB4_Msk
#define PWR_PUCRB_PB3_Pos                                                  (3UL)		/*!<PWR PUCRB: PB3 (Bit 3) */
#define PWR_PUCRB_PB3_Msk                                                  (0x8UL)		/*!< PWR PUCRB: PB3 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB3                                                      PWR_PUCRB_PB3_Msk
#define PWR_PUCRB_PB2_Pos                                                  (2UL)		/*!<PWR PUCRB: PB2 (Bit 2) */
#define PWR_PUCRB_PB2_Msk                                                  (0x4UL)		/*!< PWR PUCRB: PB2 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB2                                                      PWR_PUCRB_PB2_Msk
#define PWR_PUCRB_PB1_Pos                                                  (1UL)		/*!<PWR PUCRB: PB1 (Bit 1) */
#define PWR_PUCRB_PB1_Msk                                                  (0x2UL)		/*!< PWR PUCRB: PB1 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB1                                                      PWR_PUCRB_PB1_Msk
#define PWR_PUCRB_PB0_Pos                                                  (0UL)		/*!<PWR PUCRB: PB0 (Bit 0) */
#define PWR_PUCRB_PB0_Msk                                                  (0x1UL)		/*!< PWR PUCRB: PB0 (Bitfield-Mask: 0x01) */
#define PWR_PUCRB_PB0                                                      PWR_PUCRB_PB0_Msk

/* =====================================================    PDCRB    =====================================================*/ 
#define PWR_PDCRB_PB15_Pos                                                 (15UL)		/*!<PWR PDCRB: PB15 (Bit 15) */
#define PWR_PDCRB_PB15_Msk                                                 (0x8000UL)		/*!< PWR PDCRB: PB15 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB15                                                     PWR_PDCRB_PB15_Msk
#define PWR_PDCRB_PB14_Pos                                                 (14UL)		/*!<PWR PDCRB: PB14 (Bit 14) */
#define PWR_PDCRB_PB14_Msk                                                 (0x4000UL)		/*!< PWR PDCRB: PB14 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB14                                                     PWR_PDCRB_PB14_Msk
#define PWR_PDCRB_PB13_Pos                                                 (13UL)		/*!<PWR PDCRB: PB13 (Bit 13) */
#define PWR_PDCRB_PB13_Msk                                                 (0x2000UL)		/*!< PWR PDCRB: PB13 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB13                                                     PWR_PDCRB_PB13_Msk
#define PWR_PDCRB_PB12_Pos                                                 (12UL)		/*!<PWR PDCRB: PB12 (Bit 12) */
#define PWR_PDCRB_PB12_Msk                                                 (0x1000UL)		/*!< PWR PDCRB: PB12 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB12                                                     PWR_PDCRB_PB12_Msk
#define PWR_PDCRB_PB11_Pos                                                 (11UL)		/*!<PWR PDCRB: PB11 (Bit 11) */
#define PWR_PDCRB_PB11_Msk                                                 (0x800UL)		/*!< PWR PDCRB: PB11 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB11                                                     PWR_PDCRB_PB11_Msk
#define PWR_PDCRB_PB10_Pos                                                 (10UL)		/*!<PWR PDCRB: PB10 (Bit 10) */
#define PWR_PDCRB_PB10_Msk                                                 (0x400UL)		/*!< PWR PDCRB: PB10 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB10                                                     PWR_PDCRB_PB10_Msk
#define PWR_PDCRB_PB9_Pos                                                  (9UL)		/*!<PWR PDCRB: PB9 (Bit 9) */
#define PWR_PDCRB_PB9_Msk                                                  (0x200UL)		/*!< PWR PDCRB: PB9 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB9                                                      PWR_PDCRB_PB9_Msk
#define PWR_PDCRB_PB8_Pos                                                  (8UL)		/*!<PWR PDCRB: PB8 (Bit 8) */
#define PWR_PDCRB_PB8_Msk                                                  (0x100UL)		/*!< PWR PDCRB: PB8 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB8                                                      PWR_PDCRB_PB8_Msk
#define PWR_PDCRB_PB7_Pos                                                  (7UL)		/*!<PWR PDCRB: PB7 (Bit 7) */
#define PWR_PDCRB_PB7_Msk                                                  (0x80UL)		/*!< PWR PDCRB: PB7 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB7                                                      PWR_PDCRB_PB7_Msk
#define PWR_PDCRB_PB6_Pos                                                  (6UL)		/*!<PWR PDCRB: PB6 (Bit 6) */
#define PWR_PDCRB_PB6_Msk                                                  (0x40UL)		/*!< PWR PDCRB: PB6 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB6                                                      PWR_PDCRB_PB6_Msk
#define PWR_PDCRB_PB5_Pos                                                  (5UL)		/*!<PWR PDCRB: PB5 (Bit 5) */
#define PWR_PDCRB_PB5_Msk                                                  (0x20UL)		/*!< PWR PDCRB: PB5 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB5                                                      PWR_PDCRB_PB5_Msk
#define PWR_PDCRB_PB4_Pos                                                  (4UL)		/*!<PWR PDCRB: PB4 (Bit 4) */
#define PWR_PDCRB_PB4_Msk                                                  (0x10UL)		/*!< PWR PDCRB: PB4 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB4                                                      PWR_PDCRB_PB4_Msk
#define PWR_PDCRB_PB3_Pos                                                  (3UL)		/*!<PWR PDCRB: PB3 (Bit 3) */
#define PWR_PDCRB_PB3_Msk                                                  (0x8UL)		/*!< PWR PDCRB: PB3 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB3                                                      PWR_PDCRB_PB3_Msk
#define PWR_PDCRB_PB2_Pos                                                  (2UL)		/*!<PWR PDCRB: PB2 (Bit 2) */
#define PWR_PDCRB_PB2_Msk                                                  (0x4UL)		/*!< PWR PDCRB: PB2 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB2                                                      PWR_PDCRB_PB2_Msk
#define PWR_PDCRB_PB1_Pos                                                  (1UL)		/*!<PWR PDCRB: PB1 (Bit 1) */
#define PWR_PDCRB_PB1_Msk                                                  (0x2UL)		/*!< PWR PDCRB: PB1 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB1                                                      PWR_PDCRB_PB1_Msk
#define PWR_PDCRB_PB0_Pos                                                  (0UL)		/*!<PWR PDCRB: PB0 (Bit 0) */
#define PWR_PDCRB_PB0_Msk                                                  (0x1UL)		/*!< PWR PDCRB: PB0 (Bitfield-Mask: 0x01) */
#define PWR_PDCRB_PB0                                                      PWR_PDCRB_PB0_Msk

/* =====================================================    EWUA    =====================================================*/ 
#define PWR_EWUA_EWU15_Pos                                                 (15UL)		/*!<PWR EWUA: EWU15 (Bit 15) */
#define PWR_EWUA_EWU15_Msk                                                 (0x8000UL)		/*!< PWR EWUA: EWU15 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU15                                                     PWR_EWUA_EWU15_Msk
#define PWR_EWUA_EWU14_Pos                                                 (14UL)		/*!<PWR EWUA: EWU14 (Bit 14) */
#define PWR_EWUA_EWU14_Msk                                                 (0x4000UL)		/*!< PWR EWUA: EWU14 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU14                                                     PWR_EWUA_EWU14_Msk
#define PWR_EWUA_EWU13_Pos                                                 (13UL)		/*!<PWR EWUA: EWU13 (Bit 13) */
#define PWR_EWUA_EWU13_Msk                                                 (0x2000UL)		/*!< PWR EWUA: EWU13 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU13                                                     PWR_EWUA_EWU13_Msk
#define PWR_EWUA_EWU12_Pos                                                 (12UL)		/*!<PWR EWUA: EWU12 (Bit 12) */
#define PWR_EWUA_EWU12_Msk                                                 (0x1000UL)		/*!< PWR EWUA: EWU12 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU12                                                     PWR_EWUA_EWU12_Msk
#define PWR_EWUA_EWU11_Pos                                                 (11UL)		/*!<PWR EWUA: EWU11 (Bit 11) */
#define PWR_EWUA_EWU11_Msk                                                 (0x800UL)		/*!< PWR EWUA: EWU11 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU11                                                     PWR_EWUA_EWU11_Msk
#define PWR_EWUA_EWU10_Pos                                                 (10UL)		/*!<PWR EWUA: EWU10 (Bit 10) */
#define PWR_EWUA_EWU10_Msk                                                 (0x400UL)		/*!< PWR EWUA: EWU10 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU10                                                     PWR_EWUA_EWU10_Msk
#define PWR_EWUA_EWU9_Pos                                                  (9UL)		/*!<PWR EWUA: EWU9 (Bit 9) */
#define PWR_EWUA_EWU9_Msk                                                  (0x200UL)		/*!< PWR EWUA: EWU9 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU9                                                      PWR_EWUA_EWU9_Msk
#define PWR_EWUA_EWU8_Pos                                                  (8UL)		/*!<PWR EWUA: EWU8 (Bit 8) */
#define PWR_EWUA_EWU8_Msk                                                  (0x100UL)		/*!< PWR EWUA: EWU8 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU8                                                      PWR_EWUA_EWU8_Msk
#define PWR_EWUA_EWU7_Pos                                                  (7UL)		/*!<PWR EWUA: EWU7 (Bit 7) */
#define PWR_EWUA_EWU7_Msk                                                  (0x80UL)		/*!< PWR EWUA: EWU7 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU7                                                      PWR_EWUA_EWU7_Msk
#define PWR_EWUA_EWU6_Pos                                                  (6UL)		/*!<PWR EWUA: EWU6 (Bit 6) */
#define PWR_EWUA_EWU6_Msk                                                  (0x40UL)		/*!< PWR EWUA: EWU6 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU6                                                      PWR_EWUA_EWU6_Msk
#define PWR_EWUA_EWU5_Pos                                                  (5UL)		/*!<PWR EWUA: EWU5 (Bit 5) */
#define PWR_EWUA_EWU5_Msk                                                  (0x20UL)		/*!< PWR EWUA: EWU5 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU5                                                      PWR_EWUA_EWU5_Msk
#define PWR_EWUA_EWU4_Pos                                                  (4UL)		/*!<PWR EWUA: EWU4 (Bit 4) */
#define PWR_EWUA_EWU4_Msk                                                  (0x10UL)		/*!< PWR EWUA: EWU4 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU4                                                      PWR_EWUA_EWU4_Msk
#define PWR_EWUA_EWU3_Pos                                                  (3UL)		/*!<PWR EWUA: EWU3 (Bit 3) */
#define PWR_EWUA_EWU3_Msk                                                  (0x8UL)		/*!< PWR EWUA: EWU3 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU3                                                      PWR_EWUA_EWU3_Msk
#define PWR_EWUA_EWU2_Pos                                                  (2UL)		/*!<PWR EWUA: EWU2 (Bit 2) */
#define PWR_EWUA_EWU2_Msk                                                  (0x4UL)		/*!< PWR EWUA: EWU2 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU2                                                      PWR_EWUA_EWU2_Msk
#define PWR_EWUA_EWU1_Pos                                                  (1UL)		/*!<PWR EWUA: EWU1 (Bit 1) */
#define PWR_EWUA_EWU1_Msk                                                  (0x2UL)		/*!< PWR EWUA: EWU1 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU1                                                      PWR_EWUA_EWU1_Msk
#define PWR_EWUA_EWU0_Pos                                                  (0UL)		/*!<PWR EWUA: EWU0 (Bit 0) */
#define PWR_EWUA_EWU0_Msk                                                  (0x1UL)		/*!< PWR EWUA: EWU0 (Bitfield-Mask: 0x01) */
#define PWR_EWUA_EWU0                                                      PWR_EWUA_EWU0_Msk

/* =====================================================    WUPA    =====================================================*/ 
#define PWR_WUPA_WUP15_Pos                                                 (15UL)		/*!<PWR WUPA: WUP15 (Bit 15) */
#define PWR_WUPA_WUP15_Msk                                                 (0x8000UL)		/*!< PWR WUPA: WUP15 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP15                                                     PWR_WUPA_WUP15_Msk
#define PWR_WUPA_WUP14_Pos                                                 (14UL)		/*!<PWR WUPA: WUP14 (Bit 14) */
#define PWR_WUPA_WUP14_Msk                                                 (0x4000UL)		/*!< PWR WUPA: WUP14 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP14                                                     PWR_WUPA_WUP14_Msk
#define PWR_WUPA_WUP13_Pos                                                 (13UL)		/*!<PWR WUPA: WUP13 (Bit 13) */
#define PWR_WUPA_WUP13_Msk                                                 (0x2000UL)		/*!< PWR WUPA: WUP13 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP13                                                     PWR_WUPA_WUP13_Msk
#define PWR_WUPA_WUP12_Pos                                                 (12UL)		/*!<PWR WUPA: WUP12 (Bit 12) */
#define PWR_WUPA_WUP12_Msk                                                 (0x1000UL)		/*!< PWR WUPA: WUP12 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP12                                                     PWR_WUPA_WUP12_Msk
#define PWR_WUPA_WUP11_Pos                                                 (11UL)		/*!<PWR WUPA: WUP11 (Bit 11) */
#define PWR_WUPA_WUP11_Msk                                                 (0x800UL)		/*!< PWR WUPA: WUP11 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP11                                                     PWR_WUPA_WUP11_Msk
#define PWR_WUPA_WUP10_Pos                                                 (10UL)		/*!<PWR WUPA: WUP10 (Bit 10) */
#define PWR_WUPA_WUP10_Msk                                                 (0x400UL)		/*!< PWR WUPA: WUP10 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP10                                                     PWR_WUPA_WUP10_Msk
#define PWR_WUPA_WUP9_Pos                                                  (9UL)		/*!<PWR WUPA: WUP9 (Bit 9) */
#define PWR_WUPA_WUP9_Msk                                                  (0x200UL)		/*!< PWR WUPA: WUP9 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP9                                                      PWR_WUPA_WUP9_Msk
#define PWR_WUPA_WUP8_Pos                                                  (8UL)		/*!<PWR WUPA: WUP8 (Bit 8) */
#define PWR_WUPA_WUP8_Msk                                                  (0x100UL)		/*!< PWR WUPA: WUP8 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP8                                                      PWR_WUPA_WUP8_Msk
#define PWR_WUPA_WUP7_Pos                                                  (7UL)		/*!<PWR WUPA: WUP7 (Bit 7) */
#define PWR_WUPA_WUP7_Msk                                                  (0x80UL)		/*!< PWR WUPA: WUP7 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP7                                                      PWR_WUPA_WUP7_Msk
#define PWR_WUPA_WUP6_Pos                                                  (6UL)		/*!<PWR WUPA: WUP6 (Bit 6) */
#define PWR_WUPA_WUP6_Msk                                                  (0x40UL)		/*!< PWR WUPA: WUP6 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP6                                                      PWR_WUPA_WUP6_Msk
#define PWR_WUPA_WUP5_Pos                                                  (5UL)		/*!<PWR WUPA: WUP5 (Bit 5) */
#define PWR_WUPA_WUP5_Msk                                                  (0x20UL)		/*!< PWR WUPA: WUP5 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP5                                                      PWR_WUPA_WUP5_Msk
#define PWR_WUPA_WUP4_Pos                                                  (4UL)		/*!<PWR WUPA: WUP4 (Bit 4) */
#define PWR_WUPA_WUP4_Msk                                                  (0x10UL)		/*!< PWR WUPA: WUP4 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP4                                                      PWR_WUPA_WUP4_Msk
#define PWR_WUPA_WUP3_Pos                                                  (3UL)		/*!<PWR WUPA: WUP3 (Bit 3) */
#define PWR_WUPA_WUP3_Msk                                                  (0x8UL)		/*!< PWR WUPA: WUP3 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP3                                                      PWR_WUPA_WUP3_Msk
#define PWR_WUPA_WUP2_Pos                                                  (2UL)		/*!<PWR WUPA: WUP2 (Bit 2) */
#define PWR_WUPA_WUP2_Msk                                                  (0x4UL)		/*!< PWR WUPA: WUP2 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP2                                                      PWR_WUPA_WUP2_Msk
#define PWR_WUPA_WUP1_Pos                                                  (1UL)		/*!<PWR WUPA: WUP1 (Bit 1) */
#define PWR_WUPA_WUP1_Msk                                                  (0x2UL)		/*!< PWR WUPA: WUP1 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP1                                                      PWR_WUPA_WUP1_Msk
#define PWR_WUPA_WUP0_Pos                                                  (0UL)		/*!<PWR WUPA: WUP0 (Bit 0) */
#define PWR_WUPA_WUP0_Msk                                                  (0x1UL)		/*!< PWR WUPA: WUP0 (Bitfield-Mask: 0x01) */
#define PWR_WUPA_WUP0                                                      PWR_WUPA_WUP0_Msk

/* =====================================================    WUFA    =====================================================*/ 
#define PWR_WUFA_WUF15_Pos                                                 (15UL)		/*!<PWR WUFA: WUF15 (Bit 15) */
#define PWR_WUFA_WUF15_Msk                                                 (0x8000UL)		/*!< PWR WUFA: WUF15 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF15                                                     PWR_WUFA_WUF15_Msk
#define PWR_WUFA_WUF14_Pos                                                 (14UL)		/*!<PWR WUFA: WUF14 (Bit 14) */
#define PWR_WUFA_WUF14_Msk                                                 (0x4000UL)		/*!< PWR WUFA: WUF14 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF14                                                     PWR_WUFA_WUF14_Msk
#define PWR_WUFA_WUF13_Pos                                                 (13UL)		/*!<PWR WUFA: WUF13 (Bit 13) */
#define PWR_WUFA_WUF13_Msk                                                 (0x2000UL)		/*!< PWR WUFA: WUF13 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF13                                                     PWR_WUFA_WUF13_Msk
#define PWR_WUFA_WUF12_Pos                                                 (12UL)		/*!<PWR WUFA: WUF12 (Bit 12) */
#define PWR_WUFA_WUF12_Msk                                                 (0x1000UL)		/*!< PWR WUFA: WUF12 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF12                                                     PWR_WUFA_WUF12_Msk
#define PWR_WUFA_WUF11_Pos                                                 (11UL)		/*!<PWR WUFA: WUF11 (Bit 11) */
#define PWR_WUFA_WUF11_Msk                                                 (0x800UL)		/*!< PWR WUFA: WUF11 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF11                                                     PWR_WUFA_WUF11_Msk
#define PWR_WUFA_WUF10_Pos                                                 (10UL)		/*!<PWR WUFA: WUF10 (Bit 10) */
#define PWR_WUFA_WUF10_Msk                                                 (0x400UL)		/*!< PWR WUFA: WUF10 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF10                                                     PWR_WUFA_WUF10_Msk
#define PWR_WUFA_WUF9_Pos                                                  (9UL)		/*!<PWR WUFA: WUF9 (Bit 9) */
#define PWR_WUFA_WUF9_Msk                                                  (0x200UL)		/*!< PWR WUFA: WUF9 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF9                                                      PWR_WUFA_WUF9_Msk
#define PWR_WUFA_WUF8_Pos                                                  (8UL)		/*!<PWR WUFA: WUF8 (Bit 8) */
#define PWR_WUFA_WUF8_Msk                                                  (0x100UL)		/*!< PWR WUFA: WUF8 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF8                                                      PWR_WUFA_WUF8_Msk
#define PWR_WUFA_WUF7_Pos                                                  (7UL)		/*!<PWR WUFA: WUF7 (Bit 7) */
#define PWR_WUFA_WUF7_Msk                                                  (0x80UL)		/*!< PWR WUFA: WUF7 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF7                                                      PWR_WUFA_WUF7_Msk
#define PWR_WUFA_WUF6_Pos                                                  (6UL)		/*!<PWR WUFA: WUF6 (Bit 6) */
#define PWR_WUFA_WUF6_Msk                                                  (0x40UL)		/*!< PWR WUFA: WUF6 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF6                                                      PWR_WUFA_WUF6_Msk
#define PWR_WUFA_WUF5_Pos                                                  (5UL)		/*!<PWR WUFA: WUF5 (Bit 5) */
#define PWR_WUFA_WUF5_Msk                                                  (0x20UL)		/*!< PWR WUFA: WUF5 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF5                                                      PWR_WUFA_WUF5_Msk
#define PWR_WUFA_WUF4_Pos                                                  (4UL)		/*!<PWR WUFA: WUF4 (Bit 4) */
#define PWR_WUFA_WUF4_Msk                                                  (0x10UL)		/*!< PWR WUFA: WUF4 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF4                                                      PWR_WUFA_WUF4_Msk
#define PWR_WUFA_WUF3_Pos                                                  (3UL)		/*!<PWR WUFA: WUF3 (Bit 3) */
#define PWR_WUFA_WUF3_Msk                                                  (0x8UL)		/*!< PWR WUFA: WUF3 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF3                                                      PWR_WUFA_WUF3_Msk
#define PWR_WUFA_WUF2_Pos                                                  (2UL)		/*!<PWR WUFA: WUF2 (Bit 2) */
#define PWR_WUFA_WUF2_Msk                                                  (0x4UL)		/*!< PWR WUFA: WUF2 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF2                                                      PWR_WUFA_WUF2_Msk
#define PWR_WUFA_WUF1_Pos                                                  (1UL)		/*!<PWR WUFA: WUF1 (Bit 1) */
#define PWR_WUFA_WUF1_Msk                                                  (0x2UL)		/*!< PWR WUFA: WUF1 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF1                                                      PWR_WUFA_WUF1_Msk
#define PWR_WUFA_WUF0_Pos                                                  (0UL)		/*!<PWR WUFA: WUF0 (Bit 0) */
#define PWR_WUFA_WUF0_Msk                                                  (0x1UL)		/*!< PWR WUFA: WUF0 (Bitfield-Mask: 0x01) */
#define PWR_WUFA_WUF0                                                      PWR_WUFA_WUF0_Msk

/* =====================================================    EWUB    =====================================================*/ 
#define PWR_EWUB_EWU15_Pos                                                 (15UL)		/*!<PWR EWUB: EWU15 (Bit 15) */
#define PWR_EWUB_EWU15_Msk                                                 (0x8000UL)		/*!< PWR EWUB: EWU15 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU15                                                     PWR_EWUB_EWU15_Msk
#define PWR_EWUB_EWU14_Pos                                                 (14UL)		/*!<PWR EWUB: EWU14 (Bit 14) */
#define PWR_EWUB_EWU14_Msk                                                 (0x4000UL)		/*!< PWR EWUB: EWU14 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU14                                                     PWR_EWUB_EWU14_Msk
#define PWR_EWUB_EWU13_Pos                                                 (13UL)		/*!<PWR EWUB: EWU13 (Bit 13) */
#define PWR_EWUB_EWU13_Msk                                                 (0x2000UL)		/*!< PWR EWUB: EWU13 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU13                                                     PWR_EWUB_EWU13_Msk
#define PWR_EWUB_EWU12_Pos                                                 (12UL)		/*!<PWR EWUB: EWU12 (Bit 12) */
#define PWR_EWUB_EWU12_Msk                                                 (0x1000UL)		/*!< PWR EWUB: EWU12 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU12                                                     PWR_EWUB_EWU12_Msk
#define PWR_EWUB_EWU11_Pos                                                 (11UL)		/*!<PWR EWUB: EWU11 (Bit 11) */
#define PWR_EWUB_EWU11_Msk                                                 (0x800UL)		/*!< PWR EWUB: EWU11 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU11                                                     PWR_EWUB_EWU11_Msk
#define PWR_EWUB_EWU10_Pos                                                 (10UL)		/*!<PWR EWUB: EWU10 (Bit 10) */
#define PWR_EWUB_EWU10_Msk                                                 (0x400UL)		/*!< PWR EWUB: EWU10 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU10                                                     PWR_EWUB_EWU10_Msk
#define PWR_EWUB_EWU9_Pos                                                  (9UL)		/*!<PWR EWUB: EWU9 (Bit 9) */
#define PWR_EWUB_EWU9_Msk                                                  (0x200UL)		/*!< PWR EWUB: EWU9 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU9                                                      PWR_EWUB_EWU9_Msk
#define PWR_EWUB_EWU8_Pos                                                  (8UL)		/*!<PWR EWUB: EWU8 (Bit 8) */
#define PWR_EWUB_EWU8_Msk                                                  (0x100UL)		/*!< PWR EWUB: EWU8 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU8                                                      PWR_EWUB_EWU8_Msk
#define PWR_EWUB_EWU7_Pos                                                  (7UL)		/*!<PWR EWUB: EWU7 (Bit 7) */
#define PWR_EWUB_EWU7_Msk                                                  (0x80UL)		/*!< PWR EWUB: EWU7 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU7                                                      PWR_EWUB_EWU7_Msk
#define PWR_EWUB_EWU6_Pos                                                  (6UL)		/*!<PWR EWUB: EWU6 (Bit 6) */
#define PWR_EWUB_EWU6_Msk                                                  (0x40UL)		/*!< PWR EWUB: EWU6 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU6                                                      PWR_EWUB_EWU6_Msk
#define PWR_EWUB_EWU5_Pos                                                  (5UL)		/*!<PWR EWUB: EWU5 (Bit 5) */
#define PWR_EWUB_EWU5_Msk                                                  (0x20UL)		/*!< PWR EWUB: EWU5 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU5                                                      PWR_EWUB_EWU5_Msk
#define PWR_EWUB_EWU4_Pos                                                  (4UL)		/*!<PWR EWUB: EWU4 (Bit 4) */
#define PWR_EWUB_EWU4_Msk                                                  (0x10UL)		/*!< PWR EWUB: EWU4 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU4                                                      PWR_EWUB_EWU4_Msk
#define PWR_EWUB_EWU3_Pos                                                  (3UL)		/*!<PWR EWUB: EWU3 (Bit 3) */
#define PWR_EWUB_EWU3_Msk                                                  (0x8UL)		/*!< PWR EWUB: EWU3 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU3                                                      PWR_EWUB_EWU3_Msk
#define PWR_EWUB_EWU2_Pos                                                  (2UL)		/*!<PWR EWUB: EWU2 (Bit 2) */
#define PWR_EWUB_EWU2_Msk                                                  (0x4UL)		/*!< PWR EWUB: EWU2 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU2                                                      PWR_EWUB_EWU2_Msk
#define PWR_EWUB_EWU1_Pos                                                  (1UL)		/*!<PWR EWUB: EWU1 (Bit 1) */
#define PWR_EWUB_EWU1_Msk                                                  (0x2UL)		/*!< PWR EWUB: EWU1 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU1                                                      PWR_EWUB_EWU1_Msk
#define PWR_EWUB_EWU0_Pos                                                  (0UL)		/*!<PWR EWUB: EWU0 (Bit 0) */
#define PWR_EWUB_EWU0_Msk                                                  (0x1UL)		/*!< PWR EWUB: EWU0 (Bitfield-Mask: 0x01) */
#define PWR_EWUB_EWU0                                                      PWR_EWUB_EWU0_Msk

/* =====================================================    WUPB    =====================================================*/ 
#define PWR_WUPB_WUP15_Pos                                                 (15UL)		/*!<PWR WUPB: WUP15 (Bit 15) */
#define PWR_WUPB_WUP15_Msk                                                 (0x8000UL)		/*!< PWR WUPB: WUP15 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP15                                                     PWR_WUPB_WUP15_Msk
#define PWR_WUPB_WUP14_Pos                                                 (14UL)		/*!<PWR WUPB: WUP14 (Bit 14) */
#define PWR_WUPB_WUP14_Msk                                                 (0x4000UL)		/*!< PWR WUPB: WUP14 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP14                                                     PWR_WUPB_WUP14_Msk
#define PWR_WUPB_WUP13_Pos                                                 (13UL)		/*!<PWR WUPB: WUP13 (Bit 13) */
#define PWR_WUPB_WUP13_Msk                                                 (0x2000UL)		/*!< PWR WUPB: WUP13 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP13                                                     PWR_WUPB_WUP13_Msk
#define PWR_WUPB_WUP12_Pos                                                 (12UL)		/*!<PWR WUPB: WUP12 (Bit 12) */
#define PWR_WUPB_WUP12_Msk                                                 (0x1000UL)		/*!< PWR WUPB: WUP12 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP12                                                     PWR_WUPB_WUP12_Msk
#define PWR_WUPB_WUP11_Pos                                                 (11UL)		/*!<PWR WUPB: WUP11 (Bit 11) */
#define PWR_WUPB_WUP11_Msk                                                 (0x800UL)		/*!< PWR WUPB: WUP11 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP11                                                     PWR_WUPB_WUP11_Msk
#define PWR_WUPB_WUP10_Pos                                                 (10UL)		/*!<PWR WUPB: WUP10 (Bit 10) */
#define PWR_WUPB_WUP10_Msk                                                 (0x400UL)		/*!< PWR WUPB: WUP10 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP10                                                     PWR_WUPB_WUP10_Msk
#define PWR_WUPB_WUP9_Pos                                                  (9UL)		/*!<PWR WUPB: WUP9 (Bit 9) */
#define PWR_WUPB_WUP9_Msk                                                  (0x200UL)		/*!< PWR WUPB: WUP9 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP9                                                      PWR_WUPB_WUP9_Msk
#define PWR_WUPB_WUP8_Pos                                                  (8UL)		/*!<PWR WUPB: WUP8 (Bit 8) */
#define PWR_WUPB_WUP8_Msk                                                  (0x100UL)		/*!< PWR WUPB: WUP8 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP8                                                      PWR_WUPB_WUP8_Msk
#define PWR_WUPB_WUP7_Pos                                                  (7UL)		/*!<PWR WUPB: WUP7 (Bit 7) */
#define PWR_WUPB_WUP7_Msk                                                  (0x80UL)		/*!< PWR WUPB: WUP7 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP7                                                      PWR_WUPB_WUP7_Msk
#define PWR_WUPB_WUP6_Pos                                                  (6UL)		/*!<PWR WUPB: WUP6 (Bit 6) */
#define PWR_WUPB_WUP6_Msk                                                  (0x40UL)		/*!< PWR WUPB: WUP6 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP6                                                      PWR_WUPB_WUP6_Msk
#define PWR_WUPB_WUP5_Pos                                                  (5UL)		/*!<PWR WUPB: WUP5 (Bit 5) */
#define PWR_WUPB_WUP5_Msk                                                  (0x20UL)		/*!< PWR WUPB: WUP5 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP5                                                      PWR_WUPB_WUP5_Msk
#define PWR_WUPB_WUP4_Pos                                                  (4UL)		/*!<PWR WUPB: WUP4 (Bit 4) */
#define PWR_WUPB_WUP4_Msk                                                  (0x10UL)		/*!< PWR WUPB: WUP4 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP4                                                      PWR_WUPB_WUP4_Msk
#define PWR_WUPB_WUP3_Pos                                                  (3UL)		/*!<PWR WUPB: WUP3 (Bit 3) */
#define PWR_WUPB_WUP3_Msk                                                  (0x8UL)		/*!< PWR WUPB: WUP3 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP3                                                      PWR_WUPB_WUP3_Msk
#define PWR_WUPB_WUP2_Pos                                                  (2UL)		/*!<PWR WUPB: WUP2 (Bit 2) */
#define PWR_WUPB_WUP2_Msk                                                  (0x4UL)		/*!< PWR WUPB: WUP2 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP2                                                      PWR_WUPB_WUP2_Msk
#define PWR_WUPB_WUP1_Pos                                                  (1UL)		/*!<PWR WUPB: WUP1 (Bit 1) */
#define PWR_WUPB_WUP1_Msk                                                  (0x2UL)		/*!< PWR WUPB: WUP1 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP1                                                      PWR_WUPB_WUP1_Msk
#define PWR_WUPB_WUP0_Pos                                                  (0UL)		/*!<PWR WUPB: WUP0 (Bit 0) */
#define PWR_WUPB_WUP0_Msk                                                  (0x1UL)		/*!< PWR WUPB: WUP0 (Bitfield-Mask: 0x01) */
#define PWR_WUPB_WUP0                                                      PWR_WUPB_WUP0_Msk

/* =====================================================    WUFB    =====================================================*/ 
#define PWR_WUFB_WUF15_Pos                                                 (15UL)		/*!<PWR WUFB: WUF15 (Bit 15) */
#define PWR_WUFB_WUF15_Msk                                                 (0x8000UL)		/*!< PWR WUFB: WUF15 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF15                                                     PWR_WUFB_WUF15_Msk
#define PWR_WUFB_WUF14_Pos                                                 (14UL)		/*!<PWR WUFB: WUF14 (Bit 14) */
#define PWR_WUFB_WUF14_Msk                                                 (0x4000UL)		/*!< PWR WUFB: WUF14 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF14                                                     PWR_WUFB_WUF14_Msk
#define PWR_WUFB_WUF13_Pos                                                 (13UL)		/*!<PWR WUFB: WUF13 (Bit 13) */
#define PWR_WUFB_WUF13_Msk                                                 (0x2000UL)		/*!< PWR WUFB: WUF13 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF13                                                     PWR_WUFB_WUF13_Msk
#define PWR_WUFB_WUF12_Pos                                                 (12UL)		/*!<PWR WUFB: WUF12 (Bit 12) */
#define PWR_WUFB_WUF12_Msk                                                 (0x1000UL)		/*!< PWR WUFB: WUF12 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF12                                                     PWR_WUFB_WUF12_Msk
#define PWR_WUFB_WUF11_Pos                                                 (11UL)		/*!<PWR WUFB: WUF11 (Bit 11) */
#define PWR_WUFB_WUF11_Msk                                                 (0x800UL)		/*!< PWR WUFB: WUF11 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF11                                                     PWR_WUFB_WUF11_Msk
#define PWR_WUFB_WUF10_Pos                                                 (10UL)		/*!<PWR WUFB: WUF10 (Bit 10) */
#define PWR_WUFB_WUF10_Msk                                                 (0x400UL)		/*!< PWR WUFB: WUF10 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF10                                                     PWR_WUFB_WUF10_Msk
#define PWR_WUFB_WUF9_Pos                                                  (9UL)		/*!<PWR WUFB: WUF9 (Bit 9) */
#define PWR_WUFB_WUF9_Msk                                                  (0x200UL)		/*!< PWR WUFB: WUF9 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF9                                                      PWR_WUFB_WUF9_Msk
#define PWR_WUFB_WUF8_Pos                                                  (8UL)		/*!<PWR WUFB: WUF8 (Bit 8) */
#define PWR_WUFB_WUF8_Msk                                                  (0x100UL)		/*!< PWR WUFB: WUF8 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF8                                                      PWR_WUFB_WUF8_Msk
#define PWR_WUFB_WUF7_Pos                                                  (7UL)		/*!<PWR WUFB: WUF7 (Bit 7) */
#define PWR_WUFB_WUF7_Msk                                                  (0x80UL)		/*!< PWR WUFB: WUF7 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF7                                                      PWR_WUFB_WUF7_Msk
#define PWR_WUFB_WUF6_Pos                                                  (6UL)		/*!<PWR WUFB: WUF6 (Bit 6) */
#define PWR_WUFB_WUF6_Msk                                                  (0x40UL)		/*!< PWR WUFB: WUF6 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF6                                                      PWR_WUFB_WUF6_Msk
#define PWR_WUFB_WUF5_Pos                                                  (5UL)		/*!<PWR WUFB: WUF5 (Bit 5) */
#define PWR_WUFB_WUF5_Msk                                                  (0x20UL)		/*!< PWR WUFB: WUF5 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF5                                                      PWR_WUFB_WUF5_Msk
#define PWR_WUFB_WUF4_Pos                                                  (4UL)		/*!<PWR WUFB: WUF4 (Bit 4) */
#define PWR_WUFB_WUF4_Msk                                                  (0x10UL)		/*!< PWR WUFB: WUF4 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF4                                                      PWR_WUFB_WUF4_Msk
#define PWR_WUFB_WUF3_Pos                                                  (3UL)		/*!<PWR WUFB: WUF3 (Bit 3) */
#define PWR_WUFB_WUF3_Msk                                                  (0x8UL)		/*!< PWR WUFB: WUF3 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF3                                                      PWR_WUFB_WUF3_Msk
#define PWR_WUFB_WUF2_Pos                                                  (2UL)		/*!<PWR WUFB: WUF2 (Bit 2) */
#define PWR_WUFB_WUF2_Msk                                                  (0x4UL)		/*!< PWR WUFB: WUF2 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF2                                                      PWR_WUFB_WUF2_Msk
#define PWR_WUFB_WUF1_Pos                                                  (1UL)		/*!<PWR WUFB: WUF1 (Bit 1) */
#define PWR_WUFB_WUF1_Msk                                                  (0x2UL)		/*!< PWR WUFB: WUF1 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF1                                                      PWR_WUFB_WUF1_Msk
#define PWR_WUFB_WUF0_Pos                                                  (0UL)		/*!<PWR WUFB: WUF0 (Bit 0) */
#define PWR_WUFB_WUF0_Msk                                                  (0x1UL)		/*!< PWR WUFB: WUF0 (Bitfield-Mask: 0x01) */
#define PWR_WUFB_WUF0                                                      PWR_WUFB_WUF0_Msk

/* =====================================================    SDWN_WUEN    =====================================================*/ 
#define PWR_SDWN_WUEN_WUEN_Pos                                             (0UL)		/*!<PWR SDWN_WUEN: WUEN (Bit 0) */
#define PWR_SDWN_WUEN_WUEN_Msk                                             (0x1UL)		/*!< PWR SDWN_WUEN: WUEN (Bitfield-Mask: 0x01) */
#define PWR_SDWN_WUEN_WUEN                                                 PWR_SDWN_WUEN_WUEN_Msk

/* =====================================================    SDWN_WUPOL    =====================================================*/ 
#define PWR_SDWN_WUPOL_WUPOL_Pos                                           (0UL)		/*!<PWR SDWN_WUPOL: WUPOL (Bit 0) */
#define PWR_SDWN_WUPOL_WUPOL_Msk                                           (0x1UL)		/*!< PWR SDWN_WUPOL: WUPOL (Bitfield-Mask: 0x01) */
#define PWR_SDWN_WUPOL_WUPOL                                               PWR_SDWN_WUPOL_WUPOL_Msk

/* =====================================================    SDWN_WUF    =====================================================*/ 
#define PWR_SDWN_WUF_WUF_Pos                                               (0UL)		/*!<PWR SDWN_WUF: WUF (Bit 0) */
#define PWR_SDWN_WUF_WUF_Msk                                               (0x1UL)		/*!< PWR SDWN_WUF: WUF (Bitfield-Mask: 0x01) */
#define PWR_SDWN_WUF_WUF                                                   PWR_SDWN_WUF_WUF_Msk

/* =====================================================    BOF_TUNE    =====================================================*/ 
#define PWR_BOF_TUNE_BOF_TUNE_Pos                                          (0UL)		/*!<PWR BOF_TUNE: BOF_TUNE (Bit 0) */
#define PWR_BOF_TUNE_BOF_TUNE_Msk                                          (0x7UL)		/*!< PWR BOF_TUNE: BOF_TUNE (Bitfield-Mask: 0x07) */
#define PWR_BOF_TUNE_BOF_TUNE                                              PWR_BOF_TUNE_BOF_TUNE_Msk
#define PWR_BOF_TUNE_BOF_TUNE_0                                            (0x1U << PWR_BOF_TUNE_BOF_TUNE_Pos)
#define PWR_BOF_TUNE_BOF_TUNE_1                                            (0x2U << PWR_BOF_TUNE_BOF_TUNE_Pos)
#define PWR_BOF_TUNE_BOF_TUNE_2                                            (0x4U << PWR_BOF_TUNE_BOF_TUNE_Pos)

/* =====================================================    DBGR    =====================================================*/ 
#define PWR_DBGR_DEEPSTOP2_Pos                                             (0UL)		/*!<PWR DBGR: DEEPSTOP2 (Bit 0) */
#define PWR_DBGR_DEEPSTOP2_Msk                                             (0x1UL)		/*!< PWR DBGR: DEEPSTOP2 (Bitfield-Mask: 0x01) */
#define PWR_DBGR_DEEPSTOP2                                                 PWR_DBGR_DEEPSTOP2_Msk

/* =====================================================    EXTSRR    =====================================================*/ 
#define PWR_EXTSRR_RFPHASEF_Pos                                            (10UL)		/*!<PWR EXTSRR: RFPHASEF (Bit 10) */
#define PWR_EXTSRR_RFPHASEF_Msk                                            (0x400UL)		/*!< PWR EXTSRR: RFPHASEF (Bitfield-Mask: 0x01) */
#define PWR_EXTSRR_RFPHASEF                                                PWR_EXTSRR_RFPHASEF_Msk
#define PWR_EXTSRR_DEEPSTOPF_Pos                                           (9UL)		/*!<PWR EXTSRR: DEEPSTOPF (Bit 9) */
#define PWR_EXTSRR_DEEPSTOPF_Msk                                           (0x200UL)		/*!< PWR EXTSRR: DEEPSTOPF (Bitfield-Mask: 0x01) */
#define PWR_EXTSRR_DEEPSTOPF                                               PWR_EXTSRR_DEEPSTOPF_Msk

/* =====================================================    TRIMR    =====================================================*/ 
#define PWR_TRIMR_SMPS_TRIM_Pos                                            (8UL)		/*!<PWR TRIMR: SMPS_TRIM (Bit 8) */
#define PWR_TRIMR_SMPS_TRIM_Msk                                            (0x700UL)		/*!< PWR TRIMR: SMPS_TRIM (Bitfield-Mask: 0x07) */
#define PWR_TRIMR_SMPS_TRIM                                                PWR_TRIMR_SMPS_TRIM_Msk
#define PWR_TRIMR_SMPS_TRIM_0                                              (0x1U << PWR_TRIMR_SMPS_TRIM_Pos)
#define PWR_TRIMR_SMPS_TRIM_1                                              (0x2U << PWR_TRIMR_SMPS_TRIM_Pos)
#define PWR_TRIMR_SMPS_TRIM_2                                              (0x4U << PWR_TRIMR_SMPS_TRIM_Pos)
#define PWR_TRIMR_TRIM_MR_Pos                                              (4UL)		/*!<PWR TRIMR: TRIM_MR (Bit 4) */
#define PWR_TRIMR_TRIM_MR_Msk                                              (0xf0UL)		/*!< PWR TRIMR: TRIM_MR (Bitfield-Mask: 0x0f) */
#define PWR_TRIMR_TRIM_MR                                                  PWR_TRIMR_TRIM_MR_Msk
#define PWR_TRIMR_TRIM_MR_0                                                (0x1U << PWR_TRIMR_TRIM_MR_Pos)
#define PWR_TRIMR_TRIM_MR_1                                                (0x2U << PWR_TRIMR_TRIM_MR_Pos)
#define PWR_TRIMR_TRIM_MR_2                                                (0x4U << PWR_TRIMR_TRIM_MR_Pos)
#define PWR_TRIMR_TRIM_MR_3                                                (0x8U << PWR_TRIMR_TRIM_MR_Pos)
#define PWR_TRIMR_RFD_REG_TRIM_Pos                                         (0UL)		/*!<PWR TRIMR: RFD_REG_TRIM (Bit 0) */
#define PWR_TRIMR_RFD_REG_TRIM_Msk                                         (0x7UL)		/*!< PWR TRIMR: RFD_REG_TRIM (Bitfield-Mask: 0x07) */
#define PWR_TRIMR_RFD_REG_TRIM                                             PWR_TRIMR_RFD_REG_TRIM_Msk
#define PWR_TRIMR_RFD_REG_TRIM_0                                           (0x1U << PWR_TRIMR_RFD_REG_TRIM_Pos)
#define PWR_TRIMR_RFD_REG_TRIM_1                                           (0x2U << PWR_TRIMR_RFD_REG_TRIM_Pos)
#define PWR_TRIMR_RFD_REG_TRIM_2                                           (0x4U << PWR_TRIMR_RFD_REG_TRIM_Pos)

/* =====================================================    ENGTRIM    =====================================================*/ 
#define PWR_ENGTRIM_SMPS_TRIM_Pos                                          (11UL)		/*!<PWR ENGTRIM: SMPS_TRIM (Bit 11) */
#define PWR_ENGTRIM_SMPS_TRIM_Msk                                          (0x3800UL)		/*!< PWR ENGTRIM: SMPS_TRIM (Bitfield-Mask: 0x07) */
#define PWR_ENGTRIM_SMPS_TRIM                                              PWR_ENGTRIM_SMPS_TRIM_Msk
#define PWR_ENGTRIM_SMPS_TRIM_0                                            (0x1U << PWR_ENGTRIM_SMPS_TRIM_Pos)
#define PWR_ENGTRIM_SMPS_TRIM_1                                            (0x2U << PWR_ENGTRIM_SMPS_TRIM_Pos)
#define PWR_ENGTRIM_SMPS_TRIM_2                                            (0x4U << PWR_ENGTRIM_SMPS_TRIM_Pos)
#define PWR_ENGTRIM_SMPSTRIMEN_Pos                                         (10UL)		/*!<PWR ENGTRIM: SMPSTRIMEN (Bit 10) */
#define PWR_ENGTRIM_SMPSTRIMEN_Msk                                         (0x400UL)		/*!< PWR ENGTRIM: SMPSTRIMEN (Bitfield-Mask: 0x01) */
#define PWR_ENGTRIM_SMPSTRIMEN                                             PWR_ENGTRIM_SMPSTRIMEN_Msk
#define PWR_ENGTRIM_TRIM_MR_Pos                                            (6UL)		/*!<PWR ENGTRIM: TRIM_MR (Bit 6) */
#define PWR_ENGTRIM_TRIM_MR_Msk                                            (0x3c0UL)		/*!< PWR ENGTRIM: TRIM_MR (Bitfield-Mask: 0x0f) */
#define PWR_ENGTRIM_TRIM_MR                                                PWR_ENGTRIM_TRIM_MR_Msk
#define PWR_ENGTRIM_TRIM_MR_0                                              (0x1U << PWR_ENGTRIM_TRIM_MR_Pos)
#define PWR_ENGTRIM_TRIM_MR_1                                              (0x2U << PWR_ENGTRIM_TRIM_MR_Pos)
#define PWR_ENGTRIM_TRIM_MR_2                                              (0x4U << PWR_ENGTRIM_TRIM_MR_Pos)
#define PWR_ENGTRIM_TRIM_MR_3                                              (0x8U << PWR_ENGTRIM_TRIM_MR_Pos)
#define PWR_ENGTRIM_TRIMMREN_Pos                                           (5UL)		/*!<PWR ENGTRIM: TRIMMREN (Bit 5) */
#define PWR_ENGTRIM_TRIMMREN_Msk                                           (0x20UL)		/*!< PWR ENGTRIM: TRIMMREN (Bitfield-Mask: 0x01) */
#define PWR_ENGTRIM_TRIMMREN                                               PWR_ENGTRIM_TRIMMREN_Msk
#define PWR_ENGTRIM_TRIM_RFDREG_Pos                                        (1UL)		/*!<PWR ENGTRIM: TRIM_RFDREG (Bit 1) */
#define PWR_ENGTRIM_TRIM_RFDREG_Msk                                        (0xeUL)		/*!< PWR ENGTRIM: TRIM_RFDREG (Bitfield-Mask: 0x07) */
#define PWR_ENGTRIM_TRIM_RFDREG                                            PWR_ENGTRIM_TRIM_RFDREG_Msk
#define PWR_ENGTRIM_TRIM_RFDREG_0                                          (0x1U << PWR_ENGTRIM_TRIM_RFDREG_Pos)
#define PWR_ENGTRIM_TRIM_RFDREG_1                                          (0x2U << PWR_ENGTRIM_TRIM_RFDREG_Pos)
#define PWR_ENGTRIM_TRIM_RFDREG_2                                          (0x4U << PWR_ENGTRIM_TRIM_RFDREG_Pos)
#define PWR_ENGTRIM_TRIMRFDREGEN_Pos                                       (0UL)		/*!<PWR ENGTRIM: TRIMRFDREGEN (Bit 0) */
#define PWR_ENGTRIM_TRIMRFDREGEN_Msk                                       (0x1UL)		/*!< PWR ENGTRIM: TRIMRFDREGEN (Bitfield-Mask: 0x01) */
#define PWR_ENGTRIM_TRIMRFDREGEN                                           PWR_ENGTRIM_TRIMRFDREGEN_Msk

/* =====================================================    ENGTRIM2    =====================================================*/ 
#define PWR_ENGTRIM2_BOF_TRIM_Pos                                          (1UL)		/*!<PWR ENGTRIM2: BOF_TRIM (Bit 1) */
#define PWR_ENGTRIM2_BOF_TRIM_Msk                                          (0xeUL)		/*!< PWR ENGTRIM2: BOF_TRIM (Bitfield-Mask: 0x07) */
#define PWR_ENGTRIM2_BOF_TRIM                                              PWR_ENGTRIM2_BOF_TRIM_Msk
#define PWR_ENGTRIM2_BOF_TRIM_0                                            (0x1U << PWR_ENGTRIM2_BOF_TRIM_Pos)
#define PWR_ENGTRIM2_BOF_TRIM_1                                            (0x2U << PWR_ENGTRIM2_BOF_TRIM_Pos)
#define PWR_ENGTRIM2_BOF_TRIM_2                                            (0x4U << PWR_ENGTRIM2_BOF_TRIM_Pos)
#define PWR_ENGTRIM2_BOFTRIMEN_Pos                                         (0UL)		/*!<PWR ENGTRIM2: BOFTRIMEN (Bit 0) */
#define PWR_ENGTRIM2_BOFTRIMEN_Msk                                         (0x1UL)		/*!< PWR ENGTRIM2: BOFTRIMEN (Bitfield-Mask: 0x01) */
#define PWR_ENGTRIM2_BOFTRIMEN                                             PWR_ENGTRIM2_BOFTRIMEN_Msk


/* ============================================================================================================================*/ 
/*=====================                                        RNG                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define RNG_CR_TST_CLK_Pos                                                 (3UL)		/*!<RNG CR: TST_CLK (Bit 3) */
#define RNG_CR_TST_CLK_Msk                                                 (0x8UL)		/*!< RNG CR: TST_CLK (Bitfield-Mask: 0x01) */
#define RNG_CR_TST_CLK                                                     RNG_CR_TST_CLK_Msk
#define RNG_CR_RNG_DIS_Pos                                                 (2UL)		/*!<RNG CR: RNG_DIS (Bit 2) */
#define RNG_CR_RNG_DIS_Msk                                                 (0x4UL)		/*!< RNG CR: RNG_DIS (Bitfield-Mask: 0x01) */
#define RNG_CR_RNG_DIS                                                     RNG_CR_RNG_DIS_Msk

/* =====================================================    SR    =====================================================*/ 
#define RNG_SR_FAULT_Pos                                                   (2UL)		/*!<RNG SR: FAULT (Bit 2) */
#define RNG_SR_FAULT_Msk                                                   (0x4UL)		/*!< RNG SR: FAULT (Bitfield-Mask: 0x01) */
#define RNG_SR_FAULT                                                       RNG_SR_FAULT_Msk
#define RNG_SR_REVCLK_Pos                                                  (1UL)		/*!<RNG SR: REVCLK (Bit 1) */
#define RNG_SR_REVCLK_Msk                                                  (0x2UL)		/*!< RNG SR: REVCLK (Bitfield-Mask: 0x01) */
#define RNG_SR_REVCLK                                                      RNG_SR_REVCLK_Msk
#define RNG_SR_RNGRDY_Pos                                                  (0UL)		/*!<RNG SR: RNGRDY (Bit 0) */
#define RNG_SR_RNGRDY_Msk                                                  (0x1UL)		/*!< RNG SR: RNGRDY (Bitfield-Mask: 0x01) */
#define RNG_SR_RNGRDY                                                      RNG_SR_RNGRDY_Msk

/* =====================================================    VAL    =====================================================*/ 
#define RNG_VAL_RANDOM_VALUE_Pos                                           (0UL)		/*!<RNG VAL: RANDOM_VALUE (Bit 0) */
#define RNG_VAL_RANDOM_VALUE_Msk                                           (0xffffUL)		/*!< RNG VAL: RANDOM_VALUE (Bitfield-Mask: 0xffff) */
#define RNG_VAL_RANDOM_VALUE                                               RNG_VAL_RANDOM_VALUE_Msk
#define RNG_VAL_RANDOM_VALUE_0                                             (0x1U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_1                                             (0x2U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_2                                             (0x4U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_3                                             (0x8U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_4                                             (0x10U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_5                                             (0x20U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_6                                             (0x40U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_7                                             (0x80U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_8                                             (0x100U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_9                                             (0x200U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_10                                            (0x400U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_11                                            (0x800U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_12                                            (0x1000U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_13                                            (0x2000U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_14                                            (0x4000U << RNG_VAL_RANDOM_VALUE_Pos)
#define RNG_VAL_RANDOM_VALUE_15                                            (0x8000U << RNG_VAL_RANDOM_VALUE_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        DMA                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    ISR    =====================================================*/ 
#define DMA_ISR_TEIF8_Pos                                                  (31UL)		/*!<DMA ISR: TEIF8 (Bit 31) */
#define DMA_ISR_TEIF8_Msk                                                  (0x80000000UL)		/*!< DMA ISR: TEIF8 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF8                                                      DMA_ISR_TEIF8_Msk
#define DMA_ISR_HTIF8_Pos                                                  (30UL)		/*!<DMA ISR: HTIF8 (Bit 30) */
#define DMA_ISR_HTIF8_Msk                                                  (0x40000000UL)		/*!< DMA ISR: HTIF8 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF8                                                      DMA_ISR_HTIF8_Msk
#define DMA_ISR_TCIF8_Pos                                                  (29UL)		/*!<DMA ISR: TCIF8 (Bit 29) */
#define DMA_ISR_TCIF8_Msk                                                  (0x20000000UL)		/*!< DMA ISR: TCIF8 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF8                                                      DMA_ISR_TCIF8_Msk
#define DMA_ISR_GIF8_Pos                                                   (28UL)		/*!<DMA ISR: GIF8 (Bit 28) */
#define DMA_ISR_GIF8_Msk                                                   (0x10000000UL)		/*!< DMA ISR: GIF8 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF8                                                       DMA_ISR_GIF8_Msk
#define DMA_ISR_TEIF7_Pos                                                  (27UL)		/*!<DMA ISR: TEIF7 (Bit 27) */
#define DMA_ISR_TEIF7_Msk                                                  (0x8000000UL)		/*!< DMA ISR: TEIF7 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF7                                                      DMA_ISR_TEIF7_Msk
#define DMA_ISR_HTIF7_Pos                                                  (26UL)		/*!<DMA ISR: HTIF7 (Bit 26) */
#define DMA_ISR_HTIF7_Msk                                                  (0x4000000UL)		/*!< DMA ISR: HTIF7 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF7                                                      DMA_ISR_HTIF7_Msk
#define DMA_ISR_TCIF7_Pos                                                  (25UL)		/*!<DMA ISR: TCIF7 (Bit 25) */
#define DMA_ISR_TCIF7_Msk                                                  (0x2000000UL)		/*!< DMA ISR: TCIF7 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF7                                                      DMA_ISR_TCIF7_Msk
#define DMA_ISR_GIF7_Pos                                                   (24UL)		/*!<DMA ISR: GIF7 (Bit 24) */
#define DMA_ISR_GIF7_Msk                                                   (0x1000000UL)		/*!< DMA ISR: GIF7 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF7                                                       DMA_ISR_GIF7_Msk
#define DMA_ISR_TEIF6_Pos                                                  (23UL)		/*!<DMA ISR: TEIF6 (Bit 23) */
#define DMA_ISR_TEIF6_Msk                                                  (0x800000UL)		/*!< DMA ISR: TEIF6 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF6                                                      DMA_ISR_TEIF6_Msk
#define DMA_ISR_HTIF6_Pos                                                  (22UL)		/*!<DMA ISR: HTIF6 (Bit 22) */
#define DMA_ISR_HTIF6_Msk                                                  (0x400000UL)		/*!< DMA ISR: HTIF6 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF6                                                      DMA_ISR_HTIF6_Msk
#define DMA_ISR_TCIF6_Pos                                                  (21UL)		/*!<DMA ISR: TCIF6 (Bit 21) */
#define DMA_ISR_TCIF6_Msk                                                  (0x200000UL)		/*!< DMA ISR: TCIF6 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF6                                                      DMA_ISR_TCIF6_Msk
#define DMA_ISR_GIF6_Pos                                                   (20UL)		/*!<DMA ISR: GIF6 (Bit 20) */
#define DMA_ISR_GIF6_Msk                                                   (0x100000UL)		/*!< DMA ISR: GIF6 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF6                                                       DMA_ISR_GIF6_Msk
#define DMA_ISR_TEIF5_Pos                                                  (19UL)		/*!<DMA ISR: TEIF5 (Bit 19) */
#define DMA_ISR_TEIF5_Msk                                                  (0x80000UL)		/*!< DMA ISR: TEIF5 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF5                                                      DMA_ISR_TEIF5_Msk
#define DMA_ISR_HTIF5_Pos                                                  (18UL)		/*!<DMA ISR: HTIF5 (Bit 18) */
#define DMA_ISR_HTIF5_Msk                                                  (0x40000UL)		/*!< DMA ISR: HTIF5 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF5                                                      DMA_ISR_HTIF5_Msk
#define DMA_ISR_TCIF5_Pos                                                  (17UL)		/*!<DMA ISR: TCIF5 (Bit 17) */
#define DMA_ISR_TCIF5_Msk                                                  (0x20000UL)		/*!< DMA ISR: TCIF5 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF5                                                      DMA_ISR_TCIF5_Msk
#define DMA_ISR_GIF5_Pos                                                   (16UL)		/*!<DMA ISR: GIF5 (Bit 16) */
#define DMA_ISR_GIF5_Msk                                                   (0x10000UL)		/*!< DMA ISR: GIF5 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF5                                                       DMA_ISR_GIF5_Msk
#define DMA_ISR_TEIF4_Pos                                                  (15UL)		/*!<DMA ISR: TEIF4 (Bit 15) */
#define DMA_ISR_TEIF4_Msk                                                  (0x8000UL)		/*!< DMA ISR: TEIF4 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF4                                                      DMA_ISR_TEIF4_Msk
#define DMA_ISR_HTIF4_Pos                                                  (14UL)		/*!<DMA ISR: HTIF4 (Bit 14) */
#define DMA_ISR_HTIF4_Msk                                                  (0x4000UL)		/*!< DMA ISR: HTIF4 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF4                                                      DMA_ISR_HTIF4_Msk
#define DMA_ISR_TCIF4_Pos                                                  (13UL)		/*!<DMA ISR: TCIF4 (Bit 13) */
#define DMA_ISR_TCIF4_Msk                                                  (0x2000UL)		/*!< DMA ISR: TCIF4 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF4                                                      DMA_ISR_TCIF4_Msk
#define DMA_ISR_GIF4_Pos                                                   (12UL)		/*!<DMA ISR: GIF4 (Bit 12) */
#define DMA_ISR_GIF4_Msk                                                   (0x1000UL)		/*!< DMA ISR: GIF4 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF4                                                       DMA_ISR_GIF4_Msk
#define DMA_ISR_TEIF3_Pos                                                  (11UL)		/*!<DMA ISR: TEIF3 (Bit 11) */
#define DMA_ISR_TEIF3_Msk                                                  (0x800UL)		/*!< DMA ISR: TEIF3 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF3                                                      DMA_ISR_TEIF3_Msk
#define DMA_ISR_HTIF3_Pos                                                  (10UL)		/*!<DMA ISR: HTIF3 (Bit 10) */
#define DMA_ISR_HTIF3_Msk                                                  (0x400UL)		/*!< DMA ISR: HTIF3 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF3                                                      DMA_ISR_HTIF3_Msk
#define DMA_ISR_TCIF3_Pos                                                  (9UL)		/*!<DMA ISR: TCIF3 (Bit 9) */
#define DMA_ISR_TCIF3_Msk                                                  (0x200UL)		/*!< DMA ISR: TCIF3 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF3                                                      DMA_ISR_TCIF3_Msk
#define DMA_ISR_GIF3_Pos                                                   (8UL)		/*!<DMA ISR: GIF3 (Bit 8) */
#define DMA_ISR_GIF3_Msk                                                   (0x100UL)		/*!< DMA ISR: GIF3 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF3                                                       DMA_ISR_GIF3_Msk
#define DMA_ISR_TEIF2_Pos                                                  (7UL)		/*!<DMA ISR: TEIF2 (Bit 7) */
#define DMA_ISR_TEIF2_Msk                                                  (0x80UL)		/*!< DMA ISR: TEIF2 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF2                                                      DMA_ISR_TEIF2_Msk
#define DMA_ISR_HTIF2_Pos                                                  (6UL)		/*!<DMA ISR: HTIF2 (Bit 6) */
#define DMA_ISR_HTIF2_Msk                                                  (0x40UL)		/*!< DMA ISR: HTIF2 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF2                                                      DMA_ISR_HTIF2_Msk
#define DMA_ISR_TCIF2_Pos                                                  (5UL)		/*!<DMA ISR: TCIF2 (Bit 5) */
#define DMA_ISR_TCIF2_Msk                                                  (0x20UL)		/*!< DMA ISR: TCIF2 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF2                                                      DMA_ISR_TCIF2_Msk
#define DMA_ISR_GIF2_Pos                                                   (4UL)		/*!<DMA ISR: GIF2 (Bit 4) */
#define DMA_ISR_GIF2_Msk                                                   (0x10UL)		/*!< DMA ISR: GIF2 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF2                                                       DMA_ISR_GIF2_Msk
#define DMA_ISR_TEIF1_Pos                                                  (3UL)		/*!<DMA ISR: TEIF1 (Bit 3) */
#define DMA_ISR_TEIF1_Msk                                                  (0x8UL)		/*!< DMA ISR: TEIF1 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TEIF1                                                      DMA_ISR_TEIF1_Msk
#define DMA_ISR_HTIF1_Pos                                                  (2UL)		/*!<DMA ISR: HTIF1 (Bit 2) */
#define DMA_ISR_HTIF1_Msk                                                  (0x4UL)		/*!< DMA ISR: HTIF1 (Bitfield-Mask: 0x01) */
#define DMA_ISR_HTIF1                                                      DMA_ISR_HTIF1_Msk
#define DMA_ISR_TCIF1_Pos                                                  (1UL)		/*!<DMA ISR: TCIF1 (Bit 1) */
#define DMA_ISR_TCIF1_Msk                                                  (0x2UL)		/*!< DMA ISR: TCIF1 (Bitfield-Mask: 0x01) */
#define DMA_ISR_TCIF1                                                      DMA_ISR_TCIF1_Msk
#define DMA_ISR_GIF1_Pos                                                   (0UL)		/*!<DMA ISR: GIF1 (Bit 0) */
#define DMA_ISR_GIF1_Msk                                                   (0x1UL)		/*!< DMA ISR: GIF1 (Bitfield-Mask: 0x01) */
#define DMA_ISR_GIF1                                                       DMA_ISR_GIF1_Msk

/* =====================================================    IFCR    =====================================================*/ 
#define DMA_IFCR_CTEIF8_Pos                                                (31UL)		/*!<DMA IFCR: CTEIF8 (Bit 31) */
#define DMA_IFCR_CTEIF8_Msk                                                (0x80000000UL)		/*!< DMA IFCR: CTEIF8 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF8                                                    DMA_IFCR_CTEIF8_Msk
#define DMA_IFCR_CHTIF8_Pos                                                (30UL)		/*!<DMA IFCR: CHTIF8 (Bit 30) */
#define DMA_IFCR_CHTIF8_Msk                                                (0x40000000UL)		/*!< DMA IFCR: CHTIF8 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF8                                                    DMA_IFCR_CHTIF8_Msk
#define DMA_IFCR_CTCIF8_Pos                                                (29UL)		/*!<DMA IFCR: CTCIF8 (Bit 29) */
#define DMA_IFCR_CTCIF8_Msk                                                (0x20000000UL)		/*!< DMA IFCR: CTCIF8 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF8                                                    DMA_IFCR_CTCIF8_Msk
#define DMA_IFCR_CGIF8_Pos                                                 (28UL)		/*!<DMA IFCR: CGIF8 (Bit 28) */
#define DMA_IFCR_CGIF8_Msk                                                 (0x10000000UL)		/*!< DMA IFCR: CGIF8 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF8                                                     DMA_IFCR_CGIF8_Msk
#define DMA_IFCR_CTEIF7_Pos                                                (27UL)		/*!<DMA IFCR: CTEIF7 (Bit 27) */
#define DMA_IFCR_CTEIF7_Msk                                                (0x8000000UL)		/*!< DMA IFCR: CTEIF7 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF7                                                    DMA_IFCR_CTEIF7_Msk
#define DMA_IFCR_CHTIF7_Pos                                                (26UL)		/*!<DMA IFCR: CHTIF7 (Bit 26) */
#define DMA_IFCR_CHTIF7_Msk                                                (0x4000000UL)		/*!< DMA IFCR: CHTIF7 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF7                                                    DMA_IFCR_CHTIF7_Msk
#define DMA_IFCR_CTCIF7_Pos                                                (25UL)		/*!<DMA IFCR: CTCIF7 (Bit 25) */
#define DMA_IFCR_CTCIF7_Msk                                                (0x2000000UL)		/*!< DMA IFCR: CTCIF7 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF7                                                    DMA_IFCR_CTCIF7_Msk
#define DMA_IFCR_CGIF7_Pos                                                 (24UL)		/*!<DMA IFCR: CGIF7 (Bit 24) */
#define DMA_IFCR_CGIF7_Msk                                                 (0x1000000UL)		/*!< DMA IFCR: CGIF7 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF7                                                     DMA_IFCR_CGIF7_Msk
#define DMA_IFCR_CTEIF6_Pos                                                (23UL)		/*!<DMA IFCR: CTEIF6 (Bit 23) */
#define DMA_IFCR_CTEIF6_Msk                                                (0x800000UL)		/*!< DMA IFCR: CTEIF6 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF6                                                    DMA_IFCR_CTEIF6_Msk
#define DMA_IFCR_CHTIF6_Pos                                                (22UL)		/*!<DMA IFCR: CHTIF6 (Bit 22) */
#define DMA_IFCR_CHTIF6_Msk                                                (0x400000UL)		/*!< DMA IFCR: CHTIF6 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF6                                                    DMA_IFCR_CHTIF6_Msk
#define DMA_IFCR_CTCIF6_Pos                                                (21UL)		/*!<DMA IFCR: CTCIF6 (Bit 21) */
#define DMA_IFCR_CTCIF6_Msk                                                (0x200000UL)		/*!< DMA IFCR: CTCIF6 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF6                                                    DMA_IFCR_CTCIF6_Msk
#define DMA_IFCR_CGIF6_Pos                                                 (20UL)		/*!<DMA IFCR: CGIF6 (Bit 20) */
#define DMA_IFCR_CGIF6_Msk                                                 (0x100000UL)		/*!< DMA IFCR: CGIF6 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF6                                                     DMA_IFCR_CGIF6_Msk
#define DMA_IFCR_CTEIF5_Pos                                                (19UL)		/*!<DMA IFCR: CTEIF5 (Bit 19) */
#define DMA_IFCR_CTEIF5_Msk                                                (0x80000UL)		/*!< DMA IFCR: CTEIF5 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF5                                                    DMA_IFCR_CTEIF5_Msk
#define DMA_IFCR_CHTIF5_Pos                                                (18UL)		/*!<DMA IFCR: CHTIF5 (Bit 18) */
#define DMA_IFCR_CHTIF5_Msk                                                (0x40000UL)		/*!< DMA IFCR: CHTIF5 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF5                                                    DMA_IFCR_CHTIF5_Msk
#define DMA_IFCR_CTCIF5_Pos                                                (17UL)		/*!<DMA IFCR: CTCIF5 (Bit 17) */
#define DMA_IFCR_CTCIF5_Msk                                                (0x20000UL)		/*!< DMA IFCR: CTCIF5 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF5                                                    DMA_IFCR_CTCIF5_Msk
#define DMA_IFCR_CGIF5_Pos                                                 (16UL)		/*!<DMA IFCR: CGIF5 (Bit 16) */
#define DMA_IFCR_CGIF5_Msk                                                 (0x10000UL)		/*!< DMA IFCR: CGIF5 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF5                                                     DMA_IFCR_CGIF5_Msk
#define DMA_IFCR_CTEIF4_Pos                                                (15UL)		/*!<DMA IFCR: CTEIF4 (Bit 15) */
#define DMA_IFCR_CTEIF4_Msk                                                (0x8000UL)		/*!< DMA IFCR: CTEIF4 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF4                                                    DMA_IFCR_CTEIF4_Msk
#define DMA_IFCR_CHTIF4_Pos                                                (14UL)		/*!<DMA IFCR: CHTIF4 (Bit 14) */
#define DMA_IFCR_CHTIF4_Msk                                                (0x4000UL)		/*!< DMA IFCR: CHTIF4 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF4                                                    DMA_IFCR_CHTIF4_Msk
#define DMA_IFCR_CTCIF4_Pos                                                (13UL)		/*!<DMA IFCR: CTCIF4 (Bit 13) */
#define DMA_IFCR_CTCIF4_Msk                                                (0x2000UL)		/*!< DMA IFCR: CTCIF4 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF4                                                    DMA_IFCR_CTCIF4_Msk
#define DMA_IFCR_CGIF4_Pos                                                 (12UL)		/*!<DMA IFCR: CGIF4 (Bit 12) */
#define DMA_IFCR_CGIF4_Msk                                                 (0x1000UL)		/*!< DMA IFCR: CGIF4 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF4                                                     DMA_IFCR_CGIF4_Msk
#define DMA_IFCR_CTEIF3_Pos                                                (11UL)		/*!<DMA IFCR: CTEIF3 (Bit 11) */
#define DMA_IFCR_CTEIF3_Msk                                                (0x800UL)		/*!< DMA IFCR: CTEIF3 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF3                                                    DMA_IFCR_CTEIF3_Msk
#define DMA_IFCR_CHTIF3_Pos                                                (10UL)		/*!<DMA IFCR: CHTIF3 (Bit 10) */
#define DMA_IFCR_CHTIF3_Msk                                                (0x400UL)		/*!< DMA IFCR: CHTIF3 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF3                                                    DMA_IFCR_CHTIF3_Msk
#define DMA_IFCR_CTCIF3_Pos                                                (9UL)		/*!<DMA IFCR: CTCIF3 (Bit 9) */
#define DMA_IFCR_CTCIF3_Msk                                                (0x200UL)		/*!< DMA IFCR: CTCIF3 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF3                                                    DMA_IFCR_CTCIF3_Msk
#define DMA_IFCR_CGIF3_Pos                                                 (8UL)		/*!<DMA IFCR: CGIF3 (Bit 8) */
#define DMA_IFCR_CGIF3_Msk                                                 (0x100UL)		/*!< DMA IFCR: CGIF3 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF3                                                     DMA_IFCR_CGIF3_Msk
#define DMA_IFCR_CTEIF2_Pos                                                (7UL)		/*!<DMA IFCR: CTEIF2 (Bit 7) */
#define DMA_IFCR_CTEIF2_Msk                                                (0x80UL)		/*!< DMA IFCR: CTEIF2 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF2                                                    DMA_IFCR_CTEIF2_Msk
#define DMA_IFCR_CHTIF2_Pos                                                (6UL)		/*!<DMA IFCR: CHTIF2 (Bit 6) */
#define DMA_IFCR_CHTIF2_Msk                                                (0x40UL)		/*!< DMA IFCR: CHTIF2 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF2                                                    DMA_IFCR_CHTIF2_Msk
#define DMA_IFCR_CTCIF2_Pos                                                (5UL)		/*!<DMA IFCR: CTCIF2 (Bit 5) */
#define DMA_IFCR_CTCIF2_Msk                                                (0x20UL)		/*!< DMA IFCR: CTCIF2 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF2                                                    DMA_IFCR_CTCIF2_Msk
#define DMA_IFCR_CGIF2_Pos                                                 (4UL)		/*!<DMA IFCR: CGIF2 (Bit 4) */
#define DMA_IFCR_CGIF2_Msk                                                 (0x10UL)		/*!< DMA IFCR: CGIF2 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF2                                                     DMA_IFCR_CGIF2_Msk
#define DMA_IFCR_CTEIF1_Pos                                                (3UL)		/*!<DMA IFCR: CTEIF1 (Bit 3) */
#define DMA_IFCR_CTEIF1_Msk                                                (0x8UL)		/*!< DMA IFCR: CTEIF1 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTEIF1                                                    DMA_IFCR_CTEIF1_Msk
#define DMA_IFCR_CHTIF1_Pos                                                (2UL)		/*!<DMA IFCR: CHTIF1 (Bit 2) */
#define DMA_IFCR_CHTIF1_Msk                                                (0x4UL)		/*!< DMA IFCR: CHTIF1 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CHTIF1                                                    DMA_IFCR_CHTIF1_Msk
#define DMA_IFCR_CTCIF1_Pos                                                (1UL)		/*!<DMA IFCR: CTCIF1 (Bit 1) */
#define DMA_IFCR_CTCIF1_Msk                                                (0x2UL)		/*!< DMA IFCR: CTCIF1 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CTCIF1                                                    DMA_IFCR_CTCIF1_Msk
#define DMA_IFCR_CGIF1_Pos                                                 (0UL)		/*!<DMA IFCR: CGIF1 (Bit 0) */
#define DMA_IFCR_CGIF1_Msk                                                 (0x1UL)		/*!< DMA IFCR: CGIF1 (Bitfield-Mask: 0x01) */
#define DMA_IFCR_CGIF1                                                     DMA_IFCR_CGIF1_Msk

/* =====================================================    CCR    =====================================================*/ 
#define DMA_CCR_MEM2MEM_Pos                                                (14UL)		/*!<DMA CCR: MEM2MEM (Bit 14) */
#define DMA_CCR_MEM2MEM_Msk                                                (0x4000UL)		/*!< DMA CCR: MEM2MEM (Bitfield-Mask: 0x01) */
#define DMA_CCR_MEM2MEM                                                    DMA_CCR_MEM2MEM_Msk
#define DMA_CCR_PL_Pos                                                     (12UL)		/*!<DMA CCR: PL (Bit 12) */
#define DMA_CCR_PL_Msk                                                     (0x3000UL)		/*!< DMA CCR: PL (Bitfield-Mask: 0x03) */
#define DMA_CCR_PL                                                         DMA_CCR_PL_Msk
#define DMA_CCR_PL_0                                                       (0x1U << DMA_CCR_PL_Pos)
#define DMA_CCR_PL_1                                                       (0x2U << DMA_CCR_PL_Pos)
#define DMA_CCR_MSIZE_Pos                                                  (10UL)		/*!<DMA CCR: MSIZE (Bit 10) */
#define DMA_CCR_MSIZE_Msk                                                  (0xc00UL)		/*!< DMA CCR: MSIZE (Bitfield-Mask: 0x03) */
#define DMA_CCR_MSIZE                                                      DMA_CCR_MSIZE_Msk
#define DMA_CCR_MSIZE_0                                                    (0x1U << DMA_CCR_MSIZE_Pos)
#define DMA_CCR_MSIZE_1                                                    (0x2U << DMA_CCR_MSIZE_Pos)
#define DMA_CCR_PSIZE_Pos                                                  (8UL)		/*!<DMA CCR: PSIZE (Bit 8) */
#define DMA_CCR_PSIZE_Msk                                                  (0x300UL)		/*!< DMA CCR: PSIZE (Bitfield-Mask: 0x03) */
#define DMA_CCR_PSIZE                                                      DMA_CCR_PSIZE_Msk
#define DMA_CCR_PSIZE_0                                                    (0x1U << DMA_CCR_PSIZE_Pos)
#define DMA_CCR_PSIZE_1                                                    (0x2U << DMA_CCR_PSIZE_Pos)
#define DMA_CCR_MINC_Pos                                                   (7UL)		/*!<DMA CCR: MINC (Bit 7) */
#define DMA_CCR_MINC_Msk                                                   (0x80UL)		/*!< DMA CCR: MINC (Bitfield-Mask: 0x01) */
#define DMA_CCR_MINC                                                       DMA_CCR_MINC_Msk
#define DMA_CCR_PINC_Pos                                                   (6UL)		/*!<DMA CCR: PINC (Bit 6) */
#define DMA_CCR_PINC_Msk                                                   (0x40UL)		/*!< DMA CCR: PINC (Bitfield-Mask: 0x01) */
#define DMA_CCR_PINC                                                       DMA_CCR_PINC_Msk
#define DMA_CCR_CIRC_Pos                                                   (5UL)		/*!<DMA CCR: CIRC (Bit 5) */
#define DMA_CCR_CIRC_Msk                                                   (0x20UL)		/*!< DMA CCR: CIRC (Bitfield-Mask: 0x01) */
#define DMA_CCR_CIRC                                                       DMA_CCR_CIRC_Msk
#define DMA_CCR_DIR_Pos                                                    (4UL)		/*!<DMA CCR: DIR (Bit 4) */
#define DMA_CCR_DIR_Msk                                                    (0x10UL)		/*!< DMA CCR: DIR (Bitfield-Mask: 0x01) */
#define DMA_CCR_DIR                                                        DMA_CCR_DIR_Msk
#define DMA_CCR_TEIE_Pos                                                   (3UL)		/*!<DMA CCR: TEIE (Bit 3) */
#define DMA_CCR_TEIE_Msk                                                   (0x8UL)		/*!< DMA CCR: TEIE (Bitfield-Mask: 0x01) */
#define DMA_CCR_TEIE                                                       DMA_CCR_TEIE_Msk
#define DMA_CCR_HTIE_Pos                                                   (2UL)		/*!<DMA CCR: HTIE (Bit 2) */
#define DMA_CCR_HTIE_Msk                                                   (0x4UL)		/*!< DMA CCR: HTIE (Bitfield-Mask: 0x01) */
#define DMA_CCR_HTIE                                                       DMA_CCR_HTIE_Msk
#define DMA_CCR_TCIE_Pos                                                   (1UL)		/*!<DMA CCR: TCIE (Bit 1) */
#define DMA_CCR_TCIE_Msk                                                   (0x2UL)		/*!< DMA CCR: TCIE (Bitfield-Mask: 0x01) */
#define DMA_CCR_TCIE                                                       DMA_CCR_TCIE_Msk
#define DMA_CCR_EN_Pos                                                     (0UL)		/*!<DMA CCR: EN (Bit 0) */
#define DMA_CCR_EN_Msk                                                     (0x1UL)		/*!< DMA CCR: EN (Bitfield-Mask: 0x01) */
#define DMA_CCR_EN                                                         DMA_CCR_EN_Msk

/* =====================================================    CNDTR    =====================================================*/ 
#define DMA_CNDTR_NDT_Pos                                                  (0UL)		/*!<DMA CNDTR: NDT (Bit 0) */
#define DMA_CNDTR_NDT_Msk                                                  (0xffffUL)		/*!< DMA CNDTR: NDT (Bitfield-Mask: 0xffff) */
#define DMA_CNDTR_NDT                                                      DMA_CNDTR_NDT_Msk
#define DMA_CNDTR_NDT_0                                                    (0x1U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_1                                                    (0x2U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_2                                                    (0x4U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_3                                                    (0x8U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_4                                                    (0x10U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_5                                                    (0x20U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_6                                                    (0x40U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_7                                                    (0x80U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_8                                                    (0x100U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_9                                                    (0x200U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_10                                                   (0x400U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_11                                                   (0x800U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_12                                                   (0x1000U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_13                                                   (0x2000U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_14                                                   (0x4000U << DMA_CNDTR_NDT_Pos)
#define DMA_CNDTR_NDT_15                                                   (0x8000U << DMA_CNDTR_NDT_Pos)

/* =====================================================    CPAR    =====================================================*/ 
#define DMA_CPAR_PA_Pos                                                    (0UL)		/*!<DMA CPAR: PA (Bit 0) */
#define DMA_CPAR_PA_Msk                                                    (0xffffffffUL)		/*!< DMA CPAR: PA (Bitfield-Mask: 0xffffffff) */
#define DMA_CPAR_PA                                                        DMA_CPAR_PA_Msk
#define DMA_CPAR_PA_0                                                      (0x1U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_1                                                      (0x2U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_2                                                      (0x4U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_3                                                      (0x8U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_4                                                      (0x10U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_5                                                      (0x20U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_6                                                      (0x40U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_7                                                      (0x80U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_8                                                      (0x100U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_9                                                      (0x200U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_10                                                     (0x400U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_11                                                     (0x800U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_12                                                     (0x1000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_13                                                     (0x2000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_14                                                     (0x4000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_15                                                     (0x8000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_16                                                     (0x10000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_17                                                     (0x20000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_18                                                     (0x40000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_19                                                     (0x80000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_20                                                     (0x100000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_21                                                     (0x200000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_22                                                     (0x400000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_23                                                     (0x800000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_24                                                     (0x1000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_25                                                     (0x2000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_26                                                     (0x4000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_27                                                     (0x8000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_28                                                     (0x10000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_29                                                     (0x20000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_30                                                     (0x40000000U << DMA_CPAR_PA_Pos)
#define DMA_CPAR_PA_31                                                     (0x80000000UL << DMA_CPAR_PA_Pos)

/* =====================================================    CMAR    =====================================================*/ 
#define DMA_CMAR_MA_Pos                                                    (0UL)		/*!<DMA CMAR: MA (Bit 0) */
#define DMA_CMAR_MA_Msk                                                    (0xffffffffUL)		/*!< DMA CMAR: MA (Bitfield-Mask: 0xffffffff) */
#define DMA_CMAR_MA                                                        DMA_CMAR_MA_Msk
#define DMA_CMAR_MA_0                                                      (0x1U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_1                                                      (0x2U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_2                                                      (0x4U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_3                                                      (0x8U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_4                                                      (0x10U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_5                                                      (0x20U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_6                                                      (0x40U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_7                                                      (0x80U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_8                                                      (0x100U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_9                                                      (0x200U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_10                                                     (0x400U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_11                                                     (0x800U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_12                                                     (0x1000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_13                                                     (0x2000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_14                                                     (0x4000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_15                                                     (0x8000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_16                                                     (0x10000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_17                                                     (0x20000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_18                                                     (0x40000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_19                                                     (0x80000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_20                                                     (0x100000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_21                                                     (0x200000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_22                                                     (0x400000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_23                                                     (0x800000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_24                                                     (0x1000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_25                                                     (0x2000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_26                                                     (0x4000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_27                                                     (0x8000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_28                                                     (0x10000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_29                                                     (0x20000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_30                                                     (0x40000000U << DMA_CMAR_MA_Pos)
#define DMA_CMAR_MA_31                                                     (0x80000000UL << DMA_CMAR_MA_Pos)


/* ============================================================================================================================*/ 
/*=====================                                      DMAMUX                                      =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CxCR    =====================================================*/ 
#define DMAMUX_CxCR_DMAREQ_ID_Pos                                          (0UL)		/*!<DMAMUX CxCR: DMAREQ_ID (Bit 0) */
#define DMAMUX_CxCR_DMAREQ_ID_Msk                                          (0x1fUL)		/*!< DMAMUX CxCR: DMAREQ_ID (Bitfield-Mask: 0x1f) */
#define DMAMUX_CxCR_DMAREQ_ID                                              DMAMUX_CxCR_DMAREQ_ID_Msk
#define DMAMUX_CxCR_DMAREQ_ID_0                                            (0x1U << DMAMUX_CxCR_DMAREQ_ID_Pos)
#define DMAMUX_CxCR_DMAREQ_ID_1                                            (0x2U << DMAMUX_CxCR_DMAREQ_ID_Pos)
#define DMAMUX_CxCR_DMAREQ_ID_2                                            (0x4U << DMAMUX_CxCR_DMAREQ_ID_Pos)
#define DMAMUX_CxCR_DMAREQ_ID_3                                            (0x8U << DMAMUX_CxCR_DMAREQ_ID_Pos)
#define DMAMUX_CxCR_DMAREQ_ID_4                                            (0x10U << DMAMUX_CxCR_DMAREQ_ID_Pos)


/* ============================================================================================================================*/ 
/*=====================                                        AES                                        =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    CR    =====================================================*/ 
#define AES_CR_NPBLB_Pos                                                   (20UL)		/*!<AES CR: NPBLB (Bit 20) */
#define AES_CR_NPBLB_Msk                                                   (0xf00000UL)		/*!< AES CR: NPBLB (Bitfield-Mask: 0x0f) */
#define AES_CR_NPBLB                                                       AES_CR_NPBLB_Msk
#define AES_CR_NPBLB_0                                                     (0x1U << AES_CR_NPBLB_Pos)
#define AES_CR_NPBLB_1                                                     (0x2U << AES_CR_NPBLB_Pos)
#define AES_CR_NPBLB_2                                                     (0x4U << AES_CR_NPBLB_Pos)
#define AES_CR_NPBLB_3                                                     (0x8U << AES_CR_NPBLB_Pos)
#define AES_CR_KEYSIZE_Pos                                                 (18UL)		/*!<AES CR: KEYSIZE (Bit 18) */
#define AES_CR_KEYSIZE_Msk                                                 (0x40000UL)		/*!< AES CR: KEYSIZE (Bitfield-Mask: 0x01) */
#define AES_CR_KEYSIZE                                                     AES_CR_KEYSIZE_Msk
#define AES_CR_GCMPH_Pos                                                   (13UL)		/*!<AES CR: GCMPH (Bit 13) */
#define AES_CR_GCMPH_Msk                                                   (0x6000UL)		/*!< AES CR: GCMPH (Bitfield-Mask: 0x03) */
#define AES_CR_GCMPH                                                       AES_CR_GCMPH_Msk
#define AES_CR_GCMPH_0                                                     (0x1U << AES_CR_GCMPH_Pos)
#define AES_CR_GCMPH_1                                                     (0x2U << AES_CR_GCMPH_Pos)
#define AES_CR_DMAOUTEN_Pos                                                (12UL)		/*!<AES CR: DMAOUTEN (Bit 12) */
#define AES_CR_DMAOUTEN_Msk                                                (0x1000UL)		/*!< AES CR: DMAOUTEN (Bitfield-Mask: 0x01) */
#define AES_CR_DMAOUTEN                                                    AES_CR_DMAOUTEN_Msk
#define AES_CR_DMAINEN_Pos                                                 (11UL)		/*!<AES CR: DMAINEN (Bit 11) */
#define AES_CR_DMAINEN_Msk                                                 (0x800UL)		/*!< AES CR: DMAINEN (Bitfield-Mask: 0x01) */
#define AES_CR_DMAINEN                                                     AES_CR_DMAINEN_Msk
#define AES_CR_ERRIE_Pos                                                   (10UL)		/*!<AES CR: ERRIE (Bit 10) */
#define AES_CR_ERRIE_Msk                                                   (0x400UL)		/*!< AES CR: ERRIE (Bitfield-Mask: 0x01) */
#define AES_CR_ERRIE                                                       AES_CR_ERRIE_Msk
#define AES_CR_CCFIE_Pos                                                   (9UL)		/*!<AES CR: CCFIE (Bit 9) */
#define AES_CR_CCFIE_Msk                                                   (0x200UL)		/*!< AES CR: CCFIE (Bitfield-Mask: 0x01) */
#define AES_CR_CCFIE                                                       AES_CR_CCFIE_Msk
#define AES_CR_ERRC_Pos                                                    (8UL)		/*!<AES CR: ERRC (Bit 8) */
#define AES_CR_ERRC_Msk                                                    (0x100UL)		/*!< AES CR: ERRC (Bitfield-Mask: 0x01) */
#define AES_CR_ERRC                                                        AES_CR_ERRC_Msk
#define AES_CR_CCFC_Pos                                                    (7UL)		/*!<AES CR: CCFC (Bit 7) */
#define AES_CR_CCFC_Msk                                                    (0x80UL)		/*!< AES CR: CCFC (Bitfield-Mask: 0x01) */
#define AES_CR_CCFC                                                        AES_CR_CCFC_Msk
#define AES_CR_CHMOD_Pos                                                   (5UL)		/*!<AES CR: CHMOD (Bit 5) */
#define AES_CR_CHMOD_Msk                                                   (0x10060UL)		/*!< AES CR: CHMOD (Bitfield-Mask: 0x803) */
#define AES_CR_CHMOD                                                       AES_CR_CHMOD_Msk
#define AES_CR_CHMOD_0                                                     (0x1U << AES_CR_CHMOD_Pos)
#define AES_CR_CHMOD_1                                                     (0x2U << AES_CR_CHMOD_Pos)
#define AES_CR_CHMOD_2                                                     (0x800U << AES_CR_CHMOD_Pos)
#define AES_CR_MODE_Pos                                                    (3UL)		/*!<AES CR: MODE (Bit 3) */
#define AES_CR_MODE_Msk                                                    (0x18UL)		/*!< AES CR: MODE (Bitfield-Mask: 0x03) */
#define AES_CR_MODE                                                        AES_CR_MODE_Msk
#define AES_CR_MODE_0                                                      (0x1U << AES_CR_MODE_Pos)
#define AES_CR_MODE_1                                                      (0x2U << AES_CR_MODE_Pos)
#define AES_CR_DATATYPE_Pos                                                (1UL)		/*!<AES CR: DATATYPE (Bit 1) */
#define AES_CR_DATATYPE_Msk                                                (0x6UL)		/*!< AES CR: DATATYPE (Bitfield-Mask: 0x03) */
#define AES_CR_DATATYPE                                                    AES_CR_DATATYPE_Msk
#define AES_CR_DATATYPE_0                                                  (0x1U << AES_CR_DATATYPE_Pos)
#define AES_CR_DATATYPE_1                                                  (0x2U << AES_CR_DATATYPE_Pos)
#define AES_CR_EN_Pos                                                      (0UL)		/*!<AES CR: EN (Bit 0) */
#define AES_CR_EN_Msk                                                      (0x1UL)		/*!< AES CR: EN (Bitfield-Mask: 0x01) */
#define AES_CR_EN                                                          AES_CR_EN_Msk

/* =====================================================    SR    =====================================================*/ 
#define AES_SR_BUSY_Pos                                                    (3UL)		/*!<AES SR: BUSY (Bit 3) */
#define AES_SR_BUSY_Msk                                                    (0x8UL)		/*!< AES SR: BUSY (Bitfield-Mask: 0x01) */
#define AES_SR_BUSY                                                        AES_SR_BUSY_Msk
#define AES_SR_WRERR_Pos                                                   (2UL)		/*!<AES SR: WRERR (Bit 2) */
#define AES_SR_WRERR_Msk                                                   (0x4UL)		/*!< AES SR: WRERR (Bitfield-Mask: 0x01) */
#define AES_SR_WRERR                                                       AES_SR_WRERR_Msk
#define AES_SR_RDERR_Pos                                                   (1UL)		/*!<AES SR: RDERR (Bit 1) */
#define AES_SR_RDERR_Msk                                                   (0x2UL)		/*!< AES SR: RDERR (Bitfield-Mask: 0x01) */
#define AES_SR_RDERR                                                       AES_SR_RDERR_Msk
#define AES_SR_CCF_Pos                                                     (0UL)		/*!<AES SR: CCF (Bit 0) */
#define AES_SR_CCF_Msk                                                     (0x1UL)		/*!< AES SR: CCF (Bitfield-Mask: 0x01) */
#define AES_SR_CCF                                                         AES_SR_CCF_Msk

/* =====================================================    DINR    =====================================================*/ 
#define AES_DINR_DINR_Pos                                                  (0UL)		/*!<AES DINR: DINR (Bit 0) */
#define AES_DINR_DINR_Msk                                                  (0xffffffffUL)		/*!< AES DINR: DINR (Bitfield-Mask: 0xffffffff) */
#define AES_DINR_DINR                                                      AES_DINR_DINR_Msk
#define AES_DINR_DINR_0                                                    (0x1U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_1                                                    (0x2U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_2                                                    (0x4U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_3                                                    (0x8U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_4                                                    (0x10U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_5                                                    (0x20U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_6                                                    (0x40U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_7                                                    (0x80U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_8                                                    (0x100U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_9                                                    (0x200U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_10                                                   (0x400U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_11                                                   (0x800U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_12                                                   (0x1000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_13                                                   (0x2000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_14                                                   (0x4000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_15                                                   (0x8000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_16                                                   (0x10000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_17                                                   (0x20000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_18                                                   (0x40000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_19                                                   (0x80000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_20                                                   (0x100000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_21                                                   (0x200000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_22                                                   (0x400000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_23                                                   (0x800000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_24                                                   (0x1000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_25                                                   (0x2000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_26                                                   (0x4000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_27                                                   (0x8000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_28                                                   (0x10000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_29                                                   (0x20000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_30                                                   (0x40000000U << AES_DINR_DINR_Pos)
#define AES_DINR_DINR_31                                                   (0x80000000UL << AES_DINR_DINR_Pos)

/* =====================================================    DOUTR    =====================================================*/ 
#define AES_DOUTR_DOUTR_Pos                                                (0UL)		/*!<AES DOUTR: DOUTR (Bit 0) */
#define AES_DOUTR_DOUTR_Msk                                                (0xffffffffUL)		/*!< AES DOUTR: DOUTR (Bitfield-Mask: 0xffffffff) */
#define AES_DOUTR_DOUTR                                                    AES_DOUTR_DOUTR_Msk
#define AES_DOUTR_DOUTR_0                                                  (0x1U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_1                                                  (0x2U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_2                                                  (0x4U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_3                                                  (0x8U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_4                                                  (0x10U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_5                                                  (0x20U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_6                                                  (0x40U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_7                                                  (0x80U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_8                                                  (0x100U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_9                                                  (0x200U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_10                                                 (0x400U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_11                                                 (0x800U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_12                                                 (0x1000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_13                                                 (0x2000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_14                                                 (0x4000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_15                                                 (0x8000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_16                                                 (0x10000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_17                                                 (0x20000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_18                                                 (0x40000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_19                                                 (0x80000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_20                                                 (0x100000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_21                                                 (0x200000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_22                                                 (0x400000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_23                                                 (0x800000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_24                                                 (0x1000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_25                                                 (0x2000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_26                                                 (0x4000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_27                                                 (0x8000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_28                                                 (0x10000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_29                                                 (0x20000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_30                                                 (0x40000000U << AES_DOUTR_DOUTR_Pos)
#define AES_DOUTR_DOUTR_31                                                 (0x80000000UL << AES_DOUTR_DOUTR_Pos)

/* =====================================================    KEYR0    =====================================================*/ 
#define AES_KEYR0_KEY_Pos                                                  (0UL)		/*!<AES KEYR0: KEY (Bit 0) */
#define AES_KEYR0_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR0: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR0_KEY                                                      AES_KEYR0_KEY_Msk
#define AES_KEYR0_KEY_0                                                    (0x1U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_1                                                    (0x2U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_2                                                    (0x4U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_3                                                    (0x8U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_4                                                    (0x10U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_5                                                    (0x20U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_6                                                    (0x40U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_7                                                    (0x80U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_8                                                    (0x100U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_9                                                    (0x200U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_10                                                   (0x400U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_11                                                   (0x800U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_12                                                   (0x1000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_13                                                   (0x2000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_14                                                   (0x4000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_15                                                   (0x8000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_16                                                   (0x10000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_17                                                   (0x20000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_18                                                   (0x40000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_19                                                   (0x80000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_20                                                   (0x100000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_21                                                   (0x200000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_22                                                   (0x400000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_23                                                   (0x800000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_24                                                   (0x1000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_25                                                   (0x2000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_26                                                   (0x4000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_27                                                   (0x8000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_28                                                   (0x10000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_29                                                   (0x20000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_30                                                   (0x40000000U << AES_KEYR0_KEY_Pos)
#define AES_KEYR0_KEY_31                                                   (0x80000000UL << AES_KEYR0_KEY_Pos)

/* =====================================================    KEYR1    =====================================================*/ 
#define AES_KEYR1_KEY_Pos                                                  (0UL)		/*!<AES KEYR1: KEY (Bit 0) */
#define AES_KEYR1_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR1: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR1_KEY                                                      AES_KEYR1_KEY_Msk
#define AES_KEYR1_KEY_0                                                    (0x1U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_1                                                    (0x2U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_2                                                    (0x4U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_3                                                    (0x8U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_4                                                    (0x10U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_5                                                    (0x20U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_6                                                    (0x40U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_7                                                    (0x80U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_8                                                    (0x100U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_9                                                    (0x200U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_10                                                   (0x400U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_11                                                   (0x800U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_12                                                   (0x1000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_13                                                   (0x2000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_14                                                   (0x4000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_15                                                   (0x8000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_16                                                   (0x10000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_17                                                   (0x20000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_18                                                   (0x40000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_19                                                   (0x80000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_20                                                   (0x100000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_21                                                   (0x200000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_22                                                   (0x400000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_23                                                   (0x800000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_24                                                   (0x1000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_25                                                   (0x2000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_26                                                   (0x4000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_27                                                   (0x8000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_28                                                   (0x10000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_29                                                   (0x20000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_30                                                   (0x40000000U << AES_KEYR1_KEY_Pos)
#define AES_KEYR1_KEY_31                                                   (0x80000000UL << AES_KEYR1_KEY_Pos)

/* =====================================================    KEYR2    =====================================================*/ 
#define AES_KEYR2_KEY_Pos                                                  (0UL)		/*!<AES KEYR2: KEY (Bit 0) */
#define AES_KEYR2_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR2: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR2_KEY                                                      AES_KEYR2_KEY_Msk
#define AES_KEYR2_KEY_0                                                    (0x1U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_1                                                    (0x2U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_2                                                    (0x4U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_3                                                    (0x8U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_4                                                    (0x10U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_5                                                    (0x20U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_6                                                    (0x40U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_7                                                    (0x80U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_8                                                    (0x100U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_9                                                    (0x200U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_10                                                   (0x400U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_11                                                   (0x800U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_12                                                   (0x1000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_13                                                   (0x2000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_14                                                   (0x4000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_15                                                   (0x8000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_16                                                   (0x10000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_17                                                   (0x20000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_18                                                   (0x40000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_19                                                   (0x80000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_20                                                   (0x100000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_21                                                   (0x200000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_22                                                   (0x400000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_23                                                   (0x800000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_24                                                   (0x1000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_25                                                   (0x2000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_26                                                   (0x4000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_27                                                   (0x8000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_28                                                   (0x10000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_29                                                   (0x20000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_30                                                   (0x40000000U << AES_KEYR2_KEY_Pos)
#define AES_KEYR2_KEY_31                                                   (0x80000000UL << AES_KEYR2_KEY_Pos)

/* =====================================================    KEYR3    =====================================================*/ 
#define AES_KEYR3_KEY_Pos                                                  (0UL)		/*!<AES KEYR3: KEY (Bit 0) */
#define AES_KEYR3_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR3: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR3_KEY                                                      AES_KEYR3_KEY_Msk
#define AES_KEYR3_KEY_0                                                    (0x1U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_1                                                    (0x2U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_2                                                    (0x4U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_3                                                    (0x8U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_4                                                    (0x10U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_5                                                    (0x20U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_6                                                    (0x40U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_7                                                    (0x80U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_8                                                    (0x100U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_9                                                    (0x200U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_10                                                   (0x400U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_11                                                   (0x800U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_12                                                   (0x1000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_13                                                   (0x2000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_14                                                   (0x4000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_15                                                   (0x8000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_16                                                   (0x10000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_17                                                   (0x20000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_18                                                   (0x40000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_19                                                   (0x80000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_20                                                   (0x100000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_21                                                   (0x200000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_22                                                   (0x400000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_23                                                   (0x800000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_24                                                   (0x1000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_25                                                   (0x2000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_26                                                   (0x4000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_27                                                   (0x8000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_28                                                   (0x10000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_29                                                   (0x20000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_30                                                   (0x40000000U << AES_KEYR3_KEY_Pos)
#define AES_KEYR3_KEY_31                                                   (0x80000000UL << AES_KEYR3_KEY_Pos)

/* =====================================================    IVR0    =====================================================*/ 
#define AES_IVR0_IVI_Pos                                                   (0UL)		/*!<AES IVR0: IVI (Bit 0) */
#define AES_IVR0_IVI_Msk                                                   (0xffffffffUL)		/*!< AES IVR0: IVI (Bitfield-Mask: 0xffffffff) */
#define AES_IVR0_IVI                                                       AES_IVR0_IVI_Msk
#define AES_IVR0_IVI_0                                                     (0x1U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_1                                                     (0x2U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_2                                                     (0x4U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_3                                                     (0x8U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_4                                                     (0x10U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_5                                                     (0x20U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_6                                                     (0x40U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_7                                                     (0x80U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_8                                                     (0x100U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_9                                                     (0x200U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_10                                                    (0x400U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_11                                                    (0x800U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_12                                                    (0x1000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_13                                                    (0x2000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_14                                                    (0x4000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_15                                                    (0x8000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_16                                                    (0x10000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_17                                                    (0x20000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_18                                                    (0x40000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_19                                                    (0x80000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_20                                                    (0x100000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_21                                                    (0x200000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_22                                                    (0x400000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_23                                                    (0x800000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_24                                                    (0x1000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_25                                                    (0x2000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_26                                                    (0x4000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_27                                                    (0x8000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_28                                                    (0x10000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_29                                                    (0x20000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_30                                                    (0x40000000U << AES_IVR0_IVI_Pos)
#define AES_IVR0_IVI_31                                                    (0x80000000UL << AES_IVR0_IVI_Pos)

/* =====================================================    IVR1    =====================================================*/ 
#define AES_IVR1_IVI_Pos                                                   (0UL)		/*!<AES IVR1: IVI (Bit 0) */
#define AES_IVR1_IVI_Msk                                                   (0xffffffffUL)		/*!< AES IVR1: IVI (Bitfield-Mask: 0xffffffff) */
#define AES_IVR1_IVI                                                       AES_IVR1_IVI_Msk
#define AES_IVR1_IVI_0                                                     (0x1U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_1                                                     (0x2U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_2                                                     (0x4U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_3                                                     (0x8U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_4                                                     (0x10U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_5                                                     (0x20U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_6                                                     (0x40U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_7                                                     (0x80U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_8                                                     (0x100U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_9                                                     (0x200U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_10                                                    (0x400U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_11                                                    (0x800U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_12                                                    (0x1000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_13                                                    (0x2000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_14                                                    (0x4000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_15                                                    (0x8000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_16                                                    (0x10000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_17                                                    (0x20000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_18                                                    (0x40000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_19                                                    (0x80000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_20                                                    (0x100000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_21                                                    (0x200000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_22                                                    (0x400000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_23                                                    (0x800000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_24                                                    (0x1000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_25                                                    (0x2000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_26                                                    (0x4000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_27                                                    (0x8000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_28                                                    (0x10000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_29                                                    (0x20000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_30                                                    (0x40000000U << AES_IVR1_IVI_Pos)
#define AES_IVR1_IVI_31                                                    (0x80000000UL << AES_IVR1_IVI_Pos)

/* =====================================================    IVR2    =====================================================*/ 
#define AES_IVR2_IVI_Pos                                                   (0UL)		/*!<AES IVR2: IVI (Bit 0) */
#define AES_IVR2_IVI_Msk                                                   (0xffffffffUL)		/*!< AES IVR2: IVI (Bitfield-Mask: 0xffffffff) */
#define AES_IVR2_IVI                                                       AES_IVR2_IVI_Msk
#define AES_IVR2_IVI_0                                                     (0x1U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_1                                                     (0x2U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_2                                                     (0x4U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_3                                                     (0x8U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_4                                                     (0x10U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_5                                                     (0x20U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_6                                                     (0x40U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_7                                                     (0x80U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_8                                                     (0x100U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_9                                                     (0x200U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_10                                                    (0x400U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_11                                                    (0x800U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_12                                                    (0x1000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_13                                                    (0x2000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_14                                                    (0x4000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_15                                                    (0x8000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_16                                                    (0x10000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_17                                                    (0x20000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_18                                                    (0x40000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_19                                                    (0x80000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_20                                                    (0x100000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_21                                                    (0x200000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_22                                                    (0x400000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_23                                                    (0x800000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_24                                                    (0x1000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_25                                                    (0x2000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_26                                                    (0x4000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_27                                                    (0x8000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_28                                                    (0x10000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_29                                                    (0x20000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_30                                                    (0x40000000U << AES_IVR2_IVI_Pos)
#define AES_IVR2_IVI_31                                                    (0x80000000UL << AES_IVR2_IVI_Pos)

/* =====================================================    IVR3    =====================================================*/ 
#define AES_IVR3_IVI_Pos                                                   (0UL)		/*!<AES IVR3: IVI (Bit 0) */
#define AES_IVR3_IVI_Msk                                                   (0xffffffffUL)		/*!< AES IVR3: IVI (Bitfield-Mask: 0xffffffff) */
#define AES_IVR3_IVI                                                       AES_IVR3_IVI_Msk
#define AES_IVR3_IVI_0                                                     (0x1U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_1                                                     (0x2U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_2                                                     (0x4U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_3                                                     (0x8U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_4                                                     (0x10U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_5                                                     (0x20U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_6                                                     (0x40U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_7                                                     (0x80U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_8                                                     (0x100U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_9                                                     (0x200U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_10                                                    (0x400U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_11                                                    (0x800U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_12                                                    (0x1000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_13                                                    (0x2000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_14                                                    (0x4000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_15                                                    (0x8000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_16                                                    (0x10000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_17                                                    (0x20000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_18                                                    (0x40000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_19                                                    (0x80000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_20                                                    (0x100000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_21                                                    (0x200000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_22                                                    (0x400000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_23                                                    (0x800000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_24                                                    (0x1000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_25                                                    (0x2000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_26                                                    (0x4000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_27                                                    (0x8000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_28                                                    (0x10000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_29                                                    (0x20000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_30                                                    (0x40000000U << AES_IVR3_IVI_Pos)
#define AES_IVR3_IVI_31                                                    (0x80000000UL << AES_IVR3_IVI_Pos)

/* =====================================================    KEYR4    =====================================================*/ 
#define AES_KEYR4_KEY_Pos                                                  (0UL)		/*!<AES KEYR4: KEY (Bit 0) */
#define AES_KEYR4_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR4: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR4_KEY                                                      AES_KEYR4_KEY_Msk
#define AES_KEYR4_KEY_0                                                    (0x1U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_1                                                    (0x2U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_2                                                    (0x4U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_3                                                    (0x8U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_4                                                    (0x10U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_5                                                    (0x20U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_6                                                    (0x40U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_7                                                    (0x80U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_8                                                    (0x100U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_9                                                    (0x200U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_10                                                   (0x400U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_11                                                   (0x800U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_12                                                   (0x1000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_13                                                   (0x2000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_14                                                   (0x4000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_15                                                   (0x8000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_16                                                   (0x10000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_17                                                   (0x20000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_18                                                   (0x40000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_19                                                   (0x80000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_20                                                   (0x100000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_21                                                   (0x200000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_22                                                   (0x400000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_23                                                   (0x800000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_24                                                   (0x1000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_25                                                   (0x2000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_26                                                   (0x4000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_27                                                   (0x8000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_28                                                   (0x10000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_29                                                   (0x20000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_30                                                   (0x40000000U << AES_KEYR4_KEY_Pos)
#define AES_KEYR4_KEY_31                                                   (0x80000000UL << AES_KEYR4_KEY_Pos)

/* =====================================================    KEYR5    =====================================================*/ 
#define AES_KEYR5_KEY_Pos                                                  (0UL)		/*!<AES KEYR5: KEY (Bit 0) */
#define AES_KEYR5_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR5: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR5_KEY                                                      AES_KEYR5_KEY_Msk
#define AES_KEYR5_KEY_0                                                    (0x1U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_1                                                    (0x2U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_2                                                    (0x4U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_3                                                    (0x8U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_4                                                    (0x10U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_5                                                    (0x20U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_6                                                    (0x40U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_7                                                    (0x80U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_8                                                    (0x100U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_9                                                    (0x200U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_10                                                   (0x400U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_11                                                   (0x800U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_12                                                   (0x1000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_13                                                   (0x2000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_14                                                   (0x4000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_15                                                   (0x8000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_16                                                   (0x10000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_17                                                   (0x20000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_18                                                   (0x40000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_19                                                   (0x80000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_20                                                   (0x100000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_21                                                   (0x200000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_22                                                   (0x400000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_23                                                   (0x800000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_24                                                   (0x1000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_25                                                   (0x2000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_26                                                   (0x4000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_27                                                   (0x8000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_28                                                   (0x10000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_29                                                   (0x20000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_30                                                   (0x40000000U << AES_KEYR5_KEY_Pos)
#define AES_KEYR5_KEY_31                                                   (0x80000000UL << AES_KEYR5_KEY_Pos)

/* =====================================================    KEYR6    =====================================================*/ 
#define AES_KEYR6_KEY_Pos                                                  (0UL)		/*!<AES KEYR6: KEY (Bit 0) */
#define AES_KEYR6_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR6: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR6_KEY                                                      AES_KEYR6_KEY_Msk
#define AES_KEYR6_KEY_0                                                    (0x1U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_1                                                    (0x2U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_2                                                    (0x4U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_3                                                    (0x8U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_4                                                    (0x10U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_5                                                    (0x20U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_6                                                    (0x40U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_7                                                    (0x80U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_8                                                    (0x100U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_9                                                    (0x200U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_10                                                   (0x400U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_11                                                   (0x800U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_12                                                   (0x1000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_13                                                   (0x2000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_14                                                   (0x4000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_15                                                   (0x8000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_16                                                   (0x10000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_17                                                   (0x20000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_18                                                   (0x40000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_19                                                   (0x80000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_20                                                   (0x100000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_21                                                   (0x200000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_22                                                   (0x400000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_23                                                   (0x800000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_24                                                   (0x1000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_25                                                   (0x2000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_26                                                   (0x4000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_27                                                   (0x8000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_28                                                   (0x10000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_29                                                   (0x20000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_30                                                   (0x40000000U << AES_KEYR6_KEY_Pos)
#define AES_KEYR6_KEY_31                                                   (0x80000000UL << AES_KEYR6_KEY_Pos)

/* =====================================================    KEYR7    =====================================================*/ 
#define AES_KEYR7_KEY_Pos                                                  (0UL)		/*!<AES KEYR7: KEY (Bit 0) */
#define AES_KEYR7_KEY_Msk                                                  (0xffffffffUL)		/*!< AES KEYR7: KEY (Bitfield-Mask: 0xffffffff) */
#define AES_KEYR7_KEY                                                      AES_KEYR7_KEY_Msk
#define AES_KEYR7_KEY_0                                                    (0x1U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_1                                                    (0x2U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_2                                                    (0x4U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_3                                                    (0x8U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_4                                                    (0x10U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_5                                                    (0x20U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_6                                                    (0x40U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_7                                                    (0x80U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_8                                                    (0x100U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_9                                                    (0x200U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_10                                                   (0x400U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_11                                                   (0x800U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_12                                                   (0x1000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_13                                                   (0x2000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_14                                                   (0x4000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_15                                                   (0x8000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_16                                                   (0x10000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_17                                                   (0x20000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_18                                                   (0x40000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_19                                                   (0x80000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_20                                                   (0x100000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_21                                                   (0x200000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_22                                                   (0x400000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_23                                                   (0x800000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_24                                                   (0x1000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_25                                                   (0x2000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_26                                                   (0x4000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_27                                                   (0x8000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_28                                                   (0x10000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_29                                                   (0x20000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_30                                                   (0x40000000U << AES_KEYR7_KEY_Pos)
#define AES_KEYR7_KEY_31                                                   (0x80000000UL << AES_KEYR7_KEY_Pos)

/* =====================================================    SUSP0    =====================================================*/ 
#define AES_SUSP0_SUSP_Pos                                                 (0UL)		/*!<AES SUSP0: SUSP (Bit 0) */
#define AES_SUSP0_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP0: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP0_SUSP                                                     AES_SUSP0_SUSP_Msk
#define AES_SUSP0_SUSP_0                                                   (0x1U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_1                                                   (0x2U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_2                                                   (0x4U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_3                                                   (0x8U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_4                                                   (0x10U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_5                                                   (0x20U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_6                                                   (0x40U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_7                                                   (0x80U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_8                                                   (0x100U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_9                                                   (0x200U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_10                                                  (0x400U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_11                                                  (0x800U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_12                                                  (0x1000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_13                                                  (0x2000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_14                                                  (0x4000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_15                                                  (0x8000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_16                                                  (0x10000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_17                                                  (0x20000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_18                                                  (0x40000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_19                                                  (0x80000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_20                                                  (0x100000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_21                                                  (0x200000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_22                                                  (0x400000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_23                                                  (0x800000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_24                                                  (0x1000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_25                                                  (0x2000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_26                                                  (0x4000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_27                                                  (0x8000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_28                                                  (0x10000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_29                                                  (0x20000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_30                                                  (0x40000000U << AES_SUSP0_SUSP_Pos)
#define AES_SUSP0_SUSP_31                                                  (0x80000000UL << AES_SUSP0_SUSP_Pos)

/* =====================================================    SUSP1    =====================================================*/ 
#define AES_SUSP1_SUSP_Pos                                                 (0UL)		/*!<AES SUSP1: SUSP (Bit 0) */
#define AES_SUSP1_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP1: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP1_SUSP                                                     AES_SUSP1_SUSP_Msk
#define AES_SUSP1_SUSP_0                                                   (0x1U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_1                                                   (0x2U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_2                                                   (0x4U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_3                                                   (0x8U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_4                                                   (0x10U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_5                                                   (0x20U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_6                                                   (0x40U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_7                                                   (0x80U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_8                                                   (0x100U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_9                                                   (0x200U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_10                                                  (0x400U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_11                                                  (0x800U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_12                                                  (0x1000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_13                                                  (0x2000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_14                                                  (0x4000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_15                                                  (0x8000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_16                                                  (0x10000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_17                                                  (0x20000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_18                                                  (0x40000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_19                                                  (0x80000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_20                                                  (0x100000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_21                                                  (0x200000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_22                                                  (0x400000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_23                                                  (0x800000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_24                                                  (0x1000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_25                                                  (0x2000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_26                                                  (0x4000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_27                                                  (0x8000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_28                                                  (0x10000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_29                                                  (0x20000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_30                                                  (0x40000000U << AES_SUSP1_SUSP_Pos)
#define AES_SUSP1_SUSP_31                                                  (0x80000000UL << AES_SUSP1_SUSP_Pos)

/* =====================================================    SUSP2    =====================================================*/ 
#define AES_SUSP2_SUSP_Pos                                                 (0UL)		/*!<AES SUSP2: SUSP (Bit 0) */
#define AES_SUSP2_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP2: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP2_SUSP                                                     AES_SUSP2_SUSP_Msk
#define AES_SUSP2_SUSP_0                                                   (0x1U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_1                                                   (0x2U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_2                                                   (0x4U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_3                                                   (0x8U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_4                                                   (0x10U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_5                                                   (0x20U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_6                                                   (0x40U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_7                                                   (0x80U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_8                                                   (0x100U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_9                                                   (0x200U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_10                                                  (0x400U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_11                                                  (0x800U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_12                                                  (0x1000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_13                                                  (0x2000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_14                                                  (0x4000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_15                                                  (0x8000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_16                                                  (0x10000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_17                                                  (0x20000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_18                                                  (0x40000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_19                                                  (0x80000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_20                                                  (0x100000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_21                                                  (0x200000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_22                                                  (0x400000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_23                                                  (0x800000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_24                                                  (0x1000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_25                                                  (0x2000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_26                                                  (0x4000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_27                                                  (0x8000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_28                                                  (0x10000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_29                                                  (0x20000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_30                                                  (0x40000000U << AES_SUSP2_SUSP_Pos)
#define AES_SUSP2_SUSP_31                                                  (0x80000000UL << AES_SUSP2_SUSP_Pos)

/* =====================================================    SUSP3    =====================================================*/ 
#define AES_SUSP3_SUSP_Pos                                                 (0UL)		/*!<AES SUSP3: SUSP (Bit 0) */
#define AES_SUSP3_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP3: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP3_SUSP                                                     AES_SUSP3_SUSP_Msk
#define AES_SUSP3_SUSP_0                                                   (0x1U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_1                                                   (0x2U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_2                                                   (0x4U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_3                                                   (0x8U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_4                                                   (0x10U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_5                                                   (0x20U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_6                                                   (0x40U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_7                                                   (0x80U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_8                                                   (0x100U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_9                                                   (0x200U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_10                                                  (0x400U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_11                                                  (0x800U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_12                                                  (0x1000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_13                                                  (0x2000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_14                                                  (0x4000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_15                                                  (0x8000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_16                                                  (0x10000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_17                                                  (0x20000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_18                                                  (0x40000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_19                                                  (0x80000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_20                                                  (0x100000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_21                                                  (0x200000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_22                                                  (0x400000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_23                                                  (0x800000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_24                                                  (0x1000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_25                                                  (0x2000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_26                                                  (0x4000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_27                                                  (0x8000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_28                                                  (0x10000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_29                                                  (0x20000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_30                                                  (0x40000000U << AES_SUSP3_SUSP_Pos)
#define AES_SUSP3_SUSP_31                                                  (0x80000000UL << AES_SUSP3_SUSP_Pos)

/* =====================================================    SUSP4    =====================================================*/ 
#define AES_SUSP4_SUSP_Pos                                                 (0UL)		/*!<AES SUSP4: SUSP (Bit 0) */
#define AES_SUSP4_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP4: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP4_SUSP                                                     AES_SUSP4_SUSP_Msk
#define AES_SUSP4_SUSP_0                                                   (0x1U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_1                                                   (0x2U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_2                                                   (0x4U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_3                                                   (0x8U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_4                                                   (0x10U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_5                                                   (0x20U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_6                                                   (0x40U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_7                                                   (0x80U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_8                                                   (0x100U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_9                                                   (0x200U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_10                                                  (0x400U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_11                                                  (0x800U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_12                                                  (0x1000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_13                                                  (0x2000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_14                                                  (0x4000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_15                                                  (0x8000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_16                                                  (0x10000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_17                                                  (0x20000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_18                                                  (0x40000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_19                                                  (0x80000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_20                                                  (0x100000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_21                                                  (0x200000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_22                                                  (0x400000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_23                                                  (0x800000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_24                                                  (0x1000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_25                                                  (0x2000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_26                                                  (0x4000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_27                                                  (0x8000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_28                                                  (0x10000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_29                                                  (0x20000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_30                                                  (0x40000000U << AES_SUSP4_SUSP_Pos)
#define AES_SUSP4_SUSP_31                                                  (0x80000000UL << AES_SUSP4_SUSP_Pos)

/* =====================================================    SUSP5    =====================================================*/ 
#define AES_SUSP5_SUSP_Pos                                                 (0UL)		/*!<AES SUSP5: SUSP (Bit 0) */
#define AES_SUSP5_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP5: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP5_SUSP                                                     AES_SUSP5_SUSP_Msk
#define AES_SUSP5_SUSP_0                                                   (0x1U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_1                                                   (0x2U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_2                                                   (0x4U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_3                                                   (0x8U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_4                                                   (0x10U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_5                                                   (0x20U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_6                                                   (0x40U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_7                                                   (0x80U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_8                                                   (0x100U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_9                                                   (0x200U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_10                                                  (0x400U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_11                                                  (0x800U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_12                                                  (0x1000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_13                                                  (0x2000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_14                                                  (0x4000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_15                                                  (0x8000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_16                                                  (0x10000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_17                                                  (0x20000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_18                                                  (0x40000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_19                                                  (0x80000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_20                                                  (0x100000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_21                                                  (0x200000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_22                                                  (0x400000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_23                                                  (0x800000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_24                                                  (0x1000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_25                                                  (0x2000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_26                                                  (0x4000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_27                                                  (0x8000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_28                                                  (0x10000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_29                                                  (0x20000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_30                                                  (0x40000000U << AES_SUSP5_SUSP_Pos)
#define AES_SUSP5_SUSP_31                                                  (0x80000000UL << AES_SUSP5_SUSP_Pos)

/* =====================================================    SUSP6    =====================================================*/ 
#define AES_SUSP6_SUSP_Pos                                                 (0UL)		/*!<AES SUSP6: SUSP (Bit 0) */
#define AES_SUSP6_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP6: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP6_SUSP                                                     AES_SUSP6_SUSP_Msk
#define AES_SUSP6_SUSP_0                                                   (0x1U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_1                                                   (0x2U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_2                                                   (0x4U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_3                                                   (0x8U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_4                                                   (0x10U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_5                                                   (0x20U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_6                                                   (0x40U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_7                                                   (0x80U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_8                                                   (0x100U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_9                                                   (0x200U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_10                                                  (0x400U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_11                                                  (0x800U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_12                                                  (0x1000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_13                                                  (0x2000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_14                                                  (0x4000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_15                                                  (0x8000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_16                                                  (0x10000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_17                                                  (0x20000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_18                                                  (0x40000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_19                                                  (0x80000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_20                                                  (0x100000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_21                                                  (0x200000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_22                                                  (0x400000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_23                                                  (0x800000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_24                                                  (0x1000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_25                                                  (0x2000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_26                                                  (0x4000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_27                                                  (0x8000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_28                                                  (0x10000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_29                                                  (0x20000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_30                                                  (0x40000000U << AES_SUSP6_SUSP_Pos)
#define AES_SUSP6_SUSP_31                                                  (0x80000000UL << AES_SUSP6_SUSP_Pos)

/* =====================================================    SUSP7    =====================================================*/ 
#define AES_SUSP7_SUSP_Pos                                                 (0UL)		/*!<AES SUSP7: SUSP (Bit 0) */
#define AES_SUSP7_SUSP_Msk                                                 (0xffffffffUL)		/*!< AES SUSP7: SUSP (Bitfield-Mask: 0xffffffff) */
#define AES_SUSP7_SUSP                                                     AES_SUSP7_SUSP_Msk
#define AES_SUSP7_SUSP_0                                                   (0x1U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_1                                                   (0x2U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_2                                                   (0x4U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_3                                                   (0x8U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_4                                                   (0x10U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_5                                                   (0x20U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_6                                                   (0x40U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_7                                                   (0x80U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_8                                                   (0x100U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_9                                                   (0x200U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_10                                                  (0x400U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_11                                                  (0x800U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_12                                                  (0x1000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_13                                                  (0x2000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_14                                                  (0x4000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_15                                                  (0x8000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_16                                                  (0x10000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_17                                                  (0x20000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_18                                                  (0x40000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_19                                                  (0x80000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_20                                                  (0x100000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_21                                                  (0x200000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_22                                                  (0x400000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_23                                                  (0x800000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_24                                                  (0x1000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_25                                                  (0x2000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_26                                                  (0x4000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_27                                                  (0x8000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_28                                                  (0x10000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_29                                                  (0x20000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_30                                                  (0x40000000U << AES_SUSP7_SUSP_Pos)
#define AES_SUSP7_SUSP_31                                                  (0x80000000UL << AES_SUSP7_SUSP_Pos)


/* ============================================================================================================================*/ 
/*=====================                                      LPAWUR                                      =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    FRAME_CONFIG0    =====================================================*/ 
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos                (21UL)		/*!<LPAWUR FRAME_CONFIG0: SLOW_CLK_CYCLE_PER_BIT_CNT (Bit 21) */
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Msk                (0x3e00000UL)		/*!< LPAWUR FRAME_CONFIG0: SLOW_CLK_CYCLE_PER_BIT_CNT (Bitfield-Mask: 0x1f) */
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT                    LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Msk
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_0                  (0x1U << LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_1                  (0x2U << LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_2                  (0x4U << LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_3                  (0x8U << LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_4                  (0x10U << LPAWUR_FRAME_CONFIG0_SLOW_CLK_CYCLE_PER_BIT_CNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Pos                            (16UL)		/*!<LPAWUR FRAME_CONFIG0: PAYLOAD_LENGTH (Bit 16) */
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Msk                            (0xf0000UL)		/*!< LPAWUR FRAME_CONFIG0: PAYLOAD_LENGTH (Bitfield-Mask: 0x0f) */
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH                                LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Msk
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_0                              (0x1U << LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Pos)
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_1                              (0x2U << LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Pos)
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_2                              (0x4U << LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Pos)
#define LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_3                              (0x8U << LPAWUR_FRAME_CONFIG0_PAYLOAD_LENGTH_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos                      (10UL)		/*!<LPAWUR FRAME_CONFIG0: SYNC_THRESHOLD_COUNT (Bit 10) */
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Msk                      (0xfc00UL)		/*!< LPAWUR FRAME_CONFIG0: SYNC_THRESHOLD_COUNT (Bitfield-Mask: 0x3f) */
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT                          LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Msk
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_0                        (0x1U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_1                        (0x2U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_2                        (0x4U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_3                        (0x8U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_4                        (0x10U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_5                        (0x20U << LPAWUR_FRAME_CONFIG0_SYNC_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_SYNC_LENGTH_Pos                               (8UL)		/*!<LPAWUR FRAME_CONFIG0: SYNC_LENGTH (Bit 8) */
#define LPAWUR_FRAME_CONFIG0_SYNC_LENGTH_Msk                               (0x100UL)		/*!< LPAWUR FRAME_CONFIG0: SYNC_LENGTH (Bitfield-Mask: 0x01) */
#define LPAWUR_FRAME_CONFIG0_SYNC_LENGTH                                   LPAWUR_FRAME_CONFIG0_SYNC_LENGTH_Msk
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos                  (0UL)		/*!<LPAWUR FRAME_CONFIG0: PREAMBLE_THRESHOLD_COUNT (Bit 0) */
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Msk                  (0xffUL)		/*!< LPAWUR FRAME_CONFIG0: PREAMBLE_THRESHOLD_COUNT (Bitfield-Mask: 0xff) */
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT                      LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Msk
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_0                    (0x1U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_1                    (0x2U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_2                    (0x4U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_3                    (0x8U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_4                    (0x10U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_5                    (0x20U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_6                    (0x40U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)
#define LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_7                    (0x80U << LPAWUR_FRAME_CONFIG0_PREAMBLE_THRESHOLD_COUNT_Pos)

/* =====================================================    FRAME_CONFIG1    =====================================================*/ 
#define LPAWUR_FRAME_CONFIG1_TREC_LOOP_ALGO_SEL_Pos                        (18UL)		/*!<LPAWUR FRAME_CONFIG1: TREC_LOOP_ALGO_SEL (Bit 18) */
#define LPAWUR_FRAME_CONFIG1_TREC_LOOP_ALGO_SEL_Msk                        (0x40000UL)		/*!< LPAWUR FRAME_CONFIG1: TREC_LOOP_ALGO_SEL (Bitfield-Mask: 0x01) */
#define LPAWUR_FRAME_CONFIG1_TREC_LOOP_ALGO_SEL                            LPAWUR_FRAME_CONFIG1_TREC_LOOP_ALGO_SEL_Msk
#define LPAWUR_FRAME_CONFIG1_PREAMBLE_ENABLE_Pos                           (17UL)		/*!<LPAWUR FRAME_CONFIG1: PREAMBLE_ENABLE (Bit 17) */
#define LPAWUR_FRAME_CONFIG1_PREAMBLE_ENABLE_Msk                           (0x20000UL)		/*!< LPAWUR FRAME_CONFIG1: PREAMBLE_ENABLE (Bitfield-Mask: 0x01) */
#define LPAWUR_FRAME_CONFIG1_PREAMBLE_ENABLE                               LPAWUR_FRAME_CONFIG1_PREAMBLE_ENABLE_Msk
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos                (8UL)		/*!<LPAWUR FRAME_CONFIG1: FRAME_SYNC_COUNTER_TIMEOUT (Bit 8) */
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Msk                (0xff00UL)		/*!< LPAWUR FRAME_CONFIG1: FRAME_SYNC_COUNTER_TIMEOUT (Bitfield-Mask: 0xff) */
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT                    LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Msk
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_0                  (0x1U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_1                  (0x2U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_2                  (0x4U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_3                  (0x8U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_4                  (0x10U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_5                  (0x20U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_6                  (0x40U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_7                  (0x80U << LPAWUR_FRAME_CONFIG1_FRAME_SYNC_COUNTER_TIMEOUT_Pos)
#define LPAWUR_FRAME_CONFIG1_KP_Pos                                        (4UL)		/*!<LPAWUR FRAME_CONFIG1: KP (Bit 4) */
#define LPAWUR_FRAME_CONFIG1_KP_Msk                                        (0xf0UL)		/*!< LPAWUR FRAME_CONFIG1: KP (Bitfield-Mask: 0x0f) */
#define LPAWUR_FRAME_CONFIG1_KP                                            LPAWUR_FRAME_CONFIG1_KP_Msk
#define LPAWUR_FRAME_CONFIG1_KP_0                                          (0x1U << LPAWUR_FRAME_CONFIG1_KP_Pos)
#define LPAWUR_FRAME_CONFIG1_KP_1                                          (0x2U << LPAWUR_FRAME_CONFIG1_KP_Pos)
#define LPAWUR_FRAME_CONFIG1_KP_2                                          (0x4U << LPAWUR_FRAME_CONFIG1_KP_Pos)
#define LPAWUR_FRAME_CONFIG1_KP_3                                          (0x8U << LPAWUR_FRAME_CONFIG1_KP_Pos)
#define LPAWUR_FRAME_CONFIG1_KI_Pos                                        (0UL)		/*!<LPAWUR FRAME_CONFIG1: KI (Bit 0) */
#define LPAWUR_FRAME_CONFIG1_KI_Msk                                        (0xfUL)		/*!< LPAWUR FRAME_CONFIG1: KI (Bitfield-Mask: 0x0f) */
#define LPAWUR_FRAME_CONFIG1_KI                                            LPAWUR_FRAME_CONFIG1_KI_Msk
#define LPAWUR_FRAME_CONFIG1_KI_0                                          (0x1U << LPAWUR_FRAME_CONFIG1_KI_Pos)
#define LPAWUR_FRAME_CONFIG1_KI_1                                          (0x2U << LPAWUR_FRAME_CONFIG1_KI_Pos)
#define LPAWUR_FRAME_CONFIG1_KI_2                                          (0x4U << LPAWUR_FRAME_CONFIG1_KI_Pos)
#define LPAWUR_FRAME_CONFIG1_KI_3                                          (0x8U << LPAWUR_FRAME_CONFIG1_KI_Pos)

/* =====================================================    FRAME_SYNC_CONFIG    =====================================================*/ 
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos                  (16UL)		/*!<LPAWUR FRAME_SYNC_CONFIG: FRAME_SYNC_PATTERN_H (Bit 16) */
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Msk                  (0xffff0000UL)		/*!< LPAWUR FRAME_SYNC_CONFIG: FRAME_SYNC_PATTERN_H (Bitfield-Mask: 0xffff) */
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H                      LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Msk
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_0                    (0x1U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_1                    (0x2U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_2                    (0x4U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_3                    (0x8U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_4                    (0x10U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_5                    (0x20U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_6                    (0x40U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_7                    (0x80U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_8                    (0x100U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_9                    (0x200U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_10                   (0x400U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_11                   (0x800U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_12                   (0x1000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_13                   (0x2000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_14                   (0x4000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_15                   (0x8000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_H_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos                  (0UL)		/*!<LPAWUR FRAME_SYNC_CONFIG: FRAME_SYNC_PATTERN_L (Bit 0) */
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Msk                  (0xffffUL)		/*!< LPAWUR FRAME_SYNC_CONFIG: FRAME_SYNC_PATTERN_L (Bitfield-Mask: 0xffff) */
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L                      LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Msk
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_0                    (0x1U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_1                    (0x2U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_2                    (0x4U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_3                    (0x8U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_4                    (0x10U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_5                    (0x20U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_6                    (0x40U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_7                    (0x80U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_8                    (0x100U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_9                    (0x200U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_10                   (0x400U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_11                   (0x800U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_12                   (0x1000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_13                   (0x2000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_14                   (0x4000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)
#define LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_15                   (0x8000U << LPAWUR_FRAME_SYNC_CONFIG_FRAME_SYNC_PATTERN_L_Pos)

/* =====================================================    RFIP_CONFIG    =====================================================*/ 
#define LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_Pos                                (1UL)		/*!<LPAWUR RFIP_CONFIG: WAKEUP_LEVEL (Bit 1) */
#define LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_Msk                                (0x6UL)		/*!< LPAWUR RFIP_CONFIG: WAKEUP_LEVEL (Bitfield-Mask: 0x03) */
#define LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL                                    LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_Msk
#define LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_0                                  (0x1U << LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_Pos)
#define LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_1                                  (0x2U << LPAWUR_RFIP_CONFIG_WAKEUP_LEVEL_Pos)
#define LPAWUR_RFIP_CONFIG_LPAWUR_ENABLE_Pos                               (0UL)		/*!<LPAWUR RFIP_CONFIG: LPAWUR_ENABLE (Bit 0) */
#define LPAWUR_RFIP_CONFIG_LPAWUR_ENABLE_Msk                               (0x1UL)		/*!< LPAWUR RFIP_CONFIG: LPAWUR_ENABLE (Bitfield-Mask: 0x01) */
#define LPAWUR_RFIP_CONFIG_LPAWUR_ENABLE                                   LPAWUR_RFIP_CONFIG_LPAWUR_ENABLE_Msk

/* =====================================================    RF_CONFIG    =====================================================*/ 
#define LPAWUR_RF_CONFIG_ED_ICAL_Pos                                       (18UL)		/*!<LPAWUR RF_CONFIG: ED_ICAL (Bit 18) */
#define LPAWUR_RF_CONFIG_ED_ICAL_Msk                                       (0x1c0000UL)		/*!< LPAWUR RF_CONFIG: ED_ICAL (Bitfield-Mask: 0x07) */
#define LPAWUR_RF_CONFIG_ED_ICAL                                           LPAWUR_RF_CONFIG_ED_ICAL_Msk
#define LPAWUR_RF_CONFIG_ED_ICAL_0                                         (0x1U << LPAWUR_RF_CONFIG_ED_ICAL_Pos)
#define LPAWUR_RF_CONFIG_ED_ICAL_1                                         (0x2U << LPAWUR_RF_CONFIG_ED_ICAL_Pos)
#define LPAWUR_RF_CONFIG_ED_ICAL_2                                         (0x4U << LPAWUR_RF_CONFIG_ED_ICAL_Pos)
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Pos                                  (14UL)		/*!<LPAWUR RF_CONFIG: AGC_HIGH_LVL (Bit 14) */
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Msk                                  (0x3c000UL)		/*!< LPAWUR RF_CONFIG: AGC_HIGH_LVL (Bitfield-Mask: 0x0f) */
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL                                      LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Msk
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_0                                    (0x1U << LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Pos)
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_1                                    (0x2U << LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Pos)
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_2                                    (0x4U << LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Pos)
#define LPAWUR_RF_CONFIG_AGC_HIGH_LVL_3                                    (0x8U << LPAWUR_RF_CONFIG_AGC_HIGH_LVL_Pos)
#define LPAWUR_RF_CONFIG_ED_DC_CTRL_Pos                                    (13UL)		/*!<LPAWUR RF_CONFIG: ED_DC_CTRL (Bit 13) */
#define LPAWUR_RF_CONFIG_ED_DC_CTRL_Msk                                    (0x2000UL)		/*!< LPAWUR RF_CONFIG: ED_DC_CTRL (Bitfield-Mask: 0x01) */
#define LPAWUR_RF_CONFIG_ED_DC_CTRL                                        LPAWUR_RF_CONFIG_ED_DC_CTRL_Msk
#define LPAWUR_RF_CONFIG_AGC_LOW_LVL_Pos                                   (11UL)		/*!<LPAWUR RF_CONFIG: AGC_LOW_LVL (Bit 11) */
#define LPAWUR_RF_CONFIG_AGC_LOW_LVL_Msk                                   (0x1800UL)		/*!< LPAWUR RF_CONFIG: AGC_LOW_LVL (Bitfield-Mask: 0x03) */
#define LPAWUR_RF_CONFIG_AGC_LOW_LVL                                       LPAWUR_RF_CONFIG_AGC_LOW_LVL_Msk
#define LPAWUR_RF_CONFIG_AGC_LOW_LVL_0                                     (0x1U << LPAWUR_RF_CONFIG_AGC_LOW_LVL_Pos)
#define LPAWUR_RF_CONFIG_AGC_LOW_LVL_1                                     (0x2U << LPAWUR_RF_CONFIG_AGC_LOW_LVL_Pos)
#define LPAWUR_RF_CONFIG_CLKDIV_Pos                                        (1UL)		/*!<LPAWUR RF_CONFIG: CLKDIV (Bit 1) */
#define LPAWUR_RF_CONFIG_CLKDIV_Msk                                        (0x1eUL)		/*!< LPAWUR RF_CONFIG: CLKDIV (Bitfield-Mask: 0x0f) */
#define LPAWUR_RF_CONFIG_CLKDIV                                            LPAWUR_RF_CONFIG_CLKDIV_Msk
#define LPAWUR_RF_CONFIG_CLKDIV_0                                          (0x1U << LPAWUR_RF_CONFIG_CLKDIV_Pos)
#define LPAWUR_RF_CONFIG_CLKDIV_1                                          (0x2U << LPAWUR_RF_CONFIG_CLKDIV_Pos)
#define LPAWUR_RF_CONFIG_CLKDIV_2                                          (0x4U << LPAWUR_RF_CONFIG_CLKDIV_Pos)
#define LPAWUR_RF_CONFIG_CLKDIV_3                                          (0x8U << LPAWUR_RF_CONFIG_CLKDIV_Pos)
#define LPAWUR_RF_CONFIG_ED_SWITCH_Pos                                     (0UL)		/*!<LPAWUR RF_CONFIG: ED_SWITCH (Bit 0) */
#define LPAWUR_RF_CONFIG_ED_SWITCH_Msk                                     (0x1UL)		/*!< LPAWUR RF_CONFIG: ED_SWITCH (Bitfield-Mask: 0x01) */
#define LPAWUR_RF_CONFIG_ED_SWITCH                                         LPAWUR_RF_CONFIG_ED_SWITCH_Msk

/* =====================================================    AGC_CONFIG    =====================================================*/ 
#define LPAWUR_AGC_CONFIG_AGC_RESET_MODE_Pos                               (3UL)		/*!<LPAWUR AGC_CONFIG: AGC_RESET_MODE (Bit 3) */
#define LPAWUR_AGC_CONFIG_AGC_RESET_MODE_Msk                               (0x8UL)		/*!< LPAWUR AGC_CONFIG: AGC_RESET_MODE (Bitfield-Mask: 0x01) */
#define LPAWUR_AGC_CONFIG_AGC_RESET_MODE                                   LPAWUR_AGC_CONFIG_AGC_RESET_MODE_Msk
#define LPAWUR_AGC_CONFIG_AGC_HOLD_MODE_Pos                                (2UL)		/*!<LPAWUR AGC_CONFIG: AGC_HOLD_MODE (Bit 2) */
#define LPAWUR_AGC_CONFIG_AGC_HOLD_MODE_Msk                                (0x4UL)		/*!< LPAWUR AGC_CONFIG: AGC_HOLD_MODE (Bitfield-Mask: 0x01) */
#define LPAWUR_AGC_CONFIG_AGC_HOLD_MODE                                    LPAWUR_AGC_CONFIG_AGC_HOLD_MODE_Msk
#define LPAWUR_AGC_CONFIG_AGC_MODE_Pos                                     (0UL)		/*!<LPAWUR AGC_CONFIG: AGC_MODE (Bit 0) */
#define LPAWUR_AGC_CONFIG_AGC_MODE_Msk                                     (0x3UL)		/*!< LPAWUR AGC_CONFIG: AGC_MODE (Bitfield-Mask: 0x03) */
#define LPAWUR_AGC_CONFIG_AGC_MODE                                         LPAWUR_AGC_CONFIG_AGC_MODE_Msk
#define LPAWUR_AGC_CONFIG_AGC_MODE_0                                       (0x1U << LPAWUR_AGC_CONFIG_AGC_MODE_Pos)
#define LPAWUR_AGC_CONFIG_AGC_MODE_1                                       (0x2U << LPAWUR_AGC_CONFIG_AGC_MODE_Pos)

/* =====================================================    PAYLOAD_0    =====================================================*/ 
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos                                     (0UL)		/*!<LPAWUR PAYLOAD_0: PAYLOAD_0 (Bit 0) */
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_Msk                                     (0xffffffffUL)		/*!< LPAWUR PAYLOAD_0: PAYLOAD_0 (Bitfield-Mask: 0xffffffff) */
#define LPAWUR_PAYLOAD_0_PAYLOAD_0                                         LPAWUR_PAYLOAD_0_PAYLOAD_0_Msk
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_0                                       (0x1U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_1                                       (0x2U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_2                                       (0x4U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_3                                       (0x8U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_4                                       (0x10U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_5                                       (0x20U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_6                                       (0x40U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_7                                       (0x80U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_8                                       (0x100U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_9                                       (0x200U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_10                                      (0x400U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_11                                      (0x800U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_12                                      (0x1000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_13                                      (0x2000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_14                                      (0x4000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_15                                      (0x8000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_16                                      (0x10000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_17                                      (0x20000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_18                                      (0x40000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_19                                      (0x80000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_20                                      (0x100000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_21                                      (0x200000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_22                                      (0x400000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_23                                      (0x800000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_24                                      (0x1000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_25                                      (0x2000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_26                                      (0x4000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_27                                      (0x8000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_28                                      (0x10000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_29                                      (0x20000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_30                                      (0x40000000U << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)
#define LPAWUR_PAYLOAD_0_PAYLOAD_0_31                                      (0x80000000UL << LPAWUR_PAYLOAD_0_PAYLOAD_0_Pos)

/* =====================================================    PAYLOAD_1    =====================================================*/ 
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos                                     (0UL)		/*!<LPAWUR PAYLOAD_1: PAYLOAD_1 (Bit 0) */
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_Msk                                     (0xffffffffUL)		/*!< LPAWUR PAYLOAD_1: PAYLOAD_1 (Bitfield-Mask: 0xffffffff) */
#define LPAWUR_PAYLOAD_1_PAYLOAD_1                                         LPAWUR_PAYLOAD_1_PAYLOAD_1_Msk
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_0                                       (0x1U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_1                                       (0x2U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_2                                       (0x4U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_3                                       (0x8U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_4                                       (0x10U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_5                                       (0x20U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_6                                       (0x40U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_7                                       (0x80U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_8                                       (0x100U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_9                                       (0x200U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_10                                      (0x400U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_11                                      (0x800U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_12                                      (0x1000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_13                                      (0x2000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_14                                      (0x4000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_15                                      (0x8000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_16                                      (0x10000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_17                                      (0x20000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_18                                      (0x40000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_19                                      (0x80000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_20                                      (0x100000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_21                                      (0x200000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_22                                      (0x400000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_23                                      (0x800000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_24                                      (0x1000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_25                                      (0x2000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_26                                      (0x4000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_27                                      (0x8000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_28                                      (0x10000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_29                                      (0x20000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_30                                      (0x40000000U << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)
#define LPAWUR_PAYLOAD_1_PAYLOAD_1_31                                      (0x80000000UL << LPAWUR_PAYLOAD_1_PAYLOAD_1_Pos)

/* =====================================================    RFIP_VERSION    =====================================================*/ 
#define LPAWUR_RFIP_VERSION_PRODUCT_Pos                                    (12UL)		/*!<LPAWUR RFIP_VERSION: PRODUCT (Bit 12) */
#define LPAWUR_RFIP_VERSION_PRODUCT_Msk                                    (0xf000UL)		/*!< LPAWUR RFIP_VERSION: PRODUCT (Bitfield-Mask: 0x0f) */
#define LPAWUR_RFIP_VERSION_PRODUCT                                        LPAWUR_RFIP_VERSION_PRODUCT_Msk
#define LPAWUR_RFIP_VERSION_PRODUCT_0                                      (0x1U << LPAWUR_RFIP_VERSION_PRODUCT_Pos)
#define LPAWUR_RFIP_VERSION_PRODUCT_1                                      (0x2U << LPAWUR_RFIP_VERSION_PRODUCT_Pos)
#define LPAWUR_RFIP_VERSION_PRODUCT_2                                      (0x4U << LPAWUR_RFIP_VERSION_PRODUCT_Pos)
#define LPAWUR_RFIP_VERSION_PRODUCT_3                                      (0x8U << LPAWUR_RFIP_VERSION_PRODUCT_Pos)
#define LPAWUR_RFIP_VERSION_VERSION_Pos                                    (8UL)		/*!<LPAWUR RFIP_VERSION: VERSION (Bit 8) */
#define LPAWUR_RFIP_VERSION_VERSION_Msk                                    (0xf00UL)		/*!< LPAWUR RFIP_VERSION: VERSION (Bitfield-Mask: 0x0f) */
#define LPAWUR_RFIP_VERSION_VERSION                                        LPAWUR_RFIP_VERSION_VERSION_Msk
#define LPAWUR_RFIP_VERSION_VERSION_0                                      (0x1U << LPAWUR_RFIP_VERSION_VERSION_Pos)
#define LPAWUR_RFIP_VERSION_VERSION_1                                      (0x2U << LPAWUR_RFIP_VERSION_VERSION_Pos)
#define LPAWUR_RFIP_VERSION_VERSION_2                                      (0x4U << LPAWUR_RFIP_VERSION_VERSION_Pos)
#define LPAWUR_RFIP_VERSION_VERSION_3                                      (0x8U << LPAWUR_RFIP_VERSION_VERSION_Pos)
#define LPAWUR_RFIP_VERSION_REVISION_Pos                                   (4UL)		/*!<LPAWUR RFIP_VERSION: REVISION (Bit 4) */
#define LPAWUR_RFIP_VERSION_REVISION_Msk                                   (0xf0UL)		/*!< LPAWUR RFIP_VERSION: REVISION (Bitfield-Mask: 0x0f) */
#define LPAWUR_RFIP_VERSION_REVISION                                       LPAWUR_RFIP_VERSION_REVISION_Msk
#define LPAWUR_RFIP_VERSION_REVISION_0                                     (0x1U << LPAWUR_RFIP_VERSION_REVISION_Pos)
#define LPAWUR_RFIP_VERSION_REVISION_1                                     (0x2U << LPAWUR_RFIP_VERSION_REVISION_Pos)
#define LPAWUR_RFIP_VERSION_REVISION_2                                     (0x4U << LPAWUR_RFIP_VERSION_REVISION_Pos)
#define LPAWUR_RFIP_VERSION_REVISION_3                                     (0x8U << LPAWUR_RFIP_VERSION_REVISION_Pos)

/* =====================================================    IRQ_ENABLE    =====================================================*/ 
#define LPAWUR_IRQ_ENABLE_FRAME_VALID_E_Pos                                (3UL)		/*!<LPAWUR IRQ_ENABLE: FRAME_VALID_E (Bit 3) */
#define LPAWUR_IRQ_ENABLE_FRAME_VALID_E_Msk                                (0x8UL)		/*!< LPAWUR IRQ_ENABLE: FRAME_VALID_E (Bitfield-Mask: 0x01) */
#define LPAWUR_IRQ_ENABLE_FRAME_VALID_E                                    LPAWUR_IRQ_ENABLE_FRAME_VALID_E_Msk
#define LPAWUR_IRQ_ENABLE_FRAME_COMPLETE_E_Pos                             (2UL)		/*!<LPAWUR IRQ_ENABLE: FRAME_COMPLETE_E (Bit 2) */
#define LPAWUR_IRQ_ENABLE_FRAME_COMPLETE_E_Msk                             (0x4UL)		/*!< LPAWUR IRQ_ENABLE: FRAME_COMPLETE_E (Bitfield-Mask: 0x01) */
#define LPAWUR_IRQ_ENABLE_FRAME_COMPLETE_E                                 LPAWUR_IRQ_ENABLE_FRAME_COMPLETE_E_Msk
#define LPAWUR_IRQ_ENABLE_FRAME_SYNC_COMPLETE_E_Pos                        (1UL)		/*!<LPAWUR IRQ_ENABLE: FRAME_SYNC_COMPLETE_E (Bit 1) */
#define LPAWUR_IRQ_ENABLE_FRAME_SYNC_COMPLETE_E_Msk                        (0x2UL)		/*!< LPAWUR IRQ_ENABLE: FRAME_SYNC_COMPLETE_E (Bitfield-Mask: 0x01) */
#define LPAWUR_IRQ_ENABLE_FRAME_SYNC_COMPLETE_E                            LPAWUR_IRQ_ENABLE_FRAME_SYNC_COMPLETE_E_Msk
#define LPAWUR_IRQ_ENABLE_BIT_SYNC_DETECTED_E_Pos                          (0UL)		/*!<LPAWUR IRQ_ENABLE: BIT_SYNC_DETECTED_E (Bit 0) */
#define LPAWUR_IRQ_ENABLE_BIT_SYNC_DETECTED_E_Msk                          (0x1UL)		/*!< LPAWUR IRQ_ENABLE: BIT_SYNC_DETECTED_E (Bitfield-Mask: 0x01) */
#define LPAWUR_IRQ_ENABLE_BIT_SYNC_DETECTED_E                              LPAWUR_IRQ_ENABLE_BIT_SYNC_DETECTED_E_Msk

/* =====================================================    STATUS    =====================================================*/ 
#define LPAWUR_STATUS_ERROR_F_Pos                                          (30UL)		/*!<LPAWUR STATUS: ERROR_F (Bit 30) */
#define LPAWUR_STATUS_ERROR_F_Msk                                          (0xc0000000UL)		/*!< LPAWUR STATUS: ERROR_F (Bitfield-Mask: 0x03) */
#define LPAWUR_STATUS_ERROR_F                                              LPAWUR_STATUS_ERROR_F_Msk
#define LPAWUR_STATUS_ERROR_F_0                                            (0x1U << LPAWUR_STATUS_ERROR_F_Pos)
#define LPAWUR_STATUS_ERROR_F_1                                            (0x2U << LPAWUR_STATUS_ERROR_F_Pos)
#define LPAWUR_STATUS_FRAME_VALID_F_Pos                                    (3UL)		/*!<LPAWUR STATUS: FRAME_VALID_F (Bit 3) */
#define LPAWUR_STATUS_FRAME_VALID_F_Msk                                    (0x8UL)		/*!< LPAWUR STATUS: FRAME_VALID_F (Bitfield-Mask: 0x01) */
#define LPAWUR_STATUS_FRAME_VALID_F                                        LPAWUR_STATUS_FRAME_VALID_F_Msk
#define LPAWUR_STATUS_FRAME_COMPLETE_F_Pos                                 (2UL)		/*!<LPAWUR STATUS: FRAME_COMPLETE_F (Bit 2) */
#define LPAWUR_STATUS_FRAME_COMPLETE_F_Msk                                 (0x4UL)		/*!< LPAWUR STATUS: FRAME_COMPLETE_F (Bitfield-Mask: 0x01) */
#define LPAWUR_STATUS_FRAME_COMPLETE_F                                     LPAWUR_STATUS_FRAME_COMPLETE_F_Msk
#define LPAWUR_STATUS_FRAME_SYNC_COMPLETE_F_Pos                            (1UL)		/*!<LPAWUR STATUS: FRAME_SYNC_COMPLETE_F (Bit 1) */
#define LPAWUR_STATUS_FRAME_SYNC_COMPLETE_F_Msk                            (0x2UL)		/*!< LPAWUR STATUS: FRAME_SYNC_COMPLETE_F (Bitfield-Mask: 0x01) */
#define LPAWUR_STATUS_FRAME_SYNC_COMPLETE_F                                LPAWUR_STATUS_FRAME_SYNC_COMPLETE_F_Msk
#define LPAWUR_STATUS_BIT_SYNC_DETECTED_F_Pos                              (0UL)		/*!<LPAWUR STATUS: BIT_SYNC_DETECTED_F (Bit 0) */
#define LPAWUR_STATUS_BIT_SYNC_DETECTED_F_Msk                              (0x1UL)		/*!< LPAWUR STATUS: BIT_SYNC_DETECTED_F (Bitfield-Mask: 0x01) */
#define LPAWUR_STATUS_BIT_SYNC_DETECTED_F                                  LPAWUR_STATUS_BIT_SYNC_DETECTED_F_Msk


/* ============================================================================================================================*/ 
/*=====================                               MR_SUBG_GLOB_DYNAMIC                               =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    PCKTLEN_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos                    (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC PCKTLEN_CONFIG: PCKTLEN (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Msk                    (0xffffUL)		/*!< MR_SUBG_GLOB_DYNAMIC PCKTLEN_CONFIG: PCKTLEN (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN                        MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Msk
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_0                      (0x1U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_1                      (0x2U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_2                      (0x4U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_3                      (0x8U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_4                      (0x10U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_5                      (0x20U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_6                      (0x40U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_7                      (0x80U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_8                      (0x100U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_9                      (0x200U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_10                     (0x400U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_11                     (0x800U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_12                     (0x1000U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_13                     (0x2000U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_14                     (0x4000U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)
#define MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_15                     (0x8000U << MR_SUBG_GLOB_DYNAMIC_PCKTLEN_CONFIG_PCKTLEN_Pos)

/* =====================================================    MOD0_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_PA_CLKON_LOCKONTX_Pos             (31UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: PA_CLKON_LOCKONTX (Bit 31) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_PA_CLKON_LOCKONTX_Msk             (0x80000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: PA_CLKON_LOCKONTX (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_PA_CLKON_LOCKONTX                 MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_PA_CLKON_LOCKONTX_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_BT_SEL_Pos                        (26UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: BT_SEL (Bit 26) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_BT_SEL_Msk                        (0x4000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: BT_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_BT_SEL                            MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_BT_SEL_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_Pos                     (24UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: CONST_MAP (Bit 24) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_Msk                     (0x3000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: CONST_MAP (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP                         MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_0                       (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_1                       (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_CONST_MAP_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Pos                      (20UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: MOD_TYPE (Bit 20) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Msk                      (0x700000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: MOD_TYPE (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE                          MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_0                        (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_1                        (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_2                        (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_MOD_TYPE_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Pos                    (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: DATARATE_E (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Msk                    (0xf0000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: DATARATE_E (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E                        MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_0                      (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_1                      (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_2                      (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_3                      (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos                    (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: DATARATE_M (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Msk                    (0xffffUL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD0_CONFIG: DATARATE_M (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M                        MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_0                      (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_1                      (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_2                      (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_3                      (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_4                      (0x10U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_5                      (0x20U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_6                      (0x40U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_7                      (0x80U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_8                      (0x100U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_9                      (0x200U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_10                     (0x400U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_11                     (0x800U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_12                     (0x1000U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_13                     (0x2000U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_14                     (0x4000U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_15                     (0x8000U << MR_SUBG_GLOB_DYNAMIC_MOD0_CONFIG_DATARATE_M_Pos)

/* =====================================================    MOD1_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Pos                       (20UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: CHFLT_E (Bit 20) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Msk                       (0xf00000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: CHFLT_E (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E                           MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_0                         (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_1                         (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_2                         (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_3                         (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Pos                       (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: CHFLT_M (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Msk                       (0xf0000UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: CHFLT_M (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M                           MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_0                         (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_1                         (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_2                         (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_3                         (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_CHFLT_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Pos                        (8UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: FDEV_E (Bit 8) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Msk                        (0xf00UL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: FDEV_E (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E                            MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_0                          (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_1                          (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_2                          (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_3                          (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_E_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos                        (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: FDEV_M (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Msk                        (0xffUL)		/*!< MR_SUBG_GLOB_DYNAMIC MOD1_CONFIG: FDEV_M (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M                            MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Msk
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_0                          (0x1U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_1                          (0x2U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_2                          (0x4U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_3                          (0x8U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_4                          (0x10U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_5                          (0x20U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_6                          (0x40U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)
#define MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_7                          (0x80U << MR_SUBG_GLOB_DYNAMIC_MOD1_CONFIG_FDEV_M_Pos)

/* =====================================================    SYNTH_FREQ    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_BS_Pos                             (30UL)		/*!<MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: BS (Bit 30) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_BS_Msk                             (0x40000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: BS (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_BS                                 MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_BS_Msk
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos                      (20UL)		/*!<MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: SYNTH_INT (Bit 20) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Msk                      (0xff00000UL)		/*!< MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: SYNTH_INT (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT                          MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Msk
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_0                        (0x1U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_1                        (0x2U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_2                        (0x4U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_3                        (0x8U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_4                        (0x10U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_5                        (0x20U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_6                        (0x40U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_7                        (0x80U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_INT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos                     (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: SYNTH_FRAC (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Msk                     (0xfffffUL)		/*!< MR_SUBG_GLOB_DYNAMIC SYNTH_FREQ: SYNTH_FRAC (Bitfield-Mask: 0xfffff) */
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC                         MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Msk
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_0                       (0x1U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_1                       (0x2U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_2                       (0x4U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_3                       (0x8U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_4                       (0x10U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_5                       (0x20U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_6                       (0x40U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_7                       (0x80U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_8                       (0x100U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_9                       (0x200U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_10                      (0x400U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_11                      (0x800U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_12                      (0x1000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_13                      (0x2000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_14                      (0x4000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_15                      (0x8000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_16                      (0x10000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_17                      (0x20000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_18                      (0x40000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_19                      (0x80000U << MR_SUBG_GLOB_DYNAMIC_SYNTH_FREQ_SYNTH_FRAC_Pos)

/* =====================================================    VCO_CAL_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALIB_REQ_Pos              (31UL)		/*!<MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALIB_REQ (Bit 31) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALIB_REQ_Msk              (0x80000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALIB_REQ (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALIB_REQ                  MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALIB_REQ_Msk
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_SEL_Pos        (23UL)		/*!<MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALFREQ_EXT_SEL (Bit 23) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_SEL_Msk        (0x800000UL)		/*!< MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALFREQ_EXT_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_SEL            MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_SEL_Msk
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos            (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALFREQ_EXT (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Msk            (0x7f0000UL)		/*!< MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALFREQ_EXT (Bitfield-Mask: 0x7f) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT                MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Msk
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_0              (0x1U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_1              (0x2U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_2              (0x4U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_3              (0x8U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_4              (0x10U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_5              (0x20U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_6              (0x40U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALFREQ_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_SEL_Pos         (15UL)		/*!<MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALAMP_EXT_SEL (Bit 15) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_SEL_Msk         (0x8000UL)		/*!< MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALAMP_EXT_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_SEL             MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_SEL_Msk
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos             (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALAMP_EXT (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Msk             (0x3fffUL)		/*!< MR_SUBG_GLOB_DYNAMIC VCO_CAL_CONFIG: VCO_CALAMP_EXT (Bitfield-Mask: 0x3fff) */
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT                 MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Msk
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_0               (0x1U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_1               (0x2U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_2               (0x4U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_3               (0x8U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_4               (0x10U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_5               (0x20U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_6               (0x40U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_7               (0x80U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_8               (0x100U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_9               (0x200U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_10              (0x400U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_11              (0x800U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_12              (0x1000U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_13              (0x2000U << MR_SUBG_GLOB_DYNAMIC_VCO_CAL_CONFIG_VCO_CALAMP_EXT_Pos)

/* =====================================================    RX_TIMER    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_OR_nAND_SELECT_Pos                (31UL)		/*!<MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_OR_nAND_SELECT (Bit 31) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_OR_nAND_SELECT_Msk                (0x80000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_OR_nAND_SELECT (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_OR_nAND_SELECT                    MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_OR_nAND_SELECT_Msk
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_SQI_TIMEOUT_MASK_Pos              (30UL)		/*!<MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_SQI_TIMEOUT_MASK (Bit 30) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_SQI_TIMEOUT_MASK_Msk              (0x40000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_SQI_TIMEOUT_MASK (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_SQI_TIMEOUT_MASK                  MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_SQI_TIMEOUT_MASK_Msk
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_PQI_TIMEOUT_MASK_Pos              (29UL)		/*!<MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_PQI_TIMEOUT_MASK (Bit 29) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_PQI_TIMEOUT_MASK_Msk              (0x20000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_PQI_TIMEOUT_MASK (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_PQI_TIMEOUT_MASK                  MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_PQI_TIMEOUT_MASK_Msk
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_CS_TIMEOUT_MASK_Pos               (28UL)		/*!<MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_CS_TIMEOUT_MASK (Bit 28) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_CS_TIMEOUT_MASK_Msk               (0x10000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_CS_TIMEOUT_MASK (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_CS_TIMEOUT_MASK                   MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_CS_TIMEOUT_MASK_Msk
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos                       (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_TIMEOUT (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Msk                       (0x7fffffUL)		/*!< MR_SUBG_GLOB_DYNAMIC RX_TIMER: RX_TIMEOUT (Bitfield-Mask: 0x7fffff) */
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT                           MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Msk
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_0                         (0x1U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_1                         (0x2U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_2                         (0x4U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_3                         (0x8U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_4                         (0x10U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_5                         (0x20U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_6                         (0x40U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_7                         (0x80U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_8                         (0x100U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_9                         (0x200U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_10                        (0x400U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_11                        (0x800U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_12                        (0x1000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_13                        (0x2000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_14                        (0x4000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_15                        (0x8000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_16                        (0x10000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_17                        (0x20000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_18                        (0x40000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_19                        (0x80000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_20                        (0x100000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_21                        (0x200000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_22                        (0x400000U << MR_SUBG_GLOB_DYNAMIC_RX_TIMER_RX_TIMEOUT_Pos)

/* =====================================================    DATABUFFER_THR    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos        (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC DATABUFFER_THR: TX_ALMOST_EMPTY_THR (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Msk        (0xffff0000UL)		/*!< MR_SUBG_GLOB_DYNAMIC DATABUFFER_THR: TX_ALMOST_EMPTY_THR (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR            MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Msk
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_0          (0x1U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_1          (0x2U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_2          (0x4U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_3          (0x8U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_4          (0x10U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_5          (0x20U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_6          (0x40U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_7          (0x80U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_8          (0x100U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_9          (0x200U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_10         (0x400U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_11         (0x800U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_12         (0x1000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_13         (0x2000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_14         (0x4000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_15         (0x8000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_TX_ALMOST_EMPTY_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos         (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC DATABUFFER_THR: RX_ALMOST_FULL_THR (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Msk         (0xffffUL)		/*!< MR_SUBG_GLOB_DYNAMIC DATABUFFER_THR: RX_ALMOST_FULL_THR (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR             MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Msk
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_0           (0x1U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_1           (0x2U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_2           (0x4U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_3           (0x8U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_4           (0x10U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_5           (0x20U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_6           (0x40U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_7           (0x80U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_8           (0x100U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_9           (0x200U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_10          (0x400U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_11          (0x800U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_12          (0x1000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_13          (0x2000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_14          (0x4000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)
#define MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_15          (0x8000U << MR_SUBG_GLOB_DYNAMIC_DATABUFFER_THR_RX_ALMOST_FULL_THR_Pos)

/* =====================================================    RFSEQ_IRQ_ENABLE    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AGC_CALIB_DONE_E_Pos         (31UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: AGC_CALIB_DONE_E (Bit 31) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AGC_CALIB_DONE_E_Msk         (0x80000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: AGC_CALIB_DONE_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AGC_CALIB_DONE_E             MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AGC_CALIB_DONE_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SAFEASK_CALIB_DONE_E_Pos     (30UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SAFEASK_CALIB_DONE_E (Bit 30) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SAFEASK_CALIB_DONE_E_Msk     (0x40000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SAFEASK_CALIB_DONE_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SAFEASK_CALIB_DONE_E         MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SAFEASK_CALIB_DONE_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_END_E_Pos            (28UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RRM_CMD_END_E (Bit 28) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_END_E_Msk            (0x10000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RRM_CMD_END_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_END_E                MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_END_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_START_E_Pos          (27UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RRM_CMD_START_E (Bit 27) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_START_E_Msk          (0x8000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RRM_CMD_START_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_START_E              MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RRM_CMD_START_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SEQ_E_Pos                    (26UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SEQ_E (Bit 26) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SEQ_E_Msk                    (0x4000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SEQ_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SEQ_E                        MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SEQ_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_HW_ANA_FAILURE_E_Pos         (24UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: HW_ANA_FAILURE_E (Bit 24) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_HW_ANA_FAILURE_E_Msk         (0x1000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: HW_ANA_FAILURE_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_HW_ANA_FAILURE_E             MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_HW_ANA_FAILURE_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AHB_ACCESS_ERROR_E_Pos       (22UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: AHB_ACCESS_ERROR_E (Bit 22) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AHB_ACCESS_ERROR_E_Msk       (0x400000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: AHB_ACCESS_ERROR_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AHB_ACCESS_ERROR_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_AHB_ACCESS_ERROR_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_1_E_Pos      (21UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_ALMOST_EMPTY_1_E (Bit 21) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_1_E_Msk      (0x200000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_ALMOST_EMPTY_1_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_1_E          MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_1_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_0_E_Pos      (20UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_ALMOST_EMPTY_0_E (Bit 20) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_0_E_Msk      (0x100000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_ALMOST_EMPTY_0_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_0_E          MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_ALMOST_EMPTY_0_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_1_E_Pos       (19UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_ALMOST_FULL_1_E (Bit 19) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_1_E_Msk       (0x80000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_ALMOST_FULL_1_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_1_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_1_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_0_E_Pos       (18UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_ALMOST_FULL_0_E (Bit 18) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_0_E_Msk       (0x40000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_ALMOST_FULL_0_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_0_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_ALMOST_FULL_0_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER1_USED_E_Pos       (17UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: DATABUFFER1_USED_E (Bit 17) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER1_USED_E_Msk       (0x20000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: DATABUFFER1_USED_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER1_USED_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER1_USED_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER0_USED_E_Pos       (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: DATABUFFER0_USED_E (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER0_USED_E_Msk       (0x10000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: DATABUFFER0_USED_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER0_USED_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_DATABUFFER0_USED_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SYNC_VALID_E_Pos             (14UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SYNC_VALID_E (Bit 14) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SYNC_VALID_E_Msk             (0x4000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SYNC_VALID_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SYNC_VALID_E                 MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SYNC_VALID_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_PREAMBLE_VALID_E_Pos         (13UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: PREAMBLE_VALID_E (Bit 13) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_PREAMBLE_VALID_E_Msk         (0x2000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: PREAMBLE_VALID_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_PREAMBLE_VALID_E             MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_PREAMBLE_VALID_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_CS_E_Pos                     (12UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: CS_E (Bit 12) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_CS_E_Msk                     (0x1000UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: CS_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_CS_E                         MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_CS_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_COMMAND_REJECTED_E_Pos       (9UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: COMMAND_REJECTED_E (Bit 9) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_COMMAND_REJECTED_E_Msk       (0x200UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: COMMAND_REJECTED_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_COMMAND_REJECTED_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_COMMAND_REJECTED_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SABORT_DONE_E_Pos            (8UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SABORT_DONE_E (Bit 8) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SABORT_DONE_E_Msk            (0x100UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: SABORT_DONE_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SABORT_DONE_E                MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_SABORT_DONE_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RXTIMER_STOP_CDT_E_Pos       (7UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RXTIMER_STOP_CDT_E (Bit 7) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RXTIMER_STOP_CDT_E_Msk       (0x80UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RXTIMER_STOP_CDT_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RXTIMER_STOP_CDT_E           MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RXTIMER_STOP_CDT_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_FAST_RX_TERM_E_Pos           (4UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: FAST_RX_TERM_E (Bit 4) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_FAST_RX_TERM_E_Msk           (0x10UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: FAST_RX_TERM_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_FAST_RX_TERM_E               MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_FAST_RX_TERM_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_CRC_ERROR_E_Pos           (3UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_CRC_ERROR_E (Bit 3) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_CRC_ERROR_E_Msk           (0x8UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_CRC_ERROR_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_CRC_ERROR_E               MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_CRC_ERROR_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_TIMEOUT_E_Pos             (2UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_TIMEOUT_E (Bit 2) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_TIMEOUT_E_Msk             (0x4UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_TIMEOUT_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_TIMEOUT_E                 MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_TIMEOUT_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_OK_E_Pos                  (1UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_OK_E (Bit 1) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_OK_E_Msk                  (0x2UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: RX_OK_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_OK_E                      MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_RX_OK_E_Msk
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_DONE_E_Pos                (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_DONE_E (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_DONE_E_Msk                (0x1UL)		/*!< MR_SUBG_GLOB_DYNAMIC RFSEQ_IRQ_ENABLE: TX_DONE_E (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_DONE_E                    MR_SUBG_GLOB_DYNAMIC_RFSEQ_IRQ_ENABLE_TX_DONE_E_Msk

/* =====================================================    ADDITIONAL_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_AS_ENABLE_Pos                 (31UL)		/*!<MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: AS_ENABLE (Bit 31) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_AS_ENABLE_Msk                 (0x80000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: AS_ENABLE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_AS_ENABLE                     MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_AS_ENABLE_Msk
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Pos           (20UL)		/*!<MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: TIME_CAPTURESEL (Bit 20) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Msk           (0x700000UL)		/*!< MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: TIME_CAPTURESEL (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL               MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Msk
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_0             (0x1U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_1             (0x2U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_2             (0x4U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_TIME_CAPTURESEL_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_Pos                     (16UL)		/*!<MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: PA_FC (Bit 16) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_Msk                     (0x30000UL)		/*!< MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: PA_FC (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC                         MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_Msk
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_0                       (0x1U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_1                       (0x2U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_PA_FC_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos                (8UL)		/*!<MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: CH_SPACING (Bit 8) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Msk                (0xff00UL)		/*!< MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: CH_SPACING (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING                    MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Msk
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_0                  (0x1U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_1                  (0x2U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_2                  (0x4U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_3                  (0x8U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_4                  (0x10U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_5                  (0x20U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_6                  (0x40U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_7                  (0x80U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_SPACING_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos                    (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: CH_NUM (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Msk                    (0xffUL)		/*!< MR_SUBG_GLOB_DYNAMIC ADDITIONAL_CTRL: CH_NUM (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM                        MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Msk
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_0                      (0x1U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_1                      (0x2U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_2                      (0x4U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_3                      (0x8U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_4                      (0x10U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_5                      (0x20U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_6                      (0x40U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)
#define MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_7                      (0x80U << MR_SUBG_GLOB_DYNAMIC_ADDITIONAL_CTRL_CH_NUM_Pos)

/* =====================================================    FAST_RX_TIMER    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_CS_TERM_EN_Pos             (8UL)		/*!<MR_SUBG_GLOB_DYNAMIC FAST_RX_TIMER: FAST_CS_TERM_EN (Bit 8) */
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_CS_TERM_EN_Msk             (0x100UL)		/*!< MR_SUBG_GLOB_DYNAMIC FAST_RX_TIMER: FAST_CS_TERM_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_CS_TERM_EN                 MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_CS_TERM_EN_Msk
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos             (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC FAST_RX_TIMER: FAST_RX_TIMEOUT (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Msk             (0xffUL)		/*!< MR_SUBG_GLOB_DYNAMIC FAST_RX_TIMER: FAST_RX_TIMEOUT (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT                 MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Msk
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_0               (0x1U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_1               (0x2U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_2               (0x4U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_3               (0x8U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_4               (0x10U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_5               (0x20U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_6               (0x40U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)
#define MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_7               (0x80U << MR_SUBG_GLOB_DYNAMIC_FAST_RX_TIMER_FAST_RX_TIMEOUT_Pos)

/* =====================================================    COMMAND    =====================================================*/ 
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2LOCKON_Pos                       (26UL)		/*!<MR_SUBG_GLOB_DYNAMIC COMMAND: BACK2LOCKON (Bit 26) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2LOCKON_Msk                       (0x4000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC COMMAND: BACK2LOCKON (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2LOCKON                           MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2LOCKON_Msk
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2ACTIVE_Pos                       (25UL)		/*!<MR_SUBG_GLOB_DYNAMIC COMMAND: BACK2ACTIVE (Bit 25) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2ACTIVE_Msk                       (0x2000000UL)		/*!< MR_SUBG_GLOB_DYNAMIC COMMAND: BACK2ACTIVE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2ACTIVE                           MR_SUBG_GLOB_DYNAMIC_COMMAND_BACK2ACTIVE_Msk
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Pos                        (0UL)		/*!<MR_SUBG_GLOB_DYNAMIC COMMAND: COMMAND_ID (Bit 0) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Msk                        (0xfUL)		/*!< MR_SUBG_GLOB_DYNAMIC COMMAND: COMMAND_ID (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID                            MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Msk
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_0                          (0x1U << MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Pos)
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_1                          (0x2U << MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Pos)
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_2                          (0x4U << MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Pos)
#define MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_3                          (0x8U << MR_SUBG_GLOB_DYNAMIC_COMMAND_COMMAND_ID_Pos)


/* ============================================================================================================================*/ 
/*=====================                                 MR_SUBG_GLOB_MISC                                 =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    RFIP_VERSION    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Pos                         (12UL)		/*!<MR_SUBG_GLOB_MISC RFIP_VERSION: PRODUCT (Bit 12) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Msk                         (0xf000UL)		/*!< MR_SUBG_GLOB_MISC RFIP_VERSION: PRODUCT (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT                             MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Msk
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_0                           (0x1U << MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_1                           (0x2U << MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_2                           (0x4U << MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_3                           (0x8U << MR_SUBG_GLOB_MISC_RFIP_VERSION_PRODUCT_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Pos                         (8UL)		/*!<MR_SUBG_GLOB_MISC RFIP_VERSION: VERSION (Bit 8) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Msk                         (0xf00UL)		/*!< MR_SUBG_GLOB_MISC RFIP_VERSION: VERSION (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION                             MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Msk
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_0                           (0x1U << MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_1                           (0x2U << MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_2                           (0x4U << MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_3                           (0x8U << MR_SUBG_GLOB_MISC_RFIP_VERSION_VERSION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Pos                        (4UL)		/*!<MR_SUBG_GLOB_MISC RFIP_VERSION: REVISION (Bit 4) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Msk                        (0xf0UL)		/*!< MR_SUBG_GLOB_MISC RFIP_VERSION: REVISION (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION                            MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Msk
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_0                          (0x1U << MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_1                          (0x2U << MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_2                          (0x4U << MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Pos)
#define MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_3                          (0x8U << MR_SUBG_GLOB_MISC_RFIP_VERSION_REVISION_Pos)

/* =====================================================    RRM_UDRA_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_RRM_UDRA_CTRL_RRM_CMD_REQ_Pos                    (0UL)		/*!<MR_SUBG_GLOB_MISC RRM_UDRA_CTRL: RRM_CMD_REQ (Bit 0) */
#define MR_SUBG_GLOB_MISC_RRM_UDRA_CTRL_RRM_CMD_REQ_Msk                    (0x1UL)		/*!< MR_SUBG_GLOB_MISC RRM_UDRA_CTRL: RRM_CMD_REQ (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_RRM_UDRA_CTRL_RRM_CMD_REQ                        MR_SUBG_GLOB_MISC_RRM_UDRA_CTRL_RRM_CMD_REQ_Msk

/* =====================================================    SEQUENCER_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_DISABLE_SEQ_Pos                   (1UL)		/*!<MR_SUBG_GLOB_MISC SEQUENCER_CTRL: DISABLE_SEQ (Bit 1) */
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_DISABLE_SEQ_Msk                   (0x2UL)		/*!< MR_SUBG_GLOB_MISC SEQUENCER_CTRL: DISABLE_SEQ (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_DISABLE_SEQ                       MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_DISABLE_SEQ_Msk
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_GEN_SEQ_TRIGGER_Pos               (0UL)		/*!<MR_SUBG_GLOB_MISC SEQUENCER_CTRL: GEN_SEQ_TRIGGER (Bit 0) */
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_GEN_SEQ_TRIGGER_Msk               (0x1UL)		/*!< MR_SUBG_GLOB_MISC SEQUENCER_CTRL: GEN_SEQ_TRIGGER (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_GEN_SEQ_TRIGGER                   MR_SUBG_GLOB_MISC_SEQUENCER_CTRL_GEN_SEQ_TRIGGER_Msk

/* =====================================================    ABSOLUTE_TIME    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos                  (0UL)		/*!<MR_SUBG_GLOB_MISC ABSOLUTE_TIME: ABSOLUTE_TIME (Bit 0) */
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Msk                  (0xffffffffUL)		/*!< MR_SUBG_GLOB_MISC ABSOLUTE_TIME: ABSOLUTE_TIME (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME                      MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Msk
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_0                    (0x1U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_1                    (0x2U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_2                    (0x4U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_3                    (0x8U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_4                    (0x10U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_5                    (0x20U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_6                    (0x40U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_7                    (0x80U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_8                    (0x100U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_9                    (0x200U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_10                   (0x400U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_11                   (0x800U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_12                   (0x1000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_13                   (0x2000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_14                   (0x4000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_15                   (0x8000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_16                   (0x10000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_17                   (0x20000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_18                   (0x40000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_19                   (0x80000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_20                   (0x100000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_21                   (0x200000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_22                   (0x400000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_23                   (0x800000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_24                   (0x1000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_25                   (0x2000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_26                   (0x4000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_27                   (0x8000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_28                   (0x10000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_29                   (0x20000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_30                   (0x40000000U << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)
#define MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_31                   (0x80000000UL << MR_SUBG_GLOB_MISC_ABSOLUTE_TIME_ABSOLUTE_TIME_Pos)

/* =====================================================    SCM_COUNTER_VAL    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos          (0UL)		/*!<MR_SUBG_GLOB_MISC SCM_COUNTER_VAL: SCM_COUNTER_CURRVAL (Bit 0) */
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Msk          (0x7fffUL)		/*!< MR_SUBG_GLOB_MISC SCM_COUNTER_VAL: SCM_COUNTER_CURRVAL (Bitfield-Mask: 0x7fff) */
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL              MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Msk
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_0            (0x1U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_1            (0x2U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_2            (0x4U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_3            (0x8U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_4            (0x10U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_5            (0x20U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_6            (0x40U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_7            (0x80U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_8            (0x100U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_9            (0x200U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_10           (0x400U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_11           (0x800U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_12           (0x1000U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_13           (0x2000U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_14           (0x4000U << MR_SUBG_GLOB_MISC_SCM_COUNTER_VAL_SCM_COUNTER_CURRVAL_Pos)

/* =====================================================    SCM_MIN_MAX    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_CLEAR_MIN_MAX_Pos                    (31UL)		/*!<MR_SUBG_GLOB_MISC SCM_MIN_MAX: CLEAR_MIN_MAX (Bit 31) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_CLEAR_MIN_MAX_Msk                    (0x80000000UL)		/*!< MR_SUBG_GLOB_MISC SCM_MIN_MAX: CLEAR_MIN_MAX (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_CLEAR_MIN_MAX                        MR_SUBG_GLOB_MISC_SCM_MIN_MAX_CLEAR_MIN_MAX_Msk
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos               (16UL)		/*!<MR_SUBG_GLOB_MISC SCM_MIN_MAX: SCM_COUNTER_MAXVAL (Bit 16) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Msk               (0x7fff0000UL)		/*!< MR_SUBG_GLOB_MISC SCM_MIN_MAX: SCM_COUNTER_MAXVAL (Bitfield-Mask: 0x7fff) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL                   MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Msk
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_0                 (0x1U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_1                 (0x2U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_2                 (0x4U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_3                 (0x8U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_4                 (0x10U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_5                 (0x20U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_6                 (0x40U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_7                 (0x80U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_8                 (0x100U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_9                 (0x200U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_10                (0x400U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_11                (0x800U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_12                (0x1000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_13                (0x2000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_14                (0x4000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MAXVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos               (0UL)		/*!<MR_SUBG_GLOB_MISC SCM_MIN_MAX: SCM_COUNTER_MINVAL (Bit 0) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Msk               (0x7fffUL)		/*!< MR_SUBG_GLOB_MISC SCM_MIN_MAX: SCM_COUNTER_MINVAL (Bitfield-Mask: 0x7fff) */
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL                   MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Msk
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_0                 (0x1U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_1                 (0x2U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_2                 (0x4U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_3                 (0x8U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_4                 (0x10U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_5                 (0x20U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_6                 (0x40U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_7                 (0x80U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_8                 (0x100U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_9                 (0x200U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_10                (0x400U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_11                (0x800U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_12                (0x1000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_13                (0x2000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)
#define MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_14                (0x4000U << MR_SUBG_GLOB_MISC_SCM_MIN_MAX_SCM_COUNTER_MINVAL_Pos)

/* =====================================================    WAKEUP_IRQ_STATUS    =====================================================*/ 
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_RFIP_WAKEUP_F_Pos              (1UL)		/*!<MR_SUBG_GLOB_MISC WAKEUP_IRQ_STATUS: RFIP_WAKEUP_F (Bit 1) */
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_RFIP_WAKEUP_F_Msk              (0x2UL)		/*!< MR_SUBG_GLOB_MISC WAKEUP_IRQ_STATUS: RFIP_WAKEUP_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_RFIP_WAKEUP_F                  MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_RFIP_WAKEUP_F_Msk
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_CPU_WAKEUP_F_Pos               (0UL)		/*!<MR_SUBG_GLOB_MISC WAKEUP_IRQ_STATUS: CPU_WAKEUP_F (Bit 0) */
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_CPU_WAKEUP_F_Msk               (0x1UL)		/*!< MR_SUBG_GLOB_MISC WAKEUP_IRQ_STATUS: CPU_WAKEUP_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_CPU_WAKEUP_F                   MR_SUBG_GLOB_MISC_WAKEUP_IRQ_STATUS_CPU_WAKEUP_F_Msk


/* ============================================================================================================================*/ 
/*=====================                               MR_SUBG_GLOB_RETAINED                               =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    RFIP_WAKEUPTIME    =====================================================*/ 
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos          (0UL)		/*!<MR_SUBG_GLOB_RETAINED RFIP_WAKEUPTIME: RFIP_WAKEUPTIME (Bit 0) */
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Msk          (0xffffffffUL)		/*!< MR_SUBG_GLOB_RETAINED RFIP_WAKEUPTIME: RFIP_WAKEUPTIME (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME              MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Msk
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_0            (0x1U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_1            (0x2U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_2            (0x4U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_3            (0x8U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_4            (0x10U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_5            (0x20U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_6            (0x40U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_7            (0x80U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_8            (0x100U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_9            (0x200U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_10           (0x400U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_11           (0x800U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_12           (0x1000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_13           (0x2000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_14           (0x4000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_15           (0x8000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_16           (0x10000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_17           (0x20000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_18           (0x40000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_19           (0x80000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_20           (0x100000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_21           (0x200000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_22           (0x400000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_23           (0x800000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_24           (0x1000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_25           (0x2000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_26           (0x4000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_27           (0x8000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_28           (0x10000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_29           (0x20000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_30           (0x40000000U << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_31           (0x80000000UL << MR_SUBG_GLOB_RETAINED_RFIP_WAKEUPTIME_RFIP_WAKEUPTIME_Pos)

/* =====================================================    CPU_WAKEUPTIME    =====================================================*/ 
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos            (1UL)		/*!<MR_SUBG_GLOB_RETAINED CPU_WAKEUPTIME: CPU_WAKEUPTIME (Bit 1) */
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Msk            (0xfffffffeUL)		/*!< MR_SUBG_GLOB_RETAINED CPU_WAKEUPTIME: CPU_WAKEUPTIME (Bitfield-Mask: 0x7fffffff) */
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME                MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Msk
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_0              (0x1U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_1              (0x2U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_2              (0x4U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_3              (0x8U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_4              (0x10U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_5              (0x20U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_6              (0x40U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_7              (0x80U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_8              (0x100U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_9              (0x200U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_10             (0x400U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_11             (0x800U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_12             (0x1000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_13             (0x2000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_14             (0x4000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_15             (0x8000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_16             (0x10000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_17             (0x20000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_18             (0x40000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_19             (0x80000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_20             (0x100000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_21             (0x200000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_22             (0x400000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_23             (0x800000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_24             (0x1000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_25             (0x2000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_26             (0x4000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_27             (0x8000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_28             (0x10000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_29             (0x20000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)
#define MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_30             (0x40000000U << MR_SUBG_GLOB_RETAINED_CPU_WAKEUPTIME_CPU_WAKEUPTIME_Pos)

/* =====================================================    WAKEUP_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_RFIP_WAKEUP_EN_Pos               (31UL)		/*!<MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: RFIP_WAKEUP_EN (Bit 31) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_RFIP_WAKEUP_EN_Msk               (0x80000000UL)		/*!< MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: RFIP_WAKEUP_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_RFIP_WAKEUP_EN                   MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_RFIP_WAKEUP_EN_Msk
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_CPU_WAKEUP_EN_Pos                (30UL)		/*!<MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: CPU_WAKEUP_EN (Bit 30) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_CPU_WAKEUP_EN_Msk                (0x40000000UL)		/*!< MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: CPU_WAKEUP_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_CPU_WAKEUP_EN                    MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_CPU_WAKEUP_EN_Msk
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos            (0UL)		/*!<MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: SOC_WAKEUP_OFFSET (Bit 0) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Msk            (0xffUL)		/*!< MR_SUBG_GLOB_RETAINED WAKEUP_CTRL: SOC_WAKEUP_OFFSET (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET                MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Msk
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_0              (0x1U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_1              (0x2U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_2              (0x4U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_3              (0x8U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_4              (0x10U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_5              (0x20U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_6              (0x40U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_7              (0x80U << MR_SUBG_GLOB_RETAINED_WAKEUP_CTRL_SOC_WAKEUP_OFFSET_Pos)

/* =====================================================    RRM_CMDLIST_PTR    =====================================================*/ 
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_VALID_Pos        (31UL)		/*!<MR_SUBG_GLOB_RETAINED RRM_CMDLIST_PTR: CMDLIST_PTR_VALID (Bit 31) */
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_VALID_Msk        (0x80000000UL)		/*!< MR_SUBG_GLOB_RETAINED RRM_CMDLIST_PTR: CMDLIST_PTR_VALID (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_VALID            MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_VALID_Msk
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos       (0UL)		/*!<MR_SUBG_GLOB_RETAINED RRM_CMDLIST_PTR: CMDLIST_PTR_OFFSET (Bit 0) */
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Msk       (0xffffUL)		/*!< MR_SUBG_GLOB_RETAINED RRM_CMDLIST_PTR: CMDLIST_PTR_OFFSET (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET           MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Msk
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_0         (0x1U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_1         (0x2U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_2         (0x4U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_3         (0x8U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_4         (0x10U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_5         (0x20U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_6         (0x40U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_7         (0x80U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_8         (0x100U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_9         (0x200U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_10        (0x400U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_11        (0x800U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_12        (0x1000U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_13        (0x2000U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_14        (0x4000U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)
#define MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_15        (0x8000U << MR_SUBG_GLOB_RETAINED_RRM_CMDLIST_PTR_CMDLIST_PTR_OFFSET_Pos)

/* =====================================================    SEQ_GLOBALTABLE_PTR    =====================================================*/ 
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos  (0UL)		/*!<MR_SUBG_GLOB_RETAINED SEQ_GLOBALTABLE_PTR: SEQ_GLOBALTABLE_PTR (Bit 0) */
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Msk  (0xffffUL)		/*!< MR_SUBG_GLOB_RETAINED SEQ_GLOBALTABLE_PTR: SEQ_GLOBALTABLE_PTR (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR      MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Msk
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_0    (0x1U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_1    (0x2U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_2    (0x4U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_3    (0x8U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_4    (0x10U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_5    (0x20U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_6    (0x40U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_7    (0x80U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_8    (0x100U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_9    (0x200U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_10   (0x400U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_11   (0x800U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_12   (0x1000U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_13   (0x2000U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_14   (0x4000U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)
#define MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_15   (0x8000U << MR_SUBG_GLOB_RETAINED_SEQ_GLOBALTABLE_PTR_SEQ_GLOBALTABLE_PTR_Pos)


/* ============================================================================================================================*/ 
/*=====================                                MR_SUBG_GLOB_STATIC                                =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    PCKT_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_Pos                  (30UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: POSTAMBLE_SEQ (Bit 30) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_Msk                  (0xc0000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: POSTAMBLE_SEQ (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ                      MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_0                    (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_1                    (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_SEQ_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos               (24UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: POSTAMBLE_LENGTH (Bit 24) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Msk               (0x3f000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: POSTAMBLE_LENGTH (Bitfield-Mask: 0x3f) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH                   MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_0                 (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_1                 (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_2                 (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_3                 (0x8U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_4                 (0x10U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_5                 (0x20U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_POSTAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_Pos                   (22UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: PREAMBLE_SEQ (Bit 22) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_Msk                   (0xc00000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: PREAMBLE_SEQ (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ                       MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_0                     (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_1                     (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_SEQ_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos                (12UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: PREAMBLE_LENGTH (Bit 12) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Msk                (0x3ff000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: PREAMBLE_LENGTH (Bitfield-Mask: 0x3ff) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH                    MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_0                  (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_1                  (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_2                  (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_3                  (0x8U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_4                  (0x10U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_5                  (0x20U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_6                  (0x40U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_7                  (0x80U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_8                  (0x100U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_9                  (0x200U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_PREAMBLE_LENGTH_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_FIX_VAR_LEN_Pos                    (11UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: FIX_VAR_LEN (Bit 11) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_FIX_VAR_LEN_Msk                    (0x800UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: FIX_VAR_LEN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_FIX_VAR_LEN                        MR_SUBG_GLOB_STATIC_PCKT_CONFIG_FIX_VAR_LEN_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_LEN_WIDTH_Pos                      (10UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: LEN_WIDTH (Bit 10) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_LEN_WIDTH_Msk                      (0x400UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: LEN_WIDTH (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_LEN_WIDTH                          MR_SUBG_GLOB_STATIC_PCKT_CONFIG_LEN_WIDTH_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_PRESENT_Pos                   (9UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: SYNC_PRESENT (Bit 9) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_PRESENT_Msk                   (0x200UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: SYNC_PRESENT (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_PRESENT                       MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_PRESENT_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos                       (4UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: SYNC_LEN (Bit 4) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Msk                       (0x1f0UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: SYNC_LEN (Bitfield-Mask: 0x1f) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN                           MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_0                         (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_1                         (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_2                         (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_3                         (0x8U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_4                         (0x10U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SYNC_LEN_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SECONDARY_SYNC_SEL_Pos             (3UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: SECONDARY_SYNC_SEL (Bit 3) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SECONDARY_SYNC_SEL_Msk             (0x8UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: SECONDARY_SYNC_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SECONDARY_SYNC_SEL                 MR_SUBG_GLOB_STATIC_PCKT_CONFIG_SECONDARY_SYNC_SEL_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Pos                       (0UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CONFIG: CRC_MODE (Bit 0) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Msk                       (0x7UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CONFIG: CRC_MODE (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE                           MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_0                         (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_1                         (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_2                         (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CONFIG_CRC_MODE_Pos)

/* =====================================================    SYNC    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos                                  (0UL)		/*!<MR_SUBG_GLOB_STATIC SYNC: SYNC (Bit 0) */
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_Msk                                  (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATIC SYNC: SYNC (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC                                      MR_SUBG_GLOB_STATIC_SYNC_SYNC_Msk
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_0                                    (0x1U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_1                                    (0x2U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_2                                    (0x4U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_3                                    (0x8U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_4                                    (0x10U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_5                                    (0x20U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_6                                    (0x40U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_7                                    (0x80U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_8                                    (0x100U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_9                                    (0x200U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_10                                   (0x400U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_11                                   (0x800U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_12                                   (0x1000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_13                                   (0x2000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_14                                   (0x4000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_15                                   (0x8000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_16                                   (0x10000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_17                                   (0x20000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_18                                   (0x40000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_19                                   (0x80000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_20                                   (0x100000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_21                                   (0x200000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_22                                   (0x400000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_23                                   (0x800000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_24                                   (0x1000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_25                                   (0x2000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_26                                   (0x4000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_27                                   (0x8000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_28                                   (0x10000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_29                                   (0x20000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_30                                   (0x40000000U << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SYNC_SYNC_31                                   (0x80000000UL << MR_SUBG_GLOB_STATIC_SYNC_SYNC_Pos)

/* =====================================================    SEC_SYNC    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos                          (0UL)		/*!<MR_SUBG_GLOB_STATIC SEC_SYNC: SEC_SYNC (Bit 0) */
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Msk                          (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATIC SEC_SYNC: SEC_SYNC (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC                              MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Msk
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_0                            (0x1U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_1                            (0x2U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_2                            (0x4U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_3                            (0x8U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_4                            (0x10U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_5                            (0x20U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_6                            (0x40U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_7                            (0x80U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_8                            (0x100U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_9                            (0x200U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_10                           (0x400U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_11                           (0x800U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_12                           (0x1000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_13                           (0x2000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_14                           (0x4000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_15                           (0x8000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_16                           (0x10000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_17                           (0x20000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_18                           (0x40000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_19                           (0x80000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_20                           (0x100000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_21                           (0x200000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_22                           (0x400000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_23                           (0x800000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_24                           (0x1000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_25                           (0x2000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_26                           (0x4000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_27                           (0x8000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_28                           (0x10000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_29                           (0x20000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_30                           (0x40000000U << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)
#define MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_31                           (0x80000000UL << MR_SUBG_GLOB_STATIC_SEC_SYNC_SEC_SYNC_Pos)

/* =====================================================    CRC_INIT    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos                      (0UL)		/*!<MR_SUBG_GLOB_STATIC CRC_INIT: CRC_INIT_VAL (Bit 0) */
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Msk                      (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATIC CRC_INIT: CRC_INIT_VAL (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL                          MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Msk
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_0                        (0x1U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_1                        (0x2U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_2                        (0x4U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_3                        (0x8U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_4                        (0x10U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_5                        (0x20U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_6                        (0x40U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_7                        (0x80U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_8                        (0x100U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_9                        (0x200U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_10                       (0x400U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_11                       (0x800U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_12                       (0x1000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_13                       (0x2000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_14                       (0x4000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_15                       (0x8000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_16                       (0x10000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_17                       (0x20000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_18                       (0x40000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_19                       (0x80000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_20                       (0x100000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_21                       (0x200000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_22                       (0x400000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_23                       (0x800000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_24                       (0x1000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_25                       (0x2000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_26                       (0x4000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_27                       (0x8000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_28                       (0x10000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_29                       (0x20000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_30                       (0x40000000U << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)
#define MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_31                       (0x80000000UL << MR_SUBG_GLOB_STATIC_CRC_INIT_CRC_INIT_VAL_Pos)

/* =====================================================    PCKT_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FORCE_2FSK_SYNC_MODE_Pos             (31UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: FORCE_2FSK_SYNC_MODE (Bit 31) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FORCE_2FSK_SYNC_MODE_Msk             (0x80000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: FORCE_2FSK_SYNC_MODE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FORCE_2FSK_SYNC_MODE                 MR_SUBG_GLOB_STATIC_PCKT_CTRL_FORCE_2FSK_SYNC_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PN_SEL_Pos                           (29UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: PN_SEL (Bit 29) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PN_SEL_Msk                           (0x20000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: PN_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PN_SEL                               MR_SUBG_GLOB_STATIC_PCKT_CTRL_PN_SEL_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MOD_INTERP_EN_Pos                    (28UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: MOD_INTERP_EN (Bit 28) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MOD_INTERP_EN_Msk                    (0x10000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: MOD_INTERP_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MOD_INTERP_EN                        MR_SUBG_GLOB_STATIC_PCKT_CTRL_MOD_INTERP_EN_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FCS_TYPE_4G_Pos                      (27UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: FCS_TYPE_4G (Bit 27) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FCS_TYPE_4G_Msk                      (0x8000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: FCS_TYPE_4G (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FCS_TYPE_4G                          MR_SUBG_GLOB_STATIC_PCKT_CTRL_FCS_TYPE_4G_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FEC_TYPE_4G_Pos                      (26UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: FEC_TYPE_4G (Bit 26) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FEC_TYPE_4G_Msk                      (0x4000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: FEC_TYPE_4G (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FEC_TYPE_4G                          MR_SUBG_GLOB_STATIC_PCKT_CTRL_FEC_TYPE_4G_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_INT_EN_4G_Pos                        (25UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: INT_EN_4G (Bit 25) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_INT_EN_4G_Msk                        (0x2000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: INT_EN_4G (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_INT_EN_4G                            MR_SUBG_GLOB_STATIC_PCKT_CTRL_INT_EN_4G_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MANCHESTER_TYPE_Pos                  (24UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: MANCHESTER_TYPE (Bit 24) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MANCHESTER_TYPE_Msk                  (0x1000000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: MANCHESTER_TYPE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_MANCHESTER_TYPE                      MR_SUBG_GLOB_STATIC_PCKT_CTRL_MANCHESTER_TYPE_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_Pos                       (21UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: CODING_SEL (Bit 21) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_Msk                       (0x600000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: CODING_SEL (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL                           MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_0                         (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_1                         (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_CODING_SEL_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos                        (12UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_INIT (Bit 12) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Msk                        (0x1ff000UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_INIT (Bitfield-Mask: 0x1ff) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT                            MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_0                          (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_1                          (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_2                          (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_3                          (0x8U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_4                          (0x10U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_5                          (0x20U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_6                          (0x40U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_7                          (0x80U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_8                          (0x100U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_INIT_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_EN_Pos                          (11UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_EN (Bit 11) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_EN_Msk                          (0x800UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_EN                              MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_EN_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_BF_FEC_Pos                      (10UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_BF_FEC (Bit 10) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_BF_FEC_Msk                      (0x400UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: WHIT_BF_FEC (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_BF_FEC                          MR_SUBG_GLOB_STATIC_PCKT_CTRL_WHIT_BF_FEC_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_Pos                          (7UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: TX_MODE (Bit 7) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_Msk                          (0x180UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: TX_MODE (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE                              MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_0                            (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_1                            (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_TX_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Pos                          (4UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: RX_MODE (Bit 4) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Msk                          (0x70UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: RX_MODE (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE                              MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_0                            (0x1U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_1                            (0x2U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_2                            (0x4U << MR_SUBG_GLOB_STATIC_PCKT_CTRL_RX_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FOUR_FSK_SYM_SWAP_Pos                (3UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: FOUR_FSK_SYM_SWAP (Bit 3) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FOUR_FSK_SYM_SWAP_Msk                (0x8UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: FOUR_FSK_SYM_SWAP (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_FOUR_FSK_SYM_SWAP                    MR_SUBG_GLOB_STATIC_PCKT_CTRL_FOUR_FSK_SYM_SWAP_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_BYTE_SWAP_Pos                        (2UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: BYTE_SWAP (Bit 2) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_BYTE_SWAP_Msk                        (0x4UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: BYTE_SWAP (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_BYTE_SWAP                            MR_SUBG_GLOB_STATIC_PCKT_CTRL_BYTE_SWAP_Msk
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PCKT_FORMAT_Pos                      (0UL)		/*!<MR_SUBG_GLOB_STATIC PCKT_CTRL: PCKT_FORMAT (Bit 0) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PCKT_FORMAT_Msk                      (0x1UL)		/*!< MR_SUBG_GLOB_STATIC PCKT_CTRL: PCKT_FORMAT (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PCKT_CTRL_PCKT_FORMAT                          MR_SUBG_GLOB_STATIC_PCKT_CTRL_PCKT_FORMAT_Msk

/* =====================================================    DATABUFFER0_PTR    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos            (2UL)		/*!<MR_SUBG_GLOB_STATIC DATABUFFER0_PTR: DATABUFFER0_PTR (Bit 2) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Msk            (0xfffffffcUL)		/*!< MR_SUBG_GLOB_STATIC DATABUFFER0_PTR: DATABUFFER0_PTR (Bitfield-Mask: 0x3fffffff) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR                MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Msk
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_0              (0x1U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_1              (0x2U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_2              (0x4U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_3              (0x8U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_4              (0x10U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_5              (0x20U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_6              (0x40U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_7              (0x80U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_8              (0x100U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_9              (0x200U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_10             (0x400U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_11             (0x800U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_12             (0x1000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_13             (0x2000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_14             (0x4000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_15             (0x8000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_16             (0x10000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_17             (0x20000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_18             (0x40000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_19             (0x80000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_20             (0x100000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_21             (0x200000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_22             (0x400000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_23             (0x800000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_24             (0x1000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_25             (0x2000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_26             (0x4000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_27             (0x8000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_28             (0x10000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_29             (0x20000000U << MR_SUBG_GLOB_STATIC_DATABUFFER0_PTR_DATABUFFER0_PTR_Pos)

/* =====================================================    DATABUFFER1_PTR    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos            (2UL)		/*!<MR_SUBG_GLOB_STATIC DATABUFFER1_PTR: DATABUFFER1_PTR (Bit 2) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Msk            (0xfffffffcUL)		/*!< MR_SUBG_GLOB_STATIC DATABUFFER1_PTR: DATABUFFER1_PTR (Bitfield-Mask: 0x3fffffff) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR                MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Msk
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_0              (0x1U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_1              (0x2U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_2              (0x4U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_3              (0x8U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_4              (0x10U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_5              (0x20U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_6              (0x40U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_7              (0x80U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_8              (0x100U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_9              (0x200U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_10             (0x400U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_11             (0x800U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_12             (0x1000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_13             (0x2000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_14             (0x4000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_15             (0x8000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_16             (0x10000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_17             (0x20000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_18             (0x40000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_19             (0x80000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_20             (0x100000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_21             (0x200000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_22             (0x400000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_23             (0x800000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_24             (0x1000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_25             (0x2000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_26             (0x4000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_27             (0x8000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_28             (0x10000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_29             (0x20000000U << MR_SUBG_GLOB_STATIC_DATABUFFER1_PTR_DATABUFFER1_PTR_Pos)

/* =====================================================    DATABUFFER_SIZE    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos            (0UL)		/*!<MR_SUBG_GLOB_STATIC DATABUFFER_SIZE: DATABUFFER_SIZE (Bit 0) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Msk            (0xffffUL)		/*!< MR_SUBG_GLOB_STATIC DATABUFFER_SIZE: DATABUFFER_SIZE (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE                MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Msk
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_0              (0x1U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_1              (0x2U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_2              (0x4U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_3              (0x8U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_4              (0x10U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_5              (0x20U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_6              (0x40U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_7              (0x80U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_8              (0x100U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_9              (0x200U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_10             (0x400U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_11             (0x800U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_12             (0x1000U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_13             (0x2000U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_14             (0x4000U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)
#define MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_15             (0x8000U << MR_SUBG_GLOB_STATIC_DATABUFFER_SIZE_DATABUFFER_SIZE_Pos)

/* =====================================================    PA_LEVEL_3_0    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos                     (24UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL3 (Bit 24) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Msk                     (0xff000000UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL3 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3                         MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL3_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos                     (16UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL2 (Bit 16) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Msk                     (0xff0000UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL2 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2                         MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL2_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos                     (8UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL1 (Bit 8) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Msk                     (0xff00UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL1 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1                         MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL1_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos                     (0UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL0 (Bit 0) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Msk                     (0xffUL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_3_0: PA_LEVEL0 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0                         MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_3_0_PA_LEVEL0_Pos)

/* =====================================================    PA_LEVEL_7_4    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos                     (24UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL7 (Bit 24) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Msk                     (0xff000000UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL7 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7                         MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL7_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos                     (16UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL6 (Bit 16) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Msk                     (0xff0000UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL6 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6                         MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL6_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos                     (8UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL5 (Bit 8) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Msk                     (0xff00UL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL5 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5                         MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL5_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos                     (0UL)		/*!<MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL4 (Bit 0) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Msk                     (0xffUL)		/*!< MR_SUBG_GLOB_STATIC PA_LEVEL_7_4: PA_LEVEL4 (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4                         MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Msk
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_0                       (0x1U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_1                       (0x2U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_2                       (0x4U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_3                       (0x8U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_4                       (0x10U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_5                       (0x20U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_6                       (0x40U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)
#define MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_7                       (0x80U << MR_SUBG_GLOB_STATIC_PA_LEVEL_7_4_PA_LEVEL4_Pos)

/* =====================================================    PA_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_ENABLE_Pos                   (14UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_RAMP_ENABLE (Bit 14) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_ENABLE_Msk                   (0x4000UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_RAMP_ENABLE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_ENABLE                       MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_ENABLE_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_LIN_NLOG_Pos                         (13UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: LIN_NLOG (Bit 13) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_LIN_NLOG_Msk                         (0x2000UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: LIN_NLOG (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_LIN_NLOG                             MR_SUBG_GLOB_STATIC_PA_CONFIG_LIN_NLOG_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_Pos                          (10UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_MODE (Bit 10) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_Msk                          (0xc00UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_MODE (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE                              MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_0                            (0x1U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_1                            (0x2U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_Pos                      (8UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_DRV_MODE (Bit 8) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_Msk                      (0x300UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_DRV_MODE (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE                          MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_0                        (0x1U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_1                        (0x2U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_DRV_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_ASK_OOK_EN_Pos                       (7UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: ASK_OOK_EN (Bit 7) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_ASK_OOK_EN_Msk                       (0x80UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: ASK_OOK_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_ASK_OOK_EN                           MR_SUBG_GLOB_STATIC_PA_CONFIG_ASK_OOK_EN_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_INTERP_EN_Pos                     (6UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_INTERP_EN (Bit 6) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_INTERP_EN_Msk                     (0x40UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_INTERP_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_INTERP_EN                         MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_INTERP_EN_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Pos               (2UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_LEVEL_MAX_INDEX (Bit 2) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Msk               (0x1cUL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_LEVEL_MAX_INDEX (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX                   MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_0                 (0x1U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_1                 (0x2U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_2                 (0x4U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_LEVEL_MAX_INDEX_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_Pos               (0UL)		/*!<MR_SUBG_GLOB_STATIC PA_CONFIG: PA_RAMP_STEP_WIDTH (Bit 0) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_Msk               (0x3UL)		/*!< MR_SUBG_GLOB_STATIC PA_CONFIG: PA_RAMP_STEP_WIDTH (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH                   MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_Msk
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_0                 (0x1U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_Pos)
#define MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_1                 (0x2U << MR_SUBG_GLOB_STATIC_PA_CONFIG_PA_RAMP_STEP_WIDTH_Pos)

/* =====================================================    IF_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_MODE_Pos                            (31UL)		/*!<MR_SUBG_GLOB_STATIC IF_CTRL: IF_MODE (Bit 31) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_MODE_Msk                            (0x80000000UL)		/*!< MR_SUBG_GLOB_STATIC IF_CTRL: IF_MODE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_MODE                                MR_SUBG_GLOB_STATIC_IF_CTRL_IF_MODE_Msk
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos                      (16UL)		/*!<MR_SUBG_GLOB_STATIC IF_CTRL: IF_OFFSET_ANA (Bit 16) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Msk                      (0x1fff0000UL)		/*!< MR_SUBG_GLOB_STATIC IF_CTRL: IF_OFFSET_ANA (Bitfield-Mask: 0x1fff) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA                          MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Msk
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_0                        (0x1U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_1                        (0x2U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_2                        (0x4U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_3                        (0x8U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_4                        (0x10U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_5                        (0x20U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_6                        (0x40U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_7                        (0x80U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_8                        (0x100U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_9                        (0x200U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_10                       (0x400U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_11                       (0x800U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_12                       (0x1000U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_ANA_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos                      (0UL)		/*!<MR_SUBG_GLOB_STATIC IF_CTRL: IF_OFFSET_DIG (Bit 0) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Msk                      (0x1fffUL)		/*!< MR_SUBG_GLOB_STATIC IF_CTRL: IF_OFFSET_DIG (Bitfield-Mask: 0x1fff) */
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG                          MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Msk
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_0                        (0x1U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_1                        (0x2U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_2                        (0x4U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_3                        (0x8U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_4                        (0x10U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_5                        (0x20U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_6                        (0x40U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_7                        (0x80U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_8                        (0x100U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_9                        (0x200U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_10                       (0x400U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_11                       (0x800U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)
#define MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_12                       (0x1000U << MR_SUBG_GLOB_STATIC_IF_CTRL_IF_OFFSET_DIG_Pos)

/* =====================================================    AS_QI_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_CS_BLANKING_Pos                  (31UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_CS_BLANKING (Bit 31) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_CS_BLANKING_Msk                  (0x80000000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_CS_BLANKING (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_CS_BLANKING                      MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_CS_BLANKING_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Pos                    (28UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_MEAS_TIME (Bit 28) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Msk                    (0x70000000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_MEAS_TIME (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME                        MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_0                      (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_1                      (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_2                      (0x4U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_MEAS_TIME_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_Pos                     (26UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_EQU_CTRL (Bit 26) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_Msk                     (0xc000000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: AS_EQU_CTRL (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL                         MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_0                       (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_1                       (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_AS_EQU_CTRL_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Pos                         (16UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: SQI_THR (Bit 16) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Msk                         (0x70000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: SQI_THR (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR                             MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_0                           (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_1                           (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_2                           (0x4U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_EN_Pos                          (15UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: SQI_EN (Bit 15) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_EN_Msk                          (0x8000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: SQI_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_EN                              MR_SUBG_GLOB_STATIC_AS_QI_CTRL_SQI_EN_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_Pos                         (13UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: CS_MODE (Bit 13) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_Msk                         (0x6000UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: CS_MODE (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE                             MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_0                           (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_1                           (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_CS_MODE_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Pos                         (9UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: PQI_THR (Bit 9) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Msk                         (0x1e00UL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: PQI_THR (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR                             MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_0                           (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_1                           (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_2                           (0x4U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_3                           (0x8U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_PQI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos                        (0UL)		/*!<MR_SUBG_GLOB_STATIC AS_QI_CTRL: RSSI_THR (Bit 0) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Msk                        (0x1ffUL)		/*!< MR_SUBG_GLOB_STATIC AS_QI_CTRL: RSSI_THR (Bitfield-Mask: 0x1ff) */
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR                            MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Msk
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_0                          (0x1U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_1                          (0x2U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_2                          (0x4U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_3                          (0x8U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_4                          (0x10U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_5                          (0x20U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_6                          (0x40U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_7                          (0x80U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)
#define MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_8                          (0x100U << MR_SUBG_GLOB_STATIC_AS_QI_CTRL_RSSI_THR_Pos)

/* =====================================================    IQC_CONFIG    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_ENABLE_Pos                      (31UL)		/*!<MR_SUBG_GLOB_STATIC IQC_CONFIG: IQC_ENABLE (Bit 31) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_ENABLE_Msk                      (0x80000000UL)		/*!< MR_SUBG_GLOB_STATIC IQC_CONFIG: IQC_ENABLE (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_ENABLE                          MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_ENABLE_Msk
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_REUSE_CORRECTION_Pos                (30UL)		/*!<MR_SUBG_GLOB_STATIC IQC_CONFIG: REUSE_CORRECTION (Bit 30) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_REUSE_CORRECTION_Msk                (0x40000000UL)		/*!< MR_SUBG_GLOB_STATIC IQC_CONFIG: REUSE_CORRECTION (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_REUSE_CORRECTION                    MR_SUBG_GLOB_STATIC_IQC_CONFIG_REUSE_CORRECTION_Msk
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_LOAD_IQC_INIT_Pos                   (29UL)		/*!<MR_SUBG_GLOB_STATIC IQC_CONFIG: LOAD_IQC_INIT (Bit 29) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_LOAD_IQC_INIT_Msk                   (0x20000000UL)		/*!< MR_SUBG_GLOB_STATIC IQC_CONFIG: LOAD_IQC_INIT (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_LOAD_IQC_INIT                       MR_SUBG_GLOB_STATIC_IQC_CONFIG_LOAD_IQC_INIT_Msk
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos                  (0UL)		/*!<MR_SUBG_GLOB_STATIC IQC_CONFIG: IQC_CORRECT_IN (Bit 0) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Msk                  (0xffffffUL)		/*!< MR_SUBG_GLOB_STATIC IQC_CONFIG: IQC_CORRECT_IN (Bitfield-Mask: 0xffffff) */
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN                      MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Msk
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_0                    (0x1U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_1                    (0x2U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_2                    (0x4U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_3                    (0x8U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_4                    (0x10U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_5                    (0x20U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_6                    (0x40U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_7                    (0x80U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_8                    (0x100U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_9                    (0x200U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_10                   (0x400U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_11                   (0x800U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_12                   (0x1000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_13                   (0x2000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_14                   (0x4000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_15                   (0x8000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_16                   (0x10000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_17                   (0x20000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_18                   (0x40000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_19                   (0x80000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_20                   (0x100000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_21                   (0x200000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_22                   (0x400000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)
#define MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_23                   (0x800000U << MR_SUBG_GLOB_STATIC_IQC_CONFIG_IQC_CORRECT_IN_Pos)

/* =====================================================    DSSS_CTRL    =====================================================*/ 
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos                          (10UL)		/*!<MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_THR (Bit 10) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Msk                          (0xfc00UL)		/*!< MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_THR (Bitfield-Mask: 0x3f) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR                              MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Msk
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_0                            (0x1U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_1                            (0x2U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_2                            (0x4U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_3                            (0x8U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_4                            (0x10U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_5                            (0x20U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_THR_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_Pos                         (8UL)		/*!<MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_HITS (Bit 8) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_Msk                         (0x300UL)		/*!< MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_HITS (Bitfield-Mask: 0x03) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS                             MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_Msk
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_0                           (0x1U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_1                           (0x2U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_HITS_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_DSSS_EN_Pos                          (7UL)		/*!<MR_SUBG_GLOB_STATIC DSSS_CTRL: DSSS_EN (Bit 7) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_DSSS_EN_Msk                          (0x80UL)		/*!< MR_SUBG_GLOB_STATIC DSSS_CTRL: DSSS_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_DSSS_EN                              MR_SUBG_GLOB_STATIC_DSSS_CTRL_DSSS_EN_Msk
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Pos                    (4UL)		/*!<MR_SUBG_GLOB_STATIC DSSS_CTRL: SPREADING_EXP (Bit 4) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Msk                    (0x70UL)		/*!< MR_SUBG_GLOB_STATIC DSSS_CTRL: SPREADING_EXP (Bitfield-Mask: 0x07) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP                        MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Msk
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_0                      (0x1U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_1                      (0x2U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_2                      (0x4U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_SPREADING_EXP_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Pos                       (0UL)		/*!<MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_WINDOW (Bit 0) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Msk                       (0xfUL)		/*!< MR_SUBG_GLOB_STATIC DSSS_CTRL: ACQ_WINDOW (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW                           MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Msk
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_0                         (0x1U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_1                         (0x2U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_2                         (0x4U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Pos)
#define MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_3                         (0x8U << MR_SUBG_GLOB_STATIC_DSSS_CTRL_ACQ_WINDOW_Pos)


/* ============================================================================================================================*/ 
/*=====================                                MR_SUBG_GLOB_STATUS                                =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    RFSEQ_IRQ_STATUS    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AGC_CALIB_DONE_F_Pos          (31UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: AGC_CALIB_DONE_F (Bit 31) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AGC_CALIB_DONE_F_Msk          (0x80000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: AGC_CALIB_DONE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AGC_CALIB_DONE_F              MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AGC_CALIB_DONE_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SAFEASK_CALIB_DONE_F_Pos      (30UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SAFEASK_CALIB_DONE_F (Bit 30) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SAFEASK_CALIB_DONE_F_Msk      (0x40000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SAFEASK_CALIB_DONE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SAFEASK_CALIB_DONE_F          MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SAFEASK_CALIB_DONE_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_END_F_Pos             (28UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RRM_CMD_END_F (Bit 28) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_END_F_Msk             (0x10000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RRM_CMD_END_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_END_F                 MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_END_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_START_F_Pos           (27UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RRM_CMD_START_F (Bit 27) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_START_F_Msk           (0x8000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RRM_CMD_START_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_START_F               MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RRM_CMD_START_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SEQ_F_Pos                     (26UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SEQ_F (Bit 26) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SEQ_F_Msk                     (0x4000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SEQ_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SEQ_F                         MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SEQ_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_HW_ANA_FAILURE_F_Pos          (24UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: HW_ANA_FAILURE_F (Bit 24) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_HW_ANA_FAILURE_F_Msk          (0x1000000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: HW_ANA_FAILURE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_HW_ANA_FAILURE_F              MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_HW_ANA_FAILURE_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AHB_ACCESS_ERROR_F_Pos        (22UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: AHB_ACCESS_ERROR_F (Bit 22) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AHB_ACCESS_ERROR_F_Msk        (0x400000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: AHB_ACCESS_ERROR_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AHB_ACCESS_ERROR_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_AHB_ACCESS_ERROR_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_1_F_Pos       (21UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_ALMOST_EMPTY_1_F (Bit 21) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_1_F_Msk       (0x200000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_ALMOST_EMPTY_1_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_1_F           MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_1_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_0_F_Pos       (20UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_ALMOST_EMPTY_0_F (Bit 20) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_0_F_Msk       (0x100000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_ALMOST_EMPTY_0_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_0_F           MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_ALMOST_EMPTY_0_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_1_F_Pos        (19UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_ALMOST_FULL_1_F (Bit 19) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_1_F_Msk        (0x80000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_ALMOST_FULL_1_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_1_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_1_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_0_F_Pos        (18UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_ALMOST_FULL_0_F (Bit 18) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_0_F_Msk        (0x40000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_ALMOST_FULL_0_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_0_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_ALMOST_FULL_0_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER1_USED_F_Pos        (17UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: DATABUFFER1_USED_F (Bit 17) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER1_USED_F_Msk        (0x20000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: DATABUFFER1_USED_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER1_USED_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER1_USED_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER0_USED_F_Pos        (16UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: DATABUFFER0_USED_F (Bit 16) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER0_USED_F_Msk        (0x10000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: DATABUFFER0_USED_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER0_USED_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_DATABUFFER0_USED_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SYNC_VALID_F_Pos              (14UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SYNC_VALID_F (Bit 14) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SYNC_VALID_F_Msk              (0x4000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SYNC_VALID_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SYNC_VALID_F                  MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SYNC_VALID_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_PREAMBLE_VALID_F_Pos          (13UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: PREAMBLE_VALID_F (Bit 13) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_PREAMBLE_VALID_F_Msk          (0x2000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: PREAMBLE_VALID_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_PREAMBLE_VALID_F              MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_PREAMBLE_VALID_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_CS_F_Pos                      (12UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: CS_F (Bit 12) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_CS_F_Msk                      (0x1000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: CS_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_CS_F                          MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_CS_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_COMMAND_REJECTED_F_Pos        (9UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: COMMAND_REJECTED_F (Bit 9) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_COMMAND_REJECTED_F_Msk        (0x200UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: COMMAND_REJECTED_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_COMMAND_REJECTED_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_COMMAND_REJECTED_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SABORT_DONE_F_Pos             (8UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SABORT_DONE_F (Bit 8) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SABORT_DONE_F_Msk             (0x100UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: SABORT_DONE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SABORT_DONE_F                 MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_SABORT_DONE_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RXTIMER_STOP_CDT_F_Pos        (7UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RXTIMER_STOP_CDT_F (Bit 7) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RXTIMER_STOP_CDT_F_Msk        (0x80UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RXTIMER_STOP_CDT_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RXTIMER_STOP_CDT_F            MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RXTIMER_STOP_CDT_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_FAST_RX_TERM_F_Pos            (4UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: FAST_RX_TERM_F (Bit 4) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_FAST_RX_TERM_F_Msk            (0x10UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: FAST_RX_TERM_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_FAST_RX_TERM_F                MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_FAST_RX_TERM_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_CRC_ERROR_F_Pos            (3UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_CRC_ERROR_F (Bit 3) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_CRC_ERROR_F_Msk            (0x8UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_CRC_ERROR_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_CRC_ERROR_F                MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_CRC_ERROR_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_TIMEOUT_F_Pos              (2UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_TIMEOUT_F (Bit 2) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_TIMEOUT_F_Msk              (0x4UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_TIMEOUT_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_TIMEOUT_F                  MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_TIMEOUT_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_OK_F_Pos                   (1UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_OK_F (Bit 1) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_OK_F_Msk                   (0x2UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: RX_OK_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_OK_F                       MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_RX_OK_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_DONE_F_Pos                 (0UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_DONE_F (Bit 0) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_DONE_F_Msk                 (0x1UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_IRQ_STATUS: TX_DONE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_DONE_F                     MR_SUBG_GLOB_STATUS_RFSEQ_IRQ_STATUS_TX_DONE_F_Msk

/* =====================================================    RFSEQ_STATUS_DETAIL    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_COMPLETE_F_Pos         (15UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: SEQ_COMPLETE_F (Bit 15) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_COMPLETE_F_Msk         (0x8000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: SEQ_COMPLETE_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_COMPLETE_F             MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_COMPLETE_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_ACTIONTIMEOUT_F_Pos    (14UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: SEQ_ACTIONTIMEOUT_F (Bit 14) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_ACTIONTIMEOUT_F_Msk    (0x4000UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: SEQ_ACTIONTIMEOUT_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_ACTIONTIMEOUT_F        MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_SEQ_ACTIONTIMEOUT_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALAMP_ERROR_F_Pos     (11UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_CALAMP_ERROR_F (Bit 11) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALAMP_ERROR_F_Msk     (0x800UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_CALAMP_ERROR_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALAMP_ERROR_F         MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALAMP_ERROR_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALFREQ_ERROR_F_Pos    (10UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_CALFREQ_ERROR_F (Bit 10) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALFREQ_ERROR_F_Msk    (0x400UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_CALFREQ_ERROR_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALFREQ_ERROR_F        MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_CALFREQ_ERROR_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_UNLOCK_F_Pos           (9UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_UNLOCK_F (Bit 9) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_UNLOCK_F_Msk           (0x200UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_UNLOCK_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_UNLOCK_F               MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_UNLOCK_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_LOCK_FAIL_F_Pos        (8UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_LOCK_FAIL_F (Bit 8) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_LOCK_FAIL_F_Msk        (0x100UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: PLL_LOCK_FAIL_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_LOCK_FAIL_F            MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_PLL_LOCK_FAIL_F_Msk
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_DBM_FIFO_ERROR_F_Pos       (5UL)		/*!<MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: DBM_FIFO_ERROR_F (Bit 5) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_DBM_FIFO_ERROR_F_Msk       (0x20UL)		/*!< MR_SUBG_GLOB_STATUS RFSEQ_STATUS_DETAIL: DBM_FIFO_ERROR_F (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_DBM_FIFO_ERROR_F           MR_SUBG_GLOB_STATUS_RFSEQ_STATUS_DETAIL_DBM_FIFO_ERROR_F_Msk

/* =====================================================    RADIO_FSM_INFO    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos             (0UL)		/*!<MR_SUBG_GLOB_STATUS RADIO_FSM_INFO: RADIO_FSM_STATE (Bit 0) */
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Msk             (0x1fUL)		/*!< MR_SUBG_GLOB_STATUS RADIO_FSM_INFO: RADIO_FSM_STATE (Bitfield-Mask: 0x1f) */
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE                 MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Msk
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_0               (0x1U << MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_1               (0x2U << MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_2               (0x4U << MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_3               (0x8U << MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_4               (0x10U << MR_SUBG_GLOB_STATUS_RADIO_FSM_INFO_RADIO_FSM_STATE_Pos)

/* =====================================================    RX_INDICATOR    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_ANT_SELECT_Pos                    (31UL)		/*!<MR_SUBG_GLOB_STATUS RX_INDICATOR: ANT_SELECT (Bit 31) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_ANT_SELECT_Msk                    (0x80000000UL)		/*!< MR_SUBG_GLOB_STATUS RX_INDICATOR: ANT_SELECT (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_ANT_SELECT                        MR_SUBG_GLOB_STATUS_RX_INDICATOR_ANT_SELECT_Msk
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Pos                      (24UL)		/*!<MR_SUBG_GLOB_STATUS RX_INDICATOR: AGC_WORD (Bit 24) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Msk                      (0xf000000UL)		/*!< MR_SUBG_GLOB_STATUS RX_INDICATOR: AGC_WORD (Bitfield-Mask: 0x0f) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD                          MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Msk
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_0                        (0x1U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_1                        (0x2U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_2                        (0x4U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_3                        (0x8U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_AGC_WORD_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos                (12UL)		/*!<MR_SUBG_GLOB_STATUS RX_INDICATOR: RSSI_LEVEL_RUN (Bit 12) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Msk                (0x1ff000UL)		/*!< MR_SUBG_GLOB_STATUS RX_INDICATOR: RSSI_LEVEL_RUN (Bitfield-Mask: 0x1ff) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN                    MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Msk
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_0                  (0x1U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_1                  (0x2U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_2                  (0x4U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_3                  (0x8U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_4                  (0x10U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_5                  (0x20U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_6                  (0x40U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_7                  (0x80U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_8                  (0x100U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_RUN_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos            (0UL)		/*!<MR_SUBG_GLOB_STATUS RX_INDICATOR: RSSI_LEVEL_ON_SYNC (Bit 0) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Msk            (0x1ffUL)		/*!< MR_SUBG_GLOB_STATUS RX_INDICATOR: RSSI_LEVEL_ON_SYNC (Bitfield-Mask: 0x1ff) */
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC                MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Msk
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_0              (0x1U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_1              (0x2U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_2              (0x4U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_3              (0x8U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_4              (0x10U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_5              (0x20U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_6              (0x40U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_7              (0x80U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_8              (0x100U << MR_SUBG_GLOB_STATUS_RX_INDICATOR_RSSI_LEVEL_ON_SYNC_Pos)

/* =====================================================    RX_INFO_REG    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos                 (0UL)		/*!<MR_SUBG_GLOB_STATUS RX_INFO_REG: RX_PCKTLEN_OUT (Bit 0) */
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Msk                 (0xffffUL)		/*!< MR_SUBG_GLOB_STATUS RX_INFO_REG: RX_PCKTLEN_OUT (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT                     MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Msk
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_0                   (0x1U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_1                   (0x2U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_2                   (0x4U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_3                   (0x8U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_4                   (0x10U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_5                   (0x20U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_6                   (0x40U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_7                   (0x80U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_8                   (0x100U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_9                   (0x200U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_10                  (0x400U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_11                  (0x800U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_12                  (0x1000U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_13                  (0x2000U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_14                  (0x4000U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_15                  (0x8000U << MR_SUBG_GLOB_STATUS_RX_INFO_REG_RX_PCKTLEN_OUT_Pos)

/* =====================================================    RX_CRC_REG    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos                      (0UL)		/*!<MR_SUBG_GLOB_STATUS RX_CRC_REG: RX_CRC_OUT (Bit 0) */
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Msk                      (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATUS RX_CRC_REG: RX_CRC_OUT (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT                          MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Msk
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_0                        (0x1U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_1                        (0x2U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_2                        (0x4U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_3                        (0x8U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_4                        (0x10U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_5                        (0x20U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_6                        (0x40U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_7                        (0x80U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_8                        (0x100U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_9                        (0x200U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_10                       (0x400U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_11                       (0x800U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_12                       (0x1000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_13                       (0x2000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_14                       (0x4000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_15                       (0x8000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_16                       (0x10000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_17                       (0x20000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_18                       (0x40000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_19                       (0x80000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_20                       (0x100000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_21                       (0x200000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_22                       (0x400000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_23                       (0x800000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_24                       (0x1000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_25                       (0x2000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_26                       (0x4000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_27                       (0x8000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_28                       (0x10000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_29                       (0x20000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_30                       (0x40000000U << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_31                       (0x80000000UL << MR_SUBG_GLOB_STATUS_RX_CRC_REG_RX_CRC_OUT_Pos)

/* =====================================================    QI_INFO    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos                     (16UL)		/*!<MR_SUBG_GLOB_STATUS QI_INFO: AFC_CORRECTION (Bit 16) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Msk                     (0xff0000UL)		/*!< MR_SUBG_GLOB_STATUS QI_INFO: AFC_CORRECTION (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION                         MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Msk
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_0                       (0x1U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_1                       (0x2U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_2                       (0x4U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_3                       (0x8U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_4                       (0x10U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_5                       (0x20U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_6                       (0x40U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_7                       (0x80U << MR_SUBG_GLOB_STATUS_QI_INFO_AFC_CORRECTION_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_SEC_Pos                            (14UL)		/*!<MR_SUBG_GLOB_STATUS QI_INFO: SQI_SEC (Bit 14) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_SEC_Msk                            (0x4000UL)		/*!< MR_SUBG_GLOB_STATUS QI_INFO: SQI_SEC (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_SEC                                MR_SUBG_GLOB_STATUS_QI_INFO_SQI_SEC_Msk
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos                           (8UL)		/*!<MR_SUBG_GLOB_STATUS QI_INFO: SQI_INFO (Bit 8) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Msk                           (0x3f00UL)		/*!< MR_SUBG_GLOB_STATUS QI_INFO: SQI_INFO (Bitfield-Mask: 0x3f) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO                               MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Msk
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_0                             (0x1U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_1                             (0x2U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_2                             (0x4U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_3                             (0x8U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_4                             (0x10U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_5                             (0x20U << MR_SUBG_GLOB_STATUS_QI_INFO_SQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos                           (0UL)		/*!<MR_SUBG_GLOB_STATUS QI_INFO: PQI_INFO (Bit 0) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Msk                           (0xffUL)		/*!< MR_SUBG_GLOB_STATUS QI_INFO: PQI_INFO (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO                               MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Msk
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_0                             (0x1U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_1                             (0x2U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_2                             (0x4U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_3                             (0x8U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_4                             (0x10U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_5                             (0x20U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_6                             (0x40U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)
#define MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_7                             (0x80U << MR_SUBG_GLOB_STATUS_QI_INFO_PQI_INFO_Pos)

/* =====================================================    DATABUFFER_INFO    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_Pos         (31UL)		/*!<MR_SUBG_GLOB_STATUS DATABUFFER_INFO: CURRENT_DATABUFFER (Bit 31) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_Msk         (0x80000000UL)		/*!< MR_SUBG_GLOB_STATUS DATABUFFER_INFO: CURRENT_DATABUFFER (Bitfield-Mask: 0x01) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER             MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_Msk
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos         (16UL)		/*!<MR_SUBG_GLOB_STATUS DATABUFFER_INFO: NB_DATABUFFER_USED (Bit 16) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Msk         (0x7fff0000UL)		/*!< MR_SUBG_GLOB_STATUS DATABUFFER_INFO: NB_DATABUFFER_USED (Bitfield-Mask: 0x7fff) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED             MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Msk
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_0           (0x1U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_1           (0x2U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_2           (0x4U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_3           (0x8U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_4           (0x10U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_5           (0x20U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_6           (0x40U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_7           (0x80U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_8           (0x100U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_9           (0x200U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_10          (0x400U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_11          (0x800U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_12          (0x1000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_13          (0x2000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_14          (0x4000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_NB_DATABUFFER_USED_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos   (0UL)		/*!<MR_SUBG_GLOB_STATUS DATABUFFER_INFO: CURRENT_DATABUFFER_COUNT (Bit 0) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Msk   (0xffffUL)		/*!< MR_SUBG_GLOB_STATUS DATABUFFER_INFO: CURRENT_DATABUFFER_COUNT (Bitfield-Mask: 0xffff) */
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT       MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Msk
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_0     (0x1U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_1     (0x2U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_2     (0x4U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_3     (0x8U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_4     (0x10U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_5     (0x20U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_6     (0x40U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_7     (0x80U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_8     (0x100U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_9     (0x200U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_10    (0x400U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_11    (0x800U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_12    (0x1000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_13    (0x2000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_14    (0x4000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)
#define MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_15    (0x8000U << MR_SUBG_GLOB_STATUS_DATABUFFER_INFO_CURRENT_DATABUFFER_COUNT_Pos)

/* =====================================================    TIME_CAPTURE    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos                  (0UL)		/*!<MR_SUBG_GLOB_STATUS TIME_CAPTURE: TIME_CAPTURE (Bit 0) */
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Msk                  (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATUS TIME_CAPTURE: TIME_CAPTURE (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE                      MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Msk
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_0                    (0x1U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_1                    (0x2U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_2                    (0x4U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_3                    (0x8U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_4                    (0x10U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_5                    (0x20U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_6                    (0x40U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_7                    (0x80U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_8                    (0x100U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_9                    (0x200U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_10                   (0x400U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_11                   (0x800U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_12                   (0x1000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_13                   (0x2000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_14                   (0x4000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_15                   (0x8000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_16                   (0x10000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_17                   (0x20000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_18                   (0x40000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_19                   (0x80000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_20                   (0x100000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_21                   (0x200000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_22                   (0x400000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_23                   (0x800000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_24                   (0x1000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_25                   (0x2000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_26                   (0x4000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_27                   (0x8000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_28                   (0x10000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_29                   (0x20000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_30                   (0x40000000U << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)
#define MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_31                   (0x80000000UL << MR_SUBG_GLOB_STATUS_TIME_CAPTURE_TIME_CAPTURE_Pos)

/* =====================================================    IQC_CORRECTION_OUT    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos         (0UL)		/*!<MR_SUBG_GLOB_STATUS IQC_CORRECTION_OUT: IQC_CORRECT_OUT (Bit 0) */
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Msk         (0xffffffUL)		/*!< MR_SUBG_GLOB_STATUS IQC_CORRECTION_OUT: IQC_CORRECT_OUT (Bitfield-Mask: 0xffffff) */
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT             MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Msk
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_0           (0x1U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_1           (0x2U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_2           (0x4U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_3           (0x8U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_4           (0x10U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_5           (0x20U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_6           (0x40U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_7           (0x80U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_8           (0x100U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_9           (0x200U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_10          (0x400U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_11          (0x800U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_12          (0x1000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_13          (0x2000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_14          (0x4000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_15          (0x8000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_16          (0x10000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_17          (0x20000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_18          (0x40000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_19          (0x80000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_20          (0x100000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_21          (0x200000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_22          (0x400000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_23          (0x800000U << MR_SUBG_GLOB_STATUS_IQC_CORRECTION_OUT_IQC_CORRECT_OUT_Pos)

/* =====================================================    PA_SAFEASK_OUT    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos                  (0UL)		/*!<MR_SUBG_GLOB_STATUS PA_SAFEASK_OUT: PA_CODEMAX (Bit 0) */
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Msk                  (0xffUL)		/*!< MR_SUBG_GLOB_STATUS PA_SAFEASK_OUT: PA_CODEMAX (Bitfield-Mask: 0xff) */
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX                      MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Msk
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_0                    (0x1U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_1                    (0x2U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_2                    (0x4U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_3                    (0x8U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_4                    (0x10U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_5                    (0x20U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_6                    (0x40U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)
#define MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_7                    (0x80U << MR_SUBG_GLOB_STATUS_PA_SAFEASK_OUT_PA_CODEMAX_Pos)

/* =====================================================    VCO_CALIB_OUT    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos               (8UL)		/*!<MR_SUBG_GLOB_STATUS VCO_CALIB_OUT: VCO_CALAMP_OUT (Bit 8) */
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Msk               (0x3fff00UL)		/*!< MR_SUBG_GLOB_STATUS VCO_CALIB_OUT: VCO_CALAMP_OUT (Bitfield-Mask: 0x3fff) */
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT                   MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Msk
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_0                 (0x1U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_1                 (0x2U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_2                 (0x4U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_3                 (0x8U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_4                 (0x10U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_5                 (0x20U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_6                 (0x40U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_7                 (0x80U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_8                 (0x100U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_9                 (0x200U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_10                (0x400U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_11                (0x800U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_12                (0x1000U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_13                (0x2000U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALAMP_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos              (0UL)		/*!<MR_SUBG_GLOB_STATUS VCO_CALIB_OUT: VCO_CALFREQ_OUT (Bit 0) */
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Msk              (0x7fUL)		/*!< MR_SUBG_GLOB_STATUS VCO_CALIB_OUT: VCO_CALFREQ_OUT (Bitfield-Mask: 0x7f) */
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT                  MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Msk
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_0                (0x1U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_1                (0x2U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_2                (0x4U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_3                (0x8U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_4                (0x10U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_5                (0x20U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)
#define MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_6                (0x40U << MR_SUBG_GLOB_STATUS_VCO_CALIB_OUT_VCO_CALFREQ_OUT_Pos)

/* =====================================================    SEQ_INFO    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos                     (0UL)		/*!<MR_SUBG_GLOB_STATUS SEQ_INFO: SEQ_FSM_STATE (Bit 0) */
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Msk                     (0x1fUL)		/*!< MR_SUBG_GLOB_STATUS SEQ_INFO: SEQ_FSM_STATE (Bitfield-Mask: 0x1f) */
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE                         MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Msk
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_0                       (0x1U << MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_1                       (0x2U << MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_2                       (0x4U << MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_3                       (0x8U << MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_4                       (0x10U << MR_SUBG_GLOB_STATUS_SEQ_INFO_SEQ_FSM_STATE_Pos)

/* =====================================================    SEQ_EVENT_STATUS    =====================================================*/ 
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos          (0UL)		/*!<MR_SUBG_GLOB_STATUS SEQ_EVENT_STATUS: SEQ_EVENT_STATUS (Bit 0) */
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Msk          (0xffffffffUL)		/*!< MR_SUBG_GLOB_STATUS SEQ_EVENT_STATUS: SEQ_EVENT_STATUS (Bitfield-Mask: 0xffffffff) */
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS              MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Msk
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_0            (0x1U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_1            (0x2U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_2            (0x4U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_3            (0x8U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_4            (0x10U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_5            (0x20U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_6            (0x40U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_7            (0x80U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_8            (0x100U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_9            (0x200U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_10           (0x400U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_11           (0x800U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_12           (0x1000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_13           (0x2000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_14           (0x4000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_15           (0x8000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_16           (0x10000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_17           (0x20000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_18           (0x40000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_19           (0x80000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_20           (0x100000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_21           (0x200000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_22           (0x400000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_23           (0x800000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_24           (0x1000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_25           (0x2000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_26           (0x4000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_27           (0x8000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_28           (0x10000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_29           (0x20000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_30           (0x40000000U << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)
#define MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_31           (0x80000000UL << MR_SUBG_GLOB_STATUS_SEQ_EVENT_STATUS_SEQ_EVENT_STATUS_Pos)


/* ============================================================================================================================*/ 
/*=====================                                   MR_SUBG_RADIO                                   =====================*/ 
/* ============================================================================================================================*/ 

/* =====================================================    RF_FSM0_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos                  (0UL)		/*!<MR_SUBG_RADIO RF_FSM0_TIMEOUT: ENA_RFREG_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Msk                  (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM0_TIMEOUT: ENA_RFREG_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER                      MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_0                    (0x1U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_1                    (0x2U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_2                    (0x4U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_3                    (0x8U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_4                    (0x10U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_5                    (0x20U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_6                    (0x40U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_7                    (0x80U << MR_SUBG_RADIO_RF_FSM0_TIMEOUT_ENA_RFREG_TIMER_Pos)

/* =====================================================    RF_FSM1_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos                (0UL)		/*!<MR_SUBG_RADIO RF_FSM1_TIMEOUT: SYNTH_SETUP_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Msk                (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM1_TIMEOUT: SYNTH_SETUP_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER                    MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_0                  (0x1U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_1                  (0x2U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_2                  (0x4U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_3                  (0x8U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_4                  (0x10U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_5                  (0x20U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_6                  (0x40U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_7                  (0x80U << MR_SUBG_RADIO_RF_FSM1_TIMEOUT_SYNTH_SETUP_TIMER_Pos)

/* =====================================================    RF_FSM2_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos             (0UL)		/*!<MR_SUBG_RADIO RF_FSM2_TIMEOUT: VCO_CALIB_LOCK_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Msk             (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM2_TIMEOUT: VCO_CALIB_LOCK_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER                 MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_0               (0x1U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_1               (0x2U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_2               (0x4U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_3               (0x8U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_4               (0x10U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_5               (0x20U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_6               (0x40U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_7               (0x80U << MR_SUBG_RADIO_RF_FSM2_TIMEOUT_VCO_CALIB_LOCK_TIMER_Pos)

/* =====================================================    RF_FSM3_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos                   (0UL)		/*!<MR_SUBG_RADIO RF_FSM3_TIMEOUT: VCO_LOCK_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Msk                   (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM3_TIMEOUT: VCO_LOCK_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER                       MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_0                     (0x1U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_1                     (0x2U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_2                     (0x4U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_3                     (0x8U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_4                     (0x10U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_5                     (0x20U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_6                     (0x40U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_7                     (0x80U << MR_SUBG_RADIO_RF_FSM3_TIMEOUT_VCO_LOCK_TIMER_Pos)

/* =====================================================    RF_FSM4_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos                      (0UL)		/*!<MR_SUBG_RADIO RF_FSM4_TIMEOUT: EN_RX_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Msk                      (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM4_TIMEOUT: EN_RX_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER                          MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_0                        (0x1U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_1                        (0x2U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_2                        (0x4U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_3                        (0x8U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_4                        (0x10U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_5                        (0x20U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_6                        (0x40U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_7                        (0x80U << MR_SUBG_RADIO_RF_FSM4_TIMEOUT_EN_RX_TIMER_Pos)

/* =====================================================    RF_FSM5_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos                      (0UL)		/*!<MR_SUBG_RADIO RF_FSM5_TIMEOUT: EN_PA_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Msk                      (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM5_TIMEOUT: EN_PA_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER                          MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_0                        (0x1U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_1                        (0x2U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_2                        (0x4U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_3                        (0x8U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_4                        (0x10U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_5                        (0x20U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_6                        (0x40U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_7                        (0x80U << MR_SUBG_RADIO_RF_FSM5_TIMEOUT_EN_PA_TIMER_Pos)

/* =====================================================    RF_FSM6_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos                 (0UL)		/*!<MR_SUBG_RADIO RF_FSM6_TIMEOUT: PA_DWN_ANA_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Msk                 (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM6_TIMEOUT: PA_DWN_ANA_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER                     MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_0                   (0x1U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_1                   (0x2U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_2                   (0x4U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_3                   (0x8U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_4                   (0x10U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_5                   (0x20U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_6                   (0x40U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_7                   (0x80U << MR_SUBG_RADIO_RF_FSM6_TIMEOUT_PA_DWN_ANA_TIMER_Pos)

/* =====================================================    RF_FSM7_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos                     (0UL)		/*!<MR_SUBG_RADIO RF_FSM7_TIMEOUT: EN_LNA_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Msk                     (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM7_TIMEOUT: EN_LNA_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER                         MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_0                       (0x1U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_1                       (0x2U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_2                       (0x4U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_3                       (0x8U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_4                       (0x10U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_5                       (0x20U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_6                       (0x40U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_7                       (0x80U << MR_SUBG_RADIO_RF_FSM7_TIMEOUT_EN_LNA_TIMER_Pos)

/* =====================================================    AFC0_CONFIG    =====================================================*/ 
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Pos                   (4UL)		/*!<MR_SUBG_RADIO AFC0_CONFIG: AFC_FAST_GAIN_LOG2 (Bit 4) */
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Msk                   (0xf0UL)		/*!< MR_SUBG_RADIO AFC0_CONFIG: AFC_FAST_GAIN_LOG2 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2                       MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Msk
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_0                     (0x1U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_1                     (0x2U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_2                     (0x4U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_3                     (0x8U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_FAST_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Pos                   (0UL)		/*!<MR_SUBG_RADIO AFC0_CONFIG: AFC_SLOW_GAIN_LOG2 (Bit 0) */
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Msk                   (0xfUL)		/*!< MR_SUBG_RADIO AFC0_CONFIG: AFC_SLOW_GAIN_LOG2 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2                       MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Msk
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_0                     (0x1U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_1                     (0x2U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_2                     (0x4U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Pos)
#define MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_3                     (0x8U << MR_SUBG_RADIO_AFC0_CONFIG_AFC_SLOW_GAIN_LOG2_Pos)

/* =====================================================    AFC1_CONFIG    =====================================================*/ 
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos                      (0UL)		/*!<MR_SUBG_RADIO AFC1_CONFIG: AFC_FAST_PERIOD (Bit 0) */
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Msk                      (0xffUL)		/*!< MR_SUBG_RADIO AFC1_CONFIG: AFC_FAST_PERIOD (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD                          MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Msk
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_0                        (0x1U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_1                        (0x2U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_2                        (0x4U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_3                        (0x8U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_4                        (0x10U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_5                        (0x20U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_6                        (0x40U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)
#define MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_7                        (0x80U << MR_SUBG_RADIO_AFC1_CONFIG_AFC_FAST_PERIOD_Pos)

/* =====================================================    AFC2_CONFIG    =====================================================*/ 
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_FREEZE_ON_SYNC_Pos                   (7UL)		/*!<MR_SUBG_RADIO AFC2_CONFIG: AFC_FREEZE_ON_SYNC (Bit 7) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_FREEZE_ON_SYNC_Msk                   (0x80UL)		/*!< MR_SUBG_RADIO AFC2_CONFIG: AFC_FREEZE_ON_SYNC (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_FREEZE_ON_SYNC                       MR_SUBG_RADIO_AFC2_CONFIG_AFC_FREEZE_ON_SYNC_Msk
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_EN_Pos                               (6UL)		/*!<MR_SUBG_RADIO AFC2_CONFIG: AFC_EN (Bit 6) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_EN_Msk                               (0x40UL)		/*!< MR_SUBG_RADIO AFC2_CONFIG: AFC_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_EN                                   MR_SUBG_RADIO_AFC2_CONFIG_AFC_EN_Msk
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_MODE_Pos                             (5UL)		/*!<MR_SUBG_RADIO AFC2_CONFIG: AFC_MODE (Bit 5) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_MODE_Msk                             (0x20UL)		/*!< MR_SUBG_RADIO AFC2_CONFIG: AFC_MODE (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_MODE                                 MR_SUBG_RADIO_AFC2_CONFIG_AFC_MODE_Msk
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos                       (0UL)		/*!<MR_SUBG_RADIO AFC2_CONFIG: AFC_PD_LEAKAGE (Bit 0) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Msk                       (0x1fUL)		/*!< MR_SUBG_RADIO AFC2_CONFIG: AFC_PD_LEAKAGE (Bitfield-Mask: 0x1f) */
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE                           MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Msk
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_0                         (0x1U << MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos)
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_1                         (0x2U << MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos)
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_2                         (0x4U << MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos)
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_3                         (0x8U << MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos)
#define MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_4                         (0x10U << MR_SUBG_RADIO_AFC2_CONFIG_AFC_PD_LEAKAGE_Pos)

/* =====================================================    AFC3_CONFIG    =====================================================*/ 
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_Pos                    (6UL)		/*!<MR_SUBG_RADIO AFC3_CONFIG: AFC_REINIT_OPTION (Bit 6) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_Msk                    (0xc0UL)		/*!< MR_SUBG_RADIO AFC3_CONFIG: AFC_REINIT_OPTION (Bitfield-Mask: 0x03) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION                        MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_Msk
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_0                      (0x1U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_1                      (0x2U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_REINIT_OPTION_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Pos                     (2UL)		/*!<MR_SUBG_RADIO AFC3_CONFIG: AFC_TH_SIGN_PERM (Bit 2) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Msk                     (0x3cUL)		/*!< MR_SUBG_RADIO AFC3_CONFIG: AFC_TH_SIGN_PERM (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM                         MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Msk
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_0                       (0x1U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_1                       (0x2U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_2                       (0x4U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_3                       (0x8U << MR_SUBG_RADIO_AFC3_CONFIG_AFC_TH_SIGN_PERM_Pos)
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_SIGN_PERM_CHECK_Pos                  (1UL)		/*!<MR_SUBG_RADIO AFC3_CONFIG: AFC_SIGN_PERM_CHECK (Bit 1) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_SIGN_PERM_CHECK_Msk                  (0x2UL)		/*!< MR_SUBG_RADIO AFC3_CONFIG: AFC_SIGN_PERM_CHECK (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_SIGN_PERM_CHECK                      MR_SUBG_RADIO_AFC3_CONFIG_AFC_SIGN_PERM_CHECK_Msk
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_INIT_MODE_Pos                        (0UL)		/*!<MR_SUBG_RADIO AFC3_CONFIG: AFC_INIT_MODE (Bit 0) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_INIT_MODE_Msk                        (0x1UL)		/*!< MR_SUBG_RADIO AFC3_CONFIG: AFC_INIT_MODE (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AFC3_CONFIG_AFC_INIT_MODE                            MR_SUBG_RADIO_AFC3_CONFIG_AFC_INIT_MODE_Msk

/* =====================================================    CLKREC_CTRL0    =====================================================*/ 
#define MR_SUBG_RADIO_CLKREC_CTRL0_PSTFLT_LEN_Pos                          (7UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL0: PSTFLT_LEN (Bit 7) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_PSTFLT_LEN_Msk                          (0x80UL)		/*!< MR_SUBG_RADIO CLKREC_CTRL0: PSTFLT_LEN (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_PSTFLT_LEN                              MR_SUBG_RADIO_CLKREC_CTRL0_PSTFLT_LEN_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Pos                  (4UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL0: CLKREC_P_GAIN_FAST (Bit 4) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Msk                  (0x70UL)		/*!< MR_SUBG_RADIO CLKREC_CTRL0: CLKREC_P_GAIN_FAST (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST                      MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_0                    (0x1U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_1                    (0x2U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_2                    (0x4U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_P_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Pos                  (0UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL0: CLKREC_I_GAIN_FAST (Bit 0) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Msk                  (0xfUL)		/*!< MR_SUBG_RADIO CLKREC_CTRL0: CLKREC_I_GAIN_FAST (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST                      MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_0                    (0x1U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_1                    (0x2U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_2                    (0x4U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_3                    (0x8U << MR_SUBG_RADIO_CLKREC_CTRL0_CLKREC_I_GAIN_FAST_Pos)

/* =====================================================    CLKREC_CTRL1    =====================================================*/ 
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_ALGO_SEL_Pos                     (7UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_ALGO_SEL (Bit 7) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_ALGO_SEL_Msk                     (0x80UL)		/*!< MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_ALGO_SEL (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_ALGO_SEL                         MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_ALGO_SEL_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Pos                  (4UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_P_GAIN_SLOW (Bit 4) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Msk                  (0x70UL)		/*!< MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_P_GAIN_SLOW (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW                      MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_0                    (0x1U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_1                    (0x2U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_2                    (0x4U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_P_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Pos                  (0UL)		/*!<MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_I_GAIN_SLOW (Bit 0) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Msk                  (0xfUL)		/*!< MR_SUBG_RADIO CLKREC_CTRL1: CLKREC_I_GAIN_SLOW (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW                      MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Msk
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_0                    (0x1U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_1                    (0x2U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_2                    (0x4U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Pos)
#define MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_3                    (0x8U << MR_SUBG_RADIO_CLKREC_CTRL1_CLKREC_I_GAIN_SLOW_Pos)

/* =====================================================    DCREM_CTRL0    =====================================================*/ 
#define MR_SUBG_RADIO_DCREM_CTRL0_TRACK_GAIN_Pos                           (7UL)		/*!<MR_SUBG_RADIO DCREM_CTRL0: TRACK_GAIN (Bit 7) */
#define MR_SUBG_RADIO_DCREM_CTRL0_TRACK_GAIN_Msk                           (0x80UL)		/*!< MR_SUBG_RADIO DCREM_CTRL0: TRACK_GAIN (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_DCREM_CTRL0_TRACK_GAIN                               MR_SUBG_RADIO_DCREM_CTRL0_TRACK_GAIN_Msk
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos                           (0UL)		/*!<MR_SUBG_RADIO DCREM_CTRL0: START_GAIN (Bit 0) */
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Msk                           (0x1fUL)		/*!< MR_SUBG_RADIO DCREM_CTRL0: START_GAIN (Bitfield-Mask: 0x1f) */
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN                               MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Msk
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_0                             (0x1U << MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_1                             (0x2U << MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_2                             (0x4U << MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_3                             (0x8U << MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_4                             (0x10U << MR_SUBG_RADIO_DCREM_CTRL0_START_GAIN_Pos)

/* =====================================================    DCREM_CTRL1    =====================================================*/ 
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Pos                          (0UL)		/*!<MR_SUBG_RADIO DCREM_CTRL1: LN2TIME2SET (Bit 0) */
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Msk                          (0xfUL)		/*!< MR_SUBG_RADIO DCREM_CTRL1: LN2TIME2SET (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET                              MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Msk
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_0                            (0x1U << MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_1                            (0x2U << MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_2                            (0x4U << MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Pos)
#define MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_3                            (0x8U << MR_SUBG_RADIO_DCREM_CTRL1_LN2TIME2SET_Pos)

/* =====================================================    IQC_CTRL0    =====================================================*/ 
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Pos                              (4UL)		/*!<MR_SUBG_RADIO IQC_CTRL0: SLOW_GAIN (Bit 4) */
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Msk                              (0xf0UL)		/*!< MR_SUBG_RADIO IQC_CTRL0: SLOW_GAIN (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN                                  MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Msk
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_0                                (0x1U << MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_1                                (0x2U << MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_2                                (0x4U << MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_3                                (0x8U << MR_SUBG_RADIO_IQC_CTRL0_SLOW_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Pos                              (0UL)		/*!<MR_SUBG_RADIO IQC_CTRL0: FAST_GAIN (Bit 0) */
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Msk                              (0xfUL)		/*!< MR_SUBG_RADIO IQC_CTRL0: FAST_GAIN (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN                                  MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Msk
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_0                                (0x1U << MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_1                                (0x2U << MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_2                                (0x4U << MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Pos)
#define MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_3                                (0x8U << MR_SUBG_RADIO_IQC_CTRL0_FAST_GAIN_Pos)

/* =====================================================    IQC_CTRL1    =====================================================*/ 
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos                             (0UL)		/*!<MR_SUBG_RADIO IQC_CTRL1: QPD_ATTACK (Bit 0) */
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Msk                             (0xffUL)		/*!< MR_SUBG_RADIO IQC_CTRL1: QPD_ATTACK (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK                                 MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Msk
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_0                               (0x1U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_1                               (0x2U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_2                               (0x4U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_3                               (0x8U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_4                               (0x10U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_5                               (0x20U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_6                               (0x40U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)
#define MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_7                               (0x80U << MR_SUBG_RADIO_IQC_CTRL1_QPD_ATTACK_Pos)

/* =====================================================    IQC_CTRL2    =====================================================*/ 
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos                              (0UL)		/*!<MR_SUBG_RADIO IQC_CTRL2: QPD_DECAY (Bit 0) */
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Msk                              (0xffUL)		/*!< MR_SUBG_RADIO IQC_CTRL2: QPD_DECAY (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY                                  MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Msk
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_0                                (0x1U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_1                                (0x2U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_2                                (0x4U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_3                                (0x8U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_4                                (0x10U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_5                                (0x20U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_6                                (0x40U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)
#define MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_7                                (0x80U << MR_SUBG_RADIO_IQC_CTRL2_QPD_DECAY_Pos)

/* =====================================================    IQC_CTRL3    =====================================================*/ 
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Pos                              (0UL)		/*!<MR_SUBG_RADIO IQC_CTRL3: FAST_TIME (Bit 0) */
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Msk                              (0xfUL)		/*!< MR_SUBG_RADIO IQC_CTRL3: FAST_TIME (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME                                  MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Msk
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_0                                (0x1U << MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Pos)
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_1                                (0x2U << MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Pos)
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_2                                (0x4U << MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Pos)
#define MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_3                                (0x8U << MR_SUBG_RADIO_IQC_CTRL3_FAST_TIME_Pos)

/* =====================================================    AGC0_CTRL    =====================================================*/ 
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_EN_Pos                                 (7UL)		/*!<MR_SUBG_RADIO AGC0_CTRL: AGC_EN (Bit 7) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_EN_Msk                                 (0x80UL)		/*!< MR_SUBG_RADIO AGC0_CTRL: AGC_EN (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_EN                                     MR_SUBG_RADIO_AGC0_CTRL_AGC_EN_Msk
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_START_ONHOLD_Pos                       (6UL)		/*!<MR_SUBG_RADIO AGC0_CTRL: AGC_START_ONHOLD (Bit 6) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_START_ONHOLD_Msk                       (0x40UL)		/*!< MR_SUBG_RADIO AGC0_CTRL: AGC_START_ONHOLD (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_START_ONHOLD                           MR_SUBG_RADIO_AGC0_CTRL_AGC_START_ONHOLD_Msk
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos                          (0UL)		/*!<MR_SUBG_RADIO AGC0_CTRL: AGC_HOLD_TIME (Bit 0) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Msk                          (0x3fUL)		/*!< MR_SUBG_RADIO AGC0_CTRL: AGC_HOLD_TIME (Bitfield-Mask: 0x3f) */
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME                              MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Msk
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_0                            (0x1U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_1                            (0x2U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_2                            (0x4U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_3                            (0x8U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_4                            (0x10U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)
#define MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_5                            (0x20U << MR_SUBG_RADIO_AGC0_CTRL_AGC_HOLD_TIME_Pos)

/* =====================================================    AGC1_CTRL    =====================================================*/ 
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Pos                            (4UL)		/*!<MR_SUBG_RADIO AGC1_CTRL: AGC_MAX_THR (Bit 4) */
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Msk                            (0xf0UL)		/*!< MR_SUBG_RADIO AGC1_CTRL: AGC_MAX_THR (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR                                MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Msk
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_0                              (0x1U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_1                              (0x2U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_2                              (0x4U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_3                              (0x8U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MAX_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Pos                            (0UL)		/*!<MR_SUBG_RADIO AGC1_CTRL: AGC_MIN_THR (Bit 0) */
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Msk                            (0xfUL)		/*!< MR_SUBG_RADIO AGC1_CTRL: AGC_MIN_THR (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR                                MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Msk
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_0                              (0x1U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_1                              (0x2U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_2                              (0x4U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Pos)
#define MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_3                              (0x8U << MR_SUBG_RADIO_AGC1_CTRL_AGC_MIN_THR_Pos)

/* =====================================================    AGC2_CTRL    =====================================================*/ 
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_HIGH_ATTEN_MODE_Pos                    (7UL)		/*!<MR_SUBG_RADIO AGC2_CTRL: AGC_HIGH_ATTEN_MODE (Bit 7) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_HIGH_ATTEN_MODE_Msk                    (0x80UL)		/*!< MR_SUBG_RADIO AGC2_CTRL: AGC_HIGH_ATTEN_MODE (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_HIGH_ATTEN_MODE                        MR_SUBG_RADIO_AGC2_CTRL_AGC_HIGH_ATTEN_MODE_Msk
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_STEADY_Pos                   (6UL)		/*!<MR_SUBG_RADIO AGC2_CTRL: AGC_FREEZE_ON_STEADY (Bit 6) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_STEADY_Msk                   (0x40UL)		/*!< MR_SUBG_RADIO AGC2_CTRL: AGC_FREEZE_ON_STEADY (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_STEADY                       MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_STEADY_Msk
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_SYNC_Pos                     (5UL)		/*!<MR_SUBG_RADIO AGC2_CTRL: AGC_FREEZE_ON_SYNC (Bit 5) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_SYNC_Msk                     (0x20UL)		/*!< MR_SUBG_RADIO AGC2_CTRL: AGC_FREEZE_ON_SYNC (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_SYNC                         MR_SUBG_RADIO_AGC2_CTRL_AGC_FREEZE_ON_SYNC_Msk
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_START_MAX_ATTEN_Pos                    (4UL)		/*!<MR_SUBG_RADIO AGC2_CTRL: AGC_START_MAX_ATTEN (Bit 4) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_START_MAX_ATTEN_Msk                    (0x10UL)		/*!< MR_SUBG_RADIO AGC2_CTRL: AGC_START_MAX_ATTEN (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_START_MAX_ATTEN                        MR_SUBG_RADIO_AGC2_CTRL_AGC_START_MAX_ATTEN_Msk
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Pos                          (0UL)		/*!<MR_SUBG_RADIO AGC2_CTRL: AGC_MEAS_TIME (Bit 0) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Msk                          (0xfUL)		/*!< MR_SUBG_RADIO AGC2_CTRL: AGC_MEAS_TIME (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME                              MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Msk
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_0                            (0x1U << MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Pos)
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_1                            (0x2U << MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Pos)
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_2                            (0x4U << MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Pos)
#define MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_3                            (0x8U << MR_SUBG_RADIO_AGC2_CTRL_AGC_MEAS_TIME_Pos)

/* =====================================================    AGC3_CTRL    =====================================================*/ 
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Pos                          (4UL)		/*!<MR_SUBG_RADIO AGC3_CTRL: AGC_MAX_ATTEN (Bit 4) */
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Msk                          (0xf0UL)		/*!< MR_SUBG_RADIO AGC3_CTRL: AGC_MAX_ATTEN (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN                              MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Msk
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_0                            (0x1U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_1                            (0x2U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_2                            (0x4U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_3                            (0x8U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MAX_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Pos                          (0UL)		/*!<MR_SUBG_RADIO AGC3_CTRL: AGC_MIN_ATTEN (Bit 0) */
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Msk                          (0xfUL)		/*!< MR_SUBG_RADIO AGC3_CTRL: AGC_MIN_ATTEN (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN                              MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Msk
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_0                            (0x1U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_1                            (0x2U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_2                            (0x4U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Pos)
#define MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_3                            (0x8U << MR_SUBG_RADIO_AGC3_CTRL_AGC_MIN_ATTEN_Pos)

/* =====================================================    AGC4_CTRL    =====================================================*/ 
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Pos                         (0UL)		/*!<MR_SUBG_RADIO AGC4_CTRL: AGC_FREEZE_THR (Bit 0) */
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Msk                         (0xfUL)		/*!< MR_SUBG_RADIO AGC4_CTRL: AGC_FREEZE_THR (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR                             MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Msk
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_0                           (0x1U << MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Pos)
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_1                           (0x2U << MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Pos)
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_2                           (0x4U << MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Pos)
#define MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_3                           (0x8U << MR_SUBG_RADIO_AGC4_CTRL_AGC_FREEZE_THR_Pos)

/* =====================================================    AGC_ATTEN0    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN0: PGA_AGCGAIN_0 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN0: PGA_AGCGAIN_0 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0                             MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Msk
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN0_PGA_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN0: ATTEN_AGCGAIN_0 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN0: ATTEN_AGCGAIN_0 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0                           MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Msk
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN0_ATTEN_AGCGAIN_0_Pos)

/* =====================================================    AGC_ATTEN1    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN1: PGA_AGCGAIN_1 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN1: PGA_AGCGAIN_1 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1                             MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Msk
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN1_PGA_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN1: ATTEN_AGCGAIN_1 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN1: ATTEN_AGCGAIN_1 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1                           MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Msk
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN1_ATTEN_AGCGAIN_1_Pos)

/* =====================================================    AGC_ATTEN2    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN2: PGA_AGCGAIN_2 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN2: PGA_AGCGAIN_2 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2                             MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Msk
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN2_PGA_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN2: ATTEN_AGCGAIN_2 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN2: ATTEN_AGCGAIN_2 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2                           MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Msk
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN2_ATTEN_AGCGAIN_2_Pos)

/* =====================================================    AGC_ATTEN3    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN3: PGA_AGCGAIN_3 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN3: PGA_AGCGAIN_3 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3                             MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Msk
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN3_PGA_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN3: ATTEN_AGCGAIN_3 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN3: ATTEN_AGCGAIN_3 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3                           MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Msk
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN3_ATTEN_AGCGAIN_3_Pos)

/* =====================================================    AGC_ATTEN4    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN4: PGA_AGCGAIN_4 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN4: PGA_AGCGAIN_4 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4                             MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Msk
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN4_PGA_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN4: ATTEN_AGCGAIN_4 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN4: ATTEN_AGCGAIN_4 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4                           MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Msk
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN4_ATTEN_AGCGAIN_4_Pos)

/* =====================================================    AGC_ATTEN5    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN5: PGA_AGCGAIN_5 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN5: PGA_AGCGAIN_5 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5                             MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Msk
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN5_PGA_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN5: ATTEN_AGCGAIN_5 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN5: ATTEN_AGCGAIN_5 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5                           MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Msk
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN5_ATTEN_AGCGAIN_5_Pos)

/* =====================================================    AGC_ATTEN6    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN6: PGA_AGCGAIN_6 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN6: PGA_AGCGAIN_6 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6                             MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Msk
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN6_PGA_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN6: ATTEN_AGCGAIN_6 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN6: ATTEN_AGCGAIN_6 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6                           MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Msk
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN6_ATTEN_AGCGAIN_6_Pos)

/* =====================================================    AGC_ATTEN7    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN7: PGA_AGCGAIN_7 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN7: PGA_AGCGAIN_7 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7                             MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Msk
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN7_PGA_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN7: ATTEN_AGCGAIN_7 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN7: ATTEN_AGCGAIN_7 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7                           MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Msk
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN7_ATTEN_AGCGAIN_7_Pos)

/* =====================================================    AGC_ATTEN8    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN8: PGA_AGCGAIN_8 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN8: PGA_AGCGAIN_8 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8                             MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Msk
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN8_PGA_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN8: ATTEN_AGCGAIN_8 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN8: ATTEN_AGCGAIN_8 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8                           MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Msk
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN8_ATTEN_AGCGAIN_8_Pos)

/* =====================================================    AGC_ATTEN9    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Pos                         (4UL)		/*!<MR_SUBG_RADIO AGC_ATTEN9: PGA_AGCGAIN_9 (Bit 4) */
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Msk                         (0x70UL)		/*!< MR_SUBG_RADIO AGC_ATTEN9: PGA_AGCGAIN_9 (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9                             MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Msk
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_0                           (0x1U << MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_1                           (0x2U << MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_2                           (0x4U << MR_SUBG_RADIO_AGC_ATTEN9_PGA_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Pos                       (0UL)		/*!<MR_SUBG_RADIO AGC_ATTEN9: ATTEN_AGCGAIN_9 (Bit 0) */
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Msk                       (0xfUL)		/*!< MR_SUBG_RADIO AGC_ATTEN9: ATTEN_AGCGAIN_9 (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9                           MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Msk
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_0                         (0x1U << MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_1                         (0x2U << MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_2                         (0x4U << MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Pos)
#define MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_3                         (0x8U << MR_SUBG_RADIO_AGC_ATTEN9_ATTEN_AGCGAIN_9_Pos)

/* =====================================================    AGC1_ATTEN_TRIM    =====================================================*/ 
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos               (0UL)		/*!<MR_SUBG_RADIO AGC1_ATTEN_TRIM: AGC1_ATTEN_TRIM_IN (Bit 0) */
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Msk               (0xffUL)		/*!< MR_SUBG_RADIO AGC1_ATTEN_TRIM: AGC1_ATTEN_TRIM_IN (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN                   MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Msk
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_0                 (0x1U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_1                 (0x2U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_2                 (0x4U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_3                 (0x8U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_4                 (0x10U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_5                 (0x20U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_6                 (0x40U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_7                 (0x80U << MR_SUBG_RADIO_AGC1_ATTEN_TRIM_AGC1_ATTEN_TRIM_IN_Pos)

/* =====================================================    AGC2_ATTEN_TRIM    =====================================================*/ 
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos               (0UL)		/*!<MR_SUBG_RADIO AGC2_ATTEN_TRIM: AGC2_ATTEN_TRIM_IN (Bit 0) */
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Msk               (0xffUL)		/*!< MR_SUBG_RADIO AGC2_ATTEN_TRIM: AGC2_ATTEN_TRIM_IN (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN                   MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Msk
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_0                 (0x1U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_1                 (0x2U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_2                 (0x4U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_3                 (0x8U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_4                 (0x10U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_5                 (0x20U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_6                 (0x40U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_7                 (0x80U << MR_SUBG_RADIO_AGC2_ATTEN_TRIM_AGC2_ATTEN_TRIM_IN_Pos)

/* =====================================================    AGC3_ATTEN_TRIM    =====================================================*/ 
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos               (0UL)		/*!<MR_SUBG_RADIO AGC3_ATTEN_TRIM: AGC3_ATTEN_TRIM_IN (Bit 0) */
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Msk               (0xffUL)		/*!< MR_SUBG_RADIO AGC3_ATTEN_TRIM: AGC3_ATTEN_TRIM_IN (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN                   MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Msk
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_0                 (0x1U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_1                 (0x2U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_2                 (0x4U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_3                 (0x8U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_4                 (0x10U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_5                 (0x20U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_6                 (0x40U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_7                 (0x80U << MR_SUBG_RADIO_AGC3_ATTEN_TRIM_AGC3_ATTEN_TRIM_IN_Pos)

/* =====================================================    AGC4_ATTEN_TRIM    =====================================================*/ 
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos               (0UL)		/*!<MR_SUBG_RADIO AGC4_ATTEN_TRIM: AGC4_ATTEN_TRIM_IN (Bit 0) */
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Msk               (0xffUL)		/*!< MR_SUBG_RADIO AGC4_ATTEN_TRIM: AGC4_ATTEN_TRIM_IN (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN                   MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Msk
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_0                 (0x1U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_1                 (0x2U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_2                 (0x4U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_3                 (0x8U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_4                 (0x10U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_5                 (0x20U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_6                 (0x40U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)
#define MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_7                 (0x80U << MR_SUBG_RADIO_AGC4_ATTEN_TRIM_AGC4_ATTEN_TRIM_IN_Pos)

/* =====================================================    AGC_PGA_HWTRIM_OUT    =====================================================*/ 
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Pos               (0UL)		/*!<MR_SUBG_RADIO AGC_PGA_HWTRIM_OUT: AGC_HW_PGA_TRIM (Bit 0) */
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Msk               (0xfUL)		/*!< MR_SUBG_RADIO AGC_PGA_HWTRIM_OUT: AGC_HW_PGA_TRIM (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM                   MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Msk
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_0                 (0x1U << MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Pos)
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_1                 (0x2U << MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Pos)
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_2                 (0x4U << MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Pos)
#define MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_3                 (0x8U << MR_SUBG_RADIO_AGC_PGA_HWTRIM_OUT_AGC_HW_PGA_TRIM_Pos)

/* =====================================================    PA_REG    =====================================================*/ 
#define MR_SUBG_RADIO_PA_REG_PA_DEGEN_ON_Pos                               (3UL)		/*!<MR_SUBG_RADIO PA_REG: PA_DEGEN_ON (Bit 3) */
#define MR_SUBG_RADIO_PA_REG_PA_DEGEN_ON_Msk                               (0x8UL)		/*!< MR_SUBG_RADIO PA_REG: PA_DEGEN_ON (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_PA_REG_PA_DEGEN_ON                                   MR_SUBG_RADIO_PA_REG_PA_DEGEN_ON_Msk
#define MR_SUBG_RADIO_PA_REG_CFG_FILT_Pos                                  (0UL)		/*!<MR_SUBG_RADIO PA_REG: CFG_FILT (Bit 0) */
#define MR_SUBG_RADIO_PA_REG_CFG_FILT_Msk                                  (0x3UL)		/*!< MR_SUBG_RADIO PA_REG: CFG_FILT (Bitfield-Mask: 0x03) */
#define MR_SUBG_RADIO_PA_REG_CFG_FILT                                      MR_SUBG_RADIO_PA_REG_CFG_FILT_Msk
#define MR_SUBG_RADIO_PA_REG_CFG_FILT_0                                    (0x1U << MR_SUBG_RADIO_PA_REG_CFG_FILT_Pos)
#define MR_SUBG_RADIO_PA_REG_CFG_FILT_1                                    (0x2U << MR_SUBG_RADIO_PA_REG_CFG_FILT_Pos)

/* =====================================================    PA_HWTRIM_OUT    =====================================================*/ 
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Pos                   (4UL)		/*!<MR_SUBG_RADIO PA_HWTRIM_OUT: PA_HW_DEGEN_TRIM (Bit 4) */
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Msk                   (0xf0UL)		/*!< MR_SUBG_RADIO PA_HWTRIM_OUT: PA_HW_DEGEN_TRIM (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM                       MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Msk
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_0                     (0x1U << MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Pos)
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_1                     (0x2U << MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Pos)
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_2                     (0x4U << MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Pos)
#define MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_3                     (0x8U << MR_SUBG_RADIO_PA_HWTRIM_OUT_PA_HW_DEGEN_TRIM_Pos)

/* =====================================================    RSSI_FLT    =====================================================*/ 
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Pos                                (4UL)		/*!<MR_SUBG_RADIO RSSI_FLT: RSSI_FLT (Bit 4) */
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Msk                                (0xf0UL)		/*!< MR_SUBG_RADIO RSSI_FLT: RSSI_FLT (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT                                    MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Msk
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_0                                  (0x1U << MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_1                                  (0x2U << MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_2                                  (0x4U << MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_3                                  (0x8U << MR_SUBG_RADIO_RSSI_FLT_RSSI_FLT_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Pos                          (0UL)		/*!<MR_SUBG_RADIO RSSI_FLT: OOK_PEAK_DECAY (Bit 0) */
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Msk                          (0xfUL)		/*!< MR_SUBG_RADIO RSSI_FLT: OOK_PEAK_DECAY (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY                              MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Msk
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_0                            (0x1U << MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_1                            (0x2U << MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_2                            (0x4U << MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Pos)
#define MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_3                            (0x8U << MR_SUBG_RADIO_RSSI_FLT_OOK_PEAK_DECAY_Pos)

/* =====================================================    SYNTH2_ANA_ENG    =====================================================*/ 
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_LD_WIN_ACC_Pos                (3UL)		/*!<MR_SUBG_RADIO SYNTH2_ANA_ENG: RFD_PLL_LD_WIN_ACC (Bit 3) */
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_LD_WIN_ACC_Msk                (0x8UL)		/*!< MR_SUBG_RADIO SYNTH2_ANA_ENG: RFD_PLL_LD_WIN_ACC (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_LD_WIN_ACC                    MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_LD_WIN_ACC_Msk
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Pos               (0UL)		/*!<MR_SUBG_RADIO SYNTH2_ANA_ENG: RFD_PLL_VCO_ALC_AMP (Bit 0) */
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Msk               (0x7UL)		/*!< MR_SUBG_RADIO SYNTH2_ANA_ENG: RFD_PLL_VCO_ALC_AMP (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP                   MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Msk
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_0                 (0x1U << MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Pos)
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_1                 (0x2U << MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Pos)
#define MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_2                 (0x4U << MR_SUBG_RADIO_SYNTH2_ANA_ENG_RFD_PLL_VCO_ALC_AMP_Pos)

/* =====================================================    RXADC_HWDELAYTRIM_OUT    =====================================================*/ 
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Pos       (3UL)		/*!<MR_SUBG_RADIO RXADC_HWDELAYTRIM_OUT: RXADC_HW_DELAYTRIM_Q (Bit 3) */
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Msk       (0x38UL)		/*!< MR_SUBG_RADIO RXADC_HWDELAYTRIM_OUT: RXADC_HW_DELAYTRIM_Q (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q           MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Msk
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_0         (0x1U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Pos)
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_1         (0x2U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Pos)
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_2         (0x4U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_Q_Pos)
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Pos       (0UL)		/*!<MR_SUBG_RADIO RXADC_HWDELAYTRIM_OUT: RXADC_HW_DELAYTRIM_I (Bit 0) */
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Msk       (0x7UL)		/*!< MR_SUBG_RADIO RXADC_HWDELAYTRIM_OUT: RXADC_HW_DELAYTRIM_I (Bitfield-Mask: 0x07) */
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I           MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Msk
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_0         (0x1U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Pos)
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_1         (0x2U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Pos)
#define MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_2         (0x4U << MR_SUBG_RADIO_RXADC_HWDELAYTRIM_OUT_RXADC_HW_DELAYTRIM_I_Pos)

/* =====================================================    RX_AAF_HWTRIM_OUT    =====================================================*/ 
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Pos                  (0UL)		/*!<MR_SUBG_RADIO RX_AAF_HWTRIM_OUT: AAF_HW_FCTRIM (Bit 0) */
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Msk                  (0xfUL)		/*!< MR_SUBG_RADIO RX_AAF_HWTRIM_OUT: AAF_HW_FCTRIM (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM                      MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Msk
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_0                    (0x1U << MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Pos)
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_1                    (0x2U << MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Pos)
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_2                    (0x4U << MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Pos)
#define MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_3                    (0x8U << MR_SUBG_RADIO_RX_AAF_HWTRIM_OUT_AAF_HW_FCTRIM_Pos)

/* =====================================================    SINGEN_ANA_ENG    =====================================================*/ 
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_LBE_Pos                    (2UL)		/*!<MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_LBE (Bit 2) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_LBE_Msk                    (0x4UL)		/*!< MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_LBE (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_LBE                        MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_LBE_Msk
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_DIV2_PUP_Pos               (1UL)		/*!<MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_DIV2_PUP (Bit 1) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_DIV2_PUP_Msk               (0x2UL)		/*!< MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_DIV2_PUP (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_DIV2_PUP                   MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_DIV2_PUP_Msk
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_ENA_Pos                    (0UL)		/*!<MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_ENA (Bit 0) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_ENA_Msk                    (0x1UL)		/*!< MR_SUBG_RADIO SINGEN_ANA_ENG: RFD_SINGEN_ENA (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_ENA                        MR_SUBG_RADIO_SINGEN_ANA_ENG_RFD_SINGEN_ENA_Msk

/* =====================================================    RF_INFO_OUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Pos                           (4UL)         /*!<MR_SUBG_RADIO RF_INFO_OUT: RFSUBG_ID (Bit 4) */
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Msk                           (0xf0UL)      /*!< MR_SUBG_RADIO RF_INFO_OUT: RFSUBG_ID (Bitfield-Mask: 0x0f) */
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID                                MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Msk
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_0                             (0x1U << MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Pos)
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_1                             (0x2U << MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Pos)
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_2                             (0x4U << MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Pos)
#define MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_3                             (0x8U << MR_SUBG_RADIO_RF_INFO_OUT_RFSUBG_ID_Pos)

/* =====================================================    RF_FSM8_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos                 (0UL)		/*!<MR_SUBG_RADIO RF_FSM8_TIMEOUT: SYNTH_PDWN_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Msk                 (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM8_TIMEOUT: SYNTH_PDWN_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER                     MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_0                   (0x1U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_1                   (0x2U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_2                   (0x4U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_3                   (0x8U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_4                   (0x10U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_5                   (0x20U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_6                   (0x40U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_7                   (0x80U << MR_SUBG_RADIO_RF_FSM8_TIMEOUT_SYNTH_PDWN_TIMER_Pos)

/* =====================================================    RF_FSM9_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos                     (0UL)		/*!<MR_SUBG_RADIO RF_FSM9_TIMEOUT: END_RX_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Msk                     (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM9_TIMEOUT: END_RX_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER                         MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_0                       (0x1U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_1                       (0x2U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_2                       (0x4U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_3                       (0x8U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_4                       (0x10U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_5                       (0x20U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_6                       (0x40U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_7                       (0x80U << MR_SUBG_RADIO_RF_FSM9_TIMEOUT_END_RX_TIMER_Pos)

/* =====================================================    RF_FSM10_TIMEOUT    =====================================================*/ 
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos                    (0UL)		/*!<MR_SUBG_RADIO RF_FSM10_TIMEOUT: END_TX_TIMER (Bit 0) */
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Msk                    (0xffUL)		/*!< MR_SUBG_RADIO RF_FSM10_TIMEOUT: END_TX_TIMER (Bitfield-Mask: 0xff) */
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER                        MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Msk
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_0                      (0x1U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_1                      (0x2U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_2                      (0x4U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_3                      (0x8U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_4                      (0x10U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_5                      (0x20U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_6                      (0x40U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)
#define MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_7                      (0x80U << MR_SUBG_RADIO_RF_FSM10_TIMEOUT_END_TX_TIMER_Pos)

/* =====================================================    SUBG_DIG_CTRL0    =====================================================*/ 
#define MR_SUBG_RADIO_SUBG_DIG_CTRL0_FORCE_GPIO_OUTPUT_Pos                 (0UL)		/*!<MR_SUBG_RADIO SUBG_DIG_CTRL0: FORCE_GPIO_OUTPUT (Bit 0) */
#define MR_SUBG_RADIO_SUBG_DIG_CTRL0_FORCE_GPIO_OUTPUT_Msk                 (0x1UL)		/*!< MR_SUBG_RADIO SUBG_DIG_CTRL0: FORCE_GPIO_OUTPUT (Bitfield-Mask: 0x01) */
#define MR_SUBG_RADIO_SUBG_DIG_CTRL0_FORCE_GPIO_OUTPUT                     MR_SUBG_RADIO_SUBG_DIG_CTRL0_FORCE_GPIO_OUTPUT_Msk


/** @} */ /* End of group PosMask_peripherals */

/** @addtogroup Exported_macros
  * @{
  */

/*********************** UART Instances : Asynchronous mode *******************/
#define IS_UART_INSTANCE(INSTANCE) ((INSTANCE) == USART1)

/*********************** UART Instances : FIFO mode ***************************/
#define IS_UART_FIFO_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                           ((INSTANCE) == LPUART1))

/*********************** UART Instances : SPI Slave mode **********************/
#define IS_UART_SPI_SLAVE_INSTANCE(INSTANCE) ((INSTANCE) == USART1)

/*********************** USART Instances : Synchronous mode *******************/
#define IS_USART_INSTANCE(INSTANCE) ((INSTANCE) == USART1)

/*********************** USART Instances : Auto Baud Rate detection ***********/
#define IS_USART_AUTOBAUDRATE_DETECTION_INSTANCE(INSTANCE) ((INSTANCE) == USART1)

/*********************** UART Instances : Half-Duplex mode ********************/
#define IS_UART_HALFDUPLEX_INSTANCE(INSTANCE)   (((INSTANCE) == USART1) || \
                                                 ((INSTANCE) == LPUART1))

/*********************** UART Instances : LIN mode ****************************/
#define IS_UART_LIN_INSTANCE(INSTANCE)    ((INSTANCE) == USART1)

/*********************** UART Instances : Hardware Flow control ***************/
#define IS_UART_HWFLOW_INSTANCE(INSTANCE) (((INSTANCE) == USART1) || \
                                           ((INSTANCE) == LPUART1))

/*********************** UART Instances : Smard card mode *********************/
#define IS_SMARTCARD_INSTANCE(INSTANCE) ((INSTANCE) == USART1)


/*********************** UART Instances : Driver Enable ***********************/
#define IS_UART_DRIVER_ENABLE_INSTANCE(INSTANCE) (((INSTANCE) == USART1)|| \
                                                  ((INSTANCE) == LPUART1))

/*********************** UART Instances : IRDA mode ***************************/
#define IS_IRDA_INSTANCE(INSTANCE) ((INSTANCE) == USART1)

/*********************** UART Instances : Wake-up from Stop mode **************/
#define IS_UART_WAKEUP_FROMSTOP_INSTANCE(INSTANCE) ((INSTANCE) == LPUART1)

/******************** LPUART Instance *****************************************/
#define IS_LPUART_INSTANCE(INSTANCE)    ((INSTANCE) == LPUART1)


/******************************* ADC Instances ********************************/
#define IS_ADC_ALL_INSTANCE(INSTANCE) ((INSTANCE) == ADC1)

/******************************* CRC Instances ********************************/
#define IS_CRC_ALL_INSTANCE(INSTANCE) ((INSTANCE) == CRC)

/******************************* DAC Instances ********************************/
#define IS_DAC_ALL_INSTANCE(INSTANCE) ((INSTANCE) == DAC1)

/******************************** DMA Instances *******************************/
#define IS_DMA_ALL_INSTANCE(INSTANCE) (((INSTANCE) == DMA1_Channel1) || \
                                       ((INSTANCE) == DMA1_Channel2) || \
                                       ((INSTANCE) == DMA1_Channel3) || \
                                       ((INSTANCE) == DMA1_Channel4) || \
                                       ((INSTANCE) == DMA1_Channel5) || \
                                       ((INSTANCE) == DMA1_Channel6) || \
                                       ((INSTANCE) == DMA1_Channel7) || \
                                       ((INSTANCE) == DMA1_Channel8))

/******************************** DMAMUX Instances ****************************/
#define IS_DMAMUX_ALL_INSTANCE(INSTANCE) ((INSTANCE) == DMAMUX1)

/******************************* GPIO Instances *******************************/
#define IS_GPIO_ALL_INSTANCE(INSTANCE) (((INSTANCE) == GPIOA) || \
                                        ((INSTANCE) == GPIOB))

/******************************* GPIO AF Instances ****************************/
#define IS_GPIO_AF_INSTANCE(INSTANCE)   IS_GPIO_ALL_INSTANCE(INSTANCE)

/**************************** GPIO Lock Instances *****************************/
#define IS_GPIO_LOCK_INSTANCE(INSTANCE) IS_GPIO_ALL_INSTANCE(INSTANCE)

/******************************** I2C Instances *******************************/
#define IS_I2C_ALL_INSTANCE(INSTANCE) (((INSTANCE) == I2C1) || \
                                       ((INSTANCE) == I2C2))

/******************************* SMBUS Instances ******************************/
#define IS_SMBUS_ALL_INSTANCE(INSTANCE) (((INSTANCE) == I2C1) || \
                                        ((INSTANCE) == I2C2))

/******************************* RNG Instances ********************************/
#define IS_RNG_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == RNG)

/****************************** RTC Instances *********************************/
#define IS_RTC_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == RTC)

/******************************** SPI Instances *******************************/
#define IS_SPI_ALL_INSTANCE(INSTANCE) (((INSTANCE) == SPI1) || \
                                       ((INSTANCE) == SPI3))

/******************************** I2S Instances *******************************/
#define IS_I2S_ALL_INSTANCE(INSTANCE)  (((INSTANCE) == SPI3))

/****************************** IWDG Instances ********************************/
#define IS_IWDG_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == IWDG)

/****************** TIM Instances : All supported instances *******************/
#define IS_TIM_INSTANCE(INSTANCE)       (((INSTANCE) == TIM2) || \
					 ((INSTANCE) == TIM16))

/****************** TIM Instances : DMA requests generation *******************/
#define IS_TIM_DMA_INSTANCE(INSTANCE)  (((INSTANCE) == TIM2)   || \
                                        ((INSTANCE) == TIM16))

/****************** TIM Instances : supporting the break function *************/
#define IS_TIM_BREAK_INSTANCE(INSTANCE)    ((INSTANCE) == TIM16)

/************** TIM Instances : supporting Break source selection *************/
#define IS_TIM_BREAKSOURCE_INSTANCE(INSTANCE) ((INSTANCE) == TIM16)

/************* TIM Instances : at least 1 capture/compare channel *************/
#define IS_TIM_CC1_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
					 ((INSTANCE) == TIM16))

/************ TIM Instances : at least 2 capture/compare channels *************/
#define IS_TIM_CC2_INSTANCE(INSTANCE)   ((INSTANCE) == TIM2)

/************ TIM Instances : at least 3 capture/compare channels *************/
#define IS_TIM_CC3_INSTANCE(INSTANCE)   ((INSTANCE) == TIM2)

/************ TIM Instances : at least 4 capture/compare channels *************/
#define IS_TIM_CC4_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2))

/************ TIM Instances : DMA requests generation (TIMx_DIER.CCxDE) *******/
#define IS_TIM_DMA_CC_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2)  || \
                                            ((INSTANCE) == TIM16))

/******************** TIM Instances : DMA burst feature ***********************/
#define IS_TIM_DMABURST_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2)   || \
                                              ((INSTANCE) == TIM16))

/******************* TIM Instances : output(s) available **********************/
#define IS_TIM_CCX_INSTANCE(INSTANCE, CHANNEL) \
        ((((INSTANCE) == TIM2) &&                    \
           (((CHANNEL) == TIM_CHANNEL_1) ||          \
            ((CHANNEL) == TIM_CHANNEL_2) ||          \
            ((CHANNEL) == TIM_CHANNEL_3) ||          \
            ((CHANNEL) == TIM_CHANNEL_4)))           \
           ||                                        \
           (((INSTANCE) == TIM16) &&                 \
           (((CHANNEL) == TIM_CHANNEL_1))))

/****************** TIM Instances : supporting complementary output(s) ********/
#define IS_TIM_CCXN_INSTANCE(INSTANCE, CHANNEL) \
   (((INSTANCE) == TIM16) &&                   \
    ((CHANNEL) == TIM_CHANNEL_1))

/****************** TIM Instances : supporting clock division *****************/
#define IS_TIM_CLOCK_DIVISION_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
						    ((INSTANCE) == TIM16))

/****** TIM Instances : supporting external clock mode 1 for ETRF input *******/
#define IS_TIM_CLOCKSOURCE_ETRMODE1_INSTANCE(INSTANCE) ((INSTANCE) == TIM2)

/****** TIM Instances : supporting external clock mode 2 for ETRF input *******/
#define IS_TIM_CLOCKSOURCE_ETRMODE2_INSTANCE(INSTANCE) ((INSTANCE) == TIM2)

/****************** TIM Instances : supporting external clock mode 1 for TIX inputs*/
#define IS_TIM_CLOCKSOURCE_TIX_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
						     ((INSTANCE) == TIM16))

/****************** TIM Instances : supporting internal trigger inputs(ITRX) *******/
#define IS_TIM_CLOCKSOURCE_ITRX_INSTANCE(INSTANCE)     ((INSTANCE) == TIM16)

/****************** TIM Instances : supporting combined 3-phase PWM mode ******/
#define IS_TIM_COMBINED3PHASEPWM_INSTANCE(INSTANCE) ((INSTANCE) == TIM2)

/****************** TIM Instances : supporting commutation event generation ***/
#define IS_TIM_COMMUTATION_EVENT_INSTANCE(INSTANCE)   ((INSTANCE) == TIM16)

/****************** TIM Instances : supporting counting mode selection ********/
#define IS_TIM_COUNTER_MODE_SELECT_INSTANCE(INSTANCE)  ((INSTANCE) == TIM2)

/****************** TIM Instances : supporting encoder interface **************/
#define IS_TIM_ENCODER_INTERFACE_INSTANCE(INSTANCE)  ((INSTANCE) == TIM2)

/**************** TIM Instances : external trigger input available ************/
#define IS_TIM_ETR_INSTANCE(INSTANCE)      ((INSTANCE) == TIM2)

/************* TIM Instances : supporting ETR source selection ***************/
#define IS_TIM_ETRSEL_INSTANCE(INSTANCE)    ((INSTANCE) == TIM2)

/****** TIM Instances : Master mode available (TIMx_CR2.MMS available )********/
#define IS_TIM_MASTER_INSTANCE(INSTANCE)    (((INSTANCE) == TIM2) || \
                                             ((INSTANCE) == TIM16))

/*********** TIM Instances : Slave mode available (TIMx_SMCR available )*******/
#define IS_TIM_SLAVE_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
                                           ((INSTANCE) == TIM16))

/****************** TIM Instances : supporting OCxREF clear *******************/
#define IS_TIM_OCXREF_CLEAR_INSTANCE(INSTANCE)        ((INSTANCE) == TIM2)

/****************** TIM Instances : remapping capability **********************/
#define IS_TIM_REMAP_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
                                           ((INSTANCE) == TIM16))

/****************** TIM Instances : supporting repetition counter *************/
#define IS_TIM_REPETITION_COUNTER_INSTANCE(INSTANCE)   (((INSTANCE) == TIM2) || \
                                                        ((INSTANCE) == TIM16))

/****************** TIM Instances : supporting synchronization ****************/
#define IS_TIM_SYNCHRO_INSTANCE(INSTANCE)  IS_TIM_MASTER_INSTANCE(INSTANCE)

/****************** TIM Instances : supporting ADC triggering through TRGO2 ***/
#define IS_TIM_TRGO2_INSTANCE(INSTANCE)    (0)

/******************* TIM Instances : Timer input XOR function *****************/
#define IS_TIM_XOR_INSTANCE(INSTANCE)      ((INSTANCE) == TIM2)

/************ TIM Instances : Advanced timers  ********************************/
#define IS_TIM_ADVANCED_INSTANCE(INSTANCE)    (0)

/******************************* AES Instances ********************************/
#define IS_AES_ALL_INSTANCE(INSTANCE) (((INSTANCE) == AES))

/******************************** COMP Instances ******************************/
#define IS_COMP_ALL_INSTANCE(INSTANCE) (((INSTANCE) == COMP1))

/******************************* LCD Instances ********************************/
#define IS_LCD_ALL_INSTANCE(INSTANCE)  ((INSTANCE) == LCD)

/** @} */ /* End of group STM32WBL3x_Peripheral_Exported_macros */

/** @} */ /* End of group STM32WL3x */

/** @} */ /* End of group ST */

#ifdef __cplusplus
}
#endif

#endif /* STM32WL33_H */
